% This function demonstrates the use of the e02be & e02bb routines from the 
% NAG toolbox for fitting a cubic spline through a set of points. 
%

% NAG Copyright 2009.

function e02be_demo
% Here's the (x,y) data to be fitted, followed by the 
% weights for each point (set these to 1 if all points
% are equally important).  
x = [0.5; 1; 1.5; 2; 2.5; 3; 4; 4.5; 5; 5.5; 6; 7; 7.5];
y = [-0.372; 0.431; 1.69; 2.11; 3.1; 4.23; 4.35;
     4.81; 4.61; 4.79; 5.23; 6.35; 7.19];
w = [2; 1.5; 1; 3; 1; 0.5; 1; 2; 2.5; 1; 3; 1; 2];

% Calculate work array lengths, according to the e02be documentation.
m = length(x);
nest = m+4;
lwrk = 4*m + 16*nest + 41;

% Now initialize some parameters required by the NAG routine.
start = 'C';
n = nag_int(0);
lamda = zeros(nest,1);
wrk = zeros(lwrk, 1);
iwrk = zeros(nest, 1, nag_int_name);

pause on;

% Set up the points at which the spline is to be evaluated.
px = x(1) : 0.01 : x(m);
pf = zeros(size(px));

figure('Name', 'Fitting a cubic spline using the NAG Toolbox', ...
    'Position', [100, 200, 700, 600], ...
    'NumberTitle', 'off');
axis([0 8 -1 8]);

axes = gca;
set(gca, 'FontSize', 12);

% Create a button and wait for it to be pressed before continuing
hbutton = uicontrol('Position', [20 10 140 40], ...
                    'String', 'Start', 'Callback', @my_button_handler);
% Plot the initial data.
plot(x,y,'o', 'MarkerEdgeColor','k', 'MarkerFaceColor','k', 'MarkerSize', 5);
title('Using NAG routines e02be, e02bb to fit a cubic spline', 'FontSize', 14);

uiwait(gcf);

hold on;

s = 5.0;
while s > 0.001
  xlabel('x');
  ylabel('y', 'Rotation', 0);

  [nOut, lamdaOut, c, fp, wrkOut, iwrkOut, ifail] = ...
      e02be(start, x, y, w, s, n, lamda, wrk, iwrk);
  for i = 1 : length(px)
    [pf(i), ifail] = e02bb(lamdaOut, c, px(i), 'ncap7', nOut);
  end
  plot(px, pf, 'Color', [.8 0 0], 'Linewidth', 2);
  hold on;
  title('Using NAG routines e02be, e02bb to fit a cubic spline', 'FontSize', 14);
  % Plot the initial data.
  plot(x,y,'o', 'MarkerEdgeColor','k', 'MarkerFaceColor','k', 'MarkerSize', 5);
  x1 = 4.6;
  y1 = 2;
  t = sprintf('Smoothness = %7.3f', s);
  text(x1,y1, t, 'FontSize', 12);
  t = sprintf('Number of knots = %4d', double(nOut)-8);
  text(x1,y1-0.5, t, 'FontSize', 12);
  hold off;
  s = s / 1.2;
  pause(0.25);
end

set(hbutton, 'String', 'Close');

% This function is used to handle the control button on the graphics window.
function my_button_handler(hObject, eventData)
label = get(hObject, 'String');
if strcmp(label, 'Close')
   close;
elseif strcmp(label, 'Pause')
   set(hObject, 'String', 'Resume');
   uiwait;
elseif strcmp(label, 'Start') || strcmp(label, 'Resume')
   set(hObject, 'String', 'Pause');
   uiresume;
elseif strcmp(label, 'Next')
   uiresume;
end
