function e04gb_demo(pause_time, rolling)
% Finds the minimum of Rosenbrock's Function:
%   F = 100*(x2-x1^2)^2 + (1-x1)^2
% using NAG routine e04gb, which minimizes a sum of squares of
% m nonlinear functions in n variables.

if nargin < 2

  % rolling = 1 means use the full screen for plots (good for
  % rolling demo) and don't wait for key press before starting
  rolling = 0;

  if nargin < 1
    pause_time = 1;
  end

end

global mypause;
mypause = pause_time;

% Initial guess at solution
x = [-1.7; 1.37];

% Create data for contour plot of Rosenbrock's function
px = -3 : 0.05 : 1.5;
py = -8 : 0.05 : 3;
pf = zeros(length(py),length(px));

% User structure to pass to objective function.
%   user{1}(1) plot progress? 1 = yes
%   user{1}(2) axes of the plotting figure
%   user{1}(3) x coordinate of the previous plotted point
%   user{1}(4) y coordinate of the previous plotted point
%   user{1}(5) pause time in seconds after each point is plotted.
%   user{1}(6) number of calls to this evaluation function

global user;
user = cell(1,1);

tfj = zeros(2,2);
tfv = zeros(2);
for i = 1:length(py)

  for j = 1:length(px)
    xx = [px(j); py(i)];
    ggrad=[0; 0];
    nstate = nag_int(0);
    user{1} = zeros(6,1);
    [iflag, tfv, tfj] = e04gb_lsqfun(0, 2, 2, xx, 2);
    pf(i,j) = tfv(1,1)*tfv(1,1) + tfv(2,1)*tfv(2,1);
  end

end

scrsz = get(0, 'ScreenSize');

if rolling
  figure('Name', 'Minimum of Rosenbrock''s Function using NAG Toolbox for MATLAB', ...
         'Position', [0 -35 scrsz(3)*1.0 scrsz(4)*1.0], ...
         'NumberTitle', 'off');
else
  figure('Name', 'Minimum of Rosenbrock''s Function using NAG Toolbox for MATLAB', ...
         'Position', [scrsz(1)+100 scrsz(2)+20 scrsz(3)*0.8 scrsz(4)*0.8], ...
         'NumberTitle', 'off');
end
axes = gca;

% Heights of contours required are in vector v.
v = [5 30 100 220 400 700 1200 2000 3000 4500 6500 9000 12000];
[C,h] = contour(axes, px, py, pf, v, 'Linewidth', 1);
set(h, 'ShowText', 'on', 'TextStep', get(h,'LevelStep')*2);
colormap('prism');
hold on;

% Plot the true minimum with a green circle
plot(axes, 1, 1, 'go', 'Linewidth', 2, 'MarkerSize', 16);

title('NAG routine e04gb: minimizing Rosenbrock''s Function as a Sum of Squares', 'FontSize', 14);

% Plot the starting point with a blue circle
plot(axes, x(1), x(2), 'bo', 'Linewidth', 2, 'MarkerSize', 16);

% Add a legend
frx = [-2.85 -1.15 -1.15 -2.85];
fry = [-5 -5 -7.65 -7.65];
fill(frx, fry, 'w');
%text('Interpreter', 'latex',...
%     'String', '$f(x,y) = 100(y-x^2)^2 + (1-x)^2$',...
%     'Position', [-2.75, -0.45], 'FontSize', 12);
text(-2.75, -5.3, 'f(x,y) = 100(y-x^2)^2 + (1-x)^2', 'FontSize', 12);
plot(axes, -2.75, -5.8, 'bo', 'Linewidth', 2, 'MarkerSize', 10);
text(-2.65, -5.8, 'Starting point', 'FontSize', 10);
plot(axes, -2.75, -6.3, 'r*', 'Linewidth', 2, 'MarkerSize', 8);
text(-2.65, -6.3, 'Current evaluation point', 'FontSize', 10);
plot(axes, -2.75, -6.8, 'b*', 'Linewidth', 2, 'MarkerSize', 8);
text(-2.65, -6.8, 'Previous evaluation points', 'FontSize', 10);
plot(axes, -2.75, -7.3, 'go', 'Linewidth', 2, 'MarkerSize', 10);
text(-2.65, -7.3, 'Target minimum', 'FontSize', 10);
% Draw a green line from previous to current point
line([-2.0,-1.8], [-5.8,-5.8], 'Color', [0 1 0], 'Linewidth', 2);
text(-1.75, -5.8, 'Search path', 'FontSize', 10);


if rolling ~= 1
  % Create a button and wait for it to be pressed before continuing
  hbutton = uicontrol('Position', [10 20 140 40], ...
                      'String', 'Start', 'Callback', @my_button_handler);
  hspeed = uicontrol('Style', 'popup', 'String', 'Slow|Medium|Fast', ...
            'Position', [10 90 110 30], 'TooltipString', ...
            'Chooses the speed of animation', ...
            'Value', 1, 'Callback', @animspeed);

  uiwait(gcf); % Wait until "Start" button is pressed.

  % Disable other buttons once animation is underway
  set(hspeed, 'Enable', 'Off');
else
  % Create a pause button
  hbutton = uicontrol('Position', [10 20 140 40], ...
                      'String', 'Pause', 'Callback', @my_button_handler);
end

frx = [-0.75 1.4 1.4 -0.75];
fry = [2.85 2.85 1.5 1.5];
fill(frx, fry, 'w');

user{1} = [1, axes, x(1), x(2), mypause, 0];

m = nag_int(2);
maxcal = nag_int(150);
eta = 0.7;
xtol = 1.05418557512311e-07;
iw = zeros(1,nag_int_name);
w = zeros(105,1);

% Solve the problem using NAG routine e04gb.
[xOut, fsumsq, fvec, fjac, s, v, niter, nf, iwOut, wOut, ifail] = ...
    e04gb(m, 'e04hev', 'e04gb_lsqfun', 'e04gb_lsqmon', ...
          maxcal, eta, xtol, x, iw, w);

% Show the results
fill(frx, fry, 'w');
t = sprintf('Number of evaluations of objective function = %d', user{1}(6));
text(-0.7, 2.6, t, 'FontSize', 12);
t = sprintf('Minimum point is at (%5.2f, %5.2f)', xOut(1), xOut(2));
text(-0.7, 2.15, t, 'FontSize', 12);
t = sprintf('Objective function at minimum = %15.10f', fsumsq);
text(-0.7, 1.7, t, 'FontSize', 12);

if rolling ~= 1
  set(hbutton, 'String', 'Close');
end

function button_handler(hObject, eventData)
label = get(hObject, 'String');
if strcmp(label, 'Close') || strcmp(label, 'Quit')
   close;
elseif strcmp(label, 'Pause')
   set(hObject, 'String', 'Resume');
   uiwait;
elseif strcmp(label, 'Start') || strcmp(label, 'Resume')
   set(hObject, 'String', 'Pause');
   uiresume;
elseif strcmp(label, 'Next')
   uiresume;
end

function animspeed(hObject, eventData)
global mypause;
val = get(hObject, 'Value');
if val == 1
  mypause = 0.5;
elseif val == 2
  mypause = 0.125;
else
  mypause = 0.01;
end

