function d02tz_example

% diary d02tz_example.r
fprintf('d02tz example results\n\n');

global alpha beta eps; % For communication with local functions

% Initialize variables and arrays.
neq  = nag_int(1);
nlbc = nag_int(1);
nrbc = nag_int(1);
ncol = nag_int(5);
mmax = nag_int(2);
m    = nag_int([2]);
tols = [1.0e-05];

% Set values for problem-specific physical parameters.
alpha = -1/3;
beta  =  1/3;
eps   = 1e-3;

% Set up the mesh.
nmesh  = nag_int(7);
mxmesh = nag_int(50);

% Set location of mesh points, and specify which are fixed.
mesh =   zeros(mxmesh, 1);
ipmesh = zeros(mxmesh, 1, nag_int_name);
mesh(1:nmesh) =  [0.0; 0.15; 0.3; 0.5; 0.7; 0.85; 1.0];
ipmesh(1:2:nmesh) = 1;
ipmesh(2:2:nmesh) = 2;

% Prepare to store results for plotting.
xarray = zeros(1,1);
yarray = zeros(1,2);

% d02tv is a setup routine to be called prior to d02tk.
[work, iwork, ifail] = d02tv( ...
                              m, nlbc, nrbc, ncol, tols, nmesh, mesh, ipmesh);
ncont = 2;

% We run through the calculation ncont times with different parameter sets.
for jcont = 1:ncont

  eps = 0.1*eps;
  fprintf('\n Tolerance = %8.1e, eps = %10.3e\n\n', tols(1), eps);

  % Call d02tk to solve BVP for this set of parameters.
  [work, iwork, ifail] = d02tk( ...
                                @ffun, @fjac, @gafun, @gbfun, @gajac, ...
                                @gbjac, @guess, work, iwork);
  % Call d02tz to extract mesh from solution.
  [nmesh, mesh, ipmesh, ermx, iermx, ijermx, ifail] = ...
  d02tz( ...
         mxmesh, work, iwork);

    % Output mesh results.
  fprintf(' Used a mesh of %d points\n', nmesh);
  fprintf(' Maximum error = %10.2e in interval %d for component %d\n\n',...
          ermx, iermx, ijermx);

  % Output solution, and store it for plotting.
  fprintf(' Solution and derivative at every second point:\n');
  fprintf('      x        u          u''\n');
  for imesh = 1:nmesh

    % Call d02ty to perform interpolation on the solution.
    [y, work, ifail] = d02ty( ...
                              mesh(imesh), neq, mmax, work, iwork);
    if mod(imesh, 2) ~= 0
      fprintf(' %8.4f   %8.5f   %8.5f\n', mesh(imesh), y(1,1), y(1,2));
    end
    xarray(imesh) = mesh(imesh);
    yarray(imesh, :) = y(1, :);
  end

  % Plot results for this parameter set.
  if jcont==1
    fig1 = figure;
  else
    fig2 = figure;
  end
  display_plot(xarray, yarray, eps)
  
  % Select mesh for next calculation.
  if jcont < ncont
    nmesh = (nmesh+1)/2;

    % d02tx allows the current solution to be used as an initial
    % approximation to the solution of a related problem.
    [work, iwork, ifail] = d02tx( ...
                                  nmesh, mesh, ipmesh, work, iwork);
  end
end

% print(fig1,'-dpng','-r75','d02tz_fig1.png');
% print(fig1,'-deps','-r75','d02tz_fig1.eps');
% print(fig2,'-dpng','-r75','d02tz_fig2.png');
% print(fig2,'-deps','-r75','d02tz_fig2.eps');

% diary off

function [f] = ffun(x, y, neq, m)
  % Evaluate derivative functions (rhs of system of ODEs).

  global eps
  f = zeros(neq, 1);
  f(1) = (y(1,1) - y(1,1)*y(1,2))/eps;
  
function [dfdy] = fjac(x, y, neq, m)
  % Evaluate Jacobians (partial derivatives of f).

  dfdy = zeros(neq, neq, 1);

  machep = x02aj;
  fac = sqrt(machep);
  for i = 1:neq
    yp(i,1:m(i)) = y(i,1:m(i));
  end

  for i = 1:neq
    for j = 1:m(i)
      ptrb = max(100*machep, fac*abs(y(i,j)));
      yp(i,j) = y(i,j) + ptrb;
      f1 = ffun(x, yp, neq, m);
      yp(i,j) = y(i,j) - ptrb;
      f2 = ffun(x, yp, neq, m);
      dfdy(:,i,j) = 0.5*(f1- f2)/ptrb;
      yp(i,j) = y(i,j);
    end
  end

function [ga] = gafun(ya, neq, m, nlbc)
  % Evaluate boundary conditions at left-hand end of range.

  global alpha beta
  ga = zeros(nlbc, 1);
  ga(1) = ya(1,1) - alpha;

function [dgady] = gajac(ya, neq, m, nlbc)
  % Evaluate Jacobians (partial derivatives of ga).

  dgady = zeros(nlbc, neq, 2);
  dgady(1,1,1) = 1;

function [gb] = gbfun(yb, neq, m, nrbc)
  % Evaluate boundary conditions at right-hand end of range.

  global alpha beta
  gb = zeros(nrbc, 1);
  gb(1) = yb(1,1) - beta;

function [dgbdy] = gbjac(yb, neq, m, nrbc)
  % Evaluate Jacobians (partial derivatives of gb).

  dgbdy = zeros(nrbc, neq, 2);
  dgbdy(1,1,1) = 1;

function [y, dym] = guess(x, neq, m)
  % Evaluate initial approximations to solution components and derivatives.

  global alpha beta
  y = zeros(neq, 2);
  dym = zeros(neq, 1);
  y(1,1) = alpha + (beta - alpha)*x;
  y(1,2) = beta - alpha;
  dym(1) = 0;

function display_plot(x, y, eps)
  % Plot the results.
  [haxes, hline1, hline2] = plotyy(x, y(:,1), x, y(:,2));
  % Set the axis limits and the tick specifications to beautify the plot.
  set(haxes(1), 'YLim', [-0.5 0.5]);
  set(haxes(1), 'YMinorTick', 'on');
  set(haxes(1), 'YTick', [-0.5 -0.25 0 0.25 0.5]);
  set(haxes(2), 'YLim', [0 2]);
  set(haxes(2), 'YTick', [0 0.25 0.5 0.75 1.0 1.25 1.5 1.75 2]);
  set(haxes(2), 'YMinorTick', 'on');
  for iaxis = 1:2
    % These properties must be the same for both sets of axes.
    set(haxes(iaxis), 'XLim', [0 1]);
  end
  set(gca, 'box', 'off')
  % Add title.
  title('Largerstrom-Cole Equation (-1/3, 1/3)');
  text(0.4,0.4,['eps = ', num2str(eps)]);
  % Label the axes.
  xlabel('x');
  ylabel(haxes(1), 'Solution');
  ylabel(haxes(2), 'Derivative');
  % Add a legend.
  legend('solution','derivative','Location','Best');
  % Set some features of the three lines.
  set(hline1, 'Linewidth', 0.25, 'Marker', '+', 'LineStyle', '-');
  set(hline2, 'Linewidth', 0.25, 'Marker', 'x', 'LineStyle', '--');
