function d02za_example

% diary d02za_example.r
fprintf('d02za example results\n\n');

global isol tkeep ykeep

% Initialize setup variables and arrays.
neq    = nag_int(3);
neqmax = nag_int(neq);
nwkjac = nag_int(neqmax*(neqmax + 1));
maxord = nag_int(5);
sdysav = nag_int(maxord+1);
maxstp = nag_int(200);
mxhnil = nag_int(5);

h0    = 0;
hmax  = 10;
hmin  = 1.0e-10;
tcrit = 10;
petzld = false;

const  = zeros(6, 1);
rwork  = zeros(50+4*neqmax, 1);

% d02nv is a setup routine to be called prior to d02nb.
[const, rwork, ifail] = d02nv( ...
                               neqmax, sdysav, maxord, 'Newton', petzld, ...
                               const, tcrit, hmin, hmax, h0, maxstp, ...
                               mxhnil, 'Average-L2', rwork);
% d02ns determines how d02nb evaluates the Jacobian.
[rwork, ifail] = d02ns( ...
                        neq, neqmax, 'Analytical', nwkjac, rwork);

% Initialize integration variables and arrays.
inform(1:23) = nag_int(0);
ysave = zeros(neq, sdysav);
wkjac = zeros(nwkjac, 1);

% First case. Integrate to tout without passing tout (tcrit=tout and itask=4)
%             use B.D.F formulae with a Newton method.
%             Evaluate Jacobian numerically (d02nbz), no monitoring (d02nby).
t      = 0.0;
tout   = 10.0;
itask  = nag_int(4);
itrace = nag_int(0);
y      = [1; 0; 0];
itol   = nag_int(1);
rtol   = [0.0001];
atol   = [1e-07];

isol          = 1;
tkeep(isol)   = t;
ykeep(1:3,isol) = y;

% Output initial and final solutions.
fprintf('  Using Analytical Jacobian\n\n');
fprintf('     x           y_1          y_2           y_3\n');
fprintf(' %8.3f      %5.1f        %5.1f         %5.1f\n', t, y);

[t, y, ydot, rwork, inform, ysave, wkjac, ifail] = ...
d02nb( ...
       t, tout, y, rwork, rtol, atol, itol, inform, @fcn, ysave, ...
       @jac, wkjac, @monitr, itask, itrace);
fprintf(' %8.3f      %5.1f        %5.1f         %5.1f\n', t, y);

% Plot results.
fig1 = figure;
display_plot(tkeep, ykeep)
% print(fig1,'-dpng','-r75','d02za_fig1.png');
% print(fig1,'-deps','-r75','d02za_fig1.eps');

% diary off

function [f, ires] = fcn(neq, t, y, ires)
  % Evaluate derivative vector.
  f = zeros(3,1);
  f(1) = -0.04*y(1) + 1.0d4*y(2)*y(3);
  f(2) =  0.04*y(1) - 1.0d4*y(2)*y(3) - 3.0d7*y(2)*y(2);
  f(3) =                                3.0d7*y(2)*y(2);

function p = jac(neq, t, y, h, d, p)
  % Evaluate the Jacobian.
  p = zeros(neq, neq);
  hxd = h*d;
  p(1,1) = 1 - hxd*(-0.04);
  p(1,2) =   - hxd*(1.0d4*y(3));
  p(1,3) =   - hxd*(1.0d4*y(2));
  p(2,1) =   - hxd*(0.04);
  p(2,2) = 1 - hxd*(-1.0d4*y(3)-6.0d7*y(2));
  p(2,3) =   - hxd*(-1.0d4*y(2));
  p(3,2) =   - hxd*(6.0d7*y(2));
  p(3,3) = 1 - hxd*(0);

function [hnext, y, imon, inln, hmin, hmax] = ...
         monitr(neq, neqmax, t, hlast, hnext, y, ydot, ...
                ysave, r, acor, imon, hmin, hmax, nqu)

  global isol tkeep ykeep

  inln=nag_int(0);
  if (imon == nag_int(1))
    isol = isol + 1;
    tkeep(isol) = t;
    ykeep(1:3,isol) = y;
  end

function display_plot(x,y)
  % Formatting for title and axis labels.
  % Plot one of the curves and then add the other two.
  hline3 = plot(x, y(3,:));
  hold on;
  [haxes, hline1, hline2] = plotyy(x, 100*y(2,:), x, y(1,:));
  % Set the axis limits and the tick specifications to beautify the plot.
  set(haxes(1), 'YLim', [0 0.0045]);
  set(haxes(1), 'XMinorTick', 'on', 'YMinorTick', 'on');
  set(haxes(1), 'YTick', [0:0.001:0.004]);
  set(haxes(2), 'YLim', [0.995 1.005]);
  set(haxes(2), 'YMinorTick', 'on');
  set(haxes(2), 'YTick', [0.995:0.002:1.005]);
  set(haxes(1), 'XLim', [-0.005 0.1]);
  set(haxes(2), 'XLim', [-0.005 0.1]);
  set(gca, 'box', 'off');
  % Add title.
  th = title({'Stiff Robertson Problem','Using BDF with Modified Newton'});
  set(th,'position',[0.05,0.004]);
  % Label the x axis, and both y axes.
  xlabel('x');
  ylabel(haxes(1), 'Solution (100*b,c)');
  ylabel(haxes(2), 'Solution (a)');
  % Add a legend.
  legend('c', '100*b', 'a', 'Location', 'East');
  % Set some features of the three lines
  set(hline1, 'Linewidth', 0.5, 'Marker', '+', 'LineStyle', '-', ...
     'Color', 'Magenta');
  set(hline2, 'Linewidth', 0.5, 'Marker', '*', 'LineStyle', ':');
  set(hline3, 'Linewidth', 0.5, 'Marker', 'x', 'LineStyle', '--');
  hold off;
