function g05xd_example

% diary g05xd_example.r
fprintf('g05xd example results\n\n');

% We wish to solve the stochastic differential equation (SDE)
%   dSt = r*St*dt + sigma*St*dXt
% where X is a one dimensional Wiener process.
% This means we have
%     a = 0,   d = 1,  c = 1
% Set the other parameters of the SDE and the Euler-Maruyama scheme

% Initial value of the process
s0    = 1;
r     = 0.05;
sigma = 0.12;

% Number of paths to simulate
npaths = nag_int(3);

% The time interval [t0,T] on which to solve the SDE
t0   = 0;
tend = 1;

% The time steps in the discretization of [t0,T]
tn = 20;
t = t0 + (1:tn)*(tend-t0)/(tn+1);

% Make the bridge construction order
bgord = nag_int(3);
move  = zeros(0, 1, nag_int_name);
[times, ifail] = g05xe( ...
                        t0, tend, t, move, 'bgord', bgord);

% Generate the Z values
d = 1;
a = nag_int(0);
[z] = get_z(npaths, d, a, tn);

% Initialize the generator
[rcomm, ifail] = g05xc( ...
                        t0, tend, times);

% Get the scaled increments of the Wiener process
dif = zeros(d, 1);
c   = ones(d, 1);
[z, b, ifail] = g05xd( ...
                       npaths, dif, z, c, rcomm, 'a', a);

% Do the Euler-Maruyama time stepping
st = zeros(tn+1, npaths);

% Do first time step
st(1,:) = s0 + (t(1)-t0)*(r*s0+sigma*s0*b(1, :));
v = st(1,:);
for i=2:tn
  v = v + (t(i)-t(i-1))*(r*v+sigma*v.*b(i,:));
  st(i,:) = v;
end
% Do last time step
st(tn+1,:) = v + (tend-t(tn))*(r*v+sigma*v.*b(tn+1,:));

% Compute the analytic solution:
%    ST = S0*exp( (r-sigma^2/2)T + sigma WT )
analytic = s0*exp((r-0.5*sigma^2)*tend+sigma*sqrt(tend-t0)*z(1,:));

% Display the results
fprintf('\nEuler-Maruyama solution for Geometric Brownian motion\n');
fprintf('        Path 1     Path 2     Path 3\n');
for i = 1:tn+1
  fprintf('%2d  %10.4f %10.4f %10.4f\n', i, st(i, :));
end
fprintf('\nAnalytic solution at final time step\n');
fprintf('        Path 1     Path 2     Path 3\n');
fprintf('    %10.4f %10.4f %10.4f\n', analytic);

% diary off

function [z] = get_z(npaths, d, a, ntimes)
  idim = d*(ntimes+1-a);

  % We now need to generate the input pseudorandom points

  % First initialize the base pseudorandom number generator
  state = initialize_prng(nag_int(6), nag_int(0), [nag_int(1023401)]);

  % Scrambled quasi-random sequences preserve the good discrepancy
  % properties of quasi-random sequences while counteracting the bias
  % some applications experience when using quasi-random sequences.
  % Initialize the scrambled quasi-random generator.
  [iref, state] = initialize_scrambled_qrng(nag_int(1), nag_int(2), ...
                                            idim, state);

  % Generate the quasi-random points from N(0,1)
  xmean = zeros(idim, 1);
  std   = ones(idim, 1);
  [z, iref, ifail] = g05yj( ...
                            xmean, std, npaths, iref);
  z = z';

function [state] = initialize_prng(genid, subid, seed)
  % Initialize the generator to a repeatable sequence
  [state, ifail] = g05kf( ...
                          genid, subid, seed);


function [iref, state] = initialize_scrambled_qrng(genid,stype,idim,state)
  iskip = nag_int(0);
  nsdigits = nag_int(32);
  [iref, state, ifail] = g05yn( ...
                                genid, stype, nag_int(idim), ...
                                iskip, nsdigits, state);
