function nag_ode_bvp_coll_nlin_contin_example

% diary nag_ode_bvp_coll_nlin_contin_example.r
fprintf('nag_ode_bvp_coll_nlin_contin example results\n\n');

global en s el; % For communication with local functions

% Initialize variables and arrays.
neq  = nag_int(2);
nlbc = nag_int(3);
nrbc = nag_int(2);
ncol = nag_int(6);
mmax = nag_int(3);
m    = nag_int([3; 2]);
tols = [0.00001; 0.00001];

% Set values for problem-specific physical parameters.
el = 60;
s  = 0.24;
en = 0.2;

% Set up the mesh.
nmesh  = nag_int(21);
mxmesh = nag_int(250);
mesh = zeros(mxmesh,1);
ipmesh = zeros(mxmesh,1,nag_int_name);

% Set initial mesh with constant spacing.
dx = 1/double(nmesh-1);
mesh(1:nmesh) = [0:dx:1];

% Specify mesh end points as fixed.
ipmesh(1)         = 1;
ipmesh(2:nmesh-1) = 2;
ipmesh(nmesh)     = 1;

% Prepare to store results for plotting.
xarray = zeros(1,1);
yarray1 = zeros(1,1);
yarray2 = zeros(1,1);

% nag_ode_bvp_coll_nlin_setup is a setup routine to be called prior to nag_ode_bvp_coll_nlin.
[work, iwork, ifail] = nag_ode_bvp_coll_nlin_setup(...
                              m, nlbc, nrbc, ncol, tols, nmesh, mesh, ipmesh);

ncont = 3;

% We run through the calculation three times with different parameter sets.
for jcont = 1:ncont
  fprintf('\n Tolerance = %8.1e, L = %8.3f, S = %6.4f\n\n', tols(1), el, s);
  
  % Call nag_ode_bvp_coll_nlin to solve BVP for this set of parameters.
  [work, iwork, ifail] = nag_ode_bvp_coll_nlin(...
                               @ffun, @fjac, @gafun, @gbfun, @gajac, ...
                               @gbjac, @guess, work, iwork);

  % Call nag_ode_bvp_coll_nlin_diag to extract mesh from solution.
  [nmesh, mesh, ipmesh, ermx, iermx, ijermx, ifail] = ...
  nag_ode_bvp_coll_nlin_diag( ...
        mxmesh, work, iwork);

  % Output mesh results.
  fprintf(' Used a mesh of %d points\n', nmesh);
  fprintf(' Maximum error = %10.2e in interval %d for component %d\n\n',...
          ermx, iermx, ijermx);

  % Output solution, and store it for plotting.
  fprintf(' Solution on original interval:\n');
  fprintf('    x          f           g\n');
  for i=1:16
    xx = double(i-1)*2/el;

    % Call nag_ode_bvp_coll_nlin_interp to perform interpolation on the solution.
    [y, work, ifail] = nag_ode_bvp_coll_nlin_interp(...
                              xx, neq, mmax, work, iwork);
    fprintf(' %6.2f    %8.4f    %8.4f\n', xx*el, y(1,1), y(2,1));
    xarray(i) = xx*el;
    yarray1(i) = y(1,1);
    yarray2(i) = y(2,1);
  end
  for i=1:10
    xx = (30+(el-30)*double(i)/10)/el;

    [y, work, ifail] = nag_ode_bvp_coll_nlin_interp(...
                             xx, neq, mmax, work, iwork);
    fprintf(' %6.2f    %8.4f    %8.4f\n', xx*el, y(1,1), y(2,1));
    xarray(i+16) = xx*el;
    yarray1(i+16) = y(1,1);
    yarray2(i+16) = y(2,1);
  end

  % Plot results for this parameter set.
  if (jcont==1)
    fig1 = figure;
  elseif (jcont==2)
    fig2 = figure;
  else
    fig3 = figure;
  end    
  display_plot(xarray, yarray1, yarray2, el, s)

  % Select mesh for next calculation.
  if jcont < ncont
    el = 2*el;
    s = 0.6*s;
    nmesh = (nmesh+1)/2;

    % nag_ode_bvp_coll_nlin_contin allows the current solution to be used as an initial
    % approximation to the solution of a related problem.
    [work, iwork, ifail] = nag_ode_bvp_coll_nlin_contin(...
                                  nmesh, mesh, ipmesh, work, iwork);
  end
end

% print(fig1,'-dpng','-r75','nag_ode_bvp_coll_nlin_contin_fig1.png');
% print(fig1,'-deps','-r75','nag_ode_bvp_coll_nlin_contin_fig1.eps');
% print(fig2,'-dpng','-r75','nag_ode_bvp_coll_nlin_contin_fig2.png');
% print(fig2,'-deps','-r75','nag_ode_bvp_coll_nlin_contin_fig2.eps');
% print(fig3,'-dpng','-r75','nag_ode_bvp_coll_nlin_contin_fig3.png');
% print(fig3,'-deps','-r75','nag_ode_bvp_coll_nlin_contin_fig3.eps');

% diary off

function [f] = ffun(x, y, neq, m)
  % Evaluate derivative functions (rhs of system of ODEs).

  global en s el
  f = zeros(neq, 1);
  f(1) = el^3*(1-y(2,1)^2) + el^2*s*y(1,2) - ...
         el*(0.5*(3-en)*y(1,1)*y(1,3) + en*y(1,2)^2);
  f(2) = el^2*s*(y(2,1)-1) - ...
         el*(0.5*(3-en)*y(1,1)*y(2,2) + (en-1)*y(1,2)*y(2,1));

function [dfdy] = fjac(x, y, neq, m)
  % Evaluate Jacobians (partial derivatives of f).
         
  global en s el
  dfdy = zeros(neq, neq, 3);
  dfdy(1,2,1) = -2*el^3*y(2,1);
  dfdy(1,1,1) = -el*0.5*(3-en)*y(1,3);
  dfdy(1,1,2) = el^2*s - el*2*en*y(1,2);
  dfdy(1,1,3) = -el*0.5*(3-en)*y(1,1);
  dfdy(2,2,1) = el^2*s - el*(en-1)*y(1,2);
  dfdy(2,2,2) = -el*0.5*(3-en)*y(1,1);
  dfdy(2,1,1) = -el*0.5*(3-en)*y(2,2);
  dfdy(2,1,2) = -el*(en-1)*y(2,1);

function [ga] = gafun(ya, neq, m, nlbc)
  % Evaluate boundary conditions at left-hand end of range.

  global en s el
  ga = zeros(nlbc, 1);
  ga(1) = ya(1,1);
  ga(2) = ya(1,2);
  ga(3) = ya(2,1);

function [dgady] = gajac(ya, neq, m, nlbc)
  % Evaluate Jacobians (partial derivatives of ga).

  global en s el
  dgady = zeros(nlbc, neq, 3);
  dgady(1,1,1) = 1;
  dgady(2,1,2) = 1;
  dgady(3,2,1) = 1;

function [gb] = gbfun(yb, neq, m, nrbc)
  % Evaluate boundary conditions at right-hand end of range.

  global en s el
  gb = zeros(nrbc, 1);
  gb(1) = yb(1,2);
  gb(2) = yb(2,1) - 1;

function [dgbdy] = gbjac(yb, neq, m, nrbc)
  % Evaluate Jacobians (partial derivatives of gb).

  global en s el
  dgbdy = zeros(nrbc, neq, 3);
  dgbdy(1,1,2) = 1;
  dgbdy(2,2,1) = 1;

function [y, dym] = guess(x, neq, m)
  % Evaluate initial approximations to solution components and derivatives.

  global en s el
  y = zeros(neq, 3);
  dym = zeros(neq, 1);
  ex = x*el;
  expmx = exp(-ex);
  y(1,1) = -ex^2*expmx;
  y(1,2) = (-2*ex+ex^2)*expmx;
  y(1,3) = (-2+4*ex-ex^2)*expmx;
  y(2,1) = 1 - expmx;
  y(2,2) = expmx;
  dym(1) = (6-6*ex+ex^2)*expmx;
  dym(2) = -expmx;

function display_plot(xarray,yarray1,yarray2,el,s)
  % Formatting for title and axis labels.
  % Plot the results.
  plot(xarray, yarray1, '-+', xarray, yarray2, '--x');
  % Add title.
  title({['Swirling Flow over Disc in Axial Magnetic Field '], ...
         ['with L = ',num2str(el), ' and s = ',num2str(s)]})
  % Label the axes.
  xlabel('Radial Distance from Magnetic Field');
  ylabel('Velocities');
  % Add a legend.
  legend('axial velocity','tangential velocity','Location','Best')
