function nag_pde_1d_parab_coll_example

% diary nag_pde_1d_parab_coll_example.r
fprintf('nag_pde_1d_parab_coll example results\n\n');

% Solution of an elliptic-parabolic pair of PDEs
% (derived from a fourth-order PDE).

% Set values for problem parameters.
npde = 2;

% Number of points on interpolated mesh, number of break points.
intpts = 6;
nbkpts = 10;

% Order of Chebyshev polynomial.
npoly = nag_int(3);

npts   = (nbkpts-1)*npoly + 1;
lisave = npde*npts + 24;

% Define some arrays.
rsave   = zeros(4000, 1);
u       = zeros(npde, npts);
isave   = zeros(lisave, 1, nag_int_name);
lwsav   = false(100, 1);
iwsav   = zeros(505, 1, nag_int_name);
rwsav   = zeros(1100, 1);
cwsav   = {''; ''; ''; ''; ''; ''; ''; ''; ''; ''};
% Set up the points on the interpolation grid.
xinterp = [-1:0.4:1];


% Number of output points in time.
niter  = 20;

% Prepare to store plotting results.
tsav   = zeros(niter, 1);
usav   = zeros(2, niter, npts);
isav   = 0;

% Set the break points.
hx     = 2/(nbkpts-1);
xbkpts = [-1:hx:1];

% Set initial conditions.
m      = nag_int(0);
ts     = 0.0;
tout   = 0.1e-4;
acc    = 1.0e-4;
itask  = nag_int(1);
itrace = nag_int(0);
ind0   = nag_int(0);

alpha  = -log(tout)/(niter-1);

% Output algorithmic details and interpolation points.
fprintf('polynomial degree = %4d    no. of elements = %4d\n', npoly, nbkpts-1);
fprintf('accuracy requirement = %10.3e    number of points = %5d', acc, npts);
fprintf('\n\n t / x       ');
fprintf('%8.4f', xinterp);
fprintf('\n\n');

% Loop over exponentially increasing endpoints of integration 0.0001 --> 1. 
% Set itask = 1: normal computation of output values at t = tout.
for iter = 1:niter

  tout = exp(alpha*(iter - niter));

  % (re)start integration in time: ind0=0.
  [ts, u, x, rsave, isave, ind, user, cwsav, lwsav, iwsav, rwsav, ifail] = ...
  nag_pde_1d_parab_coll( ...
         m, ts, tout, @pdedef, ...
         @bndary, u, xbkpts, npoly, @uinit, ...
         acc, rsave, isave, itask, itrace, ind0, ...
         cwsav, lwsav, iwsav, rwsav);

  % Interpolation onto coarser grid for display
  itype  = nag_int(1);
  [uinterp, rsave, ifail] = nag_pde_1d_parab_coll_interp( ...
                                   u, xbkpts, npoly, xinterp, itype, rsave);

  % Output interpolated results for this time step.
  fprintf('%7.4f  u(1)', ts);
  fprintf('%8.4f', uinterp(1,:,1));
  fprintf('\n         u(2)');
  fprintf('%8.4f', uinterp(2,:,1));
  fprintf('\n\n');

  % Save this timestep for plotting.
  isav = isav+1;
  tsav(isav) = ts;
  usav(1:2,isav,1:npts) = u(1:2,1:npts);
end

% Output some statistics.
fprintf(' Number of integration steps in time = %6d\n', isave(1));
fprintf(' Number of function evaluations      = %6d\n', isave(2));
fprintf(' Number of Jacobian evaluations      = %6d\n', isave(3));
fprintf(' Number of iterations                = %6d\n', isave(5));

% Plot results.
fig1 = figure;
plot_results(x, tsav, squeeze(usav(1,:,:)), 'u_1');
% print(fig1,'-dpng','-r75','nag_pde_1d_parab_coll_fig1.png');
% print(fig1,'-deps','-r75','nag_pde_1d_parab_coll_fig1.eps');
fig2 = figure;
plot_results(x, tsav, squeeze(usav(2,:,:)), 'u_2');
% print(fig2,'-dpng','-r75','nag_pde_1d_parab_coll_fig2.png');
% print(fig2,'-deps','-r75','nag_pde_1d_parab_coll_fig2.eps');

% diary off

function [p, q, r, ires, user] = pdedef(npde, t, x, nptl, u, ux, ires, user)
  p = zeros(npde, npde, nptl);
  q = zeros(npde, nptl);
  r = zeros(npde, nptl);

  for i = 1:double(nptl)
    q(1,i) = u(2,i);
    q(2,i) = u(1,i)*ux(2,i) - ux(1,i)*u(2,i);
    r(1,i) = ux(1,i);
    r(2,i) = ux(2,i);
    p(2,2,i) = 1;
  end;


function [beta, gamma, ires, user] = bndary(npde, t, u, ux, ibnd, ires, user)
  beta  = zeros(npde, 1);
  gamma = zeros(npde, 1);

  beta(1)  = 1;
  beta(2)  = 0;
  gamma(1) = 0;
  if (ibnd == 0)
    gamma(2) = u(1) - 1;
  else
    gamma(2) = u(1) + 1;
  end


function [u, user] = uinit(npde, npts, x, user)
  u = zeros(npde, npts);

  piby2 = pi/2;
  u(1,:) = -sin(piby2*x);
  u(2,:) = -piby2*piby2*u(1,:);


function plot_results(x, t, u, ident)
  % Plot array as a mesh.
  mesh(x, t, u);
  set(gca, 'YScale', 'log');
  set(gca, 'YTick', [0.00001 0.0001 0.001 0.01 0.1 1]);
  set(gca, 'YMinorGrid', 'off');
  set(gca, 'YMinorTick', 'off');

  % Label the axes, and set the title.
  xlabel('x');
  ylabel('t');
  zlabel([ident,'(x,t)']);
  title({['Solution ',ident,' of elliptic-parabolic pair'], ...
      'using Chebyshev Collocation and BDF'});

  % Set the axes limits tight to the x and y range.
  axis([x(1) x(end) t(1) t(end)]);

  % Set the view to something nice (determined empirically).
  view(-165, 44);
