%% NAG Toolbox for MATLAB Demo for the Travelling Salesman Problem
% The NAG Toolbox routine nag_mip_tsp_simann (h03bb) calculates an
% approximate solution to a symmetric travelling salesman problem
% using simulated annealing via a configuration free interface.
%
% This demo illustrates such the use of the routine to find a solution
% for a set of cities in the UK and Ireland.
%
%% The Classical Travelling Salesman Problem
% Consider a salesman who has to visit a set of cities once each
% in a round trip in order to meet customers in each of these cities.
% There are many routes that can achieve this task, but there is likely
% only one (and its reverse) that will minimize the cost of that trip.
%
% In simplest terms the cost will just be the total distance travelled,
% that being the sum of the distances travelled from one city to the next.
% However, in the problem framework, the cost of travelling from one city
% to another can be any positive value, which might be a function of many
% things.
%
% For the purposes of this demo, the matrix of costs of travelling form any
% city to any other is simply the distance matrix computed from the city
% longitudes and latitudes.

%% Map of UK and Ireland and City List
% travelling_salesman_demo.m contains a list of cities and their
% longitudes and latitudes. 
% This list can be added to by editing the file:
% * add city names to the end of the array *city_names*
% * add latitude and longitude (in that order) for each cities as extra
%   rows to the matrix *latlong*.
% You can select a subset of cities in the list to visit by changing the
% values in the index array *choose*. For example,
%%
% choose = [1 3 5 7 9 11 13 15 17 19 21];
%
% travelling_salesman_demo.m also contains details of the coastlines for
% the UK, Ireland and most of their associated islands. 
% A map of the islands is drawn and then overlayed with the chosen set of
% cities. 

%% nag_mip_tsp_simann (h03bb)
% This routine is called to find a cost-efficient route through the chosen
% cities, starting and ending at the home city (Oxford by default).
% This route is then displayed on the map.

%% Controls
% * Pause: the plotting of the map, cities and route can be paused at any
%          time by pressing the Pause button on the GUI
% * Resume: plotting may be resumed by pressing the same button after a
%           pause.
% * Stop: Pressing the stop button tells the demo to stop repeating the demo
%         at the end of the current cycle.




function travelling_salesman_demo(pausetime)
  if nargin<1 || isempty(pausetime)
    pausetime = 0.2;
  end
  pause on;
  lstop = 0;
  while lstop==0
    rolling = 1;
    [lstop] = travelling_salesman_single_demo(pausetime,rolling);
    pause(max(2*pausetime, 4));
    close all;
  end
end

function [lstop] = travelling_salesman_single_demo(pausetime, rolling)
  if nargin<1 || isempty(pausetime)
    pausetime = 0.2;
  end
  
  if nargin<2 || isempty(rolling)
    rolling = 0;
  end
  lstop = 0;

homecity = 'Oxford';
city_names = char(homecity,    'Dundee',    'Cardiff',     'Edinburgh', ...
                  'Swansea',   'Perth',     'Stirling',    'Bangor', ...
                  'Plymouth',  'Holyhead',  'Exeter',      'Glasgow', ...
                  'Newport',   'Inverness', 'St. Davids',  'Aberdeen', ...
                  'St. Asaph', 'Cambridge', 'Aberystwyth', 'Birmingham', ...
                  'Dublin');
latlong = [-1.251068 51.75339;           -2.988098 56.469719;
           -3.17909  51.481581;          -3.188267 55.953252;
           -3.943646 51.62144;           -3.430838 56.394994;
           -3.936903 56.116523;          -4.129263 53.22739;
           -4.142656 50.375457;          -4.633038 53.309441;
           -3.533899 50.718412;          -4.251806 55.864237;
           -2.997664 51.584151;          -4.224721 57.477773;
           -5.265995 51.881227;          -2.094278 57.149717;
           -3.441876 53.257713;           0.121817 52.205337;
           -4.08292  52.415303;          -1.890401 52.486243;
           -6.26031  53.349805];

% Set number of cities (including home city) to visit.

% Choose which cities in the list to visit (by index).
% E.g. choose = [1 3 5 7 9 11 13 15 17 19 21] visits every other city
% in the list
choose = 1:21;
nc = size(choose,2);

for j = 1:nc
  ch_names(j,:) = city_names(choose(j),:);
  ch_pos(j,:) = latlong(choose(j),:);
end

% The following is the default distance matrix used in example program
dm = zeros(nc,nc);

% Distance matrix data for cities
% The above can be replaced by a simple crow-flies distance matrix.
% This makes it easier to add and choose cities.
for j = 1:nc-1
  for k = j+1:nc
    dm(j,k) = 100*sqrt((ch_pos(j,1)-ch_pos(k,1))^2 + ...
                       (ch_pos(j,2)-ch_pos(k,2))^2);
  end
end

[uki] = coastlines;

% Set up figure
scrsz = get(0, 'ScreenSize');
fig1 = figure('Name', 'Travelling Salesman with NAG Toolbox for MATLAB', ...
              'Position', ...
              [scrsz(3)*0.2 scrsz(4)*0.2 scrsz(3)*0.5 scrsz(4)*0.6], ...
              'NumberTitle', 'off');
% Use double buffering to avoid flashing of animation
set(fig1, 'Renderer', 'Painters', 'DoubleBuffer', 'On');

title({'Traveling Salesman problem:';'Cities in UK and Ireland'});

setappdata(fig1,'lstop',0);
if ~rolling
    % Create a button and wait for it to be pressed before continuing
    hbutton = uicontrol('Position', [20 20 140 40], ...
        'String', 'Start', 'Callback', @my_button_handler);
    uiwait(gcf);
else
    % Create a pause button.
    hbutton = uicontrol('Position', [20 20 140 40], ...
        'String', 'Pause', 'Callback', @my_button_handler);
    % Create a stop button.
    hbutton = uicontrol('Position', [200 20 140 40], ...
        'String', 'Stop', 'Callback', @my_button_handler);
end

pause on;
hold on;
num_coast = size(uki,2);

for j = 1:num_coast
  
  a = uki(j).xy;
  plot(a(:,1),a(:,2),'-k');
  if j<82
    pause(pausetime/2)
  else
    pause(pausetime*1.5)
  end
end
for j = 1:nc
  a = ch_pos(j,:);
  hc(j) = plot(a(1),a(2),'*');
  cb = mod(j-1,3)/2.0;
  cg = mod(floor((j-1)/3),3)/2.0;
  cr = mod(floor((j-1)/9),3)/2.0;
  hc(j).Color = [cr, cg, cb];
  pause(pausetime)
end
xlabel('Latitude');
ylabel('Longitude');
legend(hc(1:nc),ch_names,'Location','eastoutside');

% Calculate a lower bound internally and try to find lowest cost path.
bound = -1;
targc = -1;

% Initialize the random number state array
genid = nag_int(2);
subid = nag_int(53);
seed = nag_int([304950,889934,209094,23423990]);
[state,ifail] = g05kf( ...
                       genid,subid,seed);

%  Find low cost return path through all cities
[path, cost, tmode, alg_stats, state, ifail] = ...
h03bb( ...
       dm, bound, targc, state);

path(nc+1) = 1;
for j = 1:nc
  plot([ch_pos(path(j),1) ch_pos(path(j+1),1)], ...
       [ch_pos(path(j),2) ch_pos(path(j+1),2)], ...
       '-r');
  drawnow;
  pause(2*pausetime);
end
hold off;
pause off;

lstop = getappdata(fig1,'lstop');
set(hbutton, 'String', 'Close');

end

function my_button_handler(hObject, eventData)
  global my_stop;
  label = get(hObject, 'String');
  if strcmp(label, 'Close')
    closereq;
  elseif strcmp(label, 'Pause')
    set(hObject, 'String', 'Resume');
    uiwait;
  elseif strcmp(label, 'Start') || strcmp(label, 'Resume')
    set(hObject, 'String', 'Pause');
    uiresume;
  elseif strcmp(label, 'Next')
    uiresume;
  elseif strcmp(label, 'Stop')
    setappdata(hObject.Parent,'lstop',1);
  end
end

function [uki] = coastlines
  uki(1).xy = [ -4.943056	55.725833;
                -4.945278	55.722222;
                -4.948333	55.719167;
                -4.949722	55.715000;
                -4.951111	55.710833;
                -4.954167	55.707778;
                -4.960000	55.706667;
                -4.964722	55.708889;
                -4.967222	55.711667;
                -4.968611	55.715556;
                -4.970556	55.718889;
                -4.971389	55.722500;
                -4.969167	55.726111;
                -4.964722	55.728056;
                -4.961111	55.730833;
                -4.956111	55.732222;
                -4.950833	55.733889;
                -4.946389	55.731944;
                -4.946111	55.727778;
                -4.943056	55.725833];
  
  uki(2).xy = [ -5.164444	55.916667;
                -5.160556	55.915278;
                -5.156111	55.913056;
                -5.153056	55.910278;
                -5.149444	55.907778;
                -5.144722	55.905833;
                -5.139444	55.904167;
                -5.135833	55.901667;
                -5.132778	55.898611;
                -5.130000	55.895833;
                -5.126944	55.893056;
                -5.123333	55.890556;
                -5.116944	55.889167;
                -5.111667	55.887500;
                -5.105833	55.885833;
                -5.100556	55.884444;
                -5.095000	55.882778;
                -5.089722	55.881111;
                -5.085000	55.878889;
                -5.080278	55.876944;
                -5.077500	55.874167;
                -5.076389	55.870278;
                -5.077778	55.866389;
                -5.079444	55.862222;
                -5.078056	55.858611;
                -5.076111	55.855278;
                -5.074167	55.851944;
                -5.071667	55.848889;
                -5.068611	55.846111;
                -5.065000	55.843611;
                -5.061389	55.841111;
                -5.055556	55.839444;
                -5.048611	55.838611;
                -5.042778	55.838889;
                -5.036389	55.837778;
                -5.035278	55.833889;
                -5.035833	55.829444;
                -5.035000	55.825556;
                -5.033056	55.822222;
                -5.031111	55.819167;
                -5.027222	55.816667;
                -5.024444	55.813611;
                -5.022500	55.810278;
                -5.020556	55.806944;
                -5.021111	55.802500;
                -5.021111	55.798333;
                -5.020000	55.794722;
                -5.018056	55.791389;
                -5.015000	55.788333;
                -5.012500	55.785556;
                -5.008889	55.783056;
                -5.006944	55.779722;
                -5.005556	55.776111;
                -5.005278	55.771944;
                -5.005278	55.767778;
                -5.007500	55.764167;
                -5.011111	55.761667;
                -5.015000	55.759167;
                -5.020000	55.757500;
                -5.024444	55.755278;
                -5.028056	55.752778;
                -5.030000	55.749167;
                -5.027222	55.746389;
                -5.022778	55.744167;
                -5.019167	55.741667;
                -5.014444	55.739722;
                -5.010833	55.737222;
                -5.008056	55.734444;
                -5.006944	55.730556;
                -5.009167	55.726944;
                -5.012778	55.724444;
                -5.016389	55.721944;
                -5.023056	55.721389;
                -5.030278	55.722222;
                -5.034722	55.724167;
                -5.038333	55.726667;
                -5.042222	55.729167;
                -5.045000	55.731944;
                -5.047778	55.735000;
                -5.049722	55.738056;
                -5.051667	55.741389;
                -5.053611	55.744722;
                -5.054722	55.748611;
                -5.055000	55.752778;
                -5.056944	55.756111;
                -5.060556	55.758333;
                -5.066944	55.759722;
                -5.072222	55.761389;
                -5.076944	55.763333;
                -5.079722	55.766389;
                -5.083611	55.768611;
                -5.088889	55.770278;
                -5.096111	55.771111;
                -5.103611	55.771944;
                -5.110000	55.773056;
                -5.114444	55.775278;
                -5.118889	55.777222;
                -5.121944	55.780278;
                -5.122778	55.783889;
                -5.123056	55.788056;
                -5.123333	55.792222;
                -5.124167	55.795833;
                -5.127222	55.798889;
                -5.130000	55.801667;
                -5.133889	55.804167;
                -5.136667	55.806944;
                -5.137778	55.810833;
                -5.138611	55.814444;
                -5.140000	55.818333;
                -5.141944	55.821667;
                -5.142222	55.825556;
                -5.141667	55.829444;
                -5.138333	55.832500;
                -5.137222	55.836389;
                -5.139167	55.839722;
                -5.141944	55.842778;
                -5.148333	55.843889;
                -5.155833	55.843889;
                -5.162222	55.845000;
                -5.165833	55.847500;
                -5.170278	55.849444;
                -5.173333	55.852500;
                -5.176944	55.854722;
                -5.178889	55.858056;
                -5.181667	55.861111;
                -5.185556	55.863611;
                -5.189167	55.866111;
                -5.192778	55.868333;
                -5.195833	55.871389;
                -5.198611	55.874167;
                -5.200556	55.877500;
                -5.201667	55.881389;
                -5.202500	55.885000;
                -5.203889	55.888611;
                -5.205833	55.891944;
                -5.206667	55.895833;
                -5.206389	55.899444;
                -5.201944	55.901389;
                -5.197500	55.903611;
                -5.193056	55.905556;
                -5.188611	55.907778;
                -5.185833	55.910833;
                -5.183611	55.914444;
                -5.181667	55.918056;
                -5.176944	55.920000;
                -5.170556	55.918889;
                -5.164444	55.916667];
  
  uki(3).xy = [ -5.158056	55.800833;
                -5.156111	55.797500;
                -5.153056	55.794444;
                -5.150278	55.791667;
                -5.147500	55.788611;
                -5.145556	55.785556;
                -5.144444	55.781667;
                -5.145833	55.777778;
                -5.150278	55.775556;
                -5.155833	55.776389;
                -5.160278	55.778333;
                -5.163333	55.781111;
                -5.165278	55.784444;
                -5.168333	55.787500;
                -5.170278	55.790833;
                -5.171111	55.794444;
                -5.172500	55.798056;
                -5.170833	55.802222;
                -5.166389	55.804444;
                -5.160000	55.803889;
                -5.158056	55.800833];
  
  uki(4).xy = [ -5.063333	55.528889;
                -5.061389	55.526667;
                -5.058333	55.523611;
                -5.055556	55.520833;
                -5.052778	55.517778;
                -5.049722	55.515000;
                -5.048889	55.511389;
                -5.049444	55.507500;
                -5.055278	55.506667;
                -5.059722	55.508611;
                -5.063611	55.511111;
                -5.066389	55.513889;
                -5.069167	55.516944;
                -5.071944	55.519722;
                -5.074722	55.522500;
                -5.076667	55.525833;
                -5.076944	55.530000;
                -5.071667	55.531667;
                -5.066389	55.530000;
                -5.063333	55.528889];
  
  uki(5).xy = [ -5.265556	55.717222;
                -5.260000	55.716389;
                -5.251944	55.716111;
                -5.245556	55.714722;
                -5.239167	55.713611;
                -5.233889	55.711944;
                -5.228333	55.710278;
                -5.223889	55.708333;
                -5.219167	55.706111;
                -5.215556	55.703889;
                -5.210833	55.701667;
                -5.204722	55.700556;
                -5.198333	55.699167;
                -5.193611	55.697222;
                -5.190000	55.694722;
                -5.185278	55.692778;
                -5.181667	55.690278;
                -5.178056	55.687778;
                -5.173333	55.685833;
                -5.168056	55.684167;
                -5.164167	55.681667;
                -5.160278	55.679167;
                -5.157778	55.676389;
                -5.154722	55.673333;
                -5.152778	55.670000;
                -5.150833	55.666667;
                -5.150000	55.663056;
                -5.148611	55.659444;
                -5.148333	55.655278;
                -5.148333	55.651111;
                -5.145556	55.648056;
                -5.142500	55.645278;
                -5.140556	55.641944;
                -5.140556	55.637778;
                -5.138333	55.634444;
                -5.134722	55.631944;
                -5.131111	55.629444;
                -5.130000	55.625833;
                -5.130000	55.621667;
                -5.130556	55.617222;
                -5.129444	55.613333;
                -5.129444	55.609167;
                -5.129167	55.605000;
                -5.131944	55.601944;
                -5.135833	55.598889;
                -5.139444	55.596389;
                -5.142222	55.593333;
                -5.142778	55.588889;
                -5.140833	55.585556;
                -5.138056	55.582500;
                -5.132500	55.580833;
                -5.127222	55.579444;
                -5.120000	55.578056;
                -5.113889	55.576944;
                -5.108333	55.575278;
                -5.103889	55.573056;
                -5.100000	55.570556;
                -5.097222	55.567778;
                -5.095278	55.564444;
                -5.092500	55.561667;
                -5.088889	55.559167;
                -5.085278	55.556667;
                -5.083889	55.553056;
                -5.084722	55.548333;
                -5.087778	55.545278;
                -5.092778	55.543611;
                -5.097778	55.542222;
                -5.103889	55.541111;
                -5.108611	55.539444;
                -5.114444	55.538333;
                -5.118889	55.536111;
                -5.122500	55.533611;
                -5.123889	55.529444;
                -5.123056	55.525833;
                -5.120000	55.523056;
                -5.116389	55.520556;
                -5.111944	55.518333;
                -5.106389	55.516667;
                -5.098333	55.516389;
                -5.090278	55.516111;
                -5.083333	55.515278;
                -5.077500	55.513611;
                -5.075000	55.510833;
                -5.073611	55.506944;
                -5.075833	55.503333;
                -5.080278	55.501389;
                -5.084722	55.499167;
                -5.088333	55.496667;
                -5.091389	55.493611;
                -5.093889	55.490556;
                -5.095556	55.486389;
                -5.096944	55.482222;
                -5.095833	55.478611;
                -5.092222	55.476111;
                -5.088611	55.473611;
                -5.084722	55.471111;
                -5.080278	55.469167;
                -5.076667	55.466667;
                -5.074722	55.463333;
                -5.074444	55.459167;
                -5.075833	55.455278;
                -5.078889	55.452222;
                -5.082500	55.449444;
                -5.085833	55.446944;
                -5.089722	55.444444;
                -5.093333	55.441667;
                -5.098333	55.440278;
                -5.103611	55.438611;
                -5.110000	55.438056;
                -5.117500	55.437778;
                -5.124444	55.437778;
                -5.131111	55.437222;
                -5.137778	55.436389;
                -5.144167	55.435833;
                -5.150278	55.434722;
                -5.154444	55.432778;
                -5.159722	55.431111;
                -5.165556	55.430000;
                -5.171944	55.429444;
                -5.179167	55.429444;
                -5.187222	55.429722;
                -5.194444	55.430556;
                -5.202500	55.430833;
                -5.210556	55.431111;
                -5.216944	55.430556;
                -5.223611	55.430000;
                -5.231667	55.430278;
                -5.236944	55.431944;
                -5.240833	55.434444;
                -5.245278	55.436667;
                -5.249167	55.438056;
                -5.253611	55.440278;
                -5.260000	55.441389;
                -5.266389	55.442500;
                -5.272778	55.443889;
                -5.278056	55.445278;
                -5.283333	55.446944;
                -5.288056	55.449167;
                -5.292778	55.451111;
                -5.296389	55.453611;
                -5.300000	55.456111;
                -5.303611	55.458611;
                -5.307500	55.461111;
                -5.311111	55.463333;
                -5.315000	55.465833;
                -5.318611	55.468333;
                -5.322222	55.470833;
                -5.327500	55.472500;
                -5.333889	55.473611;
                -5.340278	55.475000;
                -5.342222	55.477222;
                -5.339722	55.480278;
                -5.341667	55.483611;
                -5.341111	55.487222;
                -5.337500	55.490000;
                -5.335278	55.493611;
                -5.338889	55.495833;
                -5.344444	55.497500;
                -5.349722	55.499167;
                -5.354444	55.501111;
                -5.357222	55.504167;
                -5.358333	55.507778;
                -5.356944	55.511944;
                -5.354722	55.515556;
                -5.354167	55.520000;
                -5.355278	55.523889;
                -5.357222	55.527222;
                -5.357500	55.531111;
                -5.353889	55.533889;
                -5.350833	55.536944;
                -5.348889	55.540556;
                -5.348056	55.545000;
                -5.350000	55.548333;
                -5.351389	55.551944;
                -5.353333	55.555278;
                -5.355833	55.558333;
                -5.358889	55.561111;
                -5.361667	55.563889;
                -5.364444	55.566944;
                -5.367500	55.569722;
                -5.370000	55.572778;
                -5.373889	55.575000;
                -5.377778	55.577500;
                -5.380278	55.580556;
                -5.383333	55.583333;
                -5.386111	55.586111;
                -5.388889	55.589167;
                -5.392778	55.591667;
                -5.395556	55.594444;
                -5.398333	55.597222;
                -5.400278	55.600556;
                -5.400556	55.604722;
                -5.400833	55.608889;
                -5.400278	55.613611;
                -5.400278	55.617778;
                -5.399167	55.620833;
                -5.397778	55.625000;
                -5.396389	55.628889;
                -5.394167	55.632500;
                -5.392778	55.636667;
                -5.390556	55.640278;
                -5.388611	55.643611;
                -5.387222	55.647778;
                -5.386389	55.652222;
                -5.385278	55.656389;
                -5.383889	55.660556;
                -5.382222	55.664444;
                -5.381111	55.668611;
                -5.378056	55.671667;
                -5.374444	55.674444;
                -5.370833	55.676944;
                -5.365000	55.678056;
                -5.360556	55.680000;
                -5.354722	55.681111;
                -5.349722	55.682778;
                -5.345278	55.685000;
                -5.341667	55.687500;
                -5.338889	55.690556;
                -5.335833	55.693611;
                -5.333056	55.696667;
                -5.330000	55.699722;
                -5.325556	55.701944;
                -5.319722	55.703056;
                -5.316667	55.702778;
                -5.309444	55.703056;
                -5.301944	55.702222;
                -5.294722	55.702500;
                -5.291111	55.705000;
                -5.291389	55.709167;
                -5.295000	55.711667;
                -5.297778	55.714444;
                -5.295278	55.716667;
                -5.287778	55.716667;
                -5.280278	55.716944;
                -5.273056	55.716944;
                -5.265556	55.717222];

  uki(6).xy = [ -5.562500	55.276944;
                -5.566111	55.275278;
                -5.571667	55.274167;
                -5.577500	55.273056;
                -5.581389	55.270278;
                -5.584722	55.267778;
                -5.589167	55.265556;
                -5.595833	55.265000;
                -5.601111	55.266667;
                -5.603889	55.269444;
                -5.604167	55.273611;
                -5.602778	55.277778;
                -5.599722	55.280833;
                -5.593333	55.281389;
                -5.585556	55.281111;
                -5.578889	55.281667;
                -5.572222	55.282500;
                -5.565000	55.282500;
                -5.560556	55.280556;
                -5.561111	55.276944;
                -5.562500	55.276944];

  uki(7).xy = [ -5.728333	55.726111;
                -5.724444	55.723611;
                -5.720833	55.721111;
                -5.717778	55.718333;
                -5.720000	55.714722;
                -5.721944	55.711111;
                -5.725556	55.708611;
                -5.730000	55.706389;
                -5.733611	55.703889;
                -5.730833	55.700833;
                -5.725000	55.701111;
                -5.721111	55.698611;
                -5.723333	55.695000;
                -5.726944	55.692500;
                -5.730556	55.690000;
                -5.733333	55.686667;
                -5.731389	55.683611;
                -5.728611	55.680556;
                -5.729722	55.677500;
                -5.733333	55.675000;
                -5.735278	55.671389;
                -5.731667	55.668889;
                -5.730556	55.665000;
                -5.731944	55.661111;
                -5.737778	55.660000;
                -5.744444	55.659167;
                -5.748889	55.657222;
                -5.751389	55.654167;
                -5.753611	55.650556;
                -5.757222	55.647778;
                -5.762500	55.646389;
                -5.768889	55.646667;
                -5.773333	55.648611;
                -5.775278	55.651944;
                -5.774167	55.655833;
                -5.771111	55.658889;
                -5.767500	55.661667;
                -5.764167	55.664167;
                -5.767222	55.666111;
                -5.774167	55.666944;
                -5.778889	55.668889;
                -5.781111	55.672222;
                -5.778889	55.675833;
                -5.775833	55.678889;
                -5.773056	55.681111;
                -5.770278	55.684167;
                -5.767500	55.687222;
                -5.765278	55.690833;
                -5.762500	55.693889;
                -5.759722	55.696944;
                -5.757500	55.700556;
                -5.755000	55.703889;
                -5.751111	55.706389;
                -5.747500	55.708889;
                -5.743889	55.711667;
                -5.745000	55.714444;
                -5.748889	55.716944;
                -5.751944	55.719722;
                -5.752222	55.723889;
                -5.746944	55.725556;
                -5.738889	55.725000;
                -5.728333	55.726111];


  uki(8).xy = [ -6.137222	55.917778;
                -6.136944	55.913611;
                -6.138056	55.909722;
                -6.140278	55.906111;
                -6.142222	55.902500;
                -6.144444	55.898889;
                -6.147222	55.895833;
                -6.149444	55.892222;
                -6.148889	55.888056;
                -6.146111	55.885278;
                -6.143056	55.882222;
                -6.139444	55.880000;
                -6.136389	55.876944;
                -6.133611	55.874167;
                -6.130556	55.871389;
                -6.129444	55.867500;
                -6.130000	55.863056;
                -6.130556	55.858333;
                -6.129444	55.854722;
                -6.126389	55.851944;
                -6.122778	55.849444;
                -6.118889	55.846944;
                -6.115833	55.844167;
                -6.113056	55.841389;
                -6.111944	55.837500;
                -6.110833	55.833889;
                -6.111389	55.829167;
                -6.111944	55.824722;
                -6.112500	55.820278;
                -6.113056	55.815556;
                -6.112778	55.811389;
                -6.111667	55.807778;
                -6.110278	55.803889;
                -6.108333	55.800833;
                -6.106389	55.797500;
                -6.105833	55.793333;
                -6.105000	55.789444;
                -6.101944	55.786667;
                -6.099167	55.783889;
                -6.095278	55.781389;
                -6.091667	55.778889;
                -6.086944	55.776944;
                -6.082222	55.775000;
                -6.076667	55.773333;
                -6.071944	55.771389;
                -6.069167	55.768333;
                -6.066111	55.765556;
                -6.063333	55.762778;
                -6.060556	55.759722;
                -6.057500	55.756944;
                -6.055556	55.753611;
                -6.053333	55.750278;
                -6.053889	55.745833;
                -6.056111	55.742222;
                -6.056667	55.738611;
                -6.054444	55.735278;
                -6.052500	55.732778;
                -6.050278	55.729722;
                -6.048333	55.726389;
                -6.044444	55.723889;
                -6.039167	55.722222;
                -6.034444	55.720278;
                -6.032500	55.716944;
                -6.033611	55.712778;
                -6.036667	55.709722;
                -6.038611	55.706111;
                -6.036667	55.703056;
                -6.033056	55.700556;
                -6.030833	55.697222;
                -6.032222	55.693056;
                -6.031667	55.688889;
                -6.028056	55.686389;
                -6.026111	55.683333;
                -6.028056	55.679722;
                -6.030278	55.676111;
                -6.033056	55.673056;
                -6.038056	55.671389;
                -6.045278	55.671111;
                -6.050556	55.669444;
                -6.055000	55.667500;
                -6.057500	55.664444;
                -6.060556	55.661111;
                -6.064167	55.658611;
                -6.070556	55.659722;
                -6.076944	55.660833;
                -6.080278	55.658333;
                -6.080000	55.654167;
                -6.078889	55.650556;
                -6.076944	55.647222;
                -6.076667	55.643056;
                -6.081111	55.640833;
                -6.087778	55.640278;
                -6.094722	55.641111;
                -6.101389	55.640278;
                -6.106389	55.638611;
                -6.111667	55.636944;
                -6.115833	55.635000;
                -6.120000	55.632778;
                -6.124444	55.630833;
                -6.128889	55.628611;
                -6.136111	55.628333;
                -6.141111	55.626667;
                -6.146111	55.625000;
                -6.152222	55.623889;
                -6.159444	55.623889;
                -6.167500	55.624167;
                -6.173056	55.625833;
                -6.178333	55.627222;
                -6.184167	55.628056;
                -6.188611	55.625833;
                -6.192500	55.623611;
                -6.198611	55.622500;
                -6.203056	55.624722;
                -6.206111	55.627500;
                -6.212500	55.628611;
                -6.218333	55.627500;
                -6.223333	55.625833;
                -6.226944	55.623333;
                -6.225833	55.619444;
                -6.224722	55.615833;
                -6.226667	55.612222;
                -6.230833	55.610000;
                -6.234722	55.607500;
                -6.238889	55.605278;
                -6.242500	55.602778;
                -6.244444	55.599167;
                -6.245833	55.595000;
                -6.246944	55.590833;
                -6.248889	55.587500;
                -6.255000	55.586111;
                -6.263056	55.586389;
                -6.266667	55.583889;
                -6.268611	55.580278;
                -6.273056	55.578056;
                -6.278056	55.576667;
                -6.284444	55.575833;
                -6.291389	55.575278;
                -6.298611	55.575000;
                -6.303889	55.576667;
                -6.309444	55.578056;
                -6.315000	55.579722;
                -6.320278	55.581389;
                -6.321667	55.584167;
                -6.325556	55.586667;
                -6.332778	55.587222;
                -6.340833	55.587500;
                -6.346111	55.589167;
                -6.348333	55.592500;
                -6.345556	55.595556;
                -6.342778	55.598611;
                -6.339722	55.601944;
                -6.340278	55.605833;
                -6.341389	55.609722;
                -6.341667	55.613889;
                -6.341389	55.618333;
                -6.339167	55.621944;
                -6.337222	55.625556;
                -6.331944	55.627222;
                -6.327500	55.629444;
                -6.325556	55.633056;
                -6.325278	55.636667;
                -6.322500	55.639722;
                -6.318889	55.642222;
                -6.315556	55.645000;
                -6.310278	55.646667;
                -6.303611	55.647222;
                -6.297222	55.647778;
                -6.289722	55.648056;
                -6.283333	55.648611;
                -6.278056	55.650278;
                -6.274444	55.653056;
                -6.273056	55.657222;
                -6.273611	55.661111;
                -6.276667	55.664167;
                -6.278611	55.667500;
                -6.280556	55.670556;
                -6.281667	55.674444;
                -6.283056	55.678056;
                -6.285000	55.681389;
                -6.287778	55.684167;
                -6.290833	55.687222;
                -6.293611	55.690000;
                -6.296667	55.692778;
                -6.299444	55.695556;
                -6.303333	55.698056;
                -6.306389	55.701111;
                -6.310000	55.703333;
                -6.313056	55.706389;
                -6.317500	55.708333;
                -6.322222	55.710278;
                -6.327500	55.711944;
                -6.335000	55.712500;
                -6.341389	55.713889;
                -6.345833	55.715833;
                -6.348889	55.718611;
                -6.350000	55.722222;
                -6.348889	55.725556;
                -6.346944	55.729167;
                -6.343333	55.731667;
                -6.339722	55.734444;
                -6.335556	55.736389;
                -6.331111	55.738611;
                -6.326944	55.740833;
                -6.321667	55.742500;
                -6.317500	55.744444;
                -6.313611	55.747222;
                -6.309444	55.749167;
                -6.304444	55.750833;
                -6.299444	55.752500;
                -6.292778	55.753333;
                -6.286944	55.754444;
                -6.281667	55.756111;
                -6.276667	55.757778;
                -6.272500	55.759722;
                -6.268056	55.761944;
                -6.266111	55.765556;
                -6.266389	55.769722;
                -6.271111	55.771667;
                -6.277222	55.772778;
                -6.284722	55.772778;
                -6.290556	55.771389;
                -6.297222	55.770833;
                -6.303611	55.770278;
                -6.310000	55.771389;
                -6.317500	55.771944;
                -6.325556	55.772222;
                -6.333056	55.772222;
                -6.340278	55.771944;
                -6.348333	55.772222;
                -6.355000	55.771667;
                -6.360000	55.770000;
                -6.362778	55.766944;
                -6.365000	55.763333;
                -6.367778	55.760278;
                -6.370000	55.756667;
                -6.371944	55.753056;
                -6.374722	55.750000;
                -6.377500	55.746667;
                -6.380000	55.743611;
                -6.382222	55.740000;
                -6.384167	55.736389;
                -6.386389	55.732778;
                -6.389167	55.729722;
                -6.391389	55.726111;
                -6.393333	55.722500;
                -6.396111	55.719444;
                -6.398056	55.715833;
                -6.400278	55.712222;
                -6.403056	55.709167;
                -6.406389	55.706667;
                -6.410833	55.704444;
                -6.414444	55.701667;
                -6.418611	55.699722;
                -6.422500	55.696944;
                -6.426389	55.695000;
                -6.430278	55.692222;
                -6.433611	55.689722;
                -6.438611	55.688056;
                -6.443889	55.686389;
                -6.448333	55.683889;
                -6.451944	55.681111;
                -6.455278	55.678611;
                -6.460556	55.676944;
                -6.465278	55.675278;
                -6.471111	55.674167;
                -6.476389	55.672500;
                -6.480833	55.670278;
                -6.485000	55.667778;
                -6.490833	55.666667;
                -6.497222	55.666667;
                -6.503611	55.667778;
                -6.510278	55.667222;
                -6.515278	55.665556;
                -6.521111	55.664444;
                -6.527500	55.665556;
                -6.529444	55.668889;
                -6.528333	55.673056;
                -6.525556	55.676111;
                -6.524444	55.680000;
                -6.525278	55.683889;
                -6.530000	55.685833;
                -6.533889	55.688333;
                -6.534444	55.691389;
                -6.531389	55.694722;
                -6.528056	55.697222;
                -6.524444	55.699722;
                -6.520278	55.701944;
                -6.516667	55.704722;
                -6.513889	55.707778;
                -6.512500	55.711667;
                -6.514722	55.715000;
                -6.515833	55.718889;
                -6.514722	55.722778;
                -6.512500	55.726389;
                -6.509167	55.729167;
                -6.505278	55.731667;
                -6.501111	55.733889;
                -6.496944	55.736111;
                -6.492500	55.738056;
                -6.487222	55.739722;
                -6.481667	55.740833;
                -6.476667	55.742500;
                -6.471389	55.744167;
                -6.471111	55.746944;
                -6.474722	55.749444;
                -6.472778	55.752222;
                -6.467778	55.753611;
                -6.463611	55.755833;
                -6.460556	55.758889;
                -6.459444	55.763056;
                -6.459722	55.767222;
                -6.460278	55.771389;
                -6.463056	55.774167;
                -6.467778	55.776111;
                -6.473056	55.777778;
                -6.478056	55.779722;
                -6.482778	55.781667;
                -6.486389	55.784167;
                -6.489167	55.786944;
                -6.491389	55.790278;
                -6.489167	55.793889;
                -6.484167	55.795556;
                -6.476944	55.795833;
                -6.471111	55.796944;
                -6.466111	55.798611;
                -6.463056	55.801667;
                -6.464444	55.805556;
                -6.465556	55.809167;
                -6.466667	55.812778;
                -6.468056	55.816667;
                -6.469167	55.820278;
                -6.468889	55.824722;
                -6.468333	55.829444;
                -6.467500	55.833889;
                -6.466389	55.838056;
                -6.464444	55.841667;
                -6.463056	55.843889;
                -6.460556	55.846944;
                -6.457500	55.850000;
                -6.452500	55.851667;
                -6.448333	55.853889;
                -6.441667	55.854444;
                -6.435833	55.855833;
                -6.429167	55.856389;
                -6.424444	55.855278;
                -6.419722	55.853333;
                -6.413889	55.853611;
                -6.408889	55.855278;
                -6.404444	55.857222;
                -6.398056	55.858056;
                -6.393611	55.860278;
                -6.390000	55.862778;
                -6.386389	55.865278;
                -6.381389	55.866944;
                -6.376111	55.868611;
                -6.371111	55.870278;
                -6.365833	55.871944;
                -6.361667	55.874167;
                -6.357500	55.876389;
                -6.353056	55.878333;
                -6.349444	55.881111;
                -6.345000	55.883056;
                -6.341111	55.881667;
                -6.340833	55.877500;
                -6.341944	55.873333;
                -6.343611	55.869444;
                -6.344722	55.865278;
                -6.346667	55.861667;
                -6.348056	55.857500;
                -6.349444	55.853611;
                -6.349722	55.848889;
                -6.351389	55.844722;
                -6.351944	55.840278;
                -6.352222	55.835833;
                -6.352778	55.831111;
                -6.352500	55.826944;
                -6.351389	55.823333;
                -6.348333	55.820556;
                -6.342778	55.818889;
                -6.337222	55.819167;
                -6.331944	55.820833;
                -6.329444	55.823889;
                -6.327778	55.827778;
                -6.329722	55.831111;
                -6.331944	55.834444;
                -6.333889	55.837778;
                -6.333611	55.842222;
                -6.331667	55.845833;
                -6.328611	55.849167;
                -6.325833	55.852222;
                -6.323056	55.855278;
                -6.319444	55.857778;
                -6.315833	55.860556;
                -6.315556	55.864167;
                -6.316667	55.867778;
                -6.312222	55.870000;
                -6.307222	55.871667;
                -6.301389	55.872778;
                -6.296944	55.875000;
                -6.293333	55.877500;
                -6.290556	55.880556;
                -6.288611	55.884167;
                -6.284167	55.886389;
                -6.279167	55.888056;
                -6.273889	55.889722;
                -6.268056	55.890833;
                -6.262222	55.891944;
                -6.256944	55.893611;
                -6.251944	55.895278;
                -6.246944	55.896944;
                -6.241667	55.898611;
                -6.237500	55.900556;
                -6.233889	55.903333;
                -6.229444	55.905278;
                -6.226111	55.908056;
                -6.222500	55.910556;
                -6.219444	55.913611;
                -6.216111	55.916389;
                -6.212500	55.918889;
                -6.205833	55.919722;
                -6.200556	55.921111;
                -6.196389	55.923333;
                -6.191111	55.925000;
                -6.183889	55.925278;
                -6.176389	55.925278;
                -6.171111	55.926944;
                -6.166111	55.928611;
                -6.161944	55.930833;
                -6.157500	55.932778;
                -6.151667	55.934167;
                -6.145000	55.934722;
                -6.137500	55.934722;
                -6.130278	55.934167;
                -6.127500	55.931389;
                -6.128056	55.926667;
                -6.133056	55.925000;
                -6.134167	55.921111;
                -6.137222	55.917778];


  uki(9).xy = [ -5.854444	55.958333;
                -5.853056	55.955556;
                -5.851111	55.952222;
                -5.849722	55.948611;
                -5.848889	55.945000;
                -5.848333	55.940833;
                -5.850556	55.937222;
                -5.853611	55.934167;
                -5.857222	55.931389;
                -5.860556	55.928889;
                -5.863611	55.925833;
                -5.865833	55.922222;
                -5.866944	55.918056;
                -5.869167	55.914444;
                -5.871389	55.910833;
                -5.873889	55.907778;
                -5.876111	55.904167;
                -5.879167	55.901111;
                -5.881111	55.897500;
                -5.880833	55.893333;
                -5.880556	55.889444;
                -5.880278	55.885278;
                -5.883889	55.882500;
                -5.888889	55.882778;
                -5.891667	55.885556;
                -5.894444	55.888611;
                -5.899722	55.886944;
                -5.902500	55.883889;
                -5.905556	55.880556;
                -5.906667	55.876667;
                -5.905556	55.872778;
                -5.903611	55.869722;
                -5.906389	55.866389;
                -5.912222	55.867222;
                -5.917778	55.868889;
                -5.924722	55.869444;
                -5.930556	55.868333;
                -5.935000	55.866389;
                -5.938056	55.863056;
                -5.940000	55.859722;
                -5.941389	55.855556;
                -5.944444	55.852500;
                -5.946944	55.849444;
                -5.950000	55.846389;
                -5.951944	55.842778;
                -5.953333	55.838611;
                -5.953889	55.835000;
                -5.951667	55.831667;
                -5.949722	55.828333;
                -5.947778	55.825000;
                -5.948333	55.820556;
                -5.951111	55.817500;
                -5.954167	55.814167;
                -5.957500	55.811667;
                -5.960556	55.808611;
                -5.962778	55.805000;
                -5.964722	55.801389;
                -5.966667	55.797778;
                -5.968056	55.793889;
                -5.969444	55.789722;
                -5.972778	55.786944;
                -5.979167	55.788333;
                -5.983056	55.790833;
                -5.987778	55.792778;
                -5.995833	55.793056;
                -6.003056	55.792778;
                -6.010556	55.792778;
                -6.018056	55.792500;
                -6.025278	55.792500;
                -6.032500	55.793056;
                -6.037222	55.795278;
                -6.041944	55.797222;
                -6.046667	55.799167;
                -6.050278	55.801667;
                -6.055833	55.803333;
                -6.062222	55.804444;
                -6.069444	55.805278;
                -6.075000	55.806667;
                -6.078889	55.809167;
                -6.082500	55.811667;
                -6.085556	55.814444;
                -6.088333	55.817222;
                -6.090278	55.820556;
                -6.092500	55.823889;
                -6.093611	55.827778;
                -6.095833	55.830833;
                -6.096667	55.834722;
                -6.099722	55.837500;
                -6.102500	55.840833;
                -6.104444	55.844167;
                -6.106667	55.847500;
                -6.105833	55.851944;
                -6.105833	55.855556;
                -6.104167	55.859722;
                -6.103056	55.863889;
                -6.101667	55.867778;
                -6.104444	55.870833;
                -6.107500	55.873611;
                -6.108611	55.877222;
                -6.110000	55.881111;
                -6.108611	55.885000;
                -6.107222	55.889167;
                -6.105000	55.892778;
                -6.102222	55.895833;
                -6.099444	55.898889;
                -6.095000	55.901111;
                -6.090833	55.903333;
                -6.086389	55.905278;
                -6.082778	55.908056;
                -6.080556	55.911667;
                -6.078611	55.915000;
                -6.075556	55.918333;
                -6.073611	55.921944;
                -6.071667	55.925278;
                -6.068889	55.928611;
                -6.065833	55.931667;
                -6.063056	55.934722;
                -6.058611	55.936944;
                -6.054444	55.938889;
                -6.050000	55.941111;
                -6.043889	55.942222;
                -6.038333	55.943333;
                -6.032222	55.944444;
                -6.026389	55.945556;
                -6.019722	55.946389;
                -6.013889	55.947500;
                -6.007222	55.948056;
                -6.000000	55.948333;
                -5.991667	55.948056;
                -5.985000	55.947778;
                -5.976944	55.947222;
                -5.970556	55.948056;
                -5.962500	55.947778;
                -5.955000	55.946944;
                -5.946667	55.946667;
                -5.939444	55.946667;
                -5.933611	55.947778;
                -5.926944	55.948611;
                -5.921111	55.949722;
                -5.915000	55.950833;
                -5.910278	55.952500;
                -5.905833	55.954444;
                -5.900000	55.955556;
                -5.893889	55.956944;
                -5.886389	55.956944;
                -5.880000	55.957778;
                -5.874722	55.959167;
                -5.870278	55.961389;
                -5.865278	55.963056;
                -5.858611	55.963611;
                -5.854722	55.961111;
                -5.854444	55.958333];


  uki(10).xy = [ -5.709444	56.140556;
                 -5.708889	56.136389;
                 -5.705278	56.133889;
                 -5.703333	56.130556;
                 -5.702222	56.126944;
                 -5.702778	56.122222;
                 -5.702500	56.118333;
                 -5.699722	56.115278;
                 -5.696667	56.112500;
                 -5.694444	56.109167;
                 -5.696111	56.105000;
                 -5.699722	56.102500;
                 -5.704167	56.100278;
                 -5.709167	56.098889;
                 -5.713611	56.096667;
                 -5.717222	56.094167;
                 -5.720278	56.090833;
                 -5.722500	56.087500;
                 -5.725000	56.084167;
                 -5.728056	56.081111;
                 -5.730833	56.078056;
                 -5.733889	56.075000;
                 -5.736944	56.071944;
                 -5.740556	56.069444;
                 -5.743056	56.066111;
                 -5.746944	56.063611;
                 -5.749722	56.060556;
                 -5.752778	56.057500;
                 -5.755556	56.054444;
                 -5.756944	56.050278;
                 -5.758333	56.046111;
                 -5.759444	56.042222;
                 -5.761111	56.038056;
                 -5.763056	56.034444;
                 -5.765000	56.030833;
                 -5.767222	56.027222;
                 -5.770278	56.024167;
                 -5.773056	56.021111;
                 -5.776667	56.018611;
                 -5.779444	56.015556;
                 -5.783333	56.012778;
                 -5.787500	56.010833;
                 -5.791389	56.008056;
                 -5.795000	56.005556;
                 -5.798611	56.002778;
                 -5.801389	55.999722;
                 -5.804167	55.996667;
                 -5.806389	55.993056;
                 -5.808333	55.989444;
                 -5.810556	55.986111;
                 -5.811944	55.981944;
                 -5.812500	55.977222;
                 -5.814722	55.973889;
                 -5.815833	55.969722;
                 -5.819444	55.967222;
                 -5.823889	55.965000;
                 -5.828333	55.962778;
                 -5.834167	55.961667;
                 -5.840833	55.961944;
                 -5.842778	55.965278;
                 -5.843889	55.969444;
                 -5.847778	55.971944;
                 -5.852222	55.973889;
                 -5.859722	55.974722;
                 -5.866111	55.975000;
                 -5.871389	55.973333;
                 -5.875833	55.971111;
                 -5.880000	55.969167;
                 -5.885278	55.967500;
                 -5.892500	55.967222;
                 -5.900556	55.967778;
                 -5.908333	55.967500;
                 -5.913333	55.965833;
                 -5.918333	55.964167;
                 -5.922778	55.962222;
                 -5.925556	55.959167;
                 -5.929167	55.956389;
                 -5.936389	55.956389;
                 -5.943333	55.955556;
                 -5.950000	55.955000;
                 -5.958056	55.955278;
                 -5.963611	55.956944;
                 -5.968889	55.958611;
                 -5.974722	55.960000;
                 -5.980278	55.961667;
                 -5.985000	55.963611;
                 -5.991111	55.965000;
                 -5.996944	55.966389;
                 -6.002222	55.968611;
                 -6.006111	55.971111;
                 -6.008056	55.974167;
                 -6.006944	55.978333;
                 -6.003889	55.981389;
                 -6.000278	55.984167;
                 -5.997222	55.987222;
                 -5.993889	55.989722;
                 -5.990833	55.992778;
                 -5.987222	55.995556;
                 -5.984444	55.998611;
                 -5.980833	56.001111;
                 -5.979444	56.005278;
                 -5.976667	56.008333;
                 -5.973889	56.011389;
                 -5.972500	56.015556;
                 -5.972778	56.019722;
                 -5.971389	56.023611;
                 -5.967778	56.026389;
                 -5.965000	56.028056;
                 -5.960000	56.029722;
                 -5.955556	56.031667;
                 -5.950556	56.033333;
                 -5.946667	56.036111;
                 -5.942500	56.038056;
                 -5.937222	56.039722;
                 -5.932778	56.041944;
                 -5.928333	56.043889;
                 -5.923333	56.045556;
                 -5.918889	56.047778;
                 -5.915556	56.050278;
                 -5.912500	56.053333;
                 -5.909722	56.056667;
                 -5.906667	56.059722;
                 -5.903056	56.062222;
                 -5.897222	56.063333;
                 -5.891389	56.064444;
                 -5.885556	56.065556;
                 -5.881111	56.067778;
                 -5.876667	56.070000;
                 -5.872222	56.071944;
                 -5.868611	56.074722;
                 -5.864167	56.076667;
                 -5.859167	56.078333;
                 -5.853889	56.080000;
                 -5.848889	56.081667;
                 -5.843889	56.083333;
                 -5.837222	56.083889;
                 -5.835000	56.087500;
                 -5.832778	56.091111;
                 -5.828333	56.093056;
                 -5.823889	56.095278;
                 -5.819444	56.097500;
                 -5.817500	56.100833;
                 -5.816944	56.105556;
                 -5.810278	56.106111;
                 -5.804722	56.104444;
                 -5.798056	56.104167;
                 -5.793056	56.105833;
                 -5.789444	56.108611;
                 -5.785000	56.110556;
                 -5.779722	56.112222;
                 -5.775278	56.114444;
                 -5.773056	56.118056;
                 -5.769167	56.120000;
                 -5.763056	56.121111;
                 -5.761667	56.125278;
                 -5.759444	56.128889;
                 -5.755833	56.131389;
                 -5.750833	56.133056;
                 -5.745000	56.134167;
                 -5.739167	56.132500;
                 -5.733611	56.131111;
                 -5.729167	56.133056;
                 -5.728056	56.137222;
                 -5.727500	56.140833;
                 -5.723889	56.143333;
                 -5.716667	56.142778;
                 -5.709444	56.140556];


  uki(11).xy = [ -5.595833	56.218611;
                 -5.594444	56.215000;
                 -5.593333	56.211389;
                 -5.593056	56.207222;
                 -5.592778	56.203056;
                 -5.594167	56.198889;
                 -5.598611	56.196944;
                 -5.603056	56.194722;
                 -5.607500	56.192778;
                 -5.611944	56.190556;
                 -5.611667	56.192222;
                 -5.615556	56.194722;
                 -5.618333	56.197778;
                 -5.618611	56.201944;
                 -5.618056	56.206389;
                 -5.617222	56.210833;
                 -5.615278	56.214444;
                 -5.612222	56.217500;
                 -5.609444	56.220833;
                 -5.604167	56.222222;
                 -5.597778	56.221111;
                 -5.595833	56.218611];
 
  uki(94).xy = [ -5.645278	56.259722;
                 -5.646111	56.255278;
                 -5.645556	56.251944;
                 -5.640833	56.250000;
                 -5.635278	56.248333;
                 -5.627222	56.248056;
                 -5.626111	56.244444;
                 -5.629722	56.241667;
                 -5.632500	56.238611;
                 -5.632222	56.234444;
                 -5.632778	56.230833;
                 -5.635833	56.227778;
                 -5.638611	56.224722;
                 -5.640000	56.220556;
                 -5.640556	56.216111;
                 -5.640278	56.211944;
                 -5.643333	56.208889;
                 -5.646944	56.206111;
                 -5.646389	56.202222;
                 -5.644444	56.198889;
                 -5.643333	56.195000;
                 -5.642222	56.191389;
                 -5.642222	56.187222;
                 -5.648611	56.186667;
                 -5.656111	56.187500;
                 -5.661667	56.188889;
                 -5.662500	56.192778;
                 -5.663056	56.196944;
                 -5.665000	56.200000;
                 -5.668056	56.203056;
                 -5.668889	56.206667;
                 -5.671111	56.210000;
                 -5.672222	56.213611;
                 -5.672500	56.217778;
                 -5.670278	56.221389;
                 -5.668333	56.225000;
                 -5.666111	56.228611;
                 -5.663056	56.231667;
                 -5.662500	56.236389;
                 -5.661944	56.240833;
                 -5.664167	56.244167;
                 -5.665000	56.247778;
                 -5.666389	56.251667;
                 -5.666389	56.255556;
                 -5.666111	56.259444;
                 -5.663056	56.262500;
                 -5.659444	56.265000;
                 -5.651944	56.265278;
                 -5.648056	56.262778;
                 -5.645278	56.259722];


  uki(12).xy = [ -5.597778	56.311111;
                 -5.598889	56.307778;
                 -5.600278	56.303889;
                 -5.601111	56.299167;
                 -5.601667	56.294722;
                 -5.601389	56.290556;
                 -5.601944	56.286111;
                 -5.603333	56.281944;
                 -5.605556	56.278333;
                 -5.608333	56.275278;
                 -5.611944	56.272778;
                 -5.615833	56.270000;
                 -5.620000	56.268056;
                 -5.627778	56.267778;
                 -5.634167	56.268889;
                 -5.638611	56.271111;
                 -5.642500	56.273611;
                 -5.644444	56.276667;
                 -5.646389	56.280000;
                 -5.647778	56.283889;
                 -5.651667	56.286389;
                 -5.657222	56.287778;
                 -5.664444	56.287778;
                 -5.670278	56.289444;
                 -5.667222	56.292778;
                 -5.664444	56.295000;
                 -5.660000	56.297222;
                 -5.655556	56.299167;
                 -5.651944	56.301944;
                 -5.648056	56.304444;
                 -5.646111	56.308056;
                 -5.642222	56.310556;
                 -5.639444	56.313889;
                 -5.633611	56.315000;
                 -5.629722	56.313333;
                 -5.625833	56.310833;
                 -5.621944	56.308333;
                 -5.615278	56.309167;
                 -5.610833	56.311111;
                 -5.607222	56.313889;
                 -5.604167	56.316944;
                 -5.599444	56.316667;
                 -5.598333	56.312778;
                 -5.597778	56.311111];


  uki(13).xy = [ -5.508889	56.406667;
                 -5.511111	56.403333;
                 -5.514722	56.400556;
                 -5.519167	56.398611;
                 -5.523056	56.395833;
                 -5.527222	56.393889;
                 -5.531111	56.391111;
                 -5.534722	56.388611;
                 -5.537500	56.385556;
                 -5.541389	56.383056;
                 -5.544167	56.380000;
                 -5.547778	56.377222;
                 -5.550833	56.374167;
                 -5.554444	56.371667;
                 -5.558056	56.368889;
                 -5.563611	56.367500;
                 -5.568889	56.368889;
                 -5.571667	56.371944;
                 -5.576667	56.373889;
                 -5.583889	56.374722;
                 -5.589444	56.376389;
                 -5.592222	56.379167;
                 -5.594444	56.382500;
                 -5.594722	56.386667;
                 -5.590000	56.388611;
                 -5.585000	56.390278;
                 -5.581389	56.393056;
                 -5.579722	56.396944;
                 -5.576111	56.399722;
                 -5.571389	56.400278;
                 -5.564444	56.400833;
                 -5.557778	56.401667;
                 -5.550278	56.401667;
                 -5.543333	56.402222;
                 -5.537500	56.403333;
                 -5.531667	56.404444;
                 -5.526389	56.406111;
                 -5.521944	56.408333;
                 -5.518333	56.410833;
                 -5.515278	56.413889;
                 -5.508611	56.414722;
                 -5.508333	56.411389;
                 -5.508889	56.408611;
                 -5.508889	56.406667];


  uki(14).xy = [ -5.694722	56.187222;
                 -5.695556	56.182500;
                 -5.693333	56.179444;
                 -5.691389	56.176111;
                 -5.690278	56.172222;
                 -5.688333	56.168889;
                 -5.688056	56.164722;
                 -5.691667	56.162222;
                 -5.697500	56.161111;
                 -5.701944	56.158889;
                 -5.706389	56.156944;
                 -5.710000	56.154167;
                 -5.715000	56.152778;
                 -5.721944	56.151944;
                 -5.730000	56.152500;
                 -5.738333	56.152778;
                 -5.745556	56.153611;
                 -5.753056	56.154167;
                 -5.756667	56.156667;
                 -5.758611	56.160000;
                 -5.755833	56.163056;
                 -5.753056	56.166111;
                 -5.749167	56.168889;
                 -5.745556	56.171389;
                 -5.742778	56.174444;
                 -5.740556	56.178056;
                 -5.738611	56.181667;
                 -5.735000	56.184167;
                 -5.729722	56.185000;
                 -5.722222	56.185000;
                 -5.716389	56.186111;
                 -5.711111	56.187778;
                 -5.706111	56.189444;
                 -5.700278	56.190556;
                 -5.694722	56.187222];


  uki(15).xy = [ -5.705556	56.211389;
                 -5.706111	56.206944;
                 -5.705000	56.203056;
                 -5.703889	56.199444;
                 -5.709722	56.198333;
                 -5.715278	56.200000;
                 -5.719167	56.202500;
                 -5.724722	56.203889;
                 -5.727500	56.206944;
                 -5.724167	56.208611;
                 -5.718889	56.210278;
                 -5.712778	56.211389;
                 -5.708611	56.213333;
                 -5.703333	56.215000;
                 -5.705556	56.211389];


  uki(16).xy = [ -5.758333	56.241667;
                 -5.760556	56.238056;
                 -5.764167	56.235556;
                 -5.769167	56.233889;
                 -5.776111	56.233333;
                 -5.782500	56.234444;
                 -5.787222	56.236944;
                 -5.785000	56.240556;
                 -5.780000	56.241111;
                 -5.773056	56.241944;
                 -5.768056	56.243333;
                 -5.763056	56.245000;
                 -5.756944	56.244444;
                 -5.758333	56.241667];


  uki(17).xy = [ -6.146944	56.116944;
                 -6.149167	56.113333;
                 -6.151111	56.109722;
                 -6.154167	56.106667;
                 -6.156944	56.103333;
                 -6.158889	56.100000;
                 -6.161944	56.096667;
                 -6.164722	56.093611;
                 -6.168333	56.091111;
                 -6.171111	56.088056;
                 -6.174167	56.085000;
                 -6.175278	56.080833;
                 -6.177500	56.077222;
                 -6.179444	56.073611;
                 -6.181667	56.070000;
                 -6.187500	56.068889;
                 -6.191944	56.066667;
                 -6.193889	56.063056;
                 -6.197500	56.060556;
                 -6.200556	56.057500;
                 -6.202500	56.053889;
                 -6.200556	56.050556;
                 -6.198333	56.047222;
                 -6.196389	56.043889;
                 -6.196667	56.039444;
                 -6.198889	56.035833;
                 -6.201667	56.032778;
                 -6.204722	56.029722;
                 -6.208889	56.027500;
                 -6.216389	56.027222;
                 -6.222778	56.028611;
                 -6.227500	56.030556;
                 -6.231111	56.033056;
                 -6.236944	56.034444;
                 -6.241944	56.032778;
                 -6.246944	56.031111;
                 -6.253056	56.030000;
                 -6.258611	56.031667;
                 -6.257222	56.035833;
                 -6.256944	56.039444;
                 -6.263056	56.040556;
                 -6.268333	56.038889;
                 -6.271944	56.036389;
                 -6.276389	56.034167;
                 -6.282222	56.033056;
                 -6.286944	56.035000;
                 -6.286389	56.038611;
                 -6.282778	56.041389;
                 -6.278333	56.043333;
                 -6.275000	56.046111;
                 -6.271111	56.048611;
                 -6.267500	56.051389;
                 -6.264167	56.053889;
                 -6.261944	56.057500;
                 -6.260833	56.061667;
                 -6.261944	56.065278;
                 -6.262222	56.069444;
                 -6.262500	56.073611;
                 -6.260556	56.077222;
                 -6.256111	56.079167;
                 -6.251667	56.081389;
                 -6.248056	56.083889;
                 -6.244444	56.086667;
                 -6.239167	56.088333;
                 -6.234167	56.090000;
                 -6.229722	56.091944;
                 -6.226389	56.094722;
                 -6.224167	56.098333;
                 -6.222222	56.101944;
                 -6.215556	56.101667;
                 -6.208333	56.100833;
                 -6.202500	56.099167;
                 -6.196389	56.098056;
                 -6.191111	56.098889;
                 -6.187500	56.101389;
                 -6.188056	56.105556;
                 -6.186389	56.109722;
                 -6.182778	56.112222;
                 -6.176944	56.113333;
                 -6.173333	56.116111;
                 -6.170000	56.118611;
                 -6.166389	56.121389;
                 -6.163333	56.124444;
                 -6.158889	56.126389;
                 -6.152222	56.127222;
                 -6.149444	56.124167;
                 -6.147222	56.121111;
                 -6.146944	56.116944];

  uki(18).xy = [ -6.224167	56.016667;
                 -6.230833	56.015833;
                 -6.235278	56.013889;
                 -6.237222	56.010278;
                 -6.239444	56.006667;
                 -6.242222	56.003611;
                 -6.248056	56.002500;
                 -6.255556	56.002222;
                 -6.257500	56.005556;
                 -6.258889	56.009167;
                 -6.262500	56.011667;
                 -6.267222	56.013611;
                 -6.272778	56.015278;
                 -6.268333	56.017500;
                 -6.261667	56.018056;
                 -6.255833	56.019167;
                 -6.250833	56.020833;
                 -6.245000	56.021944;
                 -6.238889	56.023056;
                 -6.232500	56.023889;
                 -6.224722	56.023889;
                 -6.217778	56.023333;
                 -6.213889	56.020833;
                 -6.218889	56.019167;
                 -6.224167	56.016667];


  uki(19).xy = [ -5.660278	56.441111;
                 -5.660000	56.437222;
                 -5.659444	56.433056;
                 -5.660833	56.428889;
                 -5.660556	56.424722;
                 -5.661389	56.420278;
                 -5.666389	56.418611;
                 -5.672222	56.419167;
                 -5.676944	56.421389;
                 -5.684444	56.421111;
                 -5.687222	56.418056;
                 -5.685278	56.414722;
                 -5.682500	56.411944;
                 -5.679444	56.408889;
                 -5.674722	56.406944;
                 -5.670833	56.404444;
                 -5.668889	56.401111;
                 -5.671944	56.398056;
                 -5.674722	56.395000;
                 -5.675278	56.390556;
                 -5.676667	56.386389;
                 -5.680556	56.383889;
                 -5.685556	56.382222;
                 -5.693056	56.381944;
                 -5.701111	56.382500;
                 -5.708611	56.383056;
                 -5.716667	56.383611;
                 -5.722500	56.385556;
                 -5.725278	56.388333;
                 -5.724722	56.393056;
                 -5.722778	56.396667;
                 -5.720556	56.400278;
                 -5.723333	56.403056;
                 -5.730000	56.402500;
                 -5.736111	56.401389;
                 -5.740556	56.399167;
                 -5.740833	56.394722;
                 -5.743056	56.391111;
                 -5.747778	56.388889;
                 -5.752222	56.386944;
                 -5.755833	56.384167;
                 -5.758611	56.381111;
                 -5.760833	56.377500;
                 -5.763611	56.374444;
                 -5.766667	56.371389;
                 -5.767222	56.367778;
                 -5.762500	56.365556;
                 -5.756944	56.367222;
                 -5.753333	56.370000;
                 -5.748889	56.371944;
                 -5.744722	56.374167;
                 -5.739444	56.375833;
                 -5.735000	56.376944;
                 -5.729167	56.378056;
                 -5.721667	56.378333;
                 -5.713611	56.377778;
                 -5.706111	56.377222;
                 -5.701389	56.375000;
                 -5.697778	56.372500;
                 -5.697500	56.368611;
                 -5.698611	56.364444;
                 -5.701667	56.361389;
                 -5.704722	56.358333;
                 -5.708889	56.356111;
                 -5.712778	56.353611;
                 -5.716944	56.351389;
                 -5.722500	56.349722;
                 -5.726944	56.347778;
                 -5.731111	56.345556;
                 -5.735556	56.343611;
                 -5.740000	56.341389;
                 -5.743611	56.338889;
                 -5.748056	56.336667;
                 -5.752500	56.334444;
                 -5.757500	56.332778;
                 -5.763056	56.331389;
                 -5.768056	56.329722;
                 -5.775000	56.329167;
                 -5.780833	56.327778;
                 -5.785278	56.325833;
                 -5.789444	56.323611;
                 -5.794722	56.321944;
                 -5.799722	56.320556;
                 -5.804444	56.318333;
                 -5.810278	56.317222;
                 -5.816111	56.316111;
                 -5.822222	56.315000;
                 -5.828056	56.313889;
                 -5.833889	56.312778;
                 -5.838333	56.310556;
                 -5.842778	56.308333;
                 -5.848889	56.307222;
                 -5.856944	56.307500;
                 -5.865278	56.308056;
                 -5.871944	56.307222;
                 -5.878056	56.306111;
                 -5.885278	56.306111;
                 -5.889167	56.308333;
                 -5.893056	56.310833;
                 -5.895000	56.314722;
                 -5.893611	56.318889;
                 -5.888333	56.320278;
                 -5.882500	56.321389;
                 -5.878056	56.323611;
                 -5.873889	56.325833;
                 -5.870000	56.328333;
                 -5.865556	56.330556;
                 -5.861944	56.333056;
                 -5.858056	56.335556;
                 -5.856944	56.339722;
                 -5.856111	56.344444;
                 -5.859444	56.347222;
                 -5.866667	56.346944;
                 -5.872500	56.345833;
                 -5.880000	56.345833;
                 -5.887778	56.345556;
                 -5.894167	56.345000;
                 -5.899444	56.343333;
                 -5.903889	56.341111;
                 -5.908333	56.339167;
                 -5.913611	56.337500;
                 -5.917222	56.334722;
                 -5.919167	56.331389;
                 -5.922778	56.328611;
                 -5.927222	56.326389;
                 -5.932500	56.325000;
                 -5.936111	56.322222;
                 -5.938333	56.318611;
                 -5.941111	56.315556;
                 -5.944722	56.313056;
                 -5.950556	56.311944;
                 -5.957500	56.311111;
                 -5.963889	56.312500;
                 -5.969444	56.313889;
                 -5.975833	56.315278;
                 -5.983333	56.315833;
                 -5.988611	56.314167;
                 -5.992222	56.311667;
                 -5.995833	56.309167;
                 -6.000278	56.306944;
                 -6.006111	56.305833;
                 -6.011944	56.304722;
                 -6.017778	56.303611;
                 -6.023056	56.301944;
                 -6.026944	56.299167;
                 -6.031944	56.297778;
                 -6.037222	56.296111;
                 -6.042222	56.294444;
                 -6.047500	56.292778;
                 -6.051667	56.290556;
                 -6.057778	56.289444;
                 -6.063611	56.288333;
                 -6.071111	56.289167;
                 -6.076667	56.290833;
                 -6.083056	56.291944;
                 -6.090278	56.292500;
                 -6.097778	56.292500;
                 -6.103889	56.291389;
                 -6.110556	56.290556;
                 -6.117222	56.290000;
                 -6.123056	56.288889;
                 -6.128889	56.287778;
                 -6.135000	56.286667;
                 -6.140000	56.285000;
                 -6.144444	56.282778;
                 -6.148056	56.280278;
                 -6.152500	56.278056;
                 -6.158333	56.277778;
                 -6.163889	56.279444;
                 -6.171389	56.280000;
                 -6.178611	56.280833;
                 -6.185278	56.280278;
                 -6.192778	56.280000;
                 -6.198333	56.281667;
                 -6.205833	56.281389;
                 -6.212500	56.280833;
                 -6.218889	56.281944;
                 -6.226389	56.282778;
                 -6.233889	56.282500;
                 -6.240556	56.281944;
                 -6.246944	56.281111;
                 -6.253056	56.280000;
                 -6.258333	56.278333;
                 -6.262778	56.276111;
                 -6.266111	56.273611;
                 -6.268333	56.270000;
                 -6.270556	56.266389;
                 -6.271111	56.261944;
                 -6.273889	56.258889;
                 -6.279722	56.257500;
                 -6.284444	56.259722;
                 -6.285556	56.263333;
                 -6.288611	56.266111;
                 -6.294444	56.265833;
                 -6.298889	56.263889;
                 -6.304444	56.265278;
                 -6.310833	56.266667;
                 -6.316667	56.265278;
                 -6.322778	56.264167;
                 -6.328611	56.263056;
                 -6.333611	56.264167;
                 -6.337222	56.266667;
                 -6.338333	56.270278;
                 -6.336389	56.273889;
                 -6.336111	56.276667;
                 -6.340833	56.278611;
                 -6.345556	56.280556;
                 -6.345833	56.284722;
                 -6.345278	56.289444;
                 -6.345833	56.293611;
                 -6.346944	56.297222;
                 -6.349722	56.300000;
                 -6.355556	56.301667;
                 -6.362778	56.301389;
                 -6.369722	56.300833;
                 -6.376111	56.301944;
                 -6.378056	56.305278;
                 -6.378333	56.309444;
                 -6.378056	56.313889;
                 -6.375833	56.317500;
                 -6.373056	56.320556;
                 -6.370000	56.323889;
                 -6.367222	56.326944;
                 -6.364444	56.330000;
                 -6.361667	56.333056;
                 -6.357222	56.335278;
                 -6.353611	56.337778;
                 -6.349167	56.340000;
                 -6.341667	56.340278;
                 -6.338611	56.337222;
                 -6.333333	56.335833;
                 -6.327222	56.336944;
                 -6.321389	56.338056;
                 -6.315556	56.337222;
                 -6.311667	56.335000;
                 -6.309444	56.331667;
                 -6.303889	56.330000;
                 -6.298889	56.331667;
                 -6.291389	56.330833;
                 -6.289444	56.327778;
                 -6.290000	56.323056;
                 -6.291389	56.319167;
                 -6.290000	56.315278;
                 -6.286389	56.312778;
                 -6.278889	56.313056;
                 -6.271389	56.313333;
                 -6.265000	56.312222;
                 -6.258333	56.310833;
                 -6.251111	56.310278;
                 -6.243611	56.310278;
                 -6.241389	56.313889;
                 -6.243611	56.317222;
                 -6.247222	56.319722;
                 -6.252222	56.321667;
                 -6.255833	56.324167;
                 -6.259722	56.326667;
                 -6.261944	56.329722;
                 -6.259722	56.333333;
                 -6.254722	56.335000;
                 -6.247222	56.334444;
                 -6.241667	56.332778;
                 -6.236111	56.331111;
                 -6.231389	56.329167;
                 -6.224167	56.328333;
                 -6.216667	56.328611;
                 -6.210000	56.327500;
                 -6.202500	56.327500;
                 -6.198333	56.329722;
                 -6.192500	56.330833;
                 -6.184167	56.330556;
                 -6.175833	56.330278;
                 -6.168333	56.330556;
                 -6.162222	56.331667;
                 -6.156389	56.332778;
                 -6.151389	56.334444;
                 -6.147778	56.336944;
                 -6.143333	56.339167;
                 -6.136667	56.339722;
                 -6.128333	56.339444;
                 -6.121667	56.340000;
                 -6.115833	56.341389;
                 -6.111389	56.343333;
                 -6.107778	56.346111;
                 -6.102500	56.347778;
                 -6.095000	56.347778;
                 -6.088333	56.348333;
                 -6.082500	56.349722;
                 -6.076389	56.350833;
                 -6.071389	56.352500;
                 -6.065556	56.353611;
                 -6.060000	56.355278;
                 -6.055556	56.357222;
                 -6.050556	56.358889;
                 -6.044722	56.360000;
                 -6.038889	56.361111;
                 -6.033333	56.362778;
                 -6.029167	56.365000;
                 -6.024722	56.366944;
                 -6.019444	56.368611;
                 -6.015000	56.370833;
                 -6.018056	56.373611;
                 -6.020000	56.376944;
                 -6.021111	56.380833;
                 -6.023333	56.383889;
                 -6.030833	56.384722;
                 -6.037222	56.384167;
                 -6.041944	56.381944;
                 -6.046389	56.379722;
                 -6.050833	56.377778;
                 -6.055278	56.375556;
                 -6.059444	56.373611;
                 -6.065556	56.372222;
                 -6.071944	56.371667;
                 -6.080278	56.371944;
                 -6.086389	56.370833;
                 -6.091667	56.369167;
                 -6.095833	56.367222;
                 -6.100278	56.365000;
                 -6.105556	56.363333;
                 -6.110556	56.361667;
                 -6.117500	56.361111;
                 -6.124722	56.360833;
                 -6.133056	56.361111;
                 -6.140000	56.360556;
                 -6.147222	56.360278;
                 -6.153056	56.359167;
                 -6.159167	56.358056;
                 -6.163611	56.355833;
                 -6.168611	56.354444;
                 -6.173889	56.352778;
                 -6.180556	56.351667;
                 -6.187222	56.350833;
                 -6.194722	56.350833;
                 -6.201389	56.350833;
                 -6.206944	56.352500;
                 -6.210833	56.355000;
                 -6.213889	56.356944;
                 -6.216667	56.359722;
                 -6.219722	56.362778;
                 -6.220833	56.366389;
                 -6.218333	56.369444;
                 -6.217500	56.374167;
                 -6.215556	56.377500;
                 -6.211944	56.380278;
                 -6.206667	56.381944;
                 -6.203889	56.385000;
                 -6.200833	56.388056;
                 -6.196667	56.390278;
                 -6.192222	56.392222;
                 -6.186944	56.393889;
                 -6.181944	56.395556;
                 -6.176389	56.397222;
                 -6.172500	56.399444;
                 -6.168611	56.401944;
                 -6.165000	56.404722;
                 -6.162222	56.407778;
                 -6.159167	56.410833;
                 -6.156389	56.413889;
                 -6.154167	56.417500;
                 -6.152222	56.421111;
                 -6.150000	56.424722;
                 -6.148056	56.428333;
                 -6.145833	56.431944;
                 -6.142778	56.435000;
                 -6.139167	56.438056;
                 -6.135556	56.440556;
                 -6.129722	56.441667;
                 -6.121944	56.441944;
                 -6.116389	56.440278;
                 -6.110000	56.439167;
                 -6.102778	56.438333;
                 -6.096944	56.438611;
                 -6.091667	56.440278;
                 -6.087222	56.442500;
                 -6.081944	56.444167;
                 -6.076944	56.445556;
                 -6.071667	56.447222;
                 -6.067500	56.449444;
                 -6.063611	56.451944;
                 -6.060000	56.454722;
                 -6.056389	56.457222;
                 -6.051667	56.459444;
                 -6.046667	56.461111;
                 -6.039444	56.461111;
                 -6.032500	56.461944;
                 -6.027222	56.463611;
                 -6.023056	56.465556;
                 -6.018333	56.467778;
                 -6.013889	56.469722;
                 -6.010278	56.472500;
                 -6.006944	56.475000;
                 -6.004722	56.478611;
                 -6.005000	56.482778;
                 -6.006111	56.486389;
                 -6.009167	56.489444;
                 -6.015556	56.490556;
                 -6.021389	56.489444;
                 -6.027222	56.488333;
                 -6.033333	56.487222;
                 -6.038611	56.485556;
                 -6.043611	56.483889;
                 -6.049167	56.482222;
                 -6.055000	56.481111;
                 -6.061667	56.480556;
                 -6.069167	56.480278;
                 -6.074167	56.478611;
                 -6.077778	56.476111;
                 -6.082500	56.473889;
                 -6.090556	56.474167;
                 -6.097222	56.474444;
                 -6.104722	56.474167;
                 -6.110000	56.472778;
                 -6.114444	56.470556;
                 -6.118889	56.468333;
                 -6.123889	56.466667;
                 -6.130556	56.466111;
                 -6.136111	56.467778;
                 -6.140000	56.470278;
                 -6.144167	56.472500;
                 -6.149722	56.474722;
                 -6.153333	56.476944;
                 -6.155556	56.480278;
                 -6.154167	56.484444;
                 -6.154444	56.488611;
                 -6.158333	56.491111;
                 -6.164722	56.492222;
                 -6.171389	56.493333;
                 -6.176111	56.495278;
                 -6.180556	56.497500;
                 -6.184444	56.499722;
                 -6.189444	56.501944;
                 -6.195833	56.503056;
                 -6.203056	56.503611;
                 -6.208611	56.505278;
                 -6.212778	56.507778;
                 -6.216667	56.510278;
                 -6.220556	56.512500;
                 -6.224444	56.515000;
                 -6.228889	56.516944;
                 -6.233889	56.519167;
                 -6.240278	56.520278;
                 -6.247500	56.521111;
                 -6.255833	56.521389;
                 -6.262778	56.520556;
                 -6.269167	56.520000;
                 -6.276111	56.519167;
                 -6.281944	56.518056;
                 -6.289444	56.518056;
                 -6.296111	56.519167;
                 -6.301667	56.520556;
                 -6.306389	56.522778;
                 -6.313889	56.523333;
                 -6.321389	56.524167;
                 -6.329722	56.524444;
                 -6.336944	56.525278;
                 -6.343611	56.526389;
                 -6.347222	56.528889;
                 -6.350278	56.531667;
                 -6.349722	56.535278;
                 -6.347778	56.538889;
                 -6.347222	56.543333;
                 -6.344167	56.546667;
                 -6.339722	56.548611;
                 -6.334722	56.550278;
                 -6.329722	56.551944;
                 -6.323611	56.553056;
                 -6.317500	56.554167;
                 -6.313333	56.556389;
                 -6.309444	56.558889;
                 -6.306667	56.562222;
                 -6.302222	56.564167;
                 -6.297778	56.566389;
                 -6.293333	56.568611;
                 -6.290556	56.571667;
                 -6.291667	56.574444;
                 -6.297500	56.574167;
                 -6.304444	56.573611;
                 -6.310556	56.572222;
                 -6.315556	56.570833;
                 -6.323056	56.570556;
                 -6.325278	56.573889;
                 -6.326389	56.577500;
                 -6.326667	56.581667;
                 -6.323889	56.584722;
                 -6.323611	56.588333;
                 -6.326389	56.591389;
                 -6.328611	56.594444;
                 -6.329722	56.598333;
                 -6.324722	56.600000;
                 -6.319722	56.597778;
                 -6.312222	56.598056;
                 -6.307222	56.599722;
                 -6.301111	56.600833;
                 -6.293611	56.601111;
                 -6.288056	56.599444;
                 -6.281944	56.600556;
                 -6.279167	56.603611;
                 -6.276111	56.606944;
                 -6.271111	56.608611;
                 -6.264444	56.609167;
                 -6.258889	56.607500;
                 -6.255556	56.605556;
                 -6.250833	56.603611;
                 -6.246111	56.601667;
                 -6.241111	56.599722;
                 -6.237500	56.597222;
                 -6.233611	56.594722;
                 -6.229722	56.592222;
                 -6.226944	56.589444;
                 -6.221944	56.587500;
                 -6.217500	56.589444;
                 -6.216389	56.593611;
                 -6.217500	56.597222;
                 -6.219444	56.600556;
                 -6.223333	56.603056;
                 -6.226389	56.605833;
                 -6.229167	56.608889;
                 -6.232500	56.611667;
                 -6.234444	56.615000;
                 -6.235556	56.618611;
                 -6.235000	56.623333;
                 -6.228333	56.623889;
                 -6.222778	56.622222;
                 -6.216944	56.620833;
                 -6.211944	56.621389;
                 -6.210556	56.625556;
                 -6.209167	56.629444;
                 -6.208056	56.633611;
                 -6.204444	56.636389;
                 -6.196667	56.635556;
                 -6.191667	56.637222;
                 -6.187222	56.639167;
                 -6.180556	56.640000;
                 -6.174444	56.641111;
                 -6.170278	56.643333;
                 -6.165556	56.645278;
                 -6.159722	56.645556;
                 -6.154167	56.643889;
                 -6.147778	56.642778;
                 -6.144722	56.645833;
                 -6.141944	56.649167;
                 -6.138333	56.648889;
                 -6.133889	56.646944;
                 -6.130000	56.644444;
                 -6.126944	56.641667;
                 -6.122222	56.639444;
                 -6.116389	56.638056;
                 -6.109167	56.637222;
                 -6.100833	56.636944;
                 -6.092500	56.636667;
                 -6.085278	56.635833;
                 -6.081389	56.633333;
                 -6.078056	56.631389;
                 -6.076111	56.628056;
                 -6.073889	56.625000;
                 -6.072778	56.621111;
                 -6.072500	56.616944;
                 -6.071389	56.613333;
                 -6.067500	56.610833;
                 -6.061667	56.609167;
                 -6.053333	56.608889;
                 -6.047222	56.607778;
                 -6.041389	56.606111;
                 -6.037500	56.603611;
                 -6.034722	56.600833;
                 -6.032500	56.597500;
                 -6.030556	56.594167;
                 -6.029167	56.590556;
                 -6.026389	56.587778;
                 -6.024167	56.584444;
                 -6.021111	56.581667;
                 -6.018333	56.578611;
                 -6.014444	56.576111;
                 -6.009722	56.574167;
                 -6.005833	56.571667;
                 -6.001944	56.569444;
                 -6.000000	56.566111;
                 -5.998889	56.562222;
                 -5.998611	56.558333;
                 -5.997222	56.554444;
                 -5.992778	56.552500;
                 -5.987778	56.550556;
                 -5.983889	56.548056;
                 -5.980278	56.545556;
                 -5.977222	56.542778;
                 -5.975000	56.539444;
                 -5.973056	56.536111;
                 -5.970833	56.532778;
                 -5.968056	56.530000;
                 -5.966111	56.526667;
                 -5.963889	56.523333;
                 -5.960833	56.520556;
                 -5.956944	56.518056;
                 -5.950556	56.516944;
                 -5.945000	56.515278;
                 -5.939444	56.513611;
                 -5.933889	56.511944;
                 -5.928889	56.510000;
                 -5.922500	56.509722;
                 -5.918056	56.511944;
                 -5.913611	56.513889;
                 -5.909167	56.516111;
                 -5.902222	56.516667;
                 -5.895833	56.517500;
                 -5.887500	56.517222;
                 -5.880000	56.516389;
                 -5.873611	56.515278;
                 -5.865278	56.514722;
                 -5.857778	56.514167;
                 -5.853889	56.511667;
                 -5.851944	56.508333;
                 -5.850833	56.504722;
                 -5.847778	56.501667;
                 -5.842222	56.500000;
                 -5.836389	56.501389;
                 -5.831111	56.502778;
                 -5.828056	56.506111;
                 -5.825278	56.509167;
                 -5.818611	56.509722;
                 -5.811944	56.508611;
                 -5.806389	56.506944;
                 -5.800833	56.505278;
                 -5.796111	56.503333;
                 -5.792222	56.500833;
                 -5.789444	56.498056;
                 -5.785556	56.495556;
                 -5.783333	56.492222;
                 -5.780556	56.489444;
                 -5.778611	56.486111;
                 -5.774722	56.483611;
                 -5.770000	56.481667;
                 -5.763333	56.480278;
                 -5.757778	56.478889;
                 -5.752500	56.477222;
                 -5.745000	56.476389;
                 -5.738889	56.476667;
                 -5.732500	56.477222;
                 -5.726944	56.475556;
                 -5.723889	56.472778;
                 -5.720833	56.470000;
                 -5.716944	56.467500;
                 -5.711389	56.465833;
                 -5.706667	56.463889;
                 -5.701111	56.462222;
                 -5.696667	56.460278;
                 -5.691944	56.458056;
                 -5.686944	56.456111;
                 -5.686667	56.451944;
                 -5.685556	56.448333;
                 -5.681944	56.445833;
                 -5.677500	56.448056;
                 -5.673056	56.450000;
                 -5.666389	56.448889;
                 -5.663611	56.446111;
                 -5.662222	56.442222;
                 -5.660278	56.441111];


  uki(20).xy = [ -5.443889	56.543611;
                 -5.447500	56.540833;
                 -5.451111	56.538333;
                 -5.454722	56.535556;
                 -5.458611	56.533056;
                 -5.462222	56.530556;
                 -5.466111	56.527778;
                 -5.469722	56.525278;
                 -5.473333	56.522778;
                 -5.476389	56.519722;
                 -5.480000	56.516944;
                 -5.483611	56.514444;
                 -5.487222	56.511944;
                 -5.490833	56.509167;
                 -5.494722	56.506667;
                 -5.498611	56.504167;
                 -5.502222	56.501389;
                 -5.505833	56.498889;
                 -5.509444	56.496389;
                 -5.513889	56.494167;
                 -5.517500	56.491667;
                 -5.521111	56.488889;
                 -5.525833	56.486944;
                 -5.531111	56.485278;
                 -5.536111	56.483611;
                 -5.540833	56.481667;
                 -5.544444	56.478889;
                 -5.546667	56.475556;
                 -5.549444	56.472222;
                 -5.553333	56.469722;
                 -5.558333	56.468056;
                 -5.563611	56.466389;
                 -5.569444	56.465278;
                 -5.575556	56.464167;
                 -5.580833	56.462778;
                 -5.585833	56.461111;
                 -5.591111	56.459444;
                 -5.596389	56.457778;
                 -5.601667	56.456111;
                 -5.606667	56.454722;
                 -5.613611	56.453889;
                 -5.621944	56.454167;
                 -5.625000	56.455278;
                 -5.620278	56.457500;
                 -5.615833	56.459444;
                 -5.611667	56.461667;
                 -5.607222	56.463889;
                 -5.601944	56.465278;
                 -5.598056	56.468056;
                 -5.593889	56.470000;
                 -5.590000	56.472778;
                 -5.586389	56.475278;
                 -5.584167	56.478889;
                 -5.583611	56.483333;
                 -5.583889	56.487500;
                 -5.581667	56.491111;
                 -5.575556	56.492222;
                 -5.569722	56.493333;
                 -5.564722	56.495000;
                 -5.559444	56.496667;
                 -5.555000	56.498611;
                 -5.551389	56.501389;
                 -5.547500	56.503889;
                 -5.544722	56.506944;
                 -5.538611	56.508056;
                 -5.532778	56.509167;
                 -5.526667	56.510278;
                 -5.520833	56.511389;
                 -5.515278	56.513056;
                 -5.511667	56.515556;
                 -5.508889	56.518889;
                 -5.505833	56.521944;
                 -5.503056	56.525000;
                 -5.498611	56.526944;
                 -5.495000	56.529722;
                 -5.490556	56.531667;
                 -5.486667	56.534444;
                 -5.482778	56.536944;
                 -5.478611	56.538889;
                 -5.474167	56.541111;
                 -5.471111	56.544167;
                 -5.467500	56.546667;
                 -5.460556	56.547500;
                 -5.455556	56.548889;
                 -5.450833	56.551111;
                 -5.446667	56.553333;
                 -5.442222	56.555278;
                 -5.437222	56.554167;
                 -5.436944	56.550000;
                 -5.440000	56.546944;
                 -5.443889	56.543611];


  uki(21).xy = [ -6.364167	56.289722;
                 -6.361389	56.286667;
                 -6.362500	56.282778;
                 -6.366944	56.280556;
                 -6.373889	56.280000;
                 -6.378333	56.281111;
                 -6.381389	56.283889;
                 -6.382500	56.287500;
                 -6.381389	56.291667;
                 -6.378333	56.294722;
                 -6.374167	56.296944;
                 -6.369167	56.297500;
                 -6.364167	56.295556;
                 -6.363333	56.291944;
                 -6.364167	56.289722];


  uki(22).xy = [ -6.391944	56.329722;
                 -6.393889	56.326111;
                 -6.396667	56.322778;
                 -6.399722	56.319722;
                 -6.403056	56.317222;
                 -6.407500	56.315000;
                 -6.412500	56.313333;
                 -6.416389	56.310833;
                 -6.420000	56.308056;
                 -6.424444	56.306111;
                 -6.428611	56.303889;
                 -6.432500	56.301389;
                 -6.437500	56.299167;
                 -6.444167	56.299444;
                 -6.446111	56.302778;
                 -6.450000	56.305000;
                 -6.454722	56.307222;
                 -6.457778	56.310000;
                 -6.454167	56.312500;
                 -6.449722	56.314722;
                 -6.445278	56.316944;
                 -6.441667	56.319444;
                 -6.438889	56.322500;
                 -6.441944	56.325556;
                 -6.445833	56.327778;
                 -6.445278	56.330556;
                 -6.438611	56.331389;
                 -6.432778	56.332500;
                 -6.429167	56.335000;
                 -6.425556	56.337778;
                 -6.419722	56.338056;
                 -6.413056	56.337778;
                 -6.406389	56.338333;
                 -6.398889	56.338611;
                 -6.394167	56.336667;
                 -6.393056	56.332778;
                 -6.391944	56.329722];


  uki(23).xy = [ -6.157500	56.457500;
                 -6.161944	56.455278;
                 -6.166944	56.453889;
                 -6.173056	56.452500;
                 -6.180556	56.452500;
                 -6.185278	56.454444;
                 -6.190000	56.456389;
                 -6.196667	56.457500;
                 -6.203056	56.456944;
                 -6.208611	56.455278;
                 -6.215833	56.456111;
                 -6.222500	56.457222;
                 -6.226944	56.459167;
                 -6.231667	56.461111;
                 -6.237222	56.462778;
                 -6.245000	56.462778;
                 -6.251389	56.463889;
                 -6.256111	56.465833;
                 -6.261667	56.467500;
                 -6.265556	56.469722;
                 -6.264167	56.473889;
                 -6.261389	56.476944;
                 -6.258611	56.480000;
                 -6.255556	56.483333;
                 -6.251111	56.485278;
                 -6.246111	56.486944;
                 -6.240000	56.488056;
                 -6.232778	56.488333;
                 -6.226944	56.486667;
                 -6.222222	56.484722;
                 -6.217500	56.482778;
                 -6.212778	56.480833;
                 -6.206389	56.479444;
                 -6.200556	56.478056;
                 -6.192500	56.477500;
                 -6.185000	56.476944;
                 -6.182222	56.474167;
                 -6.177222	56.471944;
                 -6.171667	56.470556;
                 -6.166944	56.468333;
                 -6.161389	56.466944;
                 -6.155833	56.465278;
                 -6.152778	56.462500;
                 -6.157500	56.457500];


  uki(24).xy = [ -6.273889	56.483333;
                 -6.275833	56.480833;
                 -6.280278	56.478611;
                 -6.285278	56.476944;
                 -6.289167	56.474444;
                 -6.295000	56.473333;
                 -6.303333	56.473611;
                 -6.310833	56.473333;
                 -6.316389	56.475000;
                 -6.319444	56.477778;
                 -6.319722	56.481944;
                 -6.317500	56.485556;
                 -6.313889	56.488056;
                 -6.310278	56.490833;
                 -6.306667	56.493333;
                 -6.301389	56.495000;
                 -6.295000	56.493889;
                 -6.291944	56.491111;
                 -6.293333	56.487778;
                 -6.293611	56.484167;
                 -6.287222	56.483889;
                 -6.282778	56.486111;
                 -6.278333	56.488056;
                 -6.273056	56.489722;
                 -6.271944	56.486111;
                 -6.273889	56.483333];


  uki(25).xy = [ -6.486111	56.450000;
                 -6.489722	56.447222;
                 -6.492778	56.444167;
                 -6.496944	56.441944;
                 -6.501944	56.442222;
                 -6.502222	56.446389;
                 -6.501111	56.450556;
                 -6.498889	56.453889;
                 -6.495278	56.456667;
                 -6.490278	56.458333;
                 -6.484722	56.456667;
                 -6.482500	56.453333;
                 -6.486111	56.450000];


  uki(26).xy = [ -6.428889	56.484444;
                 -6.429444	56.480833;
                 -6.430556	56.476667;
                 -6.433611	56.473611;
                 -6.438056	56.471667;
                 -6.444444	56.470833;
                 -6.449444	56.472778;
                 -6.450556	56.476667;
                 -6.447778	56.479722;
                 -6.444167	56.482222;
                 -6.442222	56.485833;
                 -6.438611	56.488611;
                 -6.430833	56.488611;
                 -6.426944	56.486389;
                 -6.428889	56.484444];

  uki(27).xy = [ -6.729444	56.530278;
                 -6.729722	56.526667;
                 -6.729444	56.522500;
                 -6.732222	56.519444;
                 -6.738333	56.518056;
                 -6.745000	56.517500;
                 -6.751111	56.516389;
                 -6.756111	56.514722;
                 -6.763333	56.515278;
                 -6.768611	56.513611;
                 -6.774444	56.513333;
                 -6.777222	56.516389;
                 -6.783333	56.517778;
                 -6.790556	56.517500;
                 -6.796667	56.516389;
                 -6.802500	56.515278;
                 -6.807500	56.513611;
                 -6.811944	56.511389;
                 -6.815000	56.508333;
                 -6.814444	56.504167;
                 -6.810556	56.501667;
                 -6.807500	56.498889;
                 -6.804444	56.496111;
                 -6.805000	56.492500;
                 -6.808611	56.489722;
                 -6.814444	56.488611;
                 -6.818889	56.486389;
                 -6.823333	56.484444;
                 -6.829167	56.483056;
                 -6.836667	56.483056;
                 -6.843889	56.483611;
                 -6.852222	56.483889;
                 -6.860556	56.484167;
                 -6.868889	56.484444;
                 -6.877222	56.484722;
                 -6.883056	56.483611;
                 -6.888056	56.481944;
                 -6.893333	56.480278;
                 -6.898333	56.478611;
                 -6.903056	56.476389;
                 -6.905833	56.473333;
                 -6.907778	56.469722;
                 -6.908889	56.465556;
                 -6.909444	56.461111;
                 -6.908333	56.457222;
                 -6.906944	56.453611;
                 -6.907500	56.449167;
                 -6.909444	56.445556;
                 -6.913056	56.442778;
                 -6.918333	56.441111;
                 -6.924167	56.440000;
                 -6.931667	56.439722;
                 -6.938333	56.440833;
                 -6.943889	56.442500;
                 -6.948611	56.444444;
                 -6.952500	56.446944;
                 -6.957222	56.448889;
                 -6.961944	56.450833;
                 -6.968611	56.451944;
                 -6.975278	56.451111;
                 -6.979722	56.449167;
                 -6.985556	56.447778;
                 -6.993889	56.448056;
                 -6.998611	56.450000;
                 -6.998333	56.453889;
                 -6.994722	56.456389;
                 -6.990278	56.458611;
                 -6.988333	56.462222;
                 -6.989444	56.465833;
                 -6.990833	56.469444;
                 -6.993056	56.472778;
                 -6.995000	56.476111;
                 -6.993056	56.479722;
                 -6.990833	56.483333;
                 -6.992222	56.486944;
                 -6.995278	56.490000;
                 -6.996667	56.493611;
                 -6.995278	56.497778;
                 -6.992500	56.500833;
                 -6.988889	56.503333;
                 -6.982778	56.504722;
                 -6.976944	56.505833;
                 -6.971944	56.507500;
                 -6.968333	56.510000;
                 -6.963889	56.512222;
                 -6.960556	56.515000;
                 -6.956111	56.516944;
                 -6.950556	56.518611;
                 -6.945833	56.517500;
                 -6.938611	56.517778;
                 -6.932500	56.518889;
                 -6.925000	56.519167;
                 -6.918333	56.520000;
                 -6.910000	56.519722;
                 -6.902500	56.518889;
                 -6.896111	56.517778;
                 -6.888611	56.517222;
                 -6.881944	56.517778;
                 -6.876667	56.519444;
                 -6.872222	56.521667;
                 -6.867222	56.523333;
                 -6.861944	56.525000;
                 -6.856944	56.526667;
                 -6.851944	56.528333;
                 -6.846667	56.530000;
                 -6.842222	56.532222;
                 -6.836944	56.533889;
                 -6.830278	56.534444;
                 -6.822778	56.534722;
                 -6.816389	56.533611;
                 -6.809722	56.532500;
                 -6.802500	56.531667;
                 -6.795556	56.531667;
                 -6.789167	56.532222;
                 -6.783333	56.533333;
                 -6.777778	56.535000;
                 -6.774167	56.537778;
                 -6.773056	56.541667;
                 -6.771111	56.545278;
                 -6.767222	56.548056;
                 -6.761389	56.549167;
                 -6.756667	56.548056;
                 -6.751667	56.546111;
                 -6.747778	56.543611;
                 -6.743889	56.541389;
                 -6.740000	56.538889;
                 -6.736944	56.536111;
                 -6.734167	56.533056;
                 -6.729444	56.530278];


  uki(28).xy = [ -6.460556	56.674167;
                 -6.463611	56.670833;
                 -6.467222	56.668333;
                 -6.470833	56.665833;
                 -6.474722	56.663056;
                 -6.476667	56.659444;
                 -6.478889	56.655833;
                 -6.480833	56.652222;
                 -6.482778	56.648611;
                 -6.484167	56.644722;
                 -6.486389	56.641111;
                 -6.488333	56.637500;
                 -6.490556	56.633889;
                 -6.493333	56.630833;
                 -6.497778	56.628611;
                 -6.500556	56.625556;
                 -6.503333	56.622500;
                 -6.505556	56.618889;
                 -6.508333	56.615556;
                 -6.513056	56.613611;
                 -6.518056	56.611944;
                 -6.523889	56.610833;
                 -6.528333	56.608611;
                 -6.532778	56.606389;
                 -6.535000	56.602778;
                 -6.538611	56.600278;
                 -6.545278	56.599444;
                 -6.548056	56.596389;
                 -6.550278	56.592778;
                 -6.553611	56.590278;
                 -6.560556	56.589444;
                 -6.567500	56.588889;
                 -6.570833	56.586111;
                 -6.575556	56.584167;
                 -6.580556	56.582500;
                 -6.588889	56.582778;
                 -6.596389	56.582500;
                 -6.601667	56.580833;
                 -6.606667	56.579167;
                 -6.613611	56.578611;
                 -6.620000	56.577778;
                 -6.622222	56.574167;
                 -6.626667	56.572222;
                 -6.634167	56.571944;
                 -6.640833	56.571111;
                 -6.646111	56.569444;
                 -6.653611	56.569444;
                 -6.660278	56.570556;
                 -6.663056	56.573333;
                 -6.666944	56.575833;
                 -6.672500	56.577500;
                 -6.679167	56.578611;
                 -6.685833	56.577778;
                 -6.691111	56.576111;
                 -6.694444	56.573611;
                 -6.697500	56.570556;
                 -6.698611	56.566389;
                 -6.698611	56.562222;
                 -6.698056	56.558056;
                 -6.701667	56.555556;
                 -6.706667	56.557500;
                 -6.708611	56.560833;
                 -6.710556	56.563889;
                 -6.712778	56.567222;
                 -6.715000	56.570556;
                 -6.712778	56.574167;
                 -6.710000	56.577222;
                 -6.704722	56.578889;
                 -6.699722	56.580556;
                 -6.696667	56.583611;
                 -6.693333	56.586389;
                 -6.687500	56.586667;
                 -6.682500	56.584444;
                 -6.676111	56.584444;
                 -6.671667	56.586389;
                 -6.669444	56.590000;
                 -6.668333	56.594167;
                 -6.665278	56.597222;
                 -6.660278	56.598889;
                 -6.655000	56.600556;
                 -6.653056	56.604167;
                 -6.652222	56.608889;
                 -6.649722	56.611944;
                 -6.644167	56.613611;
                 -6.636944	56.613611;
                 -6.631944	56.615278;
                 -6.628056	56.618056;
                 -6.626667	56.622222;
                 -6.627222	56.626111;
                 -6.626111	56.630278;
                 -6.621389	56.632500;
                 -6.616111	56.634167;
                 -6.608889	56.634444;
                 -6.603611	56.635833;
                 -6.598333	56.637500;
                 -6.593333	56.639167;
                 -6.588056	56.640833;
                 -6.583611	56.643056;
                 -6.578333	56.644722;
                 -6.573889	56.646944;
                 -6.570278	56.649444;
                 -6.567500	56.652500;
                 -6.563056	56.654722;
                 -6.557778	56.656389;
                 -6.552778	56.658056;
                 -6.548333	56.660278;
                 -6.546111	56.663889;
                 -6.542500	56.666389;
                 -6.537222	56.668056;
                 -6.529722	56.668333;
                 -6.523056	56.668889;
                 -6.518611	56.671111;
                 -6.515000	56.673611;
                 -6.511111	56.676389;
                 -6.506944	56.678333;
                 -6.501944	56.678333;
                 -6.494722	56.677500;
                 -6.486944	56.677778;
                 -6.480833	56.678889;
                 -6.475833	56.680556;
                 -6.470000	56.681667;
                 -6.464722	56.683333;
                 -6.458611	56.684444;
                 -6.452778	56.683889;
                 -6.453333	56.680278;
                 -6.456944	56.677500;
                 -6.460556	56.674167];


  uki(29).xy = [ -5.831111	56.783611;
                 -5.837778	56.783056;
                 -5.843889	56.781667;
                 -5.849167	56.780278;
                 -5.855833	56.779444;
                 -5.860000	56.781111;
                 -5.866389	56.782222;
                 -5.871944	56.783889;
                 -5.876667	56.785833;
                 -5.879722	56.788611;
                 -5.880833	56.792500;
                 -5.878611	56.796111;
                 -5.876667	56.799722;
                 -5.871389	56.801111;
                 -5.863889	56.800556;
                 -5.860833	56.798611;
                 -5.856111	56.796389;
                 -5.850278	56.795000;
                 -5.843889	56.793611;
                 -5.836389	56.793056;
                 -5.830833	56.791389;
                 -5.826111	56.789444;
                 -5.822222	56.786944;
                 -5.825278	56.783889;
                 -5.831111	56.783611];


  uki(30).xy = [ -6.224444	56.830556;
                 -6.225556	56.828333;
                 -6.229167	56.825556;
                 -6.234444	56.823889;
                 -6.240556	56.822778;
                 -6.246944	56.823889;
                 -6.251667	56.826111;
                 -6.256944	56.825278;
                 -6.260556	56.822500;
                 -6.268333	56.822500;
                 -6.273056	56.824444;
                 -6.275278	56.827778;
                 -6.273889	56.830833;
                 -6.271111	56.833889;
                 -6.268056	56.837222;
                 -6.269167	56.840000;
                 -6.263611	56.839167;
                 -6.256944	56.838056;
                 -6.251111	56.837500;
                 -6.246111	56.839167;
                 -6.240833	56.840833;
                 -6.233056	56.840833;
                 -6.229167	56.838333;
                 -6.228056	56.834722;
                 -6.224444	56.830556];


  uki(31).xy = [ -6.108056	56.921111;
                 -6.108889	56.916389;
                 -6.110000	56.912222;
                 -6.111667	56.908333;
                 -6.112778	56.904167;
                 -6.113611	56.899722;
                 -6.114722	56.895556;
                 -6.116111	56.891389;
                 -6.118333	56.887778;
                 -6.119722	56.883889;
                 -6.123333	56.881111;
                 -6.129444	56.880000;
                 -6.134444	56.878333;
                 -6.136667	56.874722;
                 -6.138889	56.871389;
                 -6.144722	56.870000;
                 -6.151667	56.869444;
                 -6.159167	56.869167;
                 -6.165833	56.870556;
                 -6.172500	56.871667;
                 -6.178056	56.873056;
                 -6.182778	56.875278;
                 -6.186667	56.877778;
                 -6.190556	56.880000;
                 -6.195278	56.882222;
                 -6.200833	56.883611;
                 -6.206667	56.885278;
                 -6.211389	56.887222;
                 -6.213611	56.890556;
                 -6.213889	56.894722;
                 -6.214444	56.898889;
                 -6.214722	56.903056;
                 -6.213056	56.906944;
                 -6.210556	56.910278;
                 -6.207500	56.913333;
                 -6.202778	56.915278;
                 -6.196389	56.916111;
                 -6.188611	56.916111;
                 -6.182222	56.915000;
                 -6.174444	56.914444;
                 -6.168333	56.915556;
                 -6.164722	56.918056;
                 -6.165000	56.922222;
                 -6.165556	56.926389;
                 -6.165833	56.930556;
                 -6.165278	56.935000;
                 -6.162222	56.938056;
                 -6.158611	56.940833;
                 -6.154167	56.942778;
                 -6.149722	56.945000;
                 -6.142222	56.944167;
                 -6.137500	56.943056;
                 -6.132500	56.941111;
                 -6.127778	56.939167;
                 -6.123889	56.936667;
                 -6.118056	56.934722;
                 -6.113611	56.932500;
                 -6.110278	56.929722;
                 -6.107500	56.926944;
                 -6.108056	56.921111];
  
  
  uki(32).xy = [ -6.249444	56.993333;
                 -6.250833	56.990000;
                 -6.253611	56.986944;
                 -6.256667	56.983889;
                 -6.259444	56.980833;
                 -6.257222	56.977500;
                 -6.255278	56.974167;
                 -6.255000	56.970000;
                 -6.256389	56.965833;
                 -6.259167	56.962778;
                 -6.262222	56.959722;
                 -6.265000	56.956667;
                 -6.268889	56.953889;
                 -6.273056	56.951944;
                 -6.276944	56.949167;
                 -6.281389	56.947222;
                 -6.286667	56.945556;
                 -6.291111	56.943333;
                 -6.295278	56.941389;
                 -6.300833	56.939722;
                 -6.305833	56.938056;
                 -6.311389	56.936389;
                 -6.318056	56.935556;
                 -6.325000	56.935000;
                 -6.329722	56.936944;
                 -6.331667	56.940278;
                 -6.336667	56.942222;
                 -6.342222	56.943889;
                 -6.348056	56.944444;
                 -6.352778	56.946667;
                 -6.357778	56.948611;
                 -6.365278	56.949167;
                 -6.371944	56.948611;
                 -6.379444	56.949444;
                 -6.382222	56.952222;
                 -6.384444	56.955556;
                 -6.385833	56.959167;
                 -6.386944	56.962778;
                 -6.388056	56.966667;
                 -6.391389	56.969444;
                 -6.396111	56.971389;
                 -6.403611	56.972222;
                 -6.410278	56.973333;
                 -6.413056	56.976111;
                 -6.415278	56.979444;
                 -6.418333	56.982222;
                 -6.420278	56.985556;
                 -6.423333	56.988611;
                 -6.427222	56.990833;
                 -6.433056	56.992500;
                 -6.439722	56.993611;
                 -6.446389	56.994722;
                 -6.451944	56.996389;
                 -6.456667	56.998333;
                 -6.461667	57.000278;
                 -6.466389	57.002500;
                 -6.471111	57.004444;
                 -6.474444	57.007222;
                 -6.469722	57.009444;
                 -6.461389	57.009167;
                 -6.452778	57.008889;
                 -6.446944	57.010000;
                 -6.442500	57.011944;
                 -6.438611	57.014722;
                 -6.436111	57.017778;
                 -6.432222	57.020278;
                 -6.427778	57.022500;
                 -6.424167	57.025278;
                 -6.420278	57.027778;
                 -6.417500	57.030833;
                 -6.414444	57.033889;
                 -6.410278	57.036111;
                 -6.404167	57.037222;
                 -6.399722	57.039444;
                 -6.395278	57.041667;
                 -6.391667	57.044167;
                 -6.387778	57.046667;
                 -6.384167	57.049444;
                 -6.377222	57.050000;
                 -6.370556	57.048889;
                 -6.363333	57.048333;
                 -6.357778	57.049722;
                 -6.354167	57.052500;
                 -6.349722	57.054722;
                 -6.343889	57.055833;
                 -6.336111	57.055833;
                 -6.328611	57.055278;
                 -6.323056	57.053611;
                 -6.316389	57.052500;
                 -6.309722	57.051389;
                 -6.305000	57.049167;
                 -6.300000	57.047222;
                 -6.295278	57.045278;
                 -6.293333	57.041944;
                 -6.291944	57.038333;
                 -6.289167	57.035278;
                 -6.283056	57.036667;
                 -6.275278	57.036667;
                 -6.270556	57.034722;
                 -6.265556	57.032778;
                 -6.261667	57.030278;
                 -6.258889	57.027500;
                 -6.255000	57.025000;
                 -6.250000	57.023056;
                 -6.246111	57.020556;
                 -6.243056	57.017778;
                 -6.242778	57.014444;
                 -6.245556	57.011389;
                 -6.250000	57.009167;
                 -6.255278	57.007500;
                 -6.259167	57.005000;
                 -6.253333	57.003333;
                 -6.249167	57.001944;
                 -6.246389	56.998889;
                 -6.245000	56.995278;
                 -6.249444	56.993333];


  uki(33).xy = [ -6.493611	57.055556;
                 -6.498333	57.053333;
                 -6.504167	57.052222;
                 -6.509444	57.050556;
                 -6.515556	57.049444;
                 -6.522500	57.048889;
                 -6.530000	57.048611;
                 -6.537500	57.048333;
                 -6.545278	57.049167;
                 -6.552500	57.049722;
                 -6.558611	57.048611;
                 -6.563889	57.046944;
                 -6.568333	57.044722;
                 -6.573611	57.043333;
                 -6.578889	57.041667;
                 -6.583889	57.040000;
                 -6.589444	57.037778;
                 -6.594444	57.036111;
                 -6.600556	57.035000;
                 -6.607222	57.036111;
                 -6.611111	57.038611;
                 -6.613333	57.041944;
                 -6.614444	57.045556;
                 -6.613333	57.049722;
                 -6.608889	57.051667;
                 -6.603611	57.053333;
                 -6.597500	57.054444;
                 -6.593056	57.056667;
                 -6.586944	57.057778;
                 -6.581111	57.059167;
                 -6.575556	57.060556;
                 -6.570556	57.062222;
                 -6.565278	57.063889;
                 -6.558333	57.064722;
                 -6.551111	57.063889;
                 -6.543333	57.064167;
                 -6.538056	57.065833;
                 -6.531389	57.066389;
                 -6.523611	57.066667;
                 -6.515000	57.066389;
                 -6.507778	57.065556;
                 -6.501111	57.064444;
                 -6.496389	57.062500;
                 -6.493056	57.059722;
                 -6.493611	57.055556];
 
 uki(93).xy = [  -6.471389	57.038333;
                 -6.475833	57.036944;
                 -6.480833	57.035278;
                 -6.487500	57.036667;
                 -6.491389	57.038889;
                 -6.498889	57.039722;
                 -6.506667	57.039444;
                 -6.511389	57.041389;
                 -6.508611	57.044722;
                 -6.503056	57.046389;
                 -6.497500	57.045556;
                 -6.488889	57.045278;
                 -6.481389	57.045556;
                 -6.474722	57.044444;
                 -6.470833	57.041944;
                 -6.471389	57.038333];

  uki(34).xy = [ -6.146111	57.570278;
                 -6.147500	57.566389;
                 -6.148056	57.561667;
                 -6.147778	57.557500;
                 -6.147500	57.553333;
                 -6.146111	57.549722;
                 -6.145833	57.545556;
                 -6.145556	57.541389;
                 -6.145278	57.537500;
                 -6.144167	57.533611;
                 -6.143611	57.529444;
                 -6.143333	57.525278;
                 -6.143889	57.520833;
                 -6.143611	57.516667;
                 -6.144167	57.512222;
                 -6.144722	57.507500;
                 -6.145278	57.503056;
                 -6.146667	57.498889;
                 -6.147222	57.494444;
                 -6.146111	57.490556;
                 -6.142222	57.488333;
                 -6.138056	57.485833;
                 -6.135278	57.482778;
                 -6.133889	57.479167;
                 -6.133611	57.475000;
                 -6.134167	57.470556;
                 -6.134722	57.465833;
                 -6.136944	57.462222;
                 -6.138333	57.458333;
                 -6.139722	57.454167;
                 -6.141944	57.450556;
                 -6.143333	57.446667;
                 -6.144444	57.442500;
                 -6.145278	57.438056;
                 -6.146389	57.433889;
                 -6.147222	57.429167;
                 -6.148333	57.425278;
                 -6.150556	57.421667;
                 -6.153611	57.418611;
                 -6.157222	57.415833;
                 -6.162500	57.414444;
                 -6.168056	57.412500;
                 -6.172500	57.410556;
                 -6.176944	57.408333;
                 -6.181667	57.406389;
                 -6.185278	57.403611;
                 -6.188889	57.401111;
                 -6.193611	57.398889;
                 -6.196667	57.395833;
                 -6.196111	57.392778;
                 -6.189167	57.392500;
                 -6.184722	57.394444;
                 -6.180278	57.396667;
                 -6.174167	57.397778;
                 -6.166389	57.398056;
                 -6.160278	57.398056;
                 -6.153611	57.398889;
                 -6.145278	57.398611;
                 -6.140278	57.396389;
                 -6.136111	57.394167;
                 -6.132222	57.391667;
                 -6.129444	57.388611;
                 -6.129167	57.384722;
                 -6.130278	57.380556;
                 -6.133333	57.377500;
                 -6.136111	57.374444;
                 -6.140000	57.371667;
                 -6.144444	57.369722;
                 -6.147500	57.366389;
                 -6.145556	57.363333;
                 -6.142222	57.360278;
                 -6.137500	57.358333;
                 -6.132500	57.356389;
                 -6.129722	57.353611;
                 -6.126667	57.350556;
                 -6.126111	57.346389;
                 -6.126111	57.342500;
                 -6.119444	57.341111;
                 -6.110833	57.340833;
                 -6.105278	57.339167;
                 -6.106389	57.335278;
                 -6.104444	57.331944;
                 -6.102222	57.328611;
                 -6.101111	57.325000;
                 -6.102500	57.320833;
                 -6.100278	57.317500;
                 -6.098056	57.314167;
                 -6.094167	57.311944;
                 -6.090278	57.309444;
                 -6.085556	57.307500;
                 -6.078889	57.307222;
                 -6.071111	57.307778;
                 -6.064167	57.308333;
                 -6.056389	57.308611;
                 -6.053333	57.305833;
                 -6.051389	57.302500;
                 -6.050278	57.298611;
                 -6.050833	57.294167;
                 -6.051389	57.290556;
                 -6.048056	57.287778;
                 -6.047222	57.283889;
                 -6.051389	57.281944;
                 -6.056944	57.280278;
                 -6.061389	57.278056;
                 -6.065833	57.275833;
                 -6.068889	57.272778;
                 -6.071667	57.269722;
                 -6.067778	57.267222;
                 -6.061667	57.267500;
                 -6.057500	57.269722;
                 -6.052778	57.271667;
                 -6.049167	57.274444;
                 -6.044722	57.276667;
                 -6.039444	57.278056;
                 -6.033889	57.279722;
                 -6.027222	57.280556;
                 -6.019444	57.279722;
                 -6.014722	57.277778;
                 -6.010833	57.275278;
                 -6.006944	57.272778;
                 -6.002222	57.270833;
                 -5.996389	57.269167;
                 -5.989722	57.268056;
                 -5.983056	57.266944;
                 -5.977222	57.265278;
                 -5.972778	57.263333;
                 -5.966667	57.261667;
                 -5.960278	57.260556;
                 -5.954722	57.258889;
                 -5.948056	57.257778;
                 -5.940556	57.256944;
                 -5.931944	57.256667;
                 -5.926389	57.255000;
                 -5.922500	57.252500;
                 -5.918333	57.250000;
                 -5.914444	57.247778;
                 -5.910556	57.245278;
                 -5.904722	57.243611;
                 -5.900278	57.241667;
                 -5.894167	57.240000;
                 -5.887778	57.238889;
                 -5.879444	57.238611;
                 -5.871667	57.238611;
                 -5.868056	57.241111;
                 -5.864167	57.243889;
                 -5.860278	57.246389;
                 -5.854722	57.245833;
                 -5.847222	57.245000;
                 -5.841111	57.245278;
                 -5.835833	57.246944;
                 -5.831389	57.248889;
                 -5.825833	57.250556;
                 -5.819722	57.251667;
                 -5.814444	57.253333;
                 -5.808611	57.254444;
                 -5.801111	57.253611;
                 -5.795000	57.253889;
                 -5.789722	57.255556;
                 -5.785278	57.257778;
                 -5.781389	57.260278;
                 -5.777500	57.262778;
                 -5.773056	57.265000;
                 -5.769444	57.267500;
                 -5.763333	57.268611;
                 -5.755833	57.268056;
                 -5.748889	57.268611;
                 -5.744444	57.270556;
                 -5.738889	57.272222;
                 -5.732778	57.273333;
                 -5.725278	57.272778;
                 -5.718889	57.271389;
                 -5.713056	57.270000;
                 -5.716111	57.266667;
                 -5.720833	57.264722;
                 -5.725833	57.263056;
                 -5.728889	57.260833;
                 -5.722222	57.259722;
                 -5.714722	57.259722;
                 -5.709167	57.261389;
                 -5.702500	57.262222;
                 -5.694444	57.262222;
                 -5.690000	57.260278;
                 -5.683333	57.259167;
                 -5.677222	57.259167;
                 -5.671111	57.260278;
                 -5.664444	57.261111;
                 -5.657500	57.261667;
                 -5.653611	57.259167;
                 -5.650556	57.256389;
                 -5.650278	57.252222;
                 -5.652500	57.248611;
                 -5.656389	57.246111;
                 -5.659167	57.242778;
                 -5.662222	57.239722;
                 -5.663611	57.235833;
                 -5.665000	57.231667;
                 -5.665833	57.227222;
                 -5.665556	57.223056;
                 -5.664444	57.219167;
                 -5.662222	57.216111;
                 -5.661111	57.212222;
                 -5.662222	57.208056;
                 -5.665278	57.205000;
                 -5.668333	57.201944;
                 -5.672778	57.200000;
                 -5.677500	57.197778;
                 -5.682778	57.196111;
                 -5.687222	57.194167;
                 -5.693333	57.193056;
                 -5.698611	57.191389;
                 -5.703056	57.189167;
                 -5.707778	57.187222;
                 -5.711389	57.184444;
                 -5.715000	57.181944;
                 -5.719722	57.179722;
                 -5.724167	57.177778;
                 -5.729444	57.176111;
                 -5.735000	57.174444;
                 -5.739167	57.172500;
                 -5.743889	57.170278;
                 -5.749167	57.168611;
                 -5.755278	57.167500;
                 -5.762222	57.166944;
                 -5.770556	57.167222;
                 -5.777222	57.168333;
                 -5.782778	57.170000;
                 -5.789444	57.171111;
                 -5.795278	57.170000;
                 -5.794167	57.167222;
                 -5.791944	57.163889;
                 -5.792778	57.159444;
                 -5.795000	57.155833;
                 -5.797222	57.152222;
                 -5.795833	57.148611;
                 -5.793056	57.145556;
                 -5.789167	57.142778;
                 -5.787778	57.139167;
                 -5.790000	57.135556;
                 -5.794722	57.133333;
                 -5.799722	57.131667;
                 -5.803611	57.129167;
                 -5.801389	57.125833;
                 -5.800278	57.122222;
                 -5.803333	57.118889;
                 -5.807778	57.116944;
                 -5.812222	57.114722;
                 -5.815833	57.112222;
                 -5.820556	57.110000;
                 -5.825000	57.108056;
                 -5.831111	57.106944;
                 -5.839722	57.107222;
                 -5.847222	57.106944;
                 -5.852500	57.105278;
                 -5.856111	57.102778;
                 -5.858333	57.099167;
                 -5.860000	57.095000;
                 -5.862778	57.091944;
                 -5.866389	57.089444;
                 -5.870833	57.087222;
                 -5.876111	57.085833;
                 -5.880556	57.083611;
                 -5.885278	57.081389;
                 -5.888889	57.078889;
                 -5.890278	57.074722;
                 -5.890000	57.070556;
                 -5.890556	57.066111;
                 -5.892222	57.061944;
                 -5.896389	57.060000;
                 -5.901944	57.058333;
                 -5.907778	57.057222;
                 -5.913056	57.055556;
                 -5.917778	57.053333;
                 -5.920556	57.050278;
                 -5.920278	57.046111;
                 -5.920000	57.041944;
                 -5.923611	57.039444;
                 -5.928889	57.037778;
                 -5.935833	57.037222;
                 -5.942500	57.036667;
                 -5.949444	57.035833;
                 -5.955556	57.034722;
                 -5.961667	57.033056;
                 -5.966667	57.031389;
                 -5.972222	57.030000;
                 -5.977222	57.028333;
                 -5.981944	57.026111;
                 -5.986389	57.023889;
                 -5.990833	57.021944;
                 -5.995278	57.019722;
                 -5.999722	57.017778;
                 -6.004444	57.015556;
                 -6.009444	57.013889;
                 -6.015000	57.012222;
                 -6.022500	57.012222;
                 -6.027222	57.013333;
                 -6.027500	57.017500;
                 -6.026389	57.021389;
                 -6.027500	57.025278;
                 -6.029444	57.028333;
                 -6.033333	57.030833;
                 -6.035556	57.034167;
                 -6.036944	57.037778;
                 -6.038889	57.041111;
                 -6.041111	57.044444;
                 -6.040278	57.048889;
                 -6.038333	57.052500;
                 -6.036944	57.056667;
                 -6.034722	57.060278;
                 -6.031111	57.062778;
                 -6.026667	57.065000;
                 -6.021944	57.067222;
                 -6.016944	57.068611;
                 -6.013889	57.071944;
                 -6.011667	57.075556;
                 -6.011111	57.080000;
                 -6.011389	57.084167;
                 -6.011667	57.088333;
                 -6.011111	57.092778;
                 -6.009722	57.096944;
                 -6.010000	57.101111;
                 -6.011111	57.104722;
                 -6.013056	57.108056;
                 -6.013611	57.112222;
                 -6.011389	57.115833;
                 -6.006944	57.117778;
                 -6.004167	57.120000;
                 -5.998889	57.121667;
                 -5.994444	57.123889;
                 -5.988889	57.125556;
                 -5.983889	57.126944;
                 -5.978611	57.128611;
                 -5.976389	57.132222;
                 -5.973333	57.135278;
                 -5.968889	57.137500;
                 -5.964444	57.139722;
                 -5.959167	57.141111;
                 -5.953889	57.142778;
                 -5.949167	57.145000;
                 -5.945556	57.147500;
                 -5.943333	57.151111;
                 -5.939722	57.153889;
                 -5.935278	57.155833;
                 -5.929167	57.156944;
                 -5.923056	57.158056;
                 -5.916389	57.158889;
                 -5.910278	57.160000;
                 -5.904167	57.161111;
                 -5.897222	57.161667;
                 -5.891944	57.163333;
                 -5.886667	57.165000;
                 -5.880556	57.166111;
                 -5.873889	57.166667;
                 -5.865278	57.166389;
                 -5.858333	57.166944;
                 -5.852222	57.168333;
                 -5.847778	57.170278;
                 -5.842500	57.171944;
                 -5.838889	57.174444;
                 -5.835833	57.177500;
                 -5.837222	57.181389;
                 -5.843889	57.182500;
                 -5.849722	57.181389;
                 -5.855833	57.180278;
                 -5.861111	57.178611;
                 -5.866389	57.176944;
                 -5.870833	57.175000;
                 -5.875556	57.172778;
                 -5.880556	57.171111;
                 -5.888333	57.171111;
                 -5.895833	57.171667;
                 -5.902778	57.171111;
                 -5.908056	57.169444;
                 -5.913333	57.167778;
                 -5.920000	57.169167;
                 -5.924722	57.171111;
                 -5.929444	57.173056;
                 -5.935278	57.174722;
                 -5.942778	57.175556;
                 -5.948611	57.174167;
                 -5.954167	57.172778;
                 -5.960278	57.171389;
                 -5.966389	57.170278;
                 -5.972500	57.169167;
                 -5.980833	57.169444;
                 -5.984722	57.171944;
                 -5.987500	57.175000;
                 -5.990833	57.177778;
                 -5.991944	57.181389;
                 -5.992222	57.185556;
                 -5.994444	57.188889;
                 -5.998056	57.191389;
                 -6.003056	57.193333;
                 -6.006944	57.195833;
                 -6.010000	57.198611;
                 -6.011944	57.201944;
                 -6.014167	57.205278;
                 -6.016111	57.208611;
                 -6.019167	57.211389;
                 -6.025833	57.212500;
                 -6.029444	57.210000;
                 -6.029167	57.205833;
                 -6.029167	57.201667;
                 -6.028611	57.197500;
                 -6.028333	57.193333;
                 -6.028056	57.189167;
                 -6.028611	57.184722;
                 -6.029167	57.180278;
                 -6.031389	57.176667;
                 -6.037222	57.175556;
                 -6.045278	57.175278;
                 -6.050278	57.173611;
                 -6.050833	57.170000;
                 -6.049444	57.166389;
                 -6.049444	57.162222;
                 -6.051389	57.158611;
                 -6.053611	57.155000;
                 -6.056667	57.151944;
                 -6.060278	57.149167;
                 -6.063333	57.146111;
                 -6.066111	57.143056;
                 -6.069167	57.140000;
                 -6.071944	57.136944;
                 -6.073611	57.132778;
                 -6.073333	57.128611;
                 -6.076944	57.126111;
                 -6.083611	57.127222;
                 -6.090000	57.128333;
                 -6.096667	57.129444;
                 -6.103333	57.130833;
                 -6.108889	57.132222;
                 -6.109444	57.135556;
                 -6.107222	57.139167;
                 -6.105833	57.143056;
                 -6.107222	57.146944;
                 -6.109167	57.150278;
                 -6.110278	57.153889;
                 -6.109167	57.158056;
                 -6.106944	57.161667;
                 -6.104722	57.165278;
                 -6.105833	57.168889;
                 -6.107222	57.172500;
                 -6.109444	57.175833;
                 -6.111389	57.179167;
                 -6.114444	57.181944;
                 -6.118333	57.184444;
                 -6.124167	57.183333;
                 -6.131944	57.183056;
                 -6.138611	57.183333;
                 -6.143611	57.185278;
                 -6.149444	57.186944;
                 -6.156111	57.188056;
                 -6.161944	57.188056;
                 -6.165556	57.185278;
                 -6.166944	57.181111;
                 -6.167500	57.176667;
                 -6.171111	57.174167;
                 -6.174722	57.171389;
                 -6.181667	57.170833;
                 -6.189444	57.170556;
                 -6.197778	57.170833;
                 -6.206389	57.171111;
                 -6.213889	57.171111;
                 -6.221389	57.170833;
                 -6.228333	57.170278;
                 -6.234444	57.169167;
                 -6.240556	57.168056;
                 -6.246667	57.166667;
                 -6.252778	57.165556;
                 -6.258611	57.164444;
                 -6.264722	57.163333;
                 -6.272500	57.163056;
                 -6.280833	57.163333;
                 -6.289167	57.163611;
                 -6.295278	57.162500;
                 -6.301389	57.161389;
                 -6.306667	57.159722;
                 -6.311667	57.158056;
                 -6.317778	57.156944;
                 -6.324722	57.157222;
                 -6.325000	57.161389;
                 -6.321944	57.164444;
                 -6.317500	57.166667;
                 -6.313056	57.168611;
                 -6.308611	57.170833;
                 -6.304167	57.173056;
                 -6.298889	57.174722;
                 -6.294444	57.176667;
                 -6.289722	57.178889;
                 -6.286111	57.181389;
                 -6.284722	57.185556;
                 -6.285833	57.189167;
                 -6.290000	57.191667;
                 -6.298333	57.191944;
                 -6.304444	57.190833;
                 -6.310556	57.189722;
                 -6.317500	57.189167;
                 -6.323611	57.187778;
                 -6.330278	57.187222;
                 -6.336111	57.186111;
                 -6.343889	57.185833;
                 -6.351667	57.185833;
                 -6.359167	57.186389;
                 -6.363056	57.188889;
                 -6.365833	57.191667;
                 -6.368056	57.195000;
                 -6.371111	57.197778;
                 -6.376111	57.199722;
                 -6.380833	57.201944;
                 -6.385833	57.203889;
                 -6.388611	57.206667;
                 -6.392500	57.209167;
                 -6.394722	57.212500;
                 -6.396944	57.215833;
                 -6.395833	57.218889;
                 -6.390278	57.220556;
                 -6.384167	57.221667;
                 -6.377500	57.222500;
                 -6.371667	57.223611;
                 -6.366111	57.225278;
                 -6.361667	57.227222;
                 -6.358889	57.230278;
                 -6.358333	57.234167;
                 -6.364167	57.234722;
                 -6.370278	57.233611;
                 -6.376111	57.232500;
                 -6.383056	57.231667;
                 -6.390556	57.232500;
                 -6.398333	57.232222;
                 -6.406111	57.232222;
                 -6.412500	57.233333;
                 -6.418333	57.234722;
                 -6.422500	57.237222;
                 -6.425278	57.240000;
                 -6.429167	57.242500;
                 -6.432500	57.245278;
                 -6.435278	57.248333;
                 -6.439167	57.250556;
                 -6.444167	57.252778;
                 -6.448056	57.255000;
                 -6.451111	57.258056;
                 -6.455000	57.260278;
                 -6.458333	57.263333;
                 -6.461111	57.266111;
                 -6.463333	57.269444;
                 -6.463611	57.273611;
                 -6.464167	57.277778;
                 -6.463611	57.281389;
                 -6.466667	57.284167;
                 -6.473056	57.285278;
                 -6.480278	57.284722;
                 -6.485278	57.283889;
                 -6.489167	57.286389;
                 -6.489444	57.290556;
                 -6.488333	57.294444;
                 -6.486944	57.298611;
                 -6.484167	57.301667;
                 -6.481944	57.305278;
                 -6.478333	57.307778;
                 -6.474722	57.310556;
                 -6.470833	57.313056;
                 -6.466389	57.315278;
                 -6.460833	57.316944;
                 -6.456667	57.319167;
                 -6.451111	57.320556;
                 -6.445000	57.321944;
                 -6.438333	57.322500;
                 -6.434444	57.325278;
                 -6.437778	57.328056;
                 -6.440000	57.330278;
                 -6.434444	57.331944;
                 -6.426389	57.331389;
                 -6.421389	57.329167;
                 -6.417500	57.326944;
                 -6.411667	57.325278;
                 -6.405000	57.324167;
                 -6.398333	57.323056;
                 -6.394444	57.320556;
                 -6.389722	57.318611;
                 -6.386389	57.315556;
                 -6.383611	57.312778;
                 -6.381389	57.309444;
                 -6.378333	57.306667;
                 -6.374167	57.304167;
                 -6.369444	57.302222;
                 -6.364722	57.300278;
                 -6.360000	57.298333;
                 -6.353889	57.296667;
                 -6.348333	57.295000;
                 -6.340833	57.294444;
                 -6.333333	57.293611;
                 -6.325556	57.293889;
                 -6.320278	57.295556;
                 -6.325833	57.296944;
                 -6.332778	57.298056;
                 -6.338333	57.299722;
                 -6.343333	57.301667;
                 -6.347222	57.304167;
                 -6.351111	57.306667;
                 -6.354167	57.309444;
                 -6.358056	57.311944;
                 -6.361111	57.314722;
                 -6.364167	57.317500;
                 -6.368056	57.320556;
                 -6.371111	57.323333;
                 -6.376111	57.325278;
                 -6.380000	57.327778;
                 -6.383889	57.330278;
                 -6.388056	57.332778;
                 -6.391944	57.335000;
                 -6.394722	57.338056;
                 -6.400556	57.339444;
                 -6.405556	57.341667;
                 -6.407500	57.344722;
                 -6.406944	57.349444;
                 -6.403333	57.351944;
                 -6.401111	57.355556;
                 -6.406111	57.357500;
                 -6.412222	57.356389;
                 -6.415000	57.353333;
                 -6.418889	57.350278;
                 -6.422500	57.347778;
                 -6.427778	57.345833;
                 -6.434444	57.345278;
                 -6.442500	57.345278;
                 -6.447778	57.343333;
                 -6.454444	57.343611;
                 -6.460278	57.345278;
                 -6.462500	57.348611;
                 -6.460278	57.352222;
                 -6.458056	57.355833;
                 -6.461111	57.358611;
                 -6.467778	57.359722;
                 -6.476389	57.360000;
                 -6.481944	57.361667;
                 -6.481667	57.365278;
                 -6.477778	57.367778;
                 -6.474722	57.371111;
                 -6.476111	57.374722;
                 -6.480833	57.376667;
                 -6.485000	57.379167;
                 -6.488056	57.381944;
                 -6.489167	57.385833;
                 -6.490556	57.389444;
                 -6.491944	57.393056;
                 -6.495833	57.395556;
                 -6.499167	57.395556;
                 -6.499444	57.391944;
                 -6.502500	57.388889;
                 -6.506944	57.386667;
                 -6.512500	57.385000;
                 -6.516111	57.382500;
                 -6.515000	57.378889;
                 -6.514444	57.374722;
                 -6.517222	57.371667;
                 -6.522778	57.370000;
                 -6.528611	57.370556;
                 -6.529722	57.374167;
                 -6.528611	57.378333;
                 -6.528056	57.382500;
                 -6.530000	57.385833;
                 -6.530556	57.389722;
                 -6.529167	57.393889;
                 -6.528611	57.398611;
                 -6.529722	57.402222;
                 -6.534722	57.404167;
                 -6.541389	57.405278;
                 -6.548056	57.405556;
                 -6.552778	57.403333;
                 -6.550556	57.400278;
                 -6.546667	57.397778;
                 -6.542500	57.395278;
                 -6.541389	57.391667;
                 -6.545833	57.389444;
                 -6.551111	57.387778;
                 -6.554722	57.385278;
                 -6.560833	57.384167;
                 -6.566667	57.385556;
                 -6.574167	57.386389;
                 -6.575556	57.382222;
                 -6.576944	57.378056;
                 -6.576667	57.373889;
                 -6.574444	57.370833;
                 -6.571389	57.367778;
                 -6.567500	57.365556;
                 -6.566111	57.361667;
                 -6.568056	57.358056;
                 -6.571667	57.355556;
                 -6.574722	57.352500;
                 -6.573611	57.348889;
                 -6.569444	57.346389;
                 -6.566389	57.343333;
                 -6.566111	57.339167;
                 -6.569167	57.336111;
                 -6.572778	57.333611;
                 -6.576389	57.331944;
                 -6.580833	57.329722;
                 -6.588611	57.329444;
                 -6.593333	57.331667;
                 -6.598056	57.333611;
                 -6.603056	57.335556;
                 -6.606944	57.338333;
                 -6.611111	57.340833;
                 -6.615000	57.343333;
                 -6.618889	57.345556;
                 -6.626389	57.346389;
                 -6.634167	57.346111;
                 -6.641667	57.346944;
                 -6.647500	57.348611;
                 -6.651389	57.350833;
                 -6.657222	57.352500;
                 -6.665556	57.352778;
                 -6.673333	57.352500;
                 -6.680000	57.353611;
                 -6.684722	57.355833;
                 -6.689722	57.357778;
                 -6.694444	57.359722;
                 -6.700278	57.361111;
                 -6.707778	57.361944;
                 -6.716389	57.362222;
                 -6.722222	57.363889;
                 -6.726111	57.366111;
                 -6.729167	57.369167;
                 -6.732222	57.371944;
                 -6.736944	57.373889;
                 -6.740833	57.376389;
                 -6.744167	57.379167;
                 -6.744444	57.383333;
                 -6.741667	57.386389;
                 -6.738056	57.388889;
                 -6.735833	57.392778;
                 -6.738889	57.395556;
                 -6.741944	57.398333;
                 -6.744167	57.401667;
                 -6.747222	57.404444;
                 -6.749444	57.407778;
                 -6.756111	57.408889;
                 -6.761111	57.410833;
                 -6.765000	57.413333;
                 -6.768056	57.416111;
                 -6.771111	57.418889;
                 -6.776944	57.420556;
                 -6.783611	57.420833;
                 -6.791389	57.420556;
                 -6.797222	57.422222;
                 -6.801111	57.424444;
                 -6.801389	57.428611;
                 -6.801111	57.432222;
                 -6.798333	57.435278;
                 -6.791944	57.436667;
                 -6.786944	57.438333;
                 -6.787222	57.442222;
                 -6.791944	57.444444;
                 -6.797222	57.446389;
                 -6.797500	57.449444;
                 -6.793056	57.451667;
                 -6.785278	57.451944;
                 -6.780278	57.451667;
                 -6.771667	57.451389;
                 -6.765000	57.450278;
                 -6.760278	57.448333;
                 -6.756111	57.445833;
                 -6.750556	57.444444;
                 -6.743056	57.443611;
                 -6.737500	57.444444;
                 -6.735556	57.448056;
                 -6.737778	57.451389;
                 -6.742778	57.453333;
                 -6.749167	57.454444;
                 -6.753056	57.456944;
                 -6.755278	57.460000;
                 -6.756944	57.463889;
                 -6.756111	57.468333;
                 -6.757500	57.471944;
                 -6.761389	57.474444;
                 -6.764722	57.477500;
                 -6.765833	57.481111;
                 -6.763889	57.484722;
                 -6.760833	57.487778;
                 -6.758056	57.490833;
                 -6.754167	57.493611;
                 -6.749722	57.495556;
                 -6.745278	57.497778;
                 -6.740833	57.500000;
                 -6.734722	57.501111;
                 -6.727778	57.501667;
                 -6.720833	57.500556;
                 -6.716944	57.498333;
                 -6.715833	57.494444;
                 -6.716389	57.490833;
                 -6.713056	57.488056;
                 -6.709167	57.485556;
                 -6.704444	57.483611;
                 -6.697500	57.482500;
                 -6.691944	57.480833;
                 -6.690556	57.477222;
                 -6.688333	57.473889;
                 -6.684444	57.471389;
                 -6.679444	57.469444;
                 -6.674444	57.467500;
                 -6.672500	57.464167;
                 -6.672778	57.459722;
                 -6.671667	57.456111;
                 -6.667778	57.453611;
                 -6.662778	57.451667;
                 -6.656111	57.450556;
                 -6.652222	57.448056;
                 -6.650000	57.444722;
                 -6.646111	57.442222;
                 -6.642778	57.439444;
                 -6.638889	57.436944;
                 -6.634167	57.435000;
                 -6.627500	57.433889;
                 -6.621944	57.435556;
                 -6.616667	57.437222;
                 -6.609167	57.436389;
                 -6.605278	57.434167;
                 -6.600278	57.431944;
                 -6.596389	57.429722;
                 -6.593333	57.426667;
                 -6.588056	57.426667;
                 -6.585556	57.429722;
                 -6.584722	57.434444;
                 -6.585833	57.438056;
                 -6.588056	57.441389;
                 -6.591389	57.444167;
                 -6.595278	57.446667;
                 -6.599167	57.448889;
                 -6.605833	57.450000;
                 -6.612500	57.451389;
                 -6.611944	57.455833;
                 -6.611667	57.460278;
                 -6.615556	57.462778;
                 -6.620278	57.464722;
                 -6.623611	57.467778;
                 -6.626389	57.470556;
                 -6.626111	57.475000;
                 -6.626389	57.479167;
                 -6.628611	57.482500;
                 -6.630833	57.485833;
                 -6.632778	57.489167;
                 -6.633333	57.493056;
                 -6.629722	57.495833;
                 -6.621944	57.496111;
                 -6.614167	57.496111;
                 -6.611389	57.499444;
                 -6.608333	57.502500;
                 -6.600556	57.502500;
                 -6.595000	57.501111;
                 -6.590833	57.498611;
                 -6.586944	57.496111;
                 -6.583611	57.493333;
                 -6.577778	57.492778;
                 -6.573611	57.494722;
                 -6.571389	57.498333;
                 -6.571667	57.502500;
                 -6.573889	57.505833;
                 -6.578611	57.507778;
                 -6.583611	57.509722;
                 -6.588611	57.511944;
                 -6.592500	57.514167;
                 -6.595556	57.517222;
                 -6.598611	57.520000;
                 -6.601944	57.522778;
                 -6.606667	57.524722;
                 -6.611667	57.526944;
                 -6.614722	57.528611;
                 -6.618611	57.531111;
                 -6.620833	57.534444;
                 -6.623056	57.537778;
                 -6.628056	57.539722;
                 -6.633889	57.541389;
                 -6.637778	57.543611;
                 -6.641667	57.546111;
                 -6.642222	57.550278;
                 -6.642222	57.554444;
                 -6.641944	57.558889;
                 -6.639722	57.562500;
                 -6.638333	57.566667;
                 -6.637222	57.570833;
                 -6.638333	57.574444;
                 -6.638611	57.578611;
                 -6.640000	57.582222;
                 -6.642222	57.585556;
                 -6.644167	57.588889;
                 -6.645833	57.592500;
                 -6.646111	57.596667;
                 -6.644167	57.600278;
                 -6.638611	57.601944;
                 -6.631944	57.602500;
                 -6.626111	57.601111;
                 -6.623611	57.598611;
                 -6.620556	57.595833;
                 -6.615833	57.593889;
                 -6.610000	57.592222;
                 -6.603889	57.590556;
                 -6.599167	57.588611;
                 -6.595833	57.585833;
                 -6.593056	57.583056;
                 -6.590833	57.579722;
                 -6.588611	57.576389;
                 -6.587500	57.572778;
                 -6.585278	57.569444;
                 -6.583889	57.565833;
                 -6.582500	57.561944;
                 -6.582222	57.557778;
                 -6.581667	57.553611;
                 -6.580556	57.550000;
                 -6.577500	57.547222;
                 -6.570833	57.546111;
                 -6.563333	57.546111;
                 -6.557500	57.544722;
                 -6.553333	57.542222;
                 -6.548611	57.539722;
                 -6.543611	57.537778;
                 -6.538889	57.535833;
                 -6.535000	57.533333;
                 -6.530000	57.531389;
                 -6.525000	57.529444;
                 -6.521111	57.526944;
                 -6.516111	57.525000;
                 -6.511111	57.523056;
                 -6.506389	57.520833;
                 -6.502500	57.518611;
                 -6.497500	57.516389;
                 -6.493611	57.514167;
                 -6.489444	57.511667;
                 -6.485556	57.509167;
                 -6.481667	57.506944;
                 -6.477778	57.504444;
                 -6.475556	57.501111;
                 -6.471667	57.498611;
                 -6.464722	57.498333;
                 -6.460833	57.501111;
                 -6.458333	57.504167;
                 -6.456111	57.507778;
                 -6.453056	57.510833;
                 -6.450278	57.513889;
                 -6.445556	57.516111;
                 -6.438056	57.516389;
                 -6.433056	57.514167;
                 -6.434444	57.510278;
                 -6.437222	57.507222;
                 -6.439444	57.503611;
                 -6.441667	57.500000;
                 -6.444444	57.496944;
                 -6.446667	57.493333;
                 -6.449444	57.490000;
                 -6.451667	57.486667;
                 -6.452500	57.481944;
                 -6.450278	57.478611;
                 -6.446111	57.476111;
                 -6.438611	57.475556;
                 -6.435000	57.477222;
                 -6.431111	57.479722;
                 -6.430000	57.483889;
                 -6.428333	57.488056;
                 -6.427222	57.491944;
                 -6.423333	57.494722;
                 -6.419722	57.497222;
                 -6.416667	57.500556;
                 -6.413889	57.503611;
                 -6.414444	57.507778;
                 -6.415556	57.511389;
                 -6.414167	57.515556;
                 -6.411944	57.518889;
                 -6.410000	57.522500;
                 -6.405833	57.521111;
                 -6.400278	57.519167;
                 -6.396944	57.516111;
                 -6.394722	57.513056;
                 -6.393611	57.509167;
                 -6.391389	57.505833;
                 -6.389167	57.502778;
                 -6.387222	57.499444;
                 -6.381944	57.497500;
                 -6.376389	57.495833;
                 -6.370556	57.494167;
                 -6.366667	57.491944;
                 -6.362778	57.489444;
                 -6.358889	57.486944;
                 -6.353056	57.485278;
                 -6.348056	57.483333;
                 -6.345000	57.480556;
                 -6.341944	57.477778;
                 -6.338889	57.474722;
                 -6.333889	57.472778;
                 -6.327778	57.473889;
                 -6.324167	57.476667;
                 -6.321944	57.480000;
                 -6.324167	57.483333;
                 -6.328333	57.485833;
                 -6.333056	57.487778;
                 -6.337778	57.489722;
                 -6.343889	57.491389;
                 -6.348611	57.493333;
                 -6.351667	57.496389;
                 -6.354722	57.499167;
                 -6.357778	57.501944;
                 -6.361944	57.504444;
                 -6.365833	57.506944;
                 -6.369722	57.509167;
                 -6.373611	57.511667;
                 -6.377778	57.514167;
                 -6.381667	57.516667;
                 -6.385556	57.518889;
                 -6.388611	57.521944;
                 -6.391667	57.524722;
                 -6.392778	57.528611;
                 -6.393333	57.532500;
                 -6.392500	57.537222;
                 -6.394167	57.540833;
                 -6.396944	57.543611;
                 -6.400278	57.546667;
                 -6.403056	57.549444;
                 -6.405278	57.552778;
                 -6.406389	57.556389;
                 -6.403611	57.559444;
                 -6.397500	57.560556;
                 -6.391389	57.561944;
                 -6.387778	57.564444;
                 -6.384722	57.567500;
                 -6.381944	57.570556;
                 -6.378889	57.573889;
                 -6.376111	57.576944;
                 -6.373056	57.580000;
                 -6.379722	57.581111;
                 -6.385833	57.580000;
                 -6.392500	57.580278;
                 -6.397500	57.582222;
                 -6.400278	57.585000;
                 -6.403611	57.588056;
                 -6.405833	57.591111;
                 -6.406111	57.595278;
                 -6.404722	57.599444;
                 -6.404167	57.603056;
                 -6.406389	57.606389;
                 -6.409444	57.609167;
                 -6.411667	57.612500;
                 -6.413889	57.615833;
                 -6.414444	57.620000;
                 -6.415556	57.623611;
                 -6.419444	57.626111;
                 -6.424444	57.628056;
                 -6.427500	57.630833;
                 -6.429444	57.634167;
                 -6.427500	57.637778;
                 -6.422778	57.640000;
                 -6.416667	57.641111;
                 -6.413056	57.643611;
                 -6.411667	57.647778;
                 -6.406944	57.650000;
                 -6.400278	57.650556;
                 -6.393333	57.651111;
                 -6.387222	57.652222;
                 -6.381944	57.654167;
                 -6.378056	57.656667;
                 -6.375000	57.659722;
                 -6.371944	57.662778;
                 -6.368333	57.665556;
                 -6.364722	57.668056;
                 -6.360833	57.670833;
                 -6.357778	57.673889;
                 -6.355000	57.676944;
                 -6.352778	57.680556;
                 -6.352222	57.685000;
                 -6.354444	57.688333;
                 -6.361944	57.689167;
                 -6.368889	57.690278;
                 -6.372778	57.692500;
                 -6.371667	57.695833;
                 -6.366111	57.697500;
                 -6.361667	57.699444;
                 -6.355833	57.698333;
                 -6.350833	57.696389;
                 -6.345000	57.694722;
                 -6.338333	57.693611;
                 -6.332222	57.694722;
                 -6.326111	57.696111;
                 -6.319167	57.696667;
                 -6.311389	57.695833;
                 -6.306667	57.693889;
                 -6.304444	57.690556;
                 -6.305833	57.686667;
                 -6.306944	57.684444;
                 -6.300278	57.683056;
                 -6.293333	57.681944;
                 -6.288611	57.680000;
                 -6.283611	57.678056;
                 -6.279722	57.675556;
                 -6.275833	57.673056;
                 -6.272778	57.670278;
                 -6.266944	57.668611;
                 -6.260278	57.667500;
                 -6.254444	57.665833;
                 -6.249444	57.663889;
                 -6.245556	57.661667;
                 -6.242500	57.658611;
                 -6.239444	57.655833;
                 -6.237222	57.652500;
                 -6.236944	57.648333;
                 -6.236667	57.644167;
                 -6.237222	57.639722;
                 -6.238611	57.635556;
                 -6.235556	57.632778;
                 -6.230833	57.630833;
                 -6.223889	57.629444;
                 -6.216389	57.628889;
                 -6.208611	57.628056;
                 -6.202778	57.626389;
                 -6.198056	57.624444;
                 -6.193056	57.622500;
                 -6.189167	57.620000;
                 -6.185278	57.617778;
                 -6.181389	57.615278;
                 -6.176389	57.613056;
                 -6.175278	57.609444;
                 -6.173889	57.605833;
                 -6.170833	57.602778;
                 -6.168611	57.599722;
                 -6.165833	57.596667;
                 -6.163611	57.593333;
                 -6.160278	57.590556;
                 -6.159167	57.586944;
                 -6.156389	57.584167;
                 -6.151389	57.581944;
                 -6.146389	57.580000;
                 -6.144167	57.576667;
                 -6.146111	57.570278];


  uki(35).xy = [ -5.823611	57.340833;
                 -5.819722	57.339167;
                 -5.815833	57.336667;
                 -5.813056	57.333889;
                 -5.816667	57.331389;
                 -5.824167	57.331111;
                 -5.830000	57.332778;
                 -5.833889	57.335278;
                 -5.837778	57.337778;
                 -5.841667	57.340000;
                 -5.843889	57.343333;
                 -5.841667	57.346111;
                 -5.834167	57.346111;
                 -5.829444	57.344167;
                 -5.826111	57.342778;
                 -5.823611	57.340833];


  uki(36).xy = [ -5.836389	57.273889;
                 -5.835000	57.271944;
                 -5.837778	57.268889;
                 -5.843333	57.267222;
                 -5.849444	57.266111;
                 -5.856944	57.266111;
                 -5.860000	57.268889;
                 -5.859444	57.272500;
                 -5.855833	57.275278;
                 -5.850278	57.276667;
                 -5.843611	57.277500;
                 -5.835833	57.276667;
                 -5.836389	57.273889];

  uki(37).xy = [ -5.917778	57.298611;
                 -5.917500	57.294444;
                 -5.917222	57.290278;
                 -5.920278	57.287222;
                 -5.923056	57.284167;
                 -5.927778	57.282222;
                 -5.932778	57.280556;
                 -5.937500	57.278333;
                 -5.942222	57.276111;
                 -5.946667	57.274167;
                 -5.951111	57.271944;
                 -5.958056	57.271389;
                 -5.966389	57.271667;
                 -5.974722	57.271944;
                 -5.981389	57.273056;
                 -5.986389	57.275278;
                 -5.990278	57.277778;
                 -5.995000	57.279722;
                 -5.998889	57.282222;
                 -6.003056	57.284444;
                 -6.005833	57.287500;
                 -6.009722	57.290000;
                 -6.013611	57.292222;
                 -6.015833	57.295556;
                 -6.017778	57.298889;
                 -6.018333	57.303056;
                 -6.018611	57.307222;
                 -6.018889	57.311389;
                 -6.017222	57.315278;
                 -6.014444	57.318333;
                 -6.009167	57.320556;
                 -6.005556	57.322222;
                 -5.999444	57.323333;
                 -5.991667	57.323611;
                 -5.985000	57.322222;
                 -5.978611	57.321111;
                 -5.970833	57.320278;
                 -5.965278	57.318333;
                 -5.960556	57.316389;
                 -5.955556	57.314444;
                 -5.948889	57.313056;
                 -5.941389	57.312500;
                 -5.935833	57.310833;
                 -5.930000	57.309167;
                 -5.925000	57.307222;
                 -5.922222	57.304444;
                 -5.919167	57.301389;
                 -5.918056	57.297778;
                 -5.917778	57.298611];


  uki(38).xy = [ -5.955278	57.572778;
                 -5.957222	57.569167;
                 -5.958611	57.565278;
                 -5.960833	57.561389;
                 -5.961667	57.556944;
                 -5.963889	57.553056;
                 -5.966111	57.549444;
                 -5.968333	57.545833;
                 -5.967778	57.541667;
                 -5.968611	57.537222;
                 -5.970000	57.533056;
                 -5.972222	57.529444;
                 -5.972778	57.524722;
                 -5.973333	57.520278;
                 -5.972778	57.516111;
                 -5.976111	57.513056;
                 -5.980556	57.511111;
                 -5.985000	57.508889;
                 -5.989444	57.507778;
                 -5.985000	57.509722;
                 -5.988889	57.512222;
                 -5.993611	57.514167;
                 -5.996944	57.517222;
                 -5.998889	57.520278;
                 -6.001111	57.523611;
                 -6.002222	57.527500;
                 -6.003333	57.531111;
                 -6.005556	57.534444;
                 -6.005000	57.538889;
                 -6.003611	57.543056;
                 -5.999722	57.545556;
                 -5.995278	57.547778;
                 -5.991389	57.550278;
                 -5.993611	57.553611;
                 -5.996667	57.556389;
                 -5.999722	57.559444;
                 -5.999167	57.563056;
                 -5.994722	57.565278;
                 -5.988611	57.566389;
                 -5.982500	57.567500;
                 -5.977778	57.569444;
                 -5.975556	57.573056;
                 -5.972778	57.576111;
                 -5.968056	57.578333;
                 -5.964444	57.580833;
                 -5.959722	57.583056;
                 -5.955833	57.580556;
                 -5.955556	57.576389;
                 -5.955278	57.572778];


  uki(39).xy = [ -5.981944	57.486667;
                 -5.982500	57.483056;
                 -5.984722	57.479444;
                 -5.987500	57.476389;
                 -5.990833	57.473333;
                 -5.993611	57.470278;
                 -5.995000	57.466111;
                 -5.994722	57.461944;
                 -5.996944	57.458611;
                 -5.999722	57.455278;
                 -6.002778	57.452222;
                 -6.005833	57.449167;
                 -6.009444	57.446667;
                 -6.013056	57.443889;
                 -6.017222	57.441389;
                 -6.020833	57.438889;
                 -6.023056	57.435278;
                 -6.025278	57.431667;
                 -6.026667	57.427500;
                 -6.026111	57.423333;
                 -6.024167	57.420000;
                 -6.023611	57.415833;
                 -6.025278	57.411944;
                 -6.027222	57.408333;
                 -6.029444	57.404722;
                 -6.027222	57.401389;
                 -6.023333	57.398889;
                 -6.019444	57.396389;
                 -6.016389	57.393611;
                 -6.015278	57.390000;
                 -6.015833	57.385556;
                 -6.014722	57.381667;
                 -6.013333	57.378056;
                 -6.011111	57.374722;
                 -6.008333	57.371944;
                 -6.004444	57.369444;
                 -5.999444	57.367222;
                 -5.995556	57.365000;
                 -5.992778	57.361944;
                 -5.990556	57.358611;
                 -5.989167	57.355000;
                 -5.990000	57.350556;
                 -5.991111	57.346389;
                 -5.995000	57.343889;
                 -6.000278	57.342222;
                 -6.005556	57.340556;
                 -6.009167	57.338056;
                 -6.012222	57.334722;
                 -6.016944	57.332778;
                 -6.022222	57.331111;
                 -6.029167	57.330556;
                 -6.035278	57.329167;
                 -6.039722	57.327222;
                 -6.045000	57.325556;
                 -6.051667	57.325000;
                 -6.059444	57.324722;
                 -6.065278	57.326389;
                 -6.070000	57.328333;
                 -6.071389	57.331944;
                 -6.069722	57.336111;
                 -6.067500	57.339722;
                 -6.066389	57.343889;
                 -6.068333	57.346944;
                 -6.073333	57.349167;
                 -6.080833	57.349722;
                 -6.083889	57.352778;
                 -6.082500	57.356667;
                 -6.081111	57.360833;
                 -6.079722	57.365000;
                 -6.077500	57.368611;
                 -6.074722	57.371667;
                 -6.072500	57.375278;
                 -6.071667	57.379722;
                 -6.073056	57.383333;
                 -6.074167	57.387222;
                 -6.076389	57.390556;
                 -6.077500	57.394167;
                 -6.077778	57.398333;
                 -6.076667	57.402222;
                 -6.074444	57.405833;
                 -6.073056	57.410000;
                 -6.072500	57.414722;
                 -6.073611	57.418333;
                 -6.075000	57.421944;
                 -6.074167	57.426667;
                 -6.071944	57.430278;
                 -6.070833	57.434167;
                 -6.070000	57.438889;
                 -6.068889	57.442778;
                 -6.065833	57.445833;
                 -6.062778	57.448889;
                 -6.059167	57.451667;
                 -6.053056	57.452778;
                 -6.045278	57.453056;
                 -6.036944	57.452500;
                 -6.029167	57.452778;
                 -6.023611	57.454444;
                 -6.020000	57.456944;
                 -6.016389	57.459722;
                 -6.014167	57.463333;
                 -6.011111	57.466389;
                 -6.007222	57.468889;
                 -6.006111	57.473056;
                 -6.006389	57.477222;
                 -6.005556	57.481667;
                 -6.002778	57.484722;
                 -6.000556	57.488333;
                 -5.998333	57.491944;
                 -5.995278	57.495000;
                 -5.988611	57.495556;
                 -5.983611	57.493611;
                 -5.981389	57.490278;
                 -5.981944	57.486667];


  uki(40).xy = [ -6.366389	57.896111;
                 -6.369444	57.893056;
                 -6.370000	57.889444;
                 -6.363889	57.887778;
                 -6.361111	57.885000;
                 -6.357778	57.882222;
                 -6.357500	57.878056;
                 -6.359722	57.874444;
                 -6.367500	57.875278;
                 -6.372222	57.877222;
                 -6.374444	57.880556;
                 -6.374722	57.884444;
                 -6.376944	57.887778;
                 -6.381944	57.890000;
                 -6.387778	57.891389;
                 -6.392500	57.893333;
                 -6.395000	57.895833;
                 -6.388611	57.896944;
                 -6.382500	57.898056;
                 -6.376389	57.899167;
                 -6.371667	57.897222;
                 -6.366389	57.896111];


  uki(41).xy = [ -6.430556	57.737222;
                 -6.425833	57.735278;
                 -6.421944	57.732778;
                 -6.422500	57.729167;
                 -6.428333	57.728056;
                 -6.436111	57.728611;
                 -6.441944	57.730278;
                 -6.447778	57.731944;
                 -6.451667	57.734167;
                 -6.452778	57.738056;
                 -6.447500	57.739722;
                 -6.440000	57.739722;
                 -6.435000	57.737778;
                 -6.430556	57.737222];


  uki(42).xy = [ -5.618056	57.834444;
                 -5.613889	57.832778;
                 -5.607778	57.831111;
                 -5.603056	57.829167;
                 -5.600833	57.825833;
                 -5.603889	57.822778;
                 -5.609444	57.821111;
                 -5.616111	57.820556;
                 -5.623056	57.821667;
                 -5.628056	57.823611;
                 -5.630833	57.826667;
                 -5.634167	57.829444;
                 -5.636111	57.832778;
                 -5.638056	57.836111;
                 -5.639444	57.839722;
                 -5.637222	57.842500;
                 -5.631389	57.841667;
                 -5.628333	57.838889;
                 -5.624167	57.836389;
                 -5.618056	57.834444];


  uki(43).xy = [ -5.464722	57.891667;
                 -5.463333	57.888889;
                 -5.460278	57.886111;
                 -5.458333	57.882778;
                 -5.456944	57.878889;
                 -5.458611	57.875000;
                 -5.460556	57.878333;
                 -5.466667	57.880000;
                 -5.471111	57.881944;
                 -5.473333	57.885278;
                 -5.475556	57.888611;
                 -5.477500	57.891667;
                 -5.476111	57.895833;
                 -5.471111	57.894722;
                 -5.466944	57.892222;
                 -5.464722	57.891667];


  uki(44).xy = [ -5.222778	57.938056;
                 -5.228889	57.939722;
                 -5.230833	57.943056;
                 -5.230278	57.946667;
                 -5.225556	57.948889;
                 -5.218611	57.949444;
                 -5.211944	57.948333;
                 -5.208611	57.945278;
                 -5.211944	57.942222;
                 -5.212500	57.938611;
                 -5.218611	57.939444;
                 -5.222778	57.938056];


  uki(45).xy = [ -5.400556	58.003056;
                 -5.406667	58.001944;
                 -5.413889	58.001389;
                 -5.419722	58.003056;
                 -5.424722	58.002222;
                 -5.430556	58.003889;
                 -5.434444	58.006389;
                 -5.434722	58.010556;
                 -5.432500	58.014167;
                 -5.430556	58.017778;
                 -5.427222	58.020833;
                 -5.424444	58.023889;
                 -5.417222	58.024444;
                 -5.411389	58.022778;
                 -5.405556	58.021111;
                 -5.403333	58.018889;
                 -5.404722	58.015556;
                 -5.410278	58.014167;
                 -5.410000	58.010000;
                 -5.402222	58.009167;
                 -5.395556	58.008056;
                 -5.393333	58.005556;
                 -5.398056	58.003333;
                 -5.400556	58.003056];


  uki(46).xy = [ -5.428056	57.970278;
                 -5.434167	57.970278;
                 -5.442500	57.970556;
                 -5.445556	57.973333;
                 -5.446667	57.977222;
                 -5.442778	57.979722;
                 -5.434444	57.979444;
                 -5.428333	57.977778;
                 -5.424444	57.975278;
                 -5.419722	57.973333;
                 -5.421944	57.970556;
                 -5.428056	57.970278];


  uki(47).xy = [ -5.498889	57.951667;
                 -5.505833	57.952222;
                 -5.512778	57.952500;
                 -5.520278	57.953056;
                 -5.527222	57.954444;
                 -5.525556	57.957500;
                 -5.521111	57.959722;
                 -5.517222	57.962222;
                 -5.513333	57.964722;
                 -5.507222	57.965833;
                 -5.503056	57.964444;
                 -5.503611	57.959722;
                 -5.500833	57.956944;
                 -5.495833	57.955000;
                 -5.491944	57.952500;
                 -5.493056	57.950278;
                 -5.498889	57.951667];


  uki(48).xy = [ -5.161944	58.371944;
                 -5.165833	58.369167;
                 -5.170278	58.367222;
                 -5.177500	58.366667;
                 -5.183333	58.368333;
                 -5.186389	58.371111;
                 -5.186389	58.375278;
                 -5.184444	58.378889;
                 -5.180556	58.381389;
                 -5.175000	58.383056;
                 -5.168611	58.384167;
                 -5.160556	58.384167;
                 -5.155833	58.382222;
                 -5.153889	58.378889;
                 -5.155278	58.374722;
                 -5.161944	58.371944];


  uki(49).xy = [ -6.162500	58.405000;
                 -6.164722	58.402222;
                 -6.168611	58.399722;
                 -6.173333	58.397500;
                 -6.176389	58.394444;
                 -6.180278	58.391667;
                 -6.183056	58.388611;
                 -6.186389	58.385556;
                 -6.189167	58.382500;
                 -6.192222	58.379444;
                 -6.194444	58.375833;
                 -6.197500	58.372778;
                 -6.199722	58.369167;
                 -6.201944	58.365556;
                 -6.203333	58.361389;
                 -6.203889	58.356944;
                 -6.201667	58.353611;
                 -6.199444	58.350278;
                 -6.198056	58.348611;
                 -6.193889	58.346111;
                 -6.190000	58.343611;
                 -6.184167	58.341944;
                 -6.178889	58.340000;
                 -6.173056	58.338333;
                 -6.166389	58.337222;
                 -6.160278	58.335556;
                 -6.154444	58.334167;
                 -6.152222	58.330833;
                 -6.155278	58.327778;
                 -6.161667	58.326667;
                 -6.169444	58.326389;
                 -6.175833	58.325278;
                 -6.181944	58.324167;
                 -6.187500	58.322500;
                 -6.192778	58.320833;
                 -6.196667	58.318333;
                 -6.200556	58.315556;
                 -6.204444	58.313056;
                 -6.208333	58.310278;
                 -6.213611	58.308611;
                 -6.218889	58.307222;
                 -6.223611	58.305000;
                 -6.228333	58.302778;
                 -6.232222	58.300278;
                 -6.233611	58.296111;
                 -6.231111	58.292778;
                 -6.231667	58.289167;
                 -6.237222	58.287500;
                 -6.242778	58.285833;
                 -6.248889	58.284722;
                 -6.255833	58.284167;
                 -6.262222	58.283056;
                 -6.266944	58.280833;
                 -6.271389	58.278611;
                 -6.275278	58.276111;
                 -6.277500	58.272500;
                 -6.278056	58.268056;
                 -6.278611	58.264444;
                 -6.281389	58.261389;
                 -6.285278	58.258611;
                 -6.291389	58.257500;
                 -6.297778	58.256389;
                 -6.303889	58.255278;
                 -6.310278	58.254167;
                 -6.315000	58.251944;
                 -6.316111	58.247778;
                 -6.316667	58.244167;
                 -6.315556	58.240556;
                 -6.314167	58.236667;
                 -6.314722	58.232222;
                 -6.319444	58.230000;
                 -6.325000	58.228611;
                 -6.331667	58.227778;
                 -6.338889	58.227222;
                 -6.345278	58.225833;
                 -6.347500	58.222500;
                 -6.348611	58.219167;
                 -6.353333	58.217222;
                 -6.355556	58.213611;
                 -6.353889	58.210833;
                 -6.346111	58.210000;
                 -6.340000	58.211111;
                 -6.331389	58.210833;
                 -6.324722	58.209722;
                 -6.318889	58.208056;
                 -6.312778	58.206389;
                 -6.307778	58.204444;
                 -6.301944	58.203056;
                 -6.297222	58.200833;
                 -6.290278	58.199722;
                 -6.282500	58.200000;
                 -6.276111	58.201111;
                 -6.270833	58.202778;
                 -6.265278	58.204444;
                 -6.260833	58.206389;
                 -6.256111	58.208611;
                 -6.252222	58.211389;
                 -6.246944	58.212778;
                 -6.238889	58.213056;
                 -6.231944	58.213611;
                 -6.226667	58.215278;
                 -6.221944	58.217500;
                 -6.218333	58.220000;
                 -6.214444	58.222778;
                 -6.210556	58.225278;
                 -6.205833	58.227500;
                 -6.202222	58.230000;
                 -6.198889	58.233056;
                 -6.196111	58.236111;
                 -6.192222	58.238889;
                 -6.185278	58.239444;
                 -6.177222	58.239722;
                 -6.171111	58.240833;
                 -6.166389	58.243056;
                 -6.162500	58.245556;
                 -6.159722	58.248611;
                 -6.155833	58.251389;
                 -6.151944	58.253889;
                 -6.145833	58.255000;
                 -6.138889	58.253889;
                 -6.135000	58.251389;
                 -6.131944	58.248611;
                 -6.130556	58.245000;
                 -6.132778	58.241389;
                 -6.135833	58.238333;
                 -6.136944	58.235000;
                 -6.134167	58.232222;
                 -6.132500	58.228611;
                 -6.135833	58.225278;
                 -6.140278	58.223333;
                 -6.141667	58.220000;
                 -6.138333	58.217222;
                 -6.134444	58.214722;
                 -6.132222	58.211389;
                 -6.134444	58.208056;
                 -6.138333	58.205278;
                 -6.143056	58.203056;
                 -6.147778	58.201111;
                 -6.153056	58.199444;
                 -6.158333	58.197778;
                 -6.163889	58.196111;
                 -6.169167	58.194444;
                 -6.174722	58.193056;
                 -6.180833	58.191667;
                 -6.186389	58.190000;
                 -6.190833	58.188056;
                 -6.194167	58.185000;
                 -6.196944	58.181944;
                 -6.200000	58.178611;
                 -6.205556	58.176944;
                 -6.211667	58.175833;
                 -6.218056	58.174722;
                 -6.223333	58.173056;
                 -6.229444	58.171944;
                 -6.236667	58.171389;
                 -6.243333	58.171667;
                 -6.248611	58.173611;
                 -6.252500	58.176111;
                 -6.255556	58.178889;
                 -6.259444	58.181389;
                 -6.262778	58.184167;
                 -6.266667	58.186667;
                 -6.271667	58.188611;
                 -6.276667	58.190556;
                 -6.284444	58.191389;
                 -6.290556	58.190278;
                 -6.296667	58.190000;
                 -6.303611	58.190278;
                 -6.309722	58.188889;
                 -6.315278	58.187500;
                 -6.319167	58.184722;
                 -6.323056	58.182222;
                 -6.326667	58.179444;
                 -6.333611	58.178889;
                 -6.339722	58.180556;
                 -6.343611	58.182778;
                 -6.347778	58.185278;
                 -6.349722	58.188611;
                 -6.353056	58.191389;
                 -6.355833	58.194444;
                 -6.360000	58.196667;
                 -6.365000	58.198889;
                 -6.371944	58.200000;
                 -6.379722	58.199722;
                 -6.386111	58.198611;
                 -6.388889	58.195556;
                 -6.388611	58.191389;
                 -6.387222	58.187500;
                 -6.385000	58.184444;
                 -6.381944	58.181389;
                 -6.376111	58.180000;
                 -6.371111	58.177778;
                 -6.373333	58.174444;
                 -6.375556	58.170833;
                 -6.374167	58.166944;
                 -6.370000	58.164722;
                 -6.367222	58.161667;
                 -6.368333	58.157778;
                 -6.367222	58.153889;
                 -6.363333	58.151667;
                 -6.359167	58.149167;
                 -6.357778	58.145278;
                 -6.360000	58.141944;
                 -6.362222	58.138333;
                 -6.360833	58.135556;
                 -6.353333	58.134722;
                 -6.351944	58.131111;
                 -6.353333	58.126944;
                 -6.356944	58.124167;
                 -6.361667	58.122222;
                 -6.368056	58.121111;
                 -6.371667	58.118333;
                 -6.378611	58.117778;
                 -6.386389	58.118333;
                 -6.393056	58.119722;
                 -6.400278	58.119722;
                 -6.403889	58.117222;
                 -6.408333	58.115000;
                 -6.413889	58.113333;
                 -6.419167	58.111944;
                 -6.426389	58.111111;
                 -6.431667	58.109444;
                 -6.434444	58.106389;
                 -6.436667	58.102778;
                 -6.441389	58.100556;
                 -6.446111	58.098611;
                 -6.450556	58.096389;
                 -6.455278	58.094167;
                 -6.462222	58.093611;
                 -6.470000	58.093333;
                 -6.476944	58.092778;
                 -6.482500	58.091111;
                 -6.488889	58.090000;
                 -6.495556	58.089167;
                 -6.502778	58.088611;
                 -6.508056	58.086944;
                 -6.512778	58.084722;
                 -6.518056	58.083056;
                 -6.522778	58.081111;
                 -6.528056	58.079444;
                 -6.534167	58.078333;
                 -6.529722	58.079444;
                 -6.524444	58.081111;
                 -6.517222	58.081667;
                 -6.510278	58.082500;
                 -6.504167	58.083611;
                 -6.496944	58.084167;
                 -6.490833	58.085556;
                 -6.483889	58.086111;
                 -6.475278	58.085833;
                 -6.466667	58.085556;
                 -6.458611	58.085833;
                 -6.452222	58.084444;
                 -6.445278	58.083333;
                 -6.438611	58.082222;
                 -6.430000	58.081944;
                 -6.426667	58.085000;
                 -6.424722	58.088611;
                 -6.422500	58.092222;
                 -6.417222	58.093889;
                 -6.410278	58.094444;
                 -6.406389	58.097222;
                 -6.401944	58.099167;
                 -6.395556	58.100556;
                 -6.387778	58.100556;
                 -6.380000	58.100000;
                 -6.379722	58.096667;
                 -6.385000	58.095000;
                 -6.388611	58.092500;
                 -6.382778	58.090833;
                 -6.386111	58.087778;
                 -6.388056	58.084167;
                 -6.383611	58.084444;
                 -6.380278	58.083333;
                 -6.378056	58.080278;
                 -6.376944	58.076389;
                 -6.375833	58.072778;
                 -6.371667	58.070278;
                 -6.367500	58.067778;
                 -6.368889	58.063889;
                 -6.368333	58.060556;
                 -6.364444	58.058056;
                 -6.363333	58.054444;
                 -6.363889	58.050000;
                 -6.367500	58.047222;
                 -6.371389	58.044722;
                 -6.374167	58.041667;
                 -6.370278	58.039167;
                 -6.368056	58.035833;
                 -6.369444	58.031944;
                 -6.372500	58.028611;
                 -6.374722	58.025000;
                 -6.376667	58.021667;
                 -6.378889	58.018056;
                 -6.381944	58.014722;
                 -6.385000	58.011667;
                 -6.387222	58.008056;
                 -6.390000	58.005000;
                 -6.393889	58.002500;
                 -6.398333	58.000278;
                 -6.405556	57.999722;
                 -6.410278	58.001667;
                 -6.416389	58.003056;
                 -6.422222	58.004722;
                 -6.430556	58.005000;
                 -6.437778	58.004444;
                 -6.444444	58.003611;
                 -6.450556	58.005278;
                 -6.456389	58.006944;
                 -6.464167	58.006667;
                 -6.470833	58.007778;
                 -6.476944	58.009444;
                 -6.484722	58.010000;
                 -6.490000	58.008611;
                 -6.495278	58.006944;
                 -6.502222	58.006111;
                 -6.509167	58.006389;
                 -6.515000	58.008056;
                 -6.520833	58.009444;
                 -6.526667	58.011111;
                 -6.531667	58.013056;
                 -6.537500	58.014722;
                 -6.544167	58.015833;
                 -6.549722	58.014167;
                 -6.548056	58.012222;
                 -6.543333	58.010278;
                 -6.539444	58.007778;
                 -6.534167	58.005833;
                 -6.529167	58.003889;
                 -6.524444	58.001944;
                 -6.519444	58.000000;
                 -6.511667	57.999167;
                 -6.505000	57.998056;
                 -6.498889	57.996389;
                 -6.493056	57.995000;
                 -6.488333	57.992778;
                 -6.483333	57.990833;
                 -6.478611	57.988889;
                 -6.473333	57.986944;
                 -6.470556	57.983889;
                 -6.466389	57.981667;
                 -6.463333	57.978889;
                 -6.461111	57.975556;
                 -6.459722	57.971667;
                 -6.460556	57.967222;
                 -6.461667	57.963056;
                 -6.464722	57.960000;
                 -6.467500	57.956944;
                 -6.472222	57.954722;
                 -6.476944	57.952778;
                 -6.478056	57.949444;
                 -6.476944	57.945833;
                 -6.477222	57.941111;
                 -6.480278	57.938056;
                 -6.485556	57.936389;
                 -6.492778	57.935833;
                 -6.501111	57.936111;
                 -6.506667	57.934444;
                 -6.511944	57.932778;
                 -6.517222	57.931111;
                 -6.522778	57.929444;
                 -6.530556	57.929444;
                 -6.534444	57.931667;
                 -6.537500	57.934444;
                 -6.541667	57.936944;
                 -6.549444	57.937778;
                 -6.553889	57.935556;
                 -6.554444	57.931111;
                 -6.553333	57.927222;
                 -6.551944	57.923611;
                 -6.551389	57.919444;
                 -6.551111	57.915278;
                 -6.554167	57.912222;
                 -6.560833	57.913333;
                 -6.567500	57.914444;
                 -6.571389	57.911944;
                 -6.575833	57.909722;
                 -6.581944	57.909444;
                 -6.585833	57.911944;
                 -6.588056	57.915278;
                 -6.592222	57.917500;
                 -6.598056	57.919167;
                 -6.603056	57.921111;
                 -6.606111	57.924167;
                 -6.608333	57.927222;
                 -6.610556	57.930556;
                 -6.612778	57.933889;
                 -6.615000	57.937222;
                 -6.618056	57.940000;
                 -6.621944	57.942500;
                 -6.626944	57.944444;
                 -6.633333	57.943333;
                 -6.631944	57.939444;
                 -6.630556	57.935833;
                 -6.628333	57.932500;
                 -6.626111	57.929167;
                 -6.623056	57.926389;
                 -6.620000	57.923611;
                 -6.617778	57.920278;
                 -6.616389	57.916667;
                 -6.619444	57.913611;
                 -6.623889	57.914167;
                 -6.627778	57.916667;
                 -6.630833	57.919444;
                 -6.635833	57.921389;
                 -6.642500	57.921667;
                 -6.647222	57.919444;
                 -6.653333	57.918333;
                 -6.661111	57.918889;
                 -6.664167	57.921944;
                 -6.668333	57.924167;
                 -6.671389	57.927222;
                 -6.674444	57.930000;
                 -6.678333	57.932500;
                 -6.681667	57.935278;
                 -6.684444	57.938056;
                 -6.686667	57.941389;
                 -6.690000	57.944167;
                 -6.694167	57.946667;
                 -6.698056	57.949167;
                 -6.703056	57.951111;
                 -6.706944	57.953611;
                 -6.710278	57.956389;
                 -6.712500	57.959722;
                 -6.715556	57.962500;
                 -6.718611	57.965278;
                 -6.720833	57.968611;
                 -6.723056	57.971944;
                 -6.724444	57.975556;
                 -6.725556	57.979167;
                 -6.724444	57.983333;
                 -6.721389	57.986389;
                 -6.718611	57.989444;
                 -6.714722	57.992222;
                 -6.713333	57.996111;
                 -6.716667	57.999167;
                 -6.721389	58.001111;
                 -6.723611	58.004444;
                 -6.721389	58.008056;
                 -6.718611	58.011111;
                 -6.714722	58.013611;
                 -6.710833	58.016389;
                 -6.707222	58.018889;
                 -6.703611	58.021667;
                 -6.700556	58.024722;
                 -6.696667	58.027222;
                 -6.696667	58.030000;
                 -6.701389	58.031944;
                 -6.710000	58.032222;
                 -6.716389	58.031111;
                 -6.722500	58.030000;
                 -6.727778	58.028333;
                 -6.731389	58.025556;
                 -6.736111	58.023611;
                 -6.739722	58.020833;
                 -6.743611	58.018333;
                 -6.746667	58.015278;
                 -6.748889	58.011667;
                 -6.750000	58.007500;
                 -6.750556	58.003056;
                 -6.749167	57.999167;
                 -6.748056	57.995556;
                 -6.746667	57.991944;
                 -6.746389	57.987778;
                 -6.745833	57.983611;
                 -6.744722	57.979722;
                 -6.743056	57.976111;
                 -6.740833	57.972778;
                 -6.739722	57.969167;
                 -6.738611	57.965556;
                 -6.738889	57.960833;
                 -6.739444	57.956389;
                 -6.740000	57.951667;
                 -6.741389	57.947778;
                 -6.736389	57.945833;
                 -6.731389	57.944722;
                 -6.727222	57.942222;
                 -6.724167	57.939444;
                 -6.720833	57.936667;
                 -6.718056	57.933611;
                 -6.714722	57.930833;
                 -6.710556	57.928333;
                 -6.708611	57.925000;
                 -6.707222	57.921389;
                 -6.705000	57.918056;
                 -6.703611	57.915278;
                 -6.708889	57.913611;
                 -6.716111	57.913056;
                 -6.721389	57.911389;
                 -6.720000	57.907500;
                 -6.716111	57.905278;
                 -6.711111	57.903333;
                 -6.703333	57.902500;
                 -6.696667	57.901389;
                 -6.692500	57.898889;
                 -6.689444	57.896111;
                 -6.687222	57.892778;
                 -6.685000	57.889722;
                 -6.682778	57.886389;
                 -6.682500	57.882222;
                 -6.684444	57.878611;
                 -6.690833	57.877500;
                 -6.697778	57.877500;
                 -6.705278	57.878333;
                 -6.713889	57.878611;
                 -6.722500	57.878889;
                 -6.729444	57.878056;
                 -6.737222	57.878056;
                 -6.742222	57.880000;
                 -6.748056	57.881667;
                 -6.755556	57.882222;
                 -6.763333	57.883056;
                 -6.770000	57.884167;
                 -6.775278	57.886111;
                 -6.779167	57.888611;
                 -6.783333	57.890000;
                 -6.790000	57.891111;
                 -6.797778	57.890833;
                 -6.803333	57.889167;
                 -6.803333	57.885556;
                 -6.806389	57.882500;
                 -6.812500	57.883056;
                 -6.817778	57.881389;
                 -6.817500	57.877222;
                 -6.820278	57.874167;
                 -6.823889	57.871667;
                 -6.823611	57.867500;
                 -6.818611	57.865556;
                 -6.810000	57.865278;
                 -6.803333	57.864167;
                 -6.798333	57.862222;
                 -6.795278	57.859167;
                 -6.791389	57.856944;
                 -6.787222	57.854444;
                 -6.785278	57.851111;
                 -6.784722	57.846944;
                 -6.781389	57.844167;
                 -6.777500	57.841667;
                 -6.773333	57.839444;
                 -6.768611	57.837222;
                 -6.763611	57.835278;
                 -6.759444	57.833056;
                 -6.758333	57.829167;
                 -6.760278	57.826667;
                 -6.765000	57.824444;
                 -6.771111	57.823333;
                 -6.776944	57.823889;
                 -6.781944	57.825833;
                 -6.788611	57.826944;
                 -6.793333	57.824722;
                 -6.798611	57.823333;
                 -6.805556	57.822500;
                 -6.809167	57.820000;
                 -6.811389	57.816389;
                 -6.807500	57.813889;
                 -6.804167	57.811111;
                 -6.801944	57.807778;
                 -6.801389	57.804444;
                 -6.805278	57.801944;
                 -6.811111	57.802500;
                 -6.815278	57.805000;
                 -6.819167	57.807222;
                 -6.826944	57.808056;
                 -6.834444	57.808889;
                 -6.839444	57.810833;
                 -6.843611	57.813056;
                 -6.849444	57.814722;
                 -6.855833	57.815833;
                 -6.863889	57.815556;
                 -6.862500	57.811944;
                 -6.861111	57.808333;
                 -6.861944	57.803611;
                 -6.864444	57.800556;
                 -6.869167	57.798333;
                 -6.874444	57.796667;
                 -6.881389	57.796111;
                 -6.882500	57.792778;
                 -6.884722	57.789167;
                 -6.889167	57.787222;
                 -6.893889	57.785000;
                 -6.897500	57.782500;
                 -6.898056	57.778611;
                 -6.895833	57.775556;
                 -6.893611	57.772222;
                 -6.897222	57.769444;
                 -6.902500	57.767778;
                 -6.910278	57.767778;
                 -6.917222	57.766944;
                 -6.923333	57.765833;
                 -6.922778	57.762500;
                 -6.920556	57.759167;
                 -6.921944	57.756111;
                 -6.925556	57.753611;
                 -6.930833	57.751944;
                 -6.937778	57.751111;
                 -6.943889	57.750000;
                 -6.949167	57.748333;
                 -6.953056	57.745556;
                 -6.955833	57.742500;
                 -6.957222	57.738333;
                 -6.956667	57.734167;
                 -6.959722	57.731111;
                 -6.964167	57.729167;
                 -6.966389	57.725556;
                 -6.967500	57.721389;
                 -6.971389	57.718611;
                 -6.978333	57.718889;
                 -6.983056	57.720833;
                 -6.986944	57.723889;
                 -6.990278	57.726667;
                 -6.992500	57.730000;
                 -6.995556	57.732778;
                 -6.996944	57.736389;
                 -6.998889	57.739722;
                 -7.001111	57.743056;
                 -7.004444	57.745833;
                 -7.008611	57.748333;
                 -7.013333	57.750278;
                 -7.021111	57.750833;
                 -7.028889	57.750833;
                 -7.034722	57.752222;
                 -7.036944	57.755556;
                 -7.040000	57.758333;
                 -7.043889	57.760833;
                 -7.042500	57.758889;
                 -7.047500	57.761667;
                 -7.050556	57.763611;
                 -7.055556	57.765556;
                 -7.061389	57.767222;
                 -7.066389	57.769167;
                 -7.069444	57.771944;
                 -7.071667	57.775278;
                 -7.074722	57.778056;
                 -7.078889	57.780556;
                 -7.083611	57.782500;
                 -7.089722	57.784167;
                 -7.091667	57.787500;
                 -7.095000	57.790278;
                 -7.096389	57.793889;
                 -7.098611	57.797222;
                 -7.100833	57.800278;
                 -7.106667	57.801944;
                 -7.115278	57.802222;
                 -7.121111	57.803889;
                 -7.124167	57.806667;
                 -7.129167	57.808611;
                 -7.135000	57.810278;
                 -7.140000	57.812222;
                 -7.145000	57.814167;
                 -7.146111	57.817778;
                 -7.146667	57.821944;
                 -7.144444	57.825556;
                 -7.147778	57.828333;
                 -7.155556	57.829167;
                 -7.159722	57.830556;
                 -7.155000	57.832778;
                 -7.150556	57.834722;
                 -7.143611	57.835556;
                 -7.137778	57.834167;
                 -7.132778	57.831944;
                 -7.126111	57.830833;
                 -7.118056	57.831111;
                 -7.110556	57.831389;
                 -7.105556	57.829444;
                 -7.103333	57.826944;
                 -7.100000	57.824167;
                 -7.097778	57.820833;
                 -7.096389	57.817222;
                 -7.094167	57.813889;
                 -7.091944	57.810833;
                 -7.090833	57.806944;
                 -7.089444	57.803333;
                 -7.087222	57.800000;
                 -7.081111	57.800278;
                 -7.076667	57.802500;
                 -7.074444	57.806111;
                 -7.073889	57.810556;
                 -7.073611	57.815278;
                 -7.073889	57.819444;
                 -7.071667	57.823056;
                 -7.067500	57.825000;
                 -7.059444	57.825278;
                 -7.054167	57.826944;
                 -7.049444	57.829167;
                 -7.047500	57.832778;
                 -7.041389	57.833889;
                 -7.033056	57.833611;
                 -7.026111	57.833333;
                 -7.023056	57.836667;
                 -7.023611	57.840833;
                 -7.020833	57.842778;
                 -7.016111	57.845000;
                 -7.010000	57.846389;
                 -7.005556	57.848333;
                 -7.004167	57.852500;
                 -7.003611	57.856944;
                 -7.000833	57.860000;
                 -6.996389	57.862222;
                 -6.989167	57.863056;
                 -6.980833	57.862778;
                 -6.975278	57.864444;
                 -6.970833	57.866667;
                 -6.963889	57.867222;
                 -6.957222	57.866111;
                 -6.951389	57.864444;
                 -6.944444	57.863611;
                 -6.936111	57.863333;
                 -6.928889	57.863889;
                 -6.926111	57.866944;
                 -6.928333	57.870278;
                 -6.932500	57.872778;
                 -6.939167	57.873889;
                 -6.947778	57.874167;
                 -6.956389	57.874444;
                 -6.963056	57.875556;
                 -6.968889	57.876944;
                 -6.971944	57.880000;
                 -6.975000	57.882778;
                 -6.973889	57.886944;
                 -6.969167	57.888889;
                 -6.965556	57.891667;
                 -6.966111	57.895833;
                 -6.962500	57.898333;
                 -6.956111	57.899444;
                 -6.950000	57.900833;
                 -6.945278	57.902778;
                 -6.942500	57.906111;
                 -6.934722	57.906111;
                 -6.928056	57.905000;
                 -6.921111	57.903889;
                 -6.916389	57.901944;
                 -6.914167	57.898611;
                 -6.910833	57.895833;
                 -6.903056	57.896111;
                 -6.896111	57.895833;
                 -6.889444	57.894722;
                 -6.881944	57.893889;
                 -6.875833	57.895278;
                 -6.868056	57.895556;
                 -6.861944	57.893889;
                 -6.855833	57.894167;
                 -6.848889	57.894722;
                 -6.841389	57.894167;
                 -6.835278	57.895278;
                 -6.834722	57.898889;
                 -6.838889	57.901389;
                 -6.843889	57.903333;
                 -6.849722	57.905000;
                 -6.856389	57.906111;
                 -6.861389	57.908056;
                 -6.865556	57.910278;
                 -6.865833	57.914444;
                 -6.862778	57.917778;
                 -6.857500	57.919444;
                 -6.853889	57.921944;
                 -6.855278	57.924722;
                 -6.861944	57.925833;
                 -6.868056	57.924722;
                 -6.874167	57.923611;
                 -6.882778	57.923889;
                 -6.889722	57.925278;
                 -6.895556	57.926944;
                 -6.898611	57.928056;
                 -6.902778	57.930833;
                 -6.906667	57.933333;
                 -6.911944	57.935278;
                 -6.914167	57.938611;
                 -6.918056	57.940833;
                 -6.925833	57.941667;
                 -6.933611	57.941389;
                 -6.942222	57.941667;
                 -6.948889	57.942778;
                 -6.955556	57.943889;
                 -6.962500	57.945000;
                 -6.969167	57.946111;
                 -6.975000	57.947778;
                 -6.980000	57.949722;
                 -6.985000	57.951667;
                 -6.990000	57.953611;
                 -6.997500	57.954444;
                 -7.002222	57.952222;
                 -7.005000	57.949167;
                 -7.006389	57.945000;
                 -7.013333	57.944167;
                 -7.020000	57.945278;
                 -7.026111	57.946944;
                 -7.031944	57.948611;
                 -7.039444	57.949167;
                 -7.046389	57.950278;
                 -7.052222	57.951944;
                 -7.056389	57.954167;
                 -7.060278	57.956667;
                 -7.064444	57.959167;
                 -7.071944	57.959722;
                 -7.079722	57.960556;
                 -7.084722	57.962500;
                 -7.087778	57.965278;
                 -7.090000	57.968611;
                 -7.092500	57.971667;
                 -7.096389	57.974167;
                 -7.098611	57.977500;
                 -7.100000	57.981111;
                 -7.102222	57.984444;
                 -7.107222	57.986389;
                 -7.113333	57.986111;
                 -7.119722	57.985000;
                 -7.127222	57.985556;
                 -7.130278	57.988333;
                 -7.130833	57.992500;
                 -7.126111	57.994722;
                 -7.120000	57.995833;
                 -7.114722	57.997500;
                 -7.110000	57.999722;
                 -7.107222	58.002778;
                 -7.108611	58.006667;
                 -7.109167	58.010833;
                 -7.108611	58.014444;
                 -7.102500	58.015556;
                 -7.094722	58.015000;
                 -7.088889	58.013333;
                 -7.083889	58.011389;
                 -7.078056	58.009722;
                 -7.071944	58.009167;
                 -7.067500	58.011389;
                 -7.063056	58.013611;
                 -7.060000	58.016667;
                 -7.057778	58.020278;
                 -7.055000	58.023333;
                 -7.049444	58.025000;
                 -7.043333	58.026111;
                 -7.038056	58.027778;
                 -7.031944	58.029167;
                 -7.025556	58.030278;
                 -7.020556	58.031944;
                 -7.015000	58.033611;
                 -7.013889	58.036667;
                 -7.020556	58.037778;
                 -7.028333	58.037500;
                 -7.035278	58.036944;
                 -7.042500	58.036111;
                 -7.049444	58.035556;
                 -7.057222	58.035278;
                 -7.063889	58.036389;
                 -7.068889	58.038333;
                 -7.070278	58.041944;
                 -7.065556	58.044167;
                 -7.061389	58.046389;
                 -7.055833	58.048056;
                 -7.048056	58.048333;
                 -7.044444	58.049167;
                 -7.052222	58.049722;
                 -7.059167	58.050833;
                 -7.065833	58.051944;
                 -7.071667	58.053611;
                 -7.068889	58.056667;
                 -7.068333	58.060278;
                 -7.075278	58.061389;
                 -7.083056	58.061111;
                 -7.090833	58.060833;
                 -7.095000	58.063333;
                 -7.096111	58.066944;
                 -7.095833	58.071667;
                 -7.097222	58.075278;
                 -7.099444	58.078611;
                 -7.099722	58.082778;
                 -7.100000	58.086944;
                 -7.100556	58.091111;
                 -7.102778	58.094167;
                 -7.105833	58.096944;
                 -7.109167	58.100000;
                 -7.113333	58.102222;
                 -7.118056	58.104444;
                 -7.124167	58.105833;
                 -7.129167	58.107778;
                 -7.134167	58.109722;
                 -7.134444	58.113889;
                 -7.134167	58.118611;
                 -7.133611	58.123056;
                 -7.131667	58.126667;
                 -7.123889	58.126944;
                 -7.119167	58.129167;
                 -7.118056	58.133056;
                 -7.116389	58.137222;
                 -7.112778	58.139722;
                 -7.109167	58.142500;
                 -7.104444	58.144722;
                 -7.100833	58.147222;
                 -7.096944	58.150000;
                 -7.095000	58.153611;
                 -7.096111	58.157222;
                 -7.098611	58.160556;
                 -7.102500	58.162778;
                 -7.106667	58.165278;
                 -7.105556	58.169444;
                 -7.101667	58.171944;
                 -7.097222	58.174167;
                 -7.090000	58.175000;
                 -7.082222	58.175000;
                 -7.074444	58.175278;
                 -7.065556	58.175000;
                 -7.058889	58.173889;
                 -7.051944	58.172778;
                 -7.048056	58.170278;
                 -7.043056	58.168333;
                 -7.037500	58.170000;
                 -7.033056	58.172222;
                 -7.026944	58.173333;
                 -7.020833	58.174722;
                 -7.015278	58.176389;
                 -7.015000	58.179167;
                 -7.017222	58.182222;
                 -7.020556	58.185278;
                 -7.024444	58.187500;
                 -7.031389	58.186944;
                 -7.038611	58.186111;
                 -7.045278	58.185556;
                 -7.050556	58.187500;
                 -7.052778	58.190556;
                 -7.054167	58.194444;
                 -7.057500	58.197222;
                 -7.060556	58.200000;
                 -7.059167	58.204167;
                 -7.056389	58.206389;
                 -7.053333	58.209444;
                 -7.050556	58.212500;
                 -7.047500	58.215833;
                 -7.043611	58.218333;
                 -7.040000	58.220833;
                 -7.037222	58.224167;
                 -7.033333	58.226667;
                 -7.030278	58.229722;
                 -7.029167	58.233889;
                 -7.023056	58.233333;
                 -7.018889	58.231667;
                 -7.016667	58.229444;
                 -7.013333	58.226667;
                 -7.007222	58.225833;
                 -6.999444	58.226111;
                 -6.994444	58.224167;
                 -6.990556	58.221667;
                 -6.986389	58.219444;
                 -6.982222	58.216944;
                 -6.978333	58.214444;
                 -6.972222	58.213056;
                 -6.964722	58.212222;
                 -6.958333	58.213333;
                 -6.956389	58.216944;
                 -6.958611	58.220278;
                 -6.959722	58.224167;
                 -6.956111	58.226667;
                 -6.950000	58.226111;
                 -6.946667	58.223056;
                 -6.943611	58.220278;
                 -6.940556	58.217500;
                 -6.936389	58.215000;
                 -6.932500	58.212778;
                 -6.926389	58.211111;
                 -6.920278	58.209444;
                 -6.914444	58.208056;
                 -6.911389	58.205278;
                 -6.909167	58.201944;
                 -6.908611	58.197778;
                 -6.910278	58.193611;
                 -6.917222	58.193056;
                 -6.924167	58.193333;
                 -6.932500	58.193333;
                 -6.941389	58.193611;
                 -6.947500	58.192500;
                 -6.946389	58.188889;
                 -6.942222	58.186389;
                 -6.937222	58.184444;
                 -6.933056	58.181944;
                 -6.928889	58.179722;
                 -6.925000	58.177222;
                 -6.918889	58.175833;
                 -6.912222	58.175556;
                 -6.903611	58.175278;
                 -6.897778	58.173611;
                 -6.891667	58.172222;
                 -6.886667	58.170000;
                 -6.881667	58.168056;
                 -6.876667	58.166111;
                 -6.870000	58.165000;
                 -6.865556	58.166111;
                 -6.867778	58.169444;
                 -6.871944	58.171944;
                 -6.871389	58.174722;
                 -6.863611	58.175000;
                 -6.856667	58.175556;
                 -6.855278	58.179722;
                 -6.852222	58.182778;
                 -6.843889	58.182500;
                 -6.839722	58.185278;
                 -6.834444	58.186667;
                 -6.827500	58.186667;
                 -6.822222	58.188333;
                 -6.814444	58.188333;
                 -6.807500	58.187222;
                 -6.801667	58.185833;
                 -6.795556	58.184167;
                 -6.787778	58.184444;
                 -6.783056	58.182500;
                 -6.778889	58.180000;
                 -6.771944	58.178889;
                 -6.765000	58.179444;
                 -6.756944	58.179722;
                 -6.749444	58.179167;
                 -6.745278	58.176667;
                 -6.743056	58.173333;
                 -6.741944	58.169722;
                 -6.739722	58.166389;
                 -6.736667	58.163611;
                 -6.731389	58.161667;
                 -6.722778	58.161389;
                 -6.717500	58.163056;
                 -6.714722	58.166111;
                 -6.713056	58.170000;
                 -6.709444	58.172778;
                 -6.705556	58.175278;
                 -6.702500	58.178611;
                 -6.709444	58.179722;
                 -6.718056	58.180000;
                 -6.725000	58.180000;
                 -6.730833	58.181667;
                 -6.736944	58.183333;
                 -6.741944	58.185278;
                 -6.745833	58.187778;
                 -6.748889	58.190556;
                 -6.749444	58.194722;
                 -6.746667	58.197778;
                 -6.742778	58.200278;
                 -6.745833	58.203056;
                 -6.750833	58.205278;
                 -6.756667	58.206667;
                 -6.759722	58.209444;
                 -6.763056	58.212500;
                 -6.765000	58.215833;
                 -6.767222	58.218889;
                 -6.769722	58.222222;
                 -6.773056	58.225000;
                 -6.778889	58.226667;
                 -6.785556	58.227778;
                 -6.789444	58.230278;
                 -6.788333	58.234167;
                 -6.785278	58.237500;
                 -6.791389	58.238889;
                 -6.797222	58.240556;
                 -6.800278	58.243333;
                 -6.802500	58.246667;
                 -6.804722	58.250000;
                 -6.805278	58.254167;
                 -6.808333	58.256944;
                 -6.808889	58.261111;
                 -6.805556	58.264167;
                 -6.800278	58.265833;
                 -6.795556	58.268056;
                 -6.800833	58.270000;
                 -6.809444	58.270278;
                 -6.815556	58.271667;
                 -6.818611	58.274722;
                 -6.815556	58.277778;
                 -6.810000	58.279444;
                 -6.804722	58.281111;
                 -6.798611	58.282222;
                 -6.794722	58.284722;
                 -6.795833	58.288889;
                 -6.797500	58.292778;
                 -6.795278	58.296389;
                 -6.788889	58.297500;
                 -6.782778	58.298611;
                 -6.778889	58.301389;
                 -6.773333	58.302778;
                 -6.767500	58.302222;
                 -6.762500	58.300278;
                 -6.755556	58.300000;
                 -6.749167	58.301111;
                 -6.744722	58.303333;
                 -6.740278	58.305556;
                 -6.739722	58.308333;
                 -6.740278	58.312500;
                 -6.735000	58.314167;
                 -6.729444	58.315556;
                 -6.723889	58.317222;
                 -6.720278	58.320000;
                 -6.716389	58.322500;
                 -6.712778	58.325278;
                 -6.707222	58.326944;
                 -6.701111	58.326111;
                 -6.699722	58.322500;
                 -6.694444	58.323333;
                 -6.690000	58.325556;
                 -6.685278	58.327500;
                 -6.680000	58.329167;
                 -6.674444	58.330833;
                 -6.671389	58.333889;
                 -6.672778	58.337778;
                 -6.671389	58.341667;
                 -6.665278	58.342778;
                 -6.660556	58.345000;
                 -6.654444	58.345278;
                 -6.649444	58.343333;
                 -6.648889	58.339167;
                 -6.646111	58.336389;
                 -6.638889	58.336944;
                 -6.634167	58.339167;
                 -6.627222	58.339722;
                 -6.619444	58.339167;
                 -6.612222	58.339722;
                 -6.606111	58.341111;
                 -6.600556	58.342500;
                 -6.595278	58.344167;
                 -6.591667	58.346944;
                 -6.586944	58.349167;
                 -6.579722	58.348889;
                 -6.571944	58.349167;
                 -6.566667	58.350556;
                 -6.561944	58.352778;
                 -6.556667	58.354444;
                 -6.552222	58.353889;
                 -6.550000	58.350556;
                 -6.545833	58.348056;
                 -6.540000	58.346667;
                 -6.533056	58.347222;
                 -6.530000	58.350278;
                 -6.531389	58.354167;
                 -6.534444	58.356944;
                 -6.533889	58.361389;
                 -6.532500	58.365556;
                 -6.530278	58.369167;
                 -6.528056	58.372778;
                 -6.525833	58.376389;
                 -6.523889	58.380000;
                 -6.520833	58.383056;
                 -6.517778	58.386111;
                 -6.513056	58.388333;
                 -6.506944	58.389444;
                 -6.498889	58.389444;
                 -6.493056	58.390833;
                 -6.488333	58.392778;
                 -6.485000	58.395833;
                 -6.482222	58.399167;
                 -6.478611	58.401667;
                 -6.472778	58.403333;
                 -6.468611	58.404722;
                 -6.462778	58.406111;
                 -6.458333	58.408333;
                 -6.456111	58.411944;
                 -6.451389	58.414167;
                 -6.446111	58.415833;
                 -6.440556	58.417500;
                 -6.436667	58.420000;
                 -6.434444	58.423611;
                 -6.430000	58.425556;
                 -6.424444	58.427222;
                 -6.417500	58.428056;
                 -6.411111	58.429167;
                 -6.405833	58.430833;
                 -6.401944	58.433333;
                 -6.398889	58.436389;
                 -6.393611	58.438056;
                 -6.386389	58.438889;
                 -6.379444	58.439444;
                 -6.374722	58.441667;
                 -6.372500	58.445278;
                 -6.369444	58.448333;
                 -6.361667	58.448611;
                 -6.353889	58.448611;
                 -6.348056	58.450278;
                 -6.343611	58.452500;
                 -6.339722	58.455000;
                 -6.336667	58.458056;
                 -6.333611	58.461389;
                 -6.331667	58.464722;
                 -6.329444	58.468611;
                 -6.324722	58.470556;
                 -6.315833	58.470278;
                 -6.308889	58.470000;
                 -6.303333	58.471667;
                 -6.298056	58.473333;
                 -6.294167	58.475833;
                 -6.289444	58.478056;
                 -6.285556	58.480556;
                 -6.281944	58.483333;
                 -6.277222	58.485278;
                 -6.274167	58.488333;
                 -6.273611	58.493056;
                 -6.275000	58.496667;
                 -6.278056	58.499722;
                 -6.279167	58.503333;
                 -6.275278	58.505833;
                 -6.269167	58.506944;
                 -6.261111	58.507222;
                 -6.256389	58.505278;
                 -6.251111	58.503333;
                 -6.246389	58.501111;
                 -6.242222	58.498889;
                 -6.237222	58.496667;
                 -6.232222	58.494722;
                 -6.224444	58.494167;
                 -6.218611	58.492500;
                 -6.216389	58.489167;
                 -6.218611	58.485556;
                 -6.220000	58.481389;
                 -6.216667	58.478611;
                 -6.211944	58.476667;
                 -6.206667	58.474722;
                 -6.201944	58.472500;
                 -6.196667	58.470556;
                 -6.191944	58.468611;
                 -6.186667	58.466667;
                 -6.181944	58.464722;
                 -6.176944	58.462500;
                 -6.171944	58.460556;
                 -6.174167	58.456944;
                 -6.178056	58.454444;
                 -6.182500	58.452222;
                 -6.185833	58.449167;
                 -6.182500	58.446389;
                 -6.180556	58.443056;
                 -6.181944	58.438889;
                 -6.183056	58.435000;
                 -6.180278	58.431944;
                 -6.175000	58.430000;
                 -6.169167	58.428333;
                 -6.169722	58.424722;
                 -6.167500	58.421389;
                 -6.162222	58.419444;
                 -6.158333	58.416944;
                 -6.158889	58.412500;
                 -6.160278	58.408611;
                 -6.162500	58.405000];




  uki(50).xy = [ -6.848889	58.197778;
                 -6.853889	58.198889;
                 -6.858889	58.200833;
                 -6.863889	58.202778;
                 -6.868056	58.205278;
                 -6.865833	58.208889;
                 -6.860000	58.209167;
                 -6.854722	58.207222;
                 -6.849722	58.205000;
                 -6.846667	58.202222;
                 -6.846944	58.199444;
                 -6.848889	58.197778];


  uki(51).xy = [ -7.124167	58.006111;
                 -7.128611	58.003889;
                 -7.135833	58.003333;
                 -7.143611	58.003056;
                 -7.149444	58.004444;
                 -7.156389	58.005833;
                 -7.163889	58.006389;
                 -7.171667	58.006111;
                 -7.179444	58.006667;
                 -7.183333	58.009167;
                 -7.185833	58.012500;
                 -7.188056	58.015833;
                 -7.188611	58.020000;
                 -7.186944	58.023889;
                 -7.182500	58.026111;
                 -7.176389	58.027222;
                 -7.171111	58.028889;
                 -7.166389	58.031111;
                 -7.161944	58.033333;
                 -7.155833	58.034444;
                 -7.148611	58.035278;
                 -7.142222	58.034167;
                 -7.135278	58.033056;
                 -7.130000	58.031111;
                 -7.126944	58.028056;
                 -7.127500	58.024444;
                 -7.129722	58.020833;
                 -7.130000	58.018056;
                 -7.124722	58.016111;
                 -7.120833	58.013889;
                 -7.117778	58.010833;
                 -7.119722	58.007222;
                 -7.124167	58.006111];

  
  uki(52).xy = [ -6.986111	57.893056;
                 -6.988056	57.890278;
                 -6.991944	57.887778;
                 -6.995000	57.884722;
                 -6.997500	57.881667;
                 -7.003056	57.880000;
                 -7.010833	57.879722;
                 -7.018611	57.880278;
                 -7.024444	57.881944;
                 -7.030278	57.883611;
                 -7.036944	57.884444;
                 -7.043889	57.883889;
                 -7.049444	57.882222;
                 -7.053611	57.880000;
                 -7.058333	57.877778;
                 -7.062778	57.875833;
                 -7.067500	57.873611;
                 -7.074167	57.872778;
                 -7.080000	57.874444;
                 -7.081667	57.878056;
                 -7.082778	57.881944;
                 -7.080833	57.885556;
                 -7.074722	57.886667;
                 -7.069444	57.888333;
                 -7.063333	57.889444;
                 -7.056944	57.890556;
                 -7.051667	57.892222;
                 -7.047778	57.895000;
                 -7.046667	57.899167;
                 -7.041944	57.901111;
                 -7.035278	57.901944;
                 -7.029722	57.903611;
                 -7.026111	57.906111;
                 -7.022222	57.908889;
                 -7.018611	57.911389;
                 -7.013056	57.913056;
                 -7.007778	57.914722;
                 -7.001667	57.916111;
                 -6.995833	57.914444;
                 -6.992778	57.912500;
                 -6.990556	57.909167;
                 -6.988889	57.905556;
                 -6.988611	57.901389;
                 -6.986389	57.898056;
                 -6.986111	57.893056];


  uki(53).xy = [-7.222778	57.772778;
                -7.220833	57.770833;
                -7.217500	57.768056;
                -7.218889	57.764167;
                -7.220000	57.760000;
                -7.226111	57.758889;
                -7.232222	57.757500;
                -7.237500	57.755833;
                -7.243611	57.754722;
                -7.250556	57.753889;
                -7.257222	57.755000;
                -7.264167	57.756111;
                -7.271667	57.756944;
                -7.279167	57.757500;
                -7.280833	57.761111;
                -7.278611	57.764722;
                -7.275000	57.767500;
                -7.270556	57.769722;
                -7.265000	57.771389;
                -7.259167	57.772500;
                -7.254722	57.774722;
                -7.248333	57.775833;
                -7.243889	57.778056;
                -7.236944	57.778889;
                -7.230278	57.777500;
                -7.226111	57.775278;
                -7.222778	57.772778];


  uki(54).xy = [-7.161944	57.723889;
                -7.164722	57.720833;
                -7.166111	57.716667;
                -7.168889	57.713611;
                -7.172500	57.711111;
                -7.179444	57.710278;
                -7.186389	57.709722;
                -7.190833	57.707500;
                -7.192500	57.703333;
                -7.196111	57.700833;
                -7.202778	57.700833;
                -7.208611	57.702500;
                -7.215556	57.702778;
                -7.218333	57.699444;
                -7.225278	57.698889;
                -7.232778	57.698611;
                -7.236944	57.701111;
                -7.235833	57.705000;
                -7.232778	57.708056;
                -7.229167	57.710833;
                -7.225556	57.713611;
                -7.222778	57.716667;
                -7.221389	57.720556;
                -7.219167	57.724167;
                -7.216389	57.727500;
                -7.211944	57.729444;
                -7.207500	57.731667;
                -7.201389	57.732778;
                -7.193611	57.733056;
                -7.185833	57.733333;
                -7.177222	57.733056;
                -7.170278	57.731944;
                -7.164444	57.730556;
                -7.160556	57.728056;
                -7.159167	57.724444;
                -7.161944	57.723889];


  uki(55).xy = [-7.293333	57.705278;
                -7.293333	57.701667;
                -7.291944	57.698889;
                -7.293333	57.695833;
                -7.298611	57.694167;
                -7.306389	57.693889;
                -7.311389	57.695833;
                -7.314444	57.698611;
                -7.315556	57.702222;
                -7.313611	57.705833;
                -7.309167	57.708056;
                -7.302222	57.708889;
                -7.297500	57.706944;
                -7.294167	57.703889;
                -7.293333	57.705278];
  
  
  uki(56).xy = [ -7.681389	57.692222;
                 -7.680833	57.689167;
                 -7.686111	57.687500;
                 -7.693889	57.688056;
                 -7.696944	57.690833;
                 -7.695833	57.694167;
                 -7.689722	57.695278;
                 -7.683889	57.693889;
                 -7.681389	57.692222];


  uki(57).xy = [ -8.552222	57.813889;
                 -8.555278	57.810833;
                 -8.559444	57.808611;
                 -8.565000	57.806667;
                 -8.569444	57.804444;
                 -8.571944	57.801389;
                 -8.575556	57.798611;
                 -8.582500	57.798889;
                 -8.586667	57.801111;
                 -8.591667	57.803056;
                 -8.599444	57.803889;
                 -8.606111	57.804722;
                 -8.611944	57.806389;
                 -8.613611	57.810000;
                 -8.612222	57.813889;
                 -8.614722	57.817222;
                 -8.618056	57.820000;
                 -8.621389	57.822778;
                 -8.618611	57.826111;
                 -8.612500	57.824444;
                 -8.609444	57.822778;
                 -8.603333	57.821111;
                 -8.597500	57.822500;
                 -8.591944	57.824167;
                 -8.583611	57.823889;
                 -8.578333	57.822222;
                 -8.573611	57.820278;
                 -8.565556	57.819722;
                 -8.559722	57.818056;
                 -8.554722	57.816111;
                 -8.552222	57.813889];

  
  uki(58).xy = [ -7.079722	57.625556;
                 -7.084167	57.623333;
                 -7.087778	57.620556;
                 -7.091667	57.618056;
                 -7.094444	57.615000;
                 -7.097500	57.611667;
                 -7.100000	57.608611;
                 -7.103889	57.606111;
                 -7.110833	57.605278;
                 -7.116667	57.606944;
                 -7.122222	57.608333;
                 -7.125556	57.611389;
                 -7.126944	57.615000;
                 -7.127500	57.619167;
                 -7.132222	57.621111;
                 -7.140000	57.620833;
                 -7.146944	57.620000;
                 -7.152778	57.620833;
                 -7.157778	57.622778;
                 -7.163611	57.624167;
                 -7.168333	57.626111;
                 -7.176944	57.626389;
                 -7.183889	57.626667;
                 -7.188611	57.628611;
                 -7.194444	57.630278;
                 -7.200556	57.628889;
                 -7.205000	57.626944;
                 -7.207222	57.623333;
                 -7.209167	57.619722;
                 -7.208611	57.616389;
                 -7.204722	57.613889;
                 -7.198889	57.612500;
                 -7.193056	57.610833;
                 -7.184444	57.610556;
                 -7.178056	57.609444;
                 -7.172778	57.607500;
                 -7.170556	57.604444;
                 -7.170278	57.600278;
                 -7.171667	57.596111;
                 -7.173611	57.592500;
                 -7.175833	57.588889;
                 -7.175000	57.585556;
                 -7.168333	57.585556;
                 -7.162222	57.586667;
                 -7.155278	57.587500;
                 -7.149444	57.586667;
                 -7.141667	57.586944;
                 -7.137222	57.589167;
                 -7.133611	57.591667;
                 -7.130000	57.594444;
                 -7.123889	57.594722;
                 -7.120000	57.592222;
                 -7.118333	57.588611;
                 -7.118056	57.584444;
                 -7.119444	57.580278;
                 -7.120556	57.576389;
                 -7.122500	57.572778;
                 -7.125556	57.569444;
                 -7.127778	57.566111;
                 -7.129722	57.562500;
                 -7.132500	57.559167;
                 -7.137222	57.557222;
                 -7.142222	57.555278;
                 -7.149167	57.554722;
                 -7.156944	57.554444;
                 -7.164722	57.554167;
                 -7.170556	57.553056;
                 -7.176944	57.551944;
                 -7.183889	57.551111;
                 -7.190556	57.551389;
                 -7.195556	57.553333;
                 -7.200556	57.555278;
                 -7.207222	57.554722;
                 -7.213333	57.553333;
                 -7.219444	57.552222;
                 -7.227222	57.551944;
                 -7.235000	57.552500;
                 -7.242500	57.553333;
                 -7.248611	57.551944;
                 -7.255278	57.551389;
                 -7.262222	57.552500;
                 -7.266111	57.555000;
                 -7.265833	57.558611;
                 -7.264722	57.562500;
                 -7.268611	57.565000;
                 -7.276111	57.565556;
                 -7.279167	57.562500;
                 -7.282778	57.560000;
                 -7.287222	57.557778;
                 -7.293333	57.556667;
                 -7.299444	57.555278;
                 -7.305556	57.554167;
                 -7.310833	57.552500;
                 -7.311944	57.548333;
                 -7.307778	57.546111;
                 -7.303056	57.544167;
                 -7.296389	57.543056;
                 -7.289444	57.541944;
                 -7.285556	57.539444;
                 -7.281389	57.536944;
                 -7.281111	57.533889;
                 -7.286389	57.532222;
                 -7.293889	57.531944;
                 -7.300000	57.530833;
                 -7.300278	57.528056;
                 -7.295278	57.525833;
                 -7.293333	57.522778;
                 -7.294444	57.518611;
                 -7.297500	57.515556;
                 -7.295278	57.512222;
                 -7.292778	57.508889;
                 -7.298056	57.507222;
                 -7.304167	57.506111;
                 -7.311111	57.505278;
                 -7.318889	57.505278;
                 -7.325556	57.505278;
                 -7.330556	57.507222;
                 -7.334444	57.509722;
                 -7.336667	57.513056;
                 -7.338056	57.516667;
                 -7.336944	57.520833;
                 -7.334722	57.524444;
                 -7.336111	57.528056;
                 -7.339444	57.530833;
                 -7.344167	57.532778;
                 -7.349167	57.534722;
                 -7.353333	57.537222;
                 -7.357222	57.539722;
                 -7.360278	57.542500;
                 -7.364167	57.544722;
                 -7.367500	57.547778;
                 -7.372500	57.549722;
                 -7.378056	57.551111;
                 -7.384167	57.552778;
                 -7.388889	57.554722;
                 -7.393889	57.556667;
                 -7.399722	57.558333;
                 -7.404722	57.560278;
                 -7.410556	57.561667;
                 -7.415556	57.563611;
                 -7.420278	57.565556;
                 -7.424444	57.568056;
                 -7.427500	57.570833;
                 -7.430556	57.573611;
                 -7.433889	57.576389;
                 -7.441389	57.577222;
                 -7.446667	57.575556;
                 -7.449722	57.572222;
                 -7.453333	57.569722;
                 -7.456944	57.566944;
                 -7.461389	57.565000;
                 -7.465833	57.562778;
                 -7.471111	57.561111;
                 -7.477222	57.559722;
                 -7.485000	57.560556;
                 -7.488889	57.562778;
                 -7.493056	57.565278;
                 -7.496944	57.567778;
                 -7.500278	57.570556;
                 -7.504167	57.572778;
                 -7.509167	57.574722;
                 -7.515000	57.576389;
                 -7.520833	57.578056;
                 -7.526667	57.579444;
                 -7.533333	57.580556;
                 -7.540000	57.581667;
                 -7.545833	57.583056;
                 -7.550000	57.585556;
                 -7.553889	57.588056;
                 -7.556111	57.591111;
                 -7.559444	57.594167;
                 -7.555833	57.595833;
                 -7.550556	57.597500;
                 -7.545278	57.599167;
                 -7.540833	57.601389;
                 -7.537222	57.603889;
                 -7.535833	57.608056;
                 -7.537222	57.611667;
                 -7.539444	57.615000;
                 -7.533333	57.616111;
                 -7.526667	57.616944;
                 -7.521111	57.618611;
                 -7.515000	57.620000;
                 -7.510833	57.621944;
                 -7.511111	57.626111;
                 -7.511667	57.630278;
                 -7.509444	57.633889;
                 -7.505833	57.636667;
                 -7.503056	57.639722;
                 -7.506389	57.642500;
                 -7.508611	57.644722;
                 -7.510000	57.648611;
                 -7.508056	57.651111;
                 -7.502778	57.653056;
                 -7.495833	57.653611;
                 -7.488889	57.653611;
                 -7.484167	57.651389;
                 -7.480000	57.649167;
                 -7.475000	57.647222;
                 -7.469722	57.648889;
                 -7.466111	57.651389;
                 -7.460556	57.653333;
                 -7.454722	57.654444;
                 -7.448611	57.653889;
                 -7.442778	57.652222;
                 -7.437778	57.653889;
                 -7.431389	57.655000;
                 -7.425556	57.656389;
                 -7.419444	57.657500;
                 -7.412222	57.658333;
                 -7.404722	57.658611;
                 -7.398889	57.656944;
                 -7.399167	57.653333;
                 -7.403889	57.651111;
                 -7.408333	57.648889;
                 -7.415000	57.648333;
                 -7.420278	57.646667;
                 -7.426389	57.645278;
                 -7.431944	57.643611;
                 -7.436389	57.641667;
                 -7.434167	57.638333;
                 -7.430833	57.635556;
                 -7.426667	57.633056;
                 -7.420833	57.631667;
                 -7.414444	57.630556;
                 -7.410278	57.628056;
                 -7.403333	57.626944;
                 -7.398056	57.627778;
                 -7.394444	57.630278;
                 -7.393333	57.634444;
                 -7.392222	57.638611;
                 -7.390833	57.643056;
                 -7.389722	57.647222;
                 -7.388056	57.651111;
                 -7.385556	57.654444;
                 -7.382500	57.657500;
                 -7.375833	57.658056;
                 -7.368611	57.658889;
                 -7.363611	57.660556;
                 -7.361389	57.664167;
                 -7.362778	57.668056;
                 -7.363333	57.671944;
                 -7.360278	57.675278;
                 -7.354167	57.676389;
                 -7.349722	57.678611;
                 -7.345833	57.681111;
                 -7.339722	57.682500;
                 -7.332222	57.682500;
                 -7.332500	57.678889;
                 -7.334722	57.675278;
                 -7.334167	57.671111;
                 -7.331111	57.668333;
                 -7.328889	57.665000;
                 -7.329167	57.661389;
                 -7.335278	57.660278;
                 -7.339722	57.658056;
                 -7.341111	57.654167;
                 -7.341389	57.649444;
                 -7.338333	57.646667;
                 -7.330556	57.645833;
                 -7.323056	57.646111;
                 -7.317222	57.644722;
                 -7.312222	57.642778;
                 -7.308056	57.640278;
                 -7.305833	57.636944;
                 -7.304444	57.633333;
                 -7.303333	57.629722;
                 -7.301667	57.625833;
                 -7.297500	57.628056;
                 -7.293611	57.630833;
                 -7.293333	57.635278;
                 -7.292778	57.640000;
                 -7.292222	57.644444;
                 -7.287222	57.646111;
                 -7.279167	57.645556;
                 -7.275000	57.647500;
                 -7.273611	57.650833;
                 -7.276944	57.653611;
                 -7.281111	57.656111;
                 -7.279722	57.658333;
                 -7.273611	57.659444;
                 -7.266944	57.660278;
                 -7.259722	57.660833;
                 -7.256944	57.664167;
                 -7.255000	57.667778;
                 -7.253056	57.671111;
                 -7.250833	57.674722;
                 -7.246389	57.676944;
                 -7.240278	57.678056;
                 -7.233333	57.677222;
                 -7.228889	57.679167;
                 -7.226944	57.682778;
                 -7.221667	57.684444;
                 -7.213889	57.684722;
                 -7.208889	57.682778;
                 -7.204722	57.680278;
                 -7.201667	57.677500;
                 -7.196667	57.675556;
                 -7.191944	57.673611;
                 -7.186944	57.671667;
                 -7.184722	57.668333;
                 -7.182500	57.665000;
                 -7.181111	57.661389;
                 -7.183056	57.657778;
                 -7.188611	57.656111;
                 -7.194444	57.655000;
                 -7.200000	57.653333;
                 -7.203611	57.650556;
                 -7.204722	57.646389;
                 -7.200833	57.644167;
                 -7.194167	57.643056;
                 -7.188611	57.644722;
                 -7.182500	57.645833;
                 -7.176111	57.644722;
                 -7.170278	57.643333;
                 -7.165278	57.641389;
                 -7.160278	57.639167;
                 -7.154444	57.637778;
                 -7.146667	57.636944;
                 -7.140556	57.638333;
                 -7.135556	57.640000;
                 -7.130833	57.641944;
                 -7.125833	57.643889;
                 -7.119722	57.645000;
                 -7.113333	57.646111;
                 -7.106389	57.645833;
                 -7.102500	57.643611;
                 -7.098611	57.641111;
                 -7.091667	57.640000;
                 -7.084167	57.639167;
                 -7.078333	57.637778;
                 -7.074167	57.635278;
                 -7.073889	57.631111;
                 -7.076667	57.628056;
                 -7.079722	57.625556];


  uki(59).xy = [ -7.435556	57.558333;
                 -7.432500	57.555278;
                 -7.428333	57.553056;
                 -7.425278	57.550278;
                 -7.423056	57.546944;
                 -7.427500	57.544722;
                 -7.432778	57.543056;
                 -7.437778	57.545000;
                 -7.440833	57.547778;
                 -7.444722	57.550278;
                 -7.446944	57.553611;
                 -7.450278	57.556389;
                 -7.451667	57.560000;
                 -7.444722	57.560833;
                 -7.438889	57.559167;
                 -7.435556	57.558333];


  uki(60).xy = [ -7.650278	57.513611;
                 -7.658056	57.514167;
                 -7.665556	57.513889;
                 -7.671667	57.512778;
                 -7.677778	57.511667;
                 -7.684444	57.511667;
                 -7.682500	57.515278;
                 -7.677222	57.516944;
                 -7.671111	57.518333;
                 -7.665000	57.519444;
                 -7.660556	57.521667;
                 -7.656944	57.524167;
                 -7.651111	57.524722;
                 -7.648611	57.522222;
                 -7.648889	57.518611;
                 -7.644722	57.516111;
                 -7.638056	57.515000;
                 -7.635000	57.512222;
                 -7.640000	57.512500;
                 -7.650278	57.513611];


  uki(61).xy = [ -7.605833	57.521944;
                 -7.604444	57.519167;
                 -7.605000	57.515556;
                 -7.609444	57.513333;
                 -7.614444	57.511667;
                 -7.616667	57.508056;
                 -7.621944	57.506389;
                 -7.626944	57.508333;
                 -7.627500	57.512500;
                 -7.626111	57.516667;
                 -7.623889	57.520278;
                 -7.620278	57.522778;
                 -7.615833	57.525000;
                 -7.609167	57.524722;
                 -7.605833	57.521944];


  uki(62).xy = [ -7.358889	57.522222;
                 -7.360000	57.519167;
                 -7.363889	57.516389;
                 -7.365833	57.512778;
                 -7.362500	57.510000;
                 -7.358611	57.507500;
                 -7.357778	57.504444;
                 -7.360833	57.501389;
                 -7.363056	57.497778;
                 -7.365833	57.494444;
                 -7.371111	57.492778;
                 -7.378056	57.493056;
                 -7.385556	57.493611;
                 -7.389444	57.496111;
                 -7.391944	57.499444;
                 -7.393056	57.503056;
                 -7.393611	57.507222;
                 -7.395833	57.510556;
                 -7.398056	57.513611;
                 -7.400278	57.516944;
                 -7.402500	57.520278;
                 -7.405556	57.523611;
                 -7.408611	57.526389;
                 -7.411944	57.529167;
                 -7.414167	57.532500;
                 -7.410278	57.535278;
                 -7.403889	57.534167;
                 -7.398611	57.533889;
                 -7.395833	57.536944;
                 -7.392222	57.539722;
                 -7.386111	57.540833;
                 -7.381111	57.538889;
                 -7.378056	57.536111;
                 -7.373056	57.534167;
                 -7.369167	57.531667;
                 -7.365000	57.529167;
                 -7.361111	57.526944;
                 -7.358889	57.522222];


  uki(63).xy = [ -7.230556	57.433056;
                 -7.228333	57.429722;
                 -7.226944	57.426111;
                 -7.226667	57.421944;
                 -7.226944	57.417500;
                 -7.228889	57.413889;
                 -7.233611	57.411667;
                 -7.240556	57.410833;
                 -7.248889	57.411111;
                 -7.255000	57.410833;
                 -7.258611	57.408333;
                 -7.263056	57.406111;
                 -7.269167	57.405000;
                 -7.271111	57.401389;
                 -7.278056	57.400556;
                 -7.285000	57.399722;
                 -7.291111	57.398611;
                 -7.296667	57.400000;
                 -7.301389	57.402222;
                 -7.307500	57.403611;
                 -7.315278	57.403333;
                 -7.320278	57.401667;
                 -7.323333	57.398611;
                 -7.330000	57.397778;
                 -7.336667	57.398889;
                 -7.341667	57.400833;
                 -7.345833	57.403333;
                 -7.348889	57.406111;
                 -7.352778	57.408611;
                 -7.356944	57.411111;
                 -7.362500	57.412500;
                 -7.368333	57.414167;
                 -7.374167	57.415556;
                 -7.379167	57.417500;
                 -7.385000	57.419167;
                 -7.392500	57.419722;
                 -7.399167	57.420833;
                 -7.404167	57.422778;
                 -7.407222	57.425556;
                 -7.409444	57.428889;
                 -7.411667	57.432222;
                 -7.412222	57.436389;
                 -7.416389	57.438611;
                 -7.420278	57.441111;
                 -7.423333	57.443889;
                 -7.421389	57.446667;
                 -7.418611	57.449722;
                 -7.422500	57.452222;
                 -7.426667	57.454722;
                 -7.429722	57.457500;
                 -7.430278	57.461667;
                 -7.427500	57.464722;
                 -7.422778	57.466944;
                 -7.416667	57.468056;
                 -7.410833	57.469167;
                 -7.404722	57.470556;
                 -7.400278	57.472500;
                 -7.398056	57.476111;
                 -7.396944	57.480278;
                 -7.394167	57.483333;
                 -7.390556	57.485000;
                 -7.383611	57.485833;
                 -7.376111	57.486111;
                 -7.368333	57.486389;
                 -7.360000	57.486111;
                 -7.353056	57.485000;
                 -7.348056	57.483056;
                 -7.344167	57.480556;
                 -7.341111	57.477778;
                 -7.337778	57.475000;
                 -7.331944	57.473333;
                 -7.325556	57.472222;
                 -7.319444	57.473611;
                 -7.312500	57.474167;
                 -7.304722	57.473611;
                 -7.299167	57.471944;
                 -7.295833	57.469167;
                 -7.295278	57.465000;
                 -7.296667	57.461111;
                 -7.291667	57.459167;
                 -7.285000	57.458889;
                 -7.276667	57.458611;
                 -7.268056	57.458333;
                 -7.260278	57.458611;
                 -7.254722	57.456944;
                 -7.250556	57.454722;
                 -7.251111	57.451111;
                 -7.252222	57.446944;
                 -7.250000	57.443611;
                 -7.243056	57.442500;
                 -7.237500	57.441111;
                 -7.231667	57.439444;
                 -7.230556	57.433056];


  uki(64).xy = [ -7.262222	57.335833;
                 -7.266667	57.334722;
                 -7.272778	57.333611;
                 -7.279444	57.332778;
                 -7.288056	57.333056;
                 -7.295556	57.333611;
                 -7.301389	57.335278;
                 -7.306111	57.337222;
                 -7.310278	57.339722;
                 -7.315278	57.341667;
                 -7.319167	57.343889;
                 -7.323889	57.345833;
                 -7.328889	57.348056;
                 -7.333889	57.350000;
                 -7.339722	57.351389;
                 -7.346389	57.352500;
                 -7.351389	57.354444;
                 -7.350278	57.354167;
                 -7.357778	57.354722;
                 -7.364722	57.355833;
                 -7.368611	57.358333;
                 -7.371944	57.361111;
                 -7.373056	57.364722;
                 -7.372778	57.369167;
                 -7.373056	57.373333;
                 -7.380000	57.374444;
                 -7.387500	57.375278;
                 -7.392222	57.377222;
                 -7.397222	57.379167;
                 -7.403889	57.380278;
                 -7.409722	57.381667;
                 -7.414444	57.383889;
                 -7.420278	57.385278;
                 -7.422500	57.388611;
                 -7.416667	57.388056;
                 -7.411111	57.386389;
                 -7.406111	57.384444;
                 -7.400278	57.382778;
                 -7.392500	57.383056;
                 -7.388056	57.385278;
                 -7.383056	57.386944;
                 -7.374444	57.386667;
                 -7.369167	57.388333;
                 -7.363889	57.390000;
                 -7.357778	57.391389;
                 -7.353056	57.389444;
                 -7.348889	57.386944;
                 -7.345000	57.384444;
                 -7.339167	57.383056;
                 -7.331667	57.382222;
                 -7.327500	57.380000;
                 -7.321667	57.378333;
                 -7.315278	57.377222;
                 -7.309444	57.375833;
                 -7.301667	57.375000;
                 -7.295833	57.373611;
                 -7.290000	57.371944;
                 -7.286111	57.369722;
                 -7.286389	57.366944;
                 -7.290000	57.364167;
                 -7.294444	57.362222;
                 -7.298056	57.359444;
                 -7.298333	57.356667;
                 -7.291667	57.355556;
                 -7.286944	57.353611;
                 -7.281944	57.351667;
                 -7.277778	57.349167;
                 -7.273889	57.346944;
                 -7.269167	57.345000;
                 -7.263056	57.343333;
                 -7.258333	57.341389;
                 -7.260556	57.337778;
                 -7.262222	57.335833];


  uki(65).xy = [ -7.216667	57.291111;
                 -7.218889	57.287500;
                 -7.220000	57.283611;
                 -7.221944	57.280000;
                 -7.224722	57.276667;
                 -7.229167	57.274722;
                 -7.234722	57.273056;
                 -7.242222	57.272778;
                 -7.247500	57.271111;
                 -7.251944	57.268889;
                 -7.253056	57.264722;
                 -7.256111	57.261667;
                 -7.259722	57.259167;
                 -7.262500	57.255833;
                 -7.265278	57.252778;
                 -7.266667	57.248611;
                 -7.270278	57.246111;
                 -7.273889	57.243333;
                 -7.276944	57.240278;
                 -7.278889	57.236667;
                 -7.280000	57.232500;
                 -7.281389	57.228611;
                 -7.284167	57.225278;
                 -7.288611	57.223333;
                 -7.293611	57.221667;
                 -7.301389	57.221389;
                 -7.308889	57.221944;
                 -7.313056	57.224444;
                 -7.316944	57.226667;
                 -7.321667	57.228889;
                 -7.328611	57.229722;
                 -7.334444	57.228611;
                 -7.340556	57.227500;
                 -7.347500	57.227778;
                 -7.353056	57.229167;
                 -7.357778	57.231111;
                 -7.364722	57.232222;
                 -7.365000	57.228611;
                 -7.366944	57.225000;
                 -7.371667	57.222778;
                 -7.366667	57.220833;
                 -7.362500	57.218333;
                 -7.357778	57.216389;
                 -7.349444	57.216389;
                 -7.343333	57.217500;
                 -7.337222	57.218611;
                 -7.330278	57.219444;
                 -7.324444	57.217778;
                 -7.323333	57.214167;
                 -7.321111	57.210833;
                 -7.316111	57.208889;
                 -7.308611	57.208333;
                 -7.301111	57.207500;
                 -7.294444	57.206389;
                 -7.288611	57.205000;
                 -7.285556	57.201944;
                 -7.283333	57.198889;
                 -7.281389	57.195556;
                 -7.280833	57.191389;
                 -7.281944	57.187222;
                 -7.282222	57.182778;
                 -7.279167	57.180000;
                 -7.276111	57.177222;
                 -7.272222	57.174722;
                 -7.271667	57.170556;
                 -7.272222	57.166111;
                 -7.270000	57.162778;
                 -7.266944	57.160000;
                 -7.267222	57.156111;
                 -7.270000	57.153056;
                 -7.274444	57.150833;
                 -7.279167	57.148889;
                 -7.284167	57.147222;
                 -7.289444	57.145278;
                 -7.296111	57.144722;
                 -7.301111	57.146667;
                 -7.305833	57.148611;
                 -7.310833	57.150556;
                 -7.314444	57.148056;
                 -7.318889	57.145833;
                 -7.326389	57.145556;
                 -7.333889	57.146111;
                 -7.338889	57.148056;
                 -7.344722	57.149722;
                 -7.350278	57.151389;
                 -7.358889	57.151389;
                 -7.363889	57.149722;
                 -7.360556	57.149722;
                 -7.357500	57.148611;
                 -7.355278	57.145556;
                 -7.350278	57.143611;
                 -7.347222	57.140556;
                 -7.345833	57.136944;
                 -7.342778	57.134167;
                 -7.336111	57.133056;
                 -7.329444	57.133611;
                 -7.323611	57.132222;
                 -7.316944	57.131944;
                 -7.312500	57.134167;
                 -7.307222	57.135833;
                 -7.299444	57.136111;
                 -7.292778	57.135833;
                 -7.288056	57.133889;
                 -7.283333	57.131944;
                 -7.276389	57.130833;
                 -7.269722	57.131667;
                 -7.262222	57.130833;
                 -7.258056	57.128611;
                 -7.261667	57.125833;
                 -7.265278	57.123333;
                 -7.263056	57.120000;
                 -7.257222	57.118333;
                 -7.250833	57.117222;
                 -7.245833	57.115278;
                 -7.243056	57.112500;
                 -7.240833	57.109167;
                 -7.240278	57.105000;
                 -7.238889	57.101389;
                 -7.239167	57.096667;
                 -7.242222	57.093611;
                 -7.246667	57.091389;
                 -7.252500	57.091111;
                 -7.257222	57.093333;
                 -7.263889	57.094444;
                 -7.272500	57.094444;
                 -7.278056	57.096111;
                 -7.282222	57.098611;
                 -7.287778	57.100000;
                 -7.294444	57.101111;
                 -7.301389	57.102222;
                 -7.306944	57.103889;
                 -7.314444	57.104444;
                 -7.319444	57.102778;
                 -7.323889	57.100556;
                 -7.328611	57.098333;
                 -7.335278	57.098611;
                 -7.340000	57.100556;
                 -7.345833	57.102222;
                 -7.354167	57.102500;
                 -7.361111	57.101667;
                 -7.368056	57.100833;
                 -7.374722	57.100278;
                 -7.383056	57.100556;
                 -7.388889	57.101944;
                 -7.393889	57.103889;
                 -7.396667	57.106667;
                 -7.401944	57.108611;
                 -7.407500	57.110278;
                 -7.410556	57.113056;
                 -7.411111	57.117222;
                 -7.408889	57.120833;
                 -7.411111	57.124167;
                 -7.414444	57.126944;
                 -7.417500	57.129722;
                 -7.421389	57.132222;
                 -7.425556	57.134722;
                 -7.427778	57.137778;
                 -7.430000	57.141111;
                 -7.430556	57.145278;
                 -7.429167	57.149444;
                 -7.426944	57.153056;
                 -7.425000	57.156667;
                 -7.423889	57.160556;
                 -7.426111	57.163889;
                 -7.430000	57.166389;
                 -7.434167	57.168611;
                 -7.438056	57.171111;
                 -7.440278	57.174444;
                 -7.441667	57.178056;
                 -7.442778	57.181667;
                 -7.443333	57.185833;
                 -7.443889	57.190000;
                 -7.443333	57.194722;
                 -7.442222	57.198611;
                 -7.440833	57.202778;
                 -7.440556	57.207222;
                 -7.443611	57.210278;
                 -7.447778	57.212500;
                 -7.448889	57.216389;
                 -7.448611	57.220833;
                 -7.448333	57.225278;
                 -7.448611	57.229444;
                 -7.452500	57.231944;
                 -7.460278	57.232500;
                 -7.464167	57.235000;
                 -7.462778	57.239167;
                 -7.456111	57.239722;
                 -7.450833	57.241389;
                 -7.448056	57.244722;
                 -7.446667	57.248611;
                 -7.445556	57.252778;
                 -7.444444	57.256944;
                 -7.444722	57.261111;
                 -7.448056	57.264444;
                 -7.447500	57.268333;
                 -7.443889	57.271111;
                 -7.439444	57.273333;
                 -7.439167	57.276944;
                 -7.441389	57.280278;
                 -7.439167	57.283889;
                 -7.434722	57.285833;
                 -7.428889	57.287222;
                 -7.424444	57.289167;
                 -7.420556	57.291944;
                 -7.418611	57.295556;
                 -7.417500	57.299722;
                 -7.416389	57.303611;
                 -7.415000	57.307778;
                 -7.414444	57.312500;
                 -7.414444	57.316944;
                 -7.413889	57.321389;
                 -7.413333	57.326111;
                 -7.413889	57.330278;
                 -7.414444	57.334444;
                 -7.414722	57.338333;
                 -7.416111	57.342222;
                 -7.417500	57.345833;
                 -7.418611	57.349444;
                 -7.420278	57.353333;
                 -7.422500	57.356389;
                 -7.424444	57.359722;
                 -7.427778	57.362500;
                 -7.430833	57.365556;
                 -7.433056	57.368611;
                 -7.436389	57.371667;
                 -7.439167	57.374444;
                 -7.440556	57.378056;
                 -7.436389	57.379167;
                 -7.430556	57.377778;
                 -7.422778	57.377222;
                 -7.415278	57.376389;
                 -7.409722	57.375000;
                 -7.405556	57.372500;
                 -7.403333	57.369167;
                 -7.401944	57.365556;
                 -7.402222	57.361111;
                 -7.401111	57.357222;
                 -7.398611	57.353889;
                 -7.395833	57.351111;
                 -7.391667	57.348889;
                 -7.386111	57.347222;
                 -7.378056	57.346667;
                 -7.371667	57.345556;
                 -7.367778	57.343056;
                 -7.360833	57.341944;
                 -7.353333	57.342222;
                 -7.346389	57.343056;
                 -7.339722	57.341944;
                 -7.335556	57.339444;
                 -7.330833	57.337500;
                 -7.325000	57.335833;
                 -7.320000	57.333889;
                 -7.315278	57.331944;
                 -7.309444	57.330556;
                 -7.304444	57.328611;
                 -7.299444	57.326389;
                 -7.293889	57.325000;
                 -7.289167	57.323056;
                 -7.284167	57.321111;
                 -7.279167	57.319167;
                 -7.274444	57.317222;
                 -7.268611	57.315556;
                 -7.261944	57.314444;
                 -7.254167	57.314722;
                 -7.246667	57.315000;
                 -7.239722	57.313889;
                 -7.236944	57.311111;
                 -7.237222	57.306389;
                 -7.240000	57.303333;
                 -7.243056	57.300278;
                 -7.242500	57.296944;
                 -7.235000	57.297222;
                 -7.228889	57.298333;
                 -7.220833	57.298611;
                 -7.216667	57.291111];


  uki(66).xy = [ -7.283889	57.078889;
                 -7.286111	57.075278;
                 -7.288056	57.071667;
                 -7.291111	57.068611;
                 -7.293056	57.065000;
                 -7.291667	57.061111;
                 -7.293611	57.057500;
                 -7.297500	57.055000;
                 -7.301667	57.052778;
                 -7.308611	57.052222;
                 -7.315278	57.053333;
                 -7.321667	57.054444;
                 -7.326667	57.056389;
                 -7.329722	57.059167;
                 -7.330278	57.063333;
                 -7.327500	57.066389;
                 -7.324444	57.069444;
                 -7.326667	57.072778;
                 -7.330833	57.075278;
                 -7.335556	57.077222;
                 -7.335833	57.081389;
                 -7.330278	57.080556;
                 -7.325556	57.078611;
                 -7.319444	57.078889;
                 -7.315000	57.081111;
                 -7.311389	57.083889;
                 -7.306944	57.085833;
                 -7.301111	57.087222;
                 -7.293333	57.087500;
                 -7.287500	57.085833;
                 -7.284444	57.083056;
                 -7.283889	57.078889];


  uki(67).xy = [ -7.391944	57.042500;
                 -7.396111	57.041389;
                 -7.402222	57.040000;
                 -7.408333	57.038889;
                 -7.415000	57.039167;
                 -7.418333	57.041944;
                 -7.416667	57.046111;
                 -7.417500	57.049167;
                 -7.421389	57.051667;
                 -7.417778	57.054167;
                 -7.412222	57.052778;
                 -7.406389	57.051111;
                 -7.399722	57.050000;
                 -7.393889	57.048611;
                 -7.389167	57.046667;
                 -7.384167	57.044722;
                 -7.391944	57.042500];


  uki(68).xy = [ -7.441944	57.038333;
                 -7.438889	57.035556;
                 -7.440278	57.031389;
                 -7.443611	57.028889;
                 -7.448333	57.026667;
                 -7.452500	57.024444;
                 -7.455278	57.021389;
                 -7.453056	57.018056;
                 -7.450000	57.015278;
                 -7.446111	57.013056;
                 -7.441111	57.010833;
                 -7.436389	57.008889;
                 -7.431389	57.006944;
                 -7.426389	57.005000;
                 -7.421667	57.003056;
                 -7.416667	57.001111;
                 -7.413889	56.998333;
                 -7.417500	56.995556;
                 -7.423333	56.994444;
                 -7.429444	56.993333;
                 -7.433889	56.991111;
                 -7.436389	56.988056;
                 -7.435278	56.984167;
                 -7.431389	56.981944;
                 -7.426389	56.980000;
                 -7.420000	56.978889;
                 -7.413333	56.977778;
                 -7.410278	56.974722;
                 -7.413889	56.972222;
                 -7.418889	56.970556;
                 -7.424722	56.969167;
                 -7.430833	56.968056;
                 -7.436111	56.966389;
                 -7.441389	56.964722;
                 -7.445556	56.962500;
                 -7.446667	56.958333;
                 -7.444444	56.955278;
                 -7.442500	56.951944;
                 -7.443611	56.947778;
                 -7.449722	56.946667;
                 -7.456389	56.946944;
                 -7.463056	56.946111;
                 -7.464444	56.941944;
                 -7.471944	56.941667;
                 -7.478889	56.941944;
                 -7.486111	56.942500;
                 -7.491944	56.944167;
                 -7.495833	56.946389;
                 -7.502500	56.947500;
                 -7.507500	56.945833;
                 -7.513611	56.944722;
                 -7.521111	56.945278;
                 -7.527778	56.946389;
                 -7.534444	56.945833;
                 -7.541944	56.945556;
                 -7.547778	56.946944;
                 -7.554444	56.948056;
                 -7.561944	56.947778;
                 -7.568611	56.948889;
                 -7.572778	56.951389;
                 -7.573889	56.955000;
                 -7.573611	56.959722;
                 -7.570833	56.962778;
                 -7.567222	56.965278;
                 -7.561944	56.966944;
                 -7.556944	56.968889;
                 -7.550833	56.970000;
                 -7.543889	56.970833;
                 -7.538889	56.972500;
                 -7.534444	56.974444;
                 -7.532500	56.978333;
                 -7.534722	56.981389;
                 -7.535278	56.985556;
                 -7.531389	56.988333;
                 -7.526389	56.990000;
                 -7.521111	56.991667;
                 -7.516944	56.993889;
                 -7.513889	56.996944;
                 -7.515278	57.000556;
                 -7.518333	57.003333;
                 -7.522500	57.005833;
                 -7.528056	57.007500;
                 -7.535556	57.008056;
                 -7.543333	57.008611;
                 -7.538056	57.009444;
                 -7.532778	57.011111;
                 -7.526944	57.012500;
                 -7.520000	57.013056;
                 -7.513056	57.013889;
                 -7.505556	57.014167;
                 -7.498889	57.014722;
                 -7.493056	57.016111;
                 -7.486944	57.017222;
                 -7.482500	57.019444;
                 -7.478889	57.021944;
                 -7.474444	57.024167;
                 -7.472500	57.027778;
                 -7.469444	57.031111;
                 -7.468611	57.035000;
                 -7.469722	57.038889;
                 -7.471111	57.042500;
                 -7.473333	57.045833;
                 -7.474722	57.049444;
                 -7.475000	57.053611;
                 -7.470000	57.055278;
                 -7.464167	57.054722;
                 -7.463611	57.050556;
                 -7.462222	57.046667;
                 -7.457222	57.044722;
                 -7.451667	57.043333;
                 -7.446667	57.041389;
                 -7.441944	57.038333];


  uki(69).xy = [ -7.507500	56.929444;
                 -7.511667	56.928056;
                 -7.518611	56.927500;
                 -7.526944	56.927778;
                 -7.533611	56.926944;
                 -7.536389	56.924722;
                 -7.530833	56.923333;
                 -7.523056	56.922500;
                 -7.518333	56.920556;
                 -7.517222	56.916944;
                 -7.521389	56.914722;
                 -7.526667	56.913056;
                 -7.533333	56.912222;
                 -7.540000	56.911667;
                 -7.545278	56.910000;
                 -7.549722	56.907778;
                 -7.556389	56.906944;
                 -7.563889	56.906667;
                 -7.568889	56.908611;
                 -7.570278	56.912222;
                 -7.568333	56.915833;
                 -7.563889	56.918056;
                 -7.564444	56.921389;
                 -7.570000	56.922778;
                 -7.575833	56.924444;
                 -7.580833	56.926389;
                 -7.583611	56.929167;
                 -7.579722	56.931389;
                 -7.576667	56.934444;
                 -7.578056	56.938333;
                 -7.581944	56.940556;
                 -7.576944	56.942222;
                 -7.569167	56.942500;
                 -7.560833	56.942222;
                 -7.552500	56.942222;
                 -7.547500	56.940278;
                 -7.544444	56.937222;
                 -7.538889	56.935833;
                 -7.532222	56.934722;
                 -7.523611	56.934444;
                 -7.516111	56.934722;
                 -7.507778	56.934444;
                 -7.499444	56.934167;
                 -7.498889	56.931944;
                 -7.503889	56.930278;
                 -7.507500	56.929444];


  uki(70).xy = [ -7.512778	56.882500;
                 -7.515278	56.879167;
                 -7.519167	56.876667;
                 -7.523889	56.878611;
                 -7.530556	56.879722;
                 -7.538056	56.879444;
                 -7.544722	56.880556;
                 -7.548611	56.883056;
                 -7.552500	56.885278;
                 -7.553889	56.888889;
                 -7.549444	56.891111;
                 -7.544444	56.892778;
                 -7.538333	56.894167;
                 -7.531389	56.894722;
                 -7.523889	56.895000;
                 -7.518333	56.893611;
                 -7.512500	56.891944;
                 -7.507500	56.890000;
                 -7.503611	56.887778;
                 -7.505556	56.884167;
                 -7.512778	56.882500];


  uki(71).xy = [ -7.575556	56.842778;
                 -7.583333	56.842778;
                 -7.589722	56.843611;
                 -7.598056	56.843889;
                 -7.605000	56.843333;
                 -7.613333	56.843333;
                 -7.620833	56.844167;
                 -7.619722	56.845833;
                 -7.613889	56.847222;
                 -7.609444	56.849167;
                 -7.604167	56.851111;
                 -7.599722	56.853056;
                 -7.594722	56.854722;
                 -7.587222	56.854167;
                 -7.583333	56.851944;
                 -7.578333	56.849722;
                 -7.574444	56.847500;
                 -7.570278	56.845000;
                 -7.571667	56.841944;
                 -7.575556	56.842778];


  uki(72).xy = [ -7.636389	56.817500;
                 -7.636667	56.813889;
                 -7.640278	56.811111;
                 -7.644722	56.808889;
                 -7.645833	56.805833;
                 -7.642778	56.803056;
                 -7.641389	56.799444;
                 -7.643333	56.795833;
                 -7.647778	56.793611;
                 -7.653611	56.792222;
                 -7.658889	56.790556;
                 -7.660000	56.787500;
                 -7.661944	56.783889;
                 -7.668056	56.782500;
                 -7.672778	56.783611;
                 -7.677778	56.785556;
                 -7.681667	56.788056;
                 -7.683889	56.791389;
                 -7.681944	56.795000;
                 -7.678333	56.797500;
                 -7.674167	56.799722;
                 -7.671111	56.802778;
                 -7.670000	56.806944;
                 -7.671389	56.810556;
                 -7.674444	56.813333;
                 -7.675833	56.817222;
                 -7.670556	56.818889;
                 -7.664444	56.820000;
                 -7.660278	56.822222;
                 -7.654167	56.823333;
                 -7.646667	56.823611;
                 -7.642778	56.821389;
                 -7.636389	56.817500];


  uki(73).xy = [ -7.351111	57.011111;
                 -7.346389	57.009167;
                 -7.342222	57.006944;
                 -7.340000	57.003611;
                 -7.342778	57.001389;
                 -7.351389	57.001667;
                 -7.356944	57.003056;
                 -7.362778	57.004722;
                 -7.369444	57.005833;
                 -7.371667	57.008889;
                 -7.370278	57.012222;
                 -7.364444	57.013333;
                 -7.356944	57.012778;
                 -7.351111	57.011111];


  uki(74).xy = [ -7.206111	57.388333;
                 -7.209722	57.386667;
                 -7.214167	57.384444;
                 -7.218889	57.382222;
                 -7.223889	57.380556;
                 -7.229722	57.380278;
                 -7.233889	57.382778;
                 -7.236944	57.385556;
                 -7.237500	57.389722;
                 -7.235278	57.393333;
                 -7.232500	57.396389;
                 -7.229444	57.399444;
                 -7.226111	57.402222;
                 -7.222500	57.405000;
                 -7.216944	57.406667;
                 -7.211111	57.406944;
                 -7.210556	57.402778;
                 -7.211944	57.399444;
                 -7.208611	57.396667;
                 -7.206667	57.393333;
                 -7.206111	57.388333];


  uki(75).xy = [ -7.224167	57.480556;
                 -7.223611	57.477222;
                 -7.222222	57.473611;
                 -7.224444	57.470000;
                 -7.226389	57.466389;
                 -7.230833	57.466944;
                 -7.233611	57.469722;
                 -7.236944	57.472500;
                 -7.242778	57.474167;
                 -7.250278	57.473889;
                 -7.254444	57.475278;
                 -7.259167	57.477222;
                 -7.266111	57.478333;
                 -7.271944	57.480000;
                 -7.275833	57.482500;
                 -7.276389	57.486389;
                 -7.272778	57.489167;
                 -7.265833	57.490000;
                 -7.261389	57.489167;
                 -7.253889	57.488611;
                 -7.246389	57.488889;
                 -7.241389	57.486944;
                 -7.237222	57.484444;
                 -7.232500	57.482500;
                 -7.227500	57.480556;
                 -7.224167	57.480556];

  
  uki(76).xy = [ -7.180556	57.489167;
                 -7.184444	57.486667;
                 -7.182222	57.483333;
                 -7.178889	57.480556;
                 -7.178333	57.477222;
                 -7.183611	57.475556;
                 -7.188889	57.473889;
                 -7.191944	57.470833;
                 -7.195556	57.468333;
                 -7.202500	57.467500;
                 -7.206667	57.470000;
                 -7.206667	57.473889;
                 -7.204722	57.477500;
                 -7.208056	57.480556;
                 -7.211944	57.482778;
                 -7.213333	57.486667;
                 -7.206389	57.487222;
                 -7.205000	57.491389;
                 -7.203889	57.495278;
                 -7.196944	57.496111;
                 -7.191111	57.494722;
                 -7.187222	57.492222;
                 -7.183889	57.490278;
                 -7.180556	57.489167];


  uki(77).xy = [ -7.219167	57.538333;
                 -7.221944	57.536111;
                 -7.224722	57.533056;
                 -7.226944	57.529444;
                 -7.229722	57.526389;
                 -7.230833	57.522222;
                 -7.232222	57.518333;
                 -7.230833	57.514444;
                 -7.227778	57.511667;
                 -7.225556	57.508333;
                 -7.227500	57.504722;
                 -7.233611	57.503611;
                 -7.239444	57.505278;
                 -7.244444	57.507222;
                 -7.251111	57.508333;
                 -7.257222	57.506944;
                 -7.264167	57.506389;
                 -7.270833	57.507500;
                 -7.275000	57.509722;
                 -7.277222	57.513056;
                 -7.277500	57.517222;
                 -7.275278	57.520833;
                 -7.271667	57.523611;
                 -7.268056	57.526111;
                 -7.263611	57.528333;
                 -7.260833	57.531389;
                 -7.259444	57.535556;
                 -7.258333	57.539444;
                 -7.254722	57.542222;
                 -7.249167	57.543889;
                 -7.243056	57.545000;
                 -7.235556	57.545278;
                 -7.228056	57.544722;
                 -7.223889	57.542222;
                 -7.219167	57.538333];


  uki(78).xy = [ -7.150556	57.541111;
                 -7.152778	57.537500;
                 -7.151389	57.533611;
                 -7.147500	57.531389;
                 -7.146111	57.527500;
                 -7.150000	57.525000;
                 -7.153611	57.522222;
                 -7.156389	57.519167;
                 -7.159167	57.516111;
                 -7.162222	57.513056;
                 -7.165000	57.509722;
                 -7.168611	57.507222;
                 -7.173056	57.505000;
                 -7.178333	57.503333;
                 -7.186111	57.504167;
                 -7.190833	57.506111;
                 -7.197500	57.507222;
                 -7.204444	57.508333;
                 -7.208333	57.510556;
                 -7.211389	57.513611;
                 -7.215556	57.515833;
                 -7.218611	57.518611;
                 -7.220000	57.522500;
                 -7.219444	57.526111;
                 -7.215833	57.528611;
                 -7.208333	57.528889;
                 -7.200556	57.528333;
                 -7.192778	57.527500;
                 -7.186111	57.528333;
                 -7.182500	57.531111;
                 -7.179444	57.534167;
                 -7.178333	57.538056;
                 -7.176389	57.541667;
                 -7.172500	57.544444;
                 -7.167222	57.546111;
                 -7.160278	57.546667;
                 -7.154444	57.546111;
                 -7.150556	57.541111];


  uki(79).xy = [ -4.901944	55.787222;
                 -4.900833	55.783333;
                 -4.901389	55.779722;
                 -4.902778	55.775556;
                 -4.903333	55.771944;
                 -4.902222	55.768333;
                 -4.899444	55.765278;
                 -4.895833	55.762778;
                 -4.893056	55.760000;
                 -4.894167	55.756944;
                 -4.898611	55.754722;
                 -4.901667	55.751667;
                 -4.904444	55.748611;
                 -4.908333	55.746111;
                 -4.914722	55.745556;
                 -4.921389	55.745833;
                 -4.928333	55.746667;
                 -4.935000	55.746111;
                 -4.940278	55.744444;
                 -4.946389	55.743333;
                 -4.953611	55.743333;
                 -4.955556	55.746389;
                 -4.956667	55.750278;
                 -4.955000	55.754444;
                 -4.952222	55.757500;
                 -4.948611	55.760000;
                 -4.945556	55.763056;
                 -4.941944	55.765556;
                 -4.938889	55.768611;
                 -4.937500	55.772778;
                 -4.936944	55.777222;
                 -4.936389	55.781944;
                 -4.932500	55.784444;
                 -4.927500	55.786111;
                 -4.923056	55.788056;
                 -4.919444	55.790833;
                 -4.915833	55.793333;
                 -4.908889	55.793889;
                 -4.904444	55.791944;
                 -4.901944	55.787222];

  uki(80).xy = [           -6.050000	53.380278;
                           -6.054444	53.373333;
                           -6.056389	53.369722;
                           -6.059167	53.367222;
                           -6.062222	53.364444;
                           -6.065556	53.361944;
                           -6.071667	53.361111;
                           -6.077778	53.361389;
                           -6.083333	53.362778;
                           -6.087778	53.365000;
                           -6.091389	53.367222;
                           -6.093889	53.370000;
                           -6.096944	53.373056;
                           -6.099722	53.375833;
                           -6.103333	53.378333;
                           -6.106944	53.380556;
                           -6.111389	53.383056;
                           -6.116389	53.384722;
                           -6.122500	53.385833;
                           -6.128611	53.386944;
                           -6.136111	53.386944;
                           -6.143333	53.386944;
                           -6.149444	53.386111;
                           -6.154167	53.385000;
                           -6.158889	53.383333;
                           -6.162778	53.381111;
                           -6.166389	53.378333;
                           -6.169722	53.375833;
                           -6.172222	53.372500;
                           -6.175556	53.370000;
                           -6.178611	53.367222;
                           -6.182778	53.365278;
                           -6.186667	53.362500;
                           -6.191667	53.360833;
                           -6.197222	53.359722;
                           -6.203333	53.358889;
                           -6.208611	53.358333;
                           -6.214444	53.356944;
                           -6.219722	53.355833;
                           -6.224444	53.354167;
                           -6.223889	53.351944;
                           -6.220278	53.349444;
                           -6.219722	53.346111;
                           -6.220278	53.342500;
                           -6.216667	53.340000;
                           -6.212222	53.338056;
                           -6.208611	53.335278;
                           -6.205556	53.332778;
                           -6.201944	53.330556;
                           -6.199167	53.327778;
                           -6.195556	53.325278;
                           -6.193611	53.321944;
                           -6.190556	53.319167;
                           -6.188056	53.316389;
                           -6.185000	53.313333;
                           -6.182500	53.310556;
                           -6.179444	53.307778;
                           -6.175833	53.305278;
                           -6.172222	53.303333;
                           -6.168611	53.301111;
                           -6.164167	53.299167;
                           -6.159722	53.296667;
                           -6.154444	53.295000;
                           -6.148333	53.293889;
                           -6.141667	53.293333;
                           -6.134167	53.293056;
                           -6.127222	53.292500;
                           -6.121944	53.290833;
                           -6.117500	53.288889;
                           -6.112222	53.286944;
                           -6.107778	53.284444;
                           -6.103611	53.282500;
                           -6.099167	53.280556;
                           -6.095556	53.278056;
                           -6.092778	53.275278;
                           -6.094444	53.271111;
                           -6.097222	53.268056;
                           -6.100556	53.265278;
                           -6.103889	53.262778;
                           -6.106389	53.259722;
                           -6.108889	53.256389;
                           -6.110000	53.252500;
                           -6.109722	53.248333;
                           -6.108333	53.245556;
                           -6.106389	53.242778;
                           -6.105278	53.238889;
                           -6.103333	53.235556;
                           -6.101944	53.231944;
                           -6.101667	53.227778;
                           -6.101944	53.223333;
                           -6.101667	53.219167;
                           -6.100278	53.215556;
                           -6.099167	53.212222;
                           -6.096389	53.209444;
                           -6.092778	53.206944;
                           -6.089167	53.204444;
                           -6.085556	53.202222;
                           -6.082778	53.199167;
                           -6.079722	53.196389;
                           -6.076389	53.193889;
                           -6.074167	53.190833;
                           -6.072222	53.186944;
                           -6.072778	53.183333;
                           -6.072222	53.179167;
                           -6.073611	53.174722;
                           -6.073611	53.170556;
                           -6.075000	53.166111;
                           -6.073611	53.162222;
                           -6.071667	53.158889;
                           -6.069722	53.155833;
                           -6.067778	53.153056;
                           -6.064167	53.150556;
                           -6.061389	53.147778;
                           -6.058611	53.144722;
                           -6.055000	53.142500;
                           -6.052222	53.139722;
                           -6.049444	53.136667;
                           -6.047500	53.133611;
                           -6.044722	53.130556;
                           -6.042778	53.127500;
                           -6.041389	53.124167;
                           -6.039444	53.120833;
                           -6.037222	53.117500;
                           -6.035278	53.114167;
                           -6.034167	53.110556;
                           -6.033333	53.106389;
                           -6.032778	53.102222;
                           -6.033056	53.098056;
                           -6.033333	53.094167;
                           -6.033056	53.090000;
                           -6.032778	53.085833;
                           -6.032500	53.081111;
                           -6.032778	53.076667;
                           -6.033333	53.072500;
                           -6.033333	53.068056;
                           -6.034722	53.063333;
                           -6.035833	53.059444;
                           -6.037500	53.055833;
                           -6.039444	53.052222;
                           -6.041389	53.048611;
                           -6.042500	53.044444;
                           -6.042222	53.040278;
                           -6.043333	53.035833;
                           -6.044444	53.031667;
                           -6.045556	53.027500;
                           -6.046944	53.023333;
                           -6.047778	53.019444;
                           -6.048333	53.014722;
                           -6.047222	53.011111;
                           -6.046667	53.007500;
                           -6.044722	53.004167;
                           -6.044444	53.000000;
                           -6.043333	52.995833;
                           -6.041944	52.992222;
                           -6.040000	52.988889;
                           -6.038056	52.985556;
                           -6.035278	52.982778;
                           -6.031667	52.980278;
                           -6.028889	52.978056;
                           -6.023611	52.976389;
                           -6.018611	52.974722;
                           -6.014167	52.972778;
                           -6.009167	52.971389;
                           -6.003611	52.969722;
                           -5.998611	52.968333;
                           -5.995000	52.965278;
                           -5.993889	52.961667;
                           -5.995556	52.958056;
                           -5.998889	52.955833;
                           -6.003056	52.953611;
                           -6.006944	52.951667;
                           -6.010278	52.948889;
                           -6.012778	52.945278;
                           -6.014722	52.941667;
                           -6.016667	52.938056;
                           -6.018333	52.934444;
                           -6.020278	52.930833;
                           -6.021944	52.927222;
                           -6.024722	52.924167;
                           -6.026667	52.920556;
                           -6.029722	52.917500;
                           -6.032500	52.914444;
                           -6.035000	52.911111;
                           -6.038333	52.908611;
                           -6.041389	52.905833;
                           -6.044722	52.903333;
                           -6.047500	52.900556;
                           -6.049722	52.897500;
                           -6.053333	52.895000;
                           -6.055556	52.891389;
                           -6.058333	52.888056;
                           -6.060000	52.884444;
                           -6.062778	52.881389;
                           -6.064722	52.877778;
                           -6.065556	52.873611;
                           -6.067500	52.870000;
                           -6.068611	52.866111;
                           -6.067500	52.862222;
                           -6.069444	52.858333;
                           -6.072778	52.855556;
                           -6.077500	52.853889;
                           -6.081389	52.851667;
                           -6.086111	52.850000;
                           -6.090833	52.848333;
                           -6.094722	52.846389;
                           -6.098611	52.844444;
                           -6.102778	52.841944;
                           -6.106667	52.840278;
                           -6.110000	52.837500;
                           -6.113333	52.835000;
                           -6.117222	52.832222;
                           -6.120000	52.829167;
                           -6.120833	52.825000;
                           -6.122778	52.821389;
                           -6.124444	52.817778;
                           -6.127222	52.814722;
                           -6.129722	52.811667;
                           -6.132222	52.808611;
                           -6.134167	52.805000;
                           -6.137500	52.801944;
                           -6.139167	52.798333;
                           -6.141111	52.794722;
                           -6.142778	52.791111;
                           -6.144722	52.787500;
                           -6.145833	52.783333;
                           -6.147778	52.779722;
                           -6.148889	52.775278;
                           -6.150000	52.771111;
                           -6.150278	52.766389;
                           -6.148333	52.763333;
                           -6.145833	52.760278;
                           -6.144444	52.756667;
                           -6.144167	52.752500;
                           -6.145278	52.748333;
                           -6.146111	52.743889;
                           -6.146667	52.739444;
                           -6.148333	52.736111;
                           -6.149722	52.731667;
                           -6.151389	52.728056;
                           -6.154167	52.725000;
                           -6.157222	52.722778;
                           -6.160556	52.719722;
                           -6.164444	52.716944;
                           -6.168333	52.714722;
                           -6.171944	52.712222;
                           -6.175000	52.709444;
                           -6.178333	52.706944;
                           -6.181667	52.704167;
                           -6.184722	52.701667;
                           -6.187500	52.698333;
                           -6.190556	52.695833;
                           -6.193056	52.692778;
                           -6.195833	52.689444;
                           -6.198333	52.686111;
                           -6.200833	52.682778;
                           -6.203333	52.680278;
                           -6.205833	52.676944;
                           -6.208611	52.673889;
                           -6.210833	52.670833;
                           -6.213333	52.667778;
                           -6.215278	52.664167;
                           -6.216944	52.660556;
                           -6.218889	52.656389;
                           -6.220278	52.652222;
                           -6.221111	52.648333;
                           -6.221667	52.643611;
                           -6.221944	52.639167;
                           -6.221667	52.635000;
                           -6.222778	52.630278;
                           -6.221667	52.626667;
                           -6.221111	52.622500;
                           -6.220000	52.618889;
                           -6.218056	52.615556;
                           -6.216111	52.612222;
                           -6.213056	52.609444;
                           -6.210556	52.606667;
                           -6.208611	52.603333;
                           -6.208056	52.599722;
                           -6.207778	52.595556;
                           -6.206667	52.591389;
                           -6.205278	52.587778;
                           -6.203889	52.586111;
                           -6.200556	52.580556;
                           -6.200278	52.576389;
                           -6.200000	52.572500;
                           -6.200833	52.567778;
                           -6.200000	52.564167;
                           -6.200278	52.559444;
                           -6.200000	52.555278;
                           -6.200833	52.551389;
                           -6.202500	52.547778;
                           -6.205278	52.544722;
                           -6.208611	52.541389;
                           -6.210833	52.538889;
                           -6.214444	52.536111;
                           -6.216667	52.533056;
                           -6.219167	52.530000;
                           -6.222778	52.527222;
                           -6.225000	52.524167;
                           -6.227500	52.521111;
                           -6.229444	52.517500;
                           -6.232778	52.514722;
                           -6.236111	52.511667;
                           -6.238611	52.508611;
                           -6.241667	52.505833;
                           -6.244444	52.502778;
                           -6.247500	52.500000;
                           -6.250833	52.497500;
                           -6.253889	52.494722;
                           -6.257222	52.492222;
                           -6.261111	52.490556;
                           -6.264444	52.487778;
                           -6.267500	52.485278;
                           -6.270833	52.481944;
                           -6.274167	52.479444;
                           -6.277222	52.476667;
                           -6.279722	52.473611;
                           -6.283333	52.471111;
                           -6.285556	52.467778;
                           -6.288056	52.464722;
                           -6.291389	52.461944;
                           -6.293889	52.458889;
                           -6.297222	52.456389;
                           -6.300556	52.453056;
                           -6.303611	52.450556;
                           -6.306944	52.448333;
                           -6.310000	52.445556;
                           -6.313889	52.443056;
                           -6.317222	52.440833;
                           -6.321389	52.438611;
                           -6.324444	52.436111;
                           -6.327500	52.433333;
                           -6.330833	52.430833;
                           -6.333889	52.428056;
                           -6.338056	52.425556;
                           -6.340556	52.422222;
                           -6.343889	52.419722;
                           -6.346389	52.416667;
                           -6.348889	52.413333;
                           -6.351389	52.410278;
                           -6.353333	52.406667;
                           -6.355833	52.403611;
                           -6.357500	52.400278;
                           -6.360000	52.396944;
                           -6.361944	52.393333;
                           -6.362778	52.389167;
                           -6.363889	52.385000;
                           -6.364167	52.380556;
                           -6.364722	52.375833;
                           -6.365833	52.371667;
                           -6.366944	52.367222;
                           -6.366389	52.363056;
                           -6.366111	52.358889;
                           -6.365000	52.355278;
                           -6.361944	52.352500;
                           -6.360000	52.349167;
                           -6.358889	52.345556;
                           -6.361944	52.342778;
                           -6.368056	52.343056;
                           -6.370833	52.345833;
                           -6.373889	52.348611;
                           -6.377222	52.351111;
                           -6.380000	52.353889;
                           -6.385000	52.355556;
                           -6.392500	52.355556;
                           -6.398056	52.354444;
                           -6.404722	52.354722;
                           -6.410556	52.355833;
                           -6.415000	52.357778;
                           -6.420833	52.358889;
                           -6.427778	52.358611;
                           -6.432500	52.356944;
                           -6.435556	52.354167;
                           -6.438056	52.351111;
                           -6.440000	52.347500;
                           -6.441667	52.343889;
                           -6.441944	52.339167;
                           -6.441389	52.335556;
                           -6.440556	52.331944;
                           -6.439167	52.328333;
                           -6.438611	52.324167;
                           -6.438889	52.319444;
                           -6.439444	52.315000;
                           -6.437500	52.311667;
                           -6.434444	52.309167;
                           -6.429444	52.307778;
                           -6.424444	52.306111;
                           -6.418333	52.305000;
                           -6.415000	52.302778;
                           -6.411389	52.300278;
                           -6.408056	52.298056;
                           -6.403611	52.295556;
                           -6.396667	52.295833;
                           -6.393611	52.298333;
                           -6.391944	52.301944;
                           -6.390556	52.306111;
                           -6.388889	52.310278;
                           -6.387778	52.314167;
                           -6.385278	52.317500;
                           -6.382778	52.320556;
                           -6.379722	52.323056;
                           -6.375833	52.325278;
                           -6.373611	52.323056;
                           -6.373889	52.318333;
                           -6.375833	52.314722;
                           -6.377500	52.311111;
                           -6.380000	52.307500;
                           -6.381944	52.303889;
                           -6.383056	52.300000;
                           -6.384722	52.296389;
                           -6.385833	52.292222;
                           -6.386111	52.287778;
                           -6.384167	52.284444;
                           -6.382222	52.281111;
                           -6.381111	52.277778;
                           -6.379167	52.274722;
                           -6.377778	52.271667;
                           -6.376944	52.271389;
                           -6.373611	52.268889;
                           -6.371667	52.265556;
                           -6.368056	52.263333;
                           -6.364444	52.260833;
                           -6.360000	52.258889;
                           -6.355833	52.256389;
                           -6.350556	52.255000;
                           -6.343889	52.254167;
                           -6.336667	52.254167;
                           -6.332222	52.251944;
                           -6.331667	52.248333;
                           -6.330556	52.244722;
                           -6.324722	52.243611;
                           -6.319444	52.241944;
                           -6.318333	52.239167;
                           -6.321667	52.236667;
                           -6.324722	52.233889;
                           -6.324444	52.229722;
                           -6.323333	52.226111;
                           -6.325000	52.222500;
                           -6.328889	52.220000;
                           -6.332778	52.217778;
                           -6.336944	52.215000;
                           -6.340000	52.212500;
                           -6.343889	52.210278;
                           -6.346389	52.207222;
                           -6.348056	52.203611;
                           -6.347778	52.199444;
                           -6.347500	52.195278;
                           -6.349722	52.192222;
                           -6.353889	52.189444;
                           -6.355833	52.185833;
                           -6.356667	52.181667;
                           -6.359167	52.178611;
                           -6.363056	52.176944;
                           -6.367778	52.175278;
                           -6.374444	52.175000;
                           -6.380278	52.175278;
                           -6.387222	52.175833;
                           -6.393889	52.176667;
                           -6.398889	52.178056;
                           -6.403333	52.180000;
                           -6.405278	52.183333;
                           -6.402222	52.185833;
                           -6.397500	52.187500;
                           -6.393611	52.189722;
                           -6.388611	52.191944;
                           -6.385833	52.194722;
                           -6.383889	52.198333;
                           -6.383611	52.202778;
                           -6.389444	52.203889;
                           -6.393056	52.202500;
                           -6.397222	52.200556;
                           -6.402222	52.199167;
                           -6.407778	52.198611;
                           -6.412222	52.196667;
                           -6.416944	52.195000;
                           -6.422500	52.193889;
                           -6.428333	52.193056;
                           -6.434444	52.192500;
                           -6.440556	52.191667;
                           -6.447222	52.191111;
                           -6.453889	52.191111;
                           -6.460000	52.190000;
                           -6.466111	52.189722;
                           -6.472778	52.189444;
                           -6.477778	52.191111;
                           -6.473889	52.193611;
                           -6.469167	52.194444;
                           -6.462500	52.196111;
                           -6.458056	52.195556;
                           -6.451389	52.195556;
                           -6.446667	52.197500;
                           -6.448056	52.200278;
                           -6.453056	52.201667;
                           -6.458333	52.203333;
                           -6.463333	52.205278;
                           -6.468889	52.206389;
                           -6.475000	52.207500;
                           -6.481667	52.207222;
                           -6.485556	52.205000;
                           -6.488333	52.201944;
                           -6.490000	52.198333;
                           -6.492500	52.195000;
                           -6.495556	52.192500;
                           -6.501111	52.190833;
                           -6.506389	52.189444;
                           -6.511667	52.188889;
                           -6.517778	52.187500;
                           -6.523611	52.186944;
                           -6.529722	52.186111;
                           -6.535833	52.185278;
                           -6.542500	52.185000;
                           -6.549444	52.184722;
                           -6.553889	52.183611;
                           -6.557778	52.181389;
                           -6.561111	52.178889;
                           -6.565000	52.176667;
                           -6.569444	52.175000;
                           -6.575000	52.173611;
                           -6.580000	52.172500;
                           -6.585556	52.171111;
                           -6.591667	52.171389;
                           -6.595833	52.173333;
                           -6.597778	52.175833;
                           -6.601389	52.178611;
                           -6.605000	52.180833;
                           -6.608333	52.183333;
                           -6.611944	52.185833;
                           -6.616389	52.187778;
                           -6.620000	52.190000;
                           -6.622778	52.192500;
                           -6.627222	52.194444;
                           -6.631944	52.195833;
                           -6.636389	52.198056;
                           -6.641667	52.199444;
                           -6.646667	52.200833;
                           -6.651944	52.203056;
                           -6.657778	52.203889;
                           -6.663611	52.205000;
                           -6.669444	52.206111;
                           -6.675556	52.207222;
                           -6.681389	52.208333;
                           -6.687222	52.209444;
                           -6.693889	52.210000;
                           -6.699722	52.211667;
                           -6.706667	52.212222;
                           -6.712500	52.213333;
                           -6.718333	52.214444;
                           -6.725833	52.214722;
                           -6.732500	52.214444;
                           -6.737778	52.213056;
                           -6.743056	52.211944;
                           -6.749167	52.211667;
                           -6.755833	52.211389;
                           -6.761111	52.210000;
                           -6.765833	52.208333;
                           -6.770556	52.206667;
                           -6.775833	52.205556;
                           -6.781111	52.204167;
                           -6.788611	52.204444;
                           -6.794444	52.205556;
                           -6.798889	52.208056;
                           -6.802222	52.210278;
                           -6.803333	52.213889;
                           -6.800278	52.216667;
                           -6.794444	52.215278;
                           -6.789167	52.216944;
                           -6.787222	52.220556;
                           -6.785000	52.224167;
                           -6.783333	52.227778;
                           -6.779167	52.229722;
                           -6.775278	52.231944;
                           -6.772222	52.234722;
                           -6.769444	52.237778;
                           -6.766389	52.240556;
                           -6.763889	52.243611;
                           -6.763056	52.247778;
                           -6.761111	52.251944;
                           -6.762500	52.255556;
                           -6.765833	52.257778;
                           -6.771111	52.257500;
                           -6.773056	52.253889;
                           -6.775278	52.250278;
                           -6.777778	52.247222;
                           -6.781111	52.244444;
                           -6.785000	52.242500;
                           -6.791667	52.242222;
                           -6.797222	52.241389;
                           -6.797500	52.237778;
                           -6.797500	52.233056;
                           -6.800833	52.230556;
                           -6.805556	52.228611;
                           -6.811389	52.228056;
                           -6.818056	52.228611;
                           -6.822500	52.230556;
                           -6.826944	52.232500;
                           -6.831389	52.234444;
                           -6.835556	52.233611;
                           -6.835278	52.229444;
                           -6.833333	52.226389;
                           -6.831944	52.222500;
                           -6.833611	52.218611;
                           -6.837222	52.215833;
                           -6.839722	52.212778;
                           -6.838889	52.210278;
                           -6.833056	52.211111;
                           -6.827500	52.212500;
                           -6.823056	52.214167;
                           -6.816944	52.213889;
                           -6.815556	52.210278;
                           -6.814444	52.206667;
                           -6.815556	52.202500;
                           -6.819444	52.200278;
                           -6.823889	52.198611;
                           -6.827778	52.196389;
                           -6.829722	52.192778;
                           -6.826111	52.191667;
                           -6.820000	52.190556;
                           -6.818056	52.187500;
                           -6.821389	52.184722;
                           -6.825278	52.182500;
                           -6.828333	52.180000;
                           -6.827222	52.176111;
                           -6.828056	52.172222;
                           -6.831944	52.170000;
                           -6.836667	52.169167;
                           -6.842778	52.168333;
                           -6.848611	52.168056;
                           -6.853889	52.166944;
                           -6.859444	52.165556;
                           -6.864444	52.164444;
                           -6.870000	52.163056;
                           -6.875278	52.161389;
                           -6.880000	52.159722;
                           -6.884444	52.158056;
                           -6.888333	52.155833;
                           -6.891667	52.153056;
                           -6.894722	52.150833;
                           -6.898056	52.148333;
                           -6.900278	52.145278;
                           -6.902778	52.141944;
                           -6.906111	52.139444;
                           -6.908333	52.136111;
                           -6.911667	52.133611;
                           -6.914722	52.130556;
                           -6.918611	52.128333;
                           -6.921944	52.125556;
                           -6.926389	52.123889;
                           -6.930556	52.121667;
                           -6.935556	52.120556;
                           -6.942222	52.121389;
                           -6.944444	52.124722;
                           -6.945556	52.128333;
                           -6.943889	52.132500;
                           -6.942222	52.136111;
                           -6.938611	52.138889;
                           -6.935833	52.141111;
                           -6.932500	52.143611;
                           -6.928611	52.145833;
                           -6.924722	52.148056;
                           -6.920278	52.149722;
                           -6.915556	52.151389;
                           -6.911667	52.153611;
                           -6.907778	52.155833;
                           -6.905278	52.158889;
                           -6.902778	52.162500;
                           -6.901944	52.166667;
                           -6.903056	52.170556;
                           -6.905000	52.173611;
                           -6.907778	52.176389;
                           -6.910556	52.179444;
                           -6.912500	52.182500;
                           -6.913889	52.186389;
                           -6.914444	52.190278;
                           -6.912500	52.194444;
                           -6.912222	52.199167;
                           -6.912500	52.203333;
                           -6.914722	52.206389;
                           -6.916111	52.209167;
                           -6.918333	52.211667;
                           -6.920833	52.214444;
                           -6.925278	52.216389;
                           -6.929722	52.218333;
                           -6.933889	52.220278;
                           -6.937500	52.222222;
                           -6.940556	52.225556;
                           -6.943056	52.228333;
                           -6.945833	52.231111;
                           -6.948611	52.233889;
                           -6.953056	52.235833;
                           -6.957222	52.237778;
                           -6.961667	52.239722;
                           -6.966111	52.241667;
                           -6.969722	52.244167;
                           -6.971667	52.247222;
                           -6.974722	52.250278;
                           -6.975000	52.253889;
                           -6.976389	52.257500;
                           -6.976944	52.261667;
                           -6.975556	52.265833;
                           -6.975278	52.270278;
                           -6.976667	52.274167;
                           -6.979444	52.276944;
                           -6.984722	52.278333;
                           -6.985278	52.278889;
                           -6.990278	52.281389;
                           -6.993889	52.283056;
                           -6.997778	52.280556;
                           -6.998889	52.276389;
                           -6.997500	52.272778;
                           -6.995000	52.270000;
                           -6.991111	52.267500;
                           -6.988611	52.264722;
                           -6.987222	52.260556;
                           -6.986944	52.256389;
                           -6.986944	52.251944;
                           -6.988333	52.247222;
                           -6.986111	52.244167;
                           -6.982778	52.241667;
                           -6.978889	52.239167;
                           -6.976111	52.236389;
                           -6.973333	52.233611;
                           -6.972222	52.230000;
                           -6.970833	52.226111;
                           -6.970556	52.222222;
                           -6.972222	52.218056;
                           -6.974167	52.214444;
                           -6.974167	52.210833;
                           -6.974722	52.207222;
                           -6.976944	52.203889;
                           -6.980278	52.201389;
                           -6.982778	52.198056;
                           -6.985000	52.195000;
                           -6.988333	52.190833;
                           -6.987778	52.187222;
                           -6.984167	52.185000;
                           -6.978333	52.183889;
                           -6.974722	52.181389;
                           -6.968056	52.181667;
                           -6.961944	52.182500;
                           -6.955278	52.181944;
                           -6.953333	52.178611;
                           -6.956667	52.175833;
                           -6.960833	52.174167;
                           -6.964722	52.171944;
                           -6.969444	52.170278;
                           -6.973333	52.168056;
                           -6.976667	52.165556;
                           -6.978889	52.162222;
                           -6.981389	52.158611;
                           -6.984722	52.156111;
                           -6.987778	52.153333;
                           -6.990278	52.150833;
                           -6.992778	52.147500;
                           -6.995000	52.144444;
                           -6.997500	52.141389;
                           -7.000833	52.138611;
                           -7.004722	52.136389;
                           -7.010833	52.135556;
                           -7.016944	52.135000;
                           -7.022778	52.134167;
                           -7.029444	52.133889;
                           -7.035556	52.133056;
                           -7.042222	52.133333;
                           -7.049444	52.133333;
                           -7.057222	52.133611;
                           -7.064444	52.133889;
                           -7.071944	52.133889;
                           -7.078056	52.133333;
                           -7.083333	52.131944;
                           -7.089444	52.131111;
                           -7.094722	52.130000;
                           -7.100556	52.129167;
                           -7.105833	52.130556;
                           -7.106111	52.134444;
                           -7.103333	52.136944;
                           -7.100000	52.139722;
                           -7.096111	52.141944;
                           -7.093056	52.144444;
                           -7.089167	52.146667;
                           -7.085833	52.149444;
                           -7.081944	52.151111;
                           -7.078889	52.153889;
                           -7.075556	52.156389;
                           -7.074722	52.160556;
                           -7.077500	52.163056;
                           -7.081944	52.165000;
                           -7.086111	52.167222;
                           -7.091389	52.168889;
                           -7.097222	52.170000;
                           -7.103889	52.170556;
                           -7.111389	52.170556;
                           -7.118056	52.170278;
                           -7.121944	52.168333;
                           -7.121667	52.164167;
                           -7.119722	52.161389;
                           -7.115278	52.159444;
                           -7.110000	52.157778;
                           -7.107222	52.155000;
                           -7.106667	52.150833;
                           -7.111389	52.149167;
                           -7.118056	52.148889;
                           -7.123889	52.150000;
                           -7.128333	52.151944;
                           -7.133333	52.153889;
                           -7.139444	52.155000;
                           -7.146111	52.155556;
                           -7.152222	52.154722;
                           -7.156111	52.152500;
                           -7.159722	52.150278;
                           -7.162222	52.147222;
                           -7.164444	52.143889;
                           -7.166389	52.140278;
                           -7.168611	52.137222;
                           -7.171944	52.134722;
                           -7.175833	52.132500;
                           -7.181111	52.130556;
                           -7.187222	52.130278;
                           -7.193889	52.130000;
                           -7.201389	52.130278;
                           -7.208611	52.130556;
                           -7.215556	52.130278;
                           -7.222778	52.130278;
                           -7.228889	52.130833;
                           -7.235000	52.132500;
                           -7.240833	52.133611;
                           -7.247222	52.134167;
                           -7.255000	52.134167;
                           -7.261389	52.135000;
                           -7.268333	52.135556;
                           -7.274167	52.136667;
                           -7.281389	52.137222;
                           -7.288333	52.136944;
                           -7.293333	52.135556;
                           -7.298056	52.133889;
                           -7.302778	52.132222;
                           -7.309444	52.131944;
                           -7.315556	52.131111;
                           -7.323056	52.131389;
                           -7.329722	52.130833;
                           -7.336389	52.131111;
                           -7.343889	52.131389;
                           -7.350556	52.131944;
                           -7.356389	52.133056;
                           -7.362222	52.133889;
                           -7.368056	52.134167;
                           -7.372778	52.132500;
                           -7.377500	52.130556;
                           -7.381944	52.128333;
                           -7.386389	52.127222;
                           -7.391944	52.125833;
                           -7.398056	52.125278;
                           -7.403889	52.125278;
                           -7.409722	52.126389;
                           -7.415556	52.127500;
                           -7.423056	52.127500;
                           -7.428889	52.127222;
                           -7.435000	52.126389;
                           -7.439722	52.124722;
                           -7.445000	52.123333;
                           -7.449444	52.121667;
                           -7.454167	52.120000;
                           -7.458611	52.118333;
                           -7.463333	52.116389;
                           -7.468611	52.115278;
                           -7.473889	52.114444;
                           -7.479167	52.112778;
                           -7.483611	52.111389;
                           -7.488333	52.109722;
                           -7.493056	52.108056;
                           -7.496667	52.105833;
                           -7.499722	52.103056;
                           -7.504444	52.101389;
                           -7.509722	52.100000;
                           -7.515556	52.099444;
                           -7.522500	52.098889;
                           -7.529722	52.099167;
                           -7.535833	52.098333;
                           -7.540556	52.096667;
                           -7.545000	52.095000;
                           -7.548056	52.092222;
                           -7.550556	52.089167;
                           -7.553056	52.085833;
                           -7.555556	52.082778;
                           -7.557778	52.079722;
                           -7.564444	52.079167;
                           -7.569444	52.080833;
                           -7.572500	52.083611;
                           -7.574444	52.086944;
                           -7.577500	52.090000;
                           -7.580833	52.092500;
                           -7.585278	52.094444;
                           -7.591944	52.095000;
                           -7.597778	52.094167;
                           -7.602500	52.092500;
                           -7.607222	52.090556;
                           -7.610833	52.088333;
                           -7.614722	52.086389;
                           -7.617222	52.083056;
                           -7.616667	52.078889;
                           -7.612222	52.076944;
                           -7.608611	52.074722;
                           -7.605833	52.071944;
                           -7.604444	52.068056;
                           -7.603333	52.065000;
                           -7.601389	52.061667;
                           -7.596944	52.059722;
                           -7.593889	52.058889;
                           -7.587778	52.057778;
                           -7.580556	52.057500;
                           -7.574722	52.058056;
                           -7.568611	52.058611;
                           -7.561667	52.059167;
                           -7.555278	52.059444;
                           -7.548611	52.058889;
                           -7.544167	52.056944;
                           -7.541944	52.053611;
                           -7.543611	52.050000;
                           -7.546944	52.047222;
                           -7.550000	52.044722;
                           -7.554722	52.042778;
                           -7.558333	52.040556;
                           -7.563056	52.038889;
                           -7.568333	52.037778;
                           -7.574167	52.036389;
                           -7.578056	52.034722;
                           -7.579167	52.030556;
                           -7.582222	52.027778;
                           -7.585556	52.025278;
                           -7.587778	52.021944;
                           -7.587778	52.018333;
                           -7.585833	52.015278;
                           -7.583611	52.011944;
                           -7.581667	52.008611;
                           -7.581944	52.004444;
                           -7.584444	52.001389;
                           -7.588889	51.999722;
                           -7.593611	51.997778;
                           -7.598056	51.996111;
                           -7.602500	51.994444;
                           -7.607778	51.992778;
                           -7.613333	51.991389;
                           -7.617778	51.989722;
                           -7.623056	51.988889;
                           -7.628056	51.987500;
                           -7.633611	51.986389;
                           -7.639722	51.985556;
                           -7.644722	51.984167;
                           -7.650278	51.983056;
                           -7.654444	51.981389;
                           -7.660000	51.980000;
                           -7.666389	51.980278;
                           -7.673333	51.979722;
                           -7.680000	51.979444;
                           -7.686667	51.979167;
                           -7.693333	51.978889;
                           -7.699444	51.978056;
                           -7.705278	51.977222;
                           -7.710556	51.976111;
                           -7.715278	51.973889;
                           -7.718333	51.971667;
                           -7.721389	51.968889;
                           -7.723889	51.965833;
                           -7.725556	51.961944;
                           -7.725556	51.957500;
                           -7.725000	51.953333;
                           -7.724722	51.949167;
                           -7.725556	51.945000;
                           -7.729444	51.943333;
                           -7.735278	51.942500;
                           -7.741389	51.941667;
                           -7.748056	51.941389;
                           -7.755278	51.941667;
                           -7.761389	51.942500;
                           -7.767222	51.944167;
                           -7.771667	51.945556;
                           -7.776111	51.947500;
                           -7.779444	51.949722;
                           -7.782500	51.952500;
                           -7.787500	51.954167;
                           -7.793333	51.953333;
                           -7.798889	51.951944;
                           -7.805000	51.951111;
                           -7.810000	51.950000;
                           -7.812500	51.947222;
                           -7.816944	51.945556;
                           -7.823611	51.945278;
                           -7.830278	51.945833;
                           -7.835556	51.947222;
                           -7.840556	51.948611;
                           -7.844167	51.951111;
                           -7.847778	51.953333;
                           -7.850556	51.956111;
                           -7.855000	51.958056;
                           -7.860000	51.959444;
                           -7.863333	51.956944;
                           -7.864167	51.952778;
                           -7.865000	51.948611;
                           -7.866667	51.945556;
                           -7.870000	51.941944;
                           -7.871667	51.938333;
                           -7.873889	51.935000;
                           -7.876111	51.931944;
                           -7.878611	51.928889;
                           -7.881111	51.925556;
                           -7.883611	51.922500;
                           -7.885833	51.919167;
                           -7.888056	51.916111;
                           -7.891389	51.913333;
                           -7.894167	51.910278;
                           -7.899722	51.908889;
                           -7.905556	51.910000;
                           -7.910000	51.911944;
                           -7.915833	51.912500;
                           -7.921111	51.911667;
                           -7.921944	51.908056;
                           -7.918333	51.905556;
                           -7.914722	51.903333;
                           -7.913333	51.899167;
                           -7.912222	51.895556;
                           -7.906389	51.895278;
                           -7.900278	51.895278;
                           -7.897500	51.892500;
                           -7.894722	51.889722;
                           -7.890278	51.887778;
                           -7.884167	51.886667;
                           -7.880833	51.884444;
                           -7.880000	51.880833;
                           -7.882500	51.877500;
                           -7.888056	51.876389;
                           -7.893889	51.875556;
                           -7.900000	51.874722;
                           -7.906389	51.874444;
                           -7.913056	51.874444;
                           -7.920000	51.874167;
                           -7.926389	51.873889;
                           -7.933333	51.873333;
                           -7.940000	51.873056;
                           -7.946667	51.872778;
                           -7.952500	51.871944;
                           -7.958611	51.871111;
                           -7.964722	51.870278;
                           -7.969722	51.869444;
                           -7.975000	51.868333;
                           -7.980833	51.867500;
                           -7.985278	51.865556;
                           -7.990833	51.864444;
                           -7.995833	51.863056;
                           -8.000556	51.861389;
                           -8.005000	51.859444;
                           -8.008889	51.857222;
                           -8.013333	51.856111;
                           -8.016389	51.853333;
                           -8.020278	51.851111;
                           -8.023333	51.848333;
                           -8.025556	51.844722;
                           -8.026667	51.840833;
                           -8.024722	51.837500;
                           -8.021667	51.834722;
                           -8.016667	51.833333;
                           -8.012222	51.831389;
                           -8.010278	51.827500;
                           -8.011111	51.824444;
                           -8.015556	51.823056;
                           -8.020833	51.821944;
                           -8.027500	51.821389;
                           -8.033333	51.820556;
                           -8.040000	51.820278;
                           -8.046111	51.819444;
                           -8.050556	51.817778;
                           -8.055833	51.816111;
                           -8.059444	51.814167;
                           -8.065000	51.813056;
                           -8.070833	51.812222;
                           -8.077500	51.811667;
                           -8.084167	51.811389;
                           -8.091667	51.811667;
                           -8.098333	51.811111;
                           -8.105000	51.811389;
                           -8.111667	51.811111;
                           -8.118333	51.810556;
                           -8.124167	51.809722;
                           -8.129444	51.808611;
                           -8.134722	51.807222;
                           -8.140000	51.805833;
                           -8.144444	51.804167;
                           -8.148889	51.802500;
                           -8.152778	51.800278;
                           -8.156389	51.798056;
                           -8.159722	51.795833;
                           -8.163333	51.793611;
                           -8.168611	51.792222;
                           -8.173889	51.792778;
                           -8.177500	51.795000;
                           -8.180278	51.797778;
                           -8.186944	51.798333;
                           -8.192778	51.798611;
                           -8.198611	51.797778;
                           -8.204722	51.796944;
                           -8.210556	51.796389;
                           -8.217222	51.796111;
                           -8.224722	51.796389;
                           -8.231389	51.796944;
                           -8.238056	51.797500;
                           -8.245278	51.797500;
                           -8.250556	51.798056;
                           -8.252778	51.801389;
                           -8.254722	51.805000;
                           -8.258333	51.807222;
                           -8.261111	51.810000;
                           -8.264167	51.812500;
                           -8.264722	51.816667;
                           -8.263611	51.820556;
                           -8.260833	51.823333;
                           -8.258333	51.826667;
                           -8.256111	51.829167;
                           -8.250833	51.830556;
                           -8.246389	51.828611;
                           -8.241111	51.827222;
                           -8.238889	51.830278;
                           -8.236667	51.833611;
                           -8.234167	51.836667;
                           -8.231111	51.839444;
                           -8.226667	51.841111;
                           -8.222222	51.843056;
                           -8.216667	51.844722;
                           -8.211667	51.846111;
                           -8.206944	51.847222;
                           -8.202500	51.849167;
                           -8.198611	51.851389;
                           -8.195000	51.853611;
                           -8.191111	51.855833;
                           -8.187500	51.858056;
                           -8.186389	51.860833;
                           -8.186389	51.860278;
                           -8.185833	51.864444;
                           -8.183333	51.867500;
                           -8.179444	51.869722;
                           -8.175833	51.872500;
                           -8.173333	51.875278;
                           -8.171944	51.878889;
                           -8.170833	51.882778;
                           -8.174444	51.885833;
                           -8.180556	51.885278;
                           -8.184722	51.883611;
                           -8.189444	51.881667;
                           -8.194167	51.880000;
                           -8.200000	51.878889;
                           -8.205000	51.878056;
                           -8.211111	51.877222;
                           -8.218611	51.877222;
                           -8.225278	51.877778;
                           -8.231944	51.878333;
                           -8.238611	51.878889;
                           -8.245833	51.878889;
                           -8.251944	51.878056;
                           -8.257778	51.877222;
                           -8.263056	51.875556;
                           -8.267500	51.874167;
                           -8.272222	51.872500;
                           -8.277222	51.870833;
                           -8.281389	51.868889;
                           -8.286389	51.867778;
                           -8.291389	51.866389;
                           -8.297500	51.865556;
                           -8.303333	51.866667;
                           -8.307778	51.868333;
                           -8.312222	51.870278;
                           -8.315278	51.872222;
                           -8.320278	51.873333;
                           -8.325556	51.874722;
                           -8.329167	51.877500;
                           -8.332778	51.879722;
                           -8.339444	51.880278;
                           -8.345833	51.880833;
                           -8.351111	51.881944;
                           -8.357778	51.883056;
                           -8.363889	51.883333;
                           -8.369722	51.884444;
                           -8.374722	51.885833;
                           -8.379167	51.887778;
                           -8.385833	51.888333;
                           -8.390833	51.886944;
                           -8.394722	51.884722;
                           -8.395833	51.880833;
                           -8.392778	51.877778;
                           -8.394167	51.874167;
                           -8.398056	51.871944;
                           -8.403333	51.870833;
                           -8.407222	51.868611;
                           -8.404167	51.865833;
                           -8.397778	51.865278;
                           -8.391111	51.865556;
                           -8.384167	51.865833;
                           -8.377778	51.865278;
                           -8.371111	51.864722;
                           -8.363611	51.864722;
                           -8.356944	51.865000;
                           -8.350278	51.865556;
                           -8.343611	51.865833;
                           -8.336944	51.865278;
                           -8.331111	51.864167;
                           -8.325833	51.862778;
                           -8.322222	51.860556;
                           -8.319444	51.857778;
                           -8.318889	51.853611;
                           -8.319722	51.849444;
                           -8.320556	51.845278;
                           -8.319167	51.843889;
                           -8.319167	51.843889;
                           -8.317778	51.839444;
                           -8.311944	51.837778;
                           -8.306111	51.836944;
                           -8.301667	51.835000;
                           -8.299444	51.831667;
                           -8.298333	51.828056;
                           -8.298333	51.823333;
                           -8.298611	51.819722;
                           -8.295000	51.817500;
                           -8.289167	51.816389;
                           -8.283889	51.815000;
                           -8.281944	51.811667;
                           -8.281944	51.807778;
                           -8.281389	51.803611;
                           -8.282500	51.799444;
                           -8.284722	51.796389;
                           -8.287778	51.793611;
                           -8.291389	51.791389;
                           -8.293333	51.787778;
                           -8.297500	51.785833;
                           -8.303056	51.784722;
                           -8.307500	51.782778;
                           -8.311389	51.781111;
                           -8.314167	51.778056;
                           -8.309167	51.776389;
                           -8.301667	51.775833;
                           -8.297500	51.773889;
                           -8.295278	51.770833;
                           -8.295278	51.766111;
                           -8.297222	51.762500;
                           -8.300000	51.759722;
                           -8.302500	51.756667;
                           -8.305556	51.754444;
                           -8.307778	51.751389;
                           -8.311111	51.748056;
                           -8.313333	51.745000;
                           -8.313611	51.740278;
                           -8.315833	51.736667;
                           -8.320278	51.735000;
                           -8.327500	51.735000;
                           -8.333611	51.736111;
                           -8.339722	51.735278;
                           -8.340278	51.731111;
                           -8.341389	51.726944;
                           -8.341667	51.722500;
                           -8.343889	51.719167;
                           -8.348333	51.718056;
                           -8.355000	51.717500;
                           -8.361667	51.717222;
                           -8.369167	51.716944;
                           -8.375833	51.716667;
                           -8.380833	51.715278;
                           -8.386111	51.713889;
                           -8.390556	51.712222;
                           -8.393889	51.709444;
                           -8.393056	51.705278;
                           -8.397500	51.703611;
                           -8.402778	51.705556;
                           -8.406944	51.707500;
                           -8.412778	51.708333;
                           -8.418333	51.706944;
                           -8.423333	51.705833;
                           -8.428333	51.704444;
                           -8.433889	51.703056;
                           -8.435278	51.699444;
                           -8.436944	51.695833;
                           -8.438611	51.691667;
                           -8.442500	51.689444;
                           -8.447500	51.690556;
                           -8.451944	51.691944;
                           -8.454722	51.695278;
                           -8.458889	51.696944;
                           -8.464444	51.696667;
                           -8.465000	51.692500;
                           -8.466667	51.688889;
                           -8.468889	51.685833;
                           -8.472778	51.683611;
                           -8.478056	51.681667;
                           -8.482778	51.680000;
                           -8.487778	51.678611;
                           -8.494444	51.678333;
                           -8.498056	51.679722;
                           -8.498056	51.684444;
                           -8.496389	51.687500;
                           -8.495556	51.691944;
                           -8.497500	51.694722;
                           -8.500556	51.698056;
                           -8.504167	51.700278;
                           -8.507500	51.702778;
                           -8.511944	51.704722;
                           -8.517222	51.706111;
                           -8.523611	51.706667;
                           -8.528333	51.704722;
                           -8.530556	51.701667;
                           -8.526944	51.699444;
                           -8.521111	51.698333;
                           -8.516944	51.696389;
                           -8.513889	51.693611;
                           -8.511111	51.690833;
                           -8.511111	51.687222;
                           -8.515556	51.685556;
                           -8.521389	51.684722;
                           -8.526111	51.682778;
                           -8.529722	51.680278;
                           -8.531389	51.676389;
                           -8.533611	51.673333;
                           -8.535278	51.669722;
                           -8.537500	51.666389;
                           -8.540000	51.663889;
                           -8.540833	51.659167;
                           -8.543333	51.656111;
                           -8.545278	51.653333;
                           -8.547222	51.649167;
                           -8.545556	51.645000;
                           -8.544444	51.641944;
                           -8.542222	51.638611;
                           -8.541111	51.634444;
                           -8.538056	51.632222;
                           -8.536667	51.628056;
                           -8.538333	51.625000;
                           -8.536944	51.620833;
                           -8.535000	51.618056;
                           -8.533333	51.614167;
                           -8.531944	51.610556;
                           -8.535833	51.608333;
                           -8.541667	51.608889;
                           -8.545278	51.611111;
                           -8.548333	51.613889;
                           -8.551111	51.616667;
                           -8.552500	51.620000;
                           -8.556111	51.622222;
                           -8.560278	51.624167;
                           -8.563333	51.626944;
                           -8.563333	51.631667;
                           -8.564444	51.635278;
                           -8.568056	51.637500;
                           -8.573889	51.638611;
                           -8.578333	51.639444;
                           -8.585000	51.640000;
                           -8.592222	51.640278;
                           -8.599722	51.640833;
                           -8.607222	51.640833;
                           -8.613889	51.640556;
                           -8.619722	51.639722;
                           -8.626111	51.639167;
                           -8.633889	51.639167;
                           -8.641111	51.639444;
                           -8.647778	51.640278;
                           -8.652222	51.638611;
                           -8.658056	51.637778;
                           -8.664722	51.637222;
                           -8.671389	51.637778;
                           -8.676389	51.636667;
                           -8.679722	51.633889;
                           -8.682500	51.631111;
                           -8.684722	51.628056;
                           -8.686389	51.624444;
                           -8.686389	51.619722;
                           -8.683889	51.616944;
                           -8.678611	51.615556;
                           -8.678056	51.611944;
                           -8.680556	51.608611;
                           -8.684167	51.606389;
                           -8.686944	51.604167;
                           -8.690278	51.601389;
                           -8.690833	51.597500;
                           -8.690556	51.593333;
                           -8.688333	51.589444;
                           -8.686111	51.586944;
                           -8.686389	51.582778;
                           -8.686389	51.578611;
                           -8.689444	51.576389;
                           -8.692500	51.573611;
                           -8.697500	51.572222;
                           -8.704722	51.572500;
                           -8.711389	51.572778;
                           -8.716667	51.574444;
                           -8.721944	51.575833;
                           -8.726944	51.577222;
                           -8.733333	51.577778;
                           -8.740278	51.577222;
                           -8.744722	51.576111;
                           -8.746944	51.572778;
                           -8.749167	51.569722;
                           -8.754444	51.568333;
                           -8.758056	51.570556;
                           -8.756944	51.574722;
                           -8.755556	51.578333;
                           -8.753889	51.582500;
                           -8.756111	51.585833;
                           -8.759167	51.588611;
                           -8.764167	51.590000;
                           -8.770000	51.590000;
                           -8.773056	51.587222;
                           -8.776944	51.585000;
                           -8.781389	51.583333;
                           -8.788611	51.583333;
                           -8.793056	51.585556;
                           -8.798056	51.587222;
                           -8.804722	51.587500;
                           -8.809722	51.589444;
                           -8.814167	51.591389;
                           -8.820000	51.592500;
                           -8.825833	51.592500;
                           -8.828889	51.591944;
                           -8.833333	51.590556;
                           -8.839444	51.589722;
                           -8.845833	51.590833;
                           -8.851944	51.591667;
                           -8.857778	51.592778;
                           -8.863611	51.591944;
                           -8.863889	51.587222;
                           -8.863889	51.582778;
                           -8.863889	51.578056;
                           -8.867778	51.575556;
                           -8.871944	51.573611;
                           -8.877222	51.572222;
                           -8.882500	51.571111;
                           -8.886111	51.569167;
                           -8.886944	51.564722;
                           -8.887222	51.561389;
                           -8.885833	51.557778;
                           -8.886389	51.553056;
                           -8.890278	51.551389;
                           -8.894167	51.549167;
                           -8.900000	51.548333;
                           -8.906389	51.547778;
                           -8.913333	51.547500;
                           -8.920000	51.548056;
                           -8.927222	51.548056;
                           -8.931667	51.546389;
                           -8.935278	51.544444;
                           -8.937778	51.540278;
                           -8.936944	51.536667;
                           -8.935556	51.533056;
                           -8.937222	51.529444;
                           -8.942500	51.528889;
                           -8.946667	51.530833;
                           -8.948889	51.534167;
                           -8.950833	51.537222;
                           -8.953056	51.540556;
                           -8.953889	51.544167;
                           -8.955000	51.547778;
                           -8.955833	51.551944;
                           -8.958611	51.554722;
                           -8.963056	51.556667;
                           -8.968333	51.558056;
                           -8.974167	51.559167;
                           -8.978611	51.560833;
                           -8.982778	51.562778;
                           -8.985556	51.565556;
                           -8.989167	51.568333;
                           -8.993611	51.570278;
                           -8.998889	51.571667;
                           -9.005278	51.572222;
                           -9.010556	51.570833;
                           -9.014444	51.568611;
                           -9.016667	51.565278;
                           -9.021111	51.563611;
                           -9.026111	51.562222;
                           -9.032222	51.563333;
                           -9.035556	51.566111;
                           -9.040000	51.566944;
                           -9.042500	51.563611;
                           -9.043889	51.560000;
                           -9.049167	51.560000;
                           -9.052778	51.562500;
                           -9.057222	51.561667;
                           -9.061111	51.559444;
                           -9.064722	51.556944;
                           -9.070556	51.556111;
                           -9.077222	51.556667;
                           -9.082222	51.558056;
                           -9.086667	51.560000;
                           -9.090278	51.561389;
                           -9.093889	51.564167;
                           -9.098333	51.566111;
                           -9.102778	51.567778;
                           -9.106389	51.569722;
                           -9.110556	51.571667;
                           -9.115833	51.573056;
                           -9.121944	51.573889;
                           -9.127778	51.573889;
                           -9.131944	51.572222;
                           -9.134167	51.569167;
                           -9.134444	51.565000;
                           -9.133889	51.560833;
                           -9.130833	51.558056;
                           -9.126389	51.556111;
                           -9.121944	51.554444;
                           -9.117778	51.552500;
                           -9.113333	51.550556;
                           -9.111111	51.547500;
                           -9.109167	51.544167;
                           -9.109167	51.540556;
                           -9.111944	51.537778;
                           -9.117500	51.535833;
                           -9.121944	51.534167;
                           -9.125556	51.531944;
                           -9.128333	51.529167;
                           -9.131944	51.527500;
                           -9.136111	51.524722;
                           -9.140278	51.523333;
                           -9.145556	51.521944;
                           -9.151389	51.521111;
                           -9.156389	51.519722;
                           -9.161944	51.518333;
                           -9.168333	51.518889;
                           -9.172222	51.521389;
                           -9.173333	51.525000;
                           -9.175556	51.528056;
                           -9.182222	51.528611;
                           -9.185000	51.526389;
                           -9.188333	51.523611;
                           -9.191111	51.520833;
                           -9.194167	51.518333;
                           -9.192500	51.514444;
                           -9.190556	51.511389;
                           -9.188611	51.508056;
                           -9.186944	51.503889;
                           -9.190556	51.501667;
                           -9.198333	51.501667;
                           -9.203889	51.502222;
                           -9.210556	51.501389;
                           -9.212778	51.498333;
                           -9.214444	51.494444;
                           -9.215278	51.490556;
                           -9.217500	51.487222;
                           -9.221111	51.484444;
                           -9.224722	51.482222;
                           -9.229444	51.480833;
                           -9.236111	51.480000;
                           -9.239444	51.482778;
                           -9.235833	51.485000;
                           -9.237222	51.488889;
                           -9.241667	51.490556;
                           -9.248889	51.490556;
                           -9.254167	51.489444;
                           -9.258611	51.487500;
                           -9.263889	51.486111;
                           -9.265833	51.489444;
                           -9.264444	51.493056;
                           -9.264444	51.497222;
                           -9.265833	51.500278;
                           -9.272500	51.500833;
                           -9.276944	51.499167;
                           -9.281944	51.497222;
                           -9.287222	51.497222;
                           -9.290000	51.499722;
                           -9.294444	51.501389;
                           -9.299444	51.501944;
                           -9.301667	51.498889;
                           -9.299444	51.496111;
                           -9.297500	51.492778;
                           -9.296111	51.489167;
                           -9.300556	51.487222;
                           -9.304167	51.485000;
                           -9.310000	51.484167;
                           -9.315278	51.482778;
                           -9.320556	51.481389;
                           -9.325556	51.480278;
                           -9.330833	51.479167;
                           -9.335278	51.476944;
                           -9.338889	51.475278;
                           -9.341667	51.472500;
                           -9.346389	51.470278;
                           -9.352222	51.469444;
                           -9.358056	51.470278;
                           -9.364722	51.469722;
                           -9.370556	51.468889;
                           -9.377222	51.469444;
                           -9.380833	51.471667;
                           -9.383889	51.475000;
                           -9.382222	51.478611;
                           -9.380000	51.481944;
                           -9.379167	51.485556;
                           -9.383611	51.487778;
                           -9.388056	51.489722;
                           -9.393056	51.491111;
                           -9.398889	51.491944;
                           -9.405556	51.492500;
                           -9.410000	51.493889;
                           -9.412222	51.497222;
                           -9.410000	51.500278;
                           -9.409167	51.504444;
                           -9.410278	51.508056;
                           -9.416389	51.508056;
                           -9.421389	51.508611;
                           -9.422222	51.512778;
                           -9.419167	51.515556;
                           -9.416389	51.518056;
                           -9.413333	51.520000;
                           -9.410278	51.522778;
                           -9.406389	51.525556;
                           -9.403611	51.528056;
                           -9.401389	51.531389;
                           -9.401389	51.535833;
                           -9.400000	51.539722;
                           -9.398889	51.543611;
                           -9.402500	51.546111;
                           -9.406944	51.547778;
                           -9.412778	51.548889;
                           -9.418056	51.550278;
                           -9.423056	51.551667;
                           -9.428333	51.553056;
                           -9.432500	51.555000;
                           -9.436944	51.556389;
                           -9.441389	51.558056;
                           -9.446667	51.560000;
                           -9.451667	51.559444;
                           -9.453333	51.555833;
                           -9.451944	51.552778;
                           -9.448889	51.550000;
                           -9.445278	51.547222;
                           -9.442500	51.544722;
                           -9.440833	51.540833;
                           -9.443056	51.538056;
                           -9.446389	51.535278;
                           -9.450000	51.533056;
                           -9.454444	51.531111;
                           -9.460833	51.530833;
                           -9.468333	51.530833;
                           -9.475556	51.530833;
                           -9.480278	51.528889;
                           -9.483889	51.526667;
                           -9.488889	51.525833;
                           -9.493333	51.524167;
                           -9.497778	51.522222;
                           -9.503056	51.520278;
                           -9.507500	51.518611;
                           -9.512778	51.517222;
                           -9.517222	51.515556;
                           -9.523611	51.515000;
                           -9.528889	51.516389;
                           -9.529444	51.520000;
                           -9.531667	51.523333;
                           -9.534722	51.526111;
                           -9.539167	51.527778;
                           -9.545000	51.527500;
                           -9.547778	51.524722;
                           -9.549444	51.521111;
                           -9.550000	51.516389;
                           -9.550278	51.511944;
                           -9.551111	51.507778;
                           -9.555556	51.506111;
                           -9.560556	51.504722;
                           -9.567222	51.504167;
                           -9.573611	51.504722;
                           -9.580278	51.505278;
                           -9.586944	51.505278;
                           -9.591667	51.502778;
                           -9.593889	51.500278;
                           -9.593889	51.495000;
                           -9.595833	51.492500;
                           -9.599722	51.490000;
                           -9.606111	51.489722;
                           -9.613611	51.489722;
                           -9.620000	51.489167;
                           -9.626944	51.488889;
                           -9.626667	51.490556;
                           -9.622222	51.492500;
                           -9.624444	51.495833;
                           -9.629722	51.497222;
                           -9.634167	51.498889;
                           -9.636111	51.502222;
                           -9.636944	51.506389;
                           -9.638333	51.510000;
                           -9.640556	51.513611;
                           -9.642778	51.516389;
                           -9.648056	51.517778;
                           -9.653889	51.516944;
                           -9.656667	51.514167;
                           -9.658889	51.510556;
                           -9.660000	51.506389;
                           -9.661389	51.502778;
                           -9.666389	51.501389;
                           -9.673056	51.500833;
                           -9.679722	51.500556;
                           -9.684722	51.499167;
                           -9.689167	51.497222;
                           -9.693611	51.495833;
                           -9.696667	51.493056;
                           -9.698889	51.490000;
                           -9.700833	51.486667;
                           -9.702500	51.483056;
                           -9.705556	51.480278;
                           -9.710556	51.478611;
                           -9.714444	51.476389;
                           -9.712778	51.473611;
                           -9.707778	51.472222;
                           -9.703333	51.470278;
                           -9.703333	51.467500;
                           -9.706944	51.465278;
                           -9.711389	51.463333;
                           -9.716667	51.462222;
                           -9.723333	51.461667;
                           -9.730000	51.461111;
                           -9.735833	51.460278;
                           -9.740278	51.458889;
                           -9.744722	51.457222;
                           -9.749722	51.455833;
                           -9.754167	51.453889;
                           -9.757778	51.451667;
                           -9.761389	51.449444;
                           -9.766667	51.448056;
                           -9.771111	51.449722;
                           -9.767222	51.452222;
                           -9.765833	51.455833;
                           -9.766667	51.458889;
                           -9.770278	51.461389;
                           -9.776111	51.461389;
                           -9.780556	51.459444;
                           -9.784167	51.457778;
                           -9.786389	51.454444;
                           -9.788611	51.450833;
                           -9.792222	51.448889;
                           -9.797500	51.447500;
                           -9.802778	51.446389;
                           -9.810000	51.446389;
                           -9.816667	51.445833;
                           -9.815278	51.449444;
                           -9.816667	51.453056;
                           -9.818056	51.456667;
                           -9.818056	51.461389;
                           -9.815833	51.464722;
                           -9.813611	51.467778;
                           -9.812222	51.471389;
                           -9.816389	51.473333;
                           -9.822500	51.473333;
                           -9.829167	51.472778;
                           -9.833611	51.474167;
                           -9.835556	51.477500;
                           -9.833889	51.481111;
                           -9.830278	51.483333;
                           -9.825278	51.484722;
                           -9.820833	51.485556;
                           -9.815556	51.486944;
                           -9.810556	51.488333;
                           -9.805556	51.489722;
                           -9.800278	51.490556;
                           -9.796667	51.493333;
                           -9.793056	51.495556;
                           -9.788611	51.496944;
                           -9.783333	51.498333;
                           -9.778333	51.499722;
                           -9.774444	51.501944;
                           -9.772222	51.505278;
                           -9.770000	51.508889;
                           -9.767778	51.512222;
                           -9.764167	51.514444;
                           -9.758333	51.515278;
                           -9.752500	51.516111;
                           -9.747222	51.517500;
                           -9.741111	51.518056;
                           -9.737500	51.520278;
                           -9.736111	51.523889;
                           -9.734722	51.527500;
                           -9.730833	51.529722;
                           -9.729444	51.529444;
                           -9.726667	51.528611;
                           -9.722222	51.526667;
                           -9.716389	51.525833;
                           -9.710556	51.526667;
                           -9.706667	51.528056;
                           -9.702222	51.529722;
                           -9.698611	51.531944;
                           -9.694722	51.534444;
                           -9.690556	51.535833;
                           -9.685278	51.536944;
                           -9.679444	51.538056;
                           -9.675000	51.539722;
                           -9.670556	51.541667;
                           -9.668333	51.544722;
                           -9.664444	51.547500;
                           -9.658611	51.548333;
                           -9.652778	51.549444;
                           -9.649167	51.551667;
                           -9.646944	51.554444;
                           -9.645278	51.558056;
                           -9.644444	51.562500;
                           -9.643056	51.565833;
                           -9.638611	51.567500;
                           -9.632778	51.568611;
                           -9.627778	51.569722;
                           -9.623889	51.572222;
                           -9.620833	51.574722;
                           -9.619444	51.578611;
                           -9.616389	51.581667;
                           -9.610556	51.582500;
                           -9.603889	51.583056;
                           -9.599722	51.584722;
                           -9.596667	51.587500;
                           -9.593611	51.589722;
                           -9.591389	51.593056;
                           -9.589722	51.596667;
                           -9.586111	51.598889;
                           -9.582500	51.601389;
                           -9.577778	51.603056;
                           -9.571944	51.603056;
                           -9.566111	51.602222;
                           -9.560278	51.603056;
                           -9.555278	51.604444;
                           -9.551389	51.606111;
                           -9.547500	51.608889;
                           -9.546111	51.612222;
                           -9.545278	51.615833;
                           -9.549722	51.618056;
                           -9.556389	51.617778;
                           -9.560278	51.615278;
                           -9.565556	51.613889;
                           -9.572778	51.614167;
                           -9.577778	51.615556;
                           -9.584444	51.615000;
                           -9.588889	51.613056;
                           -9.591944	51.610000;
                           -9.596389	51.608056;
                           -9.600833	51.606944;
                           -9.605833	51.605556;
                           -9.611111	51.604167;
                           -9.614722	51.601944;
                           -9.618611	51.599444;
                           -9.623056	51.597778;
                           -9.629722	51.597222;
                           -9.636111	51.597778;
                           -9.640000	51.595556;
                           -9.642778	51.592778;
                           -9.645278	51.589444;
                           -9.650278	51.588611;
                           -9.657778	51.588611;
                           -9.663611	51.587778;
                           -9.667222	51.585556;
                           -9.670833	51.583333;
                           -9.675556	51.581389;
                           -9.680556	51.580000;
                           -9.688056	51.580000;
                           -9.694444	51.579722;
                           -9.697500	51.576944;
                           -9.700556	51.573611;
                           -9.705556	51.572778;
                           -9.712500	51.572500;
                           -9.718889	51.571944;
                           -9.724722	51.571111;
                           -9.727778	51.568333;
                           -9.730000	51.565000;
                           -9.733611	51.562778;
                           -9.740278	51.562500;
                           -9.744722	51.564167;
                           -9.751389	51.564167;
                           -9.755000	51.562500;
                           -9.758889	51.560000;
                           -9.761667	51.557500;
                           -9.766111	51.555556;
                           -9.771111	51.554167;
                           -9.776389	51.552778;
                           -9.780000	51.550556;
                           -9.784444	51.548611;
                           -9.789722	51.547500;
                           -9.795000	51.546111;
                           -9.799444	51.544722;
                           -9.804444	51.543333;
                           -9.810278	51.542500;
                           -9.816944	51.541944;
                           -9.822778	51.541111;
                           -9.828889	51.540000;
                           -9.835556	51.540556;
                           -9.840556	51.541944;
                           -9.844167	51.544167;
                           -9.844167	51.547778;
                           -9.840556	51.550278;
                           -9.836667	51.551667;
                           -9.830833	51.552500;
                           -9.825000	51.553333;
                           -9.819167	51.554167;
                           -9.814722	51.556111;
                           -9.811111	51.558333;
                           -9.805833	51.559167;
                           -9.801389	51.561667;
                           -9.797222	51.563056;
                           -9.791667	51.564167;
                           -9.788056	51.566667;
                           -9.785833	51.569722;
                           -9.784444	51.573889;
                           -9.783056	51.577500;
                           -9.780833	51.580833;
                           -9.778611	51.583889;
                           -9.774167	51.585833;
                           -9.767222	51.586111;
                           -9.762222	51.587500;
                           -9.759167	51.590278;
                           -9.756389	51.593056;
                           -9.753333	51.595278;
                           -9.748889	51.597222;
                           -9.745278	51.599444;
                           -9.740278	51.600833;
                           -9.735000	51.602222;
                           -9.728889	51.603056;
                           -9.723056	51.603889;
                           -9.717222	51.605000;
                           -9.712222	51.606111;
                           -9.707500	51.608611;
                           -9.703889	51.610278;
                           -9.698611	51.611667;
                           -9.692222	51.611111;
                           -9.688611	51.613611;
                           -9.685556	51.616111;
                           -9.683333	51.619444;
                           -9.678611	51.621111;
                           -9.673611	51.622500;
                           -9.668333	51.623889;
                           -9.664722	51.625278;
                           -9.659722	51.626667;
                           -9.655000	51.628056;
                           -9.650000	51.629444;
                           -9.644167	51.630278;
                           -9.638889	51.631667;
                           -9.633056	51.632500;
                           -9.628056	51.633889;
                           -9.621944	51.634722;
                           -9.616667	51.636111;
                           -9.611667	51.637500;
                           -9.607222	51.639444;
                           -9.603611	51.641111;
                           -9.599722	51.643889;
                           -9.597500	51.646944;
                           -9.593889	51.649444;
                           -9.589444	51.651111;
                           -9.584167	51.652500;
                           -9.579167	51.653889;
                           -9.573056	51.654722;
                           -9.567222	51.655556;
                           -9.561389	51.656667;
                           -9.555556	51.656944;
                           -9.550278	51.658333;
                           -9.543333	51.658889;
                           -9.537500	51.659722;
                           -9.531111	51.660000;
                           -9.525278	51.661111;
                           -9.519722	51.662222;
                           -9.514722	51.663611;
                           -9.510278	51.665556;
                           -9.506667	51.667778;
                           -9.502222	51.669167;
                           -9.496944	51.670556;
                           -9.491667	51.671944;
                           -9.485833	51.672778;
                           -9.480833	51.674167;
                           -9.475278	51.675556;
                           -9.470278	51.676667;
                           -9.466667	51.679167;
                           -9.461389	51.681389;
                           -9.457778	51.683611;
                           -9.453889	51.685833;
                           -9.450833	51.688056;
                           -9.449444	51.690833;
                           -9.451667	51.694167;
                           -9.453889	51.697222;
                           -9.452222	51.700833;
                           -9.449167	51.703611;
                           -9.446389	51.706389;
                           -9.443333	51.709167;
                           -9.441944	51.712778;
                           -9.445556	51.714722;
                           -9.448611	51.717500;
                           -9.448889	51.721389;
                           -9.451111	51.724722;
                           -9.456389	51.726667;
                           -9.461667	51.725278;
                           -9.465278	51.723056;
                           -9.467500	51.719722;
                           -9.468889	51.716111;
                           -9.472222	51.713333;
                           -9.477222	51.711667;
                           -9.484722	51.711667;
                           -9.487500	51.713889;
                           -9.493333	51.713889;
                           -9.500278	51.713611;
                           -9.503889	51.715000;
                           -9.505278	51.718611;
                           -9.509167	51.721389;
                           -9.516389	51.721389;
                           -9.521389	51.722778;
                           -9.523611	51.726111;
                           -9.524444	51.730000;
                           -9.525278	51.734167;
                           -9.526389	51.737778;
                           -9.527778	51.741667;
                           -9.528611	51.745278;
                           -9.531389	51.748056;
                           -9.535278	51.750278;
                           -9.539444	51.752222;
                           -9.543889	51.753889;
                           -9.549444	51.755278;
                           -9.555833	51.755000;
                           -9.559444	51.752778;
                           -9.557500	51.749444;
                           -9.555278	51.746111;
                           -9.554444	51.742222;
                           -9.555278	51.738333;
                           -9.558333	51.735833;
                           -9.559167	51.731944;
                           -9.556944	51.728333;
                           -9.553333	51.726111;
                           -9.550278	51.723333;
                           -9.548056	51.720000;
                           -9.551111	51.717500;
                           -9.557500	51.717500;
                           -9.562778	51.716111;
                           -9.565278	51.712778;
                           -9.568056	51.710000;
                           -9.573333	51.708333;
                           -9.580000	51.707778;
                           -9.585833	51.706944;
                           -9.589722	51.704722;
                           -9.592500	51.701944;
                           -9.594722	51.698333;
                           -9.596111	51.694722;
                           -9.598611	51.691389;
                           -9.601667	51.688333;
                           -9.606667	51.686944;
                           -9.613889	51.687500;
                           -9.620000	51.686389;
                           -9.625278	51.685278;
                           -9.628056	51.684167;
                           -9.634167	51.683333;
                           -9.638333	51.681667;
                           -9.643056	51.679722;
                           -9.648056	51.678333;
                           -9.653333	51.676944;
                           -9.657778	51.675556;
                           -9.662778	51.674167;
                           -9.668611	51.673333;
                           -9.676389	51.673333;
                           -9.683611	51.673333;
                           -9.690833	51.673333;
                           -9.696667	51.674444;
                           -9.703611	51.675000;
                           -9.710278	51.675278;
                           -9.716111	51.675000;
                           -9.721111	51.673611;
                           -9.724167	51.670833;
                           -9.726944	51.667500;
                           -9.732500	51.666111;
                           -9.739722	51.666389;
                           -9.746944	51.666389;
                           -9.751389	51.664444;
                           -9.756944	51.663056;
                           -9.761944	51.661667;
                           -9.767222	51.660278;
                           -9.772222	51.659444;
                           -9.777222	51.658056;
                           -9.783889	51.657500;
                           -9.790833	51.658056;
                           -9.796667	51.657222;
                           -9.801111	51.655278;
                           -9.806111	51.653889;
                           -9.813611	51.654167;
                           -9.821111	51.654444;
                           -9.826944	51.655556;
                           -9.833611	51.655833;
                           -9.840000	51.656389;
                           -9.846667	51.655833;
                           -9.851944	51.654444;
                           -9.856389	51.652778;
                           -9.860833	51.650833;
                           -9.865833	51.650000;
                           -9.871944	51.649167;
                           -9.878333	51.648611;
                           -9.885833	51.648611;
                           -9.892500	51.648056;
                           -9.898333	51.647222;
                           -9.904167	51.646389;
                           -9.910278	51.645556;
                           -9.915278	51.644167;
                           -9.919167	51.642222;
                           -9.923611	51.640556;
                           -9.928611	51.639167;
                           -9.932500	51.636667;
                           -9.930833	51.633056;
                           -9.930278	51.629444;
                           -9.930278	51.625833;
                           -9.932500	51.622500;
                           -9.935278	51.619722;
                           -9.938889	51.617500;
                           -9.942500	51.615278;
                           -9.946389	51.613889;
                           -9.952500	51.613056;
                           -9.958333	51.611944;
                           -9.963333	51.613889;
                           -9.967778	51.615833;
                           -9.972778	51.617222;
                           -9.978056	51.615833;
                           -9.982500	51.613889;
                           -9.987500	51.612500;
                           -9.990556	51.609722;
                           -9.993611	51.606944;
                           -9.998056	51.605278;
                           -10.00250	51.6038890;
                           -10.00833	51.6030563;
                           -10.01333	51.6019443;
                           -10.01777	51.6002788;
                           -10.02138	51.5977789;
                           -10.02527	51.5955568;
                           -10.03027	51.5941678;
                           -10.03638	51.5933339;
                           -10.04305	51.5927786;
                           -10.04888	51.5938899;
                           -10.05250	51.5961110;
                           -10.05694	51.5977784;
                           -10.06138	51.6002789;
                           -10.06638	51.6016679;
                           -10.07305	51.6025006;
                           -10.08055	51.6025006;
                           -10.08722	51.6019442;
                           -10.09305	51.6011116;
                           -10.09750	51.5991670;
                           -10.10194	51.5975004;
                           -10.10777	51.5963898;
                           -10.11361	51.5961111;
                           -10.12027	51.5955568;
                           -10.12611	51.5947221;
                           -10.13194	51.5936114;
                           -10.13638	51.5919449;
                           -10.14000	51.5894440;
                           -10.14444	51.5872224;
                           -10.14805	51.5850006;
                           -10.15194	51.5827784;
                           -10.15694	51.5813894;
                           -10.16000	51.5841670;
                           -10.15916	51.5877787;
                           -10.15638	51.5894449;
                           -10.15250	51.5919440;
                           -10.15027	51.5950008;
                           -10.14805	51.5983336;
                           -10.14527	51.6011118;
                           -10.14611	51.6041671;
                           -10.14805	51.6075006;
                           -10.14916	51.6113897;
                           -10.14916	51.6152787;
                           -10.14444	51.6169444;
                           -10.13861	51.6161111;
                           -10.13500	51.6138890;
                           -10.13000	51.6125000;
                           -10.12388	51.6125009;
                           -10.11805	51.6133336;
                           -10.11388	51.6152789;
                           -10.11000	51.6169440;
                           -10.10555	51.6188896;
                           -10.10111	51.6208331;
                           -10.09444	51.6211114;
                           -10.08777	51.6208338;
                           -10.08138	51.6211119;
                           -10.07555	51.6222226;
                           -10.07027	51.6236118;
                           -10.06527	51.6250008;
                           -10.06194	51.6275004;
                           -10.06194	51.6322224;
                           -10.06277	51.6363898;
                           -10.06138	51.6400009;
                           -10.05972	51.6436112;
                           -10.06138	51.6472229;
                           -10.06527	51.6497228;
                           -10.06944	51.6513894;
                           -10.07555	51.6527786;
                           -10.08194	51.6533334;
                           -10.08777	51.6541678;
                           -10.09305	51.6550006;
                           -10.09777	51.6569448;
                           -10.10138	51.6591679;
                           -10.10277	51.6627788;
                           -10.09916	51.6650007;
                           -10.09472	51.6669442;
                           -10.09083	51.6691673;
                           -10.08500	51.6702780;
                           -10.07916	51.6711117;
                           -10.07166	51.6711117;
                           -10.06444	51.6705564;
                           -10.05694	51.6705564;
                           -10.05111	51.6716671;
                           -10.04527	51.6725008;
                           -10.03861	51.6730561;
                           -10.03194	51.6725004;
                           -10.02527	51.6719448;
                           -10.01861	51.6711111;
                           -10.01333	51.6725003;
                           -10.00972	51.6747222;
                           -10.00611	51.6769441;
                           -10.00111	51.6783331;
                           -9.995000	51.678056;
                           -9.987500	51.678056;
                           -9.982500	51.679444;
                           -9.978889	51.681667;
                           -9.975833	51.684444;
                           -9.973611	51.687500;
                           -9.971389	51.690278;
                           -9.968611	51.693611;
                           -9.965556	51.696389;
                           -9.962500	51.699167;
                           -9.960278	51.702222;
                           -9.957222	51.705000;
                           -9.957222	51.709444;
                           -9.962500	51.710833;
                           -9.969167	51.710556;
                           -9.974444	51.709444;
                           -9.980278	51.708611;
                           -9.985278	51.710000;
                           -9.991111	51.710833;
                           -9.995833	51.712778;
                           -9.998056	51.716111;
                           -9.996667	51.719722;
                           -9.991111	51.721111;
                           -9.986111	51.722500;
                           -9.980833	51.723611;
                           -9.978889	51.727500;
                           -9.975833	51.729722;
                           -9.971389	51.731389;
                           -9.966944	51.731389;
                           -9.962500	51.733333;
                           -9.958611	51.735556;
                           -9.954444	51.737500;
                           -9.949167	51.736111;
                           -9.944167	51.736667;
                           -9.942500	51.740278;
                           -9.939444	51.743056;
                           -9.935000	51.744722;
                           -9.930556	51.746667;
                           -9.926389	51.748333;
                           -9.922500	51.750278;
                           -9.918333	51.752222;
                           -9.913611	51.753889;
                           -9.910000	51.756111;
                           -9.905556	51.758056;
                           -9.899722	51.758889;
                           -9.893889	51.758889;
                           -9.890833	51.756111;
                           -9.890833	51.752500;
                           -9.893056	51.749444;
                           -9.895278	51.746111;
                           -9.897500	51.743056;
                           -9.898056	51.738889;
                           -9.894444	51.736389;
                           -9.889444	51.735000;
                           -9.885833	51.737500;
                           -9.884722	51.739722;
                           -9.882500	51.742778;
                           -9.878056	51.744722;
                           -9.873056	51.745556;
                           -9.867222	51.746667;
                           -9.861944	51.748056;
                           -9.858056	51.750278;
                           -9.855833	51.753333;
                           -9.855278	51.757500;
                           -9.853889	51.761667;
                           -9.853056	51.765833;
                           -9.850833	51.768889;
                           -9.847222	51.770278;
                           -9.843611	51.768056;
                           -9.841389	51.764722;
                           -9.838056	51.761944;
                           -9.833611	51.761667;
                           -9.829444	51.763889;
                           -9.824167	51.765278;
                           -9.818333	51.766111;
                           -9.811389	51.766111;
                           -9.805000	51.766667;
                           -9.797500	51.766667;
                           -9.790833	51.766111;
                           -9.785000	51.766111;
                           -9.783333	51.769722;
                           -9.786389	51.772500;
                           -9.792222	51.773333;
                           -9.799167	51.773889;
                           -9.804167	51.775278;
                           -9.808611	51.777222;
                           -9.811389	51.780000;
                           -9.812222	51.783056;
                           -9.808611	51.784444;
                           -9.803333	51.785833;
                           -9.799444	51.788056;
                           -9.796667	51.790833;
                           -9.793611	51.793611;
                           -9.789444	51.795278;
                           -9.784167	51.797222;
                           -9.779444	51.799167;
                           -9.775833	51.801389;
                           -9.773611	51.804444;
                           -9.772222	51.807778;
                           -9.768611	51.810000;
                           -9.764167	51.811944;
                           -9.759444	51.813611;
                           -9.755833	51.815833;
                           -9.753056	51.818611;
                           -9.749167	51.820833;
                           -9.746111	51.823611;
                           -9.743889	51.826944;
                           -9.741667	51.830556;
                           -9.737222	51.832500;
                           -9.733611	51.833611;
                           -9.728333	51.835000;
                           -9.722500	51.836111;
                           -9.715000	51.835556;
                           -9.708333	51.835833;
                           -9.701667	51.836389;
                           -9.695833	51.837222;
                           -9.688889	51.837778;
                           -9.683056	51.838611;
                           -9.677222	51.839167;
                           -9.672222	51.840556;
                           -9.666667	51.841667;
                           -9.660833	51.842778;
                           -9.655000	51.843611;
                           -9.650278	51.845278;
                           -9.646111	51.847222;
                           -9.640833	51.848611;
                           -9.636389	51.850556;
                           -9.631944	51.852222;
                           -9.627500	51.854167;
                           -9.622222	51.855000;
                           -9.618056	51.856667;
                           -9.613333	51.858611;
                           -9.608889	51.859444;
                           -9.603056	51.862222;
                           -9.597778	51.863611;
                           -9.592500	51.865000;
                           -9.588056	51.866667;
                           -9.582778	51.868611;
                           -9.578333	51.870278;
                           -9.576944	51.872500;
                           -9.582778	51.873889;
                           -9.589722	51.873611;
                           -9.595556	51.872778;
                           -9.600556	51.871389;
                           -9.606667	51.870556;
                           -9.611944	51.869167;
                           -9.616944	51.867778;
                           -9.622778	51.866944;
                           -9.629722	51.867778;
                           -9.635556	51.868611;
                           -9.641389	51.869722;
                           -9.647500	51.868611;
                           -9.652500	51.867500;
                           -9.658333	51.866111;
                           -9.664444	51.865278;
                           -9.670278	51.864167;
                           -9.676944	51.863333;
                           -9.683056	51.862500;
                           -9.688333	51.861667;
                           -9.691944	51.859167;
                           -9.696389	51.857500;
                           -9.703056	51.856944;
                           -9.709722	51.856667;
                           -9.714722	51.855278;
                           -9.719167	51.853333;
                           -9.723889	51.851667;
                           -9.727500	51.849722;
                           -9.733333	51.848889;
                           -9.738611	51.847500;
                           -9.743889	51.846111;
                           -9.747500	51.843889;
                           -9.751944	51.841944;
                           -9.758889	51.841667;
                           -9.763889	51.840278;
                           -9.767500	51.838889;
                           -9.771111	51.836667;
                           -9.775556	51.834722;
                           -9.779444	51.832778;
                           -9.783333	51.830556;
                           -9.786944	51.828333;
                           -9.790556	51.825556;
                           -9.793611	51.822778;
                           -9.798056	51.821111;
                           -9.802500	51.821111;
                           -9.803333	51.825278;
                           -9.805556	51.828056;
                           -9.811944	51.827500;
                           -9.818056	51.826667;
                           -9.823889	51.825278;
                           -9.829167	51.824444;
                           -9.833611	51.822500;
                           -9.838889	51.821111;
                           -9.843333	51.819167;
                           -9.841111	51.816944;
                           -9.835278	51.816111;
                           -9.831944	51.813333;
                           -9.835833	51.811111;
                           -9.841111	51.809722;
                           -9.846944	51.808889;
                           -9.851944	51.807500;
                           -9.855833	51.805000;
                           -9.858889	51.802500;
                           -9.863333	51.800556;
                           -9.866944	51.799167;
                           -9.871389	51.801111;
                           -9.871944	51.805278;
                           -9.871389	51.808333;
                           -9.870556	51.812500;
                           -9.870556	51.816111;
                           -9.875833	51.817500;
                           -9.881944	51.818333;
                           -9.888333	51.818333;
                           -9.893611	51.816944;
                           -9.897222	51.814722;
                           -9.898611	51.811111;
                           -9.901944	51.808333;
                           -9.905556	51.806111;
                           -9.910556	51.804722;
                           -9.915833	51.803333;
                           -9.923333	51.803333;
                           -9.930000	51.802778;
                           -9.934444	51.801667;
                           -9.938611	51.799722;
                           -9.945278	51.799167;
                           -9.951389	51.800278;
                           -9.956667	51.798889;
                           -9.959444	51.796944;
                           -9.962500	51.794167;
                           -9.964722	51.791111;
                           -9.968333	51.788333;
                           -9.972778	51.786389;
                           -9.977222	51.784722;
                           -9.983056	51.784167;
                           -9.988333	51.782778;
                           -9.992222	51.780556;
                           -9.995833	51.778333;
                           -10.00027	51.7763898;
                           -10.00388	51.7744449;
                           -10.00916	51.7730567;
                           -10.01638	51.7730569;
                           -10.02305	51.7736116;
                           -10.02916	51.7750007;
                           -10.03555	51.7755566;
                           -10.04083	51.7741673;
                           -10.04611	51.7727781;
                           -10.05138	51.7713899;
                           -10.05555	51.7694446;
                           -10.05777	51.7663898;
                           -10.06000	51.7630560;
                           -10.06361	51.7608331;
                           -10.06944	51.7600004;
                           -10.07500	51.7586110;
                           -10.07944	51.7566674;
                           -10.08305	51.7544446;
                           -10.08888	51.7538899;
                           -10.09555	51.7533336;
                           -10.10166	51.7525007;
                           -10.10583	51.7502783;
                           -10.11027	51.7483338;
                           -10.11472	51.7466672;
                           -10.11777	51.7438898;
                           -10.12000	51.7405560;
                           -10.12277	51.7377788;
                           -10.12722	51.7361112;
                           -10.13222	51.7369442;
                           -10.13083	51.7402783;
                           -10.12861	51.7433331;
                           -10.12722	51.7469442;
                           -10.12416	51.7502787;
                           -10.12277	51.7538898;
                           -10.12750	51.7558330;
                           -10.13472	51.7561112;
                           -10.13916	51.7580567;
                           -10.14416	51.7594447;
                           -10.15027	51.7583338;
                           -10.15638	51.7575009;
                           -10.16222	51.7583332;
                           -10.16500	51.7611110;
                           -10.16805	51.7638896;
                           -10.17111	51.7666671;
                           -10.17638	51.7680569;
                           -10.18138	51.7694449;
                           -10.18583	51.7711113;
                           -10.18944	51.7738894;
                           -10.19555	51.7747226;
                           -10.20222	51.7747222;
                           -10.20805	51.7738896;
                           -10.21388	51.7730569;
                           -10.22000	51.7730560;
                           -10.22361	51.7752781;
                           -10.22444	51.7788894;
                           -10.22000	51.7808330;
                           -10.21305	51.7811116;
                           -10.20861	51.7816671;
                           -10.20277	51.7808338;
                           -10.19555	51.7808336;
                           -10.18861	51.7808331;
                           -10.18500	51.7830560;
                           -10.18222	51.7858332;
                           -10.18138	51.7900009;
                           -10.18055	51.7941676;
                           -10.18000	51.7983330;
                           -10.17833	51.8019443;
                           -10.17555	51.8047226;
                           -10.17416	51.8083337;
                           -10.17250	51.8125000;
                           -10.17333	51.8166673;
                           -10.17416	51.8205567;
                           -10.17583	51.8241673;
                           -10.17861	51.8269441;
                           -10.18250	51.8294440;
                           -10.18666	51.8311117;
                           -10.19111	51.8330561;
                           -10.19277	51.8369448;
                           -10.19500	51.8397220;
                           -10.19722	51.8436112;
                           -10.19944	51.8461114;
                           -10.20472	51.8475002;
                           -10.21138	51.8480569;
                           -10.21805	51.8475006;
                           -10.22388	51.8466679;
                           -10.22972	51.8458332;
                           -10.23500	51.8444440;
                           -10.24027	51.8430568;
                           -10.24527	51.8416678;
                           -10.24888	51.8391679;
                           -10.25111	51.8361111;
                           -10.25194	51.8319444;
                           -10.25638	51.8300009;
                           -10.26166	51.8286117;
                           -10.26611	51.8269441;
                           -10.26916	51.8241677;
                           -10.27111	51.8208331;
                           -10.27194	51.8166674;
                           -10.27333	51.8130563;
                           -10.27722	51.8108332;
                           -10.28222	51.8094442;
                           -10.28805	51.8086116;
                           -10.29333	51.8072223;
                           -10.29777	51.8052788;
                           -10.29916	51.8016677;
                           -10.30305	51.7994446;
                           -10.30666	51.7969447;
                           -10.31111	51.7952781;
                           -10.31555	51.7933336;
                           -10.31972	51.7913892;
                           -10.32361	51.7891671;
                           -10.32722	51.7869442;
                           -10.33083	51.7847223;
                           -10.33583	51.7833333;
                           -10.33833	51.7827783;
                           -10.33777    51.7830568;
                           -10.34111	51.7850001;
                           -10.34472	51.7872222;
                           -10.34694	51.7905564;
                           -10.34805	51.7941676;
                           -10.34888	51.7983339;
                           -10.34750	51.8019440;
                           -10.34277	51.8036118;
                           -10.33916	51.8061117;
                           -10.33916	51.8097227;
                           -10.33972	51.8138892;
                           -10.33805	51.8175006;
                           -10.33527	51.8202788;
                           -10.33444	51.8241674;
                           -10.33583	51.8280563;
                           -10.33805	51.8311116;
                           -10.33972	51.8347222;
                           -10.34000	51.8388890;
                           -10.34166	51.8425007;
                           -10.34750	51.8436110;
                           -10.35388	51.8430569;
                           -10.36083	51.8427783;
                           -10.36666	51.8419447;
                           -10.37333	51.8413893;
                           -10.38000	51.8411110;
                           -10.38527	51.8425008;
                           -10.38888	51.8447229;
                           -10.39027	51.8483338;
                           -10.38805	51.8516676;
                           -10.38500	51.8541670;
                           -10.38500	51.8580560;
                           -10.38583	51.8619443;
                           -10.38416	51.8655567;
                           -10.38333	51.8697223;
                           -10.38472	51.8736112;
                           -10.38833	51.8758333;
                           -10.39277	51.8775008;
                           -10.39805	51.8772226;
                           -10.39416	51.8794447;
                           -10.38694	51.8794444;
                           -10.37972	51.8794442;
                           -10.37194	51.8794444;
                           -10.36555	51.8797226;
                           -10.35888	51.8802789;
                           -10.35277	51.8811118;
                           -10.34833	51.8827783;
                           -10.34388	51.8847229;
                           -10.33861	51.8861111;
                           -10.33361	51.8872221;
                           -10.32833	51.8886113;
                           -10.32222	51.8894442;
                           -10.31722	51.8908332;
                           -10.31277	51.8927788;
                           -10.30750	51.8941670;
                           -10.30083	51.8936113;
                           -10.29555	51.8950006;
                           -10.29111	51.8966671;
                           -10.28666	51.8986117;
                           -10.28305	51.9008336;
                           -10.28055	51.9038896;
                           -10.27750	51.9066670;
                           -10.27111	51.9072221;
                           -10.26638	51.9061119;
                           -10.26194	51.9041674;
                           -10.25750	51.9025000;
                           -10.25305	51.9005566;
                           -10.24888	51.9025009;
                           -10.24888	51.9069449;
                           -10.24861	51.9113891;
                           -10.24916	51.9155567;
                           -10.25138	51.9188899;
                           -10.25444	51.9216674;
                           -10.25805	51.9238896;
                           -10.26472	51.9244442;
                           -10.27083	51.9252783;
                           -10.26972	51.9294442;
                           -10.26916	51.9336117;
                           -10.27055	51.9372226;
                           -10.27416	51.9394447;
                           -10.27722	51.9422222;
                           -10.28083	51.9447223;
                           -10.28444	51.9469444;
                           -10.28944	51.9483334;
                           -10.29611	51.9488891;
                           -10.30194	51.9497224;
                           -10.30666	51.9516677;
                           -10.30777	51.9552788;
                           -10.30416	51.9575007;
                           -10.29972	51.9594442;
                           -10.29527	51.9611118;
                           -10.29083	51.9630563;
                           -10.28638	51.9647229;
                           -10.28027	51.9647228;
                           -10.27444	51.9638894;
                           -10.26777	51.9641678;
                           -10.26388	51.9663899;
                           -10.26166	51.9697227;
                           -10.26250	51.9738890;
                           -10.26388	51.9775009;
                           -10.26500	51.9811110;
                           -10.26583	51.9852783;
                           -10.26361	51.9883331;
                           -10.25916	51.9902787;
                           -10.25388	51.9916679;
                           -10.24944	51.9933334;
                           -10.24333	51.9941673;
                           -10.23750	51.9950000;
                           -10.23083	51.9955563;
                           -10.22555	51.9941676;
                           -10.22055	51.9927786;
                           -10.21472	51.9916672;
                           -10.20861	51.9922221;
                           -10.20555	51.9950006;
                           -10.20472	51.9980562;
                           -10.20305	52.0016676;
                           -10.20027	52.0044448;
                           -10.19583	52.0063893;
                           -10.19111	52.0080561;
                           -10.18527	52.0088898;
                           -10.17916	52.0097227;
                           -10.17472	52.0116672;
                           -10.16972	52.0130562;
                           -10.16361	52.0138891;
                           -10.15833	52.0152783;
                           -10.15388	52.0169449;
                           -10.14944	52.0186114;
                           -10.14333	52.0197223;
                           -10.13805	52.0208336;
                           -10.13444	52.0233334;
                           -10.13000	52.0250000;
                           -10.12388	52.0258339;
                           -10.11888	52.0244449;
                           -10.11194	52.0238894;
                           -10.10777	52.0258338;
                           -10.10472	52.0283332;
                           -10.10166	52.0311117;
                           -10.09638	52.0325009;
                           -10.09111	52.0338891;
                           -10.08583	52.0341673;
                           -10.07972	52.0350002;
                           -10.07388	52.0361119;
                           -10.06888	52.0372229;
                           -10.06277	52.0380568;
                           -10.05694	52.0391674;
                           -10.05000	52.0394440;
                           -10.04333	52.0397223;
                           -10.03666	52.0402787;
                           -10.02916	52.0402787;
                           -10.02250	52.0405560;
                           -10.01666	52.0413897;
                           -10.01083	52.0422223;
                           -10.00611	52.0441671;
                           -10.00111	52.0455561;
                           -9.995000	52.046389;
                           -9.990556	52.048056;
                           -9.986667	52.050278;
                           -9.984444	52.053611;
                           -9.982222	52.056667;
                           -9.979722	52.060000;
                           -9.978333	52.063611;
                           -9.976667	52.067222;
                           -9.975833	52.071389;
                           -9.974167	52.075000;
                           -9.973333	52.078889;
                           -9.971667	52.082778;
                           -9.968889	52.085278;
                           -9.965556	52.088056;
                           -9.962778	52.088889;
                           -9.961944	52.085000;
                           -9.961389	52.080833;
                           -9.960556	52.076667;
                           -9.961667	52.072500;
                           -9.963056	52.068889;
                           -9.964167	52.064722;
                           -9.961944	52.061667;
                           -9.956667	52.060000;
                           -9.952778	52.062500;
                           -9.949167	52.064722;
                           -9.946111	52.067222;
                           -9.942500	52.069722;
                           -9.936389	52.069444;
                           -9.934444	52.066389;
                           -9.933611	52.062222;
                           -9.932222	52.058611;
                           -9.926944	52.058056;
                           -9.924722	52.061111;
                           -9.923056	52.064722;
                           -9.922222	52.068889;
                           -9.923611	52.072500;
                           -9.925833	52.075833;
                           -9.927222	52.079444;
                           -9.928333	52.083056;
                           -9.928333	52.087778;
                           -9.926944	52.091389;
                           -9.923056	52.093611;
                           -9.919444	52.095833;
                           -9.915556	52.098056;
                           -9.912500	52.100833;
                           -9.910833	52.104444;
                           -9.908611	52.107778;
                           -9.907778	52.111944;
                           -9.906944	52.115833;
                           -9.906667	52.120556;
                           -9.907500	52.124722;
                           -9.906389	52.128611;
                           -9.902222	52.129722;
                           -9.899167	52.126944;
                           -9.897778	52.123056;
                           -9.896389	52.119444;
                           -9.894167	52.116389;
                           -9.891389	52.113611;
                           -9.886944	52.111667;
                           -9.881944	52.110278;
                           -9.875000	52.109722;
                           -9.869167	52.110556;
                           -9.863056	52.111389;
                           -9.857778	52.112778;
                           -9.853333	52.114444;
                           -9.848056	52.115833;
                           -9.842778	52.117222;
                           -9.838333	52.118889;
                           -9.833056	52.120278;
                           -9.827778	52.121667;
                           -9.821667	52.122500;
                           -9.815833	52.123333;
                           -9.808333	52.123056;
                           -9.801111	52.123056;
                           -9.795000	52.122222;
                           -9.789167	52.121111;
                           -9.782500	52.120556;
                           -9.775556	52.120000;
                           -9.771944	52.122222;
                           -9.774167	52.125556;
                           -9.777778	52.127778;
                           -9.783056	52.129167;
                           -9.786667	52.131667;
                           -9.791111	52.133611;
                           -9.794722	52.135833;
                           -9.799167	52.137778;
                           -9.804167	52.139167;
                           -9.810000	52.140000;
                           -9.816111	52.140278;
                           -9.821944	52.139444;
                           -9.826667	52.137500;
                           -9.830278	52.135278;
                           -9.836389	52.134444;
                           -9.841667	52.135833;
                           -9.845278	52.138056;
                           -9.848889	52.140556;
                           -9.851111	52.143611;
                           -9.853889	52.146389;
                           -9.857778	52.148889;
                           -9.862778	52.150278;
                           -9.868056	52.150833;
                           -9.875556	52.150833;
                           -9.883056	52.150833;
                           -9.889722	52.150556;
                           -9.896389	52.150000;
                           -9.903056	52.149722;
                           -9.909167	52.148889;
                           -9.914444	52.147500;
                           -9.918889	52.145833;
                           -9.924167	52.144444;
                           -9.930833	52.144167;
                           -9.936667	52.145000;
                           -9.942500	52.146111;
                           -9.950278	52.146111;
                           -9.955278	52.144722;
                           -9.960556	52.143333;
                           -9.962778	52.140278;
                           -9.960000	52.137500;
                           -9.956389	52.135000;
                           -9.951944	52.133333;
                           -9.948333	52.130833;
                           -9.947778	52.126944;
                           -9.949167	52.123056;
                           -9.947778	52.119444;
                           -9.944722	52.116667;
                           -9.941944	52.113889;
                           -9.940556	52.110278;
                           -9.944444	52.108056;
                           -9.951111	52.107500;
                           -9.955556	52.109444;
                           -9.958611	52.112222;
                           -9.960556	52.115556;
                           -9.961944	52.119167;
                           -9.963333	52.122778;
                           -9.965556	52.126111;
                           -9.968611	52.128889;
                           -9.971389	52.131667;
                           -9.975833	52.133333;
                           -9.980278	52.135278;
                           -9.985278	52.136667;
                           -9.991111	52.137778;
                           -9.998056	52.138333;
                           -10.00472	52.1377782;
                           -10.01138	52.1375009;
                           -10.01805	52.1369446;
                           -10.02500	52.1366670;
                           -10.03083	52.1358333;
                           -10.03611	52.1344441;
                           -10.04194	52.1336114;
                           -10.04805	52.1327786;
                           -10.05388	52.1319449;
                           -10.05944	52.1305564;
                           -10.06444	52.1291674;
                           -10.06972	52.1277782;
                           -10.07500	52.1266670;
                           -10.08027	52.1252788;
                           -10.08611	52.1244441;
                           -10.09305	52.1238896;
                           -10.09888	52.1230569;
                           -10.10500	52.1222220;
                           -10.11083	52.1213893;
                           -10.11694	52.1205564;
                           -10.12277	52.1197228;
                           -10.12750	52.1177780;
                           -10.13111	52.1155561;
                           -10.13611	52.1141671;
                           -10.14305	52.1138896;
                           -10.14750	52.1147220;
                           -10.15333	52.1158333;
                           -10.16027	52.1152788;
                           -10.16527	52.1138898;
                           -10.17055	52.1127786;
                           -10.17500	52.1108330;
                           -10.18055	52.1094446;
                           -10.18472	52.1077782;
                           -10.19166	52.1083337;
                           -10.19666	52.1097227;
                           -10.19972	52.1125002;
                           -10.20194	52.1155564;
                           -10.20416	52.1188897;
                           -10.20638	52.1219449;
                           -10.20916	52.1247227;
                           -10.21361	52.1266671;
                           -10.21944	52.1275004;
                           -10.22416	52.1258337;
                           -10.22694	52.1230564;
                           -10.22916	52.1200007;
                           -10.23250	52.1172220;
                           -10.23750	52.1158330;
                           -10.24361	52.1158331;
                           -10.24888	52.1172229;
                           -10.25250	52.1197220;
                           -10.25472	52.1227782;
                           -10.25750	52.1255560;
                           -10.26055	52.1283336;
                           -10.26333	52.1311113;
                           -10.26861	52.1325001;
                           -10.27611	52.1325001;
                           -10.28277	52.1322228;
                           -10.28805	52.1308336;
                           -10.29388	52.1300009;
                           -10.29972	52.1291672;
                           -10.30444	52.1272224;
                           -10.30583	52.1244443;
                           -10.30222	52.1222222;
                           -10.29638	52.1213899;
                           -10.29138	52.1197229;
                           -10.28583	52.1183333;
                           -10.28083	52.1169443;
                           -10.27500	52.1161110;
                           -10.27055	52.1141676;
                           -10.27055	52.1105566;
                           -10.27500	52.1088890;
                           -10.28111	52.1077781;
                           -10.28750	52.1075000;
                           -10.29527	52.1075008;
                           -10.30166	52.1080567;
                           -10.30833	52.1086113;
                           -10.31444	52.1094444;
                           -10.32027	52.1105568;
                           -10.32555	52.1119446;
                           -10.33000	52.1136110;
                           -10.33361	52.1161111;
                           -10.33750	52.1183330;
                           -10.34111	52.1205561;
                           -10.34694	52.1216674;
                           -10.35166	52.1233337;
                           -10.35805	52.1238896;
                           -10.36388	52.1238899;
                           -10.36861	52.1222221;
                           -10.37000	52.1186110;
                           -10.36944	52.1144444;
                           -10.36583	52.1119443;
                           -10.36055	52.1105566;
                           -10.35555	52.1091676;
                           -10.35472	52.1061112;
                           -10.35916	52.1041677;
                           -10.36666	52.1041677;
                           -10.37388	52.1044449;
                           -10.38166	52.1044447;
                           -10.38666	52.1030567;
                           -10.39277	52.1022228;
                           -10.39805	52.1008336;
                           -10.40305	52.0994446;
                           -10.40833	52.0980563;
                           -10.41444	52.0972224;
                           -10.41972	52.0958332;
                           -10.42555	52.0950006;
                           -10.43083	52.0936113;
                           -10.43666	52.0927787;
                           -10.44416	52.0927787;
                           -10.44944	52.0941674;
                           -10.45305	52.0963896;
                           -10.45611	52.0991671;
                           -10.46055	52.1011116;
                           -10.46555	52.1025006;
                           -10.47250	52.1030560;
                           -10.47388	52.1058339;
                           -10.47000	52.1080560;
                           -10.46777	52.1111118;
                           -10.46861	52.1152781;
                           -10.47000	52.1188890;
                           -10.47000	52.1227780;
                           -10.47138	52.1263899;
                           -10.47361	52.1294441;
                           -10.47277	52.1336118;
                           -10.47277	52.1383338;
                           -10.47416	52.1419447;
                           -10.47472	52.1461112;
                           -10.47250	52.1491670;
                           -10.47083	52.1519443;
                           -10.46666	52.1536117;
                           -10.46222	52.1555562;
                           -10.45833	52.1577783;
                           -10.45472	52.1600002;
                           -10.45138	52.1627789;
                           -10.44861	52.1655561;
                           -10.45138	52.1683339;
                           -10.45722	52.1691672;
                           -10.46277	52.1705568;
                           -10.46694	52.1725004;
                           -10.46694	52.1769444;
                           -10.46472	52.1802782;
                           -10.46083	52.1825003;
                           -10.45583	52.1838893;
                           -10.44972	52.1847222;
                           -10.44444	52.1861114;
                           -10.44000	52.1877780;
                           -10.43555	52.1897226;
                           -10.43083	52.1916673;
                           -10.42722	52.1938892;
                           -10.42250	52.1955560;
                           -10.41888	52.1977789;
                           -10.41388	52.1975009;
                           -10.41166	52.1941677;
                           -10.41166	52.1897227;
                           -10.41305	52.1858336;
                           -10.41305	52.1813896;
                           -10.41250	52.1772220;
                           -10.40888	52.1750009;
                           -10.40500	52.1744440;
                           -10.39916	52.1752787;
                           -10.39388	52.1750009;
                           -10.39000	52.1725000;
                           -10.38722	52.1697222;
                           -10.38194	52.1683334;
                           -10.37555	52.1688896;
                           -10.37166	52.1702787;
                           -10.36861	52.1727781;
                           -10.36861	52.1766671;
                           -10.37083	52.1797223;
                           -10.37388	52.1825009;
                           -10.37416	52.1866677;
                           -10.37416	52.1902787;
                           -10.37194	52.1936114;
                           -10.36888	52.1961119;
                           -10.36527	52.1986118;
                           -10.36083	52.2002783;
                           -10.36000	52.2044440;
                           -10.36361	52.2066671;
                           -10.36888	52.2080569;
                           -10.37194	52.2108334;
                           -10.37222	52.2150002;
                           -10.37166	52.2191677;
                           -10.36805	52.2213896;
                           -10.36472	52.2241672;
                           -10.36194	52.2269444;
                           -10.35972	52.2300002;
                           -10.35277	52.2305568;
                           -10.34527	52.2302788;
                           -10.33777	52.2302788;
                           -10.33027	52.2302788;
                           -10.32305	52.2302786;
                           -10.31694	52.2311114;
                           -10.31250	52.2330560;
                           -10.30944	52.2355564;
                           -10.30638	52.2383339;
                           -10.30388	52.2416679;
                           -10.30250	52.2452780;
                           -10.30027	52.2483338;
                           -10.29722	52.2511112;
                           -10.29333	52.2533333;
                           -10.28888	52.2552789;
                           -10.28527	52.2575008;
                           -10.28055	52.2591676;
                           -10.27750	52.2619440;
                           -10.27388	52.2641679;
                           -10.26916	52.2661117;
                           -10.26500	52.2677780;
                           -10.25944	52.2691674;
                           -10.25583	52.2705563;
                           -10.25055	52.2719446;
                           -10.24472	52.2727782;
                           -10.23777	52.2730568;
                           -10.23083	52.2736113;
                           -10.22500	52.2727780;
                           -10.21750	52.2725000;
                           -10.21472	52.2744442;
                           -10.21166	52.2772227;
                           -10.20777	52.2794448;
                           -10.20416	52.2816677;
                           -10.19722	52.2819442;
                           -10.19055	52.2825006;
                           -10.18444	52.2833334;
                           -10.17916	52.2847227;
                           -10.17416	52.2861117;
                           -10.16944	52.2877784;
                           -10.16277	52.2883338;
                           -10.15833	52.2863893;
                           -10.15694	52.2827784;
                           -10.15777	52.2786118;
                           -10.15694	52.2744444;
                           -10.15722	52.2708332;
                           -10.15861	52.2672221;
                           -10.16027	52.2636118;
                           -10.15944	52.2602784;
                           -10.15972	52.2558332;
                           -10.16111	52.2519441;
                           -10.16361	52.2488891;
                           -10.16583	52.2458333;
                           -10.16833	52.2425003;
                           -10.16972	52.2388892;
                           -10.17027	52.2347228;
                           -10.16750	52.2319440;
                           -10.16083	52.2325003;
                           -10.15555	52.2327786;
                           -10.14972	52.2336112;
                           -10.14277	52.2341678;
                           -10.13777	52.2355568;
                           -10.13388	52.2377789;
                           -10.13000	52.2400000;
                           -10.12333	52.2394443;
                           -10.11666	52.2388897;
                           -10.11000	52.2394440;
                           -10.10333	52.2397223;
                           -10.09722	52.2405562;
                           -10.09194	52.2419444;
                           -10.08666	52.2433337;
                           -10.08222	52.2441672;
                           -10.07694	52.2455564;
                           -10.07250	52.2472220;
                           -10.06861	52.2494441;
                           -10.06416	52.2513897;
                           -10.06027	52.2536118;
                           -10.05583	52.2552783;
                           -10.05277	52.2580568;
                           -10.04972	52.2608332;
                           -10.04666	52.2636117;
                           -10.04361	52.2661111;
                           -10.04138	52.2694449;
                           -10.03916	52.2725007;
                           -10.03666	52.2758337;
                           -10.03527	52.2794448;
                           -10.03333	52.2830563;
                           -10.03333	52.2877783;
                           -10.03500	52.2913890;
                           -10.03777	52.2941678;
                           -10.04138	52.2963899;
                           -10.04666	52.2977787;
                           -10.05111	52.2997221;
                           -10.05555	52.3016676;
                           -10.05694	52.3052784;
                           -10.05472	52.3075002;
                           -10.04861	52.3083331;
                           -10.04333	52.3077783;
                           -10.03833	52.3063893;
                           -10.03361	52.3044441;
                           -10.02861	52.3030561;
                           -10.02416	52.3041677;
                           -10.02111	52.3066671;
                           -10.01805	52.3094446;
                           -10.01194	52.3094444;
                           -10.00972	52.3061112;
                           -10.01083	52.3022223;
                           -10.01305	52.2988896;
                           -10.01472	52.2952782;
                           -10.01527	52.2911118;
                           -10.01472	52.2880562;
                           -10.01250	52.2847220;
                           -10.01027	52.2813898;
                           -10.00972	52.2775002;
                           -10.01111	52.2736111;
                           -10.01444	52.2711114;
                           -10.01666	52.2677787;
                           -10.01750	52.2636110;
                           -10.01694	52.2597224;
                           -10.01388	52.2569449;
                           -10.01027	52.2555568;
                           -10.00583	52.2536113;
                           -10.00222	52.2513892;
                           -9.998333	52.248889;
                           -9.994722	52.246667;
                           -9.989444	52.245278;
                           -9.984444	52.243889;
                           -9.978611	52.242778;
                           -9.973889	52.240833;
                           -9.970278	52.238611;
                           -9.965833	52.236667;
                           -9.961389	52.235000;
                           -9.956389	52.233333;
                           -9.951944	52.231667;
                           -9.945833	52.230556;
                           -9.939167	52.231111;
                           -9.933056	52.231944;
                           -9.926389	52.231389;
                           -9.920278	52.230278;
                           -9.913056	52.230278;
                           -9.906389	52.230833;
                           -9.898611	52.230556;
                           -9.891389	52.230556;
                           -9.883889	52.230556;
                           -9.877222	52.230833;
                           -9.871111	52.231667;
                           -9.867500	52.232222;
                           -9.862778	52.233056;
                           -9.857778	52.234444;
                           -9.853889	52.236667;
                           -9.850000	52.238889;
                           -9.846944	52.241667;
                           -9.843889	52.244167;
                           -9.840833	52.246944;
                           -9.837778	52.249722;
                           -9.833889	52.251944;
                           -9.829444	52.253611;
                           -9.825000	52.255556;
                           -9.817500	52.255556;
                           -9.815833	52.252778;
                           -9.819167	52.250000;
                           -9.822222	52.247222;
                           -9.824444	52.244167;
                           -9.821667	52.241389;
                           -9.817222	52.239444;
                           -9.810556	52.238889;
                           -9.803611	52.239167;
                           -9.798333	52.240556;
                           -9.793333	52.241944;
                           -9.787778	52.243333;
                           -9.782778	52.244444;
                           -9.777222	52.245833;
                           -9.770833	52.246389;
                           -9.767222	52.243889;
                           -9.760278	52.243333;
                           -9.753333	52.243889;
                           -9.746944	52.243333;
                           -9.741389	52.244444;
                           -9.740833	52.248611;
                           -9.740833	52.252222;
                           -9.740556	52.256944;
                           -9.741667	52.260556;
                           -9.743056	52.264167;
                           -9.747500	52.266111;
                           -9.753056	52.267500;
                           -9.758889	52.268611;
                           -9.763889	52.270000;
                           -9.770000	52.270833;
                           -9.776667	52.271389;
                           -9.784167	52.271667;
                           -9.791389	52.271667;
                           -9.799167	52.271667;
                           -9.805833	52.272222;
                           -9.811667	52.273333;
                           -9.818333	52.273611;
                           -9.824444	52.274722;
                           -9.830278	52.275833;
                           -9.837778	52.275833;
                           -9.845278	52.275833;
                           -9.850556	52.274444;
                           -9.853611	52.271944;
                           -9.857500	52.269444;
                           -9.861944	52.267778;
                           -9.868611	52.267500;
                           -9.872222	52.269722;
                           -9.876111	52.271944;
                           -9.878056	52.275278;
                           -9.880278	52.278611;
                           -9.881944	52.282222;
                           -9.884167	52.285278;
                           -9.886111	52.288611;
                           -9.887500	52.292222;
                           -9.888056	52.296389;
                           -9.881944	52.297222;
                           -9.874722	52.297222;
                           -9.868056	52.296667;
                           -9.862778	52.298056;
                           -9.858056	52.299722;
                           -9.853611	52.301667;
                           -9.850556	52.304167;
                           -9.846667	52.306389;
                           -9.843889	52.309167;
                           -9.840556	52.311944;
                           -9.838056	52.315000;
                           -9.835833	52.318333;
                           -9.834167	52.321944;
                           -9.833611	52.326111;
                           -9.833333	52.330556;
                           -9.833333	52.335278;
                           -9.833056	52.339722;
                           -9.833611	52.343889;
                           -9.834444	52.348056;
                           -9.834167	52.352500;
                           -9.835000	52.356667;
                           -9.834722	52.361389;
                           -9.835556	52.365278;
                           -9.835278	52.370000;
                           -9.836667	52.373611;
                           -9.838056	52.377222;
                           -9.839444	52.381111;
                           -9.842222	52.383889;
                           -9.846944	52.385556;
                           -9.851944	52.386944;
                           -9.857222	52.387500;
                           -9.863889	52.388056;
                           -9.871667	52.388056;
                           -9.878056	52.388611;
                           -9.885000	52.389167;
                           -9.891667	52.389722;
                           -9.897778	52.390833;
                           -9.904444	52.391389;
                           -9.909722	52.392778;
                           -9.915556	52.393611;
                           -9.921667	52.394722;
                           -9.927500	52.395556;
                           -9.932500	52.396944;
                           -9.937222	52.398889;
                           -9.941667	52.400833;
                           -9.945278	52.403056;
                           -9.946667	52.406667;
                           -9.944444	52.410000;
                           -9.941389	52.412500;
                           -9.938889	52.415833;
                           -9.936111	52.418611;
                           -9.932778	52.421111;
                           -9.928333	52.423056;
                           -9.921667	52.423333;
                           -9.914167	52.423333;
                           -9.908333	52.423333;
                           -9.902778	52.424722;
                           -9.897500	52.425833;
                           -9.890833	52.426389;
                           -9.884722	52.426389;
                           -9.878889	52.425278;
                           -9.871944	52.424722;
                           -9.865278	52.424167;
                           -9.857778	52.424167;
                           -9.851944	52.425000;
                           -9.845833	52.425833;
                           -9.839722	52.426667;
                           -9.834444	52.428056;
                           -9.829167	52.429444;
                           -9.823889	52.430833;
                           -9.818611	52.431944;
                           -9.813333	52.433333;
                           -9.807778	52.434722;
                           -9.802778	52.436111;
                           -9.798056	52.437778;
                           -9.793056	52.439167;
                           -9.787500	52.440278;
                           -9.782222	52.441667;
                           -9.775556	52.442222;
                           -9.769444	52.443056;
                           -9.762778	52.443333;
                           -9.758333	52.445000;
                           -9.753611	52.446944;
                           -9.750556	52.449722;
                           -9.747500	52.452222;
                           -9.744444	52.455000;
                           -9.741111	52.457778;
                           -9.738889	52.460833;
                           -9.736667	52.464167;
                           -9.733333	52.466667;
                           -9.729722	52.469167;
                           -9.725833	52.471389;
                           -9.721389	52.473056;
                           -9.716667	52.474722;
                           -9.711389	52.476111;
                           -9.706389	52.477500;
                           -9.700278	52.478333;
                           -9.693333	52.478611;
                           -9.687222	52.479444;
                           -9.680000	52.479444;
                           -9.673056	52.478889;
                           -9.666389	52.478333;
                           -9.660278	52.477500;
                           -9.655000	52.476111;
                           -9.650000	52.474444;
                           -9.644722	52.473056;
                           -9.640278	52.471111;
                           -9.635000	52.469722;
                           -9.632778	52.471944;
                           -9.633333	52.476111;
                           -9.635556	52.479444;
                           -9.637778	52.482500;
                           -9.640556	52.485278;
                           -9.644167	52.487778;
                           -9.648611	52.489722;
                           -9.656389	52.489722;
                           -9.663889	52.489722;
                           -9.669722	52.488889;
                           -9.676389	52.488611;
                           -9.679444	52.490556;
                           -9.676389	52.493056;
                           -9.674722	52.495833;
                           -9.673889	52.500000;
                           -9.673056	52.504167;
                           -9.675278	52.507222;
                           -9.678056	52.510000;
                           -9.680278	52.513333;
                           -9.679444	52.517500;
                           -9.679167	52.521944;
                           -9.678333	52.526111;
                           -9.675278	52.528889;
                           -9.670556	52.530556;
                           -9.667778	52.533333;
                           -9.667500	52.536944;
                           -9.670278	52.539722;
                           -9.673333	52.542500;
                           -9.675556	52.545833;
                           -9.673889	52.548611;
                           -9.670278	52.550833;
                           -9.664722	52.551944;
                           -9.660833	52.554167;
                           -9.658056	52.556944;
                           -9.655556	52.560278;
                           -9.651667	52.562500;
                           -9.646111	52.563611;
                           -9.645556	52.564167;
                           -9.646389	52.562778;
                           -9.643333	52.564722;
                           -9.638611	52.566389;
                           -9.633611	52.567778;
                           -9.628889	52.569444;
                           -9.622778	52.570278;
                           -9.616667	52.571111;
                           -9.610000	52.571389;
                           -9.602500	52.571389;
                           -9.595000	52.571389;
                           -9.588056	52.570833;
                           -9.581667	52.570278;
                           -9.575556	52.569167;
                           -9.569444	52.568333;
                           -9.564167	52.566944;
                           -9.558056	52.565833;
                           -9.552222	52.565000;
                           -9.546111	52.563889;
                           -9.539444	52.563333;
                           -9.531944	52.563333;
                           -9.525833	52.564167;
                           -9.521111	52.565833;
                           -9.515278	52.566667;
                           -9.508611	52.566111;
                           -9.504167	52.565278;
                           -9.498889	52.563611;
                           -9.493611	52.562222;
                           -9.487500	52.561389;
                           -9.482500	52.559722;
                           -9.476389	52.558889;
                           -9.469722	52.558333;
                           -9.465000	52.559167;
                           -9.462222	52.561944;
                           -9.459722	52.565000;
                           -9.457222	52.568333;
                           -9.454167	52.570833;
                           -9.449722	52.572778;
                           -9.445000	52.574444;
                           -9.439722	52.575833;
                           -9.432778	52.576111;
                           -9.426667	52.576944;
                           -9.420278	52.576389;
                           -9.414722	52.575000;
                           -9.408889	52.573889;
                           -9.402222	52.573333;
                           -9.396667	52.574722;
                           -9.391667	52.576111;
                           -9.384722	52.576389;
                           -9.379444	52.577778;
                           -9.374722	52.579444;
                           -9.368611	52.580278;
                           -9.365000	52.578889;
                           -9.361389	52.576389;
                           -9.360833	52.572500;
                           -9.357778	52.569722;
                           -9.351111	52.570000;
                           -9.344167	52.570278;
                           -9.338056	52.571111;
                           -9.331667	52.571389;
                           -9.324722	52.570833;
                           -9.317778	52.570278;
                           -9.311944	52.569444;
                           -9.306667	52.568056;
                           -9.301389	52.566389;
                           -9.294722	52.565833;
                           -9.288889	52.566667;
                           -9.283333	52.568056;
                           -9.278889	52.569722;
                           -9.273333	52.571111;
                           -9.268889	52.572778;
                           -9.263611	52.574167;
                           -9.257500	52.575000;
                           -9.251389	52.575833;
                           -9.245278	52.576667;
                           -9.240833	52.578333;
                           -9.236944	52.580556;
                           -9.232500	52.582500;
                           -9.226944	52.583611;
                           -9.222500	52.585556;
                           -9.216944	52.586667;
                           -9.211667	52.588056;
                           -9.207222	52.589722;
                           -9.201944	52.591111;
                           -9.196667	52.592500;
                           -9.191111	52.593611;
                           -9.185833	52.595000;
                           -9.180556	52.596111;
                           -9.175278	52.597500;
                           -9.170556	52.599167;
                           -9.165278	52.600556;
                           -9.160000	52.601944;
                           -9.154444	52.603056;
                           -9.148056	52.603611;
                           -9.141944	52.604167;
                           -9.135833	52.605000;
                           -9.130278	52.606389;
                           -9.125000	52.607778;
                           -9.119722	52.608889;
                           -9.113056	52.609444;
                           -9.106111	52.609722;
                           -9.099444	52.610000;
                           -9.093333	52.610833;
                           -9.087778	52.612222;
                           -9.083333	52.613889;
                           -9.079722	52.616111;
                           -9.075556	52.618333;
                           -9.071667	52.620556;
                           -9.067778	52.622778;
                           -9.063889	52.625000;
                           -9.059444	52.626667;
                           -9.053333	52.625833;
                           -9.051944	52.621944;
                           -9.051389	52.618056;
                           -9.049444	52.614722;
                           -9.043333	52.613611;
                           -9.037222	52.613611;
                           -9.033333	52.615833;
                           -9.030278	52.618333;
                           -9.027222	52.621111;
                           -9.020556	52.621389;
                           -9.016944	52.619167;
                           -9.012500	52.617222;
                           -9.007778	52.615278;
                           -9.001944	52.615278;
                           -8.998056	52.617500;
                           -8.991944	52.618333;
                           -8.987222	52.619167;
                           -8.985556	52.622778;
                           -8.985556	52.627222;
                           -8.983889	52.630833;
                           -8.980833	52.633611;
                           -8.977500	52.636389;
                           -8.973611	52.638611;
                           -8.970556	52.641111;
                           -8.967222	52.643889;
                           -8.964167	52.646667;
                           -8.961944	52.649722;
                           -8.958611	52.652500;
                           -8.952500	52.653333;
                           -8.945833	52.653611;
                           -8.938889	52.653889;
                           -8.932778	52.654722;
                           -8.926667	52.655556;
                           -8.921389	52.656944;
                           -8.915556	52.657778;
                           -8.909444	52.658333;
                           -8.904167	52.659722;
                           -8.898056	52.660556;
                           -8.891944	52.661389;
                           -8.886389	52.662500;
                           -8.879722	52.663056;
                           -8.872778	52.663333;
                           -8.865833	52.663611;
                           -8.860000	52.662500;
                           -8.853889	52.661667;
                           -8.848056	52.661389;
                           -8.843611	52.663333;
                           -8.838056	52.664444;
                           -8.832778	52.665833;
                           -8.825833	52.666111;
                           -8.818333	52.665833;
                           -8.813333	52.664444;
                           -8.810278	52.661667;
                           -8.806667	52.659167;
                           -8.803611	52.656389;
                           -8.801111	52.653611;
                           -8.794167	52.653056;
                           -8.788889	52.654444;
                           -8.785556	52.656944;
                           -8.783333	52.660278;
                           -8.779167	52.662500;
                           -8.775278	52.664722;
                           -8.769167	52.665556;
                           -8.763056	52.666389;
                           -8.756389	52.666667;
                           -8.749722	52.666944;
                           -8.743611	52.667778;
                           -8.738889	52.669444;
                           -8.741944	52.671389;
                           -8.747778	52.672500;
                           -8.755278	52.672500;
                           -8.763056	52.672778;
                           -8.769722	52.673333;
                           -8.776389	52.673889;
                           -8.782500	52.675000;
                           -8.786944	52.676667;
                           -8.791944	52.678333;
                           -8.797500	52.679722;
                           -8.803889	52.680278;
                           -8.811389	52.680556;
                           -8.818333	52.681111;
                           -8.824444	52.681944;
                           -8.829722	52.683611;
                           -8.833889	52.685556;
                           -8.838333	52.687500;
                           -8.843889	52.688889;
                           -8.848889	52.690278;
                           -8.855000	52.691389;
                           -8.861667	52.691944;
                           -8.868333	52.692500;
                           -8.876111	52.692500;
                           -8.882778	52.692222;
                           -8.888056	52.691111;
                           -8.894167	52.690278;
                           -8.898611	52.688611;
                           -8.903333	52.686667;
                           -8.908611	52.685556;
                           -8.914167	52.684167;
                           -8.920833	52.683889;
                           -8.928333	52.683889;
                           -8.934444	52.683056;
                           -8.940000	52.681944;
                           -8.945278	52.680556;
                           -8.950556	52.679444;
                           -8.957222	52.680000;
                           -8.960833	52.682222;
                           -8.961667	52.686389;
                           -8.959167	52.688611;
                           -8.955278	52.690833;
                           -8.951389	52.693056;
                           -8.946667	52.694722;
                           -8.942778	52.696944;
                           -8.939722	52.699722;
                           -8.936389	52.702500;
                           -8.934722	52.706111;
                           -8.932500	52.709167;
                           -8.933056	52.713333;
                           -8.936111	52.716111;
                           -8.941944	52.717222;
                           -8.947222	52.718611;
                           -8.948611	52.722222;
                           -8.946111	52.725556;
                           -8.942222	52.727778;
                           -8.939722	52.730833;
                           -8.940556	52.735000;
                           -8.943056	52.737778;
                           -8.946667	52.740000;
                           -8.950556	52.742500;
                           -8.954167	52.744722;
                           -8.956944	52.747500;
                           -8.959167	52.750833;
                           -8.959722	52.755000;
                           -8.957222	52.758056;
                           -8.955556	52.761667;
                           -8.956944	52.765556;
                           -8.962222	52.766944;
                           -8.968889	52.766667;
                           -8.974444	52.765278;
                           -8.978889	52.763611;
                           -8.983611	52.761944;
                           -8.988333	52.760000;
                           -8.993611	52.758889;
                           -8.999722	52.758056;
                           -9.004444	52.756111;
                           -9.009167	52.754444;
                           -9.013056	52.752222;
                           -9.017500	52.750556;
                           -9.021944	52.748889;
                           -9.026667	52.746944;
                           -9.031389	52.745278;
                           -9.035833	52.743611;
                           -9.039167	52.740833;
                           -9.037778	52.737222;
                           -9.034167	52.734722;
                           -9.030278	52.732500;
                           -9.027500	52.729722;
                           -9.026944	52.725556;
                           -9.029167	52.722222;
                           -9.031667	52.719167;
                           -9.035556	52.716944;
                           -9.038889	52.714167;
                           -9.041944	52.711667;
                           -9.045278	52.708889;
                           -9.048333	52.706111;
                           -9.050556	52.703056;
                           -9.053611	52.700278;
                           -9.056111	52.697222;
                           -9.059444	52.694444;
                           -9.061667	52.691389;
                           -9.064167	52.688056;
                           -9.067500	52.685556;
                           -9.070278	52.682778;
                           -9.074167	52.680556;
                           -9.078056	52.678333;
                           -9.081389	52.675556;
                           -9.083611	52.672500;
                           -9.085556	52.668889;
                           -9.087778	52.665556;
                           -9.091667	52.663333;
                           -9.095556	52.661111;
                           -9.100000	52.659444;
                           -9.104722	52.657778;
                           -9.109444	52.655833;
                           -9.112500	52.653333;
                           -9.115000	52.650000;
                           -9.117500	52.646944;
                           -9.120278	52.644167;
                           -9.122778	52.641111;
                           -9.126111	52.638333;
                           -9.129167	52.635556;
                           -9.131667	52.632500;
                           -9.134444	52.629722;
                           -9.138333	52.627500;
                           -9.142222	52.625278;
                           -9.146111	52.623056;
                           -9.150833	52.621389;
                           -9.156389	52.620000;
                           -9.161667	52.618889;
                           -9.168333	52.618333;
                           -9.175000	52.618056;
                           -9.181944	52.617778;
                           -9.188611	52.617500;
                           -9.195556	52.616944;
                           -9.202500	52.616667;
                           -9.209167	52.616389;
                           -9.215833	52.616111;
                           -9.222778	52.615556;
                           -9.229444	52.615278;
                           -9.236389	52.615000;
                           -9.242500	52.614167;
                           -9.247778	52.612778;
                           -9.251389	52.610556;
                           -9.254722	52.607778;
                           -9.257778	52.605278;
                           -9.261111	52.602500;
                           -9.263889	52.599722;
                           -9.267778	52.597500;
                           -9.270278	52.594444;
                           -9.274167	52.592222;
                           -9.277222	52.589444;
                           -9.281667	52.587778;
                           -9.287222	52.586389;
                           -9.293889	52.586111;
                           -9.301389	52.586111;
                           -9.309167	52.586389;
                           -9.315000	52.587222;
                           -9.321111	52.588333;
                           -9.326111	52.589722;
                           -9.330556	52.591667;
                           -9.332778	52.594722;
                           -9.331111	52.598611;
                           -9.328056	52.601111;
                           -9.323611	52.603056;
                           -9.318889	52.604722;
                           -9.314167	52.606389;
                           -9.309444	52.608333;
                           -9.305278	52.610000;
                           -9.300556	52.611667;
                           -9.295833	52.613611;
                           -9.291389	52.615278;
                           -9.286667	52.616944;
                           -9.282222	52.618889;
                           -9.278333	52.621111;
                           -9.274444	52.623333;
                           -9.271111	52.626111;
                           -9.269444	52.629722;
                           -9.272500	52.632500;
                           -9.278611	52.633333;
                           -9.284722	52.632500;
                           -9.290000	52.631389;
                           -9.294444	52.629444;
                           -9.298333	52.627222;
                           -9.303056	52.625000;
                           -9.308333	52.623889;
                           -9.314444	52.623056;
                           -9.321111	52.622500;
                           -9.327778	52.622222;
                           -9.334722	52.622778;
                           -9.341667	52.622500;
                           -9.346111	52.620556;
                           -9.349167	52.618056;
                           -9.353889	52.616111;
                           -9.359167	52.615000;
                           -9.365833	52.614444;
                           -9.372778	52.614167;
                           -9.379722	52.613889;
                           -9.383333	52.611667;
                           -9.387222	52.609444;
                           -9.391111	52.607222;
                           -9.395833	52.605278;
                           -9.401111	52.604167;
                           -9.408333	52.604167;
                           -9.413889	52.604722;
                           -9.419722	52.605556;
                           -9.425000	52.607222;
                           -9.430833	52.608056;
                           -9.436389	52.609444;
                           -9.442222	52.610556;
                           -9.448889	52.611111;
                           -9.456667	52.611111;
                           -9.462500	52.611389;
                           -9.467778	52.612778;
                           -9.472222	52.614722;
                           -9.476667	52.616389;
                           -9.481944	52.618056;
                           -9.488611	52.618611;
                           -9.493889	52.620000;
                           -9.497500	52.622222;
                           -9.501944	52.624167;
                           -9.507222	52.625556;
                           -9.513056	52.625556;
                           -9.519167	52.624722;
                           -9.526111	52.625278;
                           -9.530556	52.627222;
                           -9.533333	52.630000;
                           -9.535556	52.633333;
                           -9.537778	52.636944;
                           -9.539167	52.640556;
                           -9.540556	52.644444;
                           -9.541111	52.648333;
                           -9.541667	52.652500;
                           -9.541389	52.657222;
                           -9.542222	52.661389;
                           -9.544444	52.664444;
                           -9.550278	52.665556;
                           -9.555556	52.664167;
                           -9.561111	52.662778;
                           -9.566389	52.661667;
                           -9.571667	52.660278;
                           -9.576111	52.658611;
                           -9.578611	52.655278;
                           -9.577778	52.651111;
                           -9.575000	52.648333;
                           -9.571389	52.646111;
                           -9.567500	52.643889;
                           -9.563333	52.641944;
                           -9.560278	52.638611;
                           -9.561389	52.634444;
                           -9.564167	52.631944;
                           -9.567500	52.629167;
                           -9.571944	52.627222;
                           -9.577500	52.626111;
                           -9.584167	52.625556;
                           -9.587778	52.623333;
                           -9.590278	52.620278;
                           -9.593611	52.617500;
                           -9.597778	52.615833;
                           -9.602500	52.613889;
                           -9.607778	52.612778;
                           -9.613889	52.611944;
                           -9.620556	52.611389;
                           -9.627500	52.611944;
                           -9.635000	52.611944;
                           -9.641944	52.611667;
                           -9.648611	52.611389;
                           -9.654444	52.610556;
                           -9.660556	52.609722;
                           -9.666111	52.608333;
                           -9.672222	52.607500;
                           -9.677500	52.606111;
                           -9.682778	52.604722;
                           -9.688611	52.603889;
                           -9.694167	52.602778;
                           -9.698056	52.600556;
                           -9.701667	52.598333;
                           -9.704722	52.595556;
                           -9.706667	52.591944;
                           -9.706667	52.588333;
                           -9.703611	52.585556;
                           -9.701389	52.582222;
                           -9.703056	52.578611;
                           -9.707778	52.576667;
                           -9.714722	52.576389;
                           -9.721111	52.576944;
                           -9.726667	52.578333;
                           -9.730833	52.580278;
                           -9.737500	52.580833;
                           -9.741389	52.578611;
                           -9.744722	52.575833;
                           -9.747778	52.573056;
                           -9.751389	52.570833;
                           -9.756111	52.569167;
                           -9.761389	52.567778;
                           -9.767222	52.566944;
                           -9.773333	52.566111;
                           -9.779444	52.565278;
                           -9.785000	52.563889;
                           -9.790278	52.562500;
                           -9.795278	52.561111;
                           -9.802222	52.560833;
                           -9.807500	52.561389;
                           -9.812778	52.562778;
                           -9.819444	52.563333;
                           -9.826111	52.563056;
                           -9.832222	52.562222;
                           -9.839167	52.562500;
                           -9.845833	52.563056;
                           -9.851944	52.562222;
                           -9.857222	52.560833;
                           -9.861944	52.559167;
                           -9.866111	52.557500;
                           -9.870833	52.555556;
                           -9.875556	52.553889;
                           -9.880000	52.551944;
                           -9.886111	52.550833;
                           -9.892778	52.550278;
                           -9.900278	52.550278;
                           -9.906944	52.550833;
                           -9.913056	52.551944;
                           -9.920000	52.552500;
                           -9.926389	52.553056;
                           -9.932500	52.553889;
                           -9.936389	52.556111;
                           -9.934167	52.559444;
                           -9.929444	52.561111;
                           -9.924722	52.563056;
                           -9.921667	52.565556;
                           -9.918611	52.568333;
                           -9.915556	52.571111;
                           -9.911111	52.573056;
                           -9.905833	52.574167;
                           -9.899722	52.575000;
                           -9.893611	52.575833;
                           -9.888333	52.577222;
                           -9.883056	52.578611;
                           -9.878333	52.580278;
                           -9.873889	52.582222;
                           -9.868611	52.583611;
                           -9.863333	52.584722;
                           -9.857778	52.586111;
                           -9.852778	52.587500;
                           -9.848056	52.589167;
                           -9.841944	52.590000;
                           -9.835278	52.590556;
                           -9.827778	52.590278;
                           -9.821667	52.591111;
                           -9.816389	52.592500;
                           -9.811667	52.594444;
                           -9.807778	52.596667;
                           -9.805556	52.599722;
                           -9.803333	52.603056;
                           -9.802500	52.606944;
                           -9.800000	52.610278;
                           -9.794722	52.611667;
                           -9.791111	52.613889;
                           -9.788611	52.616944;
                           -9.784722	52.619167;
                           -9.780278	52.621111;
                           -9.775000	52.622222;
                           -9.769444	52.623611;
                           -9.765000	52.625556;
                           -9.761111	52.627778;
                           -9.758056	52.630278;
                           -9.754167	52.632778;
                           -9.749722	52.634444;
                           -9.745000	52.636111;
                           -9.739722	52.637500;
                           -9.735000	52.639167;
                           -9.731389	52.641667;
                           -9.727500	52.643889;
                           -9.724444	52.646389;
                           -9.721389	52.649167;
                           -9.717500	52.651389;
                           -9.713611	52.653611;
                           -9.709722	52.655833;
                           -9.706111	52.658056;
                           -9.704444	52.661944;
                           -9.703333	52.665833;
                           -9.700556	52.668611;
                           -9.694444	52.668611;
                           -9.688333	52.669444;
                           -9.684444	52.671667;
                           -9.680556	52.673889;
                           -9.676111	52.675556;
                           -9.670556	52.676944;
                           -9.664444	52.678333;
                           -9.661389	52.681111;
                           -9.660000	52.684722;
                           -9.658333	52.688333;
                           -9.656667	52.691944;
                           -9.654167	52.695000;
                           -9.650278	52.697222;
                           -9.646389	52.699722;
                           -9.642222	52.701389;
                           -9.637500	52.703056;
                           -9.633611	52.705278;
                           -9.628889	52.707222;
                           -9.626111	52.710000;
                           -9.624167	52.711667;
                           -9.623611	52.711111;
                           -9.625833	52.714444;
                           -9.628611	52.717222;
                           -9.629167	52.721389;
                           -9.625556	52.723611;
                           -9.620833	52.725278;
                           -9.614722	52.726111;
                           -9.610000	52.728056;
                           -9.607778	52.731111;
                           -9.606111	52.734722;
                           -9.603056	52.737500;
                           -9.597778	52.738889;
                           -9.592222	52.740278;
                           -9.587222	52.741389;
                           -9.582500	52.743333;
                           -9.577222	52.744444;
                           -9.571111	52.745278;
                           -9.565000	52.746111;
                           -9.559722	52.744722;
                           -9.556111	52.742500;
                           -9.553056	52.739722;
                           -9.548611	52.737778;
                           -9.543333	52.736389;
                           -9.538056	52.735000;
                           -9.535278	52.737500;
                           -9.531389	52.739722;
                           -9.527222	52.741944;
                           -9.521111	52.743056;
                           -9.514444	52.742500;
                           -9.509167	52.743611;
                           -9.505278	52.745833;
                           -9.501389	52.748056;
                           -9.498333	52.750833;
                           -9.495833	52.754167;
                           -9.493333	52.757222;
                           -9.494167	52.761389;
                           -9.496389	52.764444;
                           -9.499167	52.767222;
                           -9.499722	52.770556;
                           -9.495278	52.772222;
                           -9.490000	52.773611;
                           -9.486944	52.776389;
                           -9.488333	52.780000;
                           -9.490278	52.783333;
                           -9.490833	52.787500;
                           -9.486944	52.789722;
                           -9.486111	52.793611;
                           -9.487500	52.797500;
                           -9.485833	52.801111;
                           -9.482778	52.803611;
                           -9.477500	52.805000;
                           -9.472222	52.806389;
                           -9.468333	52.808611;
                           -9.466667	52.812222;
                           -9.464722	52.815833;
                           -9.462778	52.819167;
                           -9.458056	52.820833;
                           -9.453333	52.822500;
                           -9.448611	52.824444;
                           -9.444167	52.826111;
                           -9.440833	52.828889;
                           -9.439444	52.832500;
                           -9.436944	52.835556;
                           -9.437778	52.838889;
                           -9.442222	52.840833;
                           -9.444444	52.843889;
                           -9.443333	52.847222;
                           -9.439444	52.849444;
                           -9.434722	52.851111;
                           -9.430833	52.853333;
                           -9.427222	52.855556;
                           -9.428333	52.858333;
                           -9.433056	52.860278;
                           -9.437500	52.862222;
                           -9.438333	52.866389;
                           -9.436389	52.870000;
                           -9.434722	52.872778;
                           -9.432500	52.875833;
                           -9.429167	52.878611;
                           -9.424722	52.880278;
                           -9.419444	52.881667;
                           -9.414722	52.883333;
                           -9.410278	52.885000;
                           -9.405556	52.886944;
                           -9.401667	52.889167;
                           -9.396944	52.890833;
                           -9.392222	52.892778;
                           -9.388333	52.895000;
                           -9.385556	52.897500;
                           -9.383056	52.900833;
                           -9.380000	52.903611;
                           -9.376111	52.905833;
                           -9.372778	52.908333;
                           -9.368889	52.910556;
                           -9.365833	52.913333;
                           -9.361944	52.915556;
                           -9.358889	52.918333;
                           -9.355833	52.921111;
                           -9.353333	52.924167;
                           -9.351667	52.927778;
                           -9.351389	52.932500;
                           -9.353611	52.935556;
                           -9.358889	52.936944;
                           -9.365000	52.936111;
                           -9.369722	52.934444;
                           -9.374167	52.932778;
                           -9.378611	52.930833;
                           -9.383333	52.929167;
                           -9.388611	52.927778;
                           -9.394722	52.926944;
                           -9.400833	52.927222;
                           -9.407778	52.927778;
                           -9.413056	52.929167;
                           -9.418333	52.930556;
                           -9.424444	52.931667;
                           -9.431111	52.932222;
                           -9.438611	52.932222;
                           -9.444167	52.930833;
                           -9.448611	52.929167;
                           -9.454722	52.928333;
                           -9.460833	52.927500;
                           -9.467778	52.928056;
                           -9.471389	52.930278;
                           -9.473611	52.933611;
                           -9.475000	52.937222;
                           -9.473333	52.940833;
                           -9.470278	52.943611;
                           -9.466389	52.945833;
                           -9.460833	52.947222;
                           -9.456389	52.948889;
                           -9.451667	52.950556;
                           -9.447778	52.952778;
                           -9.443056	52.954722;
                           -9.439167	52.956944;
                           -9.435556	52.959167;
                           -9.432500	52.961944;
                           -9.430000	52.965000;
                           -9.428889	52.969167;
                           -9.427222	52.972778;
                           -9.425000	52.975833;
                           -9.422500	52.979167;
                           -9.419444	52.981944;
                           -9.416944	52.985000;
                           -9.414444	52.988056;
                           -9.411389	52.990833;
                           -9.408889	52.994167;
                           -9.406111	52.996667;
                           -9.402222	52.998889;
                           -9.398056	53.001111;
                           -9.394167	53.003333;
                           -9.391111	53.006111;
                           -9.393333	53.009444;
                           -9.395556	53.012500;
                           -9.393889	53.015278;
                           -9.390556	53.018056;
                           -9.388056	53.020278;
                           -9.385278	53.023056;
                           -9.382778	53.026111;
                           -9.380278	53.029444;
                           -9.377222	53.031944;
                           -9.374722	53.035278;
                           -9.372222	53.038333;
                           -9.369167	53.041111;
                           -9.366667	53.044167;
                           -9.364444	53.047500;
                           -9.361944	53.050556;
                           -9.359722	53.053889;
                           -9.357222	53.056944;
                           -9.355556	53.060556;
                           -9.353889	53.064167;
                           -9.353056	53.068333;
                           -9.350556	53.071389;
                           -9.347500	53.074167;
                           -9.342778	53.075833;
                           -9.337500	53.077222;
                           -9.332778	53.078889;
                           -9.328889	53.081389;
                           -9.324167	53.083056;
                           -9.321111	53.085833;
                           -9.317222	53.088056;
                           -9.313889	53.090556;
                           -9.310833	53.093333;
                           -9.307500	53.096111;
                           -9.304444	53.098889;
                           -9.301389	53.101389;
                           -9.298889	53.104722;
                           -9.295556	53.107222;
                           -9.293333	53.110556;
                           -9.291667	53.114167;
                           -9.290000	53.117778;
                           -9.289722	53.122222;
                           -9.289444	53.126944;
                           -9.288889	53.131111;
                           -9.286389	53.134167;
                           -9.283889	53.137222;
                           -9.280833	53.140000;
                           -9.277500	53.142778;
                           -9.273611	53.145000;
                           -9.270556	53.147778;
                           -9.266667	53.150000;
                           -9.260556	53.150833;
                           -9.253611	53.150278;
                           -9.247500	53.149167;
                           -9.242222	53.147778;
                           -9.237778	53.145833;
                           -9.233056	53.143889;
                           -9.228056	53.142500;
                           -9.223333	53.140556;
                           -9.218333	53.139167;
                           -9.213611	53.137222;
                           -9.209167	53.135278;
                           -9.203889	53.133889;
                           -9.197778	53.132778;
                           -9.192500	53.131389;
                           -9.187222	53.130000;
                           -9.182222	53.128611;
                           -9.176667	53.126944;
                           -9.172222	53.125000;
                           -9.167778	53.123333;
                           -9.161667	53.122222;
                           -9.156389	53.120833;
                           -9.148611	53.120556;
                           -9.143333	53.121944;
                           -9.139444	53.124167;
                           -9.135556	53.126389;
                           -9.131667	53.128611;
                           -9.125556	53.129444;
                           -9.119444	53.130278;
                           -9.114722	53.131944;
                           -9.108611	53.132778;
                           -9.102500	53.133611;
                           -9.096389	53.134444;
                           -9.091667	53.136111;
                           -9.091667	53.140000;
                           -9.097500	53.140833;
                           -9.104444	53.141389;
                           -9.111944	53.141667;
                           -9.119722	53.141667;
                           -9.125833	53.142778;
                           -9.128611	53.145556;
                           -9.127778	53.149722;
                           -9.123056	53.151389;
                           -9.116944	53.152222;
                           -9.110000	53.152500;
                           -9.103056	53.152778;
                           -9.095556	53.152778;
                           -9.087778	53.152500;
                           -9.080278	53.152500;
                           -9.072500	53.152500;
                           -9.065556	53.151944;
                           -9.058889	53.151111;
                           -9.052778	53.150278;
                           -9.046944	53.149167;
                           -9.041389	53.147778;
                           -9.036389	53.146389;
                           -9.031667	53.144444;
                           -9.026667	53.142778;
                           -9.021111	53.141389;
                           -9.014444	53.140833;
                           -9.008333	53.141667;
                           -9.003056	53.143056;
                           -8.996667	53.143889;
                           -8.991944	53.145556;
                           -8.991111	53.147778;
                           -8.996389	53.149167;
                           -9.001667	53.150833;
                           -9.006944	53.152222;
                           -9.012222	53.153611;
                           -9.017500	53.155000;
                           -9.023611	53.156111;
                           -9.030556	53.156667;
                           -9.037222	53.157222;
                           -9.045000	53.157500;
                           -9.052500	53.157500;
                           -9.060000	53.157778;
                           -9.066944	53.158333;
                           -9.072222	53.159722;
                           -9.073056	53.163056;
                           -9.068333	53.164722;
                           -9.062778	53.165833;
                           -9.055833	53.166389;
                           -9.049167	53.166667;
                           -9.042222	53.166111;
                           -9.035278	53.165556;
                           -9.028611	53.165000;
                           -9.022500	53.163889;
                           -9.015000	53.163889;
                           -9.008889	53.163611;
                           -9.002778	53.164444;
                           -8.996389	53.165278;
                           -8.990278	53.166111;
                           -8.985000	53.167500;
                           -8.978889	53.168333;
                           -8.971944	53.168611;
                           -8.965833	53.168333;
                           -8.965833	53.164722;
                           -8.968333	53.161667;
                           -8.971389	53.158889;
                           -8.973056	53.155278;
                           -8.971111	53.151944;
                           -8.968333	53.149167;
                           -8.962222	53.148333;
                           -8.954722	53.148056;
                           -8.949167	53.146667;
                           -8.946944	53.143333;
                           -8.944167	53.140556;
                           -8.939722	53.138611;
                           -8.935278	53.136667;
                           -8.929167	53.135833;
                           -8.922500	53.136111;
                           -8.918333	53.138333;
                           -8.919722	53.141944;
                           -8.923333	53.143333;
                           -8.929444	53.144444;
                           -8.933056	53.146667;
                           -8.934444	53.150556;
                           -8.935000	53.154722;
                           -8.936389	53.158333;
                           -8.937778	53.161944;
                           -8.940000	53.165278;
                           -8.940556	53.169444;
                           -8.940278	53.173889;
                           -8.939167	53.178056;
                           -8.936667	53.181111;
                           -8.935000	53.184722;
                           -8.932500	53.188056;
                           -8.930278	53.191111;
                           -8.926944	53.193889;
                           -8.923889	53.196667;
                           -8.920000	53.198611;
                           -8.916111	53.201111;
                           -8.912222	53.203056;
                           -8.908333	53.205278;
                           -8.904167	53.207500;
                           -8.900278	53.209722;
                           -8.898611	53.213611;
                           -8.900556	53.216389;
                           -8.906667	53.216389;
                           -8.912222	53.215000;
                           -8.916389	53.212778;
                           -8.920833	53.211111;
                           -8.925556	53.209444;
                           -8.930278	53.207500;
                           -8.935556	53.206389;
                           -8.941111	53.205000;
                           -8.946667	53.203889;
                           -8.952500	53.203056;
                           -8.958611	53.203889;
                           -8.962500	53.206389;
                           -8.965278	53.209167;
                           -8.968889	53.211389;
                           -8.976667	53.211667;
                           -8.984444	53.211667;
                           -8.991111	53.212222;
                           -8.997222	53.213333;
                           -9.001111	53.215833;
                           -9.003056	53.218889;
                           -9.000556	53.222222;
                           -8.995000	53.223333;
                           -8.988889	53.224167;
                           -8.985000	53.226389;
                           -8.984167	53.230556;
                           -8.979444	53.232222;
                           -8.972778	53.232500;
                           -8.965556	53.233056;
                           -8.966944	53.235833;
                           -8.972500	53.237222;
                           -8.974444	53.240556;
                           -8.972778	53.244167;
                           -8.968889	53.246389;
                           -8.964167	53.248056;
                           -8.958611	53.249444;
                           -8.953333	53.250556;
                           -8.946667	53.250833;
                           -8.939444	53.251389;
                           -8.934722	53.253056;
                           -8.933056	53.256667;
                           -8.930556	53.259722;
                           -8.931111	53.263889;
                           -8.935000	53.266389;
                           -8.941111	53.267222;
                           -8.948056	53.266944;
                           -8.953333	53.265833;
                           -8.958611	53.264444;
                           -8.964167	53.263056;
                           -8.970556	53.262222;
                           -8.976667	53.261667;
                           -8.980278	53.263889;
                           -8.986389	53.265000;
                           -8.993056	53.265556;
                           -8.999167	53.266389;
                           -9.005278	53.267500;
                           -9.012222	53.268056;
                           -9.019167	53.268611;
                           -9.025278	53.267778;
                           -9.030833	53.266667;
                           -9.035278	53.264722;
                           -9.040833	53.263611;
                           -9.045278	53.261667;
                           -9.050000	53.260000;
                           -9.054722	53.258333;
                           -9.059444	53.256667;
                           -9.065000	53.255278;
                           -9.070278	53.253889;
                           -9.075556	53.252778;
                           -9.081111	53.251389;
                           -9.086389	53.250000;
                           -9.092778	53.249167;
                           -9.098056	53.248056;
                           -9.105000	53.247778;
                           -9.112500	53.247778;
                           -9.119444	53.247500;
                           -9.125833	53.246667;
                           -9.131944	53.245833;
                           -9.137222	53.244444;
                           -9.143333	53.243611;
                           -9.149722	53.242778;
                           -9.156389	53.242500;
                           -9.162500	53.241667;
                           -9.169444	53.241389;
                           -9.175833	53.240556;
                           -9.183333	53.240556;
                           -9.190833	53.240833;
                           -9.197778	53.241389;
                           -9.205556	53.241389;
                           -9.213056	53.241667;
                           -9.219167	53.240833;
                           -9.225278	53.240000;
                           -9.231667	53.239167;
                           -9.238611	53.238889;
                           -9.245278	53.238333;
                           -9.251389	53.237500;
                           -9.258333	53.237222;
                           -9.265278	53.236944;
                           -9.272222	53.236389;
                           -9.279722	53.236667;
                           -9.287222	53.236667;
                           -9.295278	53.236944;
                           -9.302778	53.236944;
                           -9.309444	53.236667;
                           -9.315000	53.235278;
                           -9.321389	53.234444;
                           -9.328889	53.234444;
                           -9.336389	53.234722;
                           -9.343333	53.235278;
                           -9.349444	53.234444;
                           -9.355833	53.233611;
                           -9.362500	53.233333;
                           -9.369444	53.232778;
                           -9.374722	53.231667;
                           -9.381111	53.230833;
                           -9.387222	53.230000;
                           -9.394167	53.229444;
                           -9.400833	53.229167;
                           -9.408056	53.228889;
                           -9.414722	53.228333;
                           -9.422500	53.228611;
                           -9.430000	53.228611;
                           -9.436944	53.228333;
                           -9.444444	53.228333;
                           -9.452222	53.228611;
                           -9.458333	53.227778;
                           -9.464722	53.226944;
                           -9.470000	53.225556;
                           -9.476111	53.224722;
                           -9.482778	53.224167;
                           -9.488889	53.223611;
                           -9.493611	53.223611;
                           -9.500556	53.224167;
                           -9.507222	53.224722;
                           -9.514167	53.225278;
                           -9.520278	53.226111;
                           -9.527222	53.226667;
                           -9.532500	53.228056;
                           -9.537778	53.229722;
                           -9.543056	53.231111;
                           -9.547500	53.232778;
                           -9.550556	53.235556;
                           -9.549722	53.238889;
                           -9.547500	53.241944;
                           -9.548889	53.244722;
                           -9.553333	53.246667;
                           -9.557222	53.249167;
                           -9.559167	53.252222;
                           -9.560556	53.255833;
                           -9.558889	53.259722;
                           -9.555556	53.262222;
                           -9.551667	53.264444;
                           -9.549444	53.267778;
                           -9.548611	53.271944;
                           -9.549444	53.275833;
                           -9.553056	53.278333;
                           -9.559167	53.277500;
                           -9.563611	53.275556;
                           -9.567500	53.273333;
                           -9.571667	53.271111;
                           -9.574722	53.268611;
                           -9.573889	53.264444;
                           -9.572778	53.260556;
                           -9.571389	53.256944;
                           -9.571667	53.252500;
                           -9.573611	53.249167;
                           -9.576111	53.246111;
                           -9.577222	53.241944;
                           -9.578056	53.237778;
                           -9.581111	53.235278;
                           -9.585556	53.233333;
                           -9.591944	53.233333;
                           -9.598056	53.234444;
                           -9.603611	53.233056;
                           -9.610278	53.232778;
                           -9.616389	53.233611;
                           -9.620000	53.236111;
                           -9.623056	53.238889;
                           -9.626111	53.241667;
                           -9.627500	53.245278;
                           -9.627500	53.250000;
                           -9.626389	53.253889;
                           -9.623333	53.256667;
                           -9.620833	53.260000;
                           -9.617778	53.262500;
                           -9.614444	53.265278;
                           -9.611389	53.268056;
                           -9.608056	53.270833;
                           -9.605833	53.273889;
                           -9.604167	53.277500;
                           -9.603889	53.282222;
                           -9.604722	53.286111;
                           -9.607778	53.288889;
                           -9.610000	53.292222;
                           -9.610000	53.296944;
                           -9.609722	53.301389;
                           -9.611111	53.305000;
                           -9.613333	53.308333;
                           -9.616111	53.311111;
                           -9.615278	53.315278;
                           -9.611944	53.317778;
                           -9.606667	53.319167;
                           -9.599167	53.319167;
                           -9.591389	53.319167;
                           -9.585278	53.320000;
                           -9.580556	53.321667;
                           -9.577500	53.324444;
                           -9.575556	53.328056;
                           -9.574722	53.332222;
                           -9.573333	53.335833;
                           -9.572222	53.339722;
                           -9.574444	53.343056;
                           -9.577500	53.345833;
                           -9.579722	53.349167;
                           -9.585000	53.350556;
                           -9.589444	53.352500;
                           -9.592500	53.355278;
                           -9.596111	53.357500;
                           -9.602222	53.357500;
                           -9.606111	53.355278;
                           -9.608611	53.352222;
                           -9.610278	53.348611;
                           -9.611389	53.344444;
                           -9.609167	53.341111;
                           -9.605278	53.338889;
                           -9.600833	53.336944;
                           -9.605556	53.335278;
                           -9.610833	53.333889;
                           -9.616111	53.332500;
                           -9.621667	53.331389;
                           -9.626111	53.329444;
                           -9.631667	53.328333;
                           -9.638056	53.327222;
                           -9.643333	53.326111;
                           -9.649444	53.326944;
                           -9.654444	53.328333;
                           -9.660000	53.330000;
                           -9.664444	53.331667;
                           -9.663611	53.335833;
                           -9.659722	53.338056;
                           -9.653611	53.338889;
                           -9.646667	53.339444;
                           -9.642222	53.341111;
                           -9.639722	53.344444;
                           -9.639444	53.348889;
                           -9.636944	53.351944;
                           -9.633056	53.354444;
                           -9.630556	53.357500;
                           -9.630000	53.361667;
                           -9.628889	53.365556;
                           -9.627222	53.369444;
                           -9.625000	53.372500;
                           -9.631111	53.372500;
                           -9.637222	53.373611;
                           -9.640833	53.375833;
                           -9.644444	53.378333;
                           -9.650556	53.379167;
                           -9.656389	53.377778;
                           -9.659444	53.375278;
                           -9.661667	53.371944;
                           -9.664722	53.369444;
                           -9.668056	53.366667;
                           -9.671944	53.364444;
                           -9.677222	53.363056;
                           -9.681944	53.361389;
                           -9.687500	53.360000;
                           -9.692222	53.358056;
                           -9.696111	53.355833;
                           -9.698889	53.353333;
                           -9.701389	53.350000;
                           -9.703889	53.346944;
                           -9.705556	53.343333;
                           -9.706667	53.339167;
                           -9.708056	53.335556;
                           -9.711111	53.332778;
                           -9.715000	53.330556;
                           -9.718333	53.327778;
                           -9.721389	53.325000;
                           -9.723889	53.321944;
                           -9.723056	53.317778;
                           -9.723333	53.313333;
                           -9.725833	53.310000;
                           -9.728889	53.307222;
                           -9.733333	53.305556;
                           -9.738056	53.303889;
                           -9.743611	53.302500;
                           -9.749722	53.301667;
                           -9.755833	53.300833;
                           -9.761111	53.299444;
                           -9.764444	53.296667;
                           -9.769167	53.295000;
                           -9.776667	53.295000;
                           -9.782778	53.296111;
                           -9.788889	53.296944;
                           -9.795000	53.298056;
                           -9.800278	53.299444;
                           -9.804167	53.301667;
                           -9.806389	53.305000;
                           -9.808611	53.308056;
                           -9.813611	53.309722;
                           -9.821667	53.309722;
                           -9.827500	53.310556;
                           -9.831944	53.312500;
                           -9.835278	53.315278;
                           -9.838056	53.318056;
                           -9.842500	53.320000;
                           -9.848611	53.320833;
                           -9.855000	53.321944;
                           -9.861944	53.321389;
                           -9.865833	53.319167;
                           -9.868056	53.316111;
                           -9.871111	53.313333;
                           -9.876389	53.311944;
                           -9.883611	53.311667;
                           -9.889722	53.312500;
                           -9.895000	53.313889;
                           -9.899444	53.315833;
                           -9.904722	53.317222;
                           -9.905556	53.321389;
                           -9.903889	53.325000;
                           -9.903056	53.326111;
                           -9.903056	53.326389;
                           -9.900556	53.329722;
                           -9.897500	53.332500;
                           -9.894167	53.335000;
                           -9.891944	53.338333;
                           -9.888889	53.341111;
                           -9.885833	53.343611;
                           -9.884167	53.347222;
                           -9.885556	53.350000;
                           -9.891667	53.351111;
                           -9.897778	53.351944;
                           -9.898333	53.355278;
                           -9.894444	53.357500;
                           -9.890556	53.359722;
                           -9.885278	53.361111;
                           -9.880000	53.362500;
                           -9.872778	53.362778;
                           -9.866667	53.363611;
                           -9.860556	53.364444;
                           -9.855833	53.366389;
                           -9.851389	53.368056;
                           -9.847500	53.370278;
                           -9.842778	53.372222;
                           -9.836667	53.373056;
                           -9.828889	53.372778;
                           -9.822778	53.373611;
                           -9.817500	53.375000;
                           -9.812778	53.376944;
                           -9.808056	53.378611;
                           -9.803333	53.380556;
                           -9.798056	53.381667;
                           -9.793333	53.383611;
                           -9.788611	53.385278;
                           -9.787222	53.388889;
                           -9.786111	53.393056;
                           -9.783611	53.396111;
                           -9.787500	53.398611;
                           -9.793056	53.397222;
                           -9.799167	53.396389;
                           -9.804444	53.395000;
                           -9.811389	53.394722;
                           -9.816667	53.396111;
                           -9.818889	53.399444;
                           -9.818889	53.403889;
                           -9.815556	53.406667;
                           -9.810278	53.408056;
                           -9.805556	53.409722;
                           -9.801667	53.411944;
                           -9.800833	53.415278;
                           -9.804722	53.417500;
                           -9.811389	53.418056;
                           -9.817500	53.418056;
                           -9.823056	53.416667;
                           -9.827500	53.415000;
                           -9.830833	53.412222;
                           -9.833333	53.409167;
                           -9.837222	53.406944;
                           -9.841667	53.405000;
                           -9.847222	53.403611;
                           -9.852500	53.402500;
                           -9.855000	53.399167;
                           -9.858889	53.396944;
                           -9.863611	53.395278;
                           -9.870000	53.393889;
                           -9.876111	53.394722;
                           -9.878056	53.398056;
                           -9.880278	53.401389;
                           -9.881944	53.405000;
                           -9.880833	53.409167;
                           -9.876111	53.410833;
                           -9.870833	53.412222;
                           -9.864722	53.413056;
                           -9.860833	53.415278;
                           -9.857778	53.418056;
                           -9.859167	53.421667;
                           -9.865278	53.422778;
                           -9.871944	53.422222;
                           -9.877500	53.420833;
                           -9.881944	53.419167;
                           -9.889167	53.418889;
                           -9.895278	53.418889;
                           -9.902222	53.418333;
                           -9.907500	53.417222;
                           -9.911389	53.415000;
                           -9.915278	53.412500;
                           -9.917778	53.409444;
                           -9.919444	53.405833;
                           -9.920833	53.402222;
                           -9.921944	53.398056;
                           -9.922778	53.393889;
                           -9.923611	53.389722;
                           -9.924444	53.385833;
                           -9.926667	53.382500;
                           -9.930000	53.379722;
                           -9.935278	53.378611;
                           -9.942500	53.378056;
                           -9.948611	53.377222;
                           -9.951667	53.374444;
                           -9.954167	53.371389;
                           -9.959444	53.370000;
                           -9.966389	53.369722;
                           -9.970833	53.370556;
                           -9.973056	53.373889;
                           -9.974722	53.377500;
                           -9.978333	53.379722;
                           -9.983611	53.379444;
                           -9.990556	53.378889;
                           -9.996944	53.378056;
                           -10.00361	53.3777781;
                           -10.01138	53.3777789;
                           -10.01750	53.3786110;
                           -10.02194	53.3805564;
                           -10.02666	53.3825007;
                           -10.03027	53.3847228;
                           -10.03416	53.3872227;
                           -10.03722	53.3900002;
                           -10.04000	53.3927780;
                           -10.04166	53.3963897;
                           -10.04222	53.4005562;
                           -10.04083	53.4041673;
                           -10.04361	53.4069441;
                           -10.04833	53.4086113;
                           -10.05277	53.4105568;
                           -10.05805	53.4119446;
                           -10.06277	53.4138898;
                           -10.06638	53.4161119;
                           -10.06944	53.4188894;
                           -10.07500	53.4202780;
                           -10.07972	53.4202782;
                           -10.08111	53.4166671;
                           -10.08277	53.4130568;
                           -10.08750	53.4113890;
                           -10.09416	53.4108337;
                           -10.10194	53.4111114;
                           -10.10972	53.4111112;
                           -10.11361	53.4088891;
                           -10.11527	53.4050008;
                           -10.11888	53.4027789;
                           -10.12444	53.4016674;
                           -10.12888	53.4033339;
                           -10.13194	53.4061114;
                           -10.13416	53.4094447;
                           -10.13500	53.4136110;
                           -10.13722	53.4166672;
                           -10.14333	53.4177783;
                           -10.14861	53.4163891;
                           -10.15333	53.4144443;
                           -10.15722	53.4122222;
                           -10.16111	53.4100001;
                           -10.16500	53.4077780;
                           -10.17027	53.4063898;
                           -10.17805	53.4063896;
                           -10.18027	53.4097228;
                           -10.18027	53.4141678;
                           -10.17777	53.4175008;
                           -10.17472	53.4191672;
                           -10.17166	53.4219447;
                           -10.16916	53.4252787;
                           -10.16833	53.4294443;
                           -10.16888	53.4325009;
                           -10.16388	53.4338899;
                           -10.15888	53.4355569;
                           -10.15416	53.4375007;
                           -10.15027	53.4397228;
                           -10.14805	53.4427786;
                           -10.14500	53.4455560;
                           -10.14027	53.4475008;
                           -10.13583	53.4491673;
                           -10.12972	53.4500002;
                           -10.12194	53.4500004;
                           -10.11638	53.4486119;
                           -10.11500	53.4466670;
                           -10.11111	53.4444441;
                           -10.10805	53.4416676;
                           -10.10444	53.4394444;
                           -10.09972	53.4394442;
                           -10.09583	53.4416673;
                           -10.09055	53.4402786;
                           -10.08583	53.4383333;
                           -10.07972	53.4372222;
                           -10.07444	53.4377784;
                           -10.07138	53.4405569;
                           -10.06972	53.4441672;
                           -10.06750	53.4472220;
                           -10.06361	53.4497221;
                           -10.05944	53.4519444;
                           -10.05500	53.4536110;
                           -10.05166	53.4563897;
                           -10.05250	53.4594440;
                           -10.05777	53.4611118;
                           -10.06555	53.4611116;
                           -10.07250	53.4605560;
                           -10.07611	53.4630561;
                           -10.07694	53.4669444;
                           -10.07972	53.4697222;
                           -10.08444	53.4716674;
                           -10.08777	53.4744448;
                           -10.08888	53.4780569;
                           -10.09194	53.4808334;
                           -10.09583	53.4833333;
                           -10.09972	53.4855562;
                           -10.10333	53.4877783;
                           -10.10777	53.4897228;
                           -10.11333	53.4911113;
                           -10.11861	53.4925001;
                           -10.12333	53.4944443;
                           -10.12222	53.4975002;
                           -10.11944	53.5002784;
                           -10.12166	53.5036117;
                           -10.12694	53.5050004;
                           -10.13000	53.5069440;
                           -10.12666	53.5094447;
                           -10.12277	53.5119448;
                           -10.11972	53.5144442;
                           -10.12111	53.5183331;
                           -10.12583	53.5200003;
                           -10.13194	53.5211114;
                           -10.13805	53.5219446;
                           -10.14250	53.5238890;
                           -10.14638	53.5261119;
                           -10.14805	53.5297226;
                           -10.14861	53.5338891;
                           -10.14916	53.5380567;
                           -10.15388	53.5400009;
                           -10.16055	53.5405566;
                           -10.16694	53.5397224;
                           -10.17305	53.5386116;
                           -10.17833	53.5375003;
                           -10.18472	53.5363892;
                           -10.19166	53.5361117;
                           -10.19777	53.5369448;
                           -10.20250	53.5388890;
                           -10.20305	53.5422226;
                           -10.20000	53.5447220;
                           -10.19444	53.5461114;
                           -10.19055	53.5483336;
                           -10.18750	53.5511110;
                           -10.18277	53.5530568;
                           -10.17666	53.5538897;
                           -10.17027	53.5527788;
                           -10.16361	53.5533331;
                           -10.15888	53.5550009;
                           -10.15277	53.5547228;
                           -10.14805	53.5527786;
                           -10.14277	53.5513898;
                           -10.13611	53.5508331;
                           -10.13027	53.5522228;
                           -10.12361	53.5525001;
                           -10.11805	53.5511116;
                           -10.11194	53.5502784;
                           -10.10527	53.5505568;
                           -10.10194	53.5533334;
                           -10.10583	53.5555563;
                           -10.11027	53.5575008;
                           -10.11500	53.5594440;
                           -10.11638	53.5630569;
                           -10.11388	53.5661119;
                           -10.10777	53.5672228;
                           -10.10166	53.5669447;
                           -10.09472	53.5675002;
                           -10.08777	53.5669448;
                           -10.08333	53.5650003;
                           -10.07777	53.5636118;
                           -10.07166	53.5644447;
                           -10.06472	53.5638892;
                           -10.05777	53.5636118;
                           -10.05000	53.5633330;
                           -10.04333	53.5638893;
                           -10.03527	53.5638898;
                           -10.02861	53.5641671;
                           -10.02138	53.5644449;
                           -10.01472	53.5650002;
                           -10.00777	53.5652788;
                           -10.00694	53.5686114;
                           -10.01083	53.5708333;
                           -10.01444	53.5733334;
                           -10.01916	53.5750007;
                           -10.02361	53.5769441;
                           -10.02916	53.5783337;
                           -10.03277	53.5805568;
                           -10.03722	53.5825002;
                           -10.04138	53.5850009;
                           -10.04416	53.5877787;
                           -10.04722	53.5902782;
                           -10.04944	53.5936114;
                           -10.05083	53.5972223;
                           -10.04861	53.6005561;
                           -10.04305	53.6019446;
                           -10.03527	53.6016678;
                           -10.02833	53.6022223;
                           -10.02138	53.6025009;
                           -10.01611	53.6038891;
                           -10.01138	53.6055569;
                           -10.00500	53.6066670;
                           -9.998889	53.607500;
                           -9.992778	53.607222;
                           -9.986667	53.606389;
                           -9.981111	53.605000;
                           -9.975833	53.603611;
                           -9.970556	53.602222;
                           -9.966111	53.600278;
                           -9.960556	53.598889;
                           -9.955278	53.597500;
                           -9.950000	53.596111;
                           -9.942778	53.595556;
                           -9.936111	53.595000;
                           -9.928333	53.595000;
                           -9.921389	53.595278;
                           -9.916111	53.596667;
                           -9.912778	53.599444;
                           -9.909722	53.601944;
                           -9.905833	53.604167;
                           -9.901111	53.606111;
                           -9.895556	53.607500;
                           -9.889444	53.608333;
                           -9.883333	53.609167;
                           -9.879444	53.611389;
                           -9.880000	53.615556;
                           -9.880556	53.619722;
                           -9.881389	53.623611;
                           -9.881944	53.627778;
                           -9.883333	53.631667;
                           -9.885833	53.634722;
                           -9.888056	53.638056;
                           -9.890833	53.640833;
                           -9.895556	53.642778;
                           -9.899167	53.645000;
                           -9.903889	53.646944;
                           -9.908333	53.648611;
                           -9.913889	53.650278;
                           -9.915278	53.653889;
                           -9.916111	53.658056;
                           -9.915000	53.661111;
                           -9.914444	53.661111;
                           -9.914444	53.663889;
                           -9.913333	53.668056;
                           -9.912222	53.672222;
                           -9.913889	53.675833;
                           -9.916111	53.678889;
                           -9.918333	53.682222;
                           -9.921389	53.685000;
                           -9.923611	53.688333;
                           -9.924444	53.692222;
                           -9.923333	53.696389;
                           -9.921667	53.700000;
                           -9.919444	53.703611;
                           -9.916944	53.706944;
                           -9.914444	53.710000;
                           -9.912778	53.713611;
                           -9.910278	53.716944;
                           -9.908333	53.720000;
                           -9.906389	53.723889;
                           -9.906389	53.728333;
                           -9.906389	53.732778;
                           -9.906944	53.736944;
                           -9.908333	53.740833;
                           -9.909722	53.744444;
                           -9.909722	53.748889;
                           -9.908611	53.753056;
                           -9.904722	53.755278;
                           -9.898056	53.755833;
                           -9.890833	53.756111;
                           -9.884722	53.756944;
                           -9.879167	53.758333;
                           -9.873889	53.759722;
                           -9.867500	53.760556;
                           -9.861944	53.761667;
                           -9.856667	53.763056;
                           -9.851111	53.764444;
                           -9.845833	53.765833;
                           -9.839722	53.766667;
                           -9.833889	53.768056;
                           -9.827778	53.768889;
                           -9.822222	53.770000;
                           -9.816111	53.770833;
                           -9.810556	53.772222;
                           -9.804444	53.773056;
                           -9.798333	53.773889;
                           -9.792778	53.775278;
                           -9.785833	53.775556;
                           -9.778889	53.776111;
                           -9.774167	53.774167;
                           -9.771944	53.770833;
                           -9.765000	53.770556;
                           -9.757222	53.770278;
                           -9.750278	53.770833;
                           -9.743333	53.770278;
                           -9.735556	53.770000;
                           -9.728889	53.770556;
                           -9.722500	53.771389;
                           -9.716111	53.772222;
                           -9.710556	53.773611;
                           -9.705278	53.774722;
                           -9.698889	53.775556;
                           -9.693611	53.776944;
                           -9.687222	53.777778;
                           -9.680556	53.778333;
                           -9.675000	53.779444;
                           -9.668611	53.779444;
                           -9.665833	53.776667;
                           -9.661944	53.774444;
                           -9.656389	53.773056;
                           -9.648889	53.772778;
                           -9.643333	53.774167;
                           -9.640833	53.777222;
                           -9.636111	53.779167;
                           -9.631667	53.778056;
                           -9.626111	53.776667;
                           -9.620000	53.775833;
                           -9.613889	53.774722;
                           -9.606111	53.774722;
                           -9.599167	53.775000;
                           -9.593056	53.775833;
                           -9.587500	53.777222;
                           -9.582778	53.778889;
                           -9.578056	53.780833;
                           -9.572500	53.781944;
                           -9.567778	53.783889;
                           -9.563333	53.785556;
                           -9.559167	53.787778;
                           -9.554444	53.789444;
                           -9.550556	53.791944;
                           -9.547500	53.794444;
                           -9.545556	53.798056;
                           -9.547778	53.801389;
                           -9.553889	53.802500;
                           -9.561667	53.802500;
                           -9.567500	53.801111;
                           -9.573611	53.800278;
                           -9.580556	53.800000;
                           -9.585000	53.801944;
                           -9.588056	53.804722;
                           -9.591111	53.807500;
                           -9.595833	53.809167;
                           -9.601111	53.810833;
                           -9.606389	53.812222;
                           -9.613611	53.812778;
                           -9.619722	53.813611;
                           -9.625000	53.815278;
                           -9.622500	53.818333;
                           -9.617778	53.820000;
                           -9.613056	53.821944;
                           -9.607778	53.823056;
                           -9.603056	53.825000;
                           -9.598889	53.827222;
                           -9.597222	53.830000;
                           -9.602778	53.831389;
                           -9.608889	53.832222;
                           -9.615000	53.833333;
                           -9.616389	53.836944;
                           -9.613333	53.839722;
                           -9.607778	53.841111;
                           -9.601667	53.841944;
                           -9.595278	53.842778;
                           -9.589167	53.843611;
                           -9.583611	53.845000;
                           -9.578056	53.846111;
                           -9.577778	53.849722;
                           -9.576389	53.852500;
                           -9.571667	53.854444;
                           -9.567500	53.855556;
                           -9.564444	53.857500;
                           -9.561389	53.860278;
                           -9.563611	53.863333;
                           -9.569722	53.864444;
                           -9.576667	53.863889;
                           -9.583056	53.864167;
                           -9.585278	53.867222;
                           -9.583611	53.870000;
                           -9.578056	53.871389;
                           -9.572500	53.872778;
                           -9.572500	53.875278;
                           -9.574722	53.878611;
                           -9.574444	53.882222;
                           -9.570556	53.884444;
                           -9.566667	53.886667;
                           -9.566389	53.891389;
                           -9.567778	53.895000;
                           -9.574167	53.895000;
                           -9.578889	53.893333;
                           -9.583611	53.891667;
                           -9.588889	53.893056;
                           -9.595833	53.893611;
                           -9.600556	53.891667;
                           -9.607778	53.891389;
                           -9.613889	53.891389;
                           -9.620000	53.890556;
                           -9.626389	53.889722;
                           -9.633333	53.890278;
                           -9.640278	53.890833;
                           -9.646389	53.890000;
                           -9.651944	53.891389;
                           -9.657222	53.892778;
                           -9.663611	53.893889;
                           -9.670278	53.894444;
                           -9.676389	53.895278;
                           -9.682778	53.896389;
                           -9.689722	53.896944;
                           -9.697500	53.896944;
                           -9.704722	53.896667;
                           -9.710833	53.895833;
                           -9.718611	53.895833;
                           -9.725556	53.896389;
                           -9.730833	53.895000;
                           -9.735000	53.892778;
                           -9.740556	53.891389;
                           -9.747500	53.891111;
                           -9.753611	53.891944;
                           -9.758333	53.893889;
                           -9.764444	53.895000;
                           -9.770833	53.894167;
                           -9.775278	53.892222;
                           -9.779167	53.890000;
                           -9.781667	53.886944;
                           -9.785000	53.884167;
                           -9.788889	53.881944;
                           -9.793056	53.879722;
                           -9.797500	53.877778;
                           -9.803056	53.876667;
                           -9.807778	53.874722;
                           -9.813333	53.873333;
                           -9.817778	53.871667;
                           -9.823611	53.870278;
                           -9.829722	53.869444;
                           -9.835833	53.868611;
                           -9.841389	53.867222;
                           -9.848333	53.866944;
                           -9.854444	53.866111;
                           -9.860833	53.865278;
                           -9.866944	53.864444;
                           -9.873333	53.863611;
                           -9.880278	53.863056;
                           -9.885833	53.861667;
                           -9.891111	53.860556;
                           -9.896667	53.859167;
                           -9.902778	53.858333;
                           -9.910000	53.857778;
                           -9.914444	53.859722;
                           -9.919167	53.861667;
                           -9.923889	53.863611;
                           -9.930000	53.864444;
                           -9.936667	53.865000;
                           -9.940556	53.867222;
                           -9.942778	53.870556;
                           -9.939722	53.873333;
                           -9.935000	53.875000;
                           -9.931944	53.877778;
                           -9.930278	53.881389;
                           -9.930833	53.885556;
                           -9.930833	53.890000;
                           -9.929167	53.893889;
                           -9.928889	53.898333;
                           -9.929722	53.902500;
                           -9.928611	53.906667;
                           -9.925556	53.909167;
                           -9.921667	53.911667;
                           -9.918333	53.914167;
                           -9.914444	53.916667;
                           -9.911389	53.919167;
                           -9.908889	53.922500;
                           -9.909722	53.926667;
                           -9.913333	53.928889;
                           -9.915833	53.932222;
                           -9.915556	53.935833;
                           -9.912222	53.938611;
                           -9.911667	53.942500;
                           -9.913889	53.945833;
                           -9.912778	53.950000;
                           -9.908333	53.951667;
                           -9.902778	53.953056;
                           -9.896389	53.953889;
                           -9.889444	53.953333;
                           -9.882500	53.952778;
                           -9.875556	53.953333;
                           -9.870000	53.954444;
                           -9.863889	53.954444;
                           -9.859167	53.952500;
                           -9.856111	53.951111;
                           -9.850833	53.949722;
                           -9.845278	53.948333;
                           -9.840556	53.946389;
                           -9.836111	53.944722;
                           -9.831667	53.942778;
                           -9.825278	53.941667;
                           -9.817500	53.941667;
                           -9.811944	53.940278;
                           -9.809722	53.936944;
                           -9.809167	53.933056;
                           -9.807500	53.929167;
                           -9.811111	53.926667;
                           -9.812500	53.923889;
                           -9.810278	53.920556;
                           -9.808056	53.917222;
                           -9.805833	53.914167;
                           -9.802778	53.911389;
                           -9.797500	53.910000;
                           -9.791944	53.910278;
                           -9.788056	53.912500;
                           -9.785000	53.915278;
                           -9.784722	53.919722;
                           -9.786944	53.923056;
                           -9.789167	53.926389;
                           -9.790556	53.930000;
                           -9.790556	53.934444;
                           -9.789444	53.938611;
                           -9.787778	53.942222;
                           -9.789444	53.946111;
                           -9.791389	53.949167;
                           -9.796111	53.951111;
                           -9.801389	53.952500;
                           -9.803611	53.955833;
                           -9.804444	53.960000;
                           -9.805833	53.963611;
                           -9.812222	53.964444;
                           -9.820000	53.964722;
                           -9.824722	53.962778;
                           -9.823333	53.959167;
                           -9.821111	53.955833;
                           -9.820278	53.951667;
                           -9.827500	53.952222;
                           -9.831944	53.954167;
                           -9.835556	53.956389;
                           -9.838889	53.959167;
                           -9.841111	53.962500;
                           -9.842500	53.966111;
                           -9.842222	53.970833;
                           -9.841389	53.975000;
                           -9.843056	53.978611;
                           -9.846667	53.980833;
                           -9.850556	53.983333;
                           -9.850278	53.987778;
                           -9.847778	53.991111;
                           -9.846389	53.994722;
                           -9.850000	53.996944;
                           -9.854722	53.998889;
                           -9.861111	53.999722;
                           -9.865833	54.001667;
                           -9.872500	54.002222;
                           -9.879722	54.001667;
                           -9.884167	54.003611;
                           -9.886389	54.006944;
                           -9.887222	54.011111;
                           -9.889444	54.014167;
                           -9.894722	54.013889;
                           -9.901111	54.013889;
                           -9.904167	54.016667;
                           -9.904722	54.020833;
                           -9.904722	54.024444;
                           -9.904444	54.029167;
                           -9.903889	54.033056;
                           -9.902778	54.037222;
                           -9.901111	54.040833;
                           -9.900278	54.045000;
                           -9.898611	54.048611;
                           -9.896944	54.052222;
                           -9.895278	54.055833;
                           -9.891389	54.058333;
                           -9.886111	54.057778;
                           -9.881389	54.055833;
                           -9.875833	54.056111;
                           -9.872500	54.058889;
                           -9.868611	54.061111;
                           -9.865556	54.063889;
                           -9.861667	54.066111;
                           -9.865278	54.068611;
                           -9.871667	54.068611;
                           -9.876944	54.070000;
                           -9.880000	54.072778;
                           -9.878333	54.076389;
                           -9.876667	54.080000;
                           -9.871111	54.081389;
                           -9.865833	54.082778;
                           -9.859444	54.083611;
                           -9.854722	54.085278;
                           -9.852222	54.088611;
                           -9.855556	54.091389;
                           -9.858333	54.094167;
                           -9.858056	54.097778;
                           -9.855000	54.100556;
                           -9.851111	54.102778;
                           -9.845000	54.103611;
                           -9.838611	54.104444;
                           -9.832222	54.105833;
                           -9.832222	54.109444;
                           -9.836944	54.111389;
                           -9.843056	54.112222;
                           -9.850000	54.112778;
                           -9.857778	54.112778;
                           -9.864167	54.111944;
                           -9.870278	54.111111;
                           -9.876111	54.109722;
                           -9.878333	54.106667;
                           -9.876944	54.103056;
                           -9.875556	54.099167;
                           -9.878611	54.096667;
                           -9.884722	54.096667;
                           -9.892500	54.096667;
                           -9.894167	54.093056;
                           -9.892778	54.089444;
                           -9.891389	54.085556;
                           -9.892222	54.081667;
                           -9.895556	54.078889;
                           -9.898611	54.076111;
                           -9.902500	54.073889;
                           -9.906389	54.071667;
                           -9.910556	54.069444;
                           -9.915278	54.067500;
                           -9.920556	54.066389;
                           -9.926389	54.065000;
                           -9.930278	54.062778;
                           -9.935833	54.061389;
                           -9.942500	54.060833;
                           -9.949722	54.060556;
                           -9.957500	54.060556;
                           -9.961389	54.063056;
                           -9.963611	54.066111;
                           -9.963611	54.069722;
                           -9.962778	54.073889;
                           -9.965000	54.077222;
                           -9.968056	54.080000;
                           -9.971944	54.082222;
                           -9.974722	54.084167;
                           -9.978056	54.086944;
                           -9.980833	54.089722;
                           -9.984167	54.092500;
                           -9.986389	54.095556;
                           -9.987778	54.099444;
                           -9.983889	54.101667;
                           -9.976944	54.101944;
                           -9.970556	54.102778;
                           -9.964444	54.103611;
                           -9.959444	54.105556;
                           -9.954722	54.107222;
                           -9.947778	54.107778;
                           -9.943056	54.109444;
                           -9.938333	54.111389;
                           -9.931389	54.111667;
                           -9.923611	54.111667;
                           -9.916389	54.111944;
                           -9.911667	54.113889;
                           -9.911667	54.118333;
                           -9.913889	54.121667;
                           -9.916944	54.124444;
                           -9.920556	54.125833;
                           -9.926389	54.124444;
                           -9.932500	54.125278;
                           -9.936389	54.127778;
                           -9.939444	54.130556;
                           -9.943056	54.132778;
                           -9.946944	54.135278;
                           -9.951667	54.136944;
                           -9.956389	54.138889;
                           -9.959444	54.141667;
                           -9.958611	54.145833;
                           -9.953889	54.147500;
                           -9.947500	54.146667;
                           -9.942778	54.144722;
                           -9.936667	54.143889;
                           -9.932500	54.146111;
                           -9.935000	54.149167;
                           -9.938889	54.151667;
                           -9.942500	54.153889;
                           -9.945833	54.156667;
                           -9.948056	54.160000;
                           -9.951944	54.162222;
                           -9.956667	54.164167;
                           -9.961111	54.165833;
                           -9.965833	54.167778;
                           -9.969722	54.170000;
                           -9.972778	54.172778;
                           -9.974167	54.174722;
                           -9.968056	54.173889;
                           -9.962778	54.172500;
                           -9.956389	54.171389;
                           -9.950278	54.170556;
                           -9.945278	54.172222;
                           -9.943611	54.175833;
                           -9.943611	54.176667;
                           -9.942778	54.176667;
                           -9.942778	54.180556;
                           -9.940556	54.183611;
                           -9.937222	54.186389;
                           -9.940278	54.189167;
                           -9.945833	54.190556;
                           -9.951944	54.191389;
                           -9.958889	54.191944;
                           -9.964444	54.193333;
                           -9.969722	54.194722;
                           -9.973611	54.197222;
                           -9.977500	54.199444;
                           -9.980833	54.202222;
                           -9.984444	54.204722;
                           -9.988333	54.206944;
                           -9.991389	54.209722;
                           -9.995000	54.211944;
                           -9.998333	54.214722;
                           -10.00222	54.2172222;
                           -10.00611	54.2194441;
                           -10.01305	54.2200006;
                           -10.01722	54.2177782;
                           -10.01722	54.2130562;
                           -10.01805	54.2088896;
                           -10.02111	54.2063891;
                           -10.02361	54.2030561;
                           -10.02055	54.2002786;
                           -10.01750	54.1975000;
                           -10.01444	54.1947224;
                           -10.01138	54.1919449;
                           -10.00972	54.1883332;
                           -10.01000	54.1838890;
                           -10.01305	54.1811116;
                           -10.01722	54.1788892;
                           -10.02277	54.1775008;
                           -10.02944	54.1780564;
                           -10.03444	54.1797224;
                           -10.03916	54.1816677;
                           -10.04527	54.1827788;
                           -10.05138	54.1827789;
                           -10.05472	54.1800002;
                           -10.05250	54.1766670;
                           -10.04944	54.1738894;
                           -10.04527	54.1716678;
                           -10.04250	54.1688890;
                           -10.04166	54.1647227;
                           -10.04555	54.1625006;
                           -10.05277	54.1630568;
                           -10.05972	54.1636112;
                           -10.06583	54.1627783;
                           -10.06972	54.1605562;
                           -10.07472	54.1586112;
                           -10.08027	54.1572228;
                           -10.08500	54.1555560;
                           -10.08638	54.1519449;
                           -10.08361	54.1491671;
                           -10.07722	54.1480562;
                           -10.07166	54.1466677;
                           -10.06777	54.1444448;
                           -10.06500	54.1416670;
                           -10.06361	54.1380561;
                           -10.06805	54.1361116;
                           -10.07138	54.1333339;
                           -10.07305	54.1297226;
                           -10.07166	54.1261117;
                           -10.06944	54.1227784;
                           -10.07000	54.1205560;
                           -10.07555	54.1219446;
                           -10.08111	54.1233331;
                           -10.08777	54.1230568;
                           -10.09027	54.1197228;
                           -10.09138	54.1158339;
                           -10.08916	54.1125007;
                           -10.08444	54.1105564;
                           -10.08055	54.1091676;
                           -10.07416	54.1083337;
                           -10.06944	54.1063894;
                           -10.06750	54.1033330;
                           -10.06666	54.0991677;
                           -10.06444	54.0958334;
                           -10.06444	54.0922224;
                           -10.06694	54.0888894;
                           -10.07250	54.0877780;
                           -10.07861	54.0869441;
                           -10.08638	54.0869449;
                           -10.09277	54.0877788;
                           -10.09972	54.0883332;
                           -10.10583	54.0891673;
                           -10.11138	54.0908339;
                           -10.11666	54.0922227;
                           -10.12055	54.0944446;
                           -10.12444	54.0966674;
                           -10.12750	54.0994440;
                           -10.12805	54.1036116;
                           -10.12805	54.1083336;
                           -10.12888	54.1122229;
                           -10.12805	54.1163896;
                           -10.12555	54.1197226;
                           -10.12305	54.1227786;
                           -10.12083	54.1261113;
                           -10.11916	54.1297227;
                           -10.11750	54.1333330;
                           -10.11583	54.1369443;
                           -10.11416	54.1405567;
                           -10.11250	54.1441670;
                           -10.10944	54.1469444;
                           -10.10694	54.1502784;
                           -10.10444	54.1533334;
                           -10.10277	54.1569448;
                           -10.10194	54.1611114;
                           -10.10111	54.1652781;
                           -10.09861	54.1683331;
                           -10.09555	54.1711116;
                           -10.09222	54.1738892;
                           -10.09055	54.1775006;
                           -10.09055	54.1822226;
                           -10.09277	54.1852788;
                           -10.09416	54.1888897;
                           -10.09416	54.1927787;
                           -10.09250	54.1963890;
                           -10.08861	54.1986111;
                           -10.08333	54.2000003;
                           -10.07750	54.2011110;
                           -10.07194	54.2025004;
                           -10.06750	54.2044440;
                           -10.06416	54.2069447;
                           -10.06250	54.2108330;
                           -10.06250	54.2152780;
                           -10.06472	54.2186112;
                           -10.06944	54.2202784;
                           -10.07638	54.2208339;
                           -10.08277	54.2200008;
                           -10.08750	54.2183330;
                           -10.09388	54.2183339;
                           -10.09916	54.2197227;
                           -10.10222	54.2225002;
                           -10.10611	54.2247221;
                           -10.10916	54.2275007;
                           -10.11222	54.2302782;
                           -10.10916	54.2330567;
                           -10.10194	54.2333334;
                           -10.09500	54.2338890;
                           -10.08944	54.2352784;
                           -10.08861	54.2383331;
                           -10.08833	54.2430563;
                           -10.08527	54.2455568;
                           -10.08361	54.2494441;
                           -10.08361	54.2538891;
                           -10.08166	54.2575007;
                           -10.08166	54.2580567;
                           -10.07777	54.2594448;
                           -10.07388	54.2616679;
                           -10.07083	54.2644443;
                           -10.06750	54.2669440;
                           -10.06444	54.2697224;
                           -10.05944	54.2716674;
                           -10.05416	54.2730567;
                           -10.04777	54.2738898;
                           -10.04138	54.2747229;
                           -10.03583	54.2758333;
                           -10.03055	54.2772226;
                           -10.02666	54.2794447;
                           -10.02305	54.2822226;
                           -10.02250	54.2863890;
                           -10.02083	54.2900003;
                           -10.01833	54.2933333;
                           -10.01500	54.2958330;
                           -10.01194	54.2986114;
                           -10.00888	54.3013899;
                           -10.00305	54.3016676;
                           -9.997778	54.301389;
                           -9.993889	54.298889;
                           -9.991667	54.295833;
                           -9.989167	54.292500;
                           -9.986944	54.289444;
                           -9.985556	54.285556;
                           -9.983333	54.282500;
                           -9.980833	54.279167;
                           -9.979444	54.275556;
                           -9.976667	54.272778;
                           -9.973333	54.270000;
                           -9.969444	54.267778;
                           -9.965556	54.265278;
                           -9.960833	54.263333;
                           -9.955556	54.261944;
                           -9.950278	54.260556;
                           -9.942500	54.260556;
                           -9.936111	54.261389;
                           -9.931111	54.263056;
                           -9.925833	54.264444;
                           -9.921111	54.262500;
                           -9.918056	54.260000;
                           -9.915000	54.257222;
                           -9.910278	54.255278;
                           -9.904722	54.255556;
                           -9.902500	54.253333;
                           -9.901111	54.249722;
                           -9.901111	54.245000;
                           -9.901389	54.240556;
                           -9.902778	54.236944;
                           -9.905556	54.233611;
                           -9.909444	54.231389;
                           -9.914722	54.230000;
                           -9.917222	54.226944;
                           -9.920556	54.224167;
                           -9.926111	54.222778;
                           -9.932500	54.221944;
                           -9.937778	54.220556;
                           -9.943333	54.219444;
                           -9.950556	54.220000;
                           -9.955000	54.221667;
                           -9.958333	54.224444;
                           -9.961111	54.227222;
                           -9.964444	54.230000;
                           -9.966667	54.233333;
                           -9.969444	54.236111;
                           -9.973333	54.238333;
                           -9.979722	54.239444;
                           -9.985000	54.238056;
                           -9.984444	54.233889;
                           -9.981389	54.231111;
                           -9.980000	54.227500;
                           -9.980833	54.223333;
                           -9.979444	54.219722;
                           -9.977778	54.216944;
                           -9.974722	54.214167;
                           -9.970278	54.212222;
                           -9.963889	54.211389;
                           -9.956111	54.211111;
                           -9.948889	54.210833;
                           -9.942222	54.210278;
                           -9.935000	54.209722;
                           -9.928056	54.209167;
                           -9.920833	54.208611;
                           -9.913889	54.208056;
                           -9.906944	54.207500;
                           -9.900000	54.208056;
                           -9.894167	54.209167;
                           -9.888889	54.210556;
                           -9.887222	54.214167;
                           -9.890000	54.216944;
                           -9.892500	54.220278;
                           -9.890833	54.223889;
                           -9.883889	54.224167;
                           -9.877500	54.225278;
                           -9.875000	54.228333;
                           -9.877222	54.231667;
                           -9.881944	54.233333;
                           -9.885000	54.236111;
                           -9.883333	54.240000;
                           -9.880833	54.243056;
                           -9.880000	54.247222;
                           -9.879167	54.251389;
                           -9.877500	54.255000;
                           -9.875000	54.258056;
                           -9.871944	54.260833;
                           -9.868611	54.263611;
                           -9.864444	54.265833;
                           -9.860556	54.268056;
                           -9.856667	54.270278;
                           -9.851111	54.271667;
                           -9.845556	54.273056;
                           -9.838611	54.273333;
                           -9.831667	54.273611;
                           -9.823611	54.273611;
                           -9.817500	54.272778;
                           -9.812778	54.270833;
                           -9.809444	54.268056;
                           -9.805833	54.265556;
                           -9.801389	54.263889;
                           -9.796389	54.261944;
                           -9.791667	54.260000;
                           -9.787222	54.258056;
                           -9.781667	54.256667;
                           -9.776389	54.255278;
                           -9.769167	54.254722;
                           -9.765278	54.256944;
                           -9.764444	54.261111;
                           -9.767222	54.263889;
                           -9.773056	54.265278;
                           -9.777500	54.267222;
                           -9.782222	54.269167;
                           -9.787500	54.270556;
                           -9.792222	54.272500;
                           -9.796944	54.274167;
                           -9.800000	54.276944;
                           -9.801389	54.280833;
                           -9.798056	54.283333;
                           -9.791944	54.284167;
                           -9.789444	54.287500;
                           -9.793333	54.289722;
                           -9.799444	54.290833;
                           -9.807500	54.290833;
                           -9.814444	54.290556;
                           -9.821667	54.290000;
                           -9.829444	54.290278;
                           -9.835556	54.291111;
                           -9.841111	54.292500;
                           -9.846667	54.293889;
                           -9.854444	54.294167;
                           -9.860000	54.295556;
                           -9.864444	54.297500;
                           -9.868333	54.299722;
                           -9.870556	54.303056;
                           -9.870556	54.306667;
                           -9.868056	54.309722;
                           -9.866389	54.313333;
                           -9.863333	54.316111;
                           -9.858333	54.317778;
                           -9.853889	54.319722;
                           -9.849722	54.321944;
                           -9.845833	54.324167;
                           -9.841667	54.326389;
                           -9.837778	54.328611;
                           -9.833611	54.330833;
                           -9.828889	54.332778;
                           -9.822778	54.333611;
                           -9.817222	54.335000;
                           -9.810833	54.335833;
                           -9.805556	54.336944;
                           -9.799722	54.338333;
                           -9.793333	54.337500;
                           -9.788056	54.336111;
                           -9.782500	54.334444;
                           -9.776389	54.333611;
                           -9.769167	54.333889;
                           -9.762222	54.334444;
                           -9.755833	54.333333;
                           -9.752222	54.331111;
                           -9.747500	54.329167;
                           -9.742778	54.327222;
                           -9.735833	54.326667;
                           -9.730278	54.325278;
                           -9.725556	54.323333;
                           -9.721667	54.321111;
                           -9.716944	54.319167;
                           -9.711667	54.317778;
                           -9.705278	54.316944;
                           -9.700000	54.315278;
                           -9.694444	54.313889;
                           -9.687500	54.313333;
                           -9.682222	54.311944;
                           -9.676389	54.310556;
                           -9.670278	54.310556;
                           -9.664167	54.311389;
                           -9.660000	54.313611;
                           -9.656667	54.316389;
                           -9.652222	54.318056;
                           -9.645833	54.318889;
                           -9.640278	54.317500;
                           -9.634167	54.316389;
                           -9.627778	54.317222;
                           -9.621389	54.318056;
                           -9.615278	54.317222;
                           -9.610556	54.315278;
                           -9.604167	54.314444;
                           -9.597222	54.314722;
                           -9.591667	54.316111;
                           -9.583611	54.315833;
                           -9.577500	54.315000;
                           -9.574444	54.312222;
                           -9.570556	54.309722;
                           -9.565278	54.308333;
                           -9.558056	54.308611;
                           -9.551667	54.309444;
                           -9.547222	54.311389;
                           -9.542222	54.313056;
                           -9.536111	54.312222;
                           -9.532222	54.309722;
                           -9.528333	54.307500;
                           -9.522222	54.307500;
                           -9.518056	54.309722;
                           -9.515556	54.312778;
                           -9.511667	54.315000;
                           -9.506111	54.316389;
                           -9.499722	54.315278;
                           -9.493056	54.314722;
                           -9.485833	54.314167;
                           -9.480556	54.312778;
                           -9.476667	54.310556;
                           -9.471944	54.308611;
                           -9.468056	54.306111;
                           -9.463333	54.304444;
                           -9.456944	54.303333;
                           -9.452500	54.305000;
                           -9.446667	54.306389;
                           -9.440556	54.307222;
                           -9.432500	54.307222;
                           -9.425556	54.306667;
                           -9.418611	54.306111;
                           -9.412222	54.305000;
                           -9.406111	54.304167;
                           -9.400556	54.302500;
                           -9.397778	54.299722;
                           -9.394444	54.296944;
                           -9.390833	54.294722;
                           -9.386944	54.292500;
                           -9.380000	54.291944;
                           -9.375278	54.293611;
                           -9.373611	54.297222;
                           -9.371944	54.300833;
                           -9.368333	54.303611;
                           -9.363889	54.305278;
                           -9.357500	54.306111;
                           -9.351944	54.307500;
                           -9.347778	54.309722;
                           -9.345278	54.312778;
                           -9.343889	54.316389;
                           -9.339722	54.318611;
                           -9.333333	54.317778;
                           -9.328611	54.315833;
                           -9.323333	54.314444;
                           -9.318611	54.312500;
                           -9.313889	54.310556;
                           -9.308611	54.309167;
                           -9.302500	54.308056;
                           -9.296111	54.307222;
                           -9.290000	54.306111;
                           -9.283611	54.305278;
                           -9.276667	54.304444;
                           -9.271111	54.303056;
                           -9.265000	54.302222;
                           -9.261111	54.299722;
                           -9.257222	54.297500;
                           -9.254444	54.294722;
                           -9.251944	54.291389;
                           -9.250833	54.287778;
                           -9.250000	54.283611;
                           -9.250278	54.278889;
                           -9.251944	54.275278;
                           -9.253056	54.271389;
                           -9.253889	54.267222;
                           -9.250833	54.267222;
                           -9.246944	54.269444;
                           -9.240556	54.270278;
                           -9.233333	54.270556;
                           -9.227778	54.271944;
                           -9.223056	54.273611;
                           -9.220833	54.276667;
                           -9.216667	54.278889;
                           -9.211111	54.277500;
                           -9.208056	54.274722;
                           -9.207500	54.270556;
                           -9.210000	54.267500;
                           -9.211667	54.263889;
                           -9.209444	54.260556;
                           -9.208889	54.256389;
                           -9.210556	54.252778;
                           -9.212500	54.249167;
                           -9.214722	54.246111;
                           -9.217222	54.242778;
                           -9.215000	54.240556;
                           -9.208611	54.241389;
                           -9.203056	54.242500;
                           -9.197778	54.241944;
                           -9.195556	54.238889;
                           -9.197222	54.235278;
                           -9.198889	54.231667;
                           -9.198611	54.227500;
                           -9.201667	54.224722;
                           -9.208056	54.223889;
                           -9.212778	54.222222;
                           -9.216667	54.220000;
                           -9.220000	54.217222;
                           -9.220833	54.213056;
                           -9.218889	54.210000;
                           -9.214722	54.207500;
                           -9.210278	54.205556;
                           -9.204722	54.204167;
                           -9.198611	54.203056;
                           -9.192500	54.202222;
                           -9.186389	54.201111;
                           -9.180000	54.200278;
                           -9.174444	54.198611;
                           -9.170000	54.196667;
                           -9.167778	54.193611;
                           -9.164722	54.190833;
                           -9.161111	54.188333;
                           -9.158056	54.185556;
                           -9.156389	54.181944;
                           -9.154167	54.178611;
                           -9.150556	54.176389;
                           -9.146667	54.173889;
                           -9.143889	54.171111;
                           -9.139167	54.169444;
                           -9.133056	54.169167;
                           -9.128056	54.170833;
                           -9.123889	54.173056;
                           -9.120833	54.175833;
                           -9.120000	54.180000;
                           -9.117500	54.183056;
                           -9.115833	54.186667;
                           -9.113889	54.190556;
                           -9.113889	54.195000;
                           -9.116667	54.197778;
                           -9.120000	54.200556;
                           -9.115833	54.203333;
                           -9.109444	54.204167;
                           -9.103889	54.205278;
                           -9.101944	54.208889;
                           -9.101944	54.213611;
                           -9.100000	54.217222;
                           -9.096944	54.220000;
                           -9.094444	54.223056;
                           -9.091389	54.225833;
                           -9.087778	54.228611;
                           -9.083889	54.230833;
                           -9.082222	54.234444;
                           -9.080278	54.238056;
                           -9.077778	54.241111;
                           -9.075556	54.244167;
                           -9.073611	54.248056;
                           -9.074167	54.251944;
                           -9.071667	54.255278;
                           -9.068611	54.258056;
                           -9.064444	54.260278;
                           -9.062778	54.263889;
                           -9.064167	54.267500;
                           -9.063889	54.271944;
                           -9.063056	54.276111;
                           -9.060556	54.279444;
                           -9.057778	54.282500;
                           -9.053889	54.284722;
                           -9.049722	54.286944;
                           -9.043611	54.287778;
                           -9.036389	54.288056;
                           -9.028611	54.288056;
                           -9.021667	54.287500;
                           -9.013889	54.287222;
                           -9.006944	54.287500;
                           -8.999722	54.288056;
                           -8.991667	54.287778;
                           -8.985000	54.287222;
                           -8.978611	54.286111;
                           -8.970833	54.286111;
                           -8.963611	54.285556;
                           -8.957500	54.284444;
                           -8.950556	54.284722;
                           -8.944722	54.286111;
                           -8.938611	54.286944;
                           -8.931389	54.286389;
                           -8.926111	54.285000;
                           -8.920556	54.283333;
                           -8.916944	54.281111;
                           -8.912222	54.279167;
                           -8.908333	54.276667;
                           -8.903889	54.274444;
                           -8.900278	54.271944;
                           -8.896111	54.269722;
                           -8.892500	54.267222;
                           -8.888056	54.265556;
                           -8.884167	54.263056;
                           -8.878611	54.261667;
                           -8.872222	54.260556;
                           -8.865556	54.260833;
                           -8.858333	54.260278;
                           -8.853889	54.258333;
                           -8.850833	54.255556;
                           -8.846111	54.253611;
                           -8.840833	54.252222;
                           -8.833611	54.252500;
                           -8.826667	54.252778;
                           -8.820278	54.253611;
                           -8.813889	54.254444;
                           -8.807500	54.255278;
                           -8.804444	54.258056;
                           -8.801111	54.260556;
                           -8.796389	54.262500;
                           -8.790833	54.263611;
                           -8.785278	54.265000;
                           -8.780278	54.266667;
                           -8.775000	54.268056;
                           -8.770000	54.269722;
                           -8.765000	54.271389;
                           -8.759167	54.271389;
                           -8.755000	54.268889;
                           -8.752222	54.266111;
                           -8.748333	54.263889;
                           -8.744722	54.261389;
                           -8.738333	54.260556;
                           -8.731389	54.259722;
                           -8.725000	54.260556;
                           -8.718889	54.261389;
                           -8.713889	54.263056;
                           -8.708611	54.264444;
                           -8.702778	54.265833;
                           -8.697222	54.266944;
                           -8.690278	54.267222;
                           -8.683056	54.267500;
                           -8.676944	54.266667;
                           -8.670556	54.265556;
                           -8.665278	54.264167;
                           -8.660000	54.262500;
                           -8.654444	54.261111;
                           -8.650556	54.258611;
                           -8.647778	54.255833;
                           -8.647222	54.251667;
                           -8.647500	54.247222;
                           -8.646111	54.243611;
                           -8.644722	54.239722;
                           -8.642500	54.236667;
                           -8.640278	54.233333;
                           -8.636667	54.230556;
                           -8.630278	54.229444;
                           -8.624167	54.228333;
                           -8.619722	54.226667;
                           -8.616667	54.223611;
                           -8.610278	54.222778;
                           -8.605833	54.220833;
                           -8.603889	54.217500;
                           -8.598333	54.216111;
                           -8.591944	54.215000;
                           -8.588333	54.212500;
                           -8.587778	54.208611;
                           -8.584722	54.205556;
                           -8.581111	54.203333;
                           -8.576389	54.201389;
                           -8.570278	54.200278;
                           -8.566111	54.202500;
                           -8.563056	54.205278;
                           -8.558889	54.207500;
                           -8.552500	54.208333;
                           -8.545278	54.208611;
                           -8.539444	54.207500;
                           -8.533889	54.206111;
                           -8.527500	54.205000;
                           -8.520833	54.204444;
                           -8.514444	54.205278;
                           -8.511667	54.208333;
                           -8.513889	54.211667;
                           -8.516944	54.214444;
                           -8.522222	54.215833;
                           -8.527778	54.217222;
                           -8.533889	54.218333;
                           -8.540000	54.219444;
                           -8.546389	54.220556;
                           -8.550833	54.222500;
                           -8.555556	54.224444;
                           -8.559444	54.226667;
                           -8.563056	54.229167;
                           -8.565833	54.231944;
                           -8.569167	54.234722;
                           -8.571389	54.238056;
                           -8.574167	54.240833;
                           -8.576111	54.243889;
                           -8.579444	54.246667;
                           -8.583611	54.248611;
                           -8.588333	54.250556;
                           -8.596111	54.250833;
                           -8.603889	54.250833;
                           -8.611111	54.250556;
                           -8.615833	54.252500;
                           -8.613889	54.256111;
                           -8.611389	54.259444;
                           -8.607500	54.261667;
                           -8.602500	54.263333;
                           -8.600000	54.266389;
                           -8.598333	54.270000;
                           -8.596389	54.273611;
                           -8.593333	54.276389;
                           -8.587778	54.277778;
                           -8.582222	54.276111;
                           -8.579444	54.273333;
                           -8.575556	54.271111;
                           -8.570833	54.269167;
                           -8.564722	54.268056;
                           -8.558333	54.268889;
                           -8.551944	54.269722;
                           -8.545556	54.270278;
                           -8.539444	54.271111;
                           -8.533333	54.271944;
                           -8.526944	54.272778;
                           -8.519444	54.273056;
                           -8.512778	54.273333;
                           -8.506389	54.273333;
                           -8.500000	54.273889;
                           -8.492222	54.273889;
                           -8.485833	54.272778;
                           -8.480556	54.271389;
                           -8.473611	54.270833;
                           -8.470833	54.273889;
                           -8.473056	54.277222;
                           -8.476111	54.280000;
                           -8.480833	54.281944;
                           -8.486111	54.283333;
                           -8.490833	54.285278;
                           -8.494444	54.287778;
                           -8.498333	54.290000;
                           -8.503611	54.291389;
                           -8.509167	54.293056;
                           -8.515278	54.294167;
                           -8.521389	54.295000;
                           -8.528611	54.295556;
                           -8.533889	54.297222;
                           -8.541111	54.297778;
                           -8.546389	54.299167;
                           -8.551667	54.300833;
                           -8.555556	54.303056;
                           -8.557222	54.304167;
                           -8.563056	54.309167;
                           -8.564444	54.311944;
                           -8.566667	54.315278;
                           -8.567778	54.318889;
                           -8.566111	54.322500;
                           -8.561389	54.323333;
                           -8.556944	54.321389;
                           -8.554722	54.318056;
                           -8.551667	54.315278;
                           -8.547778	54.313056;
                           -8.542500	54.312500;
                           -8.536111	54.313333;
                           -8.529167	54.312500;
                           -8.524444	54.310833;
                           -8.520833	54.308333;
                           -8.515278	54.306944;
                           -8.507222	54.306667;
                           -8.501944	54.308056;
                           -8.498611	54.310556;
                           -8.497500	54.314722;
                           -8.499722	54.318056;
                           -8.502778	54.320833;
                           -8.507222	54.322778;
                           -8.513333	54.323889;
                           -8.520556	54.324444;
                           -8.526667	54.325278;
                           -8.530556	54.327778;
                           -8.532778	54.331111;
                           -8.537222	54.333056;
                           -8.544167	54.333611;
                           -8.551944	54.333611;
                           -8.559167	54.334444;
                           -8.566111	54.335000;
                           -8.573889	54.335000;
                           -8.581111	54.334722;
                           -8.585833	54.333056;
                           -8.590833	54.331389;
                           -8.596111	54.330000;
                           -8.604167	54.330278;
                           -8.611944	54.330278;
                           -8.617778	54.329167;
                           -8.622222	54.327222;
                           -8.625833	54.324722;
                           -8.628889	54.321944;
                           -8.630833	54.318333;
                           -8.635556	54.316667;
                           -8.642500	54.316389;
                           -8.648889	54.317222;
                           -8.653611	54.319167;
                           -8.655833	54.322500;
                           -8.653889	54.326111;
                           -8.651389	54.329167;
                           -8.650278	54.333333;
                           -8.652500	54.335833;
                           -8.658611	54.336667;
                           -8.665833	54.337222;
                           -8.671111	54.338889;
                           -8.674444	54.341667;
                           -8.671667	54.344722;
                           -8.670000	54.348333;
                           -8.674444	54.350278;
                           -8.674444	54.353889;
                           -8.670833	54.356667;
                           -8.666944	54.358889;
                           -8.662778	54.361111;
                           -8.658056	54.362778;
                           -8.652500	54.364167;
                           -8.646944	54.365278;
                           -8.640556	54.365278;
                           -8.635278	54.363889;
                           -8.628056	54.363056;
                           -8.623333	54.363889;
                           -8.621667	54.367500;
                           -8.620000	54.371111;
                           -8.615000	54.373056;
                           -8.609444	54.374167;
                           -8.603889	54.375556;
                           -8.597778	54.376389;
                           -8.591389	54.377222;
                           -8.585556	54.378333;
                           -8.580000	54.379722;
                           -8.575278	54.381389;
                           -8.571389	54.383611;
                           -8.569444	54.387222;
                           -8.567500	54.390833;
                           -8.566389	54.395000;
                           -8.563889	54.398056;
                           -8.559444	54.399722;
                           -8.553056	54.398889;
                           -8.549167	54.396389;
                           -8.545278	54.393889;
                           -8.540833	54.392222;
                           -8.534444	54.391111;
                           -8.529722	54.392778;
                           -8.527222	54.395833;
                           -8.525278	54.399444;
                           -8.521944	54.402222;
                           -8.518056	54.404444;
                           -8.513889	54.405833;
                           -8.509167	54.407500;
                           -8.504444	54.409167;
                           -8.500278	54.411389;
                           -8.496111	54.413611;
                           -8.492222	54.415833;
                           -8.488056	54.418056;
                           -8.484722	54.420833;
                           -8.482222	54.423889;
                           -8.478889	54.426667;
                           -8.476389	54.429722;
                           -8.472222	54.431944;
                           -8.469722	54.435000;
                           -8.468889	54.438333;
                           -8.473333	54.440278;
                           -8.477778	54.442222;
                           -8.481667	54.444444;
                           -8.482222	54.448611;
                           -8.479722	54.450833;
                           -8.474722	54.452500;
                           -8.471667	54.455278;
                           -8.468889	54.458333;
                           -8.465833	54.461111;
                           -8.461667	54.463333;
                           -8.455278	54.464167;
                           -8.448889	54.463056;
                           -8.444444	54.461111;
                           -8.442500	54.457778;
                           -8.440833	54.454167;
                           -8.440556	54.450000;
                           -8.435000	54.448611;
                           -8.430278	54.450278;
                           -8.426111	54.452500;
                           -8.421111	54.454167;
                           -8.415556	54.455556;
                           -8.409167	54.456111;
                           -8.402778	54.456944;
                           -8.396389	54.457778;
                           -8.389444	54.458056;
                           -8.383056	54.458889;
                           -8.376111	54.459722;
                           -8.369722	54.460278;
                           -8.363889	54.461667;
                           -8.358333	54.462778;
                           -8.352778	54.464167;
                           -8.347778	54.465833;
                           -8.343333	54.467500;
                           -8.337778	54.468889;
                           -8.331944	54.470000;
                           -8.326389	54.471389;
                           -8.320000	54.472222;
                           -8.313056	54.472500;
                           -8.305000	54.472222;
                           -8.297778	54.472500;
                           -8.290833	54.472778;
                           -8.284444	54.473611;
                           -8.278889	54.475000;
                           -8.274722	54.476944;
                           -8.271389	54.479722;
                           -8.268889	54.482778;
                           -8.266389	54.486111;
                           -8.265000	54.490278;
                           -8.265000	54.494722;
                           -8.260833	54.496944;
                           -8.256667	54.499167;
                           -8.255000	54.502778;
                           -8.256111	54.506389;
                           -8.258333	54.509722;
                           -8.259722	54.513333;
                           -8.260278	54.517500;
                           -8.258333	54.521111;
                           -8.255000	54.523889;
                           -8.250278	54.525556;
                           -8.246111	54.527778;
                           -8.241111	54.529444;
                           -8.238889	54.532500;
                           -8.235278	54.535278;
                           -8.231111	54.537500;
                           -8.226389	54.539167;
                           -8.220833	54.540556;
                           -8.215000	54.541667;
                           -8.210278	54.543333;
                           -8.208611	54.546944;
                           -8.207222	54.551111;
                           -8.206944	54.555833;
                           -8.208611	54.559444;
                           -8.208889	54.563611;
                           -8.205556	54.566111;
                           -8.200000	54.567500;
                           -8.195000	54.569167;
                           -8.190833	54.571389;
                           -8.188611	54.574444;
                           -8.184444	54.576667;
                           -8.178611	54.578056;
                           -8.174444	54.580278;
                           -8.172500	54.583889;
                           -8.171667	54.587778;
                           -8.168333	54.590556;
                           -8.163333	54.592222;
                           -8.159444	54.594444;
                           -8.156667	54.597500;
                           -8.155833	54.601667;
                           -8.156389	54.605833;
                           -8.157500	54.609444;
                           -8.157222	54.613056;
                           -8.153889	54.615833;
                           -8.148333	54.614444;
                           -8.146944	54.612500;
                           -8.145833	54.608889;
                           -8.144167	54.605000;
                           -8.141389	54.602222;
                           -8.138333	54.599444;
                           -8.133889	54.597500;
                           -8.127500	54.597222;
                           -8.124167	54.600000;
                           -8.120833	54.602778;
                           -8.118056	54.605833;
                           -8.111111	54.606111;
                           -8.106111	54.607778;
                           -8.104167	54.611389;
                           -8.107500	54.614167;
                           -8.111111	54.616667;
                           -8.113333	54.620000;
                           -8.113889	54.624167;
                           -8.118333	54.626111;
                           -8.124722	54.626944;
                           -8.128333	54.629444;
                           -8.128889	54.633611;
                           -8.125556	54.636111;
                           -8.123056	54.639444;
                           -8.129167	54.640556;
                           -8.134722	54.639167;
                           -8.138889	54.636944;
                           -8.142222	54.634444;
                           -8.145833	54.631667;
                           -8.150556	54.630000;
                           -8.156944	54.630000;
                           -8.161389	54.632500;
                           -8.168333	54.633056;
                           -8.173333	54.631389;
                           -8.178333	54.629722;
                           -8.183056	54.628056;
                           -8.188056	54.626389;
                           -8.193611	54.625000;
                           -8.198611	54.623333;
                           -8.204722	54.622500;
                           -8.210556	54.621389;
                           -8.216111	54.619444;
                           -8.220833	54.617778;
                           -8.225833	54.616111;
                           -8.230833	54.614444;
                           -8.235000	54.612222;
                           -8.238889	54.610000;
                           -8.243889	54.608333;
                           -8.247778	54.606111;
                           -8.252778	54.604444;
                           -8.258333	54.603056;
                           -8.263889	54.601944;
                           -8.270278	54.601111;
                           -8.276944	54.600278;
                           -8.283333	54.599444;
                           -8.291111	54.599722;
                           -8.297500	54.600833;
                           -8.297778	54.605000;
                           -8.295278	54.608056;
                           -8.291111	54.610278;
                           -8.288611	54.613333;
                           -8.286667	54.616944;
                           -8.284167	54.620278;
                           -8.280000	54.622500;
                           -8.277222	54.625556;
                           -8.277222	54.629167;
                           -8.279167	54.632500;
                           -8.281389	54.635833;
                           -8.285278	54.638056;
                           -8.292500	54.638611;
                           -8.298889	54.638056;
                           -8.303611	54.636111;
                           -8.308611	54.634444;
                           -8.313611	54.632778;
                           -8.317500	54.630556;
                           -8.320833	54.627778;
                           -8.324167	54.625278;
                           -8.329722	54.623889;
                           -8.334722	54.622222;
                           -8.339722	54.620556;
                           -8.341944	54.617500;
                           -8.344722	54.614167;
                           -8.348611	54.611944;
                           -8.354167	54.610833;
                           -8.358333	54.609444;
                           -8.363333	54.607778;
                           -8.366667	54.605000;
                           -8.370000	54.602500;
                           -8.372500	54.599167;
                           -8.375000	54.596111;
                           -8.377778	54.593056;
                           -8.380000	54.589722;
                           -8.384167	54.587500;
                           -8.389167	54.585833;
                           -8.394722	54.584722;
                           -8.400278	54.583333;
                           -8.406667	54.582500;
                           -8.412222	54.581389;
                           -8.418056	54.580000;
                           -8.422778	54.578333;
                           -8.426389	54.575556;
                           -8.428056	54.571944;
                           -8.431389	54.569444;
                           -8.434444	54.566667;
                           -8.438056	54.563889;
                           -8.442222	54.561667;
                           -8.446111	54.559722;
                           -8.451667	54.558333;
                           -8.458056	54.557500;
                           -8.459444	54.561111;
                           -8.457778	54.565000;
                           -8.453611	54.566944;
                           -8.450278	54.569722;
                           -8.449167	54.573889;
                           -8.448889	54.578333;
                           -8.446389	54.581667;
                           -8.442500	54.583889;
                           -8.437500	54.585556;
                           -8.431944	54.586667;
                           -8.426944	54.588611;
                           -8.421389	54.589722;
                           -8.416389	54.591389;
                           -8.411667	54.593333;
                           -8.406667	54.595000;
                           -8.401944	54.596667;
                           -8.398056	54.598889;
                           -8.393889	54.601111;
                           -8.390278	54.603889;
                           -8.386944	54.606389;
                           -8.383889	54.609167;
                           -8.381944	54.612778;
                           -8.382222	54.616944;
                           -8.385556	54.619722;
                           -8.390833	54.621111;
                           -8.398056	54.620833;
                           -8.403611	54.619722;
                           -8.407778	54.617500;
                           -8.413333	54.616111;
                           -8.418889	54.615000;
                           -8.426667	54.615000;
                           -8.432500	54.616667;
                           -8.437778	54.618056;
                           -8.442500	54.617222;
                           -8.445833	54.614444;
                           -8.448611	54.611389;
                           -8.451667	54.608611;
                           -8.455833	54.606389;
                           -8.459167	54.603889;
                           -8.464167	54.602222;
                           -8.468889	54.600278;
                           -8.475278	54.599722;
                           -8.480000	54.601389;
                           -8.481111	54.605278;
                           -8.479444	54.608889;
                           -8.482500	54.611667;
                           -8.484722	54.615000;
                           -8.481944	54.618056;
                           -8.479444	54.621111;
                           -8.480833	54.625000;
                           -8.486389	54.623611;
                           -8.489722	54.620833;
                           -8.494722	54.619167;
                           -8.501111	54.618333;
                           -8.508889	54.618611;
                           -8.514444	54.617222;
                           -8.519167	54.615556;
                           -8.525556	54.614722;
                           -8.533056	54.614444;
                           -8.539444	54.613611;
                           -8.545556	54.613056;
                           -8.551944	54.612222;
                           -8.557500	54.610833;
                           -8.563333	54.609722;
                           -8.568889	54.608333;
                           -8.574444	54.607222;
                           -8.580000	54.605833;
                           -8.585556	54.607222;
                           -8.591111	54.608889;
                           -8.597778	54.609444;
                           -8.605278	54.610000;
                           -8.611389	54.611111;
                           -8.616667	54.612500;
                           -8.621389	54.614444;
                           -8.626944	54.615833;
                           -8.633333	54.616111;
                           -8.637222	54.613889;
                           -8.642778	54.615278;
                           -8.650000	54.615833;
                           -8.656944	54.615556;
                           -8.662500	54.614444;
                           -8.668889	54.613611;
                           -8.675278	54.612778;
                           -8.680556	54.614167;
                           -8.685556	54.616111;
                           -8.686111	54.620278;
                           -8.686389	54.624444;
                           -8.689444	54.627222;
                           -8.695833	54.628333;
                           -8.702222	54.629167;
                           -8.707500	54.630833;
                           -8.711389	54.633056;
                           -8.716111	54.635000;
                           -8.720833	54.636944;
                           -8.725278	54.638889;
                           -8.730833	54.640278;
                           -8.738611	54.640556;
                           -8.744167	54.640556;
                           -8.743333	54.641111;
                           -8.749722	54.641111;
                           -8.755000	54.642778;
                           -8.759722	54.644444;
                           -8.764444	54.646389;
                           -8.768333	54.648889;
                           -8.773056	54.650833;
                           -8.778333	54.652222;
                           -8.783056	54.654167;
                           -8.787500	54.656111;
                           -8.792222	54.658056;
                           -8.796111	54.660278;
                           -8.797500	54.664167;
                           -8.795000	54.667222;
                           -8.790833	54.669444;
                           -8.786944	54.671667;
                           -8.784167	54.674722;
                           -8.785000	54.678889;
                           -8.788611	54.681389;
                           -8.794167	54.682778;
                           -8.799444	54.684167;
                           -8.804167	54.686111;
                           -8.806389	54.689444;
                           -8.805556	54.693611;
                           -8.802778	54.696667;
                           -8.799444	54.699444;
                           -8.794722	54.700278;
                           -8.788333	54.699167;
                           -8.781389	54.698611;
                           -8.774167	54.698889;
                           -8.768611	54.700000;
                           -8.762222	54.700833;
                           -8.756667	54.702222;
                           -8.751111	54.703611;
                           -8.747500	54.706111;
                           -8.748333	54.710278;
                           -8.749444	54.713889;
                           -8.746944	54.717222;
                           -8.744444	54.720278;
                           -8.743333	54.724444;
                           -8.743056	54.728889;
                           -8.739167	54.731111;
                           -8.733333	54.732500;
                           -8.726389	54.732778;
                           -8.720000	54.733611;
                           -8.715000	54.735278;
                           -8.710833	54.737500;
                           -8.706667	54.739722;
                           -8.703611	54.742500;
                           -8.699444	54.745000;
                           -8.695278	54.747222;
                           -8.692222	54.750000;
                           -8.688056	54.752222;
                           -8.682500	54.753611;
                           -8.677500	54.755278;
                           -8.674722	54.758333;
                           -8.671667	54.761111;
                           -8.667500	54.763333;
                           -8.663333	54.765556;
                           -8.656944	54.766389;
                           -8.649167	54.766111;
                           -8.642500	54.766944;
                           -8.636111	54.767778;
                           -8.629167	54.768056;
                           -8.621111	54.767778;
                           -8.613889	54.768056;
                           -8.606111	54.768056;
                           -8.600556	54.766667;
                           -8.593611	54.765833;
                           -8.586389	54.765278;
                           -8.579444	54.765556;
                           -8.572778	54.766389;
                           -8.565556	54.766667;
                           -8.559444	54.765833;
                           -8.557222	54.763889;
                           -8.551667	54.762500;
                           -8.545278	54.761389;
                           -8.537222	54.761111;
                           -8.531111	54.761944;
                           -8.524722	54.762778;
                           -8.518333	54.761667;
                           -8.516111	54.758333;
                           -8.515000	54.754722;
                           -8.511111	54.752500;
                           -8.504722	54.751389;
                           -8.497500	54.750833;
                           -8.489722	54.750556;
                           -8.481667	54.750278;
                           -8.474722	54.749722;
                           -8.466667	54.749722;
                           -8.459444	54.749167;
                           -8.451667	54.748889;
                           -8.445278	54.749722;
                           -8.444444	54.752778;
                           -8.448889	54.754722;
                           -8.456111	54.755278;
                           -8.460556	54.757222;
                           -8.464722	54.759722;
                           -8.467500	54.762500;
                           -8.470556	54.765278;
                           -8.476111	54.766667;
                           -8.482222	54.767778;
                           -8.489167	54.768333;
                           -8.496667	54.768889;
                           -8.504444	54.769167;
                           -8.511389	54.769722;
                           -8.518611	54.770278;
                           -8.525556	54.771111;
                           -8.532778	54.771667;
                           -8.538333	54.773056;
                           -8.541389	54.775833;
                           -8.537778	54.778611;
                           -8.531389	54.779444;
                           -8.525278	54.778333;
                           -8.518056	54.777778;
                           -8.510278	54.777500;
                           -8.503056	54.777778;
                           -8.495833	54.778056;
                           -8.488889	54.778611;
                           -8.481389	54.778889;
                           -8.473611	54.778611;
                           -8.469722	54.776389;
                           -8.466667	54.773333;
                           -8.463889	54.770556;
                           -8.460000	54.768333;
                           -8.455278	54.766389;
                           -8.450000	54.765000;
                           -8.444444	54.763333;
                           -8.440556	54.761111;
                           -8.436667	54.758611;
                           -8.432222	54.756667;
                           -8.426667	54.755278;
                           -8.419722	54.754722;
                           -8.413056	54.755278;
                           -8.407500	54.756667;
                           -8.406389	54.760833;
                           -8.409444	54.763889;
                           -8.414167	54.765833;
                           -8.419444	54.767500;
                           -8.426667	54.768056;
                           -8.432222	54.769444;
                           -8.436111	54.771944;
                           -8.438889	54.774722;
                           -8.441944	54.777500;
                           -8.445833	54.780000;
                           -8.448611	54.782778;
                           -8.453333	54.784722;
                           -8.457222	54.786944;
                           -8.462778	54.788611;
                           -8.470833	54.788611;
                           -8.477778	54.789167;
                           -8.482500	54.791111;
                           -8.486944	54.793056;
                           -8.490000	54.795833;
                           -8.493056	54.798611;
                           -8.496944	54.801111;
                           -8.500833	54.803333;
                           -8.506111	54.805000;
                           -8.511667	54.806389;
                           -8.515278	54.808889;
                           -8.518611	54.811667;
                           -8.520833	54.815000;
                           -8.526111	54.816389;
                           -8.534167	54.816389;
                           -8.540556	54.816667;
                           -8.546667	54.817778;
                           -8.549722	54.820556;
                           -8.551111	54.824167;
                           -8.550000	54.827500;
                           -8.546667	54.830000;
                           -8.541111	54.831389;
                           -8.533333	54.831111;
                           -8.526944	54.830000;
                           -8.521389	54.828611;
                           -8.515000	54.827500;
                           -8.508889	54.826667;
                           -8.503056	54.827778;
                           -8.500556	54.830833;
                           -8.498056	54.834167;
                           -8.494722	54.836667;
                           -8.489722	54.838611;
                           -8.485000	54.840278;
                           -8.479167	54.841389;
                           -8.472222	54.840833;
                           -8.467500	54.838889;
                           -8.461944	54.837500;
                           -8.456667	54.836111;
                           -8.450278	54.835000;
                           -8.442500	54.834722;
                           -8.435833	54.835556;
                           -8.431944	54.837778;
                           -8.427778	54.840000;
                           -8.422222	54.841111;
                           -8.415833	54.841111;
                           -8.409444	54.840000;
                           -8.404444	54.841667;
                           -8.396389	54.841667;
                           -8.392778	54.839167;
                           -8.388056	54.837222;
                           -8.380833	54.836667;
                           -8.374444	54.837500;
                           -8.367500	54.837778;
                           -8.362778	54.835833;
                           -8.358056	54.833889;
                           -8.351944	54.832778;
                           -8.344722	54.832222;
                           -8.336667	54.831944;
                           -8.331111	54.833333;
                           -8.325556	54.834444;
                           -8.321389	54.836667;
                           -8.321111	54.840556;
                           -8.326667	54.841944;
                           -8.334444	54.842222;
                           -8.340833	54.841389;
                           -8.347222	54.842500;
                           -8.351944	54.844167;
                           -8.357500	54.845833;
                           -8.363611	54.846944;
                           -8.371667	54.846944;
                           -8.379444	54.847222;
                           -8.383333	54.849444;
                           -8.383889	54.852778;
                           -8.381944	54.856389;
                           -8.380278	54.860000;
                           -8.382222	54.863333;
                           -8.383889	54.866944;
                           -8.378889	54.868611;
                           -8.372500	54.869444;
                           -8.368333	54.871667;
                           -8.364167	54.873889;
                           -8.357778	54.874722;
                           -8.352222	54.873056;
                           -8.346944	54.871667;
                           -8.340556	54.870556;
                           -8.333611	54.870000;
                           -8.327222	54.868889;
                           -8.320000	54.868333;
                           -8.317500	54.871389;
                           -8.314722	54.874722;
                           -8.310833	54.876944;
                           -8.305000	54.878056;
                           -8.305556	54.880833;
                           -8.311111	54.882222;
                           -8.318056	54.883056;
                           -8.326111	54.883056;
                           -8.330000	54.885556;
                           -8.333889	54.887778;
                           -8.335833	54.891111;
                           -8.335556	54.895833;
                           -8.341111	54.897222;
                           -8.345556	54.895000;
                           -8.351111	54.893611;
                           -8.357500	54.893056;
                           -8.363889	54.893889;
                           -8.370000	54.895000;
                           -8.375833	54.893889;
                           -8.376111	54.890278;
                           -8.373889	54.886944;
                           -8.373333	54.882778;
                           -8.378056	54.881111;
                           -8.384444	54.881111;
                           -8.390000	54.882778;
                           -8.394444	54.884722;
                           -8.399167	54.886667;
                           -8.403889	54.888611;
                           -8.409444	54.890000;
                           -8.414722	54.891389;
                           -8.420278	54.893056;
                           -8.425833	54.894444;
                           -8.431389	54.895833;
                           -8.436667	54.897500;
                           -8.442222	54.898889;
                           -8.446944	54.900833;
                           -8.448333	54.904444;
                           -8.448611	54.908611;
                           -8.449167	54.912778;
                           -8.448889	54.917222;
                           -8.446667	54.920556;
                           -8.446111	54.925000;
                           -8.448333	54.928333;
                           -8.447222	54.932500;
                           -8.443056	54.934722;
                           -8.438611	54.934444;
                           -8.435278	54.931667;
                           -8.430000	54.930278;
                           -8.423611	54.931111;
                           -8.417778	54.932222;
                           -8.411389	54.933056;
                           -8.403333	54.932778;
                           -8.398333	54.934722;
                           -8.394167	54.936944;
                           -8.389444	54.938611;
                           -8.383056	54.939444;
                           -8.376667	54.940000;
                           -8.372500	54.942222;
                           -8.372222	54.945833;
                           -8.377778	54.947500;
                           -8.385000	54.948056;
                           -8.392222	54.948611;
                           -8.397500	54.950278;
                           -8.402222	54.952222;
                           -8.406944	54.953889;
                           -8.408889	54.957222;
                           -8.412222	54.960000;
                           -8.415833	54.962500;
                           -8.419722	54.964722;
                           -8.423611	54.967222;
                           -8.425833	54.970556;
                           -8.426389	54.974444;
                           -8.426667	54.978611;
                           -8.431389	54.980556;
                           -8.435278	54.983056;
                           -8.438333	54.985833;
                           -8.442778	54.987778;
                           -8.446111	54.990556;
                           -8.448333	54.993889;
                           -8.445556	54.996944;
                           -8.440556	54.998611;
                           -8.434444	54.999444;
                           -8.428056	54.998333;
                           -8.422500	54.996944;
                           -8.415278	54.996389;
                           -8.408333	54.995833;
                           -8.403333	54.997500;
                           -8.400556	55.000556;
                           -8.400278	55.004167;
                           -8.402500	55.007500;
                           -8.405556	55.010278;
                           -8.404444	55.014444;
                           -8.401944	55.017500;
                           -8.402500	55.021667;
                           -8.403056	55.025833;
                           -8.401111	55.029444;
                           -8.396111	55.031111;
                           -8.389167	55.030556;
                           -8.381944	55.030000;
                           -8.373889	55.029722;
                           -8.367500	55.030556;
                           -8.362500	55.032222;
                           -8.357778	55.034167;
                           -8.354444	55.035833;
                           -8.351111	55.038611;
                           -8.349167	55.042222;
                           -8.348056	55.046111;
                           -8.348611	55.050278;
                           -8.348333	55.053889;
                           -8.343611	55.056111;
                           -8.337778	55.057500;
                           -8.330556	55.057778;
                           -8.323611	55.057222;
                           -8.321944	55.053333;
                           -8.320833	55.049722;
                           -8.316944	55.047222;
                           -8.312778	55.049444;
                           -8.310000	55.052778;
                           -8.308333	55.056389;
                           -8.307778	55.060833;
                           -8.311111	55.063611;
                           -8.315556	55.065556;
                           -8.319444	55.068056;
                           -8.323611	55.070278;
                           -8.325556	55.073611;
                           -8.322222	55.076389;
                           -8.315833	55.075278;
                           -8.308611	55.074722;
                           -8.302222	55.075833;
                           -8.300278	55.079444;
                           -8.302500	55.082778;
                           -8.305556	55.085556;
                           -8.309444	55.088056;
                           -8.311389	55.091111;
                           -8.315556	55.093611;
                           -8.317500	55.096944;
                           -8.318056	55.100833;
                           -8.317778	55.105556;
                           -8.316667	55.109722;
                           -8.314167	55.112778;
                           -8.312222	55.116389;
                           -8.308889	55.119167;
                           -8.304722	55.121389;
                           -8.301389	55.123889;
                           -8.298889	55.127222;
                           -8.296944	55.130833;
                           -8.294167	55.133889;
                           -8.292500	55.137500;
                           -8.290556	55.141111;
                           -8.288056	55.144167;
                           -8.283611	55.146389;
                           -8.278056	55.147778;
                           -8.270000	55.147500;
                           -8.263056	55.146944;
                           -8.256667	55.145278;
                           -8.249444	55.144722;
                           -8.243056	55.143611;
                           -8.236944	55.142778;
                           -8.230556	55.141667;
                           -8.223333	55.141111;
                           -8.216111	55.140278;
                           -8.209722	55.139444;
                           -8.202500	55.138611;
                           -8.194444	55.138611;
                           -8.188333	55.139167;
                           -8.181667	55.140000;
                           -8.176667	55.141667;
                           -8.171944	55.143611;
                           -8.168333	55.146111;
                           -8.167500	55.146667;
                           -8.165833	55.149444;
                           -8.162222	55.151944;
                           -8.158889	55.154722;
                           -8.154167	55.156389;
                           -8.148333	55.157500;
                           -8.140278	55.157500;
                           -8.135000	55.155556;
                           -8.133611	55.151667;
                           -8.133889	55.147222;
                           -8.136389	55.144167;
                           -8.140000	55.141389;
                           -8.143333	55.138611;
                           -8.147500	55.136389;
                           -8.151667	55.134444;
                           -8.155833	55.132222;
                           -8.158333	55.128889;
                           -8.161944	55.126389;
                           -8.162778	55.122222;
                           -8.160000	55.119444;
                           -8.153333	55.120278;
                           -8.147778	55.121389;
                           -8.139722	55.121111;
                           -8.132500	55.121389;
                           -8.127778	55.123333;
                           -8.125000	55.126389;
                           -8.122222	55.129444;
                           -8.118056	55.131667;
                           -8.114722	55.134444;
                           -8.113889	55.138611;
                           -8.115000	55.142222;
                           -8.113889	55.146389;
                           -8.110556	55.148889;
                           -8.105556	55.150556;
                           -8.099722	55.151944;
                           -8.092778	55.152222;
                           -8.084722	55.151944;
                           -8.078056	55.152778;
                           -8.072500	55.153889;
                           -8.067500	55.155833;
                           -8.063611	55.157778;
                           -8.059167	55.160000;
                           -8.055833	55.162778;
                           -8.052500	55.165556;
                           -8.047500	55.167222;
                           -8.041389	55.166111;
                           -8.033889	55.165556;
                           -8.026667	55.165833;
                           -8.022500	55.167778;
                           -8.021389	55.171944;
                           -8.021944	55.176111;
                           -8.023056	55.179722;
                           -8.023056	55.183611;
                           -8.021111	55.187222;
                           -8.016944	55.189167;
                           -8.017500	55.192500;
                           -8.019722	55.195833;
                           -8.021111	55.199444;
                           -8.019167	55.203056;
                           -8.016667	55.206111;
                           -8.013889	55.209444;
                           -8.010556	55.211944;
                           -8.005278	55.213889;
                           -8.001111	55.215833;
                           -7.995556	55.217222;
                           -7.990000	55.218333;
                           -7.983333	55.219167;
                           -7.976944	55.218889;
                           -7.972500	55.216944;
                           -7.970833	55.213333;
                           -7.969722	55.209722;
                           -7.965833	55.207222;
                           -7.959444	55.206111;
                           -7.952500	55.205556;
                           -7.946667	55.204167;
                           -7.942778	55.201667;
                           -7.943056	55.198056;
                           -7.945833	55.195000;
                           -7.948611	55.191667;
                           -7.951944	55.189167;
                           -7.954722	55.185833;
                           -7.954722	55.182222;
                           -7.950278	55.180278;
                           -7.944722	55.178889;
                           -7.939167	55.177222;
                           -7.932778	55.176111;
                           -7.925556	55.176389;
                           -7.920556	55.178333;
                           -7.915833	55.180000;
                           -7.910278	55.181111;
                           -7.902778	55.180556;
                           -7.900556	55.177222;
                           -7.901111	55.172778;
                           -7.899722	55.168889;
                           -7.895000	55.166944;
                           -7.890278	55.165000;
                           -7.893889	55.163889;
                           -7.898056	55.161667;
                           -7.900556	55.158611;
                           -7.903056	55.155278;
                           -7.905833	55.152222;
                           -7.907778	55.148611;
                           -7.903056	55.147500;
                           -7.898056	55.149167;
                           -7.892222	55.150556;
                           -7.887500	55.152222;
                           -7.882500	55.153889;
                           -7.876667	55.155000;
                           -7.871111	55.156389;
                           -7.864444	55.157222;
                           -7.858056	55.156944;
                           -7.859444	55.152778;
                           -7.863611	55.150556;
                           -7.868333	55.148889;
                           -7.873333	55.147222;
                           -7.877500	55.145000;
                           -7.883333	55.143889;
                           -7.887500	55.141667;
                           -7.891667	55.139444;
                           -7.895833	55.137222;
                           -7.899167	55.134722;
                           -7.901944	55.131389;
                           -7.902778	55.128333;
                           -7.895833	55.127778;
                           -7.890833	55.129444;
                           -7.885000	55.130556;
                           -7.878611	55.131389;
                           -7.872222	55.130278;
                           -7.868333	55.127778;
                           -7.863889	55.125833;
                           -7.857500	55.125833;
                           -7.857222	55.130278;
                           -7.856667	55.135000;
                           -7.854722	55.138611;
                           -7.850556	55.140833;
                           -7.845833	55.142500;
                           -7.842222	55.145000;
                           -7.841111	55.149167;
                           -7.840833	55.153889;
                           -7.840556	55.157500;
                           -7.839444	55.161389;
                           -7.837500	55.165000;
                           -7.835556	55.168611;
                           -7.833611	55.172500;
                           -7.832500	55.176389;
                           -7.831667	55.180556;
                           -7.831944	55.184722;
                           -7.837500	55.186111;
                           -7.845556	55.186389;
                           -7.851944	55.185556;
                           -7.860000	55.185833;
                           -7.863889	55.188333;
                           -7.865000	55.191944;
                           -7.867222	55.195278;
                           -7.871111	55.197500;
                           -7.874722	55.200000;
                           -7.878056	55.202778;
                           -7.878333	55.206944;
                           -7.877222	55.211111;
                           -7.874444	55.214167;
                           -7.871944	55.217222;
                           -7.868333	55.220000;
                           -7.864167	55.222222;
                           -7.857778	55.222778;
                           -7.849722	55.222778;
                           -7.842500	55.221944;
                           -7.836111	55.221111;
                           -7.829167	55.221389;
                           -7.825000	55.223333;
                           -7.823056	55.226944;
                           -7.824167	55.230833;
                           -7.821667	55.233889;
                           -7.817500	55.236111;
                           -7.813889	55.238611;
                           -7.811944	55.242222;
                           -7.809444	55.245556;
                           -7.803056	55.246111;
                           -7.796667	55.245278;
                           -7.793611	55.242222;
                           -7.794722	55.238333;
                           -7.796667	55.234722;
                           -7.797778	55.230556;
                           -7.796389	55.226944;
                           -7.797500	55.222778;
                           -7.801111	55.220000;
                           -7.805833	55.218333;
                           -7.811111	55.216667;
                           -7.812778	55.213056;
                           -7.813889	55.208889;
                           -7.816667	55.205833;
                           -7.820833	55.203611;
                           -7.821944	55.199444;
                           -7.822222	55.195000;
                           -7.820000	55.191667;
                           -7.816389	55.189444;
                           -7.813333	55.186389;
                           -7.813611	55.181944;
                           -7.812222	55.178333;
                           -7.810278	55.175000;
                           -7.805556	55.173056;
                           -7.798333	55.173333;
                           -7.790278	55.173056;
                           -7.785278	55.174722;
                           -7.781389	55.176944;
                           -7.775278	55.178056;
                           -7.768333	55.177500;
                           -7.763611	55.175556;
                           -7.758056	55.174167;
                           -7.753333	55.172222;
                           -7.751389	55.168889;
                           -7.747500	55.166389;
                           -7.739444	55.166111;
                           -7.735000	55.164167;
                           -7.731944	55.161389;
                           -7.727222	55.159444;
                           -7.720833	55.158333;
                           -7.716389	55.156389;
                           -7.711667	55.154444;
                           -7.707778	55.152222;
                           -7.704167	55.149722;
                           -7.700278	55.147222;
                           -7.698056	55.144167;
                           -7.696111	55.140833;
                           -7.694444	55.137222;
                           -7.694167	55.133056;
                           -7.694444	55.129167;
                           -7.694722	55.124722;
                           -7.696667	55.121111;
                           -7.700278	55.118333;
                           -7.701111	55.114444;
                           -7.699167	55.111111;
                           -7.699444	55.106389;
                           -7.702222	55.103333;
                           -7.704722	55.100278;
                           -7.706667	55.096667;
                           -7.702778	55.094167;
                           -7.696667	55.093056;
                           -7.691667	55.094722;
                           -7.688333	55.097500;
                           -7.688056	55.101944;
                           -7.687500	55.106667;
                           -7.685556	55.110278;
                           -7.683889	55.113889;
                           -7.681944	55.117500;
                           -7.680556	55.121667;
                           -7.678611	55.125278;
                           -7.676667	55.128889;
                           -7.675000	55.132500;
                           -7.671667	55.135000;
                           -7.668889	55.138333;
                           -7.666944	55.141944;
                           -7.666389	55.146389;
                           -7.668056	55.150000;
                           -7.670000	55.153333;
                           -7.673889	55.155833;
                           -7.676667	55.158611;
                           -7.677222	55.162778;
                           -7.676111	55.166944;
                           -7.675000	55.170833;
                           -7.677222	55.174167;
                           -7.682222	55.172500;
                           -7.686944	55.170833;
                           -7.691944	55.170833;
                           -7.693056	55.174722;
                           -7.690556	55.177778;
                           -7.688611	55.181389;
                           -7.690000	55.184167;
                           -7.696944	55.184722;
                           -7.699167	55.188056;
                           -7.698056	55.192222;
                           -7.696389	55.195833;
                           -7.695000	55.199722;
                           -7.695556	55.203889;
                           -7.695000	55.208611;
                           -7.695556	55.212778;
                           -7.697778	55.216111;
                           -7.701667	55.218333;
                           -7.705278	55.220833;
                           -7.712778	55.221389;
                           -7.716667	55.219167;
                           -7.719444	55.216111;
                           -7.720556	55.211944;
                           -7.720833	55.207500;
                           -7.721389	55.202778;
                           -7.722500	55.198889;
                           -7.723611	55.195556;
                           -7.722222	55.191944;
                           -7.720000	55.188611;
                           -7.716389	55.186111;
                           -7.714167	55.183056;
                           -7.712778	55.179167;
                           -7.710556	55.176111;
                           -7.709444	55.172222;
                           -7.713611	55.170000;
                           -7.718889	55.171667;
                           -7.723611	55.173611;
                           -7.728333	55.175556;
                           -7.732778	55.177500;
                           -7.738611	55.179167;
                           -7.741389	55.181944;
                           -7.741944	55.185000;
                           -7.738611	55.187778;
                           -7.734444	55.190000;
                           -7.734167	55.193611;
                           -7.739722	55.194167;
                           -7.745278	55.193056;
                           -7.750278	55.191389;
                           -7.754722	55.189167;
                           -7.760278	55.187778;
                           -7.766944	55.187222;
                           -7.771389	55.189167;
                           -7.775278	55.191389;
                           -7.776667	55.195278;
                           -7.778611	55.198333;
                           -7.783333	55.200278;
                           -7.788889	55.201944;
                           -7.793333	55.203889;
                           -7.795000	55.207500;
                           -7.793611	55.211667;
                           -7.790278	55.214167;
                           -7.786944	55.216944;
                           -7.783333	55.219722;
                           -7.778333	55.221389;
                           -7.773333	55.223056;
                           -7.768611	55.224722;
                           -7.763611	55.226389;
                           -7.758611	55.228056;
                           -7.755000	55.230833;
                           -7.753333	55.234444;
                           -7.753056	55.238056;
                           -7.751111	55.241667;
                           -7.746944	55.243889;
                           -7.741111	55.245000;
                           -7.734722	55.245833;
                           -7.727222	55.246111;
                           -7.720833	55.245000;
                           -7.715556	55.243611;
                           -7.710000	55.241944;
                           -7.704722	55.240556;
                           -7.698056	55.241111;
                           -7.693889	55.243333;
                           -7.691111	55.246389;
                           -7.690278	55.250556;
                           -7.688889	55.254722;
                           -7.686389	55.257778;
                           -7.682222	55.260000;
                           -7.676944	55.261667;
                           -7.672222	55.263333;
                           -7.667222	55.264167;
                           -7.662222	55.265833;
                           -7.658056	55.268056;
                           -7.655278	55.271111;
                           -7.652500	55.274444;
                           -7.648333	55.276389;
                           -7.642778	55.277778;
                           -7.635556	55.276944;
                           -7.630833	55.275000;
                           -7.628889	55.271944;
                           -7.627500	55.268056;
                           -7.627222	55.263889;
                           -7.627500	55.259444;
                           -7.626111	55.255833;
                           -7.621667	55.253889;
                           -7.618333	55.251944;
                           -7.617222	55.248056;
                           -7.618056	55.244167;
                           -7.616944	55.240278;
                           -7.613889	55.237500;
                           -7.611944	55.234167;
                           -7.608889	55.231389;
                           -7.607778	55.227778;
                           -7.608889	55.223611;
                           -7.610000	55.219444;
                           -7.607778	55.216389;
                           -7.608056	55.212500;
                           -7.611667	55.210000;
                           -7.616389	55.208333;
                           -7.620556	55.206111;
                           -7.623889	55.203333;
                           -7.625278	55.199444;
                           -7.623889	55.196667;
                           -7.621944	55.193333;
                           -7.619722	55.190000;
                           -7.616667	55.187222;
                           -7.613889	55.184444;
                           -7.610833	55.181667;
                           -7.605833	55.179444;
                           -7.599722	55.178611;
                           -7.593611	55.177500;
                           -7.587222	55.176389;
                           -7.582500	55.174444;
                           -7.577778	55.172500;
                           -7.576389	55.168611;
                           -7.574444	55.165278;
                           -7.571389	55.162500;
                           -7.568333	55.159722;
                           -7.565556	55.156944;
                           -7.562500	55.154167;
                           -7.559444	55.151111;
                           -7.555556	55.148889;
                           -7.552778	55.146111;
                           -7.550556	55.142778;
                           -7.547500	55.140000;
                           -7.543889	55.137500;
                           -7.539444	55.135556;
                           -7.535278	55.133056;
                           -7.530833	55.131111;
                           -7.527778	55.128333;
                           -7.525556	55.125000;
                           -7.526111	55.120556;
                           -7.525556	55.116389;
                           -7.524444	55.112778;
                           -7.526389	55.109167;
                           -7.529722	55.106389;
                           -7.533333	55.103889;
                           -7.534444	55.099722;
                           -7.531389	55.096944;
                           -7.531111	55.092778;
                           -7.531944	55.088611;
                           -7.537222	55.086944;
                           -7.541944	55.085278;
                           -7.548333	55.084444;
                           -7.555000	55.083889;
                           -7.561389	55.083056;
                           -7.566389	55.081389;
                           -7.569722	55.078611;
                           -7.573889	55.076389;
                           -7.578056	55.074444;
                           -7.581667	55.071667;
                           -7.585556	55.069444;
                           -7.589722	55.067222;
                           -7.592500	55.064167;
                           -7.595833	55.061667;
                           -7.599444	55.058889;
                           -7.604444	55.057222;
                           -7.610000	55.055833;
                           -7.616389	55.055278;
                           -7.622222	55.053889;
                           -7.627222	55.052222;
                           -7.631944	55.050556;
                           -7.635556	55.048056;
                           -7.638056	55.044722;
                           -7.640000	55.041111;
                           -7.638056	55.038056;
                           -7.631667	55.036944;
                           -7.626111	55.037222;
                           -7.620278	55.038333;
                           -7.615556	55.040000;
                           -7.611389	55.042222;
                           -7.605556	55.043611;
                           -7.598333	55.043611;
                           -7.590278	55.043611;
                           -7.583333	55.042778;
                           -7.577500	55.041389;
                           -7.572222	55.039722;
                           -7.567500	55.037778;
                           -7.564722	55.035000;
                           -7.560833	55.032500;
                           -7.560278	55.028611;
                           -7.563889	55.025833;
                           -7.566389	55.024167;
                           -7.571389	55.022222;
                           -7.575556	55.020278;
                           -7.580556	55.018611;
                           -7.585556	55.016667;
                           -7.589722	55.014722;
                           -7.594444	55.013056;
                           -7.598611	55.010833;
                           -7.603611	55.009167;
                           -7.607778	55.006944;
                           -7.612778	55.005278;
                           -7.617778	55.003611;
                           -7.623611	55.002222;
                           -7.627778	55.000000;
                           -7.628611	54.996111;
                           -7.630000	54.991944;
                           -7.634167	54.989722;
                           -7.638889	54.988056;
                           -7.642222	54.985278;
                           -7.645000	54.982222;
                           -7.646944	54.978611;
                           -7.648889	54.975000;
                           -7.650556	54.971389;
                           -7.651944	54.967222;
                           -7.653056	54.963333;
                           -7.650833	54.960000;
                           -7.645278	54.960278;
                           -7.641111	54.962500;
                           -7.636944	54.964722;
                           -7.633611	54.967222;
                           -7.629444	54.969444;
                           -7.625278	54.971667;
                           -7.621111	54.973889;
                           -7.617778	54.976389;
                           -7.613889	54.979167;
                           -7.610833	54.981667;
                           -7.607222	54.984444;
                           -7.603889	54.986944;
                           -7.600556	54.989722;
                           -7.597222	54.992500;
                           -7.592778	54.994444;
                           -7.589722	54.997222;
                           -7.585278	54.999444;
                           -7.581111	55.001667;
                           -7.577500	55.004167;
                           -7.573611	55.006389;
                           -7.569444	55.008611;
                           -7.565278	55.010833;
                           -7.560278	55.012500;
                           -7.556111	55.014722;
                           -7.551111	55.016389;
                           -7.545278	55.017500;
                           -7.538889	55.018333;
                           -7.532500	55.018889;
                           -7.529167	55.021667;
                           -7.527222	55.025278;
                           -7.525278	55.028889;
                           -7.521111	55.031111;
                           -7.516111	55.032778;
                           -7.510556	55.033889;
                           -7.504722	55.035278;
                           -7.498889	55.036389;
                           -7.493333	55.037500;
                           -7.486944	55.038333;
                           -7.481111	55.039444;
                           -7.475278	55.040833;
                           -7.470556	55.042500;
                           -7.466389	55.044722;
                           -7.462778	55.047222;
                           -7.464167	55.050833;
                           -7.466944	55.053889;
                           -7.469167	55.056944;
                           -7.469722	55.061111;
                           -7.470000	55.065278;
                           -7.472222	55.068611;
                           -7.474444	55.071944;
                           -7.477222	55.074722;
                           -7.480278	55.077500;
                           -7.483056	55.080278;
                           -7.486111	55.083056;
                           -7.487222	55.086944;
                           -7.488889	55.090556;
                           -7.488333	55.094167;
                           -7.486389	55.097778;
                           -7.482778	55.100556;
                           -7.479444	55.103056;
                           -7.476111	55.105833;
                           -7.474167	55.109444;
                           -7.473889	55.113056;
                           -7.474444	55.117222;
                           -7.474722	55.121389;
                           -7.475000	55.125556;
                           -7.473056	55.129167;
                           -7.472778	55.133611;
                           -7.475000	55.136944;
                           -7.478889	55.139444;
                           -7.483333	55.141389;
                           -7.486389	55.144167;
                           -7.488611	55.147500;
                           -7.489722	55.151111;
                           -7.491944	55.154444;
                           -7.495000	55.157222;
                           -7.498889	55.159722;
                           -7.502500	55.161944;
                           -7.505278	55.164722;
                           -7.508333	55.167778;
                           -7.510556	55.170833;
                           -7.512778	55.174167;
                           -7.515000	55.177500;
                           -7.517778	55.180278;
                           -7.521389	55.182778;
                           -7.525278	55.185000;
                           -7.530833	55.186667;
                           -7.536389	55.188056;
                           -7.542778	55.189167;
                           -7.548056	55.190833;
                           -7.551944	55.193056;
                           -7.553333	55.196944;
                           -7.551389	55.200556;
                           -7.551111	55.205000;
                           -7.550556	55.209722;
                           -7.549444	55.213611;
                           -7.547500	55.217222;
                           -7.545556	55.220833;
                           -7.543056	55.224167;
                           -7.539444	55.226667;
                           -7.534444	55.228333;
                           -7.531111	55.231111;
                           -7.532222	55.234722;
                           -7.536944	55.236667;
                           -7.536667	55.240556;
                           -7.533333	55.243056;
                           -7.528333	55.244722;
                           -7.523056	55.246389;
                           -7.520556	55.249722;
                           -7.519444	55.252778;
                           -7.520000	55.256944;
                           -7.521944	55.260278;
                           -7.525833	55.262500;
                           -7.528889	55.265278;
                           -7.531111	55.268611;
                           -7.531389	55.272778;
                           -7.527222	55.275000;
                           -7.520833	55.275833;
                           -7.514444	55.275556;
                           -7.506389	55.275278;
                           -7.498889	55.274722;
                           -7.491944	55.273889;
                           -7.486111	55.275278;
                           -7.483333	55.278333;
                           -7.485556	55.281667;
                           -7.484444	55.285556;
                           -7.478889	55.286944;
                           -7.473056	55.285278;
                           -7.471111	55.282222;
                           -7.468056	55.279167;
                           -7.465833	55.276944;
                           -7.462222	55.274444;
                           -7.456667	55.273056;
                           -7.450278	55.271944;
                           -7.445278	55.273611;
                           -7.443333	55.277222;
                           -7.440000	55.279722;
                           -7.434167	55.281111;
                           -7.427778	55.281667;
                           -7.421944	55.283056;
                           -7.416389	55.284167;
                           -7.410278	55.285278;
                           -7.404722	55.286667;
                           -7.399722	55.288333;
                           -7.394722	55.290000;
                           -7.391389	55.292500;
                           -7.388611	55.295833;
                           -7.386111	55.298889;
                           -7.384167	55.302500;
                           -7.380278	55.305000;
                           -7.374722	55.306389;
                           -7.366667	55.306111;
                           -7.361111	55.304444;
                           -7.355833	55.303056;
                           -7.351111	55.301111;
                           -7.344722	55.300000;
                           -7.339167	55.298333;
                           -7.334444	55.296389;
                           -7.329722	55.294444;
                           -7.327778	55.291111;
                           -7.329722	55.287500;
                           -7.335556	55.286389;
                           -7.341389	55.285278;
                           -7.348611	55.285000;
                           -7.352778	55.282778;
                           -7.352222	55.279444;
                           -7.347778	55.277500;
                           -7.340278	55.277778;
                           -7.332222	55.277500;
                           -7.324167	55.277222;
                           -7.317778	55.276111;
                           -7.311389	55.275000;
                           -7.306944	55.273056;
                           -7.303333	55.270833;
                           -7.296667	55.271389;
                           -7.291667	55.273056;
                           -7.283611	55.273056;
                           -7.277222	55.271944;
                           -7.271667	55.270278;
                           -7.263611	55.270000;
                           -7.259444	55.272222;
                           -7.260833	55.275833;
                           -7.263611	55.278611;
                           -7.264167	55.282778;
                           -7.266389	55.286111;
                           -7.271667	55.287778;
                           -7.278056	55.288889;
                           -7.284444	55.290000;
                           -7.291667	55.290556;
                           -7.295278	55.293056;
                           -7.300000	55.295000;
                           -7.305556	55.296389;
                           -7.310278	55.298333;
                           -7.315556	55.300000;
                           -7.321944	55.301111;
                           -7.326667	55.303056;
                           -7.329722	55.305833;
                           -7.331111	55.309722;
                           -7.333056	55.312778;
                           -7.335833	55.315833;
                           -7.340833	55.317778;
                           -7.345556	55.319722;
                           -7.349167	55.321944;
                           -7.353056	55.324444;
                           -7.356667	55.326944;
                           -7.359722	55.329722;
                           -7.363611	55.332222;
                           -7.363889	55.335278;
                           -7.357500	55.336111;
                           -7.349722	55.335833;
                           -7.342222	55.335278;
                           -7.336944	55.333611;
                           -7.331389	55.332222;
                           -7.325556	55.332500;
                           -7.327500	55.335556;
                           -7.331667	55.338056;
                           -7.335556	55.340556;
                           -7.340000	55.342500;
                           -7.345556	55.343889;
                           -7.351111	55.345556;
                           -7.355833	55.347500;
                           -7.360278	55.349444;
                           -7.365000	55.351389;
                           -7.371944	55.351944;
                           -7.379444	55.352778;
                           -7.385000	55.354167;
                           -7.388611	55.356667;
                           -7.390833	55.360000;
                           -7.394444	55.362500;
                           -7.399167	55.364444;
                           -7.404722	55.365833;
                           -7.408611	55.368333;
                           -7.406111	55.370556;
                           -7.400278	55.371667;
                           -7.393889	55.372500;
                           -7.386389	55.372778;
                           -7.380000	55.372500;
                           -7.380000	55.372500;
                           -7.371944	55.373056;
                           -7.365556	55.373056;
                           -7.360000	55.371389;
                           -7.354444	55.370000;
                           -7.348056	55.368889;
                           -7.341667	55.367778;
                           -7.337222	55.365833;
                           -7.331667	55.364167;
                           -7.325278	55.363056;
                           -7.318056	55.362500;
                           -7.310833	55.361667;
                           -7.302778	55.361389;
                           -7.294722	55.361389;
                           -7.287500	55.361389;
                           -7.281389	55.360278;
                           -7.276667	55.358333;
                           -7.271944	55.356389;
                           -7.266389	55.355000;
                           -7.259167	55.354167;
                           -7.251944	55.354444;
                           -7.245278	55.354167;
                           -7.240833	55.352222;
                           -7.237778	55.349444;
                           -7.234167	55.346944;
                           -7.229444	55.345000;
                           -7.222778	55.343889;
                           -7.215833	55.343333;
                           -7.209444	55.342222;
                           -7.204722	55.340278;
                           -7.200278	55.338333;
                           -7.195556	55.336111;
                           -7.190833	55.334167;
                           -7.186944	55.331944;
                           -7.182500	55.330000;
                           -7.179444	55.326944;
                           -7.180000	55.322500;
                           -7.180556	55.317778;
                           -7.176389	55.315556;
                           -7.171944	55.313611;
                           -7.167222	55.311389;
                           -7.163611	55.309167;
                           -7.160556	55.306389;
                           -7.158333	55.303056;
                           -7.155556	55.300278;
                           -7.152500	55.297222;
                           -7.149722	55.294444;
                           -7.146667	55.291667;
                           -7.142222	55.289722;
                           -7.134722	55.288889;
                           -7.129444	55.287500;
                           -7.123889	55.285833;
                           -7.118333	55.284444;
                           -7.112778	55.282778;
                           -7.108056	55.280833;
                           -7.102778	55.279167;
                           -7.096389	55.278056;
                           -7.092778	55.275833;
                           -7.088889	55.273333;
                           -7.085833	55.270556;
                           -7.081389	55.268611;
                           -7.077500	55.266111;
                           -7.071389	55.265000;
                           -7.065556	55.266111;
                           -7.061389	55.268333;
                           -7.056111	55.270000;
                           -7.051944	55.272222;
                           -7.047500	55.270278;
                           -7.045278	55.266944;
                           -7.042222	55.264167;
                           -7.035278	55.263333;
                           -7.028611	55.264167;
                           -7.023056	55.262500;
                           -7.020278	55.259722;
                           -7.016389	55.257222;
                           -7.012778	55.254722;
                           -7.008056	55.252778;
                           -7.001667	55.252778;
                           -6.994444	55.252778;
                           -6.988889	55.251389;
                           -6.983333	55.249722;
                           -6.975278	55.249444;
                           -6.969722	55.250556;
                           -6.962500	55.250833;
                           -6.956111	55.249722;
                           -6.951389	55.247778;
                           -6.947778	55.245278;
                           -6.943889	55.243056;
                           -6.938611	55.241389;
                           -6.934444	55.238889;
                           -6.931667	55.236111;
                           -6.932222	55.232500;
                           -6.933333	55.228333;
                           -6.934444	55.224444;
                           -6.935833	55.220278;
                           -6.941389	55.219167;
                           -6.946667	55.217500;
                           -6.950000	55.214722;
                           -6.953333	55.212222;
                           -6.958611	55.210556;
                           -6.964167	55.209167;
                           -6.970000	55.208056;
                           -6.975556	55.206944;
                           -6.980000	55.204722;
                           -6.985000	55.203056;
                           -6.989167	55.200833;
                           -6.995000	55.199722;
                           -6.999722	55.198056;
                           -7.005556	55.196944;
                           -7.012222	55.196111;
                           -7.017778	55.195000;
                           -7.023611	55.193889;
                           -7.028611	55.191944;
                           -7.033333	55.190278;
                           -7.037778	55.188333;
                           -7.042778	55.186667;
                           -7.046944	55.184444;
                           -7.051389	55.182222;
                           -7.055556	55.180000;
                           -7.059444	55.178056;
                           -7.063056	55.177222;
                           -7.067222	55.175000;
                           -7.073056	55.173889;
                           -7.077778	55.172222;
                           -7.081944	55.170000;
                           -7.087222	55.168333;
                           -7.091944	55.166667;
                           -7.096111	55.164444;
                           -7.101389	55.162778;
                           -7.106111	55.161111;
                           -7.111944	55.160000;
                           -7.116944	55.158333;
                           -7.121944	55.156667;
                           -7.127778	55.155278;
                           -7.133333	55.154167;
                           -7.138333	55.152500;
                           -7.144167	55.151389;
                           -7.149167	55.149722;
                           -7.154167	55.148056;
                           -7.158889	55.146389;
                           -7.163333	55.144167;
                           -7.167500	55.141944;
                           -7.171667	55.139444;
                           -7.175833	55.137222;
                           -7.179444	55.134444;
                           -7.183611	55.132500;
                           -7.186944	55.129722;
                           -7.190556	55.126944;
                           -7.194167	55.124444;
                           -7.197500	55.121667;
                           -7.200278	55.118611;
                           -7.203611	55.116111;
                           -7.206389	55.112778;
                           -7.208889	55.109722;
                           -7.211667	55.106667;
                           -7.215000	55.103889;
                           -7.217778	55.100833;
                           -7.221111	55.099167;
                           -7.224722	55.096389;
                           -7.228056	55.093889;
                           -7.230833	55.090833;
                           -7.233333	55.087500;
                           -7.236111	55.084444;
                           -7.239444	55.081944;
                           -7.243889	55.079722;
                           -7.247222	55.076944;
                           -7.250000	55.073889;
                           -7.252778	55.070833;
                           -7.253056	55.066111;
                           -7.253889	55.065833;
                           -7.253889	55.066667;
                           -7.252778	55.063056;
                           -7.250556	55.059722;
                           -7.251667	55.055556;
                           -7.253611	55.051944;
                           -7.255000	55.048056;
                           -7.251111	55.046389;
                           -7.245278	55.046667;
                           -7.239444	55.048056;
                           -7.233889	55.049167;
                           -7.228333	55.050278;
                           -7.221667	55.051111;
                           -7.216111	55.052222;
                           -7.209444	55.053056;
                           -7.203056	55.053611;
                           -7.196667	55.054444;
                           -7.190833	55.055556;
                           -7.183611	55.055833;
                           -7.176389	55.056111;
                           -7.170278	55.055833;
                           -7.163889	55.054722;
                           -7.159167	55.052778;
                           -7.155278	55.050278;
                           -7.150833	55.048333;
                           -7.146944	55.045833;
                           -7.141667	55.043889;
                           -7.136111	55.042500;
                           -7.130556	55.040833;
                           -7.124167	55.039722;
                           -7.118056	55.038611;
                           -7.110833	55.038056;
                           -7.103056	55.037778;
                           -7.095000	55.037500;
                           -7.087778	55.037778;
                           -7.080556	55.037778;
                           -7.073333	55.038056;
                           -7.069444	55.039444;
                           -7.064167	55.041111;
                           -7.060000	55.043333;
                           -7.055278	55.045000;
                           -7.051389	55.047500;
                           -7.047500	55.049722;
                           -7.043889	55.052222;
                           -7.040556	55.055000;
                           -7.037222	55.057500;
                           -7.033333	55.060278;
                           -7.029167	55.062500;
                           -7.025833	55.065000;
                           -7.022500	55.067778;
                           -7.019722	55.070833;
                           -7.019167	55.074444;
                           -7.022222	55.077222;
                           -7.023333	55.081111;
                           -7.020833	55.084167;
                           -7.019444	55.088333;
                           -7.019167	55.092778;
                           -7.016389	55.095833;
                           -7.010833	55.096944;
                           -7.005833	55.098611;
                           -7.002222	55.101389;
                           -6.999444	55.104444;
                           -6.996944	55.107500;
                           -6.994167	55.110833;
                           -6.992222	55.114444;
                           -6.989444	55.117500;
                           -6.986944	55.120556;
                           -6.985000	55.124167;
                           -6.981944	55.127222;
                           -6.980000	55.130833;
                           -6.977222	55.134167;
                           -6.974722	55.137222;
                           -6.972778	55.140833;
                           -6.970833	55.144444;
                           -6.968889	55.148056;
                           -6.967500	55.151944;
                           -6.966944	55.156667;
                           -6.967500	55.160833;
                           -6.966944	55.165278;
                           -6.967222	55.169444;
                           -6.966944	55.174167;
                           -6.967222	55.178056;
                           -6.968611	55.181944;
                           -6.968889	55.186111;
                           -6.967778	55.190000;
                           -6.963611	55.190556;
                           -6.956389	55.190556;
                           -6.950278	55.189444;
                           -6.944444	55.188056;
                           -6.940000	55.186111;
                           -6.935278	55.183889;
                           -6.930556	55.181944;
                           -6.926389	55.180000;
                           -6.921667	55.178056;
                           -6.916944	55.176111;
                           -6.911389	55.174444;
                           -6.906111	55.173056;
                           -6.900556	55.171389;
                           -6.895000	55.169722;
                           -6.889722	55.168333;
                           -6.882500	55.167500;
                           -6.875556	55.166944;
                           -6.867500	55.166667;
                           -6.860000	55.166944;
                           -6.852778	55.166944;
                           -6.846389	55.167778;
                           -6.839167	55.168056;
                           -6.831667	55.168056;
                           -6.824722	55.168333;
                           -6.818889	55.169444;
                           -6.812222	55.170278;
                           -6.806667	55.171389;
                           -6.799444	55.170833;
                           -6.791389	55.170556;
                           -6.784167	55.170556;
                           -6.777778	55.171389;
                           -6.769722	55.171111;
                           -6.764444	55.169444;
                           -6.758889	55.168056;
                           -6.754167	55.167778;
                           -6.750000	55.168889;
                           -6.745556	55.171111;
                           -6.740000	55.172222;
                           -6.735556	55.174444;
                           -6.732222	55.177222;
                           -6.730278	55.180833;
                           -6.725833	55.182778;
                           -6.720833	55.184444;
                           -6.716667	55.186667;
                           -6.713056	55.189167;
                           -6.707500	55.190556;
                           -6.700833	55.191111;
                           -6.696111	55.191111;
                           -6.691944	55.192222;
                           -6.685556	55.193056;
                           -6.678889	55.193611;
                           -6.672500	55.194167;
                           -6.667500	55.195833;
                           -6.665556	55.199444;
                           -6.662778	55.202778;
                           -6.657778	55.204444;
                           -6.651389	55.204167;
                           -6.644167	55.204167;
                           -6.639167	55.205833;
                           -6.632500	55.206667;
                           -6.624444	55.206389;
                           -6.618056	55.206111;
                           -6.613333	55.207778;
                           -6.605833	55.208056;
                           -6.597778	55.207778;
                           -6.592222	55.208889;
                           -6.586389	55.210000;
                           -6.581389	55.211667;
                           -6.577222	55.213889;
                           -6.571389	55.215000;
                           -6.564167	55.214167;
                           -6.557778	55.215000;
                           -6.552778	55.216667;
                           -6.547778	55.218333;
                           -6.543333	55.220278;
                           -6.539444	55.222500;
                           -6.537778	55.226667;
                           -6.537500	55.230278;
                           -6.531667	55.231389;
                           -6.527778	55.231667;
                           -6.521111	55.232500;
                           -6.515278	55.233611;
                           -6.511944	55.236111;
                           -6.509167	55.239444;
                           -6.506389	55.242500;
                           -6.502222	55.244722;
                           -6.496389	55.245833;
                           -6.489167	55.245833;
                           -6.482778	55.244722;
                           -6.478333	55.242778;
                           -6.472778	55.241111;
                           -6.468333	55.239167;
                           -6.462778	55.237778;
                           -6.458333	55.235556;
                           -6.450833	55.235000;
                           -6.443611	55.235000;
                           -6.437222	55.235833;
                           -6.430000	55.235000;
                           -6.424444	55.233611;
                           -6.418333	55.232222;
                           -6.411111	55.232500;
                           -6.404444	55.233056;
                           -6.398611	55.234444;
                           -6.393889	55.236111;
                           -6.388611	55.237778;
                           -6.383056	55.238889;
                           -6.376389	55.239444;
                           -6.368333	55.239167;
                           -6.360278	55.238889;
                           -6.353333	55.238056;
                           -6.348611	55.236111;
                           -6.343333	55.234444;
                           -6.338611	55.232500;
                           -6.333889	55.230556;
                           -6.328611	55.228889;
                           -6.321667	55.228333;
                           -6.314167	55.227500;
                           -6.307222	55.227778;
                           -6.299722	55.226944;
                           -6.295278	55.225000;
                           -6.290833	55.223056;
                           -6.286111	55.220833;
                           -6.283056	55.220000;
                           -6.278611	55.217778;
                           -6.273056	55.216389;
                           -6.266667	55.215000;
                           -6.260556	55.213889;
                           -6.255000	55.212500;
                           -6.251111	55.210000;
                           -6.247500	55.207500;
                           -6.244722	55.204722;
                           -6.240833	55.202222;
                           -6.235000	55.203333;
                           -6.230833	55.205556;
                           -6.225833	55.207222;
                           -6.219167	55.207778;
                           -6.212222	55.208056;
                           -6.206389	55.209167;
                           -6.200556	55.210278;
                           -6.194167	55.210833;
                           -6.187500	55.211667;
                           -6.182500	55.213333;
                           -6.178333	55.215278;
                           -6.173889	55.217500;
                           -6.168333	55.219167;
                           -6.162222	55.220278;
                           -6.155833	55.220833;
                           -6.149444	55.221667;
                           -6.142222	55.220833;
                           -6.137778	55.218889;
                           -6.133889	55.216389;
                           -6.130000	55.213889;
                           -6.125556	55.211944;
                           -6.118333	55.211111;
                           -6.111111	55.211389;
                           -6.103889	55.210556;
                           -6.098611	55.208889;
                           -6.095833	55.206111;
                           -6.092778	55.203333;
                           -6.089167	55.200833;
                           -6.083611	55.199167;
                           -6.076667	55.198611;
                           -6.071111	55.196944;
                           -6.066667	55.195000;
                           -6.063611	55.191944;
                           -6.060833	55.189167;
                           -6.058889	55.185833;
                           -6.056944	55.182500;
                           -6.055000	55.179167;
                           -6.052778	55.176111;
                           -6.050000	55.173056;
                           -6.047222	55.170278;
                           -6.043333	55.167778;
                           -6.040556	55.165000;
                           -6.037500	55.161944;
                           -6.035556	55.158889;
                           -6.033611	55.155556;
                           -6.033333	55.151389;
                           -6.034722	55.147222;
                           -6.036667	55.143611;
                           -6.039444	55.140556;
                           -6.041389	55.136944;
                           -6.042222	55.132500;
                           -6.043333	55.128333;
                           -6.043889	55.123889;
                           -6.043611	55.119722;
                           -6.042500	55.115833;
                           -6.042222	55.111667;
                           -6.041944	55.107500;
                           -6.042500	55.103056;
                           -6.043889	55.098889;
                           -6.045833	55.095278;
                           -6.047778	55.091944;
                           -6.049444	55.087778;
                           -6.051944	55.084722;
                           -6.054167	55.081111;
                           -6.055556	55.076944;
                           -6.056667	55.072778;
                           -6.057500	55.068333;
                           -6.056944	55.064167;
                           -6.056667	55.060000;
                           -6.056389	55.055833;
                           -6.054444	55.052500;
                           -6.048056	55.051389;
                           -6.043889	55.053611;
                           -6.040278	55.056111;
                           -6.036111	55.058333;
                           -6.030278	55.059444;
                           -6.023056	55.059722;
                           -6.016944	55.058333;
                           -6.009722	55.057778;
                           -6.002500	55.057778;
                           -5.995278	55.057222;
                           -5.988889	55.055833;
                           -5.982778	55.054722;
                           -5.978333	55.052778;
                           -5.974722	55.050278;
                           -5.970833	55.047778;
                           -5.968889	55.044444;
                           -5.966667	55.041389;
                           -5.965833	55.037500;
                           -5.965556	55.033333;
                           -5.965000	55.029167;
                           -5.965833	55.024722;
                           -5.966944	55.020556;
                           -5.968333	55.016667;
                           -5.970556	55.013056;
                           -5.972500	55.009444;
                           -5.975278	55.006389;
                           -5.978056	55.003056;
                           -5.981667	55.000556;
                           -5.984444	54.997500;
                           -5.986944	54.994444;
                           -5.990000	54.991389;
                           -5.992778	54.988056;
                           -5.993333	54.983611;
                           -5.990556	54.980833;
                           -5.982500	54.980278;
                           -5.974722	54.980000;
                           -5.966667	54.979722;
                           -5.962778	54.977222;
                           -5.959167	54.975000;
                           -5.955556	54.972500;
                           -5.950278	54.970833;
                           -5.943611	54.971389;
                           -5.935833	54.971111;
                           -5.930278	54.969444;
                           -5.926389	54.967222;
                           -5.922778	54.964722;
                           -5.919167	54.962222;
                           -5.916389	54.959444;
                           -5.913611	54.956389;
                           -5.910556	54.953611;
                           -5.908056	54.950833;
                           -5.905000	54.947778;
                           -5.903056	54.944444;
                           -5.900278	54.941667;
                           -5.896389	54.939167;
                           -5.892778	54.936667;
                           -5.888333	54.934722;
                           -5.886389	54.931389;
                           -5.884167	54.928056;
                           -5.882222	54.925000;
                           -5.880278	54.921667;
                           -5.878333	54.918333;
                           -5.876389	54.915000;
                           -5.874444	54.911667;
                           -5.872500	54.908333;
                           -5.868889	54.905833;
                           -5.865278	54.903611;
                           -5.860000	54.901944;
                           -5.853611	54.900833;
                           -5.848056	54.899167;
                           -5.844444	54.896667;
                           -5.842500	54.893333;
                           -5.841389	54.889722;
                           -5.839444	54.886389;
                           -5.837500	54.883056;
                           -5.834722	54.880278;
                           -5.831111	54.877778;
                           -5.827222	54.875278;
                           -5.822778	54.873333;
                           -5.819722	54.870278;
                           -5.816389	54.867500;
                           -5.814444	54.864167;
                           -5.811389	54.861389;
                           -5.810278	54.857500;
                           -5.809444	54.853889;
                           -5.809722	54.849167;
                           -5.811389	54.845278;
                           -5.812500	54.841111;
                           -5.814444	54.837500;
                           -5.815833	54.833611;
                           -5.817222	54.829444;
                           -5.816111	54.825833;
                           -5.814167	54.822500;
                           -5.811389	54.819444;
                           -5.807500	54.816944;
                           -5.803056	54.815000;
                           -5.798611	54.813056;
                           -5.793333	54.811389;
                           -5.787778	54.809722;
                           -5.780833	54.809167;
                           -5.773611	54.809167;
                           -5.767222	54.809722;
                           -5.759167	54.809444;
                           -5.754722	54.807500;
                           -5.750278	54.805556;
                           -5.746667	54.803056;
                           -5.743056	54.800556;
                           -5.739167	54.798056;
                           -5.735556	54.795556;
                           -5.732778	54.792778;
                           -5.730000	54.790000;
                           -5.726944	54.786944;
                           -5.724444	54.784167;
                           -5.721389	54.781389;
                           -5.717222	54.782500;
                           -5.716667	54.786111;
                           -5.718056	54.789722;
                           -5.720833	54.792778;
                           -5.723611	54.795556;
                           -5.726389	54.798333;
                           -5.730000	54.800833;
                           -5.733611	54.803333;
                           -5.736667	54.806111;
                           -5.740278	54.808611;
                           -5.744722	54.810833;
                           -5.748333	54.813056;
                           -5.751944	54.815556;
                           -5.756667	54.817778;
                           -5.760278	54.820000;
                           -5.764722	54.822222;
                           -5.768333	54.824722;
                           -5.773056	54.826667;
                           -5.776667	54.829167;
                           -5.780278	54.831389;
                           -5.784722	54.833611;
                           -5.788333	54.836111;
                           -5.791389	54.838889;
                           -5.794167	54.841667;
                           -5.795278	54.845556;
                           -5.794722	54.849167;
                           -5.791389	54.851667;
                           -5.786944	54.853889;
                           -5.781944	54.855556;
                           -5.775278	54.856111;
                           -5.769167	54.855000;
                           -5.762222	54.854167;
                           -5.755000	54.854167;
                           -5.746944	54.853889;
                           -5.741667	54.852222;
                           -5.736389	54.850833;
                           -5.731944	54.848611;
                           -5.728889	54.845833;
                           -5.725278	54.843333;
                           -5.722500	54.840556;
                           -5.718056	54.838333;
                           -5.714444	54.836111;
                           -5.710556	54.833611;
                           -5.707778	54.830556;
                           -5.705833	54.827500;
                           -5.703889	54.824167;
                           -5.701111	54.821111;
                           -5.698333	54.818333;
                           -5.695556	54.815556;
                           -5.692500	54.812500;
                           -5.690000	54.809722;
                           -5.688056	54.806389;
                           -5.688611	54.801944;
                           -5.690000	54.797778;
                           -5.690556	54.793333;
                           -5.690278	54.789167;
                           -5.689167	54.785278;
                           -5.688056	54.781667;
                           -5.686944	54.777778;
                           -5.686667	54.773889;
                           -5.686389	54.769722;
                           -5.686111	54.765556;
                           -5.687500	54.761389;
                           -5.689444	54.757778;
                           -5.690556	54.756389;
                           -5.692222	54.755556;
                           -5.693333	54.752500;
                           -5.696389	54.749444;
                           -5.699722	54.746667;
                           -5.703333	54.744167;
                           -5.707500	54.741944;
                           -5.710833	54.739444;
                           -5.716111	54.737778;
                           -5.720278	54.735833;
                           -5.724722	54.733611;
                           -5.728056	54.731111;
                           -5.731389	54.728333;
                           -5.735000	54.725833;
                           -5.738611	54.723333;
                           -5.743611	54.721667;
                           -5.750000	54.721111;
                           -5.756944	54.720833;
                           -5.765000	54.721111;
                           -5.772222	54.721111;
                           -5.779444	54.720833;
                           -5.785833	54.720278;
                           -5.790000	54.718056;
                           -5.793611	54.715556;
                           -5.797778	54.713333;
                           -5.803333	54.712222;
                           -5.810000	54.711667;
                           -5.816389	54.711111;
                           -5.821389	54.709444;
                           -5.825000	54.706667;
                           -5.829167	54.704722;
                           -5.832778	54.701944;
                           -5.836944	54.700000;
                           -5.841667	54.698333;
                           -5.846111	54.696111;
                           -5.851944	54.695000;
                           -5.856667	54.693333;
                           -5.862500	54.692222;
                           -5.866667	54.690278;
                           -5.870833	54.688056;
                           -5.874444	54.685556;
                           -5.878056	54.682778;
                           -5.881667	54.680278;
                           -5.885000	54.677500;
                           -5.888333	54.675000;
                           -5.891389	54.671944;
                           -5.893333	54.668333;
                           -5.896111	54.665278;
                           -5.898056	54.661667;
                           -5.900833	54.658611;
                           -5.903611	54.655556;
                           -5.906389	54.652500;
                           -5.910000	54.649722;
                           -5.912500	54.646667;
                           -5.915556	54.643611;
                           -5.918333	54.640556;
                           -5.920833	54.637500;
                           -5.923889	54.634167;
                           -5.925000	54.630278;
                           -5.922222	54.627222;
                           -5.916944	54.625833;
                           -5.912222	54.623611;
                           -5.909722	54.620833;
                           -5.908333	54.617222;
                           -5.909722	54.613056;
                           -5.910278	54.608611;
                           -5.908333	54.605278;
                           -5.904722	54.602778;
                           -5.898333	54.601667;
                           -5.892778	54.602778;
                           -5.887222	54.603889;
                           -5.882778	54.605833;
                           -5.880000	54.609167;
                           -5.878056	54.612500;
                           -5.876111	54.616111;
                           -5.874722	54.620278;
                           -5.870278	54.622778;
                           -5.865556	54.624444;
                           -5.861944	54.627222;
                           -5.860000	54.630556;
                           -5.858611	54.634722;
                           -5.855833	54.637778;
                           -5.851111	54.639444;
                           -5.845833	54.641111;
                           -5.841667	54.643333;
                           -5.836667	54.644722;
                           -5.833333	54.647500;
                           -5.829167	54.649444;
                           -5.824722	54.651667;
                           -5.820556	54.653889;
                           -5.816389	54.655833;
                           -5.811389	54.657500;
                           -5.807222	54.659722;
                           -5.803056	54.661667;
                           -5.797778	54.663333;
                           -5.793056	54.665000;
                           -5.786667	54.665556;
                           -5.780000	54.666389;
                           -5.773611	54.666944;
                           -5.767222	54.667500;
                           -5.763056	54.669722;
                           -5.759444	54.672222;
                           -5.756667	54.675278;
                           -5.752500	54.676667;
                           -5.746111	54.677222;
                           -5.738889	54.676389;
                           -5.733611	54.674722;
                           -5.728333	54.673333;
                           -5.723056	54.671667;
                           -5.716111	54.670833;
                           -5.708611	54.671111;
                           -5.703333	54.669444;
                           -5.697222	54.668333;
                           -5.692500	54.667222;
                           -5.685556	54.667222;
                           -5.680000	54.668333;
                           -5.674722	54.670000;
                           -5.670000	54.671667;
                           -5.663333	54.672222;
                           -5.658056	54.670556;
                           -5.651111	54.669722;
                           -5.645278	54.671111;
                           -5.641944	54.673611;
                           -5.638056	54.676111;
                           -5.634722	54.678889;
                           -5.629167	54.680000;
                           -5.621111	54.679444;
                           -5.613333	54.679167;
                           -5.605556	54.678889;
                           -5.598889	54.679444;
                           -5.591111	54.679167;
                           -5.583333	54.678889;
                           -5.576944	54.678611;
                           -5.571667	54.676944;
                           -5.569444	54.673611;
                           -5.566667	54.670556;
                           -5.565000	54.667500;
                           -5.563056	54.664167;
                           -5.559444	54.661667;
                           -5.553889	54.660000;
                           -5.547778	54.658889;
                           -5.543333	54.656667;
                           -5.538889	54.654722;
                           -5.535278	54.652222;
                           -5.533333	54.648889;
                           -5.532222	54.645278;
                           -5.531111	54.641667;
                           -5.530000	54.637778;
                           -5.528889	54.634167;
                           -5.527778	54.630278;
                           -5.527500	54.626111;
                           -5.528056	54.621667;
                           -5.527778	54.617500;
                           -5.526944	54.613889;
                           -5.525000	54.610556;
                           -5.523056	54.607222;
                           -5.521111	54.603889;
                           -5.518333	54.601111;
                           -5.515278	54.598056;
                           -5.513333	54.594722;
                           -5.510833	54.591944;
                           -5.508056	54.589167;
                           -5.504444	54.586667;
                           -5.500833	54.584167;
                           -5.496944	54.581667;
                           -5.493333	54.579167;
                           -5.488889	54.577222;
                           -5.485278	54.574722;
                           -5.481667	54.572222;
                           -5.479722	54.568889;
                           -5.477778	54.565556;
                           -5.475833	54.562500;
                           -5.474722	54.558611;
                           -5.475278	54.554167;
                           -5.476111	54.549444;
                           -5.477500	54.545556;
                           -5.478889	54.541389;
                           -5.479444	54.536944;
                           -5.477500	54.533611;
                           -5.474722	54.530556;
                           -5.473611	54.526944;
                           -5.476667	54.523889;
                           -5.477778	54.519722;
                           -5.477500	54.515556;
                           -5.474722	54.512778;
                           -5.470278	54.510833;
                           -5.467500	54.507778;
                           -5.465556	54.504444;
                           -5.464722	54.500833;
                           -5.462778	54.497500;
                           -5.458889	54.495000;
                           -5.454722	54.493056;
                           -5.448333	54.491667;
                           -5.441389	54.491111;
                           -5.435278	54.489722;
                           -5.430556	54.487778;
                           -5.429722	54.484167;
                           -5.430833	54.480000;
                           -5.434444	54.477500;
                           -5.438056	54.474722;
                           -5.439444	54.470833;
                           -5.438889	54.467500;
                           -5.436389	54.464722;
                           -5.435278	54.460833;
                           -5.436389	54.456667;
                           -5.440000	54.454167;
                           -5.443611	54.451667;
                           -5.446944	54.448889;
                           -5.449167	54.445556;
                           -5.450556	54.441389;
                           -5.453333	54.438333;
                           -5.457500	54.436111;
                           -5.461667	54.434167;
                           -5.466111	54.431944;
                           -5.468889	54.428889;
                           -5.470833	54.425278;
                           -5.468889	54.421944;
                           -5.466111	54.419167;
                           -5.461667	54.417222;
                           -5.458056	54.414722;
                           -5.455278	54.411667;
                           -5.455833	54.407222;
                           -5.457222	54.403056;
                           -5.458611	54.399167;
                           -5.459167	54.394444;
                           -5.460556	54.390278;
                           -5.461111	54.385833;
                           -5.463889	54.382778;
                           -5.468889	54.381111;
                           -5.473889	54.379444;
                           -5.478889	54.378056;
                           -5.482778	54.375833;
                           -5.487222	54.373611;
                           -5.489167	54.370278;
                           -5.492222	54.366944;
                           -5.489167	54.364167;
                           -5.484167	54.362500;
                           -5.478889	54.360833;
                           -5.475000	54.358611;
                           -5.478056	54.355278;
                           -5.482778	54.353889;
                           -5.486944	54.351667;
                           -5.491389	54.349167;
                           -5.494167	54.346111;
                           -5.494444	54.344167;
                           -5.495833	54.340278;
                           -5.496944	54.336111;
                           -5.498889	54.332500;
                           -5.504167	54.330833;
                           -5.511111	54.330833;
                           -5.516389	54.332500;
                           -5.520833	54.334444;
                           -5.524444	54.336944;
                           -5.527222	54.339722;
                           -5.528333	54.343611;
                           -5.529167	54.347222;
                           -5.531389	54.350556;
                           -5.535833	54.352500;
                           -5.539444	54.355000;
                           -5.541389	54.358333;
                           -5.542500	54.361944;
                           -5.542778	54.366111;
                           -5.543611	54.370000;
                           -5.546389	54.372778;
                           -5.550000	54.375278;
                           -5.553611	54.377778;
                           -5.557500	54.380000;
                           -5.561111	54.382500;
                           -5.563611	54.385556;
                           -5.567500	54.388056;
                           -5.570000	54.390833;
                           -5.573056	54.393611;
                           -5.574722	54.396944;
                           -5.575833	54.400833;
                           -5.576111	54.404722;
                           -5.576389	54.408889;
                           -5.575000	54.413056;
                           -5.573611	54.416111;
                           -5.571667	54.419722;
                           -5.569167	54.422778;
                           -5.565556	54.425556;
                           -5.561944	54.428056;
                           -5.558333	54.430556;
                           -5.555000	54.433333;
                           -5.551389	54.435833;
                           -5.548611	54.438889;
                           -5.545833	54.441944;
                           -5.543889	54.445556;
                           -5.541667	54.449167;
                           -5.540556	54.453333;
                           -5.540833	54.457500;
                           -5.541667	54.461111;
                           -5.543056	54.465000;
                           -5.543333	54.468889;
                           -5.543333	54.473056;
                           -5.543333	54.477222;
                           -5.544722	54.481111;
                           -5.545556	54.484722;
                           -5.546944	54.488333;
                           -5.548889	54.491667;
                           -5.549722	54.495556;
                           -5.550000	54.499722;
                           -5.551944	54.503056;
                           -5.555556	54.505278;
                           -5.558333	54.508333;
                           -5.559444	54.511944;
                           -5.559722	54.516111;
                           -5.561389	54.519444;
                           -5.564444	54.522222;
                           -5.568056	54.524722;
                           -5.574444	54.525000;
                           -5.579444	54.523333;
                           -5.585556	54.523611;
                           -5.590000	54.525833;
                           -5.591944	54.529167;
                           -5.593056	54.532778;
                           -5.594167	54.536389;
                           -5.596944	54.539444;
                           -5.599722	54.542222;
                           -5.604167	54.544167;
                           -5.609722	54.545833;
                           -5.612500	54.547500;
                           -5.616944	54.549444;
                           -5.621389	54.551389;
                           -5.626111	54.553611;
                           -5.630278	54.555556;
                           -5.635000	54.557500;
                           -5.639444	54.559722;
                           -5.644722	54.561111;
                           -5.649167	54.563333;
                           -5.654444	54.565000;
                           -5.658889	54.566944;
                           -5.664444	54.568611;
                           -5.669722	54.570278;
                           -5.675000	54.571667;
                           -5.680556	54.573333;
                           -5.686389	54.574444;
                           -5.692500	54.575833;
                           -5.698611	54.574722;
                           -5.700556	54.571111;
                           -5.698611	54.567778;
                           -5.695833	54.564722;
                           -5.693889	54.561667;
                           -5.692500	54.557778;
                           -5.692500	54.553611;
                           -5.693889	54.549722;
                           -5.696667	54.546389;
                           -5.700000	54.543889;
                           -5.704444	54.541944;
                           -5.707778	54.539167;
                           -5.711111	54.536667;
                           -5.712500	54.534444;
                           -5.704444	54.533889;
                           -5.697500	54.533333;
                           -5.691389	54.531944;
                           -5.687778	54.529722;
                           -5.686389	54.525833;
                           -5.683889	54.523056;
                           -5.678333	54.521389;
                           -5.672500	54.520278;
                           -5.666111	54.518889;
                           -5.661667	54.516944;
                           -5.660278	54.514167;
                           -5.666111	54.513056;
                           -5.671944	54.511944;
                           -5.676111	54.509722;
                           -5.678611	54.506667;
                           -5.680278	54.502778;
                           -5.681389	54.498611;
                           -5.678611	54.495833;
                           -5.671667	54.495833;
                           -5.665000	54.496389;
                           -5.658611	54.497222;
                           -5.652500	54.495833;
                           -5.648056	54.493889;
                           -5.645278	54.491111;
                           -5.643333	54.487778;
                           -5.645278	54.485000;
                           -5.648056	54.481944;
                           -5.646944	54.478333;
                           -5.648889	54.474722;
                           -5.650278	54.470556;
                           -5.647500	54.468611;
                           -5.642778	54.466667;
                           -5.642778	54.462500;
                           -5.640833	54.459167;
                           -5.634444	54.458889;
                           -5.627500	54.458889;
                           -5.623889	54.456667;
                           -5.625000	54.452500;
                           -5.628611	54.450000;
                           -5.631944	54.447222;
                           -5.635556	54.444722;
                           -5.631944	54.442222;
                           -5.629167	54.439444;
                           -5.630000	54.434722;
                           -5.632500	54.431667;
                           -5.633889	54.428611;
                           -5.630833	54.425556;
                           -5.630556	54.421389;
                           -5.633611	54.418333;
                           -5.636944	54.415833;
                           -5.639722	54.412778;
                           -5.639444	54.408611;
                           -5.640000	54.404167;
                           -5.641389	54.400000;
                           -5.643611	54.396389;
                           -5.646111	54.393333;
                           -5.648889	54.390278;
                           -5.652500	54.387500;
                           -5.656111	54.385000;
                           -5.658611	54.381944;
                           -5.661667	54.378889;
                           -5.665000	54.376111;
                           -5.669167	54.374167;
                           -5.673611	54.371944;
                           -5.678333	54.370278;
                           -5.684167	54.369167;
                           -5.688333	54.367222;
                           -5.692500	54.365000;
                           -5.696111	54.362500;
                           -5.698611	54.359444;
                           -5.701389	54.356389;
                           -5.705000	54.353611;
                           -5.707778	54.350556;
                           -5.710556	54.347500;
                           -5.708611	54.345000;
                           -5.702778	54.346111;
                           -5.699444	54.346944;
                           -5.695278	54.349167;
                           -5.691111	54.351111;
                           -5.687778	54.353889;
                           -5.684167	54.356389;
                           -5.679167	54.358056;
                           -5.674444	54.359722;
                           -5.668611	54.360833;
                           -5.662222	54.361389;
                           -5.655833	54.361944;
                           -5.649444	54.362778;
                           -5.643889	54.363889;
                           -5.638889	54.365556;
                           -5.635556	54.368056;
                           -5.630278	54.369722;
                           -5.623889	54.370278;
                           -5.618333	54.371389;
                           -5.613611	54.373056;
                           -5.610000	54.375556;
                           -5.605833	54.377778;
                           -5.600000	54.378889;
                           -5.594444	54.380000;
                           -5.589444	54.381667;
                           -5.584444	54.383056;
                           -5.578889	54.383333;
                           -5.573611	54.381667;
                           -5.571667	54.378611;
                           -5.569722	54.375278;
                           -5.568889	54.371389;
                           -5.568611	54.367222;
                           -5.565556	54.366389;
                           -5.561944	54.368889;
                           -5.557778	54.370000;
                           -5.554167	54.367500;
                           -5.553333	54.363889;
                           -5.551389	54.360556;
                           -5.549444	54.357222;
                           -5.547500	54.353889;
                           -5.545278	54.350556;
                           -5.544444	54.346944;
                           -5.543333	54.343333;
                           -5.543333	54.339167;
                           -5.543611	54.334444;
                           -5.544167	54.330000;
                           -5.544167	54.325833;
                           -5.541389	54.323056;
                           -5.538611	54.320000;
                           -5.535833	54.317222;
                           -5.533889	54.313889;
                           -5.536667	54.310833;
                           -5.540833	54.308611;
                           -5.545278	54.306667;
                           -5.548611	54.303889;
                           -5.551389	54.300833;
                           -5.554722	54.298333;
                           -5.556944	54.294722;
                           -5.558889	54.291111;
                           -5.560278	54.286944;
                           -5.562222	54.283611;
                           -5.565556	54.280833;
                           -5.570000	54.278889;
                           -5.574167	54.276667;
                           -5.577500	54.274167;
                           -5.580556	54.271111;
                           -5.583333	54.267778;
                           -5.586667	54.265278;
                           -5.591667	54.263611;
                           -5.595833	54.261667;
                           -5.598611	54.258611;
                           -5.599722	54.254444;
                           -5.603611	54.251944;
                           -5.608333	54.250278;
                           -5.615556	54.250000;
                           -5.621667	54.251389;
                           -5.626944	54.252778;
                           -5.631111	54.250833;
                           -5.632222	54.246667;
                           -5.634444	54.243056;
                           -5.636389	54.239444;
                           -5.639167	54.236389;
                           -5.642222	54.233333;
                           -5.644722	54.230278;
                           -5.648056	54.227778;
                           -5.653056	54.226111;
                           -5.658333	54.226667;
                           -5.662222	54.229167;
                           -5.664167	54.232500;
                           -5.666667	54.235278;
                           -5.672222	54.236944;
                           -5.677500	54.238611;
                           -5.680833	54.241111;
                           -5.684444	54.243611;
                           -5.690000	54.245000;
                           -5.696944	54.245833;
                           -5.703889	54.245833;
                           -5.710556	54.245000;
                           -5.718333	54.245556;
                           -5.724444	54.246667;
                           -5.730556	54.247778;
                           -5.738333	54.248056;
                           -5.746111	54.248611;
                           -5.752500	54.247778;
                           -5.758056	54.246667;
                           -5.763056	54.245000;
                           -5.767778	54.243611;
                           -5.774167	54.242778;
                           -5.781389	54.242778;
                           -5.788333	54.242500;
                           -5.796111	54.242778;
                           -5.803333	54.242778;
                           -5.809722	54.242222;
                           -5.816111	54.241389;
                           -5.822500	54.240833;
                           -5.827500	54.239167;
                           -5.830833	54.236667;
                           -5.835000	54.234444;
                           -5.839167	54.232222;
                           -5.844167	54.230833;
                           -5.849722	54.229722;
                           -5.855556	54.228333;
                           -5.859722	54.226389;
                           -5.863889	54.224167;
                           -5.866389	54.221111;
                           -5.868333	54.218889;
                           -5.868333	54.218611;
                           -5.870000	54.216667;
                           -5.873056	54.213611;
                           -5.873611	54.209167;
                           -5.873889	54.205278;
                           -5.872778	54.201667;
                           -5.871944	54.198056;
                           -5.870000	54.194722;
                           -5.868056	54.191389;
                           -5.865833	54.188056;
                           -5.864722	54.184444;
                           -5.864444	54.180278;
                           -5.864167	54.176111;
                           -5.863889	54.171944;
                           -5.864444	54.167500;
                           -5.865278	54.162778;
                           -5.867222	54.159167;
                           -5.868333	54.155278;
                           -5.871389	54.151944;
                           -5.873333	54.148333;
                           -5.875278	54.145000;
                           -5.877222	54.141389;
                           -5.879167	54.137778;
                           -5.881389	54.134167;
                           -5.882500	54.130000;
                           -5.883056	54.125556;
                           -5.883889	54.120833;
                           -5.885000	54.116944;
                           -5.887778	54.113889;
                           -5.890278	54.110556;
                           -5.893333	54.107500;
                           -5.894444	54.103611;
                           -5.896389	54.100000;
                           -5.900000	54.097222;
                           -5.903333	54.094722;
                           -5.907500	54.092500;
                           -5.911944	54.090556;
                           -5.916111	54.088333;
                           -5.920278	54.086111;
                           -5.925000	54.084722;
                           -5.930000	54.083056;
                           -5.934722	54.081389;
                           -5.940556	54.080278;
                           -5.945278	54.078611;
                           -5.949444	54.076389;
                           -5.953611	54.074444;
                           -5.956944	54.071667;
                           -5.960556	54.069167;
                           -5.963889	54.066389;
                           -5.968333	54.064444;
                           -5.973056	54.062778;
                           -5.979167	54.062222;
                           -5.985000	54.060833;
                           -5.990833	54.059722;
                           -5.995278	54.058333;
                           -5.999444	54.056111;
                           -6.003889	54.053889;
                           -6.007222	54.051389;
                           -6.010833	54.048611;
                           -6.014167	54.046111;
                           -6.017500	54.043611;
                           -6.023056	54.042500;
                           -6.028889	54.041111;
                           -6.033056	54.039167;
                           -6.036389	54.036389;
                           -6.039722	54.033889;
                           -6.043889	54.031667;
                           -6.050278	54.031111;
                           -6.057500	54.030833;
                           -6.063056	54.029722;
                           -6.067778	54.028056;
                           -6.075000	54.028056;
                           -6.081667	54.028611;
                           -6.086111	54.030833;
                           -6.090833	54.032778;
                           -6.096944	54.033889;
                           -6.103056	54.035000;
                           -6.108056	54.036667;
                           -6.111944	54.039167;
                           -6.113889	54.042500;
                           -6.113333	54.046111;
                           -6.110000	54.048611;
                           -6.104167	54.049722;
                           -6.099722	54.050556;
                           -6.092500	54.050833;
                           -6.086111	54.051389;
                           -6.081944	54.053611;
                           -6.083889	54.056944;
                           -6.089444	54.058333;
                           -6.094444	54.060000;
                           -6.100556	54.061111;
                           -6.105833	54.062778;
                           -6.111944	54.063889;
                           -6.120000	54.064167;
                           -6.127778	54.064444;
                           -6.135556	54.065000;
                           -6.140556	54.066389;
                           -6.146667	54.067778;
                           -6.153889	54.067500;
                           -6.161667	54.067778;
                           -6.169444	54.068056;
                           -6.177222	54.068333;
                           -6.183333	54.069444;
                           -6.187778	54.071667;
                           -6.189722	54.075000;
                           -6.190833	54.078611;
                           -6.191944	54.082222;
                           -6.193889	54.085556;
                           -6.197500	54.088056;
                           -6.201944	54.090000;
                           -6.208333	54.091111;
                           -6.214444	54.092500;
                           -6.222222	54.092778;
                           -6.228889	54.093333;
                           -6.236111	54.093333;
                           -6.242500	54.092500;
                           -6.250278	54.092778;
                           -6.258056	54.093333;
                           -6.263333	54.093056;
                           -6.266111	54.089722;
                           -6.265000	54.086111;
                           -6.262222	54.083333;
                           -6.258611	54.080833;
                           -6.255000	54.078333;
                           -6.250556	54.075833;
                           -6.246111	54.073889;
                           -6.242500	54.071389;
                           -6.237222	54.070000;
                           -6.231944	54.068333;
                           -6.226667	54.066667;
                           -6.220556	54.065556;
                           -6.212778	54.065278;
                           -6.208333	54.063333;
                           -6.207222	54.059444;
                           -6.206667	54.055278;
                           -6.204722	54.052222;
                           -6.201111	54.049722;
                           -6.196667	54.047500;
                           -6.192500	54.045556;
                           -6.188056	54.043611;
                           -6.186111	54.040278;
                           -6.184167	54.036944;
                           -6.180556	54.034722;
                           -6.176111	54.032500;
                           -6.170556	54.031111;
                           -6.164444	54.029722;
                           -6.156667	54.029444;
                           -6.150278	54.030278;
                           -6.143333	54.030278;
                           -6.138889	54.028333;
                           -6.136111	54.025556;
                           -6.133333	54.022500;
                           -6.130556	54.019722;
                           -6.127778	54.016944;
                           -6.125000	54.013889;
                           -6.121389	54.011667;
                           -6.118611	54.008611;
                           -6.117500	54.005000;
                           -6.118056	54.000278;
                           -6.120556	53.997222;
                           -6.124722	53.995278;
                           -6.128333	53.992500;
                           -6.133056	53.990833;
                           -6.137500	53.988889;
                           -6.142222	53.987222;
                           -6.146944	53.985556;
                           -6.152500	53.984444;
                           -6.158333	53.983333;
                           -6.163889	53.982222;
                           -6.170278	53.981389;
                           -6.177222	53.981111;
                           -6.184444	53.981111;
                           -6.191389	53.980833;
                           -6.198333	53.981667;
                           -6.205000	53.982500;
                           -6.211111	53.983889;
                           -6.217222	53.985278;
                           -6.222778	53.986667;
                           -6.228056	53.988333;
                           -6.233056	53.990000;
                           -6.239167	53.991667;
                           -6.244722	53.993056;
                           -6.249722	53.994722;
                           -6.255833	53.995833;
                           -6.261111	53.997500;
                           -6.266389	53.999167;
                           -6.271667	54.000556;
                           -6.277222	54.002222;
                           -6.283333	54.003333;
                           -6.288333	54.005000;
                           -6.293611	54.006667;
                           -6.298056	54.008611;
                           -6.305278	54.009167;
                           -6.311944	54.010000;
                           -6.318056	54.011111;
                           -6.324167	54.012222;
                           -6.330278	54.013611;
                           -6.337500	54.014167;
                           -6.344167	54.015000;
                           -6.350556	54.014167;
                           -6.355556	54.012500;
                           -6.358889	54.010000;
                           -6.355833	54.007222;
                           -6.351667	54.005000;
                           -6.347222	54.003056;
                           -6.343611	54.000556;
                           -6.342222	53.996944;
                           -6.345000	53.993889;
                           -6.349167	53.991667;
                           -6.353889	53.990000;
                           -6.358056	53.988056;
                           -6.362222	53.985833;
                           -6.366389	53.983611;
                           -6.370000	53.981111;
                           -6.370278	53.976389;
                           -6.370833	53.971944;
                           -6.371667	53.967500;
                           -6.372500	53.963333;
                           -6.375556	53.960278;
                           -6.378056	53.956944;
                           -6.380833	53.953889;
                           -6.381944	53.950000;
                           -6.380000	53.946667;
                           -6.378889	53.942778;
                           -6.378611	53.938889;
                           -6.377500	53.935000;
                           -6.378056	53.930556;
                           -6.378333	53.925833;
                           -6.379722	53.921944;
                           -6.380833	53.917778;
                           -6.380556	53.913611;
                           -6.379444	53.910000;
                           -6.377500	53.906667;
                           -6.375556	53.903333;
                           -6.373611	53.900000;
                           -6.370833	53.897222;
                           -6.368056	53.894167;
                           -6.365278	53.891389;
                           -6.362222	53.888611;
                           -6.358611	53.886111;
                           -6.355278	53.883611;
                           -6.351667	53.881389;
                           -6.347222	53.879167;
                           -6.345000	53.876944;
                           -6.340556	53.875000;
                           -6.336944	53.872500;
                           -6.331667	53.870833;
                           -6.325556	53.869722;
                           -6.317778	53.869444;
                           -6.311389	53.870000;
                           -6.304722	53.869444;
                           -6.300000	53.867222;
                           -6.295556	53.865278;
                           -6.289444	53.864167;
                           -6.281944	53.863889;
                           -6.275556	53.864444;
                           -6.267778	53.864167;
                           -6.262778	53.862778;
                           -6.258333	53.860556;
                           -6.251111	53.860000;
                           -6.244444	53.859167;
                           -6.243889	53.855833;
                           -6.246667	53.852778;
                           -6.246944	53.848333;
                           -6.247500	53.843611;
                           -6.249444	53.840000;
                           -6.252222	53.836944;
                           -6.255000	53.833889;
                           -6.256944	53.830278;
                           -6.258333	53.826111;
                           -6.256389	53.823056;
                           -6.254444	53.819722;
                           -6.254722	53.816111;
                           -6.255833	53.811944;
                           -6.255556	53.807778;
                           -6.253611	53.804444;
                           -6.250000	53.801944;
                           -6.245000	53.800556;
                           -6.239444	53.798889;
                           -6.234444	53.797222;
                           -6.228333	53.796111;
                           -6.222778	53.794444;
                           -6.221944	53.790833;
                           -6.226111	53.788611;
                           -6.230833	53.786944;
                           -6.235556	53.785556;
                           -6.239722	53.783333;
                           -6.240556	53.778611;
                           -6.240000	53.774722;
                           -6.241111	53.770556;
                           -6.242778	53.766389;
                           -6.244444	53.762778;
                           -6.245833	53.758889;
                           -6.246944	53.754722;
                           -6.246667	53.750556;
                           -6.247222	53.746111;
                           -6.247778	53.741389;
                           -6.248333	53.736944;
                           -6.249444	53.732778;
                           -6.250833	53.728611;
                           -6.251111	53.724167;
                           -6.251111	53.720000;
                           -6.248333	53.717222;
                           -6.244722	53.714722;
                           -6.242778	53.711389;
                           -6.241389	53.707778;
                           -6.241111	53.703611;
                           -6.241667	53.698889;
                           -6.242222	53.694444;
                           -6.242778	53.689722;
                           -6.242500	53.685833;
                           -6.241389	53.681944;
                           -6.240278	53.678333;
                           -6.238333	53.675000;
                           -6.236389	53.671667;
                           -6.233611	53.668889;
                           -6.230833	53.665833;
                           -6.228056	53.663056;
                           -6.226111	53.659722;
                           -6.224167	53.656389;
                           -6.222222	53.653333;
                           -6.221944	53.649167;
                           -6.220000	53.645833;
                           -6.218056	53.642500;
                           -6.216111	53.639167;
                           -6.214167	53.635833;
                           -6.209722	53.633889;
                           -6.204722	53.632222;
                           -6.198611	53.631111;
                           -6.191667	53.630556;
                           -6.185556	53.629167;
                           -6.183611	53.626111;
                           -6.182500	53.622222;
                           -6.181389	53.618611;
                           -6.180278	53.614722;
                           -6.177500	53.611944;
                           -6.174444	53.610833;
                           -6.169167	53.609444;
                           -6.164722	53.607222;
                           -6.161389	53.605000;
                           -6.158333	53.601944;
                           -6.156389	53.598611;
                           -6.153889	53.595833;
                           -6.151944	53.593611;
                           -6.148889	53.590556;
                           -6.144444	53.588611;
                           -6.137778	53.587778;
                           -6.130000	53.587500;
                           -6.123889	53.586389;
                           -6.119722	53.584444;
                           -6.114444	53.582778;
                           -6.108056	53.583333;
                           -6.101111	53.583611;
                           -6.099167	53.580278;
                           -6.099722	53.575833;
                           -6.096944	53.572778;
                           -6.092500	53.570833;
                           -6.088056	53.568889;
                           -6.084722	53.566389;
                           -6.081111	53.563889;
                           -6.080000	53.560278;
                           -6.079722	53.556111;
                           -6.078611	53.552500;
                           -6.077500	53.548611;
                           -6.077222	53.544444;
                           -6.076944	53.540278;
                           -6.078056	53.536389;
                           -6.080000	53.532778;
                           -6.080556	53.529167;
                           -6.077778	53.526111;
                           -6.078889	53.523056;
                           -6.082222	53.520556;
                           -6.086389	53.518333;
                           -6.090556	53.516111;
                           -6.094722	53.514167;
                           -6.098611	53.511944;
                           -6.101944	53.509444;
                           -6.103889	53.505833;
                           -6.105556	53.501667;
                           -6.103611	53.498333;
                           -6.100556	53.495556;
                           -6.100278	53.491389;
                           -6.103056	53.488333;
                           -6.107778	53.486667;
                           -6.111389	53.483889;
                           -6.114722	53.481389;
                           -6.117500	53.478333;
                           -6.119444	53.474722;
                           -6.121944	53.471667;
                           -6.123889	53.468056;
                           -6.126667	53.465000;
                           -6.129444	53.461944;
                           -6.131944	53.458611;
                           -6.135278	53.456111;
                           -6.139444	53.453889;
                           -6.143611	53.451944;
                           -6.144167	53.448333;
                           -6.139722	53.446111;
                           -6.134167	53.444722;
                           -6.130000	53.442500;
                           -6.127222	53.439722;
                           -6.125278	53.436389;
                           -6.126111	53.431944;
                           -6.126389	53.427222;
                           -6.126111	53.423056;
                           -6.124167	53.420000;
                           -6.122222	53.416667;
                           -6.119722	53.413611;
                           -6.117500	53.410556;
                           -6.115833	53.407222;
                           -6.116111	53.402500;
                           -6.118889	53.399444;
                           -6.123056	53.397222;
                           -6.122500	53.394167;
                           -6.118056	53.392222;
                           -6.111944	53.391944;
                           -6.105278	53.391111;
                           -6.098056	53.391389;
                           -6.090556	53.390833;
                           -6.083056	53.390556;
                           -6.076111	53.390000;
                           -6.069444	53.389167;
                           -6.063333	53.388056;
                           -6.057222	53.386944;
                           -6.052778	53.384722;
                           -6.050000	53.381944;
                           -6.049444	53.380556;
                           -6.050000	53.380278];

  uki(81).xy = [                 -9.959722	54.009722;
                                 -9.955000	54.007500;
                                 -9.951111	54.005278;
                                 -9.947222	54.002778;
                                 -9.944444	54.000000;
                                 -9.940556	53.997778;
                                 -9.938333	53.994444;
                                 -9.936111	53.991389;
                                 -9.938611	53.988056;
                                 -9.943056	53.986389;
                                 -9.949167	53.985556;
                                 -9.955556	53.985556;
                                 -9.957222	53.981944;
                                 -9.953333	53.979722;
                                 -9.951111	53.976389;
                                 -9.954444	53.973611;
                                 -9.956667	53.970556;
                                 -9.955278	53.966667;
                                 -9.953889	53.963056;
                                 -9.956944	53.960278;
                                 -9.959444	53.957222;
                                 -9.962778	53.954444;
                                 -9.964167	53.950833;
                                 -9.961944	53.947500;
                                 -9.955833	53.947500;
                                 -9.951944	53.949722;
                                 -9.949444	53.953056;
                                 -9.945556	53.955278;
                                 -9.938611	53.955556;
                                 -9.932222	53.954722;
                                 -9.926111	53.953611;
                                 -9.923889	53.950556;
                                 -9.923889	53.945833;
                                 -9.926389	53.942778;
                                 -9.930278	53.940556;
                                 -9.933333	53.937778;
                                 -9.935833	53.934444;
                                 -9.936111	53.930833;
                                 -9.931389	53.929167;
                                 -9.926111	53.927500;
                                 -9.926111	53.923889;
                                 -9.929167	53.921111;
                                 -9.932500	53.918611;
                                 -9.934722	53.915278;
                                 -9.938056	53.912500;
                                 -9.940556	53.909444;
                                 -9.942778	53.906389;
                                 -9.942222	53.902222;
                                 -9.940556	53.898333;
                                 -9.940833	53.893889;
                                 -9.939444	53.890278;
                                 -9.940833	53.886667;
                                 -9.943333	53.883333;
                                 -9.946667	53.880556;
                                 -9.950556	53.878333;
                                 -9.955000	53.876667;
                                 -9.961389	53.875833;
                                 -9.967500	53.876667;
                                 -9.973056	53.878056;
                                 -9.978611	53.879444;
                                 -9.983056	53.881389;
                                 -9.987500	53.883333;
                                 -9.990833	53.886111;
                                 -9.996111	53.887500;
                                 -9.997500	53.891111;
                                 -9.995000	53.894444;
                                 -9.995833	53.896667;
                                 -9.999722	53.898889;
                                 -10.00500	53.9002780;
                                 -10.01055	53.9016676;
                                 -10.01500	53.9036110;
                                 -10.01916	53.9061117;
                                 -10.02361	53.9077781;
                                 -10.02916	53.9091677;
                                 -10.03694	53.9094444;
                                 -10.04305	53.9086116;
                                 -10.05083	53.9086113;
                                 -10.05555	53.9094446;
                                 -10.05750	53.9127780;
                                 -10.05916	53.9163897;
                                 -10.06138	53.9197229;
                                 -10.06277	53.9233338;
                                 -10.06277	53.9277788;
                                 -10.06111	53.9313891;
                                 -10.05944	53.9352784;
                                 -10.05722	53.9383332;
                                 -10.05472	53.9416672;
                                 -10.05222	53.9438892;
                                 -10.05055	53.9475006;
                                 -10.04972	53.9516672;
                                 -10.05194	53.9547224;
                                 -10.05527	53.9575008;
                                 -10.05888	53.9600009;
                                 -10.06361	53.9616671;
                                 -10.06750	53.9641670;
                                 -10.07194	53.9658334;
                                 -10.07805	53.9669446;
                                 -10.08444	53.9661114;
                                 -10.08972	53.9647222;
                                 -10.09472	53.9627782;
                                 -10.09944	53.9611114;
                                 -10.10388	53.9591679;
                                 -10.10944	53.9580564;
                                 -10.11583	53.9588893;
                                 -10.12027	53.9608338;
                                 -10.12583	53.9622223;
                                 -10.13277	53.9616678;
                                 -10.13972	53.9613892;
                                 -10.14611	53.9605561;
                                 -10.15222	53.9597222;
                                 -10.16027	53.9597228;
                                 -10.16694	53.9602784;
                                 -10.17416	53.9608337;
                                 -10.17805	53.9583336;
                                 -10.18194	53.9561114;
                                 -10.18888	53.9558339;
                                 -10.19583	53.9563893;
                                 -10.20111	53.9577781;
                                 -10.20611	53.9594441;
                                 -10.20972	53.9619442;
                                 -10.21444	53.9636114;
                                 -10.22000	53.9650000;
                                 -10.22527	53.9666678;
                                 -10.23083	53.9680563;
                                 -10.23694	53.9688894;
                                 -10.24388	53.9694449;
                                 -10.25083	53.9700003;
                                 -10.25694	53.9691674;
                                 -10.26416	53.9686117;
                                 -10.26638	53.9719449;
                                 -10.26861	53.9750001;
                                 -10.26555	53.9777786;
                                 -10.26000	53.9783330;
                                 -10.25388	53.9791679;
                                 -10.24694	53.9794444;
                                 -10.24055	53.9802786;
                                 -10.23444	53.9813894;
                                 -10.22722	53.9808332;
                                 -10.22055	53.9802786;
                                 -10.21416	53.9811117;
                                 -10.21083	53.9838893;
                                 -10.20777	53.9866678;
                                 -10.20388	53.9888899;
                                 -10.20000	53.9911110;
                                 -10.19666	53.9938897;
                                 -10.19583	53.9980563;
                                 -10.19833	54.0011113;
                                 -10.19861	54.0052781;
                                 -10.19666	54.0083337;
                                 -10.18944	54.0088894;
                                 -10.18416	54.0075007;
                                 -10.17777	54.0063898;
                                 -10.17055	54.0069446;
                                 -10.16611	54.0086111;
                                 -10.15888	54.0091679;
                                 -10.15416	54.0072227;
                                 -10.14972	54.0052782;
                                 -10.14500	54.0036110;
                                 -10.13888	54.0025009;
                                 -10.13194	54.0019444;
                                 -10.12500	54.0016670;
                                 -10.11944	54.0002784;
                                 -10.11500	53.9983330;
                                 -10.10944	53.9969444;
                                 -10.10250	53.9963890;
                                 -10.09694	53.9977784;
                                 -10.09444	54.0008334;
                                 -10.09388	54.0050009;
                                 -10.09388	54.0086119;
                                 -10.09138	54.0119449;
                                 -10.08638	54.0136119;
                                 -10.08166	54.0155567;
                                 -10.07722	54.0172222;
                                 -10.07333	54.0194443;
                                 -10.06916	54.0216677;
                                 -10.06305	54.0225006;
                                 -10.05666	54.0216677;
                                 -10.05305	54.0194446;
                                 -10.05361	54.0161111;
                                 -10.04916	54.0177787;
                                 -10.04277	54.0188898;
                                 -10.03972	54.0169442;
                                 -10.03666	54.0141677;
                                 -10.03361	54.0113891;
                                 -10.02805	54.0100006;
                                 -10.02333	54.0108333;
                                 -10.01805	54.0122226;
                                 -10.01111	54.0125001;
                                 -10.00388	54.0119449;
                                 -9.996111	54.011944;
                                 -9.990833	54.013333;
                                 -9.986111	54.015000;
                                 -9.981944	54.017500;
                                 -9.979444	54.020556;
                                 -9.974167	54.021944;
                                 -9.968056	54.020833;
                                 -9.964167	54.018611;
                                 -9.959722	54.009722];

  uki(82).xy = [                 -8.221111	51.851667;
                                 -8.223333	51.848333;
                                 -8.225833	51.845278;
                                 -8.228889	51.842500;
                                 -8.233333	51.840833;
                                 -8.237778	51.839167;
                                 -8.243056	51.837778;
                                 -8.249722	51.837500;
                                 -8.256944	51.837500;
                                 -8.263889	51.838611;
                                 -8.269722	51.839167;
                                 -8.275556	51.838333;
                                 -8.281667	51.837500;
                                 -8.286944	51.838889;
                                 -8.290278	51.841111;
                                 -8.291111	51.845278;
                                 -8.289444	51.848889;
                                 -8.286944	51.852222;
                                 -8.285278	51.855833;
                                 -8.283056	51.858889;
                                 -8.280000	51.861667;
                                 -8.273333	51.861111;
                                 -8.267222	51.860000;
                                 -8.261389	51.860833;
                                 -8.256111	51.862222;
                                 -8.249444	51.862500;
                                 -8.243056	51.863056;
                                 -8.237500	51.864167;
                                 -8.231667	51.865000;
                                 -8.225833	51.864167;
                                 -8.220000	51.862500;
                                 -8.218611	51.859444;
                                 -8.221111	51.851667];

  uki(83).xy = [ -5.003056	58.628333;
                 -4.996944	58.627500;
                 -4.990833	58.625833;
                 -4.988056	58.623056;
                 -4.983889	58.620556;
                 -4.980833	58.617778;
                 -4.975000	58.616111;
                 -4.969167	58.614444;
                 -4.962500	58.613333;
                 -4.955278	58.612778;
                 -4.949722	58.614444;
                 -4.944444	58.616111;
                 -4.936389	58.616111;
                 -4.928333	58.616389;
                 -4.919722	58.615833;
                 -4.910833	58.615556;
                 -4.903889	58.616111;
                 -4.895833	58.616111;
                 -4.888056	58.615278;
                 -4.885000	58.612500;
                 -4.882778	58.609167;
                 -4.878889	58.606667;
                 -4.873056	58.605000;
                 -4.865278	58.604167;
                 -4.858333	58.603056;
                 -4.853611	58.600833;
                 -4.850556	58.598056;
                 -4.846389	58.595556;
                 -4.840556	58.593889;
                 -4.834722	58.592222;
                 -4.831667	58.590278;
                 -4.831389	58.586111;
                 -4.833056	58.582222;
                 -4.832778	58.578056;
                 -4.832500	58.573889;
                 -4.830556	58.570556;
                 -4.827500	58.567778;
                 -4.823611	58.565278;
                 -4.817500	58.563611;
                 -4.811944	58.565000;
                 -4.808056	58.567778;
                 -4.804167	58.570278;
                 -4.799444	58.572222;
                 -4.793889	58.573889;
                 -4.785833	58.574167;
                 -4.778056	58.574167;
                 -4.773056	58.576111;
                 -4.770000	58.579167;
                 -4.771111	58.583056;
                 -4.775278	58.585556;
                 -4.780000	58.587500;
                 -4.785278	58.589444;
                 -4.789167	58.591944;
                 -4.793056	58.594444;
                 -4.791389	58.598611;
                 -4.787500	58.601111;
                 -4.782778	58.603333;
                 -4.776389	58.604444;
                 -4.770556	58.602778;
                 -4.768611	58.599444;
                 -4.765278	58.596389;
                 -4.761389	58.593889;
                 -4.756667	58.591944;
                 -4.751667	58.589722;
                 -4.750556	58.586111;
                 -4.750278	58.581944;
                 -4.750278	58.577778;
                 -4.748056	58.574722;
                 -4.745000	58.571667;
                 -4.738056	58.571389;
                 -4.730833	58.571944;
                 -4.724167	58.570833;
                 -4.721111	58.567778;
                 -4.720000	58.564167;
                 -4.716111	58.561667;
                 -4.709167	58.560278;
                 -4.702500	58.559167;
                 -4.699167	58.556111;
                 -4.692500	58.555000;
                 -4.686667	58.553333;
                 -4.678889	58.552500;
                 -4.672778	58.551667;
                 -4.670000	58.548889;
                 -4.666944	58.546111;
                 -4.663889	58.543056;
                 -4.662500	58.539444;
                 -4.663611	58.534722;
                 -4.666667	58.531667;
                 -4.670556	58.529167;
                 -4.674444	58.526667;
                 -4.679444	58.524444;
                 -4.683333	58.521944;
                 -4.687222	58.519444;
                 -4.691111	58.516944;
                 -4.695000	58.514444;
                 -4.698333	58.511111;
                 -4.702500	58.508611;
                 -4.705278	58.505556;
                 -4.708611	58.502500;
                 -4.711667	58.499444;
                 -4.715556	58.496944;
                 -4.718889	58.493889;
                 -4.721944	58.490833;
                 -4.725000	58.487778;
                 -4.727500	58.484167;
                 -4.730000	58.480556;
                 -4.733056	58.477500;
                 -4.735278	58.474167;
                 -4.738611	58.471111;
                 -4.741667	58.468056;
                 -4.745000	58.464722;
                 -4.748056	58.461944;
                 -4.751944	58.459167;
                 -4.755000	58.456111;
                 -4.759167	58.453611;
                 -4.762222	58.450556;
                 -4.765278	58.447500;
                 -4.765000	58.444167;
                 -4.760000	58.442222;
                 -4.753056	58.442778;
                 -4.748333	58.445000;
                 -4.743611	58.446944;
                 -4.739444	58.449444;
                 -4.736389	58.452500;
                 -4.734167	58.456111;
                 -4.732500	58.460278;
                 -4.729167	58.463333;
                 -4.724722	58.465278;
                 -4.718333	58.466389;
                 -4.711944	58.467500;
                 -4.708611	58.470556;
                 -4.705556	58.473611;
                 -4.701667	58.476111;
                 -4.696111	58.477778;
                 -4.690556	58.479167;
                 -4.685000	58.480833;
                 -4.680556	58.482778;
                 -4.676389	58.485556;
                 -4.676667	58.488611;
                 -4.680556	58.491111;
                 -4.683611	58.494167;
                 -4.685833	58.497500;
                 -4.682500	58.500556;
                 -4.677778	58.502500;
                 -4.673056	58.504722;
                 -4.669167	58.507222;
                 -4.666111	58.510278;
                 -4.663611	58.513889;
                 -4.659722	58.516389;
                 -4.655000	58.518333;
                 -4.649444	58.520000;
                 -4.643056	58.521111;
                 -4.638056	58.523056;
                 -4.631944	58.524167;
                 -4.624722	58.524722;
                 -4.618333	58.525833;
                 -4.614444	58.528333;
                 -4.610556	58.531111;
                 -4.606667	58.533611;
                 -4.602778	58.536111;
                 -4.599722	58.539167;
                 -4.597778	58.543333;
                 -4.600000	58.546389;
                 -4.605000	58.548611;
                 -4.606944	58.551944;
                 -4.608056	58.555556;
                 -4.611111	58.558611;
                 -4.613889	58.561389;
                 -4.615278	58.565000;
                 -4.612778	58.568611;
                 -4.610278	58.572222;
                 -4.608056	58.575833;
                 -4.603889	58.578333;
                 -4.597778	58.579444;
                 -4.590833	58.578056;
                 -4.582222	58.577778;
                 -4.574167	58.577778;
                 -4.567222	58.578333;
                 -4.559444	58.578333;
                 -4.551944	58.579167;
                 -4.545278	58.577778;
                 -4.539444	58.576111;
                 -4.531667	58.575278;
                 -4.523056	58.575000;
                 -4.515000	58.574167;
                 -4.510278	58.571944;
                 -4.505278	58.570000;
                 -4.499444	58.568333;
                 -4.494722	58.566111;
                 -4.488889	58.564444;
                 -4.482778	58.562778;
                 -4.476111	58.561667;
                 -4.471111	58.559444;
                 -4.466389	58.557500;
                 -4.466944	58.553056;
                 -4.461111	58.551389;
                 -4.455000	58.551389;
                 -4.448611	58.552500;
                 -4.440833	58.551667;
                 -4.436111	58.549722;
                 -4.432222	58.547222;
                 -4.430000	58.543889;
                 -4.428889	58.540000;
                 -4.428611	58.535833;
                 -4.431944	58.532778;
                 -4.436667	58.530278;
                 -4.433889	58.527500;
                 -4.430000	58.525000;
                 -4.427778	58.521667;
                 -4.430278	58.518056;
                 -4.434444	58.515556;
                 -4.438889	58.513611;
                 -4.443611	58.511389;
                 -4.449167	58.510000;
                 -4.453056	58.507222;
                 -4.456389	58.504167;
                 -4.456389	58.500278;
                 -4.458611	58.496667;
                 -4.460833	58.493056;
                 -4.464167	58.490000;
                 -4.467500	58.486944;
                 -4.468056	58.483333;
                 -4.468056	58.479167;
                 -4.470278	58.475556;
                 -4.473333	58.472500;
                 -4.475000	58.468611;
                 -4.477500	58.465000;
                 -4.480833	58.461944;
                 -4.483889	58.458889;
                 -4.486111	58.455278;
                 -4.489167	58.452222;
                 -4.492500	58.449167;
                 -4.495000	58.445556;
                 -4.491667	58.443611;
                 -4.486389	58.445278;
                 -4.481389	58.447222;
                 -4.477500	58.449722;
                 -4.473611	58.452500;
                 -4.470556	58.455556;
                 -4.467222	58.458333;
                 -4.463056	58.461111;
                 -4.459167	58.463611;
                 -4.455278	58.466111;
                 -4.451389	58.468611;
                 -4.445833	58.470278;
                 -4.440278	58.471944;
                 -4.436389	58.474444;
                 -4.432500	58.476944;
                 -4.430556	58.481111;
                 -4.430833	58.485000;
                 -4.431944	58.488889;
                 -4.431111	58.493333;
                 -4.428056	58.496389;
                 -4.423333	58.498611;
                 -4.419167	58.501111;
                 -4.413889	58.502778;
                 -4.408889	58.504722;
                 -4.403333	58.506389;
                 -4.398611	58.508333;
                 -4.394444	58.510833;
                 -4.390556	58.513333;
                 -4.386667	58.516111;
                 -4.382778	58.518611;
                 -4.379722	58.521667;
                 -4.376389	58.524722;
                 -4.373333	58.527778;
                 -4.369167	58.530278;
                 -4.364444	58.532222;
                 -4.359722	58.534444;
                 -4.353333	58.535556;
                 -4.346944	58.536389;
                 -4.339722	58.537222;
                 -4.336111	58.537778;
                 -4.329167	58.538333;
                 -4.321944	58.538889;
                 -4.314722	58.539444;
                 -4.308611	58.540556;
                 -4.300556	58.540556;
                 -4.291667	58.540278;
                 -4.285278	58.538889;
                 -4.280000	58.536944;
                 -4.277222	58.533889;
                 -4.277222	58.529722;
                 -4.272222	58.527778;
                 -4.265278	58.526389;
                 -4.258611	58.525278;
                 -4.253611	58.523056;
                 -4.247778	58.521389;
                 -4.241944	58.519722;
                 -4.236389	58.521389;
                 -4.239444	58.524167;
                 -4.244444	58.526389;
                 -4.248333	58.528889;
                 -4.246944	58.531111;
                 -4.239722	58.531667;
                 -4.232778	58.532222;
                 -4.226944	58.533611;
                 -4.226389	58.538333;
                 -4.226389	58.542500;
                 -4.228611	58.545833;
                 -4.226111	58.549167;
                 -4.221111	58.551389;
                 -4.215556	58.552778;
                 -4.208611	58.552500;
                 -4.203889	58.550556;
                 -4.200833	58.547500;
                 -4.196944	58.545000;
                 -4.191944	58.543056;
                 -4.185278	58.541667;
                 -4.177500	58.540833;
                 -4.171111	58.541944;
                 -4.165556	58.543333;
                 -4.163056	58.546944;
                 -4.163333	58.551111;
                 -4.163333	58.555278;
                 -4.162500	58.559722;
                 -4.159444	58.562778;
                 -4.151389	58.562778;
                 -4.145000	58.563889;
                 -4.139444	58.565556;
                 -4.134722	58.567500;
                 -4.128611	58.566944;
                 -4.125833	58.563889;
                 -4.120833	58.561667;
                 -4.112778	58.561944;
                 -4.106944	58.561111;
                 -4.099167	58.560278;
                 -4.091389	58.559444;
                 -4.085556	58.557778;
                 -4.080556	58.555556;
                 -4.074167	58.556667;
                 -4.071667	58.560278;
                 -4.068611	58.563333;
                 -4.063056	58.564722;
                 -4.059167	58.567500;
                 -4.057500	58.571389;
                 -4.057500	58.574722;
                 -4.054444	58.577500;
                 -4.051389	58.580833;
                 -4.050278	58.585278;
                 -4.047222	58.588333;
                 -4.040833	58.589444;
                 -4.036944	58.591944;
                 -4.032778	58.594444;
                 -4.028889	58.596944;
                 -4.022500	58.598056;
                 -4.021111	58.594444;
                 -4.022778	58.591944;
                 -4.022500	58.588056;
                 -4.021389	58.584167;
                 -4.019444	58.580833;
                 -4.018611	58.577222;
                 -4.017222	58.573333;
                 -4.015278	58.570278;
                 -4.012500	58.567222;
                 -4.006667	58.565556;
                 -3.999444	58.566111;
                 -3.994722	58.568056;
                 -3.987500	58.568611;
                 -3.980833	58.567500;
                 -3.972778	58.566667;
                 -3.965000	58.566667;
                 -3.959444	58.568056;
                 -3.952500	58.568611;
                 -3.946667	58.566944;
                 -3.937778	58.566667;
                 -3.930556	58.567222;
                 -3.922500	58.567222;
                 -3.917778	58.565000;
                 -3.913889	58.562500;
                 -3.910000	58.560000;
                 -3.903333	58.558611;
                 -3.896944	58.559722;
                 -3.892222	58.561944;
                 -3.885833	58.562778;
                 -3.879444	58.563889;
                 -3.871667	58.563056;
                 -3.863889	58.562222;
                 -3.857222	58.560833;
                 -3.848333	58.560556;
                 -3.842778	58.561944;
                 -3.838889	58.564444;
                 -3.834722	58.566944;
                 -3.829167	58.568611;
                 -3.820278	58.568056;
                 -3.812500	58.568056;
                 -3.806111	58.569167;
                 -3.799167	58.569722;
                 -3.793333	58.568056;
                 -3.785278	58.567222;
                 -3.779722	58.568611;
                 -3.775833	58.571111;
                 -3.771667	58.573889;
                 -3.767778	58.576389;
                 -3.763056	58.578333;
                 -3.758333	58.580278;
                 -3.753333	58.582500;
                 -3.748611	58.584444;
                 -3.743611	58.586389;
                 -3.739722	58.589167;
                 -3.735833	58.591667;
                 -3.730000	58.593056;
                 -3.726111	58.595556;
                 -3.722222	58.598056;
                 -3.718056	58.600833;
                 -3.713889	58.603333;
                 -3.709167	58.605278;
                 -3.702778	58.606389;
                 -3.696667	58.607222;
                 -3.689167	58.607778;
                 -3.683611	58.609444;
                 -3.678056	58.610833;
                 -3.674167	58.613611;
                 -3.670833	58.616389;
                 -3.666111	58.618611;
                 -3.660278	58.620000;
                 -3.652500	58.619167;
                 -3.646667	58.617500;
                 -3.639167	58.616667;
                 -3.631944	58.617222;
                 -3.625556	58.618056;
                 -3.619167	58.619167;
                 -3.611944	58.619722;
                 -3.604722	58.620278;
                 -3.596944	58.620278;
                 -3.588889	58.620278;
                 -3.581667	58.620833;
                 -3.576111	58.622222;
                 -3.570833	58.623056;
                 -3.561944	58.622500;
                 -3.553333	58.621944;
                 -3.547500	58.620278;
                 -3.543611	58.617778;
                 -3.545278	58.614722;
                 -3.548333	58.611667;
                 -3.553333	58.609722;
                 -3.552778	58.607222;
                 -3.552222	58.605278;
                 -3.545278	58.604167;
                 -3.539722	58.602500;
                 -3.535000	58.600278;
                 -3.528056	58.598889;
                 -3.520278	58.598056;
                 -3.513889	58.599167;
                 -3.511944	58.599444;
                 -3.507222	58.601389;
                 -3.501111	58.602500;
                 -3.496111	58.604722;
                 -3.489722	58.605556;
                 -3.484167	58.607222;
                 -3.476111	58.607222;
                 -3.469722	58.608056;
                 -3.464167	58.609722;
                 -3.455556	58.609167;
                 -3.446667	58.608611;
                 -3.438611	58.608611;
                 -3.432222	58.607500;
                 -3.435278	58.604444;
                 -3.432500	58.601389;
                 -3.425278	58.601944;
                 -3.416389	58.601389;
                 -3.408611	58.600556;
                 -3.401944	58.599167;
                 -3.394722	58.599722;
                 -3.386111	58.599444;
                 -3.379444	58.598056;
                 -3.373611	58.596389;
                 -3.364722	58.595833;
                 -3.360833	58.598333;
                 -3.358056	58.601944;
                 -3.356389	58.605833;
                 -3.353889	58.609444;
                 -3.351667	58.613056;
                 -3.350833	58.616667;
                 -3.354722	58.619167;
                 -3.362222	58.620000;
                 -3.367222	58.622222;
                 -3.369167	58.625556;
                 -3.373056	58.628056;
                 -3.378889	58.629722;
                 -3.384722	58.631389;
                 -3.391667	58.632778;
                 -3.398056	58.633889;
                 -3.402222	58.636389;
                 -3.404167	58.639722;
                 -3.406944	58.642778;
                 -3.408889	58.646111;
                 -3.410000	58.649722;
                 -3.408889	58.654444;
                 -3.406389	58.657778;
                 -3.403333	58.660833;
                 -3.400000	58.663889;
                 -3.396111	58.666389;
                 -3.391111	58.668333;
                 -3.386111	58.670556;
                 -3.380556	58.671944;
                 -3.374167	58.673056;
                 -3.365556	58.672500;
                 -3.360556	58.670556;
                 -3.356667	58.667778;
                 -3.354722	58.664444;
                 -3.353889	58.660833;
                 -3.353889	58.656667;
                 -3.351944	58.653333;
                 -3.348889	58.650556;
                 -3.345278	58.648056;
                 -3.337500	58.646944;
                 -3.330000	58.647500;
                 -3.322222	58.647500;
                 -3.314444	58.646667;
                 -3.308611	58.645000;
                 -3.300833	58.644167;
                 -3.294722	58.644167;
                 -3.289722	58.646111;
                 -3.285556	58.648611;
                 -3.280278	58.650278;
                 -3.273056	58.650833;
                 -3.265000	58.650833;
                 -3.258056	58.650278;
                 -3.252500	58.651944;
                 -3.245278	58.652222;
                 -3.236667	58.651944;
                 -3.228889	58.650833;
                 -3.220000	58.650556;
                 -3.214722	58.651111;
                 -3.208333	58.651944;
                 -3.201944	58.653056;
                 -3.195556	58.654167;
                 -3.186667	58.653611;
                 -3.180833	58.651944;
                 -3.176111	58.649722;
                 -3.172222	58.647222;
                 -3.169444	58.644167;
                 -3.166389	58.641389;
                 -3.162500	58.638889;
                 -3.156667	58.637222;
                 -3.149722	58.637500;
                 -3.145000	58.638611;
                 -3.137778	58.639167;
                 -3.130556	58.639722;
                 -3.125000	58.641389;
                 -3.120000	58.643333;
                 -3.113889	58.644167;
                 -3.105833	58.644167;
                 -3.097778	58.644167;
                 -3.089722	58.644167;
                 -3.083611	58.645278;
                 -3.077222	58.646111;
                 -3.069722	58.646667;
                 -3.061389	58.646111;
                 -3.053333	58.645278;
                 -3.046111	58.645833;
                 -3.039722	58.646944;
                 -3.033333	58.647778;
                 -3.025556	58.647778;
                 -3.019722	58.646111;
                 -3.015833	58.643611;
                 -3.015000	58.639722;
                 -3.014722	58.637500;
                 -3.013611	58.633889;
                 -3.015278	58.629722;
                 -3.018611	58.626667;
                 -3.022778	58.624167;
                 -3.026944	58.621667;
                 -3.028611	58.617778;
                 -3.028611	58.613611;
                 -3.028611	58.609444;
                 -3.028611	58.605278;
                 -3.029444	58.600833;
                 -3.033056	58.597778;
                 -3.038611	58.596111;
                 -3.045000	58.595278;
                 -3.051389	58.594167;
                 -3.057500	58.593056;
                 -3.058333	58.590556;
                 -3.054444	58.588056;
                 -3.053056	58.586389;
                 -3.051111	58.584167;
                 -3.052778	58.580000;
                 -3.055833	58.576944;
                 -3.060000	58.574444;
                 -3.063611	58.571389;
                 -3.066667	58.568333;
                 -3.069167	58.565000;
                 -3.071111	58.560833;
                 -3.074167	58.557778;
                 -3.076667	58.554444;
                 -3.080000	58.551389;
                 -3.083333	58.548333;
                 -3.088056	58.546389;
                 -3.093056	58.544167;
                 -3.097222	58.541667;
                 -3.102778	58.540278;
                 -3.106667	58.537778;
                 -3.111944	58.535278;
                 -3.115000	58.532222;
                 -3.119167	58.529722;
                 -3.121667	58.526111;
                 -3.125000	58.523056;
                 -3.127500	58.519722;
                 -3.130000	58.516111;
                 -3.131667	58.511944;
                 -3.135000	58.509167;
                 -3.137500	58.505556;
                 -3.140000	58.501944;
                 -3.141667	58.498889;
                 -3.141667	58.494722;
                 -3.138611	58.491944;
                 -3.134722	58.489167;
                 -3.130833	58.486667;
                 -3.128056	58.483889;
                 -3.124167	58.481389;
                 -3.121389	58.478333;
                 -3.117500	58.475833;
                 -3.113611	58.473333;
                 -3.110000	58.470833;
                 -3.105000	58.468611;
                 -3.097222	58.467778;
                 -3.090833	58.468889;
                 -3.084444	58.469722;
                 -3.078056	58.470833;
                 -3.072500	58.472222;
                 -3.066944	58.473889;
                 -3.060556	58.474722;
                 -3.055000	58.476389;
                 -3.048611	58.477222;
                 -3.042222	58.478333;
                 -3.034444	58.478333;
                 -3.028611	58.476667;
                 -3.024722	58.474167;
                 -3.022778	58.470833;
                 -3.021667	58.466944;
                 -3.021944	58.462778;
                 -3.023611	58.458889;
                 -3.027500	58.456389;
                 -3.031667	58.453889;
                 -3.035278	58.450833;
                 -3.039167	58.448333;
                 -3.041389	58.444722;
                 -3.045000	58.441667;
                 -3.047500	58.438333;
                 -3.051389	58.435556;
                 -3.056389	58.433611;
                 -3.060278	58.431111;
                 -3.063611	58.428056;
                 -3.065833	58.425556;
                 -3.066944	58.420833;
                 -3.067778	58.416389;
                 -3.071389	58.413333;
                 -3.075278	58.410833;
                 -3.080000	58.408889;
                 -3.083333	58.405833;
                 -3.084167	58.401389;
                 -3.085556	58.396667;
                 -3.087222	58.392778;
                 -3.089722	58.389167;
                 -3.092778	58.386111;
                 -3.096944	58.383611;
                 -3.102500	58.382222;
                 -3.106667	58.379722;
                 -3.107500	58.376111;
                 -3.105556	58.372778;
                 -3.104444	58.368889;
                 -3.103611	58.365278;
                 -3.105833	58.361667;
                 -3.110000	58.359167;
                 -3.113889	58.356667;
                 -3.118889	58.354722;
                 -3.125278	58.353611;
                 -3.133333	58.353611;
                 -3.140278	58.353056;
                 -3.145833	58.351667;
                 -3.148333	58.348056;
                 -3.150833	58.344722;
                 -3.154167	58.341667;
                 -3.158889	58.339444;
                 -3.163889	58.337500;
                 -3.168333	58.335556;
                 -3.173333	58.333611;
                 -3.178889	58.331944;
                 -3.182778	58.329444;
                 -3.187778	58.327500;
                 -3.190278	58.323889;
                 -3.193611	58.320833;
                 -3.196111	58.317500;
                 -3.197778	58.313333;
                 -3.200278	58.309722;
                 -3.203611	58.306667;
                 -3.208333	58.304722;
                 -3.215556	58.303889;
                 -3.222778	58.303333;
                 -3.228333	58.301667;
                 -3.233889	58.300278;
                 -3.240278	58.299167;
                 -3.246667	58.298333;
                 -3.253611	58.297778;
                 -3.260000	58.296667;
                 -3.266111	58.295833;
                 -3.269722	58.295000;
                 -3.275278	58.293611;
                 -3.281111	58.291944;
                 -3.285556	58.290000;
                 -3.289722	58.287500;
                 -3.293889	58.285000;
                 -3.299444	58.283333;
                 -3.307500	58.283333;
                 -3.315278	58.283333;
                 -3.320556	58.281944;
                 -3.326111	58.280278;
                 -3.331111	58.278333;
                 -3.337500	58.277222;
                 -3.345278	58.277222;
                 -3.353056	58.277222;
                 -3.359444	58.276389;
                 -3.364167	58.274167;
                 -3.368056	58.271667;
                 -3.372222	58.269167;
                 -3.376111	58.266667;
                 -3.381944	58.265278;
                 -3.385833	58.262500;
                 -3.390000	58.260000;
                 -3.393056	58.256944;
                 -3.396111	58.254167;
                 -3.398889	58.250556;
                 -3.403611	58.248611;
                 -3.409167	58.246944;
                 -3.415556	58.245833;
                 -3.420278	58.243889;
                 -3.423611	58.240833;
                 -3.426111	58.237500;
                 -3.429167	58.234444;
                 -3.433056	58.231944;
                 -3.438056	58.229722;
                 -3.441944	58.227222;
                 -3.444444	58.223611;
                 -3.446389	58.219722;
                 -3.448611	58.216944;
                 -3.450833	58.213611;
                 -3.453333	58.210000;
                 -3.455833	58.206389;
                 -3.458889	58.203333;
                 -3.462500	58.200278;
                 -3.466389	58.197778;
                 -3.471111	58.195833;
                 -3.474444	58.192778;
                 -3.477500	58.189722;
                 -3.480000	58.186111;
                 -3.483056	58.183056;
                 -3.487222	58.180556;
                 -3.491944	58.178611;
                 -3.496944	58.176667;
                 -3.500833	58.174167;
                 -3.504722	58.171667;
                 -3.508056	58.168611;
                 -3.511111	58.165556;
                 -3.515000	58.163056;
                 -3.520000	58.160833;
                 -3.524722	58.158889;
                 -3.529444	58.156944;
                 -3.535000	58.155278;
                 -3.540556	58.153889;
                 -3.546111	58.152222;
                 -3.551667	58.150833;
                 -3.556389	58.148611;
                 -3.559444	58.145833;
                 -3.562778	58.142778;
                 -3.566111	58.139722;
                 -3.570000	58.136944;
                 -3.575556	58.135000;
                 -3.581389	58.133611;
                 -3.586667	58.131944;
                 -3.592222	58.130556;
                 -3.597778	58.128889;
                 -3.602500	58.126944;
                 -3.608056	58.125556;
                 -3.612778	58.123333;
                 -3.617778	58.121389;
                 -3.621667	58.118889;
                 -3.625556	58.116389;
                 -3.630278	58.114167;
                 -3.634167	58.112222;
                 -3.633333	58.112222;
                 -3.638611	58.111389;
                 -3.645833	58.111111;
                 -3.652222	58.110000;
                 -3.657500	58.108333;
                 -3.662222	58.106389;
                 -3.666389	58.103889;
                 -3.670278	58.101389;
                 -3.673333	58.098333;
                 -3.676667	58.095278;
                 -3.680556	58.092778;
                 -3.684444	58.090278;
                 -3.689444	58.088056;
                 -3.694167	58.086111;
                 -3.698611	58.084167;
                 -3.704444	58.082500;
                 -3.710000	58.081111;
                 -3.715278	58.079444;
                 -3.720833	58.078056;
                 -3.726389	58.076389;
                 -3.730556	58.073889;
                 -3.733611	58.070833;
                 -3.736111	58.067222;
                 -3.739167	58.064167;
                 -3.743056	58.061667;
                 -3.747778	58.059722;
                 -3.753333	58.058056;
                 -3.759167	58.056667;
                 -3.764444	58.055000;
                 -3.771389	58.054444;
                 -3.779167	58.054444;
                 -3.785556	58.053333;
                 -3.791111	58.051944;
                 -3.795278	58.049444;
                 -3.799167	58.046944;
                 -3.804444	58.045278;
                 -3.810000	58.043889;
                 -3.816389	58.042778;
                 -3.822500	58.041667;
                 -3.825556	58.038611;
                 -3.828056	58.035000;
                 -3.831389	58.031944;
                 -3.835556	58.029444;
                 -3.839444	58.026944;
                 -3.842500	58.023889;
                 -3.845000	58.020278;
                 -3.846389	58.016389;
                 -3.847500	58.011944;
                 -3.847500	58.007778;
                 -3.847500	58.003611;
                 -3.847778	58.000000;
                 -3.853611	57.998333;
                 -3.858889	57.996667;
                 -3.863056	57.994167;
                 -3.867778	57.992222;
                 -3.873889	57.991111;
                 -3.880833	57.990556;
                 -3.886389	57.989167;
                 -3.891944	57.987500;
                 -3.896667	57.985556;
                 -3.902222	57.983889;
                 -3.906944	57.981944;
                 -3.911667	57.980000;
                 -3.917778	57.978889;
                 -3.924167	57.977222;
                 -3.930556	57.976389;
                 -3.936111	57.974722;
                 -3.940556	57.972778;
                 -3.944444	57.970278;
                 -3.949167	57.968056;
                 -3.954722	57.966667;
                 -3.962778	57.966667;
                 -3.968056	57.965000;
                 -3.973611	57.963333;
                 -3.978333	57.961389;
                 -3.983056	57.959444;
                 -3.987778	57.957222;
                 -3.991667	57.954722;
                 -3.995556	57.952222;
                 -3.998889	57.949167;
                 -4.000556	57.945278;
                 -4.001111	57.940556;
                 -4.003056	57.936667;
                 -4.003611	57.933056;
                 -4.004444	57.928333;
                 -4.006111	57.924444;
                 -4.010833	57.922222;
                 -4.008889	57.918889;
                 -4.005000	57.917222;
                 -3.998889	57.915833;
                 -3.993333	57.913889;
                 -3.989444	57.911389;
                 -3.989167	57.907500;
                 -3.990833	57.903333;
                 -3.991944	57.898889;
                 -3.994167	57.895278;
                 -3.997222	57.892222;
                 -4.001111	57.889722;
                 -4.005278	57.887222;
                 -4.009167	57.884444;
                 -4.013056	57.881944;
                 -4.017222	57.879444;
                 -4.018889	57.875278;
                 -4.016944	57.872222;
                 -4.013889	57.869167;
                 -4.011111	57.866389;
                 -4.009167	57.863056;
                 -4.010556	57.859722;
                 -4.016667	57.858889;
                 -4.024167	57.859722;
                 -4.031111	57.860833;
                 -4.038611	57.860833;
                 -4.044167	57.859444;
                 -4.051389	57.858889;
                 -4.058889	57.858611;
                 -4.067500	57.859167;
                 -4.075833	57.859722;
                 -4.084444	57.860000;
                 -4.090833	57.858889;
                 -4.095556	57.856944;
                 -4.098611	57.853889;
                 -4.101944	57.850833;
                 -4.105833	57.848333;
                 -4.109722	57.845833;
                 -4.115833	57.844722;
                 -4.123889	57.844722;
                 -4.128333	57.846667;
                 -4.134167	57.848333;
                 -4.142500	57.848889;
                 -4.149722	57.848333;
                 -4.156389	57.848611;
                 -4.161389	57.850833;
                 -4.168056	57.851944;
                 -4.175556	57.852778;
                 -4.179444	57.855278;
                 -4.183333	57.857778;
                 -4.189167	57.858611;
                 -4.196389	57.858056;
                 -4.203056	57.857500;
                 -4.200556	57.854722;
                 -4.194444	57.853056;
                 -4.190556	57.850556;
                 -4.186944	57.848056;
                 -4.183889	57.845000;
                 -4.182778	57.841389;
                 -4.182778	57.837222;
                 -4.180833	57.833889;
                 -4.176944	57.831389;
                 -4.168333	57.830833;
                 -4.160833	57.830000;
                 -4.154167	57.828889;
                 -4.145833	57.828333;
                 -4.139444	57.829444;
                 -4.132222	57.830000;
                 -4.125556	57.830556;
                 -4.116944	57.830000;
                 -4.108333	57.829722;
                 -4.100833	57.828889;
                 -4.095000	57.827222;
                 -4.089444	57.825278;
                 -4.084444	57.823333;
                 -4.078889	57.821667;
                 -4.073056	57.820000;
                 -4.067500	57.818333;
                 -4.060833	57.816667;
                 -4.053889	57.815278;
                 -4.046389	57.814444;
                 -4.038889	57.814444;
                 -4.031667	57.815000;
                 -4.025278	57.816111;
                 -4.020833	57.818056;
                 -4.016944	57.820556;
                 -4.012222	57.822778;
                 -4.007500	57.824722;
                 -4.002778	57.826944;
                 -3.998056	57.828889;
                 -3.993333	57.830833;
                 -3.987778	57.832500;
                 -3.982500	57.833889;
                 -3.976944	57.835556;
                 -3.971389	57.836944;
                 -3.963889	57.836111;
                 -3.957222	57.835000;
                 -3.950556	57.833611;
                 -3.943889	57.832500;
                 -3.937222	57.831111;
                 -3.930556	57.830000;
                 -3.924722	57.828056;
                 -3.921111	57.825556;
                 -3.920833	57.822500;
                 -3.925556	57.820278;
                 -3.930278	57.818333;
                 -3.935556	57.816944;
                 -3.940556	57.814722;
                 -3.945000	57.812778;
                 -3.949722	57.810833;
                 -3.951111	57.807500;
                 -3.945556	57.805833;
                 -3.938611	57.806389;
                 -3.933889	57.808333;
                 -3.928333	57.810000;
                 -3.923611	57.811944;
                 -3.918333	57.813611;
                 -3.912500	57.815000;
                 -3.906389	57.816111;
                 -3.900278	57.817222;
                 -3.893333	57.817778;
                 -3.886111	57.818333;
                 -3.879167	57.818889;
                 -3.871389	57.818889;
                 -3.864444	57.819444;
                 -3.857500	57.820000;
                 -3.851389	57.820833;
                 -3.845833	57.822500;
                 -3.841111	57.824444;
                 -3.836389	57.826667;
                 -3.833333	57.829722;
                 -3.830833	57.833056;
                 -3.829167	57.837222;
                 -3.827500	57.841389;
                 -3.825833	57.845278;
                 -3.823611	57.848889;
                 -3.821111	57.852500;
                 -3.817778	57.855556;
                 -3.814722	57.858333;
                 -3.810000	57.860556;
                 -3.805278	57.862500;
                 -3.799167	57.863611;
                 -3.791944	57.864167;
                 -3.783333	57.863611;
                 -3.777778	57.861944;
                 -3.775833	57.858611;
                 -3.775000	57.855000;
                 -3.775556	57.850278;
                 -3.776667	57.845833;
                 -3.779722	57.842778;
                 -3.783056	57.839722;
                 -3.786111	57.836667;
                 -3.790000	57.834167;
                 -3.794722	57.832222;
                 -3.798611	57.829722;
                 -3.802500	57.826944;
                 -3.805556	57.823889;
                 -3.807500	57.820000;
                 -3.809722	57.816389;
                 -3.812222	57.812778;
                 -3.815556	57.809722;
                 -3.819444	57.807222;
                 -3.823333	57.804722;
                 -3.827222	57.802222;
                 -3.831389	57.799722;
                 -3.835278	57.797222;
                 -3.839167	57.794444;
                 -3.843056	57.791944;
                 -3.846111	57.788889;
                 -3.848611	57.785556;
                 -3.851111	57.781944;
                 -3.854167	57.778889;
                 -3.857500	57.775833;
                 -3.859722	57.772222;
                 -3.863611	57.769722;
                 -3.867500	57.767222;
                 -3.872222	57.765278;
                 -3.877778	57.763611;
                 -3.883056	57.761944;
                 -3.889444	57.761111;
                 -3.894722	57.759444;
                 -3.899722	57.757500;
                 -3.904167	57.755278;
                 -3.908333	57.752778;
                 -3.912222	57.750278;
                 -3.914444	57.746667;
                 -3.917778	57.743611;
                 -3.920000	57.740278;
                 -3.923056	57.737222;
                 -3.926389	57.734167;
                 -3.930278	57.731389;
                 -3.933333	57.728611;
                 -3.937222	57.725833;
                 -3.940556	57.722778;
                 -3.942222	57.718889;
                 -3.944444	57.715278;
                 -3.948611	57.712778;
                 -3.953889	57.711111;
                 -3.958611	57.709167;
                 -3.963056	57.707222;
                 -3.966944	57.705556;
                 -3.970833	57.703056;
                 -3.974722	57.700556;
                 -3.978056	57.697500;
                 -3.981944	57.694722;
                 -3.985000	57.691667;
                 -3.987222	57.688333;
                 -3.990556	57.685278;
                 -3.996111	57.683611;
                 -4.000833	57.681667;
                 -4.006944	57.680556;
                 -4.014722	57.680556;
                 -4.020278	57.682222;
                 -4.024167	57.684722;
                 -4.025278	57.688333;
                 -4.025278	57.692500;
                 -4.027222	57.695833;
                 -4.030000	57.698889;
                 -4.031944	57.701944;
                 -4.033333	57.705833;
                 -4.032500	57.709444;
                 -4.028611	57.711944;
                 -4.024722	57.714444;
                 -4.020833	57.717222;
                 -4.017500	57.720278;
                 -4.015833	57.724167;
                 -4.016944	57.727778;
                 -4.019722	57.730833;
                 -4.025556	57.732500;
                 -4.032500	57.731944;
                 -4.038056	57.730556;
                 -4.042778	57.728333;
                 -4.048056	57.726944;
                 -4.055278	57.726389;
                 -4.062778	57.726111;
                 -4.070556	57.726111;
                 -4.077500	57.725556;
                 -4.085278	57.725556;
                 -4.092222	57.725000;
                 -4.098611	57.723889;
                 -4.103333	57.721944;
                 -4.108611	57.720278;
                 -4.113333	57.718333;
                 -4.118056	57.716389;
                 -4.121111	57.713333;
                 -4.121944	57.708611;
                 -4.125000	57.705556;
                 -4.129167	57.703056;
                 -4.133889	57.701111;
                 -4.137778	57.698333;
                 -4.142222	57.695833;
                 -4.146944	57.693889;
                 -4.151667	57.691944;
                 -4.155556	57.689167;
                 -4.160278	57.687222;
                 -4.164167	57.684722;
                 -4.169722	57.683056;
                 -4.175000	57.681667;
                 -4.182500	57.682500;
                 -4.190278	57.683333;
                 -4.197222	57.682778;
                 -4.204722	57.682778;
                 -4.210556	57.684444;
                 -4.217222	57.685556;
                 -4.224722	57.686389;
                 -4.230833	57.685278;
                 -4.236667	57.683889;
                 -4.241944	57.682222;
                 -4.246667	57.680278;
                 -4.251944	57.678611;
                 -4.256667	57.676667;
                 -4.263056	57.675556;
                 -4.270556	57.675556;
                 -4.279167	57.675833;
                 -4.287500	57.676389;
                 -4.294722	57.675833;
                 -4.299444	57.673889;
                 -4.302222	57.670833;
                 -4.303333	57.666111;
                 -4.302222	57.662500;
                 -4.301944	57.658333;
                 -4.303333	57.655278;
                 -4.308889	57.653611;
                 -4.315833	57.653056;
                 -4.322778	57.652500;
                 -4.328889	57.651389;
                 -4.332778	57.648889;
                 -4.335000	57.646111;
                 -4.338056	57.643056;
                 -4.340556	57.639722;
                 -4.343611	57.636667;
                 -4.346667	57.633611;
                 -4.350556	57.630833;
                 -4.355278	57.628889;
                 -4.360556	57.627222;
                 -4.366111	57.625833;
                 -4.370000	57.623333;
                 -4.374722	57.621111;
                 -4.379444	57.619167;
                 -4.383056	57.616389;
                 -4.388056	57.614444;
                 -4.392500	57.612500;
                 -4.397222	57.610278;
                 -4.402500	57.608889;
                 -4.407222	57.606667;
                 -4.412500	57.605000;
                 -4.418056	57.603611;
                 -4.419444	57.600556;
                 -4.415556	57.598056;
                 -4.416389	57.593333;
                 -4.420278	57.590833;
                 -4.424167	57.588333;
                 -4.428056	57.585833;
                 -4.431944	57.583056;
                 -4.434722	57.581389;
                 -4.437778	57.578333;
                 -4.433889	57.576944;
                 -4.428889	57.574722;
                 -4.432222	57.571667;
                 -4.436111	57.569167;
                 -4.440000	57.566667;
                 -4.436389	57.567222;
                 -4.431667	57.569444;
                 -4.425556	57.570556;
                 -4.419444	57.571667;
                 -4.413889	57.573056;
                 -4.412222	57.577222;
                 -4.409167	57.580278;
                 -4.403889	57.581667;
                 -4.397778	57.582778;
                 -4.392222	57.584444;
                 -4.386667	57.585833;
                 -4.383056	57.588611;
                 -4.380000	57.591667;
                 -4.376111	57.594167;
                 -4.371944	57.596667;
                 -4.367500	57.598611;
                 -4.362778	57.600833;
                 -4.358889	57.603333;
                 -4.354167	57.605278;
                 -4.349722	57.607500;
                 -4.345000	57.609444;
                 -4.341111	57.611944;
                 -4.337778	57.615000;
                 -4.333889	57.617778;
                 -4.331111	57.620833;
                 -4.327778	57.623889;
                 -4.323889	57.626389;
                 -4.320000	57.628889;
                 -4.315556	57.631111;
                 -4.310000	57.632500;
                 -4.304722	57.634167;
                 -4.299167	57.635556;
                 -4.294444	57.637778;
                 -4.289722	57.639722;
                 -4.285833	57.642222;
                 -4.281944	57.644722;
                 -4.277222	57.646944;
                 -4.271944	57.648611;
                 -4.267222	57.650556;
                 -4.261944	57.652222;
                 -4.258056	57.654722;
                 -4.254167	57.657222;
                 -4.250278	57.659722;
                 -4.245556	57.661944;
                 -4.240833	57.663889;
                 -4.236389	57.665833;
                 -4.230833	57.667500;
                 -4.223889	57.668056;
                 -4.216667	57.668611;
                 -4.210000	57.669167;
                 -4.203611	57.670278;
                 -4.198333	57.671667;
                 -4.192222	57.672778;
                 -4.184444	57.672778;
                 -4.176667	57.671944;
                 -4.172222	57.670000;
                 -4.170000	57.666667;
                 -4.172500	57.663056;
                 -4.175556	57.660000;
                 -4.169722	57.659167;
                 -4.162500	57.659722;
                 -4.155833	57.660278;
                 -4.148056	57.660556;
                 -4.141389	57.659167;
                 -4.134722	57.657778;
                 -4.127222	57.656944;
                 -4.119444	57.657222;
                 -4.113333	57.658056;
                 -4.106111	57.658611;
                 -4.100833	57.660278;
                 -4.094722	57.661389;
                 -4.089167	57.662778;
                 -4.084444	57.665000;
                 -4.079722	57.666944;
                 -4.075278	57.668889;
                 -4.070556	57.671111;
                 -4.064167	57.672222;
                 -4.058889	57.673611;
                 -4.052778	57.674722;
                 -4.047778	57.676667;
                 -4.042500	57.678333;
                 -4.035833	57.676944;
                 -4.031111	57.675000;
                 -4.024444	57.673611;
                 -4.016944	57.672778;
                 -4.008611	57.672222;
                 -4.000000	57.671944;
                 -3.998889	57.668056;
                 -4.003611	57.666111;
                 -4.009167	57.664444;
                 -4.014444	57.663056;
                 -4.019167	57.661111;
                 -4.023056	57.658333;
                 -4.026944	57.655833;
                 -4.032500	57.654444;
                 -4.037222	57.652222;
                 -4.039444	57.648611;
                 -4.042500	57.645556;
                 -4.046389	57.643056;
                 -4.051944	57.641667;
                 -4.057500	57.640000;
                 -4.059722	57.636389;
                 -4.061944	57.633056;
                 -4.064444	57.629444;
                 -4.067500	57.626389;
                 -4.070833	57.623333;
                 -4.074722	57.620833;
                 -4.079444	57.618611;
                 -4.083889	57.616667;
                 -4.087778	57.614167;
                 -4.090278	57.610556;
                 -4.093333	57.607500;
                 -4.096389	57.604444;
                 -4.101111	57.602500;
                 -4.107500	57.601389;
                 -4.111389	57.598889;
                 -4.112222	57.597500;
                 -4.114444	57.595000;
                 -4.116944	57.591389;
                 -4.116667	57.588056;
                 -4.113889	57.585278;
                 -4.111944	57.581944;
                 -4.108889	57.578889;
                 -4.105833	57.576111;
                 -4.106389	57.573333;
                 -4.113611	57.572778;
                 -4.120000	57.574167;
                 -4.127778	57.574722;
                 -4.136111	57.575278;
                 -4.143056	57.574722;
                 -4.149167	57.573611;
                 -4.155556	57.572778;
                 -4.160833	57.571111;
                 -4.166389	57.569444;
                 -4.171667	57.568056;
                 -4.175556	57.565556;
                 -4.179444	57.562778;
                 -4.182500	57.559722;
                 -4.183333	57.555278;
                 -4.181389	57.551944;
                 -4.179444	57.548611;
                 -4.180278	57.545000;
                 -4.186389	57.543889;
                 -4.194722	57.544444;
                 -4.202500	57.545278;
                 -4.208889	57.546389;
                 -4.216667	57.547500;
                 -4.224167	57.547222;
                 -4.232778	57.547778;
                 -4.241111	57.548333;
                 -4.247778	57.549444;
                 -4.255556	57.549444;
                 -4.260278	57.547222;
                 -4.259167	57.544444;
                 -4.251389	57.543611;
                 -4.243889	57.542778;
                 -4.236389	57.541944;
                 -4.227778	57.541667;
                 -4.220278	57.540556;
                 -4.213611	57.539444;
                 -4.210556	57.536389;
                 -4.210556	57.534167;
                 -4.211111	57.529722;
                 -4.212778	57.525556;
                 -4.215833	57.522500;
                 -4.219722	57.520000;
                 -4.224444	57.518056;
                 -4.228889	57.515833;
                 -4.230278	57.513333;
                 -4.228889	57.513056;
                 -4.232500	57.510556;
                 -4.235000	57.506944;
                 -4.237222	57.503333;
                 -4.240278	57.500278;
                 -4.244167	57.497778;
                 -4.248889	57.495833;
                 -4.242222	57.494444;
                 -4.234722	57.493611;
                 -4.230000	57.491667;
                 -4.225000	57.489444;
                 -4.219444	57.487778;
                 -4.212778	57.486667;
                 -4.204444	57.486111;
                 -4.196667	57.486111;
                 -4.191389	57.487778;
                 -4.185278	57.488889;
                 -4.178889	57.489722;
                 -4.173611	57.491389;
                 -4.168889	57.493333;
                 -4.163611	57.495000;
                 -4.158889	57.496944;
                 -4.155833	57.500000;
                 -4.153611	57.503611;
                 -4.152778	57.507222;
                 -4.153889	57.511111;
                 -4.150556	57.514167;
                 -4.144444	57.515000;
                 -4.137500	57.515556;
                 -4.130556	57.516111;
                 -4.123889	57.516667;
                 -4.117500	57.517778;
                 -4.112778	57.520000;
                 -4.108889	57.522500;
                 -4.105278	57.525000;
                 -4.101111	57.527500;
                 -4.098056	57.530556;
                 -4.095000	57.533611;
                 -4.091667	57.536667;
                 -4.087778	57.539167;
                 -4.082500	57.540833;
                 -4.076389	57.541944;
                 -4.070000	57.542778;
                 -4.065556	57.545000;
                 -4.061389	57.547500;
                 -4.056111	57.549167;
                 -4.051389	57.551111;
                 -4.046111	57.552778;
                 -4.041389	57.554722;
                 -4.038333	57.557778;
                 -4.037222	57.562222;
                 -4.037500	57.566389;
                 -4.040556	57.569444;
                 -4.044167	57.571944;
                 -4.049722	57.573611;
                 -4.057500	57.574444;
                 -4.065000	57.575278;
                 -4.072500	57.576111;
                 -4.078333	57.577778;
                 -4.081944	57.580278;
                 -4.077500	57.582222;
                 -4.070556	57.582778;
                 -4.063611	57.583333;
                 -4.058056	57.585000;
                 -4.052778	57.586667;
                 -4.045556	57.587222;
                 -4.041389	57.587222;
                 -4.035278	57.588333;
                 -4.029167	57.589444;
                 -4.021944	57.590000;
                 -4.015000	57.590556;
                 -4.008056	57.591111;
                 -4.000556	57.591111;
                 -3.992778	57.591111;
                 -3.984167	57.590833;
                 -3.976667	57.590833;
                 -3.968056	57.590278;
                 -3.959444	57.589722;
                 -3.950833	57.589444;
                 -3.942500	57.588889;
                 -3.934167	57.588333;
                 -3.926389	57.588333;
                 -3.917778	57.588056;
                 -3.909444	57.587500;
                 -3.900833	57.587222;
                 -3.893333	57.586111;
                 -3.884722	57.585833;
                 -3.877222	57.585833;
                 -3.870000	57.586389;
                 -3.863889	57.587500;
                 -3.857778	57.588333;
                 -3.854167	57.589722;
                 -3.853333	57.590000;
                 -3.849722	57.590278;
                 -3.844167	57.591667;
                 -3.838056	57.592778;
                 -3.832778	57.594444;
                 -3.826389	57.595556;
                 -3.820278	57.596389;
                 -3.815556	57.598611;
                 -3.810000	57.600000;
                 -3.806111	57.602500;
                 -3.801389	57.604722;
                 -3.797500	57.607222;
                 -3.793056	57.609167;
                 -3.788333	57.611389;
                 -3.783333	57.613889;
                 -3.779444	57.616389;
                 -3.775000	57.618333;
                 -3.771111	57.620833;
                 -3.767222	57.623333;
                 -3.761389	57.625000;
                 -3.754722	57.625556;
                 -3.747500	57.626111;
                 -3.742222	57.627500;
                 -3.737500	57.629722;
                 -3.733611	57.632222;
                 -3.728889	57.634167;
                 -3.724444	57.636111;
                 -3.720278	57.638611;
                 -3.715556	57.640833;
                 -3.710833	57.642778;
                 -3.706389	57.644722;
                 -3.700833	57.646389;
                 -3.696111	57.648333;
                 -3.691389	57.650556;
                 -3.687500	57.653056;
                 -3.685000	57.656667;
                 -3.679722	57.658056;
                 -3.671111	57.657500;
                 -3.663333	57.657500;
                 -3.657222	57.658611;
                 -3.651944	57.660278;
                 -3.646389	57.660833;
                 -3.638611	57.660833;
                 -3.631944	57.659444;
                 -3.625556	57.658333;
                 -3.617778	57.658333;
                 -3.611667	57.659167;
                 -3.605278	57.660278;
                 -3.598333	57.660833;
                 -3.589722	57.660278;
                 -3.581389	57.660000;
                 -3.572778	57.659444;
                 -3.565278	57.659444;
                 -3.556667	57.658889;
                 -3.548889	57.658889;
                 -3.542778	57.660000;
                 -3.536389	57.661111;
                 -3.531111	57.662500;
                 -3.526389	57.664444;
                 -3.522500	57.667222;
                 -3.518333	57.669722;
                 -3.514444	57.672222;
                 -3.511111	57.675278;
                 -3.507222	57.677778;
                 -3.504167	57.680833;
                 -3.500278	57.683333;
                 -3.497778	57.686944;
                 -3.496111	57.690833;
                 -3.493056	57.693889;
                 -3.487500	57.695556;
                 -3.480556	57.695833;
                 -3.471944	57.695556;
                 -3.466667	57.696111;
                 -3.461111	57.697778;
                 -3.456667	57.699722;
                 -3.451944	57.701667;
                 -3.445556	57.702778;
                 -3.438611	57.703333;
                 -3.431667	57.703611;
                 -3.424444	57.704167;
                 -3.418333	57.705278;
                 -3.412222	57.706389;
                 -3.406111	57.707222;
                 -3.399722	57.708333;
                 -3.394167	57.709722;
                 -3.388056	57.710833;
                 -3.381944	57.711944;
                 -3.375556	57.712778;
                 -3.370000	57.714444;
                 -3.364444	57.715833;
                 -3.358333	57.716944;
                 -3.351944	57.718056;
                 -3.345278	57.718333;
                 -3.338056	57.718889;
                 -3.331667	57.717778;
                 -3.325000	57.716389;
                 -3.318611	57.717500;
                 -3.311111	57.716389;
                 -3.305556	57.714722;
                 -3.297500	57.714722;
                 -3.290833	57.715278;
                 -3.283611	57.715833;
                 -3.279167	57.713611;
                 -3.277222	57.710278;
                 -3.274167	57.707500;
                 -3.268611	57.705833;
                 -3.263889	57.703611;
                 -3.257222	57.702222;
                 -3.255556	57.701667;
                 -3.249722	57.700833;
                 -3.244167	57.699167;
                 -3.237500	57.697778;
                 -3.231944	57.696111;
                 -3.225000	57.694722;
                 -3.219444	57.693056;
                 -3.212778	57.691944;
                 -3.206389	57.690556;
                 -3.199722	57.689167;
                 -3.193056	57.687778;
                 -3.186389	57.686667;
                 -3.180000	57.685278;
                 -3.173333	57.683889;
                 -3.166667	57.682778;
                 -3.160278	57.681389;
                 -3.153611	57.680000;
                 -3.146944	57.678611;
                 -3.140000	57.677500;
                 -3.133611	57.676111;
                 -3.126111	57.675278;
                 -3.119444	57.673889;
                 -3.111944	57.673056;
                 -3.105278	57.671667;
                 -3.097778	57.670833;
                 -3.091111	57.669444;
                 -3.084722	57.668056;
                 -3.077222	57.667222;
                 -3.069444	57.666111;
                 -3.061944	57.665278;
                 -3.054444	57.664444;
                 -3.046944	57.663611;
                 -3.039444	57.662500;
                 -3.030833	57.661944;
                 -3.022222	57.661667;
                 -3.015278	57.661944;
                 -3.008333	57.662500;
                 -3.002222	57.663611;
                 -2.995833	57.664444;
                 -2.991111	57.666389;
                 -2.985556	57.668056;
                 -2.980833	57.670000;
                 -2.975556	57.671389;
                 -2.969167	57.672500;
                 -2.961389	57.672500;
                 -2.954722	57.673056;
                 -2.949722	57.675000;
                 -2.944444	57.676389;
                 -2.938611	57.678056;
                 -2.931111	57.678056;
                 -2.926944	57.680556;
                 -2.930833	57.683056;
                 -2.932500	57.686389;
                 -2.931944	57.690000;
                 -2.928056	57.692500;
                 -2.920278	57.692500;
                 -2.913056	57.693056;
                 -2.908333	57.695000;
                 -2.902222	57.696111;
                 -2.893889	57.695556;
                 -2.886111	57.695556;
                 -2.880000	57.696389;
                 -2.873611	57.697500;
                 -2.868056	57.698889;
                 -2.861944	57.700000;
                 -2.855556	57.700833;
                 -2.849722	57.700000;
                 -2.847778	57.696667;
                 -2.846944	57.693056;
                 -2.845000	57.689722;
                 -2.840278	57.687500;
                 -2.832778	57.687500;
                 -2.826389	57.688611;
                 -2.820833	57.690000;
                 -2.814722	57.691111;
                 -2.807500	57.691667;
                 -2.801389	57.692500;
                 -2.795833	57.693889;
                 -2.790556	57.695556;
                 -2.785000	57.693889;
                 -2.781944	57.690833;
                 -2.773333	57.690278;
                 -2.765833	57.689444;
                 -2.761111	57.688889;
                 -2.755000	57.687222;
                 -2.749444	57.685556;
                 -2.745833	57.683056;
                 -2.743056	57.680000;
                 -2.738333	57.678056;
                 -2.732222	57.678056;
                 -2.727500	57.680000;
                 -2.722222	57.681389;
                 -2.714444	57.681389;
                 -2.708611	57.679722;
                 -2.704167	57.677778;
                 -2.696667	57.676667;
                 -2.689444	57.677222;
                 -2.684167	57.678611;
                 -2.678333	57.680278;
                 -2.670556	57.680000;
                 -2.665000	57.678333;
                 -2.658333	57.676944;
                 -2.650833	57.676111;
                 -2.643611	57.675000;
                 -2.635833	57.675000;
                 -2.628611	57.675556;
                 -2.622500	57.676667;
                 -2.615000	57.675556;
                 -2.609444	57.673889;
                 -2.602778	57.672500;
                 -2.595278	57.671667;
                 -2.588056	57.671944;
                 -2.581944	57.673056;
                 -2.575000	57.673611;
                 -2.568333	57.672222;
                 -2.564722	57.669722;
                 -2.560000	57.667500;
                 -2.554444	57.665833;
                 -2.545833	57.665278;
                 -2.539444	57.666111;
                 -2.532500	57.666667;
                 -2.526111	57.665278;
                 -2.519167	57.663889;
                 -2.511944	57.663056;
                 -2.505000	57.663611;
                 -2.500278	57.665556;
                 -2.494722	57.666944;
                 -2.488889	57.668333;
                 -2.481389	57.668333;
                 -2.474722	57.668056;
                 -2.466944	57.666944;
                 -2.460556	57.665556;
                 -2.452222	57.665000;
                 -2.443611	57.664444;
                 -2.435833	57.664444;
                 -2.427222	57.663889;
                 -2.418889	57.663333;
                 -2.410278	57.662778;
                 -2.402778	57.661944;
                 -2.395278	57.660833;
                 -2.388611	57.659444;
                 -2.381111	57.659444;
                 -2.374722	57.660556;
                 -2.369167	57.661944;
                 -2.363056	57.663056;
                 -2.355833	57.663333;
                 -2.348333	57.663333;
                 -2.340833	57.662500;
                 -2.333889	57.661944;
                 -2.326944	57.662500;
                 -2.322222	57.664444;
                 -2.317500	57.666389;
                 -2.315000	57.670000;
                 -2.313333	57.673889;
                 -2.310556	57.677500;
                 -2.306667	57.680000;
                 -2.301667	57.681944;
                 -2.297222	57.683889;
                 -2.291389	57.685278;
                 -2.285833	57.683611;
                 -2.281389	57.681389;
                 -2.276667	57.679167;
                 -2.271111	57.677500;
                 -2.265278	57.675833;
                 -2.258889	57.674444;
                 -2.250278	57.673889;
                 -2.242778	57.672778;
                 -2.235000	57.671944;
                 -2.226944	57.671389;
                 -2.219167	57.670278;
                 -2.210556	57.669722;
                 -2.204167	57.668333;
                 -2.199444	57.666389;
                 -2.193889	57.664444;
                 -2.187222	57.663056;
                 -2.180556	57.663611;
                 -2.174722	57.665000;
                 -2.168333	57.666111;
                 -2.163056	57.667500;
                 -2.158333	57.669444;
                 -2.152778	57.671111;
                 -2.148056	57.673056;
                 -2.143333	57.675000;
                 -2.139167	57.677500;
                 -2.136111	57.680556;
                 -2.131944	57.683056;
                 -2.127778	57.685278;
                 -2.122222	57.686944;
                 -2.114444	57.686944;
                 -2.106667	57.686667;
                 -2.100556	57.687778;
                 -2.093611	57.688056;
                 -2.086389	57.688611;
                 -2.078889	57.688611;
                 -2.071111	57.688333;
                 -2.062500	57.687778;
                 -2.055000	57.686944;
                 -2.048333	57.685556;
                 -2.041111	57.684722;
                 -2.032500	57.683889;
                 -2.026111	57.685000;
                 -2.020000	57.685833;
                 -2.013056	57.686389;
                 -2.005000	57.686389;
                 -1.997778	57.685278;
                 -1.995833	57.681944;
                 -1.995000	57.678056;
                 -1.993333	57.674722;
                 -1.988889	57.672778;
                 -1.984167	57.670556;
                 -1.978611	57.668611;
                 -1.970000	57.668056;
                 -1.963056	57.667778;
                 -1.956111	57.668056;
                 -1.949167	57.668611;
                 -1.942778	57.669444;
                 -1.935833	57.670000;
                 -1.928889	57.670278;
                 -1.921389	57.669444;
                 -1.915833	57.667778;
                 -1.913056	57.664722;
                 -1.911111	57.661389;
                 -1.908333	57.658333;
                 -1.905833	57.655278;
                 -1.902222	57.652778;
                 -1.899167	57.651111;
                 -1.896111	57.648333;
                 -1.892500	57.645833;
                 -1.890000	57.642778;
                 -1.886111	57.640278;
                 -1.881667	57.638056;
                 -1.878889	57.635000;
                 -1.875278	57.632500;
                 -1.873611	57.629167;
                 -1.870556	57.626111;
                 -1.866944	57.623611;
                 -1.863333	57.621111;
                 -1.857778	57.619444;
                 -1.852222	57.617500;
                 -1.847500	57.615278;
                 -1.841944	57.613611;
                 -1.836389	57.611944;
                 -1.829722	57.610556;
                 -1.823333	57.609167;
                 -1.816667	57.607778;
                 -1.811944	57.605556;
                 -1.811111	57.602778;
                 -1.813056	57.598611;
                 -1.815556	57.595278;
                 -1.818056	57.591667;
                 -1.820556	57.588333;
                 -1.822500	57.584167;
                 -1.823611	57.579722;
                 -1.823611	57.575556;
                 -1.823889	57.571389;
                 -1.822222	57.568056;
                 -1.821389	57.564167;
                 -1.818889	57.560833;
                 -1.815278	57.558333;
                 -1.811389	57.555833;
                 -1.806944	57.553611;
                 -1.803333	57.551111;
                 -1.799444	57.548333;
                 -1.796944	57.545556;
                 -1.797778	57.540833;
                 -1.799167	57.536389;
                 -1.800833	57.532222;
                 -1.800000	57.528611;
                 -1.799444	57.524722;
                 -1.797500	57.521667;
                 -1.795556	57.518333;
                 -1.793056	57.515278;
                 -1.789444	57.512778;
                 -1.784722	57.510556;
                 -1.780000	57.508333;
                 -1.775278	57.506111;
                 -1.770833	57.503889;
                 -1.766111	57.501667;
                 -1.763611	57.498889;
                 -1.761667	57.495556;
                 -1.761111	57.495000;
                 -1.760278	57.495000;
                 -1.765000	57.493056;
                 -1.773333	57.493611;
                 -1.781111	57.494444;
                 -1.786389	57.493056;
                 -1.785556	57.489444;
                 -1.780000	57.487500;
                 -1.777222	57.484722;
                 -1.779722	57.481111;
                 -1.783333	57.478056;
                 -1.782500	57.474444;
                 -1.779722	57.471389;
                 -1.773056	57.470000;
                 -1.766944	57.471111;
                 -1.759444	57.470000;
                 -1.756944	57.466944;
                 -1.760000	57.463889;
                 -1.763333	57.461111;
                 -1.766667	57.458889;
                 -1.769722	57.455833;
                 -1.773889	57.453333;
                 -1.777222	57.450556;
                 -1.780556	57.447500;
                 -1.784444	57.445000;
                 -1.788611	57.442500;
                 -1.792500	57.440000;
                 -1.797500	57.438056;
                 -1.801389	57.435556;
                 -1.805556	57.433056;
                 -1.804167	57.431111;
                 -1.797500	57.429722;
                 -1.796944	57.426111;
                 -1.800000	57.423056;
                 -1.805556	57.421667;
                 -1.810278	57.419722;
                 -1.813611	57.416667;
                 -1.816111	57.413056;
                 -1.820278	57.410556;
                 -1.823611	57.407778;
                 -1.829167	57.406111;
                 -1.835278	57.405278;
                 -1.843056	57.405278;
                 -1.850556	57.405278;
                 -1.855556	57.403333;
                 -1.857222	57.399444;
                 -1.856389	57.395556;
                 -1.853889	57.392778;
                 -1.851944	57.389444;
                 -1.850000	57.386111;
                 -1.851944	57.381944;
                 -1.856944	57.380000;
                 -1.861667	57.378056;
                 -1.866111	57.376111;
                 -1.871111	57.374167;
                 -1.875833	57.372222;
                 -1.880556	57.370278;
                 -1.885278	57.368333;
                 -1.889444	57.365833;
                 -1.894167	57.363889;
                 -1.898056	57.361389;
                 -1.901389	57.358333;
                 -1.904722	57.355278;
                 -1.909444	57.353333;
                 -1.914444	57.351389;
                 -1.918333	57.348889;
                 -1.921389	57.345833;
                 -1.924167	57.342500;
                 -1.926389	57.338889;
                 -1.929167	57.335278;
                 -1.933056	57.332778;
                 -1.937222	57.330278;
                 -1.941944	57.328333;
                 -1.945833	57.325833;
                 -1.950000	57.323333;
                 -1.953056	57.320278;
                 -1.956667	57.317500;
                 -1.959722	57.314444;
                 -1.963056	57.311389;
                 -1.966389	57.308333;
                 -1.969444	57.305556;
                 -1.972778	57.302500;
                 -1.976944	57.300000;
                 -1.980278	57.296944;
                 -1.983333	57.293889;
                 -1.987222	57.291389;
                 -1.990833	57.288611;
                 -1.993889	57.285556;
                 -1.998056	57.283056;
                 -2.000556	57.279444;
                 -2.003889	57.276389;
                 -2.006389	57.273056;
                 -2.009444	57.270000;
                 -2.012222	57.266389;
                 -2.014722	57.263056;
                 -2.017222	57.259444;
                 -2.019167	57.255556;
                 -2.022222	57.252500;
                 -2.024167	57.248333;
                 -2.027222	57.245278;
                 -2.030000	57.241389;
                 -2.032500	57.238056;
                 -2.035278	57.234444;
                 -2.038333	57.231389;
                 -2.040833	57.228056;
                 -2.044167	57.225000;
                 -2.046667	57.221389;
                 -2.049444	57.217778;
                 -2.051667	57.214444;
                 -2.055000	57.211389;
                 -2.057500	57.207778;
                 -2.060000	57.204444;
                 -2.061944	57.200278;
                 -2.063611	57.197222;
                 -2.063611	57.193056;
                 -2.063611	57.188889;
                 -2.064722	57.184444;
                 -2.066667	57.180278;
                 -2.068333	57.176389;
                 -2.069444	57.171944;
                 -2.071389	57.167778;
                 -2.072222	57.163333;
                 -2.073611	57.158611;
                 -2.074444	57.154167;
                 -2.074444	57.150000;
                 -2.073611	57.146389;
                 -2.071111	57.143333;
                 -2.067500	57.140833;
                 -2.063611	57.138333;
                 -2.061111	57.135278;
                 -2.059444	57.131944;
                 -2.055556	57.129444;
                 -2.051111	57.127222;
                 -2.046389	57.125000;
                 -2.042778	57.122500;
                 -2.041944	57.118611;
                 -2.043611	57.115556;
                 -2.044722	57.111111;
                 -2.047778	57.108056;
                 -2.050556	57.104722;
                 -2.053611	57.101667;
                 -2.056944	57.098611;
                 -2.059444	57.095000;
                 -2.062778	57.091944;
                 -2.065833	57.089167;
                 -2.070833	57.087222;
                 -2.075556	57.085000;
                 -2.079444	57.082500;
                 -2.081667	57.079167;
                 -2.083611	57.075000;
                 -2.084722	57.070556;
                 -2.086389	57.066389;
                 -2.088333	57.062500;
                 -2.090000	57.058333;
                 -2.092778	57.055000;
                 -2.096667	57.052500;
                 -2.099722	57.049444;
                 -2.102222	57.045833;
                 -2.105000	57.042500;
                 -2.107500	57.038889;
                 -2.110556	57.035833;
                 -2.114444	57.033333;
                 -2.120000	57.031389;
                 -2.125556	57.030000;
                 -2.129722	57.027500;
                 -2.131389	57.023611;
                 -2.134444	57.020556;
                 -2.139167	57.018611;
                 -2.144722	57.016944;
                 -2.148611	57.014722;
                 -2.151944	57.011667;
                 -2.154167	57.008056;
                 -2.155556	57.003611;
                 -2.155556	56.999444;
                 -2.156389	56.994722;
                 -2.157778	56.990278;
                 -2.158611	56.985833;
                 -2.161111	56.982222;
                 -2.165278	56.979722;
                 -2.170556	56.978333;
                 -2.175278	56.976389;
                 -2.180000	56.974444;
                 -2.183056	56.971389;
                 -2.182500	56.967500;
                 -2.179722	56.964722;
                 -2.178056	56.961389;
                 -2.176389	56.958056;
                 -2.174444	56.954722;
                 -2.171944	56.951111;
                 -2.170000	56.947778;
                 -2.170278	56.943611;
                 -2.170278	56.939444;
                 -2.171389	56.935000;
                 -2.173056	56.931111;
                 -2.175556	56.927500;
                 -2.177222	56.923333;
                 -2.177500	56.919444;
                 -2.176667	56.915556;
                 -2.177778	56.911111;
                 -2.180833	56.908056;
                 -2.183333	56.904444;
                 -2.186389	56.901389;
                 -2.189167	56.898056;
                 -2.192500	56.895000;
                 -2.195556	56.891944;
                 -2.198056	56.888333;
                 -2.200556	56.885000;
                 -2.204722	56.882500;
                 -2.207778	56.879444;
                 -2.210833	56.876389;
                 -2.212778	56.872500;
                 -2.213889	56.867778;
                 -2.214722	56.863333;
                 -2.216667	56.859444;
                 -2.220556	56.856944;
                 -2.224722	56.854444;
                 -2.229167	56.852222;
                 -2.233056	56.849722;
                 -2.236944	56.847222;
                 -2.240278	56.844444;
                 -2.244167	56.841944;
                 -2.248889	56.840000;
                 -2.254444	56.838333;
                 -2.258333	56.835833;
                 -2.259167	56.831389;
                 -2.261111	56.827222;
                 -2.264167	56.824444;
                 -2.267222	56.821389;
                 -2.271389	56.818889;
                 -2.274722	56.815833;
                 -2.278611	56.813333;
                 -2.282500	56.810833;
                 -2.286389	56.808333;
                 -2.290278	56.805833;
                 -2.294167	56.803333;
                 -2.298056	56.800833;
                 -2.302222	56.798333;
                 -2.306944	56.796389;
                 -2.310833	56.793889;
                 -2.315556	56.791944;
                 -2.320000	56.790000;
                 -2.323889	56.787500;
                 -2.328611	56.785278;
                 -2.333333	56.783333;
                 -2.339444	56.782500;
                 -2.344722	56.780833;
                 -2.349444	56.778889;
                 -2.352500	56.775833;
                 -2.355833	56.773056;
                 -2.360556	56.770833;
                 -2.368056	56.771111;
                 -2.375556	56.771944;
                 -2.383889	56.772500;
                 -2.390000	56.771667;
                 -2.394444	56.769444;
                 -2.399167	56.767500;
                 -2.403056	56.765000;
                 -2.406389	56.761944;
                 -2.408611	56.758611;
                 -2.410556	56.756944;
                 -2.406389	56.758333;
                 -2.408333	56.755833;
                 -2.410833	56.752222;
                 -2.413333	56.748611;
                 -2.414444	56.747778;
                 -2.416389	56.743889;
                 -2.418056	56.739722;
                 -2.419722	56.735833;
                 -2.421389	56.731667;
                 -2.423056	56.727778;
                 -2.424722	56.723611;
                 -2.426389	56.719722;
                 -2.428333	56.715556;
                 -2.430833	56.712222;
                 -2.433333	56.708611;
                 -2.437222	56.706111;
                 -2.440556	56.703056;
                 -2.441111	56.699444;
                 -2.436389	56.697222;
                 -2.430278	56.695833;
                 -2.424722	56.694167;
                 -2.422778	56.690833;
                 -2.423056	56.686667;
                 -2.424722	56.682778;
                 -2.428056	56.679722;
                 -2.431111	56.676667;
                 -2.435000	56.674167;
                 -2.438889	56.671667;
                 -2.443611	56.669722;
                 -2.448889	56.668056;
                 -2.455000	56.667222;
                 -2.461944	56.666667;
                 -2.468056	56.665833;
                 -2.473333	56.664167;
                 -2.478056	56.662222;
                 -2.482778	56.660278;
                 -2.485833	56.657222;
                 -2.488889	56.654167;
                 -2.492222	56.651111;
                 -2.493889	56.647222;
                 -2.496389	56.643611;
                 -2.498056	56.639722;
                 -2.498056	56.635556;
                 -2.496389	56.632222;
                 -2.491667	56.630000;
                 -2.487222	56.627778;
                 -2.480833	56.626667;
                 -2.476389	56.624444;
                 -2.474722	56.621111;
                 -2.474722	56.616944;
                 -2.477778	56.613889;
                 -2.480833	56.610833;
                 -2.485000	56.608333;
                 -2.488889	56.605833;
                 -2.493333	56.603889;
                 -2.496667	56.600833;
                 -2.500556	56.598333;
                 -2.503611	56.595278;
                 -2.506111	56.591944;
                 -2.507778	56.587778;
                 -2.511111	56.584722;
                 -2.515556	56.582778;
                 -2.519444	56.580278;
                 -2.522778	56.577222;
                 -2.524444	56.573333;
                 -2.526944	56.569722;
                 -2.530000	56.566667;
                 -2.533889	56.564167;
                 -2.537778	56.561667;
                 -2.540833	56.560000;
                 -2.545278	56.558056;
                 -2.550833	56.556667;
                 -2.555833	56.555000;
                 -2.561944	56.554167;
                 -2.568056	56.553056;
                 -2.575000	56.552500;
                 -2.581667	56.552222;
                 -2.587778	56.551111;
                 -2.593889	56.550278;
                 -2.598333	56.548333;
                 -2.603056	56.546111;
                 -2.606944	56.543611;
                 -2.609444	56.540278;
                 -2.612500	56.537222;
                 -2.615000	56.533611;
                 -2.618056	56.530556;
                 -2.621944	56.528056;
                 -2.626111	56.525556;
                 -2.630556	56.523611;
                 -2.635278	56.521667;
                 -2.640278	56.520000;
                 -2.645000	56.518056;
                 -2.649722	56.516111;
                 -2.652778	56.513056;
                 -2.656111	56.510000;
                 -2.661389	56.508611;
                 -2.668333	56.508056;
                 -2.674167	56.507222;
                 -2.679444	56.505556;
                 -2.683333	56.503056;
                 -2.688056	56.501111;
                 -2.693333	56.499722;
                 -2.698611	56.498056;
                 -2.704722	56.497222;
                 -2.710000	56.495556;
                 -2.714722	56.493611;
                 -2.718611	56.491111;
                 -2.720833	56.487500;
                 -2.723333	56.484167;
                 -2.725000	56.480000;
                 -2.726111	56.475556;
                 -2.726944	56.470833;
                 -2.726111	56.467222;
                 -2.726944	56.462778;
                 -2.732500	56.461111;
                 -2.740833	56.461667;
                 -2.746944	56.463056;
                 -2.752500	56.464722;
                 -2.758056	56.466667;
                 -2.763333	56.468333;
                 -2.768889	56.470000;
                 -2.775278	56.471389;
                 -2.781111	56.472222;
                 -2.787222	56.473611;
                 -2.794722	56.474444;
                 -2.801944	56.474444;
                 -2.808056	56.473611;
                 -2.814167	56.472500;
                 -2.820278	56.471389;
                 -2.825556	56.470000;
                 -2.830000	56.468056;
                 -2.835556	56.466389;
                 -2.840000	56.464444;
                 -2.845556	56.463056;
                 -2.850000	56.461111;
                 -2.855278	56.459444;
                 -2.860556	56.458056;
                 -2.868056	56.458056;
                 -2.873611	56.459722;
                 -2.879167	56.461389;
                 -2.884444	56.463333;
                 -2.890833	56.464444;
                 -2.898333	56.464444;
                 -2.903889	56.463056;
                 -2.909722	56.461944;
                 -2.918056	56.462500;
                 -2.925278	56.463611;
                 -2.932222	56.463056;
                 -2.937222	56.461944;
                 -2.937222	56.462778;
                 -2.940833	56.461389;
                 -2.945278	56.459167;
                 -2.950000	56.457222;
                 -2.954722	56.455278;
                 -2.958889	56.453333;
                 -2.962778	56.450833;
                 -2.967500	56.448611;
                 -2.972778	56.447222;
                 -2.979722	56.446667;
                 -2.986944	56.446667;
                 -2.993611	56.448056;
                 -3.000000	56.449444;
                 -3.006389	56.450556;
                 -3.014444	56.451111;
                 -3.021667	56.452222;
                 -3.030000	56.452500;
                 -3.036944	56.452222;
                 -3.043611	56.451667;
                 -3.048889	56.450000;
                 -3.054167	56.448611;
                 -3.059444	56.447222;
                 -3.065000	56.445556;
                 -3.069444	56.443611;
                 -3.073333	56.441111;
                 -3.077778	56.439167;
                 -3.081667	56.436667;
                 -3.086111	56.434444;
                 -3.092222	56.433611;
                 -3.097778	56.431944;
                 -3.102778	56.430556;
                 -3.108889	56.429444;
                 -3.114167	56.428056;
                 -3.119722	56.426389;
                 -3.124722	56.425000;
                 -3.129444	56.423056;
                 -3.133333	56.420278;
                 -3.136944	56.417778;
                 -3.140833	56.415278;
                 -3.144167	56.412222;
                 -3.148611	56.410278;
                 -3.153056	56.408333;
                 -3.157778	56.406389;
                 -3.163056	56.404722;
                 -3.166667	56.402222;
                 -3.171389	56.400278;
                 -3.175278	56.397778;
                 -3.178333	56.394722;
                 -3.182222	56.392222;
                 -3.186667	56.390000;
                 -3.191389	56.388056;
                 -3.196667	56.386667;
                 -3.201111	56.384444;
                 -3.205833	56.382500;
                 -3.210556	56.380556;
                 -3.214167	56.378056;
                 -3.218056	56.375556;
                 -3.221111	56.372500;
                 -3.224722	56.370000;
                 -3.228889	56.367500;
                 -3.234167	56.365833;
                 -3.240000	56.365000;
                 -3.246944	56.364444;
                 -3.253056	56.363333;
                 -3.258889	56.362222;
                 -3.264167	56.360833;
                 -3.269444	56.359167;
                 -3.274722	56.357778;
                 -3.280833	56.356667;
                 -3.288889	56.357222;
                 -3.296111	56.358056;
                 -3.301667	56.360000;
                 -3.307222	56.361667;
                 -3.312778	56.363333;
                 -3.319167	56.364722;
                 -3.324167	56.363056;
                 -3.325000	56.360278;
                 -3.321389	56.357778;
                 -3.316667	56.355833;
                 -3.311944	56.353611;
                 -3.306667	56.351944;
                 -3.301389	56.350278;
                 -3.294722	56.348889;
                 -3.287222	56.348056;
                 -3.280278	56.346944;
                 -3.271944	56.346667;
                 -3.264444	56.346667;
                 -3.258611	56.347500;
                 -3.252500	56.348611;
                 -3.246944	56.350000;
                 -3.241111	56.351111;
                 -3.235833	56.352778;
                 -3.229167	56.353056;
                 -3.223889	56.354722;
                 -3.216944	56.355278;
                 -3.210833	56.356111;
                 -3.205000	56.357222;
                 -3.198889	56.358333;
                 -3.193056	56.359167;
                 -3.187778	56.360833;
                 -3.183056	56.362778;
                 -3.178611	56.364722;
                 -3.174167	56.366667;
                 -3.168611	56.368333;
                 -3.162500	56.369444;
                 -3.156667	56.370278;
                 -3.151389	56.371944;
                 -3.146111	56.373333;
                 -3.141667	56.375278;
                 -3.136944	56.377778;
                 -3.133056	56.380278;
                 -3.129167	56.382778;
                 -3.124722	56.385000;
                 -3.119444	56.386389;
                 -3.113333	56.387500;
                 -3.107222	56.388333;
                 -3.101389	56.389444;
                 -3.095278	56.390556;
                 -3.089167	56.391389;
                 -3.083889	56.393056;
                 -3.079444	56.395000;
                 -3.074722	56.396944;
                 -3.070278	56.398889;
                 -3.065556	56.401111;
                 -3.060278	56.402500;
                 -3.053333	56.403056;
                 -3.046944	56.403611;
                 -3.040833	56.404444;
                 -3.036111	56.406389;
                 -3.032500	56.409167;
                 -3.027222	56.410556;
                 -3.021111	56.411667;
                 -3.015000	56.412500;
                 -3.008333	56.413056;
                 -3.002222	56.414167;
                 -2.995278	56.414444;
                 -2.990278	56.416111;
                 -2.985000	56.417500;
                 -2.980278	56.419444;
                 -2.975556	56.421667;
                 -2.971111	56.423611;
                 -2.966667	56.425556;
                 -2.961111	56.427222;
                 -2.955833	56.428611;
                 -2.951389	56.430556;
                 -2.946667	56.432500;
                 -2.942222	56.434722;
                 -2.938889	56.437778;
                 -2.935000	56.440278;
                 -2.932222	56.443333;
                 -2.928333	56.445833;
                 -2.923611	56.447778;
                 -2.918889	56.449722;
                 -2.913889	56.451111;
                 -2.906944	56.451667;
                 -2.900833	56.452778;
                 -2.892500	56.452222;
                 -2.884444	56.451667;
                 -2.877222	56.450833;
                 -2.870000	56.449722;
                 -2.864167	56.448056;
                 -2.860000	56.445833;
                 -2.855278	56.443889;
                 -2.851667	56.441389;
                 -2.847778	56.438889;
                 -2.840556	56.437778;
                 -2.834722	56.438889;
                 -2.828611	56.439722;
                 -2.821667	56.440278;
                 -2.815278	56.440833;
                 -2.808889	56.439444;
                 -2.805833	56.436389;
                 -2.804167	56.433056;
                 -2.804167	56.428889;
                 -2.803333	56.425278;
                 -2.801389	56.421944;
                 -2.798889	56.418889;
                 -2.798056	56.415278;
                 -2.799722	56.411111;
                 -2.801944	56.407778;
                 -2.804444	56.404167;
                 -2.806944	56.400556;
                 -2.809444	56.397222;
                 -2.810278	56.392500;
                 -2.809444	56.388889;
                 -2.808611	56.385000;
                 -2.809444	56.380556;
                 -2.811389	56.376389;
                 -2.815278	56.373889;
                 -2.818889	56.371389;
                 -2.823611	56.369444;
                 -2.828889	56.368056;
                 -2.833889	56.366389;
                 -2.837778	56.363889;
                 -2.836111	56.360556;
                 -2.831667	56.358611;
                 -2.828333	56.358333;
                 -2.823056	56.359722;
                 -2.819167	56.362222;
                 -2.813333	56.363333;
                 -2.809444	56.361667;
                 -2.807500	56.358333;
                 -2.806667	56.354444;
                 -2.805833	56.350833;
                 -2.803889	56.347500;
                 -2.800278	56.345000;
                 -2.795833	56.342778;
                 -2.788611	56.341944;
                 -2.782222	56.340556;
                 -2.779444	56.337500;
                 -2.775833	56.335000;
                 -2.771111	56.332778;
                 -2.765000	56.331667;
                 -2.759167	56.332500;
                 -2.751389	56.332500;
                 -2.744167	56.331667;
                 -2.738889	56.330000;
                 -2.735000	56.327222;
                 -2.730833	56.325278;
                 -2.723333	56.324167;
                 -2.718333	56.324722;
                 -2.710556	56.324722;
                 -2.704167	56.325278;
                 -2.695833	56.324722;
                 -2.687778	56.324167;
                 -2.680278	56.324167;
                 -2.675278	56.323889;
                 -2.668889	56.322500;
                 -2.664444	56.320278;
                 -2.658889	56.318611;
                 -2.651667	56.317778;
                 -2.643611	56.317222;
                 -2.638056	56.315278;
                 -2.633611	56.313333;
                 -2.631944	56.310000;
                 -2.630000	56.306667;
                 -2.628056	56.303333;
                 -2.623889	56.301111;
                 -2.619167	56.298611;
                 -2.614722	56.296389;
                 -2.609444	56.294722;
                 -2.603889	56.292778;
                 -2.599444	56.290833;
                 -2.594722	56.288611;
                 -2.590278	56.286389;
                 -2.585556	56.284167;
                 -2.582222	56.281667;
                 -2.579722	56.278889;
                 -2.577778	56.275556;
                 -2.579444	56.271389;
                 -2.582500	56.268333;
                 -2.586389	56.265833;
                 -2.591111	56.263889;
                 -2.595833	56.261944;
                 -2.600278	56.260000;
                 -2.603889	56.257500;
                 -2.607778	56.255000;
                 -2.612500	56.252778;
                 -2.616389	56.250278;
                 -2.620833	56.248333;
                 -2.624722	56.245833;
                 -2.628611	56.243333;
                 -2.633333	56.241389;
                 -2.637222	56.238889;
                 -2.640833	56.236389;
                 -2.644722	56.233889;
                 -2.649444	56.231944;
                 -2.653333	56.229444;
                 -2.657778	56.227222;
                 -2.661667	56.224722;
                 -2.665556	56.222222;
                 -2.669444	56.219722;
                 -2.673056	56.217222;
                 -2.677778	56.215278;
                 -2.683056	56.213889;
                 -2.688889	56.212778;
                 -2.696667	56.212778;
                 -2.704167	56.212778;
                 -2.710000	56.211944;
                 -2.716111	56.210833;
                 -2.721944	56.210000;
                 -2.726667	56.207778;
                 -2.731111	56.205833;
                 -2.735556	56.203889;
                 -2.741667	56.202778;
                 -2.748333	56.202500;
                 -2.754444	56.201389;
                 -2.760556	56.200556;
                 -2.765556	56.198889;
                 -2.771111	56.196944;
                 -2.775556	56.195000;
                 -2.780278	56.193056;
                 -2.784167	56.190556;
                 -2.787778	56.188056;
                 -2.793889	56.186944;
                 -2.800556	56.186389;
                 -2.807500	56.186111;
                 -2.813611	56.185000;
                 -2.820000	56.184444;
                 -2.826944	56.184167;
                 -2.834167	56.184167;
                 -2.841667	56.185000;
                 -2.846944	56.186667;
                 -2.851667	56.188889;
                 -2.856944	56.190556;
                 -2.861389	56.192778;
                 -2.865833	56.195000;
                 -2.870278	56.196944;
                 -2.875000	56.199167;
                 -2.880278	56.200833;
                 -2.886667	56.202222;
                 -2.894167	56.203056;
                 -2.901389	56.204167;
                 -2.909444	56.204722;
                 -2.916667	56.205556;
                 -2.924167	56.205556;
                 -2.931667	56.205556;
                 -2.938333	56.205000;
                 -2.945000	56.204444;
                 -2.951111	56.203611;
                 -2.956944	56.202500;
                 -2.961667	56.200556;
                 -2.966667	56.199167;
                 -2.971389	56.196944;
                 -2.976111	56.195000;
                 -2.979722	56.192500;
                 -2.983611	56.190000;
                 -2.986944	56.186944;
                 -2.990556	56.184444;
                 -2.993611	56.181389;
                 -2.997500	56.178889;
                 -3.001389	56.176389;
                 -3.005000	56.173889;
                 -3.009167	56.171389;
                 -3.013333	56.169444;
                 -3.018056	56.167222;
                 -3.022778	56.165278;
                 -3.027778	56.163889;
                 -3.033333	56.162222;
                 -3.038333	56.160833;
                 -3.044444	56.159722;
                 -3.049167	56.157778;
                 -3.051944	56.154722;
                 -3.055833	56.152222;
                 -3.059167	56.149167;
                 -3.062778	56.146667;
                 -3.066667	56.144167;
                 -3.071389	56.142222;
                 -3.075000	56.139722;
                 -3.078889	56.137222;
                 -3.083333	56.135000;
                 -3.088611	56.133611;
                 -3.093333	56.131667;
                 -3.097778	56.129444;
                 -3.102222	56.127500;
                 -3.105833	56.125000;
                 -3.110000	56.122500;
                 -3.113056	56.119444;
                 -3.115833	56.116389;
                 -3.121389	56.115000;
                 -3.128056	56.114444;
                 -3.134722	56.113889;
                 -3.138333	56.111389;
                 -3.141667	56.108333;
                 -3.143333	56.104444;
                 -3.145556	56.100833;
                 -3.148056	56.097222;
                 -3.150278	56.093611;
                 -3.152500	56.090278;
                 -3.154167	56.086111;
                 -3.151667	56.083056;
                 -3.150000	56.079722;
                 -3.150000	56.075833;
                 -3.150556	56.071111;
                 -3.152222	56.067222;
                 -3.154722	56.063611;
                 -3.158056	56.060556;
                 -3.161667	56.058056;
                 -3.165556	56.055556;
                 -3.172222	56.055000;
                 -3.179444	56.055000;
                 -3.187778	56.055556;
                 -3.194722	56.056389;
                 -3.203056	56.056944;
                 -3.211111	56.057500;
                 -3.216667	56.055833;
                 -3.220833	56.053889;
                 -3.226944	56.052778;
                 -3.229167	56.053611;
                 -3.228333	56.054444;
                 -3.234167	56.054167;
                 -3.240556	56.055556;
                 -3.246944	56.056944;
                 -3.254167	56.056944;
                 -3.260278	56.055833;
                 -3.265278	56.054444;
                 -3.270000	56.052222;
                 -3.274444	56.050278;
                 -3.278333	56.047778;
                 -3.283333	56.046389;
                 -3.290278	56.045833;
                 -3.295278	56.044167;
                 -3.299444	56.041667;
                 -3.302222	56.038611;
                 -3.306111	56.036111;
                 -3.311944	56.035000;
                 -3.318889	56.034722;
                 -3.324722	56.033611;
                 -3.329444	56.031667;
                 -3.333611	56.029444;
                 -3.337778	56.026944;
                 -3.341389	56.024444;
                 -3.346667	56.023056;
                 -3.353889	56.023056;
                 -3.360278	56.024167;
                 -3.367500	56.025278;
                 -3.374167	56.024722;
                 -3.378056	56.022222;
                 -3.381111	56.019167;
                 -3.384167	56.016111;
                 -3.388056	56.013611;
                 -3.392222	56.011389;
                 -3.399167	56.010833;
                 -3.404444	56.012222;
                 -3.404444	56.012778;
                 -3.409444	56.013056;
                 -3.414167	56.015000;
                 -3.419444	56.016944;
                 -3.425833	56.018056;
                 -3.432778	56.018889;
                 -3.441111	56.019444;
                 -3.446667	56.021111;
                 -3.451111	56.023333;
                 -3.455556	56.025556;
                 -3.460833	56.027222;
                 -3.468333	56.028056;
                 -3.474444	56.026944;
                 -3.480278	56.026111;
                 -3.487500	56.026944;
                 -3.491944	56.028889;
                 -3.494722	56.031944;
                 -3.497222	56.034722;
                 -3.503611	56.036111;
                 -3.511111	56.036111;
                 -3.517222	56.035000;
                 -3.523056	56.034167;
                 -3.530278	56.035000;
                 -3.535833	56.036667;
                 -3.540278	56.038889;
                 -3.545000	56.040833;
                 -3.548611	56.043333;
                 -3.553056	56.045556;
                 -3.556667	56.048056;
                 -3.561389	56.050278;
                 -3.565556	56.052222;
                 -3.571389	56.053889;
                 -3.578333	56.054722;
                 -3.582500	56.055556;
                 -3.588333	56.054444;
                 -3.594444	56.053611;
                 -3.600278	56.052500;
                 -3.606389	56.051389;
                 -3.613056	56.050833;
                 -3.620278	56.050833;
                 -3.628611	56.051389;
                 -3.634444	56.050278;
                 -3.641389	56.049722;
                 -3.648611	56.049722;
                 -3.656111	56.050556;
                 -3.662222	56.051111;
                 -3.668333	56.050000;
                 -3.675000	56.049444;
                 -3.682500	56.049444;
                 -3.688611	56.048056;
                 -3.692500	56.045556;
                 -3.694444	56.041944;
                 -3.695278	56.038333;
                 -3.690000	56.036667;
                 -3.684444	56.034722;
                 -3.681667	56.031944;
                 -3.681667	56.027778;
                 -3.684444	56.024722;
                 -3.686111	56.021667;
                 -3.680556	56.020000;
                 -3.675000	56.018333;
                 -3.672500	56.015278;
                 -3.670556	56.011944;
                 -3.666944	56.009444;
                 -3.662222	56.007222;
                 -3.655000	56.006389;
                 -3.648333	56.006944;
                 -3.643333	56.008611;
                 -3.637222	56.009444;
                 -3.631944	56.011111;
                 -3.626667	56.012500;
                 -3.620000	56.013056;
                 -3.613611	56.013611;
                 -3.606667	56.014167;
                 -3.600000	56.014722;
                 -3.593611	56.015278;
                 -3.585833	56.015278;
                 -3.579444	56.015000;
                 -3.573056	56.013611;
                 -3.567500	56.011944;
                 -3.560556	56.011111;
                 -3.555000	56.009167;
                 -3.549444	56.007500;
                 -3.545000	56.005556;
                 -3.540278	56.005278;
                 -3.533333	56.005833;
                 -3.526944	56.006111;
                 -3.519167	56.006389;
                 -3.511111	56.005833;
                 -3.505833	56.004167;
                 -3.501111	56.001944;
                 -3.496944	55.999722;
                 -3.492222	55.997778;
                 -3.487500	55.995556;
                 -3.482222	55.993889;
                 -3.474167	55.993333;
                 -3.468889	55.994167;
                 -3.461667	55.994167;
                 -3.454167	55.994167;
                 -3.447500	55.994444;
                 -3.440278	55.994722;
                 -3.434444	55.993889;
                 -3.429167	55.991944;
                 -3.423611	55.990278;
                 -3.417222	55.989167;
                 -3.410000	55.989167;
                 -3.403333	55.989722;
                 -3.395278	55.989167;
                 -3.388889	55.987778;
                 -3.382500	55.986389;
                 -3.376111	55.985278;
                 -3.369722	55.984722;
                 -3.365278	55.986944;
                 -3.360556	55.988889;
                 -3.356944	55.991389;
                 -3.353056	55.993889;
                 -3.347222	55.995000;
                 -3.340556	55.994444;
                 -3.335278	55.992778;
                 -3.330556	55.990556;
                 -3.326111	55.988611;
                 -3.321667	55.986389;
                 -3.317222	55.984444;
                 -3.311667	55.982500;
                 -3.307222	55.980556;
                 -3.302778	55.978333;
                 -3.297222	55.976667;
                 -3.290833	55.975278;
                 -3.283333	55.975278;
                 -3.277500	55.976389;
                 -3.270278	55.975556;
                 -3.263889	55.974167;
                 -3.257500	55.973889;
                 -3.252222	55.975278;
                 -3.246944	55.976667;
                 -3.240833	55.977778;
                 -3.233889	55.976944;
                 -3.226389	55.976944;
                 -3.220556	55.978056;
                 -3.213611	55.978333;
                 -3.206667	55.977500;
                 -3.198611	55.976944;
                 -3.191944	55.976667;
                 -3.184444	55.976667;
                 -3.177778	55.977222;
                 -3.170278	55.977222;
                 -3.162222	55.976667;
                 -3.155000	55.975556;
                 -3.148333	55.975278;
                 -3.141389	55.974444;
                 -3.135000	55.973056;
                 -3.129722	55.971389;
                 -3.125000	55.969167;
                 -3.120556	55.966944;
                 -3.116111	55.965000;
                 -3.112500	55.962500;
                 -3.107778	55.960278;
                 -3.103611	55.958056;
                 -3.098889	55.956111;
                 -3.094444	55.953889;
                 -3.089722	55.951667;
                 -3.084444	55.950000;
                 -3.079167	55.948333;
                 -3.072778	55.946944;
                 -3.066389	55.945556;
                 -3.059444	55.944722;
                 -3.051389	55.944167;
                 -3.045278	55.945278;
                 -3.039444	55.946389;
                 -3.033333	55.947222;
                 -3.025278	55.946667;
                 -3.018056	55.945833;
                 -3.012500	55.944167;
                 -3.006111	55.942778;
                 -3.001111	55.943333;
                 -2.995833	55.945000;
                 -2.991389	55.946944;
                 -2.988333	55.950000;
                 -2.984722	55.952500;
                 -2.980000	55.954444;
                 -2.975278	55.956389;
                 -2.971667	55.958889;
                 -2.966944	55.961111;
                 -2.963333	55.963611;
                 -2.959444	55.966111;
                 -2.955000	55.968056;
                 -2.948889	55.969167;
                 -2.941667	55.968889;
                 -2.933611	55.968611;
                 -2.925278	55.968056;
                 -2.918056	55.968056;
                 -2.911389	55.968333;
                 -2.905278	55.969444;
                 -2.900278	55.970833;
                 -2.895556	55.973056;
                 -2.891944	55.975556;
                 -2.888056	55.978056;
                 -2.884167	55.980556;
                 -2.881111	55.983611;
                 -2.879444	55.987500;
                 -2.878333	55.992222;
                 -2.878333	55.996389;
                 -2.879167	56.000000;
                 -2.877778	56.004167;
                 -2.874444	56.006944;
                 -2.869167	56.008611;
                 -2.861944	56.008611;
                 -2.854444	56.008611;
                 -2.851944	56.011944;
                 -2.853889	56.015278;
                 -2.856389	56.018333;
                 -2.858056	56.021667;
                 -2.859167	56.025278;
                 -2.858056	56.030000;
                 -2.853889	56.031944;
                 -2.847778	56.033056;
                 -2.841111	56.033333;
                 -2.834167	56.033889;
                 -2.831389	56.036944;
                 -2.830278	56.041389;
                 -2.830000	56.045556;
                 -2.827222	56.047778;
                 -2.821389	56.048611;
                 -2.814722	56.049167;
                 -2.810833	56.051667;
                 -2.809167	56.055833;
                 -2.806667	56.059167;
                 -2.801389	56.060833;
                 -2.793889	56.060833;
                 -2.789167	56.061389;
                 -2.782222	56.061944;
                 -2.774167	56.061389;
                 -2.768889	56.059444;
                 -2.763056	56.057778;
                 -2.755000	56.057222;
                 -2.746944	56.056667;
                 -2.740833	56.057778;
                 -2.735833	56.059167;
                 -2.730556	56.060833;
                 -2.724444	56.061667;
                 -2.716944	56.061667;
                 -2.711667	56.060000;
                 -2.705278	56.058611;
                 -2.698333	56.057778;
                 -2.690000	56.057222;
                 -2.682500	56.057222;
                 -2.676389	56.058056;
                 -2.670556	56.059167;
                 -2.664444	56.060000;
                 -2.657222	56.060000;
                 -2.650000	56.059167;
                 -2.643611	56.057778;
                 -2.637222	56.056389;
                 -2.631111	56.055000;
                 -2.625833	56.053333;
                 -2.620000	56.051667;
                 -2.615833	56.049444;
                 -2.611111	56.047222;
                 -2.605833	56.045556;
                 -2.600278	56.043889;
                 -2.596944	56.041389;
                 -2.594167	56.038333;
                 -2.591667	56.035278;
                 -2.589722	56.031944;
                 -2.587222	56.029167;
                 -2.584444	56.026111;
                 -2.579722	56.023889;
                 -2.574444	56.022222;
                 -2.571111	56.019722;
                 -2.570000	56.015833;
                 -2.571667	56.011944;
                 -2.575000	56.008889;
                 -2.578056	56.005833;
                 -2.580278	56.002500;
                 -2.580000	56.000556;
                 -2.579444	56.000000;
                 -2.577500	55.998056;
                 -2.572778	55.995833;
                 -2.568056	55.994722;
                 -2.563333	55.994444;
                 -2.560000	55.997222;
                 -2.556944	56.000278;
                 -2.552500	56.002222;
                 -2.547222	56.000556;
                 -2.542500	55.998333;
                 -2.537222	55.996667;
                 -2.529722	55.996667;
                 -2.525833	55.999167;
                 -2.521389	56.001111;
                 -2.516111	56.002778;
                 -2.510000	56.003611;
                 -2.501944	56.003056;
                 -2.497222	56.001944;
                 -2.493056	55.999722;
                 -2.489167	55.997222;
                 -2.485833	55.994444;
                 -2.479444	55.993333;
                 -2.472778	55.992778;
                 -2.466667	55.991389;
                 -2.461111	55.989722;
                 -2.457778	55.987222;
                 -2.454167	55.984722;
                 -2.448611	55.982778;
                 -2.440556	55.982222;
                 -2.433333	55.981389;
                 -2.428889	55.979167;
                 -2.423611	55.977500;
                 -2.419167	55.975278;
                 -2.415556	55.972778;
                 -2.411111	55.970556;
                 -2.406389	55.968333;
                 -2.401111	55.966667;
                 -2.394722	55.965278;
                 -2.388611	55.963889;
                 -2.384167	55.961944;
                 -2.379722	55.959722;
                 -2.375278	55.957500;
                 -2.371667	55.955000;
                 -2.368056	55.952500;
                 -2.364444	55.949722;
                 -2.360000	55.947778;
                 -2.356389	55.945278;
                 -2.351944	55.943056;
                 -2.346667	55.941111;
                 -2.340278	55.940000;
                 -2.334167	55.938611;
                 -2.328611	55.936667;
                 -2.324167	55.934722;
                 -2.320833	55.931944;
                 -2.316111	55.930000;
                 -2.310000	55.928611;
                 -2.301667	55.928056;
                 -2.294722	55.926944;
                 -2.288333	55.925556;
                 -2.283056	55.923889;
                 -2.276944	55.922500;
                 -2.270556	55.921111;
                 -2.265556	55.920833;
                 -2.260278	55.919167;
                 -2.252222	55.918611;
                 -2.245278	55.918889;
                 -2.242500	55.921944;
                 -2.238611	55.924444;
                 -2.232500	55.925556;
                 -2.225000	55.925278;
                 -2.219444	55.924444;
                 -2.212222	55.924444;
                 -2.204167	55.923889;
                 -2.198611	55.922222;
                 -2.193333	55.920278;
                 -2.186944	55.918889;
                 -2.180556	55.917778;
                 -2.174444	55.916389;
                 -2.169167	55.914444;
                 -2.163889	55.912778;
                 -2.156389	55.911667;
                 -2.149444	55.910833;
                 -2.141111	55.910278;
                 -2.133889	55.910278;
                 -2.129444	55.908056;
                 -2.126667	55.905000;
                 -2.126944	55.900833;
                 -2.128611	55.896944;
                 -2.131111	55.893333;
                 -2.132778	55.889444;
                 -2.130000	55.886389;
                 -2.125833	55.884167;
                 -2.120278	55.882500;
                 -2.115000	55.880833;
                 -2.111667	55.878056;
                 -2.107778	55.875556;
                 -2.102500	55.873889;
                 -2.095556	55.872778;
                 -2.089167	55.871389;
                 -2.083611	55.869722;
                 -2.079444	55.867500;
                 -2.075833	55.865000;
                 -2.074167	55.861667;
                 -2.072500	55.858333;
                 -2.072500	55.854167;
                 -2.069722	55.851111;
                 -2.068056	55.847778;
                 -2.064722	55.845278;
                 -2.061944	55.842222;
                 -2.060833	55.840278;
                 -2.057500	55.837778;
                 -2.053889	55.835278;
                 -2.049444	55.833056;
                 -2.046944	55.830278;
                 -2.043333	55.827500;
                 -2.039722	55.825000;
                 -2.037222	55.821944;
                 -2.033611	55.819444;
                 -2.030000	55.816944;
                 -2.027500	55.813889;
                 -2.025000	55.811111;
                 -2.022222	55.808056;
                 -2.020556	55.804722;
                 -2.018056	55.801667;
                 -2.015278	55.798889;
                 -2.013611	55.795556;
                 -2.011111	55.792500;
                 -2.008611	55.789444;
                 -2.005833	55.786667;
                 -2.003056	55.783611;
                 -2.000833	55.780556;
                 -1.996944	55.778056;
                 -1.994722	55.775278;
                 -1.990833	55.772500;
                 -1.988333	55.769722;
                 -1.985833	55.766667;
                 -1.984167	55.763333;
                 -1.981389	55.760278;
                 -1.979722	55.756944;
                 -1.976389	55.754444;
                 -1.972778	55.751944;
                 -1.968333	55.749722;
                 -1.963889	55.747500;
                 -1.958611	55.745833;
                 -1.954167	55.743611;
                 -1.949722	55.741389;
                 -1.946389	55.738889;
                 -1.942500	55.736389;
                 -1.939167	55.733889;
                 -1.936667	55.730833;
                 -1.934167	55.727778;
                 -1.931389	55.725000;
                 -1.928889	55.721944;
                 -1.925278	55.719444;
                 -1.922778	55.716389;
                 -1.918333	55.714167;
                 -1.914722	55.711667;
                 -1.910278	55.709444;
                 -1.906111	55.707222;
                 -1.900556	55.705556;
                 -1.896111	55.703333;
                 -1.891944	55.701111;
                 -1.887500	55.699167;
                 -1.884167	55.696389;
                 -1.880278	55.693889;
                 -1.876944	55.691389;
                 -1.875278	55.688056;
                 -1.872500	55.685000;
                 -1.870000	55.682222;
                 -1.867500	55.679167;
                 -1.865000	55.676111;
                 -1.862222	55.673333;
                 -1.860556	55.670000;
                 -1.857778	55.666944;
                 -1.856389	55.663611;
                 -1.853889	55.660556;
                 -1.851111	55.657500;
                 -1.847778	55.655000;
                 -1.843333	55.652778;
                 -1.839722	55.650278;
                 -1.836389	55.647778;
                 -1.831667	55.645556;
                 -1.828333	55.643056;
                 -1.825000	55.640556;
                 -1.822222	55.637500;
                 -1.819722	55.634444;
                 -1.815556	55.632500;
                 -1.808333	55.631389;
                 -1.801389	55.631944;
                 -1.797778	55.634444;
                 -1.794722	55.637222;
                 -1.792778	55.641389;
                 -1.791667	55.645833;
                 -1.790833	55.650278;
                 -1.787222	55.651944;
                 -1.782500	55.650833;
                 -1.779167	55.648056;
                 -1.775278	55.645556;
                 -1.773056	55.642500;
                 -1.770278	55.639722;
                 -1.766944	55.637222;
                 -1.764167	55.634167;
                 -1.760833	55.631667;
                 -1.756389	55.629444;
                 -1.751111	55.627500;
                 -1.746667	55.625278;
                 -1.745000	55.621944;
                 -1.748889	55.619444;
                 -1.755000	55.618611;
                 -1.760833	55.617778;
                 -1.766111	55.616111;
                 -1.768333	55.612778;
                 -1.766944	55.609444;
                 -1.763056	55.606944;
                 -1.758056	55.605000;
                 -1.750833	55.605000;
                 -1.746944	55.607500;
                 -1.743056	55.610000;
                 -1.740000	55.611944;
                 -1.734167	55.613056;
                 -1.727500	55.613333;
                 -1.719444	55.612778;
                 -1.713056	55.611389;
                 -1.707778	55.609722;
                 -1.701389	55.609167;
                 -1.695278	55.607778;
                 -1.690000	55.605833;
                 -1.685556	55.603889;
                 -1.682222	55.601111;
                 -1.678333	55.598611;
                 -1.674444	55.596389;
                 -1.670000	55.594167;
                 -1.664444	55.592500;
                 -1.659167	55.590556;
                 -1.654167	55.588889;
                 -1.648611	55.587222;
                 -1.643333	55.585278;
                 -1.638056	55.583611;
                 -1.633889	55.581389;
                 -1.631944	55.578056;
                 -1.634444	55.574444;
                 -1.637778	55.571389;
                 -1.639444	55.567500;
                 -1.637778	55.564167;
                 -1.634167	55.561667;
                 -1.630000	55.559444;
                 -1.626389	55.556944;
                 -1.623056	55.554167;
                 -1.620278	55.551389;
                 -1.620556	55.547222;
                 -1.623889	55.544167;
                 -1.626944	55.541111;
                 -1.630000	55.538056;
                 -1.633056	55.535278;
                 -1.631667	55.531667;
                 -1.624444	55.530833;
                 -1.618056	55.529444;
                 -1.614722	55.526944;
                 -1.610278	55.524722;
                 -1.607778	55.521667;
                 -1.604167	55.519167;
                 -1.605556	55.514722;
                 -1.607222	55.510556;
                 -1.608056	55.506944;
                 -1.608056	55.502778;
                 -1.606389	55.499444;
                 -1.603889	55.496389;
                 -1.600556	55.493889;
                 -1.596111	55.491667;
                 -1.591667	55.489444;
                 -1.585556	55.488056;
                 -1.581389	55.485833;
                 -1.580556	55.482222;
                 -1.580556	55.478056;
                 -1.581667	55.473611;
                 -1.581944	55.469444;
                 -1.583056	55.464722;
                 -1.583333	55.460833;
                 -1.582500	55.456944;
                 -1.581667	55.453333;
                 -1.581667	55.449167;
                 -1.583056	55.444444;
                 -1.583056	55.440278;
                 -1.581667	55.436944;
                 -1.579722	55.433611;
                 -1.578056	55.430278;
                 -1.575556	55.427222;
                 -1.573889	55.423889;
                 -1.573333	55.420278;
                 -1.574167	55.415833;
                 -1.575556	55.411111;
                 -1.577222	55.407222;
                 -1.579722	55.403611;
                 -1.581944	55.400000;
                 -1.585278	55.397222;
                 -1.589167	55.394722;
                 -1.593056	55.392222;
                 -1.596111	55.389167;
                 -1.599167	55.386111;
                 -1.598333	55.382500;
                 -1.597222	55.380556;
                 -1.594722	55.377500;
                 -1.592222	55.374722;
                 -1.590556	55.371111;
                 -1.587778	55.368333;
                 -1.586389	55.365000;
                 -1.584722	55.361667;
                 -1.583056	55.358056;
                 -1.581667	55.354722;
                 -1.580833	55.351111;
                 -1.580833	55.346944;
                 -1.579444	55.343611;
                 -1.578333	55.339722;
                 -1.574167	55.337778;
                 -1.567222	55.336667;
                 -1.561667	55.334722;
                 -1.558333	55.332222;
                 -1.555833	55.329444;
                 -1.552222	55.326667;
                 -1.549722	55.323889;
                 -1.547500	55.320833;
                 -1.547500	55.316667;
                 -1.549444	55.312500;
                 -1.551667	55.309167;
                 -1.555000	55.306111;
                 -1.557500	55.302778;
                 -1.559722	55.299167;
                 -1.562222	55.295556;
                 -1.563889	55.291667;
                 -1.565556	55.287500;
                 -1.567500	55.283611;
                 -1.568611	55.278889;
                 -1.567778	55.275278;
                 -1.566944	55.272500;
                 -1.565278	55.269167;
                 -1.563611	55.265833;
                 -1.561111	55.262778;
                 -1.559444	55.259444;
                 -1.557778	55.256111;
                 -1.555278	55.253056;
                 -1.552778	55.250000;
                 -1.549444	55.247500;
                 -1.545833	55.245000;
                 -1.542500	55.242500;
                 -1.538056	55.240278;
                 -1.534722	55.237500;
                 -1.531389	55.235000;
                 -1.528611	55.232222;
                 -1.526111	55.229167;
                 -1.523611	55.226111;
                 -1.524722	55.221667;
                 -1.526389	55.217500;
                 -1.528889	55.214167;
                 -1.530833	55.210000;
                 -1.528056	55.207222;
                 -1.524722	55.204444;
                 -1.521111	55.201944;
                 -1.516944	55.199722;
                 -1.513333	55.197222;
                 -1.510000	55.194722;
                 -1.507500	55.191667;
                 -1.505833	55.188333;
                 -1.506111	55.184167;
                 -1.508611	55.180556;
                 -1.511667	55.177222;
                 -1.515000	55.174167;
                 -1.518056	55.171111;
                 -1.521111	55.168333;
                 -1.520556	55.164444;
                 -1.518889	55.161111;
                 -1.518056	55.157500;
                 -1.516667	55.153611;
                 -1.515000	55.150278;
                 -1.512500	55.147222;
                 -1.509167	55.144722;
                 -1.505556	55.141944;
                 -1.502222	55.139444;
                 -1.498889	55.136944;
                 -1.496944	55.133611;
                 -1.496389	55.129722;
                 -1.494722	55.126389;
                 -1.493889	55.122778;
                 -1.492500	55.119444;
                 -1.491667	55.115556;
                 -1.490000	55.112222;
                 -1.489167	55.108611;
                 -1.487500	55.105000;
                 -1.486944	55.101389;
                 -1.486389	55.097500;
                 -1.485556	55.093889;
                 -1.485000	55.090000;
                 -1.482500	55.086667;
                 -1.480833	55.083333;
                 -1.477222	55.080833;
                 -1.472778	55.078611;
                 -1.466667	55.077222;
                 -1.459722	55.076944;
                 -1.453333	55.076667;
                 -1.450556	55.073611;
                 -1.451944	55.069167;
                 -1.452778	55.064444;
                 -1.453889	55.060833;
                 -1.454167	55.056667;
                 -1.453333	55.053056;
                 -1.450556	55.050000;
                 -1.448333	55.046944;
                 -1.445278	55.045833;
                 -1.446111	55.046111;
                 -1.445278	55.045833;
                 -1.442500	55.043889;
                 -1.439167	55.041389;
                 -1.435833	55.038611;
                 -1.432500	55.036111;
                 -1.429722	55.033056;
                 -1.428333	55.029722;
                 -1.426389	55.026389;
                 -1.425833	55.022778;
                 -1.425000	55.018889;
                 -1.423611	55.015556;
                 -1.422778	55.011944;
                 -1.420278	55.008889;
                 -1.418611	55.005556;
                 -1.415278	55.002778;
                 -1.412778	55.000000;
                 -1.409444	54.997500;
                 -1.405278	54.994722;
                 -1.401944	54.992222;
                 -1.398333	54.989444;
                 -1.395000	54.986944;
                 -1.391667	54.984444;
                 -1.388056	54.981944;
                 -1.384722	54.979167;
                 -1.381389	54.976667;
                 -1.376944	54.974444;
                 -1.373611	54.971944;
                 -1.369167	54.969722;
                 -1.365000	54.967500;
                 -1.360556	54.965278;
                 -1.358056	54.962500;
                 -1.356667	54.958611;
                 -1.356944	54.954444;
                 -1.357778	54.950000;
                 -1.359167	54.945278;
                 -1.360833	54.941389;
                 -1.361944	54.936667;
                 -1.363056	54.932222;
                 -1.364722	54.928333;
                 -1.365000	54.924167;
                 -1.363333	54.920833;
                 -1.361944	54.917500;
                 -1.359167	54.914444;
                 -1.355833	54.911944;
                 -1.352500	54.909167;
                 -1.351944	54.905556;
                 -1.352778	54.900833;
                 -1.353889	54.896389;
                 -1.353889	54.892222;
                 -1.352500	54.888889;
                 -1.351944	54.885278;
                 -1.350000	54.881667;
                 -1.348611	54.878333;
                 -1.346944	54.875000;
                 -1.345556	54.871667;
                 -1.342778	54.868889;
                 -1.340278	54.865833;
                 -1.338889	54.862500;
                 -1.336111	54.859444;
                 -1.334722	54.856111;
                 -1.332222	54.853056;
                 -1.330556	54.849722;
                 -1.329167	54.846389;
                 -1.328333	54.842500;
                 -1.326667	54.839167;
                 -1.325278	54.835833;
                 -1.324444	54.832222;
                 -1.321667	54.829167;
                 -1.320278	54.825833;
                 -1.317778	54.822778;
                 -1.316111	54.819444;
                 -1.314722	54.816111;
                 -1.311389	54.813611;
                 -1.308611	54.810556;
                 -1.306111	54.807500;
                 -1.303611	54.804722;
                 -1.303056	54.800833;
                 -1.303333	54.796667;
                 -1.303333	54.792500;
                 -1.304444	54.788056;
                 -1.304722	54.783889;
                 -1.305000	54.779722;
                 -1.305278	54.775556;
                 -1.304444	54.771944;
                 -1.301944	54.768889;
                 -1.299444	54.765833;
                 -1.297500	54.763889;
                 -1.294167	54.761389;
                 -1.290833	54.758889;
                 -1.287222	54.756111;
                 -1.283889	54.753611;
                 -1.279722	54.751389;
                 -1.275278	54.749167;
                 -1.271111	54.746944;
                 -1.266944	54.744722;
                 -1.263333	54.742222;
                 -1.261111	54.739167;
                 -1.258611	54.736389;
                 -1.256944	54.733056;
                 -1.254444	54.730000;
                 -1.251944	54.726944;
                 -1.248611	54.724444;
                 -1.244444	54.722222;
                 -1.239167	54.720556;
                 -1.235000	54.718333;
                 -1.229722	54.716389;
                 -1.225278	54.714167;
                 -1.220278	54.712500;
                 -1.215000	54.710556;
                 -1.208889	54.709167;
                 -1.203889	54.707500;
                 -1.197778	54.706111;
                 -1.192500	54.704167;
                 -1.186389	54.702778;
                 -1.181389	54.701111;
                 -1.175556	54.699444;
                 -1.171111	54.697222;
                 -1.168611	54.694444;
                 -1.172500	54.691944;
                 -1.177778	54.690556;
                 -1.183333	54.689722;
                 -1.189167	54.688611;
                 -1.192500	54.685556;
                 -1.190000	54.682778;
                 -1.188333	54.679444;
                 -1.186667	54.676111;
                 -1.186111	54.672222;
                 -1.185278	54.668611;
                 -1.183889	54.665000;
                 -1.183056	54.661389;
                 -1.181389	54.658056;
                 -1.178889	54.655000;
                 -1.173889	54.653333;
                 -1.166111	54.652500;
                 -1.160833	54.650833;
                 -1.162222	54.647778;
                 -1.166944	54.645833;
                 -1.169444	54.642222;
                 -1.169722	54.638056;
                 -1.172222	54.634444;
                 -1.176389	54.632500;
                 -1.181667	54.631111;
                 -1.186111	54.629167;
                 -1.190556	54.627222;
                 -1.193611	54.624444;
                 -1.195556	54.620278;
                 -1.195556	54.616111;
                 -1.195000	54.612500;
                 -1.193333	54.609167;
                 -1.191944	54.605556;
                 -1.188611	54.603056;
                 -1.185833	54.601111;
                 -1.182500	54.598333;
                 -1.176389	54.596944;
                 -1.169444	54.596111;
                 -1.163056	54.595556;
                 -1.156667	54.595833;
                 -1.150278	54.596389;
                 -1.145000	54.597778;
                 -1.140556	54.599722;
                 -1.136667	54.602222;
                 -1.135000	54.606111;
                 -1.134722	54.610278;
                 -1.137222	54.613333;
                 -1.141667	54.615556;
                 -1.144167	54.618333;
                 -1.143889	54.622500;
                 -1.142222	54.626667;
                 -1.138889	54.629444;
                 -1.136389	54.633056;
                 -1.131389	54.634444;
                 -1.126667	54.634167;
                 -1.120556	54.632778;
                 -1.116111	54.630556;
                 -1.110278	54.629167;
                 -1.105278	54.627222;
                 -1.100000	54.625556;
                 -1.094722	54.623611;
                 -1.088889	54.622222;
                 -1.082778	54.620833;
                 -1.075000	54.620278;
                 -1.068056	54.619167;
                 -1.061111	54.618056;
                 -1.055000	54.616667;
                 -1.049167	54.615278;
                 -1.043889	54.613333;
                 -1.038889	54.611667;
                 -1.033611	54.609722;
                 -1.030278	54.607222;
                 -1.025833	54.605000;
                 -1.023056	54.603056;
                 -1.023056	54.603333;
                 -1.023889	54.603333;
                 -1.020278	54.601667;
                 -1.016111	54.599722;
                 -1.011944	54.597500;
                 -1.006944	54.595556;
                 -1.001667	54.593889;
                 -0.996667	54.591944;
                 -0.990556	54.590556;
                 -0.985278	54.588611;
                 -0.980278	54.586944;
                 -0.975000	54.585000;
                 -0.968056	54.584167;
                 -0.960833	54.583889;
                 -0.954722	54.584167;
                 -0.948056	54.584722;
                 -0.941389	54.585000;
                 -0.935000	54.585556;
                 -0.927222	54.584722;
                 -0.921111	54.583333;
                 -0.916111	54.581667;
                 -0.910833	54.579722;
                 -0.906667	54.577500;
                 -0.902222	54.575278;
                 -0.897222	54.573611;
                 -0.890278	54.572500;
                 -0.883611	54.571389;
                 -0.875833	54.570833;
                 -0.869167	54.571111;
                 -0.862778	54.571389;
                 -0.856111	54.571944;
                 -0.849167	54.571667;
                 -0.842222	54.570833;
                 -0.837222	54.568889;
                 -0.831944	54.566944;
                 -0.827500	54.565000;
                 -0.822500	54.563056;
                 -0.817500	54.561111;
                 -0.811389	54.559722;
                 -0.804444	54.558611;
                 -0.797500	54.558611;
                 -0.790278	54.558333;
                 -0.783056	54.558333;
                 -0.777222	54.556944;
                 -0.773056	54.554722;
                 -0.769444	54.551944;
                 -0.766111	54.549444;
                 -0.763611	54.546389;
                 -0.760556	54.543889;
                 -0.756944	54.541389;
                 -0.753056	54.539167;
                 -0.747778	54.537222;
                 -0.741667	54.535833;
                 -0.736667	54.533889;
                 -0.734167	54.531111;
                 -0.730000	54.528889;
                 -0.725556	54.526667;
                 -0.720833	54.524722;
                 -0.713611	54.523611;
                 -0.706667	54.522778;
                 -0.700556	54.521111;
                 -0.693889	54.520278;
                 -0.686944	54.519167;
                 -0.680833	54.517778;
                 -0.675833	54.515833;
                 -0.670556	54.514167;
                 -0.666389	54.511389;
                 -0.662222	54.509167;
                 -0.659167	54.506667;
                 -0.655833	54.503889;
                 -0.652222	54.501389;
                 -0.649167	54.498889;
                 -0.644722	54.496667;
                 -0.640000	54.494722;
                 -0.634722	54.492778;
                 -0.629722	54.491111;
                 -0.623611	54.489722;
                 -0.618333	54.487778;
                 -0.611667	54.486667;
                 -0.604722	54.485556;
                 -0.599722	54.486944;
                 -0.593056	54.487500;
                 -0.587222	54.486111;
                 -0.583611	54.483333;
                 -0.578611	54.481667;
                 -0.571667	54.481389;
                 -0.564722	54.480278;
                 -0.562222	54.477222;
                 -0.560833	54.473889;
                 -0.558333	54.471111;
                 -0.555833	54.468056;
                 -0.552500	54.465278;
                 -0.549444	54.462778;
                 -0.545000	54.460556;
                 -0.539722	54.458889;
                 -0.534722	54.456944;
                 -0.530556	54.454722;
                 -0.525278	54.452778;
                 -0.521111	54.450556;
                 -0.518056	54.448056;
                 -0.515556	54.445000;
                 -0.515000	54.441389;
                 -0.515000	54.437222;
                 -0.516389	54.432778;
                 -0.517500	54.428056;
                 -0.518889	54.423611;
                 -0.519167	54.419444;
                 -0.518333	54.415833;
                 -0.515833	54.412778;
                 -0.511667	54.410556;
                 -0.505833	54.409167;
                 -0.499722	54.407500;
                 -0.493056	54.406389;
                 -0.487778	54.404722;
                 -0.480833	54.403611;
                 -0.477500	54.401111;
                 -0.474444	54.398333;
                 -0.471944	54.395278;
                 -0.468889	54.392778;
                 -0.466389	54.389722;
                 -0.463889	54.386944;
                 -0.461389	54.383889;
                 -0.460000	54.380556;
                 -0.457500	54.377500;
                 -0.454167	54.375000;
                 -0.450833	54.372222;
                 -0.449444	54.368889;
                 -0.448056	54.365556;
                 -0.446389	54.362222;
                 -0.444722	54.358889;
                 -0.441667	54.356111;
                 -0.438333	54.353611;
                 -0.435833	54.350556;
                 -0.434444	54.347222;
                 -0.432778	54.343889;
                 -0.432500	54.340000;
                 -0.428889	54.337500;
                 -0.423889	54.335556;
                 -0.421389	54.332778;
                 -0.420000	54.329167;
                 -0.419444	54.325556;
                 -0.419722	54.321389;
                 -0.418889	54.317500;
                 -0.417500	54.314167;
                 -0.416111	54.310833;
                 -0.414444	54.307500;
                 -0.414722	54.303333;
                 -0.413333	54.300000;
                 -0.411944	54.296667;
                 -0.410278	54.293333;
                 -0.408611	54.290000;
                 -0.408333	54.286111;
                 -0.405833	54.283056;
                 -0.398889	54.281944;
                 -0.392222	54.281111;
                 -0.388056	54.278889;
                 -0.390000	54.276111;
                 -0.393333	54.273333;
                 -0.394167	54.269722;
                 -0.392778	54.266111;
                 -0.389444	54.263611;
                 -0.387778	54.261667;
                 -0.384167	54.258889;
                 -0.381944	54.256111;
                 -0.378611	54.253333;
                 -0.375556	54.250833;
                 -0.371944	54.248056;
                 -0.368056	54.245833;
                 -0.364444	54.243333;
                 -0.360278	54.241111;
                 -0.357222	54.238611;
                 -0.353056	54.236389;
                 -0.348889	54.234167;
                 -0.343889	54.232222;
                 -0.336944	54.231111;
                 -0.329167	54.230556;
                 -0.322778	54.230833;
                 -0.315833	54.229722;
                 -0.310833	54.227778;
                 -0.305556	54.226111;
                 -0.300833	54.224167;
                 -0.294722	54.222778;
                 -0.288889	54.221389;
                 -0.283611	54.219444;
                 -0.278611	54.217500;
                 -0.273611	54.215833;
                 -0.269444	54.213611;
                 -0.266111	54.210833;
                 -0.264722	54.207500;
                 -0.267778	54.204722;
                 -0.271667	54.202222;
                 -0.273333	54.198056;
                 -0.273056	54.194444;
                 -0.271389	54.191111;
                 -0.270000	54.187778;
                 -0.268611	54.184167;
                 -0.266111	54.181389;
                 -0.264722	54.178056;
                 -0.262222	54.175000;
                 -0.259167	54.172222;
                 -0.255556	54.169722;
                 -0.251389	54.167500;
                 -0.247222	54.165278;
                 -0.243056	54.163056;
                 -0.238056	54.161111;
                 -0.232222	54.159722;
                 -0.226389	54.157778;
                 -0.220556	54.156389;
                 -0.213611	54.155278;
                 -0.207500	54.153889;
                 -0.201667	54.152222;
                 -0.194722	54.151111;
                 -0.188889	54.149722;
                 -0.183056	54.148333;
                 -0.176944	54.146944;
                 -0.170278	54.145833;
                 -0.164444	54.144167;
                 -0.158333	54.142778;
                 -0.151667	54.141667;
                 -0.145556	54.140278;
                 -0.140556	54.138333;
                 -0.134444	54.136944;
                 -0.128611	54.135556;
                 -0.122778	54.133889;
                 -0.116667	54.132500;
                 -0.111944	54.130833;
                 -0.106667	54.128889;
                 -0.101667	54.126944;
                 -0.096667	54.125000;
                 -0.092500	54.122778;
                 -0.087500	54.121111;
                 -0.082500	54.119167;
                 -0.078333	54.116944;
                 -0.075833	54.113889;
                 -0.075556	54.113056;
                 -0.074722	54.112500;
                 -0.077778	54.109444;
                 -0.081111	54.106667;
                 -0.086111	54.105278;
                 -0.091389	54.103889;
                 -0.096389	54.102500;
                 -0.102222	54.101667;
                 -0.107222	54.100278;
                 -0.112222	54.098889;
                 -0.118056	54.098056;
                 -0.124444	54.097500;
                 -0.130278	54.096667;
                 -0.136111	54.095833;
                 -0.141111	54.094444;
                 -0.146111	54.093056;
                 -0.150556	54.091111;
                 -0.155833	54.089722;
                 -0.160833	54.088333;
                 -0.165278	54.086389;
                 -0.170000	54.084444;
                 -0.173611	54.082222;
                 -0.176944	54.078611;
                 -0.179444	54.075278;
                 -0.182500	54.072222;
                 -0.185000	54.068889;
                 -0.188611	54.066389;
                 -0.191944	54.063333;
                 -0.196111	54.060556;
                 -0.198889	54.057500;
                 -0.202778	54.055278;
                 -0.205278	54.051667;
                 -0.208611	54.048611;
                 -0.209722	54.044167;
                 -0.211667	54.040278;
                 -0.213333	54.036111;
                 -0.214722	54.031667;
                 -0.215833	54.027222;
                 -0.216111	54.023056;
                 -0.215556	54.019167;
                 -0.214722	54.015556;
                 -0.213333	54.012222;
                 -0.211944	54.008889;
                 -0.209444	54.005833;
                 -0.208056	54.002500;
                 -0.206667	53.998889;
                 -0.205000	53.995556;
                 -0.203611	53.992222;
                 -0.202222	53.988889;
                 -0.199722	53.985833;
                 -0.198333	53.982500;
                 -0.196111	53.979444;
                 -0.193611	53.976389;
                 -0.192222	53.973056;
                 -0.190000	53.970278;
                 -0.188333	53.966667;
                 -0.186111	53.963889;
                 -0.184444	53.960278;
                 -0.183056	53.956944;
                 -0.181667	53.953611;
                 -0.180278	53.950278;
                 -0.178611	53.946944;
                 -0.177222	53.943611;
                 -0.175833	53.940278;
                 -0.174167	53.936667;
                 -0.173611	53.933056;
                 -0.173056	53.929167;
                 -0.171667	53.925833;
                 -0.170833	53.922222;
                 -0.168611	53.919167;
                 -0.167222	53.915833;
                 -0.164722	53.912778;
                 -0.162222	53.909722;
                 -0.160278	53.906667;
                 -0.157778	53.903889;
                 -0.155556	53.900833;
                 -0.153056	53.897778;
                 -0.150556	53.894722;
                 -0.148056	53.891667;
                 -0.146111	53.888889;
                 -0.143611	53.885833;
                 -0.141389	53.882778;
                 -0.138889	53.879722;
                 -0.136389	53.876944;
                 -0.134167	53.873333;
                 -0.131944	53.870278;
                 -0.129722	53.867500;
                 -0.127222	53.864444;
                 -0.125000	53.861389;
                 -0.121667	53.858889;
                 -0.118333	53.856111;
                 -0.115278	53.853611;
                 -0.112778	53.851111;
                 -0.104167	53.844167;
                 -0.101667	53.843056;
                 -0.099167	53.840000;
                 -0.095833	53.837500;
                 -0.092778	53.834722;
                 -0.088611	53.832500;
                 -0.084444	53.830278;
                 -0.080278	53.828056;
                 -0.076111	53.825833;
                 -0.071944	53.823611;
                 -0.067778	53.821389;
                 -0.064722	53.818889;
                 -0.062500	53.815833;
                 -0.060000	53.812778;
                 -0.058611	53.809444;
                 -0.056389	53.806389;
                 -0.053889	53.803333;
                 -0.050833	53.800833;
                 -0.048333	53.797778;
                 -0.045278	53.795278;
                 -0.041944	53.792500;
                 -0.039444	53.789444;
                 -0.036389	53.786944;
                 -0.033333	53.784444;
                 -0.029722	53.781667;
                 -0.026667	53.779167;
                 -0.023333	53.776389;
                 -0.019167	53.774167;
                 -0.016111	53.771667;
                 -0.013056	53.769167;
                 -0.008889	53.766667;
                 -0.005556	53.764167;
                 -0.001389	53.761944;
                 0.001944 	53.759444;
                 0.005278 	53.756667;
                 0.008333 	53.754167;
                 0.010833 	53.751111;
                 0.013056 	53.748056;
                 0.015278 	53.745000;
                 0.018611 	53.742500;
                 0.020833 	53.739444;
                 0.023333 	53.736389;
                 0.026389 	53.733889;
                 0.028889 	53.730833;
                 0.031944 	53.728333;
                 0.034444 	53.725278;
                 0.036667 	53.722222;
                 0.038889 	53.719167;
                 0.041667 	53.717222;
                 0.045556 	53.715000;
                 0.048889 	53.712222;
                 0.053056 	53.710000;
                 0.057222 	53.707778;
                 0.061111 	53.705556;
                 0.064444 	53.703056;
                 0.066667 	53.700000;
                 0.069167 	53.696944;
                 0.071389 	53.693889;
                 0.073611 	53.691111;
                 0.076111 	53.688056;
                 0.079167 	53.685278;
                 0.082500 	53.682778;
                 0.085556 	53.680000;
                 0.088889 	53.677500;
                 0.092222 	53.675000;
                 0.095278 	53.672222;
                 0.099444 	53.670000;
                 0.101667 	53.666944;
                 0.105000 	53.664444;
                 0.108056 	53.661667;
                 0.110556 	53.658889;
                 0.113611 	53.656111;
                 0.116944 	53.653611;
                 0.120000 	53.650833;
                 0.123333 	53.648333;
                 0.126389 	53.645556;
                 0.128889 	53.642778;
                 0.131944 	53.639722;
                 0.134167 	53.636667;
                 0.136389 	53.633611;
                 0.138889 	53.630556;
                 0.141111 	53.627500;
                 0.143333 	53.624722;
                 0.145000 	53.621111;
                 0.147222 	53.618333;
                 0.149444 	53.615278;
                 0.150833 	53.611944;
                 0.152500 	53.608611;
                 0.153889 	53.605000;
                 0.154444 	53.601389;
                 0.153056 	53.596944;
                 0.151944 	53.592222;
                 0.149444 	53.588889;
                 0.146944 	53.585278;
                 0.143889 	53.582500;
                 0.140556 	53.579444;
                 0.136944 	53.576944;
                 0.133056 	53.574722;
                 0.129444 	53.572222;
                 0.125556 	53.569722;
                 0.121944 	53.567222;
                 0.118056 	53.565000;
                 0.113056 	53.563611;
                 0.113889 	53.567222;
                 0.117222 	53.570000;
                 0.119722 	53.573611;
                 0.123333 	53.576111;
                 0.126389 	53.578889;
                 0.129722 	53.581944;
                 0.132778 	53.584722;
                 0.135833 	53.587778;
                 0.137778 	53.591667;
                 0.138889 	53.596389;
                 0.138333 	53.600000;
                 0.136111 	53.603056;
                 0.133889 	53.606111;
                 0.130556 	53.608611;
                 0.126389 	53.610833;
                 0.122500 	53.613056;
                 0.117500 	53.615000;
                 0.113333 	53.617222;
                 0.109167 	53.619444;
                 0.104444 	53.621389;
                 0.099444 	53.623333;
                 0.094444 	53.625000;
                 0.089444 	53.626944;
                 0.084444 	53.628889;
                 0.078611 	53.630278;
                 0.072778 	53.631667;
                 0.067778 	53.633611;
                 0.061944 	53.635000;
                 0.056111 	53.636667;
                 0.050278 	53.638056;
                 0.044444 	53.639444;
                 0.036667 	53.640278;
                 0.029722 	53.640556;
                 0.023333 	53.640000;
                 0.016944 	53.639722;
                 0.011389 	53.638889;
                 0.006389 	53.637500;
                 0.001111 	53.636111;
                 -0.004167	53.634722;
                 -0.009167	53.633333;
                 -0.014167	53.631944;
                 -0.019167	53.630278;
                 -0.024444	53.628889;
                 -0.029167	53.627500;
                 -0.034444	53.626111;
                 -0.040000	53.625278;
                 -0.045833	53.624444;
                 -0.051389	53.623333;
                 -0.058611	53.623611;
                 -0.065000	53.623333;
                 -0.071389	53.623056;
                 -0.078889	53.623611;
                 -0.085556	53.624722;
                 -0.090556	53.626667;
                 -0.095556	53.628333;
                 -0.100556	53.630278;
                 -0.105556	53.632222;
                 -0.109722	53.634444;
                 -0.114444	53.636389;
                 -0.118611	53.638611;
                 -0.123611	53.640833;
                 -0.127778	53.643056;
                 -0.131944	53.645278;
                 -0.135833	53.647500;
                 -0.140000	53.649722;
                 -0.144167	53.651944;
                 -0.148056	53.654167;
                 -0.152222	53.656389;
                 -0.156389	53.658611;
                 -0.159722	53.661389;
                 -0.163889	53.663611;
                 -0.168611	53.665278;
                 -0.172778	53.667500;
                 -0.176667	53.669722;
                 -0.180278	53.672500;
                 -0.184167	53.674722;
                 -0.187500	53.677222;
                 -0.191667	53.679444;
                 -0.195556	53.681667;
                 -0.199722	53.683889;
                 -0.203889	53.686111;
                 -0.208889	53.688056;
                 -0.212778	53.690278;
                 -0.216389	53.692778;
                 -0.220278	53.695000;
                 -0.222778	53.698056;
                 -0.225833	53.700556;
                 -0.228333	53.703611;
                 -0.232500	53.705833;
                 -0.235556	53.708611;
                 -0.238056	53.711389;
                 -0.240278	53.714444;
                 -0.242778	53.717500;
                 -0.244167	53.720833;
                 -0.248333	53.723056;
                 -0.251944	53.723889;
                 -0.259167	53.724167;
                 -0.266667	53.724722;
                 -0.272500	53.726111;
                 -0.277222	53.728056;
                 -0.283333	53.729444;
                 -0.290000	53.730556;
                 -0.297222	53.730833;
                 -0.303333	53.730556;
                 -0.309722	53.730000;
                 -0.316111	53.729722;
                 -0.321667	53.728889;
                 -0.328056	53.728611;
                 -0.332222	53.727500;
                 -0.337500	53.726111;
                 -0.342500	53.724722;
                 -0.348056	53.723889;
                 -0.353333	53.722500;
                 -0.358889	53.721667;
                 -0.363889	53.720278;
                 -0.369167	53.718889;
                 -0.373889	53.717500;
                 -0.379167	53.716111;
                 -0.383611	53.714167;
                 -0.388056	53.712222;
                 -0.392222	53.710278;
                 -0.397500	53.708889;
                 -0.403056	53.708056;
                 -0.408889	53.706944;
                 -0.415278	53.706667;
                 -0.422222	53.706944;
                 -0.428333	53.706389;
                 -0.435556	53.706667;
                 -0.442500	53.706944;
                 -0.450278	53.707500;
                 -0.457778	53.708333;
                 -0.465278	53.708889;
                 -0.472500	53.709167;
                 -0.478889	53.708611;
                 -0.485000	53.708333;
                 -0.492778	53.708889;
                 -0.498611	53.708056;
                 -0.504167	53.707222;
                 -0.509722	53.706389;
                 -0.515000	53.705000;
                 -0.520556	53.704167;
                 -0.526389	53.702778;
                 -0.532778	53.702500;
                 -0.539167	53.701944;
                 -0.545278	53.701667;
                 -0.553056	53.702222;
                 -0.559167	53.703889;
                 -0.563889	53.705556;
                 -0.568056	53.707778;
                 -0.571389	53.710556;
                 -0.576389	53.712222;
                 -0.582222	53.713611;
                 -0.587500	53.715556;
                 -0.592222	53.717500;
                 -0.598056	53.718889;
                 -0.605000	53.720000;
                 -0.610833	53.721389;
                 -0.618333	53.721944;
                 -0.626111	53.722778;
                 -0.631944	53.721667;
                 -0.637500	53.720833;
                 -0.643333	53.720000;
                 -0.648333	53.718611;
                 -0.652500	53.716667;
                 -0.656944	53.714722;
                 -0.661389	53.712778;
                 -0.665833	53.710833;
                 -0.670278	53.708889;
                 -0.674167	53.706389;
                 -0.678333	53.704722;
                 -0.682778	53.702778;
                 -0.687222	53.700833;
                 -0.692222	53.699444;
                 -0.698056	53.698333;
                 -0.704167	53.698056;
                 -0.711111	53.698333;
                 -0.718333	53.698333;
                 -0.723889	53.697500;
                 -0.723333	53.693611;
                 -0.718333	53.691389;
                 -0.710556	53.690833;
                 -0.704722	53.689444;
                 -0.700556	53.687222;
                 -0.698333	53.684167;
                 -0.696667	53.680833;
                 -0.692500	53.679444;
                 -0.688333	53.680556;
                 -0.684444	53.683056;
                 -0.681389	53.685833;
                 -0.676944	53.687778;
                 -0.672500	53.689722;
                 -0.667778	53.691111;
                 -0.661944	53.691944;
                 -0.656389	53.693056;
                 -0.650556	53.693889;
                 -0.644722	53.694722;
                 -0.640000	53.696111;
                 -0.634167	53.696944;
                 -0.629167	53.698333;
                 -0.623889	53.699722;
                 -0.619722	53.701667;
                 -0.614722	53.703056;
                 -0.608333	53.703611;
                 -0.601667	53.702500;
                 -0.596389	53.700556;
                 -0.592500	53.698333;
                 -0.589167	53.695833;
                 -0.586667	53.692778;
                 -0.584444	53.689722;
                 -0.581944	53.686944;
                 -0.579722	53.683889;
                 -0.576389	53.681389;
                 -0.572222	53.679167;
                 -0.568056	53.676944;
                 -0.563333	53.675000;
                 -0.557222	53.673611;
                 -0.551389	53.672222;
                 -0.544722	53.671111;
                 -0.537500	53.670833;
                 -0.530556	53.670833;
                 -0.525000	53.671667;
                 -0.519167	53.672500;
                 -0.513611	53.673333;
                 -0.507778	53.674167;
                 -0.502778	53.675556;
                 -0.497778	53.676944;
                 -0.492778	53.678333;
                 -0.487500	53.680000;
                 -0.483333	53.681667;
                 -0.478333	53.683333;
                 -0.473889	53.685000;
                 -0.469444	53.686944;
                 -0.464444	53.688333;
                 -0.459444	53.689722;
                 -0.453056	53.690278;
                 -0.445278	53.689444;
                 -0.438611	53.688333;
                 -0.431944	53.687500;
                 -0.424722	53.687222;
                 -0.417778	53.686944;
                 -0.411667	53.687500;
                 -0.405833	53.688333;
                 -0.400278	53.689167;
                 -0.393889	53.689444;
                 -0.388056	53.690278;
                 -0.382222	53.691389;
                 -0.376667	53.692222;
                 -0.371111	53.693056;
                 -0.365278	53.693889;
                 -0.359722	53.694722;
                 -0.353889	53.695556;
                 -0.348889	53.696944;
                 -0.343333	53.697778;
                 -0.337500	53.698889;
                 -0.331667	53.699722;
                 -0.326667	53.701111;
                 -0.321111	53.701944;
                 -0.315278	53.702778;
                 -0.309444	53.703611;
                 -0.303333	53.703889;
                 -0.296944	53.704444;
                 -0.290000	53.704167;
                 -0.283333	53.703056;
                 -0.279167	53.700833;
                 -0.275000	53.698611;
                 -0.271667	53.696111;
                 -0.269167	53.693056;
                 -0.266111	53.690556;
                 -0.263056	53.687778;
                 -0.260556	53.684722;
                 -0.258056	53.681944;
                 -0.255000	53.679167;
                 -0.251667	53.676667;
                 -0.248611	53.673889;
                 -0.245000	53.671389;
                 -0.243056	53.668333;
                 -0.239722	53.665833;
                 -0.237222	53.662778;
                 -0.235000	53.659722;
                 -0.232778	53.656944;
                 -0.230278	53.653889;
                 -0.228056	53.650833;
                 -0.224722	53.647778;
                 -0.222500	53.644722;
                 -0.219167	53.642222;
                 -0.216944	53.639167;
                 -0.213611	53.636667;
                 -0.210556	53.633889;
                 -0.207222	53.631389;
                 -0.202500	53.629444;
                 -0.198333	53.627222;
                 -0.194167	53.625000;
                 -0.190000	53.622778;
                 -0.185833	53.620556;
                 -0.181667	53.618333;
                 -0.176667	53.616389;
                 -0.172500	53.614167;
                 -0.167778	53.612500;
                 -0.163611	53.610278;
                 -0.158611	53.608333;
                 -0.154444	53.606111;
                 -0.150278	53.603889;
                 -0.146111	53.601667;
                 -0.143056	53.599167;
                 -0.138889	53.596944;
                 -0.135000	53.594722;
                 -0.130833	53.592500;
                 -0.127778	53.589722;
                 -0.123611	53.587500;
                 -0.120278	53.585000;
                 -0.116944	53.582222;
                 -0.113056	53.580000;
                 -0.108889	53.577778;
                 -0.104722	53.575556;
                 -0.100556	53.573333;
                 -0.095833	53.571389;
                 -0.090833	53.569722;
                 -0.085000	53.568056;
                 -0.077778	53.568056;
                 -0.072222	53.568889;
                 -0.067778	53.570833;
                 -0.062778	53.572222;
                 -0.057222	53.573056;
                 -0.050000	53.572778;
                 -0.045833	53.570556;
                 -0.042778	53.568056;
                 -0.039444	53.565278;
                 -0.037222	53.562500;
                 -0.034167	53.559722;
                 -0.030833	53.557222;
                 -0.028611	53.554167;
                 -0.025278	53.551667;
                 -0.022222	53.548889;
                 -0.019722	53.545833;
                 -0.016667	53.543333;
                 -0.014167	53.540278;
                 -0.011111	53.537778;
                 -0.007778	53.535000;
                 -0.003611	53.532778;
                 0.000556 	53.530556;
                 0.004444 	53.528333;
                 0.008611 	53.526111;
                 0.012778 	53.523889;
                 0.016944 	53.521667;
                 0.020833 	53.519444;
                 0.023333 	53.516389;
                 0.024722 	53.513056;
                 0.026944 	53.510000;
                 0.032778 	53.508611;
                 0.039444 	53.507500;
                 0.046389 	53.506389;
                 0.052222 	53.505000;
                 0.058056 	53.503333;
                 0.063889 	53.501944;
                 0.067778 	53.499722;
                 0.070278 	53.496667;
                 0.073333 	53.493611;
                 0.076389 	53.491111;
                 0.080556 	53.488889;
                 0.085556 	53.486944;
                 0.091389 	53.485556;
                 0.097222 	53.483889;
                 0.103056 	53.482500;
                 0.108056 	53.480556;
                 0.112778 	53.478889;
                 0.117778 	53.476944;
                 0.122778 	53.475000;
                 0.128889 	53.474444;
                 0.134444 	53.475278;
                 0.140278 	53.476111;
                 0.146944 	53.475000;
                 0.150000 	53.472500;
                 0.151667 	53.469167;
                 0.155556 	53.466944;
                 0.159722 	53.464722;
                 0.164722 	53.462778;
                 0.168611 	53.460556;
                 0.171944 	53.457778;
                 0.174167 	53.455000;
                 0.173889 	53.450833;
                 0.172778 	53.446111;
                 0.173056 	53.442500;
                 0.175556 	53.439444;
                 0.177778 	53.436389;
                 0.179167 	53.433056;
                 0.182500 	53.430556;
                 0.186389 	53.428056;
                 0.190556 	53.425833;
                 0.195556 	53.424167;
                 0.200278 	53.422222;
                 0.206111 	53.420833;
                 0.211111 	53.418889;
                 0.216944 	53.417500;
                 0.218333 	53.413889;
                 0.215833 	53.410556;
                 0.215833 	53.407222;
                 0.218889 	53.404722;
                 0.224722 	53.403333;
                 0.230556 	53.401667;
                 0.235556 	53.399722;
                 0.238889 	53.397222;
                 0.241111 	53.394167;
                 0.242500 	53.390833;
                 0.244722 	53.387778;
                 0.246111 	53.384444;
                 0.247500 	53.381111;
                 0.250000 	53.378056;
                 0.251389 	53.374722;
                 0.252778 	53.371389;
                 0.255000 	53.368333;
                 0.256389 	53.365000;
                 0.258611 	53.361944;
                 0.261111 	53.358889;
                 0.263333 	53.355833;
                 0.265556 	53.353056;
                 0.268056 	53.350000;
                 0.270278 	53.346944;
                 0.273333 	53.344167;
                 0.274722 	53.340833;
                 0.277222 	53.338056;
                 0.278611 	53.334444;
                 0.280000 	53.331111;
                 0.282222 	53.328056;
                 0.283611 	53.324722;
                 0.285000 	53.321389;
                 0.286389 	53.318056;
                 0.288611 	53.315000;
                 0.290278 	53.311667;
                 0.291667 	53.308333;
                 0.293056 	53.304722;
                 0.295278 	53.301944;
                 0.296667 	53.298611;
                 0.298056 	53.295000;
                 0.300278 	53.292222;
                 0.302778 	53.289167;
                 0.305000 	53.286111;
                 0.307222 	53.283056;
                 0.309444 	53.280000;
                 0.311667 	53.277222;
                 0.313333 	53.273611;
                 0.315556 	53.270556;
                 0.317778 	53.267778;
                 0.320000 	53.264722;
                 0.322222 	53.261667;
                 0.324722 	53.258611;
                 0.326111 	53.255278;
                 0.327500 	53.251944;
                 0.329722 	53.248889;
                 0.331944 	53.245833;
                 0.334167 	53.242778;
                 0.336667 	53.240000;
                 0.338889 	53.236944;
                 0.340278 	53.233611;
                 0.340833 	53.229722;
                 0.341389 	53.226111;
                 0.341111 	53.221944;
                 0.340556 	53.217778;
                 0.340278 	53.213611;
                 0.340833 	53.209722;
                 0.342222 	53.206389;
                 0.343611 	53.203056;
                 0.346667 	53.200556;
                 0.349167 	53.197500;
                 0.351389 	53.194444;
                 0.352778 	53.191111;
                 0.354167 	53.187778;
                 0.354722 	53.183889;
                 0.356111 	53.180556;
                 0.356667 	53.176667;
                 0.357222 	53.173056;
                 0.356667 	53.168889;
                 0.357222 	53.165000;
                 0.357778 	53.161389;
                 0.357500 	53.157222;
                 0.357222 	53.153056;
                 0.356667 	53.148889;
                 0.355556 	53.144444;
                 0.354167 	53.140000;
                 0.352500 	53.135833;
                 0.350000 	53.132500;
                 0.346944 	53.129444;
                 0.345000 	53.125556;
                 0.344722 	53.121389;
                 0.345278 	53.117500;
                 0.343889 	53.113056;
                 0.342778 	53.108611;
                 0.342500 	53.104444;
                 0.342778 	53.100556;
                 0.341667 	53.096111;
                 0.339167 	53.092778;
                 0.336667 	53.089167;
                 0.333056 	53.086944;
                 0.329167 	53.084444;
                 0.324167 	53.083056;
                 0.318611 	53.082222;
                 0.311667 	53.082500;
                 0.306111 	53.081667;
                 0.300556 	53.080833;
                 0.296111 	53.078889;
                 0.292222 	53.076389;
                 0.288056 	53.074444;
                 0.284167 	53.071944;
                 0.280556 	53.069722;
                 0.276667 	53.067222;
                 0.273611 	53.064444;
                 0.270000 	53.061944;
                 0.266111 	53.059444;
                 0.262500 	53.056944;
                 0.258611 	53.054722;
                 0.255000 	53.052222;
                 0.250556 	53.050278;
                 0.246111 	53.048333;
                 0.242500 	53.046111;
                 0.238056 	53.044167;
                 0.233889 	53.042222;
                 0.229444 	53.040278;
                 0.225833 	53.038056;
                 0.221389 	53.036111;
                 0.216944 	53.034167;
                 0.212500 	53.032222;
                 0.208333 	53.030278;
                 0.203889 	53.028333;
                 0.200278 	53.026111;
                 0.196389 	53.023611;
                 0.192778 	53.021111;
                 0.188889 	53.018889;
                 0.185278 	53.016389;
                 0.181667 	53.013889;
                 0.177778 	53.011389;
                 0.173333 	53.009722;
                 0.169722 	53.007222;
                 0.166111 	53.004722;
                 0.162222 	53.002222;
                 0.158611 	53.000000;
                 0.154722 	52.997500;
                 0.151667 	52.994444;
                 0.148056 	52.992222;
                 0.145000 	52.989167;
                 0.141944 	52.986111;
                 0.138889 	52.983333;
                 0.135556 	52.980278;
                 0.132500 	52.977500;
                 0.128889 	52.975000;
                 0.125000 	52.972500;
                 0.121389 	52.970000;
                 0.116944 	52.968333;
                 0.113333 	52.965833;
                 0.109722 	52.963333;
                 0.105833 	52.960833;
                 0.102778 	52.958056;
                 0.099167 	52.955556;
                 0.095278 	52.953056;
                 0.091667 	52.950833;
                 0.088611 	52.947778;
                 0.085556 	52.944722;
                 0.082500 	52.941944;
                 0.079444 	52.938889;
                 0.076944 	52.935556;
                 0.075000 	52.931389;
                 0.072500 	52.928056;
                 0.070278 	52.924444;
                 0.066389 	52.922222;
                 0.061389 	52.920833;
                 0.056389 	52.919444;
                 0.050278 	52.918889;
                 0.043333 	52.919167;
                 0.037222 	52.918889;
                 0.032222 	52.917500;
                 0.029722 	52.914167;
                 0.029444 	52.910000;
                 0.028333 	52.905278;
                 0.025278 	52.902500;
                 0.021389 	52.900000;
                 0.017778 	52.897500;
                 0.014722 	52.894722;
                 0.011111 	52.892222;
                 0.007778 	52.889167;
                 0.004722 	52.886389;
                 0.001667 	52.883333;
                 -0.000278	52.879444;
                 0.006667 	52.878333;
                 0.011389 	52.878611;
                 0.016944 	52.879722;
                 0.022500 	52.880556;
                 0.028056 	52.881389;
                 0.033056 	52.882778;
                 0.038056 	52.884167;
                 0.043056 	52.885556;
                 0.048056 	52.886944;
                 0.053056 	52.888333;
                 0.057778 	52.889444;
                 0.063611 	52.890556;
                 0.069167 	52.891389;
                 0.075278 	52.891667;
                 0.082778 	52.890833;
                 0.086389 	52.890000;
                 0.091389 	52.888333;
                 0.096111 	52.886389;
                 0.101111 	52.884444;
                 0.106944 	52.883056;
                 0.111667 	52.881111;
                 0.117500 	52.879722;
                 0.122500 	52.877778;
                 0.127222 	52.876111;
                 0.132222 	52.874167;
                 0.136944 	52.872222;
                 0.141944 	52.870278;
                 0.147500 	52.868889;
                 0.152500 	52.866944;
                 0.157500 	52.865278;
                 0.161389 	52.863056;
                 0.165278 	52.860833;
                 0.168611 	52.858056;
                 0.171667 	52.855556;
                 0.173889 	52.852500;
                 0.176111 	52.849444;
                 0.179444 	52.846944;
                 0.181667 	52.843889;
                 0.184722 	52.841111;
                 0.186944 	52.838333;
                 0.191111 	52.836111;
                 0.194167 	52.833333;
                 0.196389 	52.830278;
                 0.199722 	52.827778;
                 0.201944 	52.824722;
                 0.204167 	52.821667;
                 0.207222 	52.818611;
                 0.209444 	52.815556;
                 0.212500 	52.813056;
                 0.214722 	52.810000;
                 0.218056 	52.807500;
                 0.221944 	52.805278;
                 0.226944 	52.803333;
                 0.230833 	52.801111;
                 0.233889 	52.798333;
                 0.238889 	52.796667;
                 0.243889 	52.795556;
                 0.250278 	52.795833;
                 0.255833 	52.796667;
                 0.260833 	52.798056;
                 0.266389 	52.798889;
                 0.271389 	52.800278;
                 0.276944 	52.801111;
                 0.282500 	52.801944;
                 0.288056 	52.802778;
                 0.294167 	52.803333;
                 0.300556 	52.803611;
                 0.307222 	52.803333;
                 0.314167 	52.803056;
                 0.320833 	52.801944;
                 0.326389 	52.800556;
                 0.331389 	52.798611;
                 0.335278 	52.796389;
                 0.339444 	52.794167;
                 0.342500 	52.791667;
                 0.346389 	52.789167;
                 0.351389 	52.787500;
                 0.356111 	52.785556;
                 0.360278 	52.783333;
                 0.363333 	52.780556;
                 0.365556 	52.777778;
                 0.366944 	52.774167;
                 0.369167 	52.771389;
                 0.371389 	52.768333;
                 0.374722 	52.765556;
                 0.379444 	52.763889;
                 0.381111 	52.766944;
                 0.380556 	52.770556;
                 0.380000 	52.774444;
                 0.379444 	52.778056;
                 0.378889 	52.781944;
                 0.379444 	52.786111;
                 0.380556 	52.790556;
                 0.383611 	52.793611;
                 0.386111 	52.796944;
                 0.390556 	52.798889;
                 0.393611 	52.801667;
                 0.396667 	52.804722;
                 0.399722 	52.807778;
                 0.402222 	52.811111;
                 0.405833 	52.813611;
                 0.410833 	52.815000;
                 0.415833 	52.816389;
                 0.420278 	52.818056;
                 0.423889 	52.820556;
                 0.427778 	52.823056;
                 0.430833 	52.825833;
                 0.433333 	52.829444;
                 0.435000 	52.833333;
                 0.436944 	52.837500;
                 0.438056 	52.841944;
                 0.438611 	52.846111;
                 0.439722 	52.850556;
                 0.441111 	52.855000;
                 0.442222 	52.859444;
                 0.443333 	52.864167;
                 0.445278 	52.868056;
                 0.447778 	52.871667;
                 0.450278 	52.875000;
                 0.453333 	52.878056;
                 0.455833 	52.881389;
                 0.458889 	52.884444;
                 0.461944 	52.887222;
                 0.463889 	52.891389;
                 0.466389 	52.894722;
                 0.468333 	52.898611;
                 0.470000 	52.902778;
                 0.472500 	52.906111;
                 0.475000 	52.909722;
                 0.477500 	52.913056;
                 0.479444 	52.917222;
                 0.481944 	52.920556;
                 0.484444 	52.923889;
                 0.486111 	52.928056;
                 0.488611 	52.931389;
                 0.491111 	52.935000;
                 0.494444 	52.937778;
                 0.498056 	52.940278;
                 0.501667 	52.942778;
                 0.506111 	52.944444;
                 0.511111 	52.945833;
                 0.516111 	52.947222;
                 0.521111 	52.948611;
                 0.525556 	52.950556;
                 0.529167 	52.953056;
                 0.532222 	52.955833;
                 0.534722 	52.959444;
                 0.537222 	52.962778;
                 0.541111 	52.965278;
                 0.546111 	52.966667;
                 0.551667 	52.967500;
                 0.558611 	52.967222;
                 0.566111 	52.966389;
                 0.570833 	52.964444;
                 0.575833 	52.962778;
                 0.578889 	52.960000;
                 0.583889 	52.958056;
                 0.588611 	52.956389;
                 0.595278 	52.955278;
                 0.600556 	52.955000;
                 0.606944 	52.955278;
                 0.613056 	52.955556;
                 0.618889 	52.956389;
                 0.624444 	52.957222;
                 0.630000 	52.958056;
                 0.635556 	52.958889;
                 0.640556 	52.960278;
                 0.645556 	52.961667;
                 0.651111 	52.962500;
                 0.656111 	52.963889;
                 0.662500 	52.964167;
                 0.668611 	52.964444;
                 0.675556 	52.964167;
                 0.682222 	52.963056;
                 0.688889 	52.961944;
                 0.695556 	52.960833;
                 0.702778 	52.960000;
                 0.709444 	52.958889;
                 0.716944 	52.958056;
                 0.724444 	52.957500;
                 0.731389 	52.957222;
                 0.738333 	52.956944;
                 0.744444 	52.957222;
                 0.748889 	52.959167;
                 0.753889 	52.960556;
                 0.758889 	52.961944;
                 0.764444 	52.962778;
                 0.770833 	52.963056;
                 0.777500 	52.962778;
                 0.783333 	52.963611;
                 0.790000 	52.963333;
                 0.796389 	52.963611;
                 0.803333 	52.963333;
                 0.810000 	52.963056;
                 0.817500 	52.962222;
                 0.825000 	52.961667;
                 0.831667 	52.960556;
                 0.836389 	52.958611;
                 0.841389 	52.956667;
                 0.848889 	52.955833;
                 0.853889 	52.957222;
                 0.858333 	52.959167;
                 0.861389 	52.961944;
                 0.865278 	52.964444;
                 0.868889 	52.966944;
                 0.872778 	52.969167;
                 0.880278 	52.968333;
                 0.885000 	52.966667;
                 0.890000 	52.964722;
                 0.893056 	52.961944;
                 0.895278 	52.958889;
                 0.898333 	52.956389;
                 0.901389 	52.953889;
                 0.906389 	52.951944;
                 0.911944 	52.952778;
                 0.918889 	52.952500;
                 0.921667 	52.952222;
                 0.928056 	52.952500;
                 0.934722 	52.952222;
                 0.942222 	52.951389;
                 0.949167 	52.951111;
                 0.956111 	52.950833;
                 0.963056 	52.950556;
                 0.969722 	52.950278;
                 0.976111 	52.950556;
                 0.982222 	52.950833;
                 0.988611 	52.951111;
                 0.994722 	52.951389;
                 1.000278 	52.952222;
                 1.006111 	52.953056;
                 1.011111 	52.954444;
                 1.016111 	52.955833;
                 1.014722 	52.959167;
                 1.008056 	52.960278;
                 1.000556 	52.961111;
                 0.993611 	52.961389;
                 0.986944 	52.961667;
                 0.980000 	52.961944;
                 0.973333 	52.963056;
                 0.967778 	52.964444;
                 0.967778 	52.967778;
                 0.972222 	52.969722;
                 0.977222 	52.971111;
                 0.982778 	52.971667;
                 0.988333 	52.972500;
                 0.995278 	52.972222;
                 1.002222 	52.972222;
                 1.009167 	52.971944;
                 1.015556 	52.970833;
                 1.021389 	52.969167;
                 1.027222 	52.967778;
                 1.031944 	52.965833;
                 1.037778 	52.964167;
                 1.042500 	52.962222;
                 1.047500 	52.960556;
                 1.052222 	52.958611;
                 1.058056 	52.956944;
                 1.063611 	52.955556;
                 1.069444 	52.953889;
                 1.075278 	52.952500;
                 1.080833 	52.950833;
                 1.086667 	52.949444;
                 1.092500 	52.947778;
                 1.098056 	52.946389;
                 1.104722 	52.945278;
                 1.112222 	52.944444;
                 1.119722 	52.943611;
                 1.127222 	52.942778;
                 1.134722 	52.941944;
                 1.141389 	52.941667;
                 1.148333 	52.941389;
                 1.155278 	52.941389;
                 1.162778 	52.940556;
                 1.169444 	52.940278;
                 1.176389 	52.940000;
                 1.183889 	52.939167;
                 1.191389 	52.938333;
                 1.198889 	52.937500;
                 1.206389 	52.936667;
                 1.212778 	52.935556;
                 1.220278 	52.934722;
                 1.226944 	52.933611;
                 1.233611 	52.932500;
                 1.239167 	52.931111;
                 1.246667 	52.930278;
                 1.254167 	52.929444;
                 1.261111 	52.929167;
                 1.268056 	52.928889;
                 1.274722 	52.928611;
                 1.282222 	52.927778;
                 1.288889 	52.926667;
                 1.294722 	52.925000;
                 1.300278 	52.923611;
                 1.306111 	52.921944;
                 1.310833 	52.920000;
                 1.316667 	52.918611;
                 1.323056 	52.917500;
                 1.328889 	52.915833;
                 1.335556 	52.914722;
                 1.341111 	52.913056;
                 1.346944 	52.911667;
                 1.352500 	52.910000;
                 1.357500 	52.908056;
                 1.362222 	52.906389;
                 1.368056 	52.904722;
                 1.372778 	52.902778;
                 1.377222 	52.902222;
                 1.379722 	52.900278;
                 1.383611 	52.898056;
                 1.388611 	52.896111;
                 1.393333 	52.894167;
                 1.397222 	52.891944;
                 1.402222 	52.890000;
                 1.406111 	52.887500;
                 1.410833 	52.885833;
                 1.414722 	52.883333;
                 1.419722 	52.881389;
                 1.423611 	52.879167;
                 1.427500 	52.876944;
                 1.432500 	52.875000;
                 1.436389 	52.872778;
                 1.440278 	52.870556;
                 1.444167 	52.868056;
                 1.448056 	52.865833;
                 1.452222 	52.863611;
                 1.456944 	52.861667;
                 1.460833 	52.859444;
                 1.464722 	52.857222;
                 1.468611 	52.854722;
                 1.473611 	52.853056;
                 1.477500 	52.850556;
                 1.482222 	52.848611;
                 1.486389 	52.846389;
                 1.491111 	52.844444;
                 1.495000 	52.842222;
                 1.499722 	52.840278;
                 1.503889 	52.838056;
                 1.508611 	52.836111;
                 1.513333 	52.834167;
                 1.517222 	52.831944;
                 1.522222 	52.830000;
                 1.526944 	52.828056;
                 1.530833 	52.825833;
                 1.535833 	52.823889;
                 1.540556 	52.821944;
                 1.545278 	52.820000;
                 1.549167 	52.817778;
                 1.554167 	52.815833;
                 1.558056 	52.813611;
                 1.561944 	52.811111;
                 1.565833 	52.808889;
                 1.569722 	52.806667;
                 1.574722 	52.804722;
                 1.578611 	52.802500;
                 1.582500 	52.800278;
                 1.587222 	52.798333;
                 1.591111 	52.795833;
                 1.595000 	52.793611;
                 1.600000 	52.791667;
                 1.603889 	52.789444;
                 1.607778 	52.787222;
                 1.612500 	52.785278;
                 1.617500 	52.783333;
                 1.622222 	52.781389;
                 1.626944 	52.779444;
                 1.631944 	52.777500;
                 1.635833 	52.775278;
                 1.640556 	52.773333;
                 1.645278 	52.771389;
                 1.649167 	52.769167;
                 1.653333 	52.766944;
                 1.657222 	52.764444;
                 1.660278 	52.761944;
                 1.663333 	52.759167;
                 1.666111 	52.756667;
                 1.669167 	52.753889;
                 1.672222 	52.751389;
                 1.675278 	52.748611;
                 1.677500 	52.745556;
                 1.680556 	52.742778;
                 1.683333 	52.739722;
                 1.686389 	52.737222;
                 1.688611 	52.734167;
                 1.690833 	52.731111;
                 1.693889 	52.728333;
                 1.696111 	52.725278;
                 1.698056 	52.722222;
                 1.700278 	52.719444;
                 1.702500 	52.716389;
                 1.703889 	52.712778;
                 1.705000 	52.709444;
                 1.707222 	52.706389;
                 1.708611 	52.703056;
                 1.709722 	52.699722;
                 1.711111 	52.696389;
                 1.712222 	52.692778;
                 1.713611 	52.689444;
                 1.715833 	52.686389;
                 1.716944 	52.683056;
                 1.719167 	52.680000;
                 1.721389 	52.676944;
                 1.722500 	52.673611;
                 1.723889 	52.670278;
                 1.725278 	52.666944;
                 1.726389 	52.663333;
                 1.727778 	52.660000;
                 1.730000 	52.656944;
                 1.731111 	52.653611;
                 1.732500 	52.650278;
                 1.734444 	52.647222;
                 1.735833 	52.643889;
                 1.738056 	52.640833;
                 1.739167 	52.637222;
                 1.740556 	52.633889;
                 1.742778 	52.630833;
                 1.743889 	52.627500;
                 1.745278 	52.624167;
                 1.746389 	52.620833;
                 1.746944 	52.616944;
                 1.746389 	52.612778;
                 1.745000 	52.608333;
                 1.744722 	52.604167;
                 1.744167 	52.600000;
                 1.744444 	52.596389;
                 1.745000 	52.592500;
                 1.746111 	52.589167;
                 1.746667 	52.585278;
                 1.747778 	52.581944;
                 1.748889 	52.578056;
                 1.747500 	52.573611;
                 1.746389 	52.569167;
                 1.743611 	52.565833;
                 1.742222 	52.561111;
                 1.741389 	52.557500;
                 1.741667 	52.553889;
                 1.741944 	52.550000;
                 1.743333 	52.546667;
                 1.744722 	52.543333;
                 1.745833 	52.540000;
                 1.747222 	52.536389;
                 1.748333 	52.533056;
                 1.749722 	52.529722;
                 1.750000 	52.525833;
                 1.751389 	52.522500;
                 1.752500 	52.519167;
                 1.753889 	52.515833;
                 1.755278 	52.512222;
                 1.756389 	52.508889;
                 1.758611 	52.505833;
                 1.759722 	52.502500;
                 1.761944 	52.499444;
                 1.764167 	52.496389;
                 1.766389 	52.493333;
                 1.768333 	52.490000;
                 1.769444 	52.486667;
                 1.770833 	52.483056;
                 1.771111 	52.479444;
                 1.770278 	52.476667;
                 1.769167 	52.473056;
                 1.767222 	52.469167;
                 1.764722 	52.465833;
                 1.761111 	52.463333;
                 1.757222 	52.460833;
                 1.753611 	52.458611;
                 1.749722 	52.456111;
                 1.746667 	52.453333;
                 1.743333 	52.450278;
                 1.740833 	52.446944;
                 1.739444 	52.442500;
                 1.737500 	52.438611;
                 1.736111 	52.433889;
                 1.734722 	52.429444;
                 1.733611 	52.425000;
                 1.732222 	52.420556;
                 1.730833 	52.416111;
                 1.730278 	52.411944;
                 1.728889 	52.407500;
                 1.729444 	52.403611;
                 1.730556 	52.400278;
                 1.731944 	52.396944;
                 1.733056 	52.393333;
                 1.730556 	52.390000;
                 1.728056 	52.386667;
                 1.726667 	52.381944;
                 1.724167 	52.378611;
                 1.720278 	52.376389;
                 1.717222 	52.373333;
                 1.714167 	52.370556;
                 1.710833 	52.367500;
                 1.708333 	52.364167;
                 1.705278 	52.361111;
                 1.703333 	52.357222;
                 1.700833 	52.353889;
                 1.698889 	52.349722;
                 1.696944 	52.345833;
                 1.694167 	52.342500;
                 1.692222 	52.338611;
                 1.690278 	52.334444;
                 1.687778 	52.331111;
                 1.685278 	52.327500;
                 1.682778 	52.324167;
                 1.680278 	52.320833;
                 1.678333 	52.316667;
                 1.676389 	52.312778;
                 1.673889 	52.309444;
                 1.671111 	52.305833;
                 1.667500 	52.303611;
                 1.663611 	52.301111;
                 1.660556 	52.298333;
                 1.656944 	52.295833;
                 1.653611 	52.293056;
                 1.650556 	52.290000;
                 1.647500 	52.287222;
                 1.644167 	52.284444;
                 1.642222 	52.280278;
                 1.639722 	52.276944;
                 1.637778 	52.273056;
                 1.635833 	52.268889;
                 1.634444 	52.264444;
                 1.633333 	52.260000;
                 1.631944 	52.255556;
                 1.630556 	52.251111;
                 1.630000 	52.246944;
                 1.628889 	52.242222;
                 1.627500 	52.237778;
                 1.626111 	52.233333;
                 1.625556 	52.229167;
                 1.625278 	52.225000;
                 1.625556 	52.221389;
                 1.626944 	52.218056;
                 1.627222 	52.214167;
                 1.628611 	52.210833;
                 1.628889 	52.206944;
                 1.629444 	52.203333;
                 1.629722 	52.199444;
                 1.630000 	52.195833;
                 1.628889 	52.191111;
                 1.628333 	52.187222;
                 1.627778 	52.183056;
                 1.626667 	52.178333;
                 1.625278 	52.173889;
                 1.622778 	52.170556;
                 1.620833 	52.166667;
                 1.617500 	52.163611;
                 1.615000 	52.160278;
                 1.613056 	52.156389;
                 1.611111 	52.152222;
                 1.610833 	52.148056;
                 1.610278 	52.143889;
                 1.608889 	52.139444;
                 1.606944 	52.135556;
                 1.604444 	52.132222;
                 1.602500 	52.128056;
                 1.601111 	52.123611;
                 1.600000 	52.119167;
                 1.599444 	52.115000;
                 1.598056 	52.110556;
                 1.596111 	52.106667;
                 1.593611 	52.103056;
                 1.591667 	52.099167;
                 1.590556 	52.094722;
                 1.590000 	52.090556;
                 1.588611 	52.086111;
                 1.586111 	52.082500;
                 1.583056 	52.079722;
                 1.580000 	52.076667;
                 1.575556 	52.075000;
                 1.571111 	52.073056;
                 1.566389 	52.071667;
                 1.561389 	52.070556;
                 1.555833 	52.069722;
                 1.550278 	52.068889;
                 1.544722 	52.068056;
                 1.538611 	52.067778;
                 1.533056 	52.066944;
                 1.528333 	52.065833;
                 1.523333 	52.064444;
                 1.519167 	52.062500;
                 1.514722 	52.060833;
                 1.510278 	52.058889;
                 1.506111 	52.056944;
                 1.502500 	52.054722;
                 1.498056 	52.052778;
                 1.493611 	52.051111;
                 1.489444 	52.049167;
                 1.483889 	52.048333;
                 1.479167 	52.050278;
                 1.472778 	52.051389;
                 1.468333 	52.049722;
                 1.465833 	52.046111;
                 1.464444 	52.041667;
                 1.465000 	52.037778;
                 1.463611 	52.033333;
                 1.460556 	52.030556;
                 1.458056 	52.026944;
                 1.456111 	52.025278;
                 1.453611 	52.021944;
                 1.450556 	52.019167;
                 1.446667 	52.016667;
                 1.443056 	52.014444;
                 1.439444 	52.011944;
                 1.435556 	52.009444;
                 1.431944 	52.007222;
                 1.428056 	52.004722;
                 1.425000 	52.001944;
                 1.421944 	51.998889;
                 1.418889 	51.996111;
                 1.416389 	51.992500;
                 1.413889 	51.989167;
                 1.410833 	51.986389;
                 1.406944 	51.983889;
                 1.403333 	51.981389;
                 1.399722 	51.979167;
                 1.396667 	51.976111;
                 1.393333 	51.973333;
                 1.390278 	51.970278;
                 1.386667 	51.968056;
                 1.382778 	51.965556;
                 1.379167 	51.963333;
                 1.376111 	51.960278;
                 1.372500 	51.958056;
                 1.368056 	51.956111;
                 1.364444 	51.953611;
                 1.360833 	51.951389;
                 1.356389 	51.949444;
                 1.352778 	51.947222;
                 1.349722 	51.944167;
                 1.345833 	51.941944;
                 1.342778 	51.938889;
                 1.339722 	51.936111;
                 1.337222 	51.932500;
                 1.334167 	51.929722;
                 1.328611 	51.928889;
                 1.327222 	51.932222;
                 1.325278 	51.935278;
                 1.323056 	51.938333;
                 1.320000 	51.940833;
                 1.317778 	51.943889;
                 1.317500 	51.947778;
                 1.315278 	51.950833;
                 1.312222 	51.953333;
                 1.307500 	51.955278;
                 1.302778 	51.957222;
                 1.298889 	51.959444;
                 1.295000 	51.961667;
                 1.292222 	51.964444;
                 1.290000 	51.967500;
                 1.287778 	51.970556;
                 1.286667 	51.973889;
                 1.283611 	51.976389;
                 1.281389 	51.979444;
                 1.278333 	51.982222;
                 1.275278 	51.984722;
                 1.271389 	51.986944;
                 1.267500 	51.989444;
                 1.262778 	51.991389;
                 1.258056 	51.993056;
                 1.253333 	51.995000;
                 1.246944 	51.996111;
                 1.240278 	51.996389;
                 1.238611 	51.993333;
                 1.240833 	51.990278;
                 1.244722 	51.988056;
                 1.249444 	51.986111;
                 1.255000 	51.984722;
                 1.260556 	51.983056;
                 1.265278 	51.981111;
                 1.269167 	51.978889;
                 1.271389 	51.975833;
                 1.273333 	51.973056;
                 1.273889 	51.969167;
                 1.273333 	51.965000;
                 1.275556 	51.961944;
                 1.276111 	51.958333;
                 1.275556 	51.954167;
                 1.272500 	51.951111;
                 1.267500 	51.950000;
                 1.263611 	51.951389;
                 1.259722 	51.953611;
                 1.255556 	51.955000;
                 1.250833 	51.956944;
                 1.244167 	51.957222;
                 1.238611 	51.956389;
                 1.233611 	51.955000;
                 1.228056 	51.954167;
                 1.222778 	51.953333;
                 1.215833 	51.953611;
                 1.208611 	51.954444;
                 1.203056 	51.955833;
                 1.197500 	51.957500;
                 1.192778 	51.959444;
                 1.187222 	51.960833;
                 1.181667 	51.962500;
                 1.175000 	51.963611;
                 1.168333 	51.963889;
                 1.164167 	51.961944;
                 1.159722 	51.960278;
                 1.156111 	51.957778;
                 1.151944 	51.955833;
                 1.146944 	51.954444;
                 1.143056 	51.954444;
                 1.137500 	51.952778;
                 1.131389 	51.952500;
                 1.124722 	51.952778;
                 1.118611 	51.952500;
                 1.113056 	51.951667;
                 1.116111 	51.949167;
                 1.120000 	51.946944;
                 1.124722 	51.945000;
                 1.131111 	51.943889;
                 1.137222 	51.943889;
                 1.143889 	51.943889;
                 1.150000 	51.944167;
                 1.156111 	51.944167;
                 1.161667 	51.945000;
                 1.167778 	51.945278;
                 1.173889 	51.945556;
                 1.180000 	51.945833;
                 1.186667 	51.945556;
                 1.193889 	51.945000;
                 1.200278 	51.943611;
                 1.207778 	51.943056;
                 1.213611 	51.942222;
                 1.219722 	51.942500;
                 1.225000 	51.943333;
                 1.230000 	51.944722;
                 1.235000 	51.946111;
                 1.239722 	51.947222;
                 1.245833 	51.947500;
                 1.253056 	51.946944;
                 1.257778 	51.945000;
                 1.261667 	51.942500;
                 1.265556 	51.940278;
                 1.271111 	51.938889;
                 1.276111 	51.940278;
                 1.280556 	51.941944;
                 1.285278 	51.943333;
                 1.290556 	51.943333;
                 1.292778 	51.940278;
                 1.289722 	51.937222;
                 1.286667 	51.934444;
                 1.283333 	51.931389;
                 1.280278 	51.928611;
                 1.276667 	51.926111;
                 1.273611 	51.923333;
                 1.270556 	51.920278;
                 1.267500 	51.917500;
                 1.265556 	51.913333;
                 1.263611 	51.909444;
                 1.260556 	51.906667;
                 1.255000 	51.905833;
                 1.248611 	51.906944;
                 1.243056 	51.908333;
                 1.237500 	51.907778;
                 1.233333 	51.905833;
                 1.230000 	51.902778;
                 1.231389 	51.899444;
                 1.233611 	51.896389;
                 1.231111 	51.893056;
                 1.226111 	51.891667;
                 1.219444 	51.891944;
                 1.213889 	51.891111;
                 1.211389 	51.887778;
                 1.210556 	51.884167;
                 1.208611 	51.880278;
                 1.204444 	51.878333;
                 1.201111 	51.875278;
                 1.201111 	51.872222;
                 1.205833 	51.870278;
                 1.213056 	51.869444;
                 1.219167 	51.869722;
                 1.224722 	51.868333;
                 1.228611 	51.865833;
                 1.232500 	51.863611;
                 1.237222 	51.861667;
                 1.243611 	51.860556;
                 1.250278 	51.860278;
                 1.256389 	51.860556;
                 1.261389 	51.861944;
                 1.265556 	51.863889;
                 1.268611 	51.866667;
                 1.270000 	51.871111;
                 1.269444 	51.875000;
                 1.270000 	51.879167;
                 1.273056 	51.881944;
                 1.279167 	51.882222;
                 1.285556 	51.881111;
                 1.290278 	51.879167;
                 1.294167 	51.876944;
                 1.297222 	51.874167;
                 1.297500 	51.870556;
                 1.295833 	51.867500;
                 1.294722 	51.862778;
                 1.292222 	51.859444;
                 1.289167 	51.856667;
                 1.286111 	51.853611;
                 1.283056 	51.850833;
                 1.279167 	51.848333;
                 1.275556 	51.845833;
                 1.272500 	51.843056;
                 1.268889 	51.840556;
                 1.265000 	51.838333;
                 1.261389 	51.835833;
                 1.257778 	51.833611;
                 1.253333 	51.831667;
                 1.249722 	51.829167;
                 1.246111 	51.826944;
                 1.243056 	51.823889;
                 1.240000 	51.821111;
                 1.236111 	51.818611;
                 1.232500 	51.816389;
                 1.228889 	51.813889;
                 1.225278 	51.811389;
                 1.222222 	51.808611;
                 1.218333 	51.806111;
                 1.214722 	51.803889;
                 1.210556 	51.801944;
                 1.206667 	51.799722;
                 1.202500 	51.797778;
                 1.198333 	51.795833;
                 1.193889 	51.793889;
                 1.189722 	51.792222;
                 1.184722 	51.790833;
                 1.180000 	51.789444;
                 1.175000 	51.788056;
                 1.170833 	51.786389;
                 1.165833 	51.785000;
                 1.161111 	51.783611;
                 1.156111 	51.782222;
                 1.151389 	51.781111;
                 1.146389 	51.779722;
                 1.140833 	51.778889;
                 1.136944 	51.777778;
                 1.131944 	51.776667;
                 1.127222 	51.775278;
                 1.121667 	51.774444;
                 1.116111 	51.773611;
                 1.110000 	51.773333;
                 1.104722 	51.772500;
                 1.098611 	51.772222;
                 1.092500 	51.771944;
                 1.086944 	51.771111;
                 1.080833 	51.770833;
                 1.074722 	51.770556;
                 1.068889 	51.770278;
                 1.062778 	51.770000;
                 1.056667 	51.769722;
                 1.050000 	51.770000;
                 1.042778 	51.770833;
                 1.038056 	51.772778;
                 1.035000 	51.775278;
                 1.033611 	51.778889;
                 1.032222 	51.782222;
                 1.030278 	51.785278;
                 1.028889 	51.788611;
                 1.027500 	51.791944;
                 1.024444 	51.794722;
                 1.021667 	51.797222;
                 1.018611 	51.800000;
                 1.014722 	51.802222;
                 1.010000 	51.804167;
                 1.004444 	51.805556;
                 0.998889 	51.807222;
                 0.992778 	51.806944;
                 0.987222 	51.806111;
                 0.981667 	51.805278;
                 0.976389 	51.804444;
                 0.969444 	51.804722;
                 0.962778 	51.805000;
                 0.956111 	51.805000;
                 0.950278 	51.804722;
                 0.944722 	51.804167;
                 0.939167 	51.803333;
                 0.934444 	51.801944;
                 0.930000 	51.800000;
                 0.926389 	51.797500;
                 0.922222 	51.795833;
                 0.917778 	51.793889;
                 0.913611 	51.791944;
                 0.909444 	51.790278;
                 0.905000 	51.788333;
                 0.900278 	51.786944;
                 0.895278 	51.785556;
                 0.891111 	51.783611;
                 0.886667 	51.781944;
                 0.881944 	51.780556;
                 0.879444 	51.776944;
                 0.881111 	51.774444;
                 0.882500 	51.772222;
                 0.883056 	51.768333;
                 0.880556 	51.765000;
                 0.877500 	51.761944;
                 0.873333 	51.760000;
                 0.868333 	51.758611;
                 0.863611 	51.757500;
                 0.860000 	51.755000;
                 0.856944 	51.752222;
                 0.854444 	51.748611;
                 0.850278 	51.746667;
                 0.845833 	51.745000;
                 0.840556 	51.744167;
                 0.835000 	51.743333;
                 0.828889 	51.743056;
                 0.822222 	51.743333;
                 0.815556 	51.743333;
                 0.808333 	51.744167;
                 0.801944 	51.745278;
                 0.796944 	51.743889;
                 0.792222 	51.742778;
                 0.787222 	51.741389;
                 0.782500 	51.740000;
                 0.777500 	51.738611;
                 0.772778 	51.737222;
                 0.767222 	51.736389;
                 0.762222 	51.735000;
                 0.757500 	51.733889;
                 0.752500 	51.732500;
                 0.745833 	51.732778;
                 0.739444 	51.733889;
                 0.733889 	51.735278;
                 0.727222 	51.735556;
                 0.722500 	51.734167;
                 0.718056 	51.732222;
                 0.715833 	51.728889;
                 0.716944 	51.725556;
                 0.722778 	51.723889;
                 0.728333 	51.722500;
                 0.734722 	51.721389;
                 0.740278 	51.719722;
                 0.746667 	51.718611;
                 0.752222 	51.717222;
                 0.756944 	51.715278;
                 0.761667 	51.713333;
                 0.765556 	51.711111;
                 0.767778 	51.708056;
                 0.769167 	51.704722;
                 0.770278 	51.701389;
                 0.775833 	51.699722;
                 0.778889 	51.702778;
                 0.781111 	51.705278;
                 0.783611 	51.708889;
                 0.786111 	51.712222;
                 0.789167 	51.715278;
                 0.792222 	51.718056;
                 0.795833 	51.720556;
                 0.800000 	51.722222;
                 0.806111 	51.722778;
                 0.812500 	51.721389;
                 0.818056 	51.720000;
                 0.824444 	51.718333;
                 0.831667 	51.717500;
                 0.836667 	51.718889;
                 0.841389 	51.720278;
                 0.845556 	51.722222;
                 0.850000 	51.724167;
                 0.854722 	51.725556;
                 0.858333 	51.727778;
                 0.862778 	51.729722;
                 0.867500 	51.731111;
                 0.871667 	51.732778;
                 0.876667 	51.734167;
                 0.882222 	51.735000;
                 0.886944 	51.736389;
                 0.891944 	51.737778;
                 0.896667 	51.739167;
                 0.901667 	51.740556;
                 0.906944 	51.741111;
                 0.911944 	51.742500;
                 0.917500 	51.743333;
                 0.923333 	51.743611;
                 0.929167 	51.743056;
                 0.935833 	51.741944;
                 0.940556 	51.740000;
                 0.945000 	51.738056;
                 0.948889 	51.735833;
                 0.951944 	51.733333;
                 0.955000 	51.730556;
                 0.957222 	51.727500;
                 0.958611 	51.724167;
                 0.958889 	51.720278;
                 0.959444 	51.716667;
                 0.960000 	51.712778;
                 0.959444 	51.708889;
                 0.957500 	51.704722;
                 0.958056 	51.701111;
                 0.959444 	51.697500;
                 0.960556 	51.694167;
                 0.960278 	51.690000;
                 0.957778 	51.686667;
                 0.956111 	51.682778;
                 0.954167 	51.678611;
                 0.952222 	51.674722;
                 0.951111 	51.670278;
                 0.949167 	51.666389;
                 0.948056 	51.661667;
                 0.945556 	51.658333;
                 0.943056 	51.654722;
                 0.940556 	51.651389;
                 0.938056 	51.648056;
                 0.935833 	51.644444;
                 0.932778 	51.641667;
                 0.929722 	51.638611;
                 0.925833 	51.636389;
                 0.921667 	51.634444;
                 0.917500 	51.632500;
                 0.913333 	51.630556;
                 0.907778 	51.629722;
                 0.902778 	51.628611;
                 0.898056 	51.627222;
                 0.892500 	51.626389;
                 0.886667 	51.626111;
                 0.880556 	51.625833;
                 0.873889 	51.626111;
                 0.867222 	51.626389;
                 0.861111 	51.626111;
                 0.855000 	51.625556;
                 0.848333 	51.625833;
                 0.841667 	51.626111;
                 0.835000 	51.626389;
                 0.827778 	51.627222;
                 0.821389 	51.628333;
                 0.816667 	51.630278;
                 0.812222 	51.629722;
                 0.814444 	51.627778;
                 0.818333 	51.625278;
                 0.823056 	51.623611;
                 0.829444 	51.622222;
                 0.835833 	51.621111;
                 0.843056 	51.620556;
                 0.849722 	51.619444;
                 0.856111 	51.618056;
                 0.860833 	51.616389;
                 0.863611 	51.613611;
                 0.864167 	51.610000;
                 0.864722 	51.606111;
                 0.865000 	51.602500;
                 0.867222 	51.599444;
                 0.872778 	51.597778;
                 0.877778 	51.599167;
                 0.880556 	51.601111;
                 0.882778 	51.604722;
                 0.885278 	51.608056;
                 0.888333 	51.611111;
                 0.891944 	51.613333;
                 0.896944 	51.614722;
                 0.901667 	51.616111;
                 0.907778 	51.616389;
                 0.913333 	51.617222;
                 0.919167 	51.617500;
                 0.925278 	51.617778;
                 0.931389 	51.618056;
                 0.937500 	51.618333;
                 0.943611 	51.618611;
                 0.950833 	51.617778;
                 0.955556 	51.615833;
                 0.956667 	51.612500;
                 0.953611 	51.609722;
                 0.950556 	51.606667;
                 0.947222 	51.605278;
                 0.943611 	51.602778;
                 0.939444 	51.601111;
                 0.935833 	51.598611;
                 0.932222 	51.596111;
                 0.928333 	51.593889;
                 0.924722 	51.591389;
                 0.920556 	51.589444;
                 0.916944 	51.587222;
                 0.913333 	51.584722;
                 0.910278 	51.581944;
                 0.905833 	51.579444;
                 0.902222 	51.577222;
                 0.898056 	51.575278;
                 0.894444 	51.572778;
                 0.890556 	51.570556;
                 0.886389 	51.568611;
                 0.882778 	51.566111;
                 0.879167 	51.563889;
                 0.875556 	51.561389;
                 0.871389 	51.559444;
                 0.866944 	51.557778;
                 0.861667 	51.556944;
                 0.856667 	51.555556;
                 0.852500 	51.553611;
                 0.848333 	51.551667;
                 0.845278 	51.548889;
                 0.841667 	51.546389;
                 0.837222 	51.544444;
                 0.833056 	51.542222;
                 0.828611 	51.540278;
                 0.824444 	51.538611;
                 0.820833 	51.536111;
                 0.816667 	51.534167;
                 0.812500 	51.532500;
                 0.808056 	51.530556;
                 0.803889 	51.528611;
                 0.799167 	51.527222;
                 0.794167 	51.525833;
                 0.789444 	51.524444;
                 0.784722 	51.523333;
                 0.779167 	51.522500;
                 0.773056 	51.522222;
                 0.766944 	51.521944;
                 0.760556 	51.523056;
                 0.755000 	51.524444;
                 0.747778 	51.525278;
                 0.742222 	51.526667;
                 0.735833 	51.527778;
                 0.728611 	51.528611;
                 0.721944 	51.528889;
                 0.715278 	51.529167;
                 0.708889 	51.529444;
                 0.701389 	51.530000;
                 0.694167 	51.530833;
                 0.687778 	51.531944;
                 0.680556 	51.532778;
                 0.674167 	51.533889;
                 0.667778 	51.535000;
                 0.661389 	51.536111;
                 0.655000 	51.537222;
                 0.648889 	51.536944;
                 0.644722 	51.535000;
                 0.643056 	51.531111;
                 0.642778 	51.527778;
                 0.642500 	51.523611;
                 0.638889 	51.521389;
                 0.634722 	51.519444;
                 0.629167 	51.518611;
                 0.623056 	51.518333;
                 0.616389 	51.518611;
                 0.609167 	51.519167;
                 0.602500 	51.519444;
                 0.596667 	51.519167;
                 0.591667 	51.517778;
                 0.587500 	51.516111;
                 0.583889 	51.513611;
                 0.580278 	51.511111;
                 0.576111 	51.509444;
                 0.571111 	51.508056;
                 0.565556 	51.509444;
                 0.561667 	51.511667;
                 0.558889 	51.514444;
                 0.553333 	51.515833;
                 0.547778 	51.515000;
                 0.542222 	51.514167;
                 0.537500 	51.512778;
                 0.532778 	51.511389;
                 0.527222 	51.510556;
                 0.521944 	51.509722;
                 0.515833 	51.509444;
                 0.510278 	51.508611;
                 0.505000 	51.507778;
                 0.498889 	51.507500;
                 0.492778 	51.507222;
                 0.487500 	51.506389;
                 0.481944 	51.505556;
                 0.476667 	51.504722;
                 0.470556 	51.504444;
                 0.465000 	51.503611;
                 0.460278 	51.502222;
                 0.456111 	51.500278;
                 0.451944 	51.498333;
                 0.448889 	51.495556;
                 0.446389 	51.491944;
                 0.446111 	51.487778;
                 0.445833 	51.483889;
                 0.445278 	51.479722;
                 0.444167 	51.475278;
                 0.441111 	51.472222;
                 0.438889 	51.468889;
                 0.435278 	51.466389;
                 0.431667 	51.463889;
                 0.427500 	51.461944;
                 0.422500 	51.460556;
                 0.417778 	51.459444;
                 0.412222 	51.458611;
                 0.407500 	51.457222;
                 0.401944 	51.456389;
                 0.397222 	51.455000;
                 0.391944 	51.454167;
                 0.386389 	51.453333;
                 0.382778 	51.450833;
                 0.386667 	51.448611;
                 0.393056 	51.447500;
                 0.399167 	51.447778;
                 0.405833 	51.447500;
                 0.412500 	51.447222;
                 0.417778 	51.448333;
                 0.423333 	51.449167;
                 0.428611 	51.450000;
                 0.433333 	51.450278;
                 0.438611 	51.451111;
                 0.444167 	51.451944;
                 0.448333 	51.453889;
                 0.452500 	51.455833;
                 0.456111 	51.458056;
                 0.459167 	51.461111;
                 0.461389 	51.464444;
                 0.463889 	51.468056;
                 0.466389 	51.471389;
                 0.468056 	51.475556;
                 0.471111 	51.478333;
                 0.474722 	51.480833;
                 0.478889 	51.482778;
                 0.483889 	51.484167;
                 0.489167 	51.485000;
                 0.495278 	51.485278;
                 0.501944 	51.485000;
                 0.508611 	51.484722;
                 0.513889 	51.485556;
                 0.519444 	51.486389;
                 0.525278 	51.486667;
                 0.530833 	51.487500;
                 0.536111 	51.488333;
                 0.542222 	51.488611;
                 0.548889 	51.488611;
                 0.556111 	51.487778;
                 0.561667 	51.486111;
                 0.567222 	51.484722;
                 0.573333 	51.485000;
                 0.578056 	51.486389;
                 0.583056 	51.487778;
                 0.588333 	51.488611;
                 0.595556 	51.487778;
                 0.601944 	51.486667;
                 0.606667 	51.484722;
                 0.610556 	51.482500;
                 0.615278 	51.480556;
                 0.620833 	51.479167;
                 0.627222 	51.478056;
                 0.633889 	51.477778;
                 0.640556 	51.477500;
                 0.646389 	51.477778;
                 0.653056 	51.477778;
                 0.660278 	51.476944;
                 0.665833 	51.475278;
                 0.671389 	51.473889;
                 0.677778 	51.472778;
                 0.683889 	51.473056;
                 0.691111 	51.472222;
                 0.697500 	51.471111;
                 0.702222 	51.469167;
                 0.707778 	51.467778;
                 0.711667 	51.465556;
                 0.716389 	51.463611;
                 0.720278 	51.461389;
                 0.723333 	51.458889;
                 0.725278 	51.455833;
                 0.726667 	51.452500;
                 0.727222 	51.448611;
                 0.725278 	51.444722;
                 0.723056 	51.441111;
                 0.719167 	51.438889;
                 0.714444 	51.437500;
                 0.709167 	51.436667;
                 0.703056 	51.436389;
                 0.696944 	51.436111;
                 0.690833 	51.435833;
                 0.684444 	51.435833;
                 0.678056 	51.437222;
                 0.671667 	51.438333;
                 0.666111 	51.439722;
                 0.659444 	51.440000;
                 0.652222 	51.440833;
                 0.645000 	51.441389;
                 0.640000 	51.440000;
                 0.637778 	51.436667;
                 0.635278 	51.433333;
                 0.629167 	51.432778;
                 0.625000 	51.431111;
                 0.621944 	51.428056;
                 0.618889 	51.425278;
                 0.615278 	51.422778;
                 0.611667 	51.420278;
                 0.606944 	51.418889;
                 0.602778 	51.417222;
                 0.598056 	51.415833;
                 0.592500 	51.415000;
                 0.586944 	51.414167;
                 0.581111 	51.413889;
                 0.575000 	51.413611;
                 0.568889 	51.413333;
                 0.563056 	51.412778;
                 0.558056 	51.411667;
                 0.556667 	51.408333;
                 0.558889 	51.405556;
                 0.561667 	51.402778;
                 0.564722 	51.400278;
                 0.568611 	51.398056;
                 0.572500 	51.395556;
                 0.575556 	51.393056;
                 0.578611 	51.390556;
                 0.582500 	51.388056;
                 0.588056 	51.386667;
                 0.593889 	51.386944;
                 0.600278 	51.385833;
                 0.605000 	51.383889;
                 0.608056 	51.381389;
                 0.611944 	51.379167;
                 0.617222 	51.380000;
                 0.619722 	51.383333;
                 0.622778 	51.386389;
                 0.626389 	51.388611;
                 0.632500 	51.388889;
                 0.638056 	51.388333;
                 0.644167 	51.388611;
                 0.647778 	51.391111;
                 0.653333 	51.391944;
                 0.659722 	51.391667;
                 0.663611 	51.389444;
                 0.666667 	51.386944;
                 0.669722 	51.384167;
                 0.671944 	51.381111;
                 0.677500 	51.379722;
                 0.681667 	51.381667;
                 0.685278 	51.383889;
                 0.690556 	51.384722;
                 0.696667 	51.385000;
                 0.701389 	51.386389;
                 0.706389 	51.387778;
                 0.710000 	51.390278;
                 0.712222 	51.393611;
                 0.711944 	51.397500;
                 0.709722 	51.400278;
                 0.706667 	51.403056;
                 0.704444 	51.406111;
                 0.703333 	51.409444;
                 0.704444 	51.413889;
                 0.706111 	51.417778;
                 0.708056 	51.421944;
                 0.711667 	51.424167;
                 0.717778 	51.424722;
                 0.722500 	51.422778;
                 0.725278 	51.420000;
                 0.726667 	51.416667;
                 0.727222 	51.413056;
                 0.728611 	51.409444;
                 0.734167 	51.408056;
                 0.736944 	51.410833;
                 0.739444 	51.414444;
                 0.741389 	51.418333;
                 0.740833 	51.422222;
                 0.740278 	51.425833;
                 0.740833 	51.430000;
                 0.744444 	51.432500;
                 0.746667 	51.435833;
                 0.747222 	51.440000;
                 0.748889 	51.443889;
                 0.754444 	51.444722;
                 0.760278 	51.445000;
                 0.766667 	51.443889;
                 0.771389 	51.442222;
                 0.777778 	51.440833;
                 0.783889 	51.441389;
                 0.791111 	51.440556;
                 0.796667 	51.438889;
                 0.801389 	51.437222;
                 0.806111 	51.435278;
                 0.812500 	51.434167;
                 0.818889 	51.433056;
                 0.825278 	51.431667;
                 0.831667 	51.430556;
                 0.838056 	51.429444;
                 0.844444 	51.428333;
                 0.851667 	51.427500;
                 0.858056 	51.426389;
                 0.863611 	51.425000;
                 0.870833 	51.424167;
                 0.878056 	51.423333;
                 0.885278 	51.422778;
                 0.892500 	51.421944;
                 0.898889 	51.420833;
                 0.904167 	51.419167;
                 0.908889 	51.417222;
                 0.911944 	51.414722;
                 0.914167 	51.411667;
                 0.916389 	51.408611;
                 0.918333 	51.405556;
                 0.922222 	51.403333;
                 0.926944 	51.401389;
                 0.930833 	51.399167;
                 0.933889 	51.396667;
                 0.936667 	51.393889;
                 0.938889 	51.390833;
                 0.941111 	51.388056;
                 0.944167 	51.385278;
                 0.945278 	51.381944;
                 0.947500 	51.378889;
                 0.948056 	51.375278;
                 0.946944 	51.371667;
                 0.942778 	51.369722;
                 0.936667 	51.369444;
                 0.929444 	51.370278;
                 0.925278 	51.368333;
                 0.921111 	51.366389;
                 0.917500 	51.364167;
                 0.913889 	51.361667;
                 0.909444 	51.359722;
                 0.904722 	51.358333;
                 0.899444 	51.357500;
                 0.892778 	51.357778;
                 0.885556 	51.358611;
                 0.880000 	51.360278;
                 0.873611 	51.361389;
                 0.867222 	51.362500;
                 0.860833 	51.363611;
                 0.853611 	51.364444;
                 0.847222 	51.365556;
                 0.840000 	51.366389;
                 0.833333 	51.366389;
                 0.826944 	51.366667;
                 0.819722 	51.367500;
                 0.812500 	51.368333;
                 0.805278 	51.368889;
                 0.798056 	51.369722;
                 0.793611 	51.368056;
                 0.795833 	51.365000;
                 0.798889 	51.362222;
                 0.803611 	51.360278;
                 0.809167 	51.358889;
                 0.814722 	51.357222;
                 0.821111 	51.356111;
                 0.827500 	51.355000;
                 0.834722 	51.354444;
                 0.841667 	51.353611;
                 0.848889 	51.352778;
                 0.856111 	51.351944;
                 0.863333 	51.351389;
                 0.870556 	51.350556;
                 0.877778 	51.349722;
                 0.884167 	51.348611;
                 0.888889 	51.346667;
                 0.893611 	51.344722;
                 0.900000 	51.343611;
                 0.906389 	51.342500;
                 0.913611 	51.341667;
                 0.919444 	51.341944;
                 0.925000 	51.342778;
                 0.929722 	51.344167;
                 0.935278 	51.345000;
                 0.940000 	51.346389;
                 0.944722 	51.347778;
                 0.950278 	51.348611;
                 0.956944 	51.348333;
                 0.963611 	51.348056;
                 0.970833 	51.347222;
                 0.976667 	51.347500;
                 0.983333 	51.347222;
                 0.989444 	51.347500;
                 0.996111 	51.347222;
                 1.001389 	51.348056;
                 1.006944 	51.348889;
                 1.011111 	51.350833;
                 1.015833 	51.352222;
                 1.020556 	51.353333;
                 1.025000 	51.355278;
                 1.028611 	51.357778;
                 1.032778 	51.359444;
                 1.036944 	51.361389;
                 1.041111 	51.363333;
                 1.046111 	51.364722;
                 1.050833 	51.365833;
                 1.055833 	51.367222;
                 1.061111 	51.368056;
                 1.067222 	51.368333;
                 1.073056 	51.368611;
                 1.079167 	51.368889;
                 1.085278 	51.369167;
                 1.091944 	51.368889;
                 1.097778 	51.369167;
                 1.103889 	51.369444;
                 1.110000 	51.369722;
                 1.116389 	51.369444;
                 1.121944 	51.370278;
                 1.127500 	51.371111;
                 1.132778 	51.371944;
                 1.137500 	51.373333;
                 1.142500 	51.374444;
                 1.147778 	51.375278;
                 1.153889 	51.375556;
                 1.160000 	51.375833;
                 1.165833 	51.376111;
                 1.171389 	51.376944;
                 1.177500 	51.377222;
                 1.182778 	51.378056;
                 1.187778 	51.379444;
                 1.193333 	51.380556;
                 1.198611 	51.381389;
                 1.203333 	51.382778;
                 1.208889 	51.383611;
                 1.215000 	51.383889;
                 1.220833 	51.384167;
                 1.227500 	51.383889;
                 1.234167 	51.383611;
                 1.241389 	51.382778;
                 1.248056 	51.382500;
                 1.254722 	51.382222;
                 1.261111 	51.381944;
                 1.268333 	51.381111;
                 1.274444 	51.381389;
                 1.279444 	51.382778;
                 1.284167 	51.384167;
                 1.288889 	51.385556;
                 1.294444 	51.386111;
                 1.300556 	51.386389;
                 1.306944 	51.386111;
                 1.313056 	51.386389;
                 1.319722 	51.386111;
                 1.325000 	51.386944;
                 1.331667 	51.386667;
                 1.337222 	51.387500;
                 1.342500 	51.388333;
                 1.347500 	51.389722;
                 1.352778 	51.390556;
                 1.358889 	51.390833;
                 1.364444 	51.391389;
                 1.370278 	51.391667;
                 1.376944 	51.391389;
                 1.383056 	51.391667;
                 1.388333 	51.392500;
                 1.393333 	51.393889;
                 1.398611 	51.394722;
                 1.404722 	51.395000;
                 1.410833 	51.395278;
                 1.416667 	51.395278;
                 1.422778 	51.395556;
                 1.428889 	51.395833;
                 1.435556 	51.395556;
                 1.441944 	51.395278;
                 1.448333 	51.394167;
                 1.452222 	51.391944;
                 1.452222 	51.391944;
                 1.453056 	51.390000;
                 1.453889 	51.387222;
                 1.454167 	51.383333;
                 1.454722 	51.379444;
                 1.454167 	51.375556;
                 1.453056 	51.370833;
                 1.451667 	51.366389;
                 1.449722 	51.362500;
                 1.447778 	51.358611;
                 1.446111 	51.354444;
                 1.444167 	51.350556;
                 1.442222 	51.346667;
                 1.440278 	51.342778;
                 1.437222 	51.339722;
                 1.434167 	51.336944;
                 1.431111 	51.333889;
                 1.427500 	51.331667;
                 1.423333 	51.329722;
                 1.419167 	51.327778;
                 1.413611 	51.326944;
                 1.406944 	51.327222;
                 1.400278 	51.327500;
                 1.394444 	51.327222;
                 1.388889 	51.326667;
                 1.384167 	51.325278;
                 1.380000 	51.323333;
                 1.376944 	51.320556;
                 1.375556 	51.316111;
                 1.376944 	51.312500;
                 1.380556 	51.310278;
                 1.382778 	51.307222;
                 1.384722 	51.304167;
                 1.385278 	51.300556;
                 1.383889 	51.296111;
                 1.382778 	51.291389;
                 1.383056 	51.287778;
                 1.384444 	51.284444;
                 1.387222 	51.281667;
                 1.389444 	51.278611;
                 1.391667 	51.275556;
                 1.393611 	51.272778;
                 1.395833 	51.269722;
                 1.396944 	51.266389;
                 1.399167 	51.263333;
                 1.400278 	51.259722;
                 1.401667 	51.256389;
                 1.402778 	51.253056;
                 1.405000 	51.250000;
                 1.405556 	51.246389;
                 1.406667 	51.242778;
                 1.407222 	51.239167;
                 1.407500 	51.235278;
                 1.408056 	51.231667;
                 1.408333 	51.227778;
                 1.408611 	51.224167;
                 1.410000 	51.220833;
                 1.410556 	51.216944;
                 1.410833 	51.213056;
                 1.411111 	51.209444;
                 1.410833 	51.205278;
                 1.411111 	51.201667;
                 1.410833 	51.197500;
                 1.409444 	51.193056;
                 1.409167 	51.188889;
                 1.407778 	51.184167;
                 1.406389 	51.179722;
                 1.404722 	51.175833;
                 1.402778 	51.171944;
                 1.400278 	51.168333;
                 1.398333 	51.164444;
                 1.396111 	51.161111;
                 1.393056 	51.158056;
                 1.390556 	51.154722;
                 1.386667 	51.151944;
                 1.383611 	51.148889;
                 1.380556 	51.146111;
                 1.377500 	51.143056;
                 1.373889 	51.140833;
                 1.371111 	51.138611;
                 1.368889 	51.136111;
                 1.365278 	51.133889;
                 1.361667 	51.131389;
                 1.358056 	51.129167;
                 1.353889 	51.127222;
                 1.349167 	51.125833;
                 1.344167 	51.124444;
                 1.338889 	51.123611;
                 1.333333 	51.123056;
                 1.327500 	51.122778;
                 1.322778 	51.121389;
                 1.317778 	51.120000;
                 1.313611 	51.118333;
                 1.309444 	51.116389;
                 1.304722 	51.115000;
                 1.300278 	51.113056;
                 1.295556 	51.111944;
                 1.290833 	51.110556;
                 1.285833 	51.109167;
                 1.281111 	51.107778;
                 1.276944 	51.106111;
                 1.272222 	51.104722;
                 1.267222 	51.103333;
                 1.262500 	51.101944;
                 1.257222 	51.101389;
                 1.251667 	51.100556;
                 1.245833 	51.100278;
                 1.239167 	51.100556;
                 1.231944 	51.101389;
                 1.226111 	51.101111;
                 1.220556 	51.100278;
                 1.216944 	51.097778;
                 1.213333 	51.095278;
                 1.210833 	51.091944;
                 1.208056 	51.089167;
                 1.205000 	51.086111;
                 1.201389 	51.083889;
                 1.196944 	51.081944;
                 1.193611 	51.080556;
                 1.189444 	51.078611;
                 1.184167 	51.077778;
                 1.179444 	51.076389;
                 1.173889 	51.075556;
                 1.168611 	51.074722;
                 1.161944 	51.075000;
                 1.155833 	51.074722;
                 1.150000 	51.074444;
                 1.143889 	51.074167;
                 1.137500 	51.074444;
                 1.130833 	51.074722;
                 1.123611 	51.075556;
                 1.117222 	51.075833;
                 1.111111 	51.075556;
                 1.105833 	51.074722;
                 1.100278 	51.073889;
                 1.095556 	51.072500;
                 1.091389 	51.070833;
                 1.087222 	51.068889;
                 1.083056 	51.066944;
                 1.078889 	51.065278;
                 1.074444 	51.063333;
                 1.070278 	51.061389;
                 1.066111 	51.059722;
                 1.061944 	51.057778;
                 1.058333 	51.055278;
                 1.054167 	51.053611;
                 1.050000 	51.051667;
                 1.045833 	51.049722;
                 1.042222 	51.047500;
                 1.038056 	51.045556;
                 1.034444 	51.043056;
                 1.030000 	51.041389;
                 1.026389 	51.038889;
                 1.023056 	51.036389;
                 1.019444 	51.034167;
                 1.015833 	51.031667;
                 1.012222 	51.029444;
                 1.008611 	51.026944;
                 1.005000 	51.024444;
                 1.001389 	51.022222;
                 0.998333 	51.019167;
                 0.994722 	51.016944;
                 0.991667 	51.013889;
                 0.988611 	51.011111;
                 0.986111 	51.007500;
                 0.983889 	51.004167;
                 0.981389 	51.000556;
                 0.978889 	50.997222;
                 0.977222 	50.993333;
                 0.974722 	50.989722;
                 0.973056 	50.985833;
                 0.971111 	50.981944;
                 0.970000 	50.977222;
                 0.968611 	50.972778;
                 0.968333 	50.968611;
                 0.968611 	50.965000;
                 0.969167 	50.961111;
                 0.969722 	50.957500;
                 0.969167 	50.953333;
                 0.969722 	50.949444;
                 0.970000 	50.945833;
                 0.971389 	50.942500;
                 0.972778 	50.938889;
                 0.973889 	50.935556;
                 0.975278 	50.932222;
                 0.978333 	50.929722;
                 0.979444 	50.926111;
                 0.980000 	50.922500;
                 0.978611 	50.918056;
                 0.975833 	50.915000;
                 0.970833 	50.913611;
                 0.965556 	50.912778;
                 0.959722 	50.912500;
                 0.955833 	50.912500;
                 0.949444 	50.912778;
                 0.942222 	50.913611;
                 0.935000 	50.914444;
                 0.928056 	50.915000;
                 0.920833 	50.915833;
                 0.913611 	50.916667;
                 0.908056 	50.917222;
                 0.900833 	50.918056;
                 0.893611 	50.918889;
                 0.886389 	50.919722;
                 0.880278 	50.920833;
                 0.873889 	50.921944;
                 0.867500 	50.923056;
                 0.862222 	50.924444;
                 0.856667 	50.926111;
                 0.850278 	50.927222;
                 0.844167 	50.928333;
                 0.838611 	50.929722;
                 0.832222 	50.930833;
                 0.826667 	50.932500;
                 0.820556 	50.933611;
                 0.814167 	50.934722;
                 0.806944 	50.935556;
                 0.800000 	50.936389;
                 0.793333 	50.936389;
                 0.787222 	50.936111;
                 0.781389 	50.935833;
                 0.777222 	50.934167;
                 0.773611 	50.931667;
                 0.770278 	50.930278;
                 0.766111 	50.928333;
                 0.761944 	50.926389;
                 0.757778 	50.924444;
                 0.753611 	50.922778;
                 0.749444 	50.920833;
                 0.745278 	50.918889;
                 0.741111 	50.916944;
                 0.736944 	50.915278;
                 0.732778 	50.913333;
                 0.729167 	50.910833;
                 0.725000 	50.909167;
                 0.720833 	50.907222;
                 0.716667 	50.905278;
                 0.713056 	50.902778;
                 0.709444 	50.900556;
                 0.705833 	50.898056;
                 0.702778 	50.895278;
                 0.699444 	50.892778;
                 0.696389 	50.889722;
                 0.692778 	50.887500;
                 0.689167 	50.885000;
                 0.686111 	50.882222;
                 0.682500 	50.879722;
                 0.679167 	50.877222;
                 0.675556 	50.875000;
                 0.671389 	50.873056;
                 0.667222 	50.871111;
                 0.662500 	50.869722;
                 0.656944 	50.868889;
                 0.652222 	50.867778;
                 0.646944 	50.866944;
                 0.642222 	50.865556;
                 0.636944 	50.864722;
                 0.631389 	50.863889;
                 0.626667 	50.862500;
                 0.621389 	50.861667;
                 0.616667 	50.860278;
                 0.611111 	50.859444;
                 0.605833 	50.858611;
                 0.600556 	50.857778;
                 0.595278 	50.856944;
                 0.589722 	50.856111;
                 0.585000 	50.854722;
                 0.580833 	50.853056;
                 0.576111 	50.851667;
                 0.571944 	50.849722;
                 0.567222 	50.848333;
                 0.561944 	50.847500;
                 0.555833 	50.847222;
                 0.550556 	50.846389;
                 0.544722 	50.846111;
                 0.538611 	50.845833;
                 0.532778 	50.845556;
                 0.526667 	50.845278;
                 0.521389 	50.844444;
                 0.516667 	50.843056;
                 0.511944 	50.841667;
                 0.508611 	50.840556;
                 0.503333 	50.839722;
                 0.498611 	50.838333;
                 0.493333 	50.837500;
                 0.488056 	50.836667;
                 0.482500 	50.835833;
                 0.476667 	50.835556;
                 0.470833 	50.835278;
                 0.465278 	50.834444;
                 0.459444 	50.834167;
                 0.452778 	50.834444;
                 0.447500 	50.833611;
                 0.441667 	50.833056;
                 0.436111 	50.832500;
                 0.430833 	50.831389;
                 0.426111 	50.830278;
                 0.420833 	50.829444;
                 0.416111 	50.828056;
                 0.410556 	50.827222;
                 0.405833 	50.825833;
                 0.400556 	50.825000;
                 0.395833 	50.823611;
                 0.391111 	50.822222;
                 0.386389 	50.820833;
                 0.381667 	50.819444;
                 0.376944 	50.818056;
                 0.372222 	50.816667;
                 0.368056 	50.815000;
                 0.363333 	50.813611;
                 0.358333 	50.812222;
                 0.354444 	50.810278;
                 0.349444 	50.808889;
                 0.346111 	50.806389;
                 0.343056 	50.803611;
                 0.341944 	50.799167;
                 0.340833 	50.794444;
                 0.337222 	50.792222;
                 0.333611 	50.789722;
                 0.328889 	50.788333;
                 0.324167 	50.786944;
                 0.320833 	50.785556;
                 0.316111 	50.784167;
                 0.311389 	50.782778;
                 0.307222 	50.780833;
                 0.303611 	50.778333;
                 0.300000 	50.776111;
                 0.296667 	50.773611;
                 0.293611 	50.770556;
                 0.290833 	50.767778;
                 0.287222 	50.765278;
                 0.284722 	50.761944;
                 0.281944 	50.758889;
                 0.279444 	50.755556;
                 0.277222 	50.751944;
                 0.274167 	50.749167;
                 0.271389 	50.746111;
                 0.267778 	50.743611;
                 0.263611 	50.741944;
                 0.258889 	50.740556;
                 0.254167 	50.739167;
                 0.248333 	50.738889;
                 0.242222 	50.738333;
                 0.235278 	50.739167;
                 0.228056 	50.740000;
                 0.221667 	50.741111;
                 0.215556 	50.742222;
                 0.210000 	50.743611;
                 0.205278 	50.745556;
                 0.199722 	50.746944;
                 0.195278 	50.748889;
                 0.190556 	50.750556;
                 0.186667 	50.752778;
                 0.181944 	50.754722;
                 0.177500 	50.756667;
                 0.171944 	50.758056;
                 0.166389 	50.759444;
                 0.160278 	50.760833;
                 0.153056 	50.761389;
                 0.146389 	50.761667;
                 0.140556 	50.761389;
                 0.135278 	50.760556;
                 0.129444 	50.760000;
                 0.123333 	50.759722;
                 0.116389 	50.760556;
                 0.110833 	50.761944;
                 0.106111 	50.763889;
                 0.102222 	50.766111;
                 0.098611 	50.768333;
                 0.093889 	50.770278;
                 0.089167 	50.771944;
                 0.084444 	50.773889;
                 0.080000 	50.775833;
                 0.075278 	50.777500;
                 0.069722 	50.779167;
                 0.064167 	50.780556;
                 0.057222 	50.781389;
                 0.051111 	50.780833;
                 0.046389 	50.779444;
                 0.041111 	50.778611;
                 0.034722 	50.779722;
                 0.029722 	50.779722;
                 0.023333 	50.780833;
                 0.017778 	50.782500;
                 0.013333 	50.784167;
                 0.007778 	50.785833;
                 0.002222 	50.787222;
                 -0.003333	50.788611;
                 -0.008889	50.790000;
                 -0.013333	50.791944;
                 -0.019167	50.793333;
                 -0.024444	50.795000;
                 -0.030000	50.796389;
                 -0.035278	50.797778;
                 -0.041111	50.799444;
                 -0.047500	50.800278;
                 -0.052778	50.801944;
                 -0.058333	50.803333;
                 -0.064444	50.804444;
                 -0.070833	50.805556;
                 -0.077222	50.806667;
                 -0.082778	50.808056;
                 -0.089167	50.809167;
                 -0.094444	50.810556;
                 -0.100833	50.811667;
                 -0.107222	50.812778;
                 -0.114167	50.813611;
                 -0.120000	50.815000;
                 -0.126111	50.816111;
                 -0.132222	50.817222;
                 -0.138056	50.818611;
                 -0.144167	50.819722;
                 -0.150556	50.820833;
                 -0.156667	50.821944;
                 -0.160556	50.821667;
                 -0.164167	50.822500;
                 -0.171111	50.823333;
                 -0.177500	50.824444;
                 -0.183889	50.825556;
                 -0.190833	50.826111;
                 -0.198056	50.826944;
                 -0.204722	50.826944;
                 -0.210556	50.826667;
                 -0.216667	50.826389;
                 -0.222500	50.826111;
                 -0.226944	50.824722;
                 -0.231111	50.822778;
                 -0.236944	50.823333;
                 -0.241667	50.825000;
                 -0.248056	50.826111;
                 -0.255000	50.826944;
                 -0.260556	50.826111;
                 -0.266389	50.825556;
                 -0.271667	50.824722;
                 -0.277500	50.824444;
                 -0.283056	50.823611;
                 -0.287778	50.822222;
                 -0.293056	50.821389;
                 -0.297222	50.819444;
                 -0.301389	50.819722;
                 -0.306944	50.818889;
                 -0.311389	50.817500;
                 -0.316944	50.816667;
                 -0.322222	50.815833;
                 -0.326944	50.814444;
                 -0.331667	50.813056;
                 -0.336389	50.811667;
                 -0.341111	50.810278;
                 -0.345833	50.808889;
                 -0.350556	50.807500;
                 -0.355833	50.806667;
                 -0.361111	50.805833;
                 -0.366389	50.804722;
                 -0.372222	50.804444;
                 -0.378333	50.804167;
                 -0.384167	50.803889;
                 -0.390000	50.803333;
                 -0.396111	50.803056;
                 -0.402222	50.802778;
                 -0.406667	50.801389;
                 -0.410833	50.799444;
                 -0.415833	50.798056;
                 -0.419722	50.796111;
                 -0.425000	50.795278;
                 -0.430556	50.794444;
                 -0.435833	50.793333;
                 -0.441667	50.793056;
                 -0.448611	50.793889;
                 -0.455000	50.794722;
                 -0.461389	50.795833;
                 -0.467222	50.795556;
                 -0.473889	50.795833;
                 -0.479722	50.795278;
                 -0.485833	50.795000;
                 -0.491667	50.794722;
                 -0.496944	50.793611;
                 -0.503056	50.793333;
                 -0.508333	50.792500;
                 -0.513611	50.791667;
                 -0.519167	50.790833;
                 -0.524167	50.790000;
                 -0.529444	50.788889;
                 -0.535000	50.788056;
                 -0.538889	50.787222;
                 -0.544722	50.786667;
                 -0.550000	50.785833;
                 -0.555833	50.785556;
                 -0.562500	50.785556;
                 -0.569167	50.785833;
                 -0.575000	50.785556;
                 -0.580278	50.784444;
                 -0.585000	50.783056;
                 -0.590278	50.782222;
                 -0.595833	50.781389;
                 -0.601667	50.781111;
                 -0.606944	50.780000;
                 -0.612778	50.779722;
                 -0.618056	50.778889;
                 -0.623889	50.778611;
                 -0.629444	50.777500;
                 -0.634722	50.776667;
                 -0.640556	50.776389;
                 -0.646111	50.775556;
                 -0.651389	50.774722;
                 -0.656667	50.773611;
                 -0.661944	50.772778;
                 -0.667222	50.771944;
                 -0.673056	50.771667;
                 -0.679167	50.771111;
                 -0.685000	50.770833;
                 -0.690833	50.770556;
                 -0.696667	50.770000;
                 -0.702778	50.769722;
                 -0.708611	50.769444;
                 -0.714167	50.768333;
                 -0.718889	50.766944;
                 -0.722778	50.765000;
                 -0.726389	50.762778;
                 -0.729722	50.760278;
                 -0.733333	50.757778;
                 -0.737500	50.755833;
                 -0.742222	50.756667;
                 -0.746389	50.758056;
                 -0.746111	50.762222;
                 -0.746944	50.766111;
                 -0.751389	50.767778;
                 -0.757222	50.767500;
                 -0.760278	50.764444;
                 -0.761111	50.760000;
                 -0.758333	50.757222;
                 -0.754444	50.755278;
                 -0.750556	50.753056;
                 -0.748889	50.749444;
                 -0.748611	50.745833;
                 -0.749444	50.741389;
                 -0.751111	50.737222;
                 -0.753056	50.733333;
                 -0.755000	50.729722;
                 -0.758056	50.726944;
                 -0.762222	50.725000;
                 -0.766944	50.723611;
                 -0.772778	50.723056;
                 -0.778611	50.722778;
                 -0.785000	50.723889;
                 -0.790556	50.725278;
                 -0.795833	50.726667;
                 -0.799722	50.728889;
                 -0.803611	50.731111;
                 -0.807500	50.733333;
                 -0.812222	50.735000;
                 -0.816111	50.737222;
                 -0.820000	50.739444;
                 -0.823889	50.741667;
                 -0.828611	50.743611;
                 -0.833333	50.745278;
                 -0.837778	50.747222;
                 -0.842500	50.748889;
                 -0.847500	50.750833;
                 -0.852778	50.752222;
                 -0.857500	50.754167;
                 -0.863056	50.755556;
                 -0.867778	50.757222;
                 -0.873333	50.758611;
                 -0.878611	50.760278;
                 -0.884167	50.761667;
                 -0.888889	50.763333;
                 -0.894444	50.764722;
                 -0.895278	50.764444;
                 -0.898611	50.765278;
                 -0.904167	50.766667;
                 -0.908333	50.768889;
                 -0.911944	50.771111;
                 -0.915000	50.773611;
                 -0.916389	50.776944;
                 -0.915556	50.780556;
                 -0.911667	50.782500;
                 -0.905556	50.782778;
                 -0.899722	50.783333;
                 -0.897500	50.786667;
                 -0.895278	50.790278;
                 -0.892222	50.793333;
                 -0.889444	50.796111;
                 -0.885278	50.798056;
                 -0.878889	50.798056;
                 -0.872778	50.798333;
                 -0.868889	50.800278;
                 -0.865833	50.803333;
                 -0.864722	50.807778;
                 -0.868056	50.810278;
                 -0.871944	50.812500;
                 -0.877222	50.813889;
                 -0.883056	50.814444;
                 -0.886111	50.817222;
                 -0.887500	50.820556;
                 -0.888889	50.823889;
                 -0.890278	50.827222;
                 -0.893611	50.829722;
                 -0.898889	50.831111;
                 -0.904167	50.830278;
                 -0.907222	50.827222;
                 -0.908333	50.822778;
                 -0.908889	50.819167;
                 -0.910556	50.815278;
                 -0.914167	50.812778;
                 -0.918889	50.811389;
                 -0.923889	50.812222;
                 -0.926667	50.814722;
                 -0.927500	50.818611;
                 -0.926389	50.823056;
                 -0.924167	50.826667;
                 -0.922500	50.830556;
                 -0.921389	50.835000;
                 -0.922778	50.838611;
                 -0.926111	50.841111;
                 -0.931389	50.842500;
                 -0.937222	50.842222;
                 -0.943333	50.841667;
                 -0.949167	50.841389;
                 -0.954167	50.840000;
                 -0.958056	50.838056;
                 -0.962222	50.836111;
                 -0.965000	50.833056;
                 -0.961944	50.830556;
                 -0.957222	50.828611;
                 -0.951667	50.827222;
                 -0.947778	50.825000;
                 -0.944167	50.822778;
                 -0.943333	50.819167;
                 -0.945556	50.815556;
                 -0.949167	50.813056;
                 -0.951944	50.810278;
                 -0.954167	50.806667;
                 -0.954444	50.802500;
                 -0.952778	50.799167;
                 -0.953056	50.795000;
                 -0.954722	50.791111;
                 -0.955000	50.786944;
                 -0.952778	50.783889;
                 -0.950556	50.781111;
                 -0.950000	50.777222;
                 -0.953333	50.774722;
                 -0.959444	50.774444;
                 -0.965833	50.774444;
                 -0.973056	50.775278;
                 -0.979444	50.776111;
                 -0.985833	50.777222;
                 -0.991111	50.778611;
                 -0.997778	50.778889;
                 -1.001944	50.776944;
                 -1.004722	50.773889;
                 -1.008889	50.771944;
                 -1.014167	50.771111;
                 -1.018056	50.773333;
                 -1.020278	50.776111;
                 -1.020833	50.780000;
                 -1.018611	50.783611;
                 -1.015556	50.786389;
                 -1.012222	50.788889;
                 -1.008333	50.790833;
                 -1.004167	50.792778;
                 -1.000000	50.794722;
                 -0.995833	50.796667;
                 -0.992500	50.799167;
                 -0.990278	50.802500;
                 -0.988611	50.806667;
                 -0.988333	50.810833;
                 -0.989722	50.814167;
                 -0.988889	50.817778;
                 -0.986944	50.821111;
                 -0.984444	50.824722;
                 -0.982778	50.828611;
                 -0.982778	50.832778;
                 -0.984167	50.836111;
                 -0.986944	50.838889;
                 -0.991667	50.840556;
                 -0.998056	50.841667;
                 -1.005278	50.842222;
                 -1.012500	50.842778;
                 -1.017778	50.841944;
                 -1.022500	50.840556;
                 -1.025833	50.838056;
                 -1.028889	50.835278;
                 -1.031111	50.831667;
                 -1.032222	50.827222;
                 -1.033611	50.823056;
                 -1.033889	50.818889;
                 -1.035556	50.815000;
                 -1.037222	50.811111;
                 -1.038333	50.806389;
                 -1.037500	50.802778;
                 -1.036944	50.798889;
                 -1.036944	50.795833;
                 -1.039444	50.792222;
                 -1.042778	50.789722;
                 -1.047500	50.788333;
                 -1.051944	50.786944;
                 -1.056944	50.785556;
                 -1.061389	50.784167;
                 -1.065556	50.782222;
                 -1.069722	50.780278;
                 -1.075000	50.779444;
                 -1.079722	50.777778;
                 -1.086111	50.778056;
                 -1.092500	50.779167;
                 -1.097222	50.780833;
                 -1.100278	50.783333;
                 -1.103611	50.786111;
                 -1.105833	50.788889;
                 -1.106111	50.792778;
                 -1.106111	50.796944;
                 -1.104444	50.800833;
                 -1.101111	50.803333;
                 -1.096111	50.804722;
                 -1.092222	50.806667;
                 -1.089444	50.809722;
                 -1.087778	50.813611;
                 -1.090000	50.816667;
                 -1.092222	50.819722;
                 -1.091944	50.823889;
                 -1.090000	50.826389;
                 -1.083611	50.826111;
                 -1.078056	50.827222;
                 -1.073889	50.829167;
                 -1.070556	50.831389;
                 -1.068333	50.835000;
                 -1.069722	50.838333;
                 -1.072778	50.840833;
                 -1.076667	50.843056;
                 -1.080556	50.845278;
                 -1.086111	50.846667;
                 -1.091667	50.847222;
                 -1.097222	50.846389;
                 -1.101944	50.845000;
                 -1.106667	50.843611;
                 -1.110556	50.841667;
                 -1.115833	50.840556;
                 -1.121944	50.840278;
                 -1.129167	50.840833;
                 -1.136111	50.841667;
                 -1.141944	50.843056;
                 -1.148056	50.843889;
                 -1.154444	50.845000;
                 -1.160000	50.844167;
                 -1.163333	50.841667;
                 -1.163333	50.838333;
                 -1.160833	50.835556;
                 -1.155556	50.834167;
                 -1.148056	50.833333;
                 -1.142778	50.831944;
                 -1.141389	50.828611;
                 -1.139167	50.825556;
                 -1.136111	50.823056;
                 -1.131944	50.820833;
                 -1.128056	50.818611;
                 -1.124167	50.816389;
                 -1.120278	50.814444;
                 -1.118056	50.811389;
                 -1.117778	50.807500;
                 -1.117778	50.803333;
                 -1.120000	50.800000;
                 -1.122778	50.796944;
                 -1.124444	50.793056;
                 -1.125556	50.788333;
                 -1.126389	50.783889;
                 -1.129444	50.781111;
                 -1.134167	50.779444;
                 -1.139444	50.778611;
                 -1.145278	50.778333;
                 -1.151111	50.778889;
                 -1.157500	50.779722;
                 -1.163611	50.780833;
                 -1.168333	50.782500;
                 -1.173889	50.783889;
                 -1.178611	50.785833;
                 -1.182500	50.788056;
                 -1.186389	50.790278;
                 -1.190278	50.792500;
                 -1.194167	50.794722;
                 -1.197222	50.797222;
                 -1.199722	50.799167;
                 -1.202778	50.801667;
                 -1.205833	50.804444;
                 -1.208889	50.806944;
                 -1.211944	50.809444;
                 -1.215833	50.811667;
                 -1.219722	50.813889;
                 -1.223611	50.816111;
                 -1.228333	50.818056;
                 -1.233056	50.819722;
                 -1.237778	50.821389;
                 -1.242500	50.823333;
                 -1.247222	50.825000;
                 -1.253056	50.826389;
                 -1.258333	50.828056;
                 -1.263889	50.829444;
                 -1.269167	50.830833;
                 -1.275556	50.831667;
                 -1.281389	50.833056;
                 -1.285833	50.835000;
                 -1.290833	50.836667;
                 -1.294722	50.838889;
                 -1.298611	50.841111;
                 -1.302500	50.843333;
                 -1.306389	50.845556;
                 -1.309444	50.848056;
                 -1.313333	50.850278;
                 -1.316389	50.852778;
                 -1.320278	50.855000;
                 -1.324167	50.857222;
                 -1.328056	50.859444;
                 -1.331111	50.861944;
                 -1.335000	50.864167;
                 -1.338889	50.866389;
                 -1.342778	50.868611;
                 -1.346667	50.870833;
                 -1.350556	50.872778;
                 -1.354444	50.875000;
                 -1.357778	50.877778;
                 -1.361667	50.879722;
                 -1.366111	50.881667;
                 -1.370278	50.883889;
                 -1.373889	50.886111;
                 -1.378889	50.887778;
                 -1.385278	50.888889;
                 -1.391944	50.888889;
                 -1.396389	50.890833;
                 -1.399722	50.893333;
                 -1.403611	50.895556;
                 -1.406389	50.898056;
                 -1.410278	50.900278;
                 -1.414444	50.902500;
                 -1.418333	50.904722;
                 -1.423056	50.906389;
                 -1.428611	50.907778;
                 -1.434167	50.909167;
                 -1.439722	50.910556;
                 -1.445278	50.911944;
                 -1.450556	50.913333;
                 -1.456389	50.914722;
                 -1.463611	50.915556;
                 -1.468889	50.914167;
                 -1.472500	50.911667;
                 -1.472500	50.908333;
                 -1.469167	50.905833;
                 -1.465000	50.904444;
                 -1.459722	50.903056;
                 -1.454722	50.901389;
                 -1.449167	50.900000;
                 -1.443889	50.898611;
                 -1.438333	50.897222;
                 -1.433611	50.895278;
                 -1.428056	50.893889;
                 -1.422500	50.892500;
                 -1.416944	50.891111;
                 -1.412222	50.889444;
                 -1.407500	50.887500;
                 -1.404167	50.885000;
                 -1.401111	50.882500;
                 -1.400000	50.879167;
                 -1.396667	50.876389;
                 -1.394167	50.873611;
                 -1.392222	50.870556;
                 -1.388056	50.868333;
                 -1.383611	50.866667;
                 -1.378611	50.864722;
                 -1.375000	50.862500;
                 -1.369444	50.861111;
                 -1.364444	50.859444;
                 -1.360000	50.857500;
                 -1.355278	50.855833;
                 -1.351389	50.853611;
                 -1.347500	50.851389;
                 -1.343611	50.849167;
                 -1.340278	50.846667;
                 -1.337500	50.844167;
                 -1.334167	50.841389;
                 -1.331111	50.838889;
                 -1.328889	50.835833;
                 -1.326667	50.833056;
                 -1.324167	50.830000;
                 -1.322778	50.826667;
                 -1.320556	50.823611;
                 -1.317500	50.821111;
                 -1.310278	50.820556;
                 -1.306389	50.818333;
                 -1.307500	50.813889;
                 -1.309444	50.810278;
                 -1.311667	50.806944;
                 -1.315556	50.804444;
                 -1.318889	50.801944;
                 -1.322222	50.799444;
                 -1.325278	50.796389;
                 -1.328611	50.793889;
                 -1.331389	50.791111;
                 -1.334167	50.788056;
                 -1.338333	50.786111;
                 -1.341667	50.783611;
                 -1.345833	50.781667;
                 -1.349722	50.779722;
                 -1.355278	50.778889;
                 -1.360556	50.777778;
                 -1.365833	50.776944;
                 -1.370556	50.775556;
                 -1.375833	50.774444;
                 -1.381111	50.773611;
                 -1.386944	50.773333;
                 -1.393056	50.772778;
                 -1.398889	50.772500;
                 -1.405556	50.772500;
                 -1.411389	50.772222;
                 -1.417222	50.771667;
                 -1.422500	50.770833;
                 -1.428333	50.770278;
                 -1.433889	50.769444;
                 -1.439167	50.768611;
                 -1.444444	50.767500;
                 -1.448889	50.766111;
                 -1.454444	50.765278;
                 -1.458889	50.763889;
                 -1.463611	50.762500;
                 -1.467778	50.760556;
                 -1.471944	50.758333;
                 -1.476667	50.756944;
                 -1.480833	50.755000;
                 -1.484722	50.753056;
                 -1.488889	50.751111;
                 -1.493333	50.749722;
                 -1.497500	50.747778;
                 -1.502222	50.746389;
                 -1.506111	50.744444;
                 -1.510278	50.742500;
                 -1.514444	50.740556;
                 -1.517778	50.738056;
                 -1.521944	50.736111;
                 -1.525278	50.733611;
                 -1.528889	50.731111;
                 -1.532222	50.728611;
                 -1.536111	50.726667;
                 -1.539722	50.724167;
                 -1.541667	50.720833;
                 -1.543889	50.717222;
                 -1.546389	50.713611;
                 -1.549722	50.711111;
                 -1.555000	50.710278;
                 -1.561389	50.711389;
                 -1.566111	50.713056;
                 -1.570000	50.715278;
                 -1.573889	50.717500;
                 -1.577778	50.719722;
                 -1.581667	50.721667;
                 -1.587222	50.723056;
                 -1.592778	50.724722;
                 -1.597500	50.726389;
                 -1.600278	50.726667;
                 -1.606667	50.727500;
                 -1.612222	50.728889;
                 -1.618611	50.730000;
                 -1.623889	50.731389;
                 -1.629722	50.732778;
                 -1.636111	50.733889;
                 -1.643056	50.734444;
                 -1.649444	50.735278;
                 -1.655833	50.736389;
                 -1.662222	50.737500;
                 -1.668333	50.738333;
                 -1.675556	50.738889;
                 -1.682500	50.739722;
                 -1.689167	50.739722;
                 -1.694444	50.738611;
                 -1.700556	50.738333;
                 -1.705000	50.736944;
                 -1.709722	50.735556;
                 -1.714722	50.733889;
                 -1.719167	50.732500;
                 -1.723889	50.731111;
                 -1.728611	50.729722;
                 -1.733889	50.728611;
                 -1.738611	50.727222;
                 -1.743611	50.726389;
                 -1.749722	50.725833;
                 -1.756389	50.725833;
                 -1.756944	50.723333;
                 -1.753056	50.721111;
                 -1.746667	50.720000;
                 -1.739167	50.719444;
                 -1.732778	50.718611;
                 -1.730000	50.715833;
                 -1.731389	50.711944;
                 -1.736111	50.710556;
                 -1.740833	50.708889;
                 -1.747222	50.709167;
                 -1.754444	50.709722;
                 -1.761111	50.710556;
                 -1.767222	50.711667;
                 -1.773333	50.712778;
                 -1.779722	50.713611;
                 -1.785000	50.714444;
                 -1.790278	50.715833;
                 -1.795833	50.717222;
                 -1.802222	50.718333;
                 -1.808889	50.718333;
                 -1.815278	50.718333;
                 -1.820556	50.717500;
                 -1.825833	50.716667;
                 -1.831667	50.716111;
                 -1.837222	50.715278;
                 -1.841667	50.713611;
                 -1.846944	50.712778;
                 -1.852222	50.711944;
                 -1.857500	50.710833;
                 -1.862778	50.710000;
                 -1.868611	50.709444;
                 -1.873889	50.708611;
                 -1.878611	50.707222;
                 -1.883333	50.705556;
                 -1.888056	50.704167;
                 -1.891389	50.701667;
                 -1.895556	50.699722;
                 -1.898889	50.697222;
                 -1.902778	50.695278;
                 -1.906389	50.692778;
                 -1.909722	50.690278;
                 -1.913889	50.688333;
                 -1.916389	50.685278;
                 -1.920000	50.682778;
                 -1.923333	50.680278;
                 -1.927500	50.678333;
                 -1.932778	50.677500;
                 -1.938333	50.678889;
                 -1.940556	50.681944;
                 -1.940556	50.685833;
                 -1.939444	50.690556;
                 -1.939444	50.694722;
                 -1.939444	50.698611;
                 -1.940833	50.702222;
                 -1.943056	50.705000;
                 -1.945278	50.708056;
                 -1.948611	50.710556;
                 -1.954167	50.711944;
                 -1.960556	50.712778;
                 -1.966389	50.712500;
                 -1.972222	50.711944;
                 -1.977500	50.711111;
                 -1.982778	50.710000;
                 -1.988056	50.709167;
                 -1.992778	50.707778;
                 -1.998056	50.706667;
                 -2.003889	50.706389;
                 -2.009722	50.705833;
                 -2.015000	50.705000;
                 -2.022222	50.705556;
                 -2.027778	50.706944;
                 -2.031111	50.709444;
                 -2.033889	50.711944;
                 -2.038889	50.713889;
                 -2.045278	50.713889;
                 -2.050000	50.712500;
                 -2.054722	50.710833;
                 -2.058611	50.708889;
                 -2.061944	50.706389;
                 -2.063611	50.702500;
                 -2.064444	50.697778;
                 -2.066667	50.694444;
                 -2.070000	50.691944;
                 -2.072778	50.688889;
                 -2.076111	50.686389;
                 -2.079722	50.683889;
                 -2.079722	50.679722;
                 -2.075000	50.678056;
                 -2.071389	50.678333;
                 -2.067500	50.680278;
                 -2.063333	50.682222;
                 -2.059722	50.684722;
                 -2.055833	50.686667;
                 -2.051667	50.688611;
                 -2.048333	50.691111;
                 -2.043611	50.693056;
                 -2.039444	50.695000;
                 -2.035000	50.696389;
                 -2.029167	50.696944;
                 -2.024444	50.695000;
                 -2.023611	50.691389;
                 -2.024722	50.686944;
                 -2.027222	50.683889;
                 -2.029167	50.679722;
                 -2.028889	50.676667;
                 -2.025556	50.674167;
                 -2.020278	50.672778;
                 -2.013056	50.672222;
                 -2.008333	50.673611;
                 -2.004444	50.675556;
                 -2.001111	50.678056;
                 -1.997500	50.680556;
                 -1.993333	50.682500;
                 -1.988056	50.681111;
                 -1.984167	50.678889;
                 -1.980278	50.676667;
                 -1.974722	50.675278;
                 -1.967500	50.674722;
                 -1.961111	50.673889;
                 -1.954722	50.672778;
                 -1.948611	50.671944;
                 -1.941389	50.671389;
                 -1.935000	50.670278;
                 -1.930278	50.668611;
                 -1.928056	50.665556;
                 -1.930556	50.662500;
                 -1.934167	50.660000;
                 -1.938333	50.658056;
                 -1.941667	50.655556;
                 -1.943889	50.652222;
                 -1.942222	50.648889;
                 -1.938333	50.646667;
                 -1.932222	50.645556;
                 -1.925833	50.644722;
                 -1.920278	50.643333;
                 -1.914444	50.641944;
                 -1.910556	50.639722;
                 -1.908333	50.636944;
                 -1.908333	50.632778;
                 -1.911944	50.630278;
                 -1.916389	50.628611;
                 -1.921111	50.627222;
                 -1.926389	50.626389;
                 -1.931111	50.625000;
                 -1.935833	50.623333;
                 -1.938611	50.620278;
                 -1.941389	50.617500;
                 -1.940556	50.613611;
                 -1.938611	50.610833;
                 -1.936944	50.607500;
                 -1.936944	50.603333;
                 -1.940556	50.600833;
                 -1.943056	50.597778;
                 -1.945556	50.593889;
                 -1.948333	50.590833;
                 -1.951667	50.588333;
                 -1.956389	50.586944;
                 -1.962222	50.586389;
                 -1.969444	50.586944;
                 -1.975556	50.588056;
                 -1.982778	50.588611;
                 -1.990000	50.589167;
                 -1.996944	50.589722;
                 -2.003611	50.589722;
                 -2.009444	50.589444;
                 -2.015278	50.588889;
                 -2.020833	50.588056;
                 -2.025833	50.586944;
                 -2.030556	50.585556;
                 -2.035278	50.584167;
                 -2.038611	50.581667;
                 -2.042778	50.579722;
                 -2.047500	50.578056;
                 -2.052500	50.577222;
                 -2.057222	50.578889;
                 -2.059444	50.581944;
                 -2.061944	50.585000;
                 -2.065000	50.587500;
                 -2.068056	50.590000;
                 -2.071944	50.592222;
                 -2.077500	50.593611;
                 -2.083611	50.594444;
                 -2.091111	50.595000;
                 -2.097500	50.595000;
                 -2.103889	50.596111;
                 -2.108611	50.597778;
                 -2.111667	50.600556;
                 -2.115833	50.602500;
                 -2.120278	50.604444;
                 -2.126667	50.605278;
                 -2.133889	50.605833;
                 -2.140833	50.606389;
                 -2.147500	50.606667;
                 -2.153056	50.607778;
                 -2.156944	50.610000;
                 -2.161667	50.611944;
                 -2.167222	50.612222;
                 -2.172500	50.611389;
                 -2.179167	50.611389;
                 -2.183056	50.613611;
                 -2.186944	50.615556;
                 -2.192500	50.616944;
                 -2.198889	50.618056;
                 -2.204722	50.617500;
                 -2.208611	50.615556;
                 -2.214167	50.614722;
                 -2.220000	50.614167;
                 -2.226389	50.614167;
                 -2.233611	50.614722;
                 -2.240000	50.615833;
                 -2.246944	50.616389;
                 -2.253333	50.617222;
                 -2.259722	50.618333;
                 -2.266944	50.618889;
                 -2.273889	50.619444;
                 -2.281389	50.620000;
                 -2.288333	50.620556;
                 -2.295278	50.621111;
                 -2.301944	50.621111;
                 -2.309167	50.621667;
                 -2.316111	50.622222;
                 -2.321667	50.623611;
                 -2.328056	50.624444;
                 -2.332778	50.626389;
                 -2.338333	50.627778;
                 -2.345556	50.628333;
                 -2.351389	50.627778;
                 -2.356667	50.626944;
                 -2.363333	50.626944;
                 -2.368611	50.628333;
                 -2.374167	50.629444;
                 -2.380556	50.630556;
                 -2.386944	50.631389;
                 -2.394167	50.631944;
                 -2.398611	50.630556;
                 -2.403333	50.629167;
                 -2.410000	50.629167;
                 -2.415833	50.628611;
                 -2.421667	50.628333;
                 -2.426944	50.626667;
                 -2.431667	50.625278;
                 -2.435000	50.622778;
                 -2.438611	50.620278;
                 -2.440556	50.616667;
                 -2.441389	50.612222;
                 -2.441667	50.608056;
                 -2.440556	50.604444;
                 -2.440833	50.600278;
                 -2.444444	50.597778;
                 -2.447500	50.595278;
                 -2.450833	50.592778;
                 -2.454444	50.590278;
                 -2.457778	50.587778;
                 -2.460000	50.584167;
                 -2.460000	50.580000;
                 -2.457778	50.577222;
                 -2.455278	50.574167;
                 -2.451389	50.571944;
                 -2.447500	50.570000;
                 -2.441944	50.568611;
                 -2.435278	50.568611;
                 -2.429167	50.567500;
                 -2.426111	50.565000;
                 -2.423611	50.562222;
                 -2.421389	50.559167;
                 -2.418889	50.556111;
                 -2.417500	50.552778;
                 -2.415000	50.550000;
                 -2.414444	50.546111;
                 -2.414444	50.541944;
                 -2.416389	50.538611;
                 -2.420556	50.536667;
                 -2.424444	50.534444;
                 -2.428056	50.532222;
                 -2.431389	50.529722;
                 -2.434167	50.526667;
                 -2.436667	50.523611;
                 -2.438889	50.520000;
                 -2.441667	50.516944;
                 -2.443889	50.513611;
                 -2.445833	50.510000;
                 -2.448611	50.506944;
                 -2.452222	50.504444;
                 -2.456111	50.502500;
                 -2.455278	50.503056;
                 -2.458611	50.503611;
                 -2.460833	50.506667;
                 -2.462778	50.510000;
                 -2.461667	50.514444;
                 -2.460833	50.518056;
                 -2.458889	50.521667;
                 -2.456111	50.524722;
                 -2.453333	50.527500;
                 -2.450000	50.530000;
                 -2.449167	50.534722;
                 -2.450556	50.538056;
                 -2.453056	50.540833;
                 -2.453611	50.544722;
                 -2.452778	50.549167;
                 -2.450556	50.552778;
                 -2.450000	50.557222;
                 -2.452222	50.560278;
                 -2.455278	50.562778;
                 -2.459167	50.565000;
                 -2.464167	50.566667;
                 -2.468889	50.568333;
                 -2.473333	50.570000;
                 -2.476667	50.572778;
                 -2.481111	50.574444;
                 -2.485278	50.576667;
                 -2.488611	50.579167;
                 -2.492500	50.581111;
                 -2.496389	50.583333;
                 -2.499444	50.585833;
                 -2.503333	50.588056;
                 -2.507222	50.590278;
                 -2.511111	50.592222;
                 -2.515000	50.594444;
                 -2.519167	50.596667;
                 -2.523889	50.598333;
                 -2.527778	50.600556;
                 -2.532500	50.602222;
                 -2.536389	50.604444;
                 -2.540278	50.606667;
                 -2.544444	50.608611;
                 -2.548333	50.610833;
                 -2.552222	50.613056;
                 -2.555556	50.615556;
                 -2.559444	50.617778;
                 -2.563889	50.619722;
                 -2.566667	50.621111;
                 -2.570556	50.623056;
                 -2.574444	50.625278;
                 -2.578333	50.627500;
                 -2.581667	50.630000;
                 -2.585556	50.631944;
                 -2.588889	50.634722;
                 -2.592778	50.636667;
                 -2.595833	50.639167;
                 -2.599722	50.641389;
                 -2.603889	50.643611;
                 -2.608333	50.645278;
                 -2.612222	50.647500;
                 -2.617222	50.649167;
                 -2.621111	50.651389;
                 -2.626667	50.652778;
                 -2.631944	50.653889;
                 -2.636944	50.655833;
                 -2.641667	50.657500;
                 -2.647222	50.658889;
                 -2.652222	50.660556;
                 -2.657500	50.661944;
                 -2.663056	50.663333;
                 -2.668611	50.664722;
                 -2.674444	50.665833;
                 -2.678889	50.667778;
                 -2.684444	50.668889;
                 -2.689167	50.670833;
                 -2.694722	50.672222;
                 -2.699722	50.673889;
                 -2.704444	50.675556;
                 -2.709167	50.677222;
                 -2.713889	50.679167;
                 -2.718889	50.680833;
                 -2.723333	50.682500;
                 -2.728056	50.684167;
                 -2.732222	50.686389;
                 -2.736111	50.688611;
                 -2.740000	50.690556;
                 -2.744722	50.692500;
                 -2.748889	50.694444;
                 -2.752778	50.696667;
                 -2.756667	50.698889;
                 -2.760556	50.700833;
                 -2.764722	50.703056;
                 -2.769167	50.704722;
                 -2.773056	50.706944;
                 -2.778889	50.708333;
                 -2.785278	50.709167;
                 -2.792222	50.709722;
                 -2.799444	50.710278;
                 -2.806667	50.710833;
                 -2.813056	50.711667;
                 -2.819444	50.712778;
                 -2.825556	50.713611;
                 -2.831944	50.714444;
                 -2.836111	50.715833;
                 -2.842500	50.716667;
                 -2.849167	50.717500;
                 -2.855278	50.718611;
                 -2.861944	50.719444;
                 -2.867222	50.720833;
                 -2.872778	50.722222;
                 -2.879167	50.723056;
                 -2.885556	50.723889;
                 -2.891944	50.725000;
                 -2.899167	50.725556;
                 -2.905833	50.725556;
                 -2.912222	50.725556;
                 -2.917500	50.724444;
                 -2.922500	50.723333;
                 -2.927222	50.721944;
                 -2.931389	50.720000;
                 -2.935278	50.718056;
                 -2.939167	50.715833;
                 -2.944167	50.714444;
                 -2.948611	50.713056;
                 -2.953056	50.711389;
                 -2.957778	50.710000;
                 -2.962500	50.708333;
                 -2.966944	50.706944;
                 -2.971667	50.705556;
                 -2.975833	50.703333;
                 -2.980556	50.701944;
                 -2.985000	50.700278;
                 -2.989444	50.698889;
                 -2.994167	50.697500;
                 -2.998889	50.695833;
                 -3.004167	50.695000;
                 -3.008889	50.693333;
                 -3.015278	50.693333;
                 -3.021667	50.693333;
                 -3.029167	50.693889;
                 -3.036111	50.694444;
                 -3.042500	50.695278;
                 -3.048889	50.696389;
                 -3.056111	50.696944;
                 -3.061944	50.696389;
                 -3.067778	50.695833;
                 -3.072500	50.694167;
                 -3.077222	50.692778;
                 -3.081667	50.691389;
                 -3.085556	50.689167;
                 -3.089722	50.687222;
                 -3.093889	50.685278;
                 -3.097778	50.683333;
                 -3.102222	50.681667;
                 -3.107500	50.680833;
                 -3.113611	50.680278;
                 -3.120000	50.680278;
                 -3.126944	50.680833;
                 -3.134167	50.681389;
                 -3.141389	50.681667;
                 -3.148056	50.681667;
                 -3.155000	50.682222;
                 -3.162222	50.682778;
                 -3.168611	50.682778;
                 -3.175278	50.682778;
                 -3.181111	50.682222;
                 -3.187778	50.682222;
                 -3.193611	50.681667;
                 -3.199444	50.681389;
                 -3.205278	50.680833;
                 -3.210556	50.679722;
                 -3.215833	50.678889;
                 -3.221667	50.678333;
                 -3.226944	50.677222;
                 -3.232500	50.676111;
                 -3.237500	50.675278;
                 -3.243056	50.674167;
                 -3.247222	50.672778;
                 -3.251944	50.671111;
                 -3.256667	50.669722;
                 -3.260556	50.667778;
                 -3.264722	50.665556;
                 -3.267778	50.663056;
                 -3.271111	50.660556;
                 -3.274722	50.658056;
                 -3.277778	50.655556;
                 -3.280833	50.652500;
                 -3.283333	50.649444;
                 -3.286944	50.646389;
                 -3.289444	50.643611;
                 -3.292222	50.640556;
                 -3.294167	50.636944;
                 -3.296944	50.633889;
                 -3.300278	50.631389;
                 -3.305000	50.630000;
                 -3.308889	50.627778;
                 -3.313889	50.626944;
                 -3.319444	50.625833;
                 -3.323889	50.624167;
                 -3.327778	50.622222;
                 -3.331389	50.619722;
                 -3.334444	50.617222;
                 -3.337778	50.614722;
                 -3.341389	50.612222;
                 -3.344444	50.609722;
                 -3.349167	50.608056;
                 -3.354444	50.607222;
                 -3.359722	50.606111;
                 -3.365000	50.605000;
                 -3.370833	50.604722;
                 -3.377222	50.604444;
                 -3.383889	50.604444;
                 -3.390278	50.604444;
                 -3.396667	50.605556;
                 -3.402222	50.606667;
                 -3.407778	50.608056;
                 -3.412500	50.609722;
                 -3.417222	50.611389;
                 -3.419722	50.614444;
                 -3.419167	50.618056;
                 -3.416944	50.621667;
                 -3.415833	50.625556;
                 -3.415833	50.629722;
                 -3.418333	50.632778;
                 -3.420278	50.635556;
                 -3.424444	50.637778;
                 -3.427778	50.640278;
                 -3.430833	50.642778;
                 -3.433333	50.645556;
                 -3.435556	50.648611;
                 -3.437222	50.651944;
                 -3.439722	50.654722;
                 -3.441111	50.658056;
                 -3.442778	50.661389;
                 -3.445278	50.664444;
                 -3.447778	50.667222;
                 -3.450833	50.669722;
                 -3.454167	50.672222;
                 -3.460556	50.673333;
                 -3.465556	50.672222;
                 -3.468889	50.669722;
                 -3.469722	50.666111;
                 -3.467222	50.663056;
                 -3.464167	50.660556;
                 -3.460833	50.658056;
                 -3.457778	50.655556;
                 -3.455278	50.652500;
                 -3.452222	50.650000;
                 -3.449722	50.647222;
                 -3.447222	50.644167;
                 -3.445556	50.640833;
                 -3.444722	50.637222;
                 -3.444722	50.633056;
                 -3.444722	50.628889;
                 -3.445556	50.624444;
                 -3.446944	50.620278;
                 -3.448611	50.616389;
                 -3.449167	50.611667;
                 -3.449167	50.607778;
                 -3.448333	50.603889;
                 -3.441944	50.603056;
                 -3.437222	50.601389;
                 -3.435556	50.598056;
                 -3.436389	50.594444;
                 -3.438889	50.591389;
                 -3.442222	50.588889;
                 -3.445556	50.586389;
                 -3.448333	50.583333;
                 -3.449722	50.579167;
                 -3.450556	50.574722;
                 -3.451944	50.570556;
                 -3.453889	50.567222;
                 -3.456667	50.564167;
                 -3.460000	50.561667;
                 -3.463056	50.558889;
                 -3.466667	50.556389;
                 -3.469722	50.553889;
                 -3.472500	50.550833;
                 -3.475833	50.548333;
                 -3.479167	50.545833;
                 -3.482500	50.543333;
                 -3.485833	50.540833;
                 -3.488611	50.537778;
                 -3.491667	50.535278;
                 -3.494444	50.532222;
                 -3.496944	50.529167;
                 -3.499722	50.526111;
                 -3.501667	50.522500;
                 -3.503889	50.519167;
                 -3.505833	50.515556;
                 -3.507222	50.511389;
                 -3.508889	50.507500;
                 -3.509444	50.503056;
                 -3.510278	50.498333;
                 -3.509444	50.494722;
                 -3.508611	50.490833;
                 -3.506944	50.487500;
                 -3.505278	50.484167;
                 -3.503056	50.481389;
                 -3.500556	50.478333;
                 -3.498333	50.475556;
                 -3.495833	50.472500;
                 -3.492778	50.470000;
                 -3.489444	50.467500;
                 -3.486944	50.464722;
                 -3.483889	50.462222;
                 -3.482222	50.458889;
                 -3.485556	50.456389;
                 -3.490833	50.455278;
                 -3.497222	50.455278;
                 -3.504444	50.455833;
                 -3.511111	50.455833;
                 -3.517222	50.455833;
                 -3.522778	50.454722;
                 -3.527778	50.453611;
                 -3.533056	50.452778;
                 -3.537500	50.451111;
                 -3.541389	50.449167;
                 -3.545000	50.446667;
                 -3.546944	50.443056;
                 -3.549167	50.439444;
                 -3.548889	50.435278;
                 -3.548056	50.431667;
                 -3.548056	50.427500;
                 -3.547500	50.423889;
                 -3.548056	50.419167;
                 -3.548056	50.415000;
                 -3.546944	50.412222;
                 -3.544722	50.409444;
                 -3.541389	50.406944;
                 -3.535833	50.405556;
                 -3.528889	50.405000;
                 -3.522500	50.404167;
                 -3.518611	50.402222;
                 -3.515000	50.399444;
                 -3.511944	50.396944;
                 -3.507222	50.395278;
                 -3.501111	50.394444;
                 -3.495000	50.395000;
                 -3.489167	50.395556;
                 -3.483611	50.395000;
                 -3.480556	50.392500;
                 -3.480833	50.388889;
                 -3.485000	50.386944;
                 -3.489444	50.385556;
                 -3.493333	50.383333;
                 -3.497222	50.381389;
                 -3.501389	50.379444;
                 -3.504722	50.376944;
                 -3.507222	50.373889;
                 -3.508889	50.369722;
                 -3.509444	50.365278;
                 -3.511389	50.361667;
                 -3.513611	50.358056;
                 -3.515000	50.354167;
                 -3.513333	50.350833;
                 -3.512500	50.346944;
                 -3.512500	50.342778;
                 -3.517778	50.341944;
                 -3.523056	50.340833;
                 -3.526111	50.338333;
                 -3.530278	50.336389;
                 -3.536111	50.335833;
                 -3.541389	50.336944;
                 -3.546389	50.338889;
                 -3.550278	50.340833;
                 -3.554167	50.343056;
                 -3.557500	50.345556;
                 -3.559722	50.348333;
                 -3.563056	50.350833;
                 -3.568611	50.352222;
                 -3.571389	50.349167;
                 -3.572500	50.345278;
                 -3.572500	50.341111;
                 -3.570000	50.338056;
                 -3.567778	50.335278;
                 -3.565556	50.332222;
                 -3.562222	50.329722;
                 -3.559722	50.326944;
                 -3.559167	50.323056;
                 -3.563056	50.321111;
                 -3.568889	50.320556;
                 -3.573889	50.319444;
                 -3.579722	50.318889;
                 -3.585000	50.318056;
                 -3.590278	50.316944;
                 -3.595556	50.315833;
                 -3.600000	50.314444;
                 -3.604444	50.312778;
                 -3.609722	50.311944;
                 -3.613889	50.309722;
                 -3.617778	50.307778;
                 -3.620833	50.305278;
                 -3.624167	50.302778;
                 -3.627500	50.300278;
                 -3.630000	50.297222;
                 -3.632778	50.294167;
                 -3.635556	50.291111;
                 -3.637500	50.287500;
                 -3.640000	50.284444;
                 -3.642222	50.281111;
                 -3.644722	50.278056;
                 -3.646667	50.274444;
                 -3.648611	50.270833;
                 -3.650278	50.266944;
                 -3.651667	50.262778;
                 -3.653611	50.259167;
                 -3.654167	50.254722;
                 -3.655000	50.250278;
                 -3.655833	50.245556;
                 -3.655000	50.241944;
                 -3.654167	50.238056;
                 -3.652500	50.235000;
                 -3.650833	50.231667;
                 -3.648611	50.228611;
                 -3.646111	50.225833;
                 -3.646667	50.222222;
                 -3.651389	50.220556;
                 -3.658333	50.221111;
                 -3.664722	50.221111;
                 -3.671944	50.221389;
                 -3.678333	50.221389;
                 -3.683611	50.220556;
                 -3.688333	50.218889;
                 -3.693333	50.217778;
                 -3.698056	50.216389;
                 -3.702500	50.214722;
                 -3.706389	50.212778;
                 -3.710278	50.210833;
                 -3.714722	50.209167;
                 -3.719444	50.207778;
                 -3.724722	50.206667;
                 -3.730000	50.207778;
                 -3.734167	50.210000;
                 -3.738056	50.212222;
                 -3.742778	50.214167;
                 -3.748333	50.215556;
                 -3.753611	50.216944;
                 -3.758611	50.218611;
                 -3.762500	50.220556;
                 -3.767222	50.222222;
                 -3.771111	50.224444;
                 -3.776111	50.224167;
                 -3.778333	50.220833;
                 -3.779167	50.216111;
                 -3.783056	50.214167;
                 -3.787222	50.212500;
                 -3.793056	50.213889;
                 -3.799444	50.214722;
                 -3.803889	50.216389;
                 -3.809444	50.217778;
                 -3.814167	50.219444;
                 -3.819167	50.221111;
                 -3.823611	50.222778;
                 -3.829444	50.224167;
                 -3.833333	50.226111;
                 -3.836389	50.228611;
                 -3.840556	50.230833;
                 -3.843889	50.233333;
                 -3.845833	50.236111;
                 -3.848333	50.239167;
                 -3.850000	50.242500;
                 -3.850833	50.246111;
                 -3.852500	50.249444;
                 -3.853333	50.253333;
                 -3.855000	50.256389;
                 -3.856667	50.259722;
                 -3.859167	50.262778;
                 -3.863056	50.264722;
                 -3.866944	50.266944;
                 -3.871944	50.268611;
                 -3.875833	50.270833;
                 -3.878056	50.273611;
                 -3.880556	50.276667;
                 -3.883889	50.279167;
                 -3.887778	50.281111;
                 -3.891667	50.283333;
                 -3.897222	50.284444;
                 -3.902778	50.285833;
                 -3.908333	50.286944;
                 -3.913889	50.288333;
                 -3.920278	50.289167;
                 -3.926389	50.290000;
                 -3.932222	50.291389;
                 -3.936667	50.293056;
                 -3.938611	50.296389;
                 -3.938611	50.300556;
                 -3.939167	50.304167;
                 -3.942500	50.306667;
                 -3.948333	50.306944;
                 -3.951389	50.304444;
                 -3.955278	50.302500;
                 -3.960000	50.300833;
                 -3.965833	50.300278;
                 -3.972222	50.300278;
                 -3.979444	50.300833;
                 -3.985556	50.301667;
                 -3.993056	50.302222;
                 -3.999167	50.301944;
                 -4.005000	50.301389;
                 -4.010278	50.300556;
                 -4.014167	50.298333;
                 -4.018056	50.296389;
                 -4.021944	50.294444;
                 -4.027222	50.293333;
                 -4.033333	50.292778;
                 -4.037778	50.294444;
                 -4.043333	50.295833;
                 -4.050556	50.296111;
                 -4.057500	50.296667;
                 -4.062222	50.298333;
                 -4.063333	50.301944;
                 -4.065000	50.305278;
                 -4.069444	50.306944;
                 -4.075833	50.307778;
                 -4.082222	50.308611;
                 -4.086389	50.310833;
                 -4.091111	50.312500;
                 -4.097500	50.313333;
                 -4.102500	50.312222;
                 -4.108889	50.313056;
                 -4.111944	50.315556;
                 -4.113889	50.318889;
                 -4.115556	50.322222;
                 -4.115556	50.326389;
                 -4.116389	50.330000;
                 -4.115833	50.334722;
                 -4.115833	50.338889;
                 -4.117500	50.342222;
                 -4.120000	50.344167;
                 -4.122222	50.346944;
                 -4.123889	50.350278;
                 -4.125833	50.353611;
                 -4.129722	50.355833;
                 -4.136111	50.355556;
                 -4.141944	50.356111;
                 -4.147500	50.357222;
                 -4.153056	50.358611;
                 -4.158333	50.357500;
                 -4.161389	50.355833;
                 -4.167222	50.355278;
                 -4.172500	50.356667;
                 -4.175833	50.359167;
                 -4.179167	50.361667;
                 -4.183889	50.363333;
                 -4.189444	50.364444;
                 -4.195833	50.365278;
                 -4.200833	50.364167;
                 -4.206389	50.363333;
                 -4.211944	50.362778;
                 -4.217778	50.362222;
                 -4.221111	50.359444;
                 -4.218889	50.356667;
                 -4.211389	50.356111;
                 -4.204444	50.355833;
                 -4.198889	50.354444;
                 -4.195556	50.351944;
                 -4.193333	50.349167;
                 -4.186667	50.349167;
                 -4.181667	50.350278;
                 -4.175000	50.350278;
                 -4.170278	50.348611;
                 -4.170278	50.345278;
                 -4.172500	50.342222;
                 -4.176389	50.340278;
                 -4.181111	50.338611;
                 -4.184444	50.336111;
                 -4.187778	50.333611;
                 -4.189722	50.330000;
                 -4.190556	50.325556;
                 -4.189444	50.321944;
                 -4.188611	50.318056;
                 -4.190556	50.314444;
                 -4.195833	50.313611;
                 -4.202222	50.313333;
                 -4.208056	50.312778;
                 -4.212778	50.311389;
                 -4.218889	50.312222;
                 -4.222222	50.314722;
                 -4.222222	50.318889;
                 -4.220833	50.322778;
                 -4.224722	50.325000;
                 -4.229722	50.326667;
                 -4.234444	50.328333;
                 -4.239167	50.330000;
                 -4.243056	50.331944;
                 -4.247222	50.334167;
                 -4.250556	50.336667;
                 -4.253611	50.339167;
                 -4.256944	50.341667;
                 -4.260000	50.344167;
                 -4.264167	50.346111;
                 -4.269722	50.347500;
                 -4.275278	50.348611;
                 -4.281389	50.349444;
                 -4.287222	50.350833;
                 -4.292778	50.351944;
                 -4.298333	50.353333;
                 -4.302500	50.353611;
                 -4.308889	50.354444;
                 -4.315833	50.354722;
                 -4.321667	50.354167;
                 -4.328333	50.354167;
                 -4.334722	50.354167;
                 -4.341389	50.354167;
                 -4.348333	50.354444;
                 -4.355000	50.354444;
                 -4.361389	50.354444;
                 -4.367222	50.353889;
                 -4.373611	50.353611;
                 -4.380833	50.354167;
                 -4.387222	50.354167;
                 -4.393889	50.353889;
                 -4.400833	50.354444;
                 -4.408056	50.355000;
                 -4.414444	50.354722;
                 -4.420278	50.354167;
                 -4.425556	50.353056;
                 -4.430556	50.352222;
                 -4.435278	50.350556;
                 -4.438889	50.348611;
                 -4.442778	50.346389;
                 -4.446389	50.343889;
                 -4.448611	50.340833;
                 -4.452222	50.338333;
                 -4.455278	50.335556;
                 -4.458611	50.333056;
                 -4.462500	50.331111;
                 -4.466389	50.328889;
                 -4.470833	50.327500;
                 -4.476944	50.326944;
                 -4.482778	50.326389;
                 -4.489722	50.326667;
                 -4.496111	50.326667;
                 -4.501111	50.325556;
                 -4.505833	50.323889;
                 -4.511667	50.323333;
                 -4.517500	50.322778;
                 -4.523889	50.322778;
                 -4.530556	50.322778;
                 -4.536944	50.322778;
                 -4.543889	50.323056;
                 -4.551389	50.323611;
                 -4.558333	50.323889;
                 -4.565556	50.324444;
                 -4.571667	50.325278;
                 -4.578056	50.326111;
                 -4.583611	50.327222;
                 -4.589722	50.326667;
                 -4.594722	50.325556;
                 -4.600556	50.325000;
                 -4.606944	50.325833;
                 -4.611389	50.324167;
                 -4.616389	50.323056;
                 -4.621111	50.321667;
                 -4.628056	50.321944;
                 -4.634444	50.322778;
                 -4.640000	50.324167;
                 -4.646389	50.323889;
                 -4.651111	50.322500;
                 -4.654167	50.320833;
                 -4.657500	50.318333;
                 -4.661389	50.316111;
                 -4.665278	50.314167;
                 -4.670278	50.313056;
                 -4.676667	50.313889;
                 -4.680000	50.316389;
                 -4.680833	50.320000;
                 -4.680833	50.324167;
                 -4.681111	50.328333;
                 -4.682222	50.331944;
                 -4.683056	50.335833;
                 -4.686389	50.338333;
                 -4.691944	50.339444;
                 -4.697500	50.339722;
                 -4.701944	50.338333;
                 -4.705833	50.336111;
                 -4.709722	50.334167;
                 -4.714444	50.332500;
                 -4.719444	50.331389;
                 -4.725278	50.330833;
                 -4.731667	50.330833;
                 -4.737500	50.330278;
                 -4.743333	50.329444;
                 -4.748611	50.328611;
                 -4.753889	50.327500;
                 -4.757500	50.325278;
                 -4.761111	50.322778;
                 -4.762222	50.318611;
                 -4.763333	50.314722;
                 -4.762500	50.310833;
                 -4.760000	50.308056;
                 -4.756944	50.305556;
                 -4.753611	50.303056;
                 -4.752500	50.300278;
                 -4.757778	50.299167;
                 -4.762222	50.297500;
                 -4.767222	50.296667;
                 -4.771111	50.293889;
                 -4.773056	50.290556;
                 -4.774722	50.286389;
                 -4.776389	50.282778;
                 -4.779167	50.279722;
                 -4.781667	50.276667;
                 -4.783611	50.273056;
                 -4.785000	50.269167;
                 -4.782500	50.266111;
                 -4.777500	50.264722;
                 -4.773611	50.262500;
                 -4.771944	50.259167;
                 -4.773056	50.255278;
                 -4.775833	50.252222;
                 -4.778333	50.249167;
                 -4.781389	50.246389;
                 -4.784167	50.243333;
                 -4.786111	50.240000;
                 -4.788611	50.236667;
                 -4.790556	50.233333;
                 -4.792500	50.229722;
                 -4.794444	50.226111;
                 -4.798333	50.224167;
                 -4.804722	50.223889;
                 -4.809444	50.225556;
                 -4.810833	50.225556;
                 -4.815000	50.227778;
                 -4.818056	50.230278;
                 -4.823056	50.232222;
                 -4.827500	50.233889;
                 -4.833889	50.234722;
                 -4.840556	50.234722;
                 -4.846389	50.234167;
                 -4.851667	50.233056;
                 -4.855278	50.230833;
                 -4.859167	50.228889;
                 -4.863056	50.226667;
                 -4.866944	50.224722;
                 -4.871389	50.223056;
                 -4.876111	50.221667;
                 -4.880000	50.219444;
                 -4.884167	50.217778;
                 -4.888056	50.215833;
                 -4.892222	50.213611;
                 -4.895278	50.211111;
                 -4.898056	50.208056;
                 -4.899722	50.204444;
                 -4.901111	50.200556;
                 -4.902222	50.196389;
                 -4.908056	50.195833;
                 -4.915278	50.196389;
                 -4.920556	50.197500;
                 -4.926389	50.198611;
                 -4.932500	50.199444;
                 -4.939722	50.200000;
                 -4.944722	50.198889;
                 -4.949444	50.197222;
                 -4.953333	50.195278;
                 -4.956667	50.192500;
                 -4.959722	50.190000;
                 -4.962778	50.187500;
                 -4.965278	50.184444;
                 -4.967222	50.180833;
                 -4.970000	50.177778;
                 -4.971944	50.174167;
                 -4.973056	50.170000;
                 -4.974444	50.166111;
                 -4.975556	50.161944;
                 -4.976944	50.158056;
                 -4.979444	50.154444;
                 -4.983333	50.152500;
                 -4.986944	50.150278;
                 -4.990833	50.148333;
                 -4.995556	50.146667;
                 -5.000000	50.145000;
                 -5.004444	50.143333;
                 -5.008333	50.141389;
                 -5.013333	50.140278;
                 -5.012778	50.141944;
                 -5.008056	50.143611;
                 -5.005000	50.146111;
                 -5.001111	50.148333;
                 -4.997778	50.150833;
                 -4.995833	50.154444;
                 -4.996111	50.157778;
                 -5.001667	50.158889;
                 -5.006389	50.157222;
                 -5.010000	50.155278;
                 -5.015278	50.154167;
                 -5.017222	50.156389;
                 -5.016667	50.160278;
                 -5.014167	50.163333;
                 -5.012222	50.166667;
                 -5.010278	50.170278;
                 -5.009167	50.174444;
                 -5.010000	50.178056;
                 -5.013056	50.180556;
                 -5.016389	50.183056;
                 -5.019444	50.185556;
                 -5.022222	50.188333;
                 -5.023056	50.192222;
                 -5.023056	50.196111;
                 -5.021111	50.199722;
                 -5.020000	50.203889;
                 -5.018056	50.207500;
                 -5.016389	50.211111;
                 -5.015000	50.215000;
                 -5.015000	50.219167;
                 -5.019167	50.221111;
                 -5.025000	50.220556;
                 -5.026944	50.216944;
                 -5.028889	50.213611;
                 -5.030000	50.209444;
                 -5.032778	50.206389;
                 -5.035278	50.203333;
                 -5.038333	50.200833;
                 -5.043056	50.199167;
                 -5.048056	50.198056;
                 -5.053889	50.197500;
                 -5.057500	50.195278;
                 -5.060278	50.192222;
                 -5.058611	50.188889;
                 -5.056111	50.186111;
                 -5.052778	50.183611;
                 -5.050278	50.180833;
                 -5.047778	50.177778;
                 -5.046944	50.174167;
                 -5.048056	50.170000;
                 -5.050833	50.166944;
                 -5.053056	50.165278;
                 -5.058056	50.164167;
                 -5.063611	50.163056;
                 -5.066667	50.160556;
                 -5.066389	50.157500;
                 -5.063611	50.154444;
                 -5.059722	50.152500;
                 -5.055000	50.150833;
                 -5.050278	50.149167;
                 -5.043889	50.148333;
                 -5.041389	50.145556;
                 -5.043333	50.142778;
                 -5.047500	50.141111;
                 -5.053333	50.140556;
                 -5.060000	50.140556;
                 -5.065556	50.140000;
                 -5.069444	50.137778;
                 -5.072222	50.134722;
                 -5.074722	50.131667;
                 -5.079167	50.130000;
                 -5.082222	50.127500;
                 -5.083611	50.123611;
                 -5.083611	50.119444;
                 -5.083333	50.115278;
                 -5.083889	50.110556;
                 -5.085556	50.107222;
                 -5.088889	50.104444;
                 -5.093889	50.103333;
                 -5.099722	50.102778;
                 -5.105833	50.102222;
                 -5.111667	50.101667;
                 -5.116667	50.100556;
                 -5.120278	50.098333;
                 -5.122778	50.096389;
                 -5.118056	50.094722;
                 -5.111944	50.093889;
                 -5.106111	50.092778;
                 -5.101389	50.091111;
                 -5.096667	50.089444;
                 -5.092500	50.087222;
                 -5.087778	50.085833;
                 -5.081667	50.085000;
                 -5.075278	50.085000;
                 -5.071111	50.083056;
                 -5.068611	50.080000;
                 -5.068611	50.075833;
                 -5.067500	50.072222;
                 -5.065000	50.069444;
                 -5.061667	50.066944;
                 -5.057778	50.064722;
                 -5.054444	50.062500;
                 -5.053611	50.058611;
                 -5.054167	50.054167;
                 -5.055556	50.050000;
                 -5.057500	50.046389;
                 -5.059444	50.043056;
                 -5.062500	50.040278;
                 -5.065556	50.037778;
                 -5.070000	50.036111;
                 -5.074722	50.034722;
                 -5.079167	50.033056;
                 -5.083056	50.030833;
                 -5.085556	50.027778;
                 -5.086111	50.023333;
                 -5.087500	50.019167;
                 -5.089167	50.015833;
                 -5.091111	50.012222;
                 -5.094167	50.009444;
                 -5.098056	50.007500;
                 -5.103889	50.006944;
                 -5.111111	50.007222;
                 -5.117222	50.008056;
                 -5.124167	50.008333;
                 -5.131389	50.008889;
                 -5.137222	50.008333;
                 -5.143056	50.007500;
                 -5.148611	50.006944;
                 -5.153889	50.005833;
                 -5.158889	50.004722;
                 -5.162222	50.002222;
                 -5.163611	49.998056;
                 -5.164722	49.994167;
                 -5.167222	49.991111;
                 -5.171111	49.988889;
                 -5.173611	49.985833;
                 -5.176667	49.983333;
                 -5.178611	49.979722;
                 -5.181111	49.976667;
                 -5.183056	49.973056;
                 -5.184444	49.969167;
                 -5.185556	49.965000;
                 -5.186111	49.960556;
                 -5.188611	49.957500;
                 -5.193056	49.955833;
                 -5.198611	49.956111;
                 -5.202778	49.958056;
                 -5.204722	49.961389;
                 -5.206389	49.964722;
                 -5.209444	49.967222;
                 -5.214167	49.968889;
                 -5.219722	49.970000;
                 -5.224722	49.971667;
                 -5.228611	49.973611;
                 -5.231667	49.976111;
                 -5.234444	49.979167;
                 -5.237500	49.981389;
                 -5.240000	49.984444;
                 -5.241667	49.987778;
                 -5.244167	49.990556;
                 -5.247500	49.993056;
                 -5.250000	49.995833;
                 -5.252500	49.998889;
                 -5.255556	50.001389;
                 -5.257222	50.004444;
                 -5.256944	50.008056;
                 -5.255000	50.011667;
                 -5.252500	50.014722;
                 -5.251111	50.018889;
                 -5.250833	50.023333;
                 -5.252500	50.026667;
                 -5.253333	50.030556;
                 -5.255833	50.033333;
                 -5.260000	50.035278;
                 -5.263056	50.037778;
                 -5.266389	50.040278;
                 -5.268889	50.043056;
                 -5.270556	50.046389;
                 -5.272222	50.049722;
                 -5.273056	50.053333;
                 -5.275000	50.056667;
                 -5.276667	50.060000;
                 -5.280000	50.062500;
                 -5.283333	50.065000;
                 -5.286389	50.067500;
                 -5.289722	50.070000;
                 -5.293611	50.071944;
                 -5.297222	50.073333;
                 -5.301389	50.075556;
                 -5.305278	50.077500;
                 -5.309167	50.079722;
                 -5.313333	50.081667;
                 -5.317778	50.083333;
                 -5.321389	50.085833;
                 -5.325278	50.087778;
                 -5.329444	50.089722;
                 -5.333333	50.091944;
                 -5.338889	50.093056;
                 -5.345278	50.093889;
                 -5.351111	50.093333;
                 -5.356667	50.092500;
                 -5.362500	50.092778;
                 -5.366389	50.095000;
                 -5.370556	50.096944;
                 -5.373889	50.099444;
                 -5.378056	50.101389;
                 -5.381944	50.103611;
                 -5.387500	50.104722;
                 -5.394444	50.105000;
                 -5.401667	50.105278;
                 -5.408056	50.105278;
                 -5.414444	50.105278;
                 -5.421667	50.105556;
                 -5.426389	50.107222;
                 -5.431111	50.108611;
                 -5.435000	50.110833;
                 -5.440000	50.112500;
                 -5.444167	50.114444;
                 -5.448056	50.116389;
                 -5.452222	50.118611;
                 -5.456111	50.120556;
                 -5.460833	50.122222;
                 -5.465556	50.123889;
                 -5.470556	50.125278;
                 -5.476111	50.126667;
                 -5.482500	50.127500;
                 -5.489444	50.127778;
                 -5.495278	50.127222;
                 -5.501111	50.126389;
                 -5.506111	50.125278;
                 -5.511111	50.124167;
                 -5.515833	50.122500;
                 -5.521111	50.121389;
                 -5.526111	50.120000;
                 -5.530556	50.118333;
                 -5.534444	50.116389;
                 -5.537500	50.113611;
                 -5.539444	50.110000;
                 -5.541389	50.106389;
                 -5.541667	50.101944;
                 -5.541111	50.098333;
                 -5.540000	50.094444;
                 -5.539167	50.090833;
                 -5.538056	50.087222;
                 -5.537778	50.083056;
                 -5.538333	50.078333;
                 -5.539444	50.074444;
                 -5.541389	50.070833;
                 -5.543889	50.067778;
                 -5.546389	50.065556;
                 -5.549444	50.063056;
                 -5.551944	50.060000;
                 -5.555556	50.057778;
                 -5.558889	50.055278;
                 -5.562778	50.053056;
                 -5.567222	50.051389;
                 -5.572222	50.050278;
                 -5.577500	50.049167;
                 -5.582500	50.048056;
                 -5.588333	50.047500;
                 -5.594722	50.047500;
                 -5.601389	50.047222;
                 -5.607778	50.047222;
                 -5.613611	50.046389;
                 -5.619167	50.045833;
                 -5.624167	50.044722;
                 -5.629444	50.043611;
                 -5.634444	50.042500;
                 -5.639167	50.040833;
                 -5.643611	50.039167;
                 -5.648611	50.038056;
                 -5.653889	50.036944;
                 -5.659444	50.036389;
                 -5.666667	50.036667;
                 -5.672222	50.038056;
                 -5.677778	50.039167;
                 -5.682500	50.040833;
                 -5.687222	50.042222;
                 -5.690556	50.044722;
                 -5.693056	50.047500;
                 -5.697222	50.049722;
                 -5.701944	50.051389;
                 -5.708333	50.051944;
                 -5.712500	50.054167;
                 -5.714722	50.056944;
                 -5.717222	50.059722;
                 -5.718333	50.063611;
                 -5.718611	50.067500;
                 -5.716667	50.071111;
                 -5.714722	50.074722;
                 -5.711667	50.077500;
                 -5.707222	50.078889;
                 -5.702222	50.080000;
                 -5.698889	50.082778;
                 -5.696667	50.085833;
                 -5.696111	50.090278;
                 -5.696111	50.094444;
                 -5.697222	50.098056;
                 -5.698889	50.101389;
                 -5.700000	50.105278;
                 -5.701667	50.108333;
                 -5.704167	50.111389;
                 -5.706111	50.114722;
                 -5.708611	50.117500;
                 -5.709444	50.121111;
                 -5.710556	50.125000;
                 -5.710000	50.129444;
                 -5.706667	50.131944;
                 -5.704444	50.135000;
                 -5.701111	50.137778;
                 -5.699167	50.141389;
                 -5.698611	50.145833;
                 -5.697500	50.150000;
                 -5.695556	50.153611;
                 -5.691944	50.155556;
                 -5.688056	50.157778;
                 -5.682778	50.158889;
                 -5.677778	50.160000;
                 -5.672500	50.161111;
                 -5.666667	50.161667;
                 -5.661667	50.162778;
                 -5.656389	50.163889;
                 -5.651389	50.165000;
                 -5.646944	50.166667;
                 -5.642222	50.168333;
                 -5.638056	50.170000;
                 -5.632778	50.171111;
                 -5.627778	50.172222;
                 -5.623333	50.173611;
                 -5.618056	50.175000;
                 -5.613611	50.176389;
                 -5.609167	50.178056;
                 -5.605278	50.180278;
                 -5.601667	50.182222;
                 -5.597778	50.184444;
                 -5.593333	50.186111;
                 -5.587500	50.186667;
                 -5.582222	50.187778;
                 -5.577500	50.189444;
                 -5.573889	50.191389;
                 -5.570000	50.193611;
                 -5.566111	50.195556;
                 -5.562500	50.197778;
                 -5.558611	50.199722;
                 -5.554167	50.201389;
                 -5.550278	50.203611;
                 -5.546389	50.205556;
                 -5.541389	50.206667;
                 -5.535278	50.207500;
                 -5.529722	50.208056;
                 -5.524722	50.209167;
                 -5.520000	50.210833;
                 -5.516111	50.212778;
                 -5.511667	50.214444;
                 -5.506667	50.215556;
                 -5.501389	50.216667;
                 -5.495556	50.217222;
                 -5.489167	50.216667;
                 -5.483611	50.215278;
                 -5.478889	50.213611;
                 -5.476667	50.210833;
                 -5.473889	50.208056;
                 -5.472222	50.204722;
                 -5.471111	50.201111;
                 -5.468889	50.198056;
                 -5.463056	50.196944;
                 -5.456111	50.196667;
                 -5.449722	50.195833;
                 -5.444167	50.194444;
                 -5.439167	50.193056;
                 -5.432778	50.193056;
                 -5.427778	50.194167;
                 -5.423889	50.196389;
                 -5.420000	50.198333;
                 -5.416667	50.200833;
                 -5.413611	50.203611;
                 -5.410278	50.206111;
                 -5.407222	50.208611;
                 -5.404167	50.211389;
                 -5.401667	50.214444;
                 -5.399167	50.217500;
                 -5.397222	50.221111;
                 -5.396667	50.225556;
                 -5.394722	50.229167;
                 -5.393611	50.233333;
                 -5.390278	50.235833;
                 -5.386389	50.237778;
                 -5.380556	50.238611;
                 -5.375000	50.237222;
                 -5.369444	50.236944;
                 -5.364167	50.238056;
                 -5.359167	50.239167;
                 -5.351944	50.238889;
                 -5.345833	50.238056;
                 -5.339167	50.238056;
                 -5.335833	50.240833;
                 -5.332778	50.243333;
                 -5.329722	50.245833;
                 -5.326389	50.248611;
                 -5.323333	50.251111;
                 -5.318889	50.252778;
                 -5.315000	50.254722;
                 -5.310278	50.256389;
                 -5.306667	50.258611;
                 -5.302778	50.260556;
                 -5.298889	50.262778;
                 -5.294444	50.264167;
                 -5.290556	50.266389;
                 -5.286111	50.268056;
                 -5.281389	50.269444;
                 -5.276389	50.270556;
                 -5.272500	50.272778;
                 -5.268056	50.274444;
                 -5.264722	50.276944;
                 -5.261111	50.278889;
                 -5.257778	50.281667;
                 -5.254722	50.284167;
                 -5.250833	50.286389;
                 -5.247500	50.288889;
                 -5.243611	50.290833;
                 -5.240556	50.293611;
                 -5.237222	50.296111;
                 -5.234167	50.298611;
                 -5.230833	50.301389;
                 -5.228333	50.304444;
                 -5.226944	50.308333;
                 -5.225000	50.311944;
                 -5.223889	50.316111;
                 -5.220833	50.318611;
                 -5.216111	50.320278;
                 -5.210833	50.321389;
                 -5.204722	50.321389;
                 -5.198611	50.321944;
                 -5.194722	50.324167;
                 -5.191111	50.326111;
                 -5.187778	50.328889;
                 -5.184444	50.331389;
                 -5.181389	50.333889;
                 -5.178333	50.336389;
                 -5.173889	50.338056;
                 -5.168056	50.338611;
                 -5.162500	50.339722;
                 -5.158333	50.341389;
                 -5.153611	50.343056;
                 -5.150278	50.345556;
                 -5.148056	50.348611;
                 -5.146667	50.352778;
                 -5.146111	50.357222;
                 -5.145556	50.361667;
                 -5.146389	50.365556;
                 -5.145833	50.370000;
                 -5.146111	50.374167;
                 -5.147778	50.377500;
                 -5.148611	50.381111;
                 -5.150278	50.384444;
                 -5.150556	50.388611;
                 -5.149444	50.392778;
                 -5.146667	50.395833;
                 -5.146111	50.400278;
                 -5.145000	50.404444;
                 -5.140278	50.405833;
                 -5.137778	50.405278;
                 -5.130556	50.405000;
                 -5.124722	50.403611;
                 -5.119444	50.402500;
                 -5.115556	50.404722;
                 -5.115833	50.407778;
                 -5.113333	50.410833;
                 -5.107778	50.411944;
                 -5.103611	50.413611;
                 -5.099722	50.415556;
                 -5.095278	50.417222;
                 -5.089444	50.416111;
                 -5.084722	50.414444;
                 -5.078333	50.413611;
                 -5.073889	50.414444;
                 -5.070556	50.416944;
                 -5.066111	50.418611;
                 -5.061111	50.419444;
                 -5.055833	50.420556;
                 -5.051944	50.422778;
                 -5.048889	50.425278;
                 -5.046111	50.428333;
                 -5.044167	50.431944;
                 -5.043056	50.436111;
                 -5.041111	50.439444;
                 -5.039722	50.443611;
                 -5.038611	50.447778;
                 -5.036667	50.451111;
                 -5.035278	50.455278;
                 -5.032778	50.458333;
                 -5.030278	50.461389;
                 -5.027500	50.464444;
                 -5.024444	50.466944;
                 -5.023889	50.470833;
                 -5.025556	50.473889;
                 -5.027222	50.477222;
                 -5.029167	50.480556;
                 -5.030833	50.483889;
                 -5.033333	50.486667;
                 -5.035278	50.490000;
                 -5.037500	50.493056;
                 -5.039444	50.496389;
                 -5.038611	50.500833;
                 -5.036111	50.503889;
                 -5.033056	50.506389;
                 -5.029444	50.509167;
                 -5.027222	50.512222;
                 -5.025278	50.515556;
                 -5.023056	50.519167;
                 -5.020556	50.522222;
                 -5.018056	50.525278;
                 -5.018333	50.529444;
                 -5.020833	50.532500;
                 -5.021667	50.536111;
                 -5.021111	50.540556;
                 -5.017778	50.543333;
                 -5.013889	50.545278;
                 -5.008889	50.546389;
                 -5.003056	50.546111;
                 -4.998056	50.544444;
                 -4.994167	50.542500;
                 -4.990833	50.540000;
                 -4.986111	50.538333;
                 -4.981389	50.540000;
                 -4.978333	50.541667;
                 -4.975000	50.544167;
                 -4.972778	50.547222;
                 -4.968889	50.549167;
                 -4.964722	50.551389;
                 -4.961667	50.553889;
                 -4.958333	50.556389;
                 -4.955833	50.559444;
                 -4.953056	50.562500;
                 -4.949167	50.564722;
                 -4.944444	50.565000;
                 -4.941667	50.561944;
                 -4.941667	50.557778;
                 -4.940556	50.554167;
                 -4.940556	50.550000;
                 -4.935556	50.548333;
                 -4.931111	50.549167;
                 -4.928889	50.549722;
                 -4.926944	50.553333;
                 -4.925000	50.556944;
                 -4.923056	50.560556;
                 -4.920556	50.563611;
                 -4.917222	50.566111;
                 -4.914722	50.569167;
                 -4.912778	50.572778;
                 -4.914444	50.576111;
                 -4.918611	50.578056;
                 -4.922500	50.580278;
                 -4.926111	50.582778;
                 -4.926667	50.586389;
                 -4.922500	50.588056;
                 -4.916667	50.586667;
                 -4.911944	50.585000;
                 -4.908611	50.582778;
                 -4.904444	50.580556;
                 -4.899722	50.578889;
                 -4.892500	50.578611;
                 -4.885556	50.578056;
                 -4.878889	50.578333;
                 -4.873889	50.579444;
                 -4.870000	50.581389;
                 -4.865833	50.583611;
                 -4.861944	50.585556;
                 -4.858056	50.587778;
                 -4.852222	50.588333;
                 -4.846389	50.588889;
                 -4.839722	50.588889;
                 -4.833333	50.588889;
                 -4.826944	50.589167;
                 -4.820278	50.589167;
                 -4.813333	50.588889;
                 -4.807500	50.589444;
                 -4.801389	50.590000;
                 -4.796111	50.591111;
                 -4.791111	50.592222;
                 -4.785833	50.593333;
                 -4.781389	50.594722;
                 -4.778056	50.597500;
                 -4.775000	50.600000;
                 -4.771389	50.602500;
                 -4.769444	50.606111;
                 -4.767222	50.609167;
                 -4.764444	50.612222;
                 -4.761667	50.615278;
                 -4.760556	50.619444;
                 -4.759722	50.623889;
                 -4.760000	50.628056;
                 -4.758889	50.631944;
                 -4.756944	50.635556;
                 -4.754167	50.638611;
                 -4.751389	50.641667;
                 -4.752500	50.645556;
                 -4.755000	50.648333;
                 -4.756667	50.651667;
                 -4.758333	50.655000;
                 -4.758611	50.659167;
                 -4.757222	50.663056;
                 -4.754722	50.666111;
                 -4.749444	50.667222;
                 -4.743611	50.667778;
                 -4.740278	50.670556;
                 -4.736944	50.673056;
                 -4.731389	50.671667;
                 -4.726389	50.672778;
                 -4.722778	50.675278;
                 -4.719722	50.678056;
                 -4.716944	50.681111;
                 -4.713056	50.683056;
                 -4.706667	50.683333;
                 -4.701389	50.684167;
                 -4.700556	50.688889;
                 -4.698333	50.691944;
                 -4.692500	50.692500;
                 -4.687222	50.693611;
                 -4.681944	50.694722;
                 -4.678611	50.697222;
                 -4.678333	50.700833;
                 -4.674722	50.703333;
                 -4.669722	50.704444;
                 -4.665000	50.706111;
                 -4.661111	50.708056;
                 -4.658056	50.710556;
                 -4.654444	50.713333;
                 -4.651389	50.715833;
                 -4.648611	50.718889;
                 -4.646667	50.722500;
                 -4.646111	50.726944;
                 -4.645556	50.730556;
                 -4.644167	50.734722;
                 -4.640278	50.736667;
                 -4.635278	50.737778;
                 -4.630000	50.738889;
                 -4.627222	50.741944;
                 -4.625556	50.745556;
                 -4.620833	50.746944;
                 -4.615556	50.748056;
                 -4.613056	50.751111;
                 -4.611111	50.754722;
                 -4.608333	50.756944;
                 -4.604444	50.758889;
                 -4.600000	50.760556;
                 -4.595556	50.762222;
                 -4.592222	50.764722;
                 -4.588056	50.766667;
                 -4.585000	50.769167;
                 -4.579722	50.770278;
                 -4.573333	50.770556;
                 -4.568611	50.771944;
                 -4.564722	50.774167;
                 -4.561389	50.776667;
                 -4.558056	50.779167;
                 -4.556111	50.782778;
                 -4.553333	50.785833;
                 -4.552222	50.790000;
                 -4.551111	50.793889;
                 -4.550278	50.798611;
                 -4.549444	50.803056;
                 -4.549167	50.807500;
                 -4.548333	50.812222;
                 -4.547500	50.816667;
                 -4.547778	50.820833;
                 -4.547222	50.825278;
                 -4.547222	50.829444;
                 -4.548056	50.833333;
                 -4.548333	50.837222;
                 -4.548333	50.841389;
                 -4.547500	50.846111;
                 -4.547222	50.850556;
                 -4.546389	50.855000;
                 -4.546389	50.859167;
                 -4.548056	50.862500;
                 -4.552222	50.864722;
                 -4.554722	50.867500;
                 -4.555556	50.871389;
                 -4.557500	50.874444;
                 -4.558333	50.878333;
                 -4.558333	50.882500;
                 -4.558611	50.886667;
                 -4.557222	50.890556;
                 -4.556667	50.895278;
                 -4.556667	50.899444;
                 -4.557500	50.903056;
                 -4.556944	50.907500;
                 -4.556111	50.912222;
                 -4.555000	50.916111;
                 -4.552222	50.919167;
                 -4.549444	50.922222;
                 -4.546389	50.925000;
                 -4.543333	50.927500;
                 -4.541389	50.931111;
                 -4.539722	50.935000;
                 -4.540000	50.939167;
                 -4.539444	50.943611;
                 -4.536667	50.946667;
                 -4.533889	50.949722;
                 -4.530833	50.952500;
                 -4.529167	50.956389;
                 -4.528056	50.960556;
                 -4.526944	50.964444;
                 -4.526111	50.969167;
                 -4.526111	50.973333;
                 -4.526389	50.977500;
                 -4.526389	50.981389;
                 -4.527222	50.985278;
                 -4.527500	50.989444;
                 -4.527500	50.993611;
                 -4.527500	50.997500;
                 -4.527222	51.002222;
                 -4.527222	51.006389;
                 -4.527222	51.010556;
                 -4.525556	51.012222;
                 -4.525556	51.012778;
                 -4.523056	51.015833;
                 -4.517222	51.016389;
                 -4.510556	51.015556;
                 -4.505000	51.014167;
                 -4.499167	51.013056;
                 -4.492778	51.013056;
                 -4.487222	51.014167;
                 -4.481389	51.014722;
                 -4.475556	51.015278;
                 -4.468333	51.014722;
                 -4.463333	51.013056;
                 -4.459444	51.011111;
                 -4.455278	51.008889;
                 -4.448889	51.008056;
                 -4.442500	51.008333;
                 -4.435833	51.007500;
                 -4.430278	51.006111;
                 -4.425278	51.004444;
                 -4.420278	51.002778;
                 -4.417222	51.000278;
                 -4.413056	50.998333;
                 -4.409167	50.996111;
                 -4.405000	50.994167;
                 -4.400278	50.992500;
                 -4.395278	50.990833;
                 -4.389722	50.989444;
                 -4.384722	50.987778;
                 -4.379167	50.986667;
                 -4.375000	50.985556;
                 -4.368611	50.984722;
                 -4.362778	50.983333;
                 -4.356389	50.982500;
                 -4.349167	50.982222;
                 -4.341944	50.981667;
                 -4.336111	50.982222;
                 -4.330833	50.983333;
                 -4.325556	50.984444;
                 -4.321111	50.985833;
                 -4.317222	50.988056;
                 -4.313056	50.990000;
                 -4.309167	50.991944;
                 -4.305278	50.994167;
                 -4.301667	50.996667;
                 -4.297778	50.998611;
                 -4.294722	51.001389;
                 -4.290556	51.003333;
                 -4.286667	51.005278;
                 -4.282778	51.007500;
                 -4.278889	51.009444;
                 -4.275000	51.011667;
                 -4.271389	51.014167;
                 -4.268056	51.016667;
                 -4.265000	51.019167;
                 -4.261389	51.021667;
                 -4.258333	51.024444;
                 -4.254444	51.026389;
                 -4.250278	51.028333;
                 -4.246389	51.030556;
                 -4.241111	51.031667;
                 -4.236667	51.033056;
                 -4.232500	51.035278;
                 -4.229167	51.037778;
                 -4.227778	51.041667;
                 -4.225833	51.045278;
                 -4.223056	51.048333;
                 -4.220556	51.051389;
                 -4.217778	51.054444;
                 -4.215833	51.058056;
                 -4.214722	51.061944;
                 -4.210556	51.063333;
                 -4.205556	51.063333;
                 -4.204167	51.067500;
                 -4.205000	51.071111;
                 -4.206667	51.074444;
                 -4.210000	51.076944;
                 -4.212500	51.080000;
                 -4.215000	51.082778;
                 -4.216667	51.086111;
                 -4.216667	51.090278;
                 -4.216111	51.094722;
                 -4.214722	51.098889;
                 -4.214167	51.103333;
                 -4.214167	51.107500;
                 -4.216667	51.110278;
                 -4.220000	51.112778;
                 -4.225556	51.114167;
                 -4.231111	51.115556;
                 -4.236111	51.117222;
                 -4.236389	51.120278;
                 -4.234167	51.123889;
                 -4.234444	51.128056;
                 -4.239167	51.129722;
                 -4.244167	51.131389;
                 -4.245833	51.134722;
                 -4.245000	51.138333;
                 -4.241111	51.140278;
                 -4.235278	51.140833;
                 -4.228889	51.141111;
                 -4.222222	51.141111;
                 -4.216667	51.142222;
                 -4.212222	51.143611;
                 -4.210278	51.147222;
                 -4.208333	51.150833;
                 -4.206667	51.154722;
                 -4.205278	51.158889;
                 -4.204722	51.163333;
                 -4.203333	51.167500;
                 -4.204444	51.171111;
                 -4.207500	51.173611;
                 -4.213056	51.175000;
                 -4.218056	51.176667;
                 -4.217500	51.180278;
                 -4.212778	51.181667;
                 -4.207500	51.182778;
                 -4.203611	51.185000;
                 -4.200278	51.187500;
                 -4.196944	51.190000;
                 -4.192500	51.191667;
                 -4.186389	51.192222;
                 -4.180556	51.191667;
                 -4.174444	51.192222;
                 -4.169167	51.193333;
                 -4.164444	51.195000;
                 -4.160556	51.196944;
                 -4.156111	51.198611;
                 -4.150833	51.199722;
                 -4.144167	51.199722;
                 -4.138889	51.200556;
                 -4.134167	51.202222;
                 -4.128889	51.203333;
                 -4.123056	51.203889;
                 -4.116389	51.203889;
                 -4.110000	51.203889;
                 -4.104444	51.205000;
                 -4.097778	51.205000;
                 -4.091389	51.205000;
                 -4.085278	51.205556;
                 -4.079444	51.206111;
                 -4.071944	51.205833;
                 -4.065556	51.205000;
                 -4.059167	51.204167;
                 -4.053333	51.202778;
                 -4.048611	51.201111;
                 -4.043056	51.199722;
                 -4.036667	51.198889;
                 -4.031389	51.200000;
                 -4.026667	51.201667;
                 -4.021944	51.203056;
                 -4.018056	51.205278;
                 -4.013333	51.206667;
                 -4.009167	51.208611;
                 -4.003889	51.210278;
                 -3.998889	51.211389;
                 -3.992778	51.211944;
                 -3.986111	51.211944;
                 -3.979444	51.211944;
                 -3.972222	51.211389;
                 -3.965833	51.210556;
                 -3.958611	51.210278;
                 -3.952500	51.210556;
                 -3.948056	51.212222;
                 -3.943333	51.213889;
                 -3.938611	51.215278;
                 -3.934167	51.216944;
                 -3.930000	51.218889;
                 -3.924722	51.220000;
                 -3.918611	51.220556;
                 -3.911667	51.220000;
                 -3.905833	51.218889;
                 -3.900278	51.217500;
                 -3.893889	51.216667;
                 -3.887222	51.216667;
                 -3.881944	51.217778;
                 -3.877222	51.219167;
                 -3.873333	51.220278;
                 -3.868611	51.221944;
                 -3.862778	51.222500;
                 -3.857500	51.223611;
                 -3.851944	51.224444;
                 -3.846111	51.225000;
                 -3.839722	51.225000;
                 -3.832222	51.224722;
                 -3.827500	51.223056;
                 -3.822500	51.221389;
                 -3.817500	51.219444;
                 -3.811389	51.218611;
                 -3.805833	51.219722;
                 -3.801389	51.221389;
                 -3.797500	51.223333;
                 -3.793333	51.225278;
                 -3.789444	51.227500;
                 -3.783889	51.228333;
                 -3.777222	51.228333;
                 -3.771111	51.227500;
                 -3.764444	51.226667;
                 -3.758056	51.225833;
                 -3.751389	51.225833;
                 -3.745278	51.226389;
                 -3.738889	51.226389;
                 -3.733889	51.225556;
                 -3.728333	51.224444;
                 -3.722500	51.223056;
                 -3.715278	51.222500;
                 -3.708056	51.222222;
                 -3.703056	51.220556;
                 -3.699167	51.218333;
                 -3.694444	51.216667;
                 -3.688333	51.217222;
                 -3.681667	51.217222;
                 -3.675833	51.217778;
                 -3.669722	51.218333;
                 -3.663889	51.218889;
                 -3.656389	51.218333;
                 -3.651944	51.216667;
                 -3.646111	51.215278;
                 -3.640278	51.213889;
                 -3.633056	51.213611;
                 -3.626667	51.212778;
                 -3.620000	51.212778;
                 -3.615278	51.211111;
                 -3.610278	51.209167;
                 -3.603333	51.208889;
                 -3.597778	51.209722;
                 -3.591667	51.210278;
                 -3.585278	51.210278;
                 -3.580000	51.211389;
                 -3.576667	51.213889;
                 -3.574444	51.217500;
                 -3.571667	51.220556;
                 -3.568333	51.223056;
                 -3.562500	51.223611;
                 -3.556111	51.222778;
                 -3.550556	51.221389;
                 -3.543889	51.220556;
                 -3.537500	51.219722;
                 -3.531111	51.218611;
                 -3.525278	51.217500;
                 -3.519167	51.216389;
                 -3.511667	51.216111;
                 -3.506111	51.214722;
                 -3.501111	51.213056;
                 -3.495556	51.211667;
                 -3.488333	51.211389;
                 -3.480833	51.210833;
                 -3.475278	51.209444;
                 -3.469722	51.208056;
                 -3.467778	51.206111;
                 -3.465556	51.203333;
                 -3.461389	51.201111;
                 -3.455833	51.200000;
                 -3.450000	51.200278;
                 -3.443056	51.200278;
                 -3.436667	51.199167;
                 -3.434444	51.197500;
                 -3.431667	51.194444;
                 -3.428333	51.191944;
                 -3.424444	51.190000;
                 -3.420278	51.187778;
                 -3.416389	51.185833;
                 -3.413056	51.183333;
                 -3.410000	51.180833;
                 -3.406667	51.178056;
                 -3.401944	51.176389;
                 -3.395556	51.175556;
                 -3.388889	51.175556;
                 -3.382222	51.175556;
                 -3.375000	51.175000;
                 -3.368333	51.175000;
                 -3.361111	51.174722;
                 -3.354444	51.174722;
                 -3.347222	51.174444;
                 -3.340000	51.174167;
                 -3.333611	51.174167;
                 -3.326111	51.173611;
                 -3.318889	51.173056;
                 -3.311667	51.172500;
                 -3.304444	51.172222;
                 -3.297222	51.171667;
                 -3.289722	51.171111;
                 -3.282778	51.170556;
                 -3.276389	51.169722;
                 -3.271111	51.170833;
                 -3.265556	51.171667;
                 -3.261111	51.173333;
                 -3.255556	51.174167;
                 -3.250278	51.175278;
                 -3.245556	51.176667;
                 -3.240833	51.178333;
                 -3.236389	51.179722;
                 -3.232500	51.181944;
                 -3.228333	51.183889;
                 -3.223611	51.185278;
                 -3.218889	51.186944;
                 -3.214444	51.188333;
                 -3.209444	51.190000;
                 -3.204722	51.191389;
                 -3.199722	51.192500;
                 -3.193056	51.192500;
                 -3.186389	51.192500;
                 -3.180000	51.191389;
                 -3.172778	51.191111;
                 -3.166667	51.191389;
                 -3.162222	51.193056;
                 -3.158056	51.195000;
                 -3.154167	51.196944;
                 -3.149444	51.198611;
                 -3.143333	51.199167;
                 -3.138056	51.200000;
                 -3.131944	51.200556;
                 -3.125556	51.200556;
                 -3.119167	51.199722;
                 -3.113611	51.198333;
                 -3.106944	51.197500;
                 -3.099722	51.196944;
                 -3.093889	51.197500;
                 -3.088056	51.196111;
                 -3.082500	51.194722;
                 -3.076111	51.193889;
                 -3.069444	51.193889;
                 -3.063611	51.194444;
                 -3.058056	51.195278;
                 -3.054167	51.197222;
                 -3.050000	51.199444;
                 -3.046111	51.201389;
                 -3.041389	51.202778;
                 -3.037222	51.205000;
                 -3.032222	51.205833;
                 -3.026111	51.206389;
                 -3.018889	51.206389;
                 -3.013056	51.206667;
                 -3.008056	51.208333;
                 -3.005000	51.209444;
                 -3.001667	51.211944;
                 -2.999444	51.215556;
                 -2.998889	51.220000;
                 -2.996944	51.224167;
                 -2.996389	51.228611;
                 -2.995000	51.232778;
                 -2.995556	51.236389;
                 -2.997222	51.239722;
                 -2.999722	51.242778;
                 -3.003056	51.245278;
                 -3.006111	51.247778;
                 -3.009167	51.250278;
                 -3.011667	51.253056;
                 -3.013333	51.256389;
                 -3.014167	51.260278;
                 -3.014167	51.264444;
                 -3.013056	51.268889;
                 -3.013056	51.273056;
                 -3.012500	51.277500;
                 -3.011111	51.281667;
                 -3.010278	51.286111;
                 -3.009167	51.290833;
                 -3.008611	51.295278;
                 -3.006944	51.299167;
                 -3.006111	51.303889;
                 -3.006944	51.307500;
                 -3.008611	51.310833;
                 -3.011667	51.313333;
                 -3.015833	51.315556;
                 -3.019722	51.317778;
                 -3.022222	51.320556;
                 -3.018333	51.321667;
                 -3.012500	51.321389;
                 -3.009167	51.318889;
                 -3.004444	51.316944;
                 -2.998889	51.315833;
                 -2.992222	51.315833;
                 -2.988333	51.317778;
                 -2.985278	51.320833;
                 -2.984722	51.325278;
                 -2.983611	51.329722;
                 -2.983611	51.333889;
                 -2.985278	51.337222;
                 -2.986944	51.340556;
                 -2.989167	51.343611;
                 -2.989444	51.346667;
                 -2.986111	51.349444;
                 -2.980833	51.350278;
                 -2.975278	51.351389;
                 -2.971389	51.353333;
                 -2.967222	51.355278;
                 -2.964722	51.358333;
                 -2.963056	51.362500;
                 -2.961667	51.366389;
                 -2.960556	51.370833;
                 -2.960000	51.375556;
                 -2.963889	51.377778;
                 -2.969444	51.378889;
                 -2.971944	51.381944;
                 -2.970556	51.385833;
                 -2.965833	51.387500;
                 -2.960556	51.388333;
                 -2.956667	51.390556;
                 -2.951667	51.391944;
                 -2.945278	51.391111;
                 -2.939722	51.389722;
                 -2.932500	51.389167;
                 -2.926944	51.390278;
                 -2.922500	51.391667;
                 -2.917500	51.393056;
                 -2.913611	51.395278;
                 -2.909444	51.397222;
                 -2.906111	51.399722;
                 -2.903333	51.402778;
                 -2.900000	51.405278;
                 -2.896389	51.406944;
                 -2.893333	51.409444;
                 -2.890278	51.412500;
                 -2.887778	51.415278;
                 -2.884167	51.417778;
                 -2.880833	51.420278;
                 -2.878056	51.423333;
                 -2.874444	51.425833;
                 -2.871111	51.428333;
                 -2.868056	51.430833;
                 -2.865000	51.433889;
                 -2.861667	51.436389;
                 -2.858889	51.439444;
                 -2.855556	51.441944;
                 -2.852500	51.445000;
                 -2.849167	51.447500;
                 -2.845833	51.450000;
                 -2.842222	51.452500;
                 -2.838889	51.455000;
                 -2.836111	51.458056;
                 -2.831944	51.460000;
                 -2.828611	51.462778;
                 -2.824444	51.464722;
                 -2.820556	51.466667;
                 -2.817222	51.469167;
                 -2.813056	51.471111;
                 -2.808333	51.472778;
                 -2.804167	51.474722;
                 -2.800000	51.476667;
                 -2.796111	51.478611;
                 -2.792778	51.481111;
                 -2.788611	51.483333;
                 -2.783333	51.484167;
                 -2.779167	51.486111;
                 -2.774444	51.487778;
                 -2.770278	51.489722;
                 -2.765000	51.490556;
                 -2.759722	51.490833;
                 -2.753056	51.490833;
                 -2.745833	51.490278;
                 -2.738611	51.489722;
                 -2.731944	51.489722;
                 -2.726667	51.490556;
                 -2.721111	51.491667;
                 -2.716944	51.493611;
                 -2.713611	51.496111;
                 -2.710833	51.499167;
                 -2.708611	51.502778;
                 -2.706667	51.506111;
                 -2.702500	51.508056;
                 -2.698611	51.510278;
                 -2.695000	51.512778;
                 -2.691667	51.515278;
                 -2.688333	51.517778;
                 -2.685278	51.520833;
                 -2.681944	51.523333;
                 -2.681111	51.524167;
                 -2.681111	51.523611;
                 -2.677778	51.526111;
                 -2.674722	51.529167;
                 -2.672222	51.532222;
                 -2.670000	51.535556;
                 -2.666944	51.538611;
                 -2.665000	51.542222;
                 -2.662778	51.545833;
                 -2.660556	51.549167;
                 -2.659167	51.553333;
                 -2.657778	51.557222;
                 -2.655556	51.560833;
                 -2.653333	51.564444;
                 -2.650278	51.567500;
                 -2.647778	51.570278;
                 -2.645000	51.573333;
                 -2.642222	51.576389;
                 -2.639444	51.579444;
                 -2.635833	51.581944;
                 -2.633056	51.585000;
                 -2.629722	51.587500;
                 -2.626111	51.590000;
                 -2.621944	51.591944;
                 -2.618611	51.594444;
                 -2.615833	51.597500;
                 -2.612222	51.600000;
                 -2.608333	51.601944;
                 -2.605000	51.604444;
                 -2.600833	51.606389;
                 -2.596667	51.608611;
                 -2.592500	51.610556;
                 -2.588333	51.612500;
                 -2.585000	51.615000;
                 -2.581667	51.617500;
                 -2.579444	51.621111;
                 -2.577222	51.624444;
                 -2.575556	51.628611;
                 -2.573611	51.632222;
                 -2.571389	51.635556;
                 -2.569444	51.639167;
                 -2.566389	51.642222;
                 -2.564167	51.645556;
                 -2.561389	51.648611;
                 -2.558611	51.651667;
                 -2.555833	51.654722;
                 -2.553056	51.657778;
                 -2.549444	51.660278;
                 -2.546111	51.662778;
                 -2.542500	51.665833;
                 -2.539167	51.668333;
                 -2.535556	51.670833;
                 -2.531389	51.672778;
                 -2.527222	51.674722;
                 -2.523333	51.676667;
                 -2.518611	51.678056;
                 -2.514444	51.680278;
                 -2.510278	51.682222;
                 -2.506389	51.684167;
                 -2.502222	51.686111;
                 -2.498889	51.688611;
                 -2.495278	51.691111;
                 -2.492500	51.694167;
                 -2.489444	51.697222;
                 -2.486944	51.700278;
                 -2.485278	51.704167;
                 -2.483056	51.707778;
                 -2.481389	51.711667;
                 -2.480000	51.715833;
                 -2.477778	51.719167;
                 -2.475000	51.722222;
                 -2.471389	51.724722;
                 -2.467222	51.726944;
                 -2.463333	51.728889;
                 -2.459167	51.730833;
                 -2.454444	51.732222;
                 -2.450556	51.733333;
                 -2.445000	51.734444;
                 -2.439722	51.735278;
                 -2.434444	51.736389;
                 -2.428889	51.737222;
                 -2.422778	51.737778;
                 -2.417500	51.738611;
                 -2.412222	51.739722;
                 -2.406111	51.740000;
                 -2.401389	51.741667;
                 -2.396389	51.743056;
                 -2.391944	51.744444;
                 -2.388056	51.746944;
                 -2.384722	51.749444;
                 -2.381944	51.752500;
                 -2.380000	51.756111;
                 -2.378889	51.760556;
                 -2.379722	51.764444;
                 -2.381111	51.767778;
                 -2.383611	51.770556;
                 -2.386667	51.773333;
                 -2.390833	51.775278;
                 -2.396389	51.776667;
                 -2.400000	51.774167;
                 -2.399167	51.771389;
                 -2.397500	51.768056;
                 -2.396667	51.764167;
                 -2.396667	51.760278;
                 -2.397778	51.755556;
                 -2.402222	51.753056;
                 -2.406667	51.751667;
                 -2.412222	51.750833;
                 -2.418333	51.750278;
                 -2.424444	51.749722;
                 -2.430278	51.749444;
                 -2.436944	51.749444;
                 -2.442778	51.748889;
                 -2.449722	51.748889;
                 -2.455000	51.748056;
                 -2.460556	51.746944;
                 -2.465000	51.745556;
                 -2.470000	51.744167;
                 -2.474167	51.741944;
                 -2.478889	51.740556;
                 -2.482500	51.738056;
                 -2.485833	51.735556;
                 -2.489167	51.733056;
                 -2.492222	51.730000;
                 -2.495000	51.726944;
                 -2.497778	51.724167;
                 -2.500556	51.721111;
                 -2.503333	51.718056;
                 -2.506944	51.715556;
                 -2.509444	51.712500;
                 -2.513056	51.710000;
                 -2.517222	51.708056;
                 -2.521111	51.706111;
                 -2.524722	51.703611;
                 -2.528889	51.701389;
                 -2.533056	51.699444;
                 -2.537222	51.697500;
                 -2.540556	51.695000;
                 -2.544722	51.693056;
                 -2.548889	51.691111;
                 -2.552222	51.688611;
                 -2.556389	51.686389;
                 -2.560278	51.684444;
                 -2.565278	51.683056;
                 -2.569722	51.681667;
                 -2.573889	51.679444;
                 -2.578889	51.678056;
                 -2.582778	51.676111;
                 -2.586944	51.674167;
                 -2.591111	51.671944;
                 -2.594444	51.669444;
                 -2.598611	51.667500;
                 -2.601944	51.665000;
                 -2.606111	51.663056;
                 -2.609722	51.660556;
                 -2.613889	51.658611;
                 -2.616389	51.655556;
                 -2.620000	51.653056;
                 -2.622778	51.650000;
                 -2.625556	51.646944;
                 -2.628889	51.644444;
                 -2.631944	51.641389;
                 -2.634167	51.638056;
                 -2.636667	51.635000;
                 -2.640000	51.632500;
                 -2.643056	51.629444;
                 -2.645833	51.626389;
                 -2.650000	51.624444;
                 -2.654167	51.622500;
                 -2.657500	51.620000;
                 -2.661667	51.618056;
                 -2.665000	51.615556;
                 -2.668333	51.612778;
                 -2.671944	51.610278;
                 -2.674722	51.607500;
                 -2.678333	51.605000;
                 -2.681667	51.602500;
                 -2.685000	51.600000;
                 -2.688611	51.597222;
                 -2.691944	51.594722;
                 -2.695278	51.592222;
                 -2.698611	51.589722;
                 -2.702778	51.587778;
                 -2.706389	51.585278;
                 -2.710556	51.583333;
                 -2.714722	51.581389;
                 -2.719167	51.579722;
                 -2.724722	51.578889;
                 -2.730833	51.578333;
                 -2.736667	51.577778;
                 -2.742778	51.577500;
                 -2.748889	51.576944;
                 -2.754167	51.575833;
                 -2.759167	51.575000;
                 -2.765000	51.573889;
                 -2.769444	51.572500;
                 -2.774167	51.570833;
                 -2.779444	51.570000;
                 -2.783611	51.568056;
                 -2.788333	51.566389;
                 -2.793333	51.565000;
                 -2.797222	51.563056;
                 -2.801944	51.561389;
                 -2.805833	51.559444;
                 -2.810833	51.558056;
                 -2.814722	51.555833;
                 -2.819444	51.554444;
                 -2.823611	51.552500;
                 -2.828333	51.551111;
                 -2.833056	51.549444;
                 -2.837778	51.548056;
                 -2.843333	51.546944;
                 -2.847778	51.545556;
                 -2.853333	51.544444;
                 -2.858611	51.543611;
                 -2.863889	51.542500;
                 -2.870000	51.541944;
                 -2.875278	51.541111;
                 -2.881389	51.540556;
                 -2.886667	51.539444;
                 -2.892778	51.539167;
                 -2.899444	51.539167;
                 -2.905278	51.538611;
                 -2.911389	51.538056;
                 -2.918611	51.538056;
                 -2.926111	51.538611;
                 -2.933333	51.539167;
                 -2.940556	51.539722;
                 -2.947222	51.539722;
                 -2.954722	51.540278;
                 -2.960556	51.540833;
                 -2.966111	51.541944;
                 -2.967778	51.545278;
                 -2.965556	51.548889;
                 -2.964722	51.552500;
                 -2.966667	51.555833;
                 -2.970556	51.558056;
                 -2.976667	51.557500;
                 -2.981111	51.556944;
                 -2.985833	51.555278;
                 -2.988056	51.551944;
                 -2.989444	51.547778;
                 -2.991667	51.544167;
                 -2.995000	51.541667;
                 -2.998889	51.539722;
                 -3.004444	51.538889;
                 -3.009722	51.537778;
                 -3.013889	51.535833;
                 -3.014722	51.531111;
                 -3.017500	51.528333;
                 -3.021111	51.525556;
                 -3.025000	51.523611;
                 -3.029722	51.522222;
                 -3.035556	51.521667;
                 -3.041111	51.520556;
                 -3.045278	51.518611;
                 -3.049722	51.517222;
                 -3.053889	51.515000;
                 -3.057778	51.513056;
                 -3.061944	51.511111;
                 -3.066111	51.509167;
                 -3.070000	51.506944;
                 -3.074167	51.505000;
                 -3.078333	51.503056;
                 -3.082222	51.501111;
                 -3.086944	51.499444;
                 -3.091111	51.497500;
                 -3.095833	51.496111;
                 -3.100556	51.494444;
                 -3.105278	51.493056;
                 -3.110556	51.491944;
                 -3.115278	51.490556;
                 -3.118611	51.488056;
                 -3.121944	51.485556;
                 -3.125000	51.482500;
                 -3.127778	51.479444;
                 -3.129722	51.475833;
                 -3.131944	51.472222;
                 -3.134444	51.469167;
                 -3.133889	51.465556;
                 -3.135278	51.461389;
                 -3.138056	51.458611;
                 -3.140833	51.455556;
                 -3.145556	51.453889;
                 -3.151667	51.453333;
                 -3.158056	51.454444;
                 -3.164444	51.455278;
                 -3.167222	51.452222;
                 -3.170000	51.449167;
                 -3.168333	51.445833;
                 -3.166111	51.443056;
                 -3.163611	51.440000;
                 -3.160278	51.437500;
                 -3.158611	51.434167;
                 -3.158056	51.430556;
                 -3.158056	51.426389;
                 -3.158611	51.421667;
                 -3.160278	51.417778;
                 -3.161667	51.413611;
                 -3.163611	51.410278;
                 -3.166389	51.407222;
                 -3.170000	51.404722;
                 -3.173333	51.402222;
                 -3.176389	51.399444;
                 -3.180556	51.397500;
                 -3.185278	51.396111;
                 -3.192500	51.396667;
                 -3.198333	51.397778;
                 -3.204722	51.398889;
                 -3.210556	51.398333;
                 -3.216667	51.397778;
                 -3.222778	51.397222;
                 -3.228056	51.396111;
                 -3.233333	51.395278;
                 -3.238889	51.394167;
                 -3.243333	51.392778;
                 -3.247500	51.390556;
                 -3.251389	51.388611;
                 -3.255556	51.386667;
                 -3.259444	51.384722;
                 -3.264444	51.383056;
                 -3.268333	51.381111;
                 -3.273611	51.380000;
                 -3.280278	51.380000;
                 -3.287500	51.380556;
                 -3.293333	51.382222;
                 -3.298056	51.384167;
                 -3.303611	51.385278;
                 -3.309444	51.386667;
                 -3.315833	51.386667;
                 -3.320556	51.385000;
                 -3.324722	51.383056;
                 -3.328889	51.381111;
                 -3.334167	51.380000;
                 -3.340000	51.379444;
                 -3.346111	51.379167;
                 -3.352778	51.379167;
                 -3.359444	51.379167;
                 -3.365833	51.380000;
                 -3.372222	51.380833;
                 -3.378611	51.381667;
                 -3.385278	51.381667;
                 -3.391389	51.381111;
                 -3.397222	51.380833;
                 -3.403333	51.380278;
                 -3.410556	51.380556;
                 -3.416389	51.381111;
                 -3.421111	51.382778;
                 -3.425278	51.385000;
                 -3.431944	51.385833;
                 -3.438889	51.386389;
                 -3.445000	51.385833;
                 -3.450833	51.385278;
                 -3.457778	51.385278;
                 -3.465000	51.385833;
                 -3.470833	51.386944;
                 -3.476944	51.387778;
                 -3.482778	51.389167;
                 -3.488611	51.390556;
                 -3.494167	51.391944;
                 -3.499722	51.393056;
                 -3.504722	51.394722;
                 -3.509444	51.396389;
                 -3.516944	51.396944;
                 -3.523056	51.397778;
                 -3.529722	51.397778;
                 -3.535833	51.397222;
                 -3.542222	51.398333;
                 -3.547222	51.400000;
                 -3.551389	51.401944;
                 -3.555556	51.404167;
                 -3.560278	51.405833;
                 -3.562778	51.408889;
                 -3.565278	51.411667;
                 -3.566944	51.415000;
                 -3.569444	51.418056;
                 -3.571667	51.420833;
                 -3.575556	51.423056;
                 -3.579722	51.425000;
                 -3.584722	51.426667;
                 -3.589722	51.428611;
                 -3.593611	51.430556;
                 -3.597778	51.432778;
                 -3.599444	51.436111;
                 -3.602500	51.438611;
                 -3.606667	51.440556;
                 -3.610556	51.442778;
                 -3.615556	51.444444;
                 -3.619722	51.446667;
                 -3.624444	51.448333;
                 -3.628611	51.450278;
                 -3.632500	51.452500;
                 -3.636111	51.455000;
                 -3.638333	51.457778;
                 -3.640000	51.461111;
                 -3.643333	51.463611;
                 -3.646389	51.466111;
                 -3.650556	51.468333;
                 -3.654722	51.470556;
                 -3.659722	51.472222;
                 -3.665278	51.473333;
                 -3.671944	51.474167;
                 -3.676944	51.473333;
                 -3.680556	51.470833;
                 -3.684444	51.468611;
                 -3.689722	51.468611;
                 -3.692500	51.469722;
                 -3.697222	51.471389;
                 -3.703333	51.470833;
                 -3.710000	51.470833;
                 -3.714722	51.472500;
                 -3.718889	51.474722;
                 -3.721389	51.477500;
                 -3.723056	51.480833;
                 -3.725556	51.483889;
                 -3.727222	51.487222;
                 -3.731111	51.489167;
                 -3.736111	51.490833;
                 -3.740278	51.493056;
                 -3.744444	51.495000;
                 -3.746111	51.498333;
                 -3.745000	51.503056;
                 -3.744444	51.507500;
                 -3.745278	51.511111;
                 -3.746111	51.515000;
                 -3.746944	51.518611;
                 -3.747778	51.522500;
                 -3.749444	51.525833;
                 -3.751111	51.529167;
                 -3.753056	51.532500;
                 -3.754722	51.535833;
                 -3.756944	51.538611;
                 -3.758889	51.541944;
                 -3.761111	51.545000;
                 -3.763611	51.547778;
                 -3.766111	51.550833;
                 -3.768611	51.553611;
                 -3.771111	51.556667;
                 -3.773333	51.559444;
                 -3.776944	51.561944;
                 -3.779167	51.565000;
                 -3.781667	51.567778;
                 -3.785000	51.570278;
                 -3.790000	51.571944;
                 -3.793889	51.574167;
                 -3.797222	51.576944;
                 -3.798889	51.580278;
                 -3.801389	51.583333;
                 -3.803611	51.586111;
                 -3.807222	51.588611;
                 -3.812778	51.590000;
                 -3.816944	51.592222;
                 -3.821111	51.594167;
                 -3.825000	51.596389;
                 -3.828333	51.598889;
                 -3.831667	51.601389;
                 -3.834167	51.604167;
                 -3.835000	51.608056;
                 -3.833611	51.611944;
                 -3.833611	51.616111;
                 -3.835556	51.619444;
                 -3.840278	51.621111;
                 -3.845556	51.621111;
                 -3.851667	51.620556;
                 -3.856944	51.619444;
                 -3.863611	51.619444;
                 -3.869444	51.618889;
                 -3.875556	51.618333;
                 -3.881667	51.617778;
                 -3.886944	51.616667;
                 -3.892778	51.616111;
                 -3.897500	51.614722;
                 -3.902222	51.613056;
                 -3.906389	51.611111;
                 -3.910833	51.609444;
                 -3.917778	51.609444;
                 -3.924167	51.610278;
                 -3.930000	51.611667;
                 -3.935556	51.612778;
                 -3.942778	51.613333;
                 -3.949444	51.613333;
                 -3.954722	51.612222;
                 -3.960278	51.611111;
                 -3.965556	51.610278;
                 -3.970278	51.608611;
                 -3.974722	51.607222;
                 -3.979722	51.605556;
                 -3.984444	51.603889;
                 -3.988333	51.601944;
                 -3.991667	51.599444;
                 -3.995833	51.597500;
                 -3.998611	51.594444;
                 -4.001111	51.591389;
                 -4.002778	51.587222;
                 -4.000000	51.584444;
                 -3.996944	51.581944;
                 -3.993333	51.579444;
                 -3.990278	51.576944;
                 -3.986111	51.574722;
                 -3.981944	51.572500;
                 -3.977778	51.570556;
                 -3.973889	51.568333;
                 -3.972222	51.565000;
                 -3.973611	51.561111;
                 -3.976944	51.558611;
                 -3.983611	51.558611;
                 -3.990000	51.559444;
                 -3.995000	51.561111;
                 -3.999722	51.562778;
                 -4.005000	51.564444;
                 -4.010000	51.563333;
                 -4.013333	51.560278;
                 -4.015000	51.556389;
                 -4.016944	51.552778;
                 -4.023056	51.552222;
                 -4.027778	51.553889;
                 -4.031389	51.556389;
                 -4.033056	51.559722;
                 -4.036944	51.561667;
                 -4.041667	51.563611;
                 -4.048333	51.564444;
                 -4.053056	51.562778;
                 -4.057222	51.560833;
                 -4.060278	51.558333;
                 -4.064444	51.556111;
                 -4.069722	51.555000;
                 -4.076389	51.555000;
                 -4.081389	51.556667;
                 -4.086111	51.558333;
                 -4.090278	51.560556;
                 -4.094444	51.562500;
                 -4.099444	51.564167;
                 -4.105000	51.565556;
                 -4.111944	51.565556;
                 -4.117222	51.564444;
                 -4.122222	51.563333;
                 -4.126944	51.561944;
                 -4.132222	51.560833;
                 -4.137778	51.559722;
                 -4.142222	51.558056;
                 -4.147222	51.556667;
                 -4.150278	51.554167;
                 -4.151111	51.549444;
                 -4.150278	51.545833;
                 -4.150278	51.541667;
                 -4.153611	51.539167;
                 -4.158889	51.538056;
                 -4.165278	51.538889;
                 -4.171944	51.539722;
                 -4.176667	51.541389;
                 -4.181667	51.543056;
                 -4.188333	51.543889;
                 -4.194722	51.543889;
                 -4.200278	51.542778;
                 -4.201667	51.538889;
                 -4.202222	51.535278;
                 -4.205556	51.532500;
                 -4.211389	51.531944;
                 -4.217222	51.533333;
                 -4.222778	51.534444;
                 -4.227778	51.536111;
                 -4.232778	51.537778;
                 -4.237500	51.539444;
                 -4.242778	51.541111;
                 -4.246667	51.543889;
                 -4.250833	51.545833;
                 -4.255000	51.548056;
                 -4.259167	51.550000;
                 -4.263056	51.552222;
                 -4.267222	51.554167;
                 -4.271111	51.556389;
                 -4.277778	51.557222;
                 -4.283611	51.556667;
                 -4.289722	51.556111;
                 -4.295278	51.557222;
                 -4.296389	51.561111;
                 -4.294444	51.564444;
                 -4.291667	51.567500;
                 -4.287500	51.569722;
                 -4.283611	51.571667;
                 -4.280278	51.574167;
                 -4.277500	51.577222;
                 -4.276944	51.581944;
                 -4.278611	51.585278;
                 -4.281111	51.588056;
                 -4.283611	51.591111;
                 -4.286111	51.593889;
                 -4.288611	51.596667;
                 -4.291389	51.599722;
                 -4.293056	51.603056;
                 -4.293056	51.607222;
                 -4.291111	51.610833;
                 -4.286111	51.612222;
                 -4.281111	51.612500;
                 -4.277222	51.613611;
                 -4.271667	51.614722;
                 -4.267778	51.616667;
                 -4.265000	51.619722;
                 -4.262500	51.622778;
                 -4.257778	51.624444;
                 -4.253611	51.626389;
                 -4.250556	51.628889;
                 -4.247778	51.631944;
                 -4.245000	51.635000;
                 -4.242222	51.638056;
                 -4.236944	51.639167;
                 -4.232500	51.640833;
                 -4.230556	51.637500;
                 -4.231944	51.633333;
                 -4.233889	51.629722;
                 -4.231389	51.626944;
                 -4.227222	51.624722;
                 -4.220833	51.623889;
                 -4.215556	51.625000;
                 -4.211389	51.627222;
                 -4.206111	51.628056;
                 -4.200278	51.628611;
                 -4.192778	51.628333;
                 -4.188056	51.626667;
                 -4.184444	51.625000;
                 -4.180278	51.622778;
                 -4.174444	51.622500;
                 -4.168333	51.623056;
                 -4.162222	51.623611;
                 -4.157222	51.624722;
                 -4.152222	51.626389;
                 -4.146944	51.627222;
                 -4.142222	51.628889;
                 -4.136944	51.630000;
                 -4.133056	51.631944;
                 -4.127778	51.633056;
                 -4.122222	51.634167;
                 -4.116111	51.634722;
                 -4.111667	51.636111;
                 -4.106944	51.637778;
                 -4.103611	51.640278;
                 -4.100278	51.642778;
                 -4.095556	51.644444;
                 -4.089722	51.645000;
                 -4.084167	51.646111;
                 -4.078056	51.646667;
                 -4.074722	51.649167;
                 -4.073611	51.653056;
                 -4.070556	51.656111;
                 -4.068056	51.659167;
                 -4.065278	51.662222;
                 -4.063889	51.666389;
                 -4.064722	51.670000;
                 -4.067500	51.673056;
                 -4.070556	51.675556;
                 -4.069722	51.675833;
                 -4.074722	51.677500;
                 -4.078056	51.675000;
                 -4.080833	51.671944;
                 -4.082778	51.668333;
                 -4.086111	51.665833;
                 -4.089722	51.663333;
                 -4.094167	51.661667;
                 -4.098889	51.660278;
                 -4.105000	51.659722;
                 -4.111667	51.659722;
                 -4.118056	51.660556;
                 -4.125556	51.660833;
                 -4.130000	51.659444;
                 -4.134167	51.657222;
                 -4.138889	51.655833;
                 -4.144722	51.656111;
                 -4.149722	51.657778;
                 -4.153889	51.660000;
                 -4.158056	51.661944;
                 -4.160278	51.665000;
                 -4.163889	51.667500;
                 -4.166944	51.670000;
                 -4.171111	51.671944;
                 -4.176111	51.673611;
                 -4.180278	51.675833;
                 -4.184444	51.677778;
                 -4.190000	51.679167;
                 -4.197222	51.679444;
                 -4.203333	51.678889;
                 -4.210000	51.678889;
                 -4.215278	51.677778;
                 -4.222222	51.677778;
                 -4.228056	51.677222;
                 -4.235000	51.677222;
                 -4.241389	51.677222;
                 -4.248056	51.676944;
                 -4.255000	51.676944;
                 -4.262222	51.677500;
                 -4.268056	51.676944;
                 -4.272778	51.675278;
                 -4.277500	51.673611;
                 -4.281111	51.671111;
                 -4.285556	51.669722;
                 -4.292222	51.669444;
                 -4.298889	51.670278;
                 -4.304444	51.671667;
                 -4.309444	51.673333;
                 -4.314167	51.675000;
                 -4.318333	51.677222;
                 -4.322500	51.679167;
                 -4.326667	51.681111;
                 -4.330833	51.683333;
                 -4.334167	51.685833;
                 -4.337500	51.688333;
                 -4.340833	51.690833;
                 -4.343889	51.693333;
                 -4.347500	51.695833;
                 -4.350833	51.698333;
                 -4.355000	51.700278;
                 -4.359167	51.702500;
                 -4.363056	51.704444;
                 -4.367222	51.706667;
                 -4.370556	51.709167;
                 -4.373889	51.711667;
                 -4.377222	51.714167;
                 -4.380000	51.716944;
                 -4.380556	51.720833;
                 -4.379444	51.723889;
                 -4.376111	51.726389;
                 -4.369444	51.726389;
                 -4.363611	51.726111;
                 -4.356944	51.725278;
                 -4.351389	51.724167;
                 -4.345556	51.722778;
                 -4.338889	51.721944;
                 -4.333333	51.720556;
                 -4.326667	51.719722;
                 -4.320000	51.718889;
                 -4.314167	51.719444;
                 -4.310000	51.721667;
                 -4.311111	51.725278;
                 -4.313611	51.728333;
                 -4.317500	51.730278;
                 -4.322500	51.731944;
                 -4.329167	51.732778;
                 -4.336389	51.733333;
                 -4.343056	51.734167;
                 -4.349722	51.735000;
                 -4.356111	51.735833;
                 -4.361111	51.737500;
                 -4.365278	51.739444;
                 -4.368333	51.741944;
                 -4.369444	51.745833;
                 -4.369722	51.750000;
                 -4.367500	51.753333;
                 -4.365556	51.756944;
                 -4.365278	51.758889;
                 -4.364444	51.759167;
                 -4.362778	51.760833;
                 -4.360556	51.764444;
                 -4.359444	51.768611;
                 -4.358611	51.773056;
                 -4.359722	51.776944;
                 -4.360278	51.780556;
                 -4.363056	51.783611;
                 -4.368056	51.785278;
                 -4.372500	51.783611;
                 -4.375278	51.780556;
                 -4.378056	51.777500;
                 -4.380556	51.774444;
                 -4.382500	51.770833;
                 -4.386111	51.768333;
                 -4.389444	51.765833;
                 -4.392222	51.762778;
                 -4.395556	51.760000;
                 -4.399444	51.758056;
                 -4.404722	51.756944;
                 -4.410556	51.757222;
                 -4.415556	51.758889;
                 -4.419722	51.761111;
                 -4.424722	51.762778;
                 -4.428889	51.764722;
                 -4.429722	51.768611;
                 -4.428889	51.773056;
                 -4.431667	51.776111;
                 -4.437222	51.777222;
                 -4.442222	51.775833;
                 -4.446111	51.773611;
                 -4.448611	51.770556;
                 -4.451389	51.767500;
                 -4.453611	51.763889;
                 -4.450833	51.761111;
                 -4.447500	51.758611;
                 -4.444444	51.756111;
                 -4.440278	51.753889;
                 -4.436667	51.751389;
                 -4.432500	51.749444;
                 -4.430278	51.746667;
                 -4.429167	51.742778;
                 -4.431944	51.739722;
                 -4.436111	51.737778;
                 -4.441111	51.736667;
                 -4.447222	51.736111;
                 -4.454167	51.736111;
                 -4.461389	51.736389;
                 -4.468889	51.736944;
                 -4.475278	51.736667;
                 -4.480833	51.735556;
                 -4.485278	51.734167;
                 -4.490833	51.733056;
                 -4.495278	51.731389;
                 -4.500833	51.730278;
                 -4.507500	51.730278;
                 -4.515000	51.730833;
                 -4.521111	51.731667;
                 -4.527222	51.732778;
                 -4.534444	51.733333;
                 -4.539444	51.733889;
                 -4.546111	51.733889;
                 -4.553333	51.734444;
                 -4.561111	51.734722;
                 -4.567500	51.734722;
                 -4.574167	51.734722;
                 -4.579722	51.733611;
                 -4.584167	51.731944;
                 -4.589722	51.730833;
                 -4.596389	51.730833;
                 -4.601944	51.729722;
                 -4.608333	51.729722;
                 -4.615000	51.729444;
                 -4.621944	51.729444;
                 -4.628333	51.729444;
                 -4.633889	51.728333;
                 -4.639722	51.727778;
                 -4.645833	51.727222;
                 -4.651111	51.726111;
                 -4.656389	51.725000;
                 -4.661944	51.723889;
                 -4.667222	51.722778;
                 -4.671944	51.721389;
                 -4.675833	51.719167;
                 -4.679167	51.716667;
                 -4.682500	51.714167;
                 -4.684444	51.710556;
                 -4.685833	51.706389;
                 -4.684167	51.703056;
                 -4.682500	51.699722;
                 -4.681389	51.696111;
                 -4.683333	51.692500;
                 -4.686667	51.690000;
                 -4.690556	51.687778;
                 -4.693333	51.684722;
                 -4.694722	51.680833;
                 -4.693889	51.676944;
                 -4.691389	51.674167;
                 -4.689444	51.670833;
                 -4.690278	51.667222;
                 -4.693333	51.664722;
                 -4.698056	51.663056;
                 -4.702222	51.661111;
                 -4.704722	51.658056;
                 -4.706667	51.654444;
                 -4.708611	51.650833;
                 -4.713333	51.649167;
                 -4.719444	51.648611;
                 -4.725000	51.650000;
                 -4.730833	51.651111;
                 -4.736667	51.652222;
                 -4.743889	51.652778;
                 -4.750000	51.652222;
                 -4.755278	51.651111;
                 -4.758056	51.648056;
                 -4.757778	51.643889;
                 -4.761667	51.641944;
                 -4.769167	51.642222;
                 -4.775278	51.641667;
                 -4.777778	51.638611;
                 -4.779722	51.635000;
                 -4.783333	51.632500;
                 -4.789444	51.633333;
                 -4.794722	51.635000;
                 -4.798889	51.636944;
                 -4.803056	51.639167;
                 -4.807222	51.641111;
                 -4.813056	51.640556;
                 -4.818611	51.641667;
                 -4.822778	51.643889;
                 -4.827778	51.645556;
                 -4.834167	51.646389;
                 -4.840278	51.645556;
                 -4.844167	51.643611;
                 -4.849722	51.642500;
                 -4.856389	51.642500;
                 -4.861667	51.641389;
                 -4.866389	51.639722;
                 -4.869722	51.637222;
                 -4.872222	51.634167;
                 -4.876111	51.631944;
                 -4.880833	51.630556;
                 -4.885556	51.628889;
                 -4.890278	51.627222;
                 -4.893611	51.624722;
                 -4.896944	51.622222;
                 -4.899444	51.619167;
                 -4.899444	51.615000;
                 -4.897778	51.611667;
                 -4.898056	51.608056;
                 -4.903611	51.606944;
                 -4.909444	51.606389;
                 -4.915556	51.605833;
                 -4.920278	51.604167;
                 -4.923333	51.601667;
                 -4.926667	51.598889;
                 -4.929444	51.595833;
                 -4.934722	51.594722;
                 -4.941389	51.594722;
                 -4.946389	51.596389;
                 -4.950556	51.598333;
                 -4.956389	51.599722;
                 -4.962222	51.600000;
                 -4.965278	51.602500;
                 -4.968889	51.605000;
                 -4.972222	51.607222;
                 -4.976944	51.608889;
                 -4.983611	51.609722;
                 -4.988889	51.608611;
                 -4.993611	51.607222;
                 -4.999444	51.606389;
                 -5.005278	51.606944;
                 -5.009444	51.608889;
                 -5.013056	51.611389;
                 -5.016389	51.613889;
                 -5.021111	51.615556;
                 -5.027778	51.616389;
                 -5.035278	51.616667;
                 -5.042500	51.616944;
                 -5.047500	51.618611;
                 -5.051389	51.620833;
                 -5.051667	51.625000;
                 -5.047500	51.626944;
                 -5.043611	51.629167;
                 -5.040556	51.631667;
                 -5.037222	51.634167;
                 -5.037222	51.638333;
                 -5.041389	51.640278;
                 -5.043889	51.643333;
                 -5.045000	51.646944;
                 -5.045278	51.651111;
                 -5.049444	51.653333;
                 -5.054167	51.654722;
                 -5.058333	51.656944;
                 -5.062500	51.658889;
                 -5.067500	51.660556;
                 -5.071667	51.662500;
                 -5.076667	51.664167;
                 -5.082500	51.664444;
                 -5.088611	51.663889;
                 -5.094444	51.663333;
                 -5.100278	51.664444;
                 -5.104444	51.666667;
                 -5.107778	51.669167;
                 -5.111111	51.671667;
                 -5.115278	51.673611;
                 -5.118056	51.676389;
                 -5.118889	51.680278;
                 -5.117778	51.683333;
                 -5.114444	51.685833;
                 -5.109167	51.686111;
                 -5.103889	51.686389;
                 -5.096667	51.685833;
                 -5.090833	51.684722;
                 -5.086667	51.682778;
                 -5.082500	51.680556;
                 -5.080000	51.677778;
                 -5.075000	51.676111;
                 -5.070000	51.675278;
                 -5.062778	51.675000;
                 -5.056667	51.675556;
                 -5.050000	51.675833;
                 -5.043889	51.676389;
                 -5.038611	51.677500;
                 -5.034722	51.679444;
                 -5.037222	51.682500;
                 -5.041389	51.684444;
                 -5.043889	51.687222;
                 -5.042778	51.690556;
                 -5.037222	51.691667;
                 -5.030000	51.691111;
                 -5.023333	51.690278;
                 -5.019167	51.688333;
                 -5.015000	51.686389;
                 -5.008611	51.685556;
                 -5.003056	51.685278;
                 -4.996111	51.685278;
                 -4.989444	51.685278;
                 -4.982778	51.685556;
                 -4.976944	51.686111;
                 -4.970833	51.686667;
                 -4.965278	51.687778;
                 -4.960556	51.689444;
                 -4.955278	51.690556;
                 -4.950556	51.691944;
                 -4.946111	51.693611;
                 -4.940556	51.694722;
                 -4.934722	51.695278;
                 -4.928611	51.695833;
                 -4.922222	51.696111;
                 -4.916111	51.696667;
                 -4.912222	51.698611;
                 -4.907500	51.700278;
                 -4.902222	51.701389;
                 -4.896111	51.701944;
                 -4.891389	51.703611;
                 -4.886667	51.705000;
                 -4.882778	51.707222;
                 -4.879444	51.709722;
                 -4.877500	51.713333;
                 -4.880000	51.716111;
                 -4.885833	51.717500;
                 -4.891944	51.716944;
                 -4.896389	51.715278;
                 -4.901111	51.713611;
                 -4.905000	51.711667;
                 -4.910278	51.710556;
                 -4.916389	51.710000;
                 -4.923056	51.709722;
                 -4.929167	51.709167;
                 -4.935833	51.709167;
                 -4.942500	51.708889;
                 -4.948611	51.708333;
                 -4.954722	51.707778;
                 -4.959167	51.706111;
                 -4.963056	51.704167;
                 -4.966944	51.701944;
                 -4.971667	51.700556;
                 -4.976944	51.699444;
                 -4.983056	51.698889;
                 -4.989722	51.699722;
                 -4.995278	51.700833;
                 -4.999444	51.702778;
                 -5.004722	51.704444;
                 -5.009444	51.706111;
                 -5.015278	51.707500;
                 -5.021111	51.708611;
                 -5.027778	51.708611;
                 -5.033611	51.707778;
                 -5.039167	51.706667;
                 -5.044444	51.705556;
                 -5.050278	51.705000;
                 -5.056389	51.704444;
                 -5.062222	51.703889;
                 -5.067500	51.702778;
                 -5.073611	51.702222;
                 -5.080278	51.703056;
                 -5.084444	51.705000;
                 -5.086944	51.708056;
                 -5.089722	51.710833;
                 -5.093056	51.713333;
                 -5.097778	51.715000;
                 -5.105278	51.715278;
                 -5.110000	51.713611;
                 -5.113889	51.711667;
                 -5.118611	51.710000;
                 -5.123889	51.709722;
                 -5.128056	51.711944;
                 -5.133889	51.712222;
                 -5.138333	51.710556;
                 -5.143611	51.710556;
                 -5.148611	51.711944;
                 -5.152778	51.714167;
                 -5.157778	51.715833;
                 -5.162778	51.716389;
                 -5.165556	51.713333;
                 -5.163889	51.710000;
                 -5.161944	51.706667;
                 -5.160833	51.703056;
                 -5.160833	51.698889;
                 -5.158889	51.695556;
                 -5.157222	51.692222;
                 -5.157778	51.688611;
                 -5.160833	51.686111;
                 -5.164444	51.683611;
                 -5.168889	51.681944;
                 -5.175556	51.682778;
                 -5.180556	51.684444;
                 -5.184444	51.686389;
                 -5.186389	51.689722;
                 -5.185833	51.693333;
                 -5.183333	51.696389;
                 -5.183611	51.700556;
                 -5.186111	51.703333;
                 -5.189444	51.705833;
                 -5.191111	51.709167;
                 -5.194444	51.711667;
                 -5.199444	51.713333;
                 -5.204722	51.715000;
                 -5.208611	51.716944;
                 -5.212222	51.719444;
                 -5.217778	51.720556;
                 -5.223611	51.721944;
                 -5.228611	51.723611;
                 -5.231944	51.725833;
                 -5.235278	51.728333;
                 -5.240833	51.729722;
                 -5.246944	51.730833;
                 -5.243611	51.731944;
                 -5.237778	51.732778;
                 -5.232500	51.733889;
                 -5.225000	51.733333;
                 -5.218611	51.732500;
                 -5.212778	51.731389;
                 -5.205278	51.731111;
                 -5.200000	51.732222;
                 -5.199444	51.735833;
                 -5.198889	51.740278;
                 -5.197500	51.744444;
                 -5.195000	51.747500;
                 -5.190556	51.749167;
                 -5.184444	51.749722;
                 -5.176944	51.749167;
                 -5.172778	51.751389;
                 -5.170278	51.754444;
                 -5.167778	51.757500;
                 -5.164444	51.760000;
                 -5.161111	51.762778;
                 -5.156389	51.764167;
                 -5.151111	51.765278;
                 -5.145000	51.766111;
                 -5.137778	51.765556;
                 -5.131667	51.766111;
                 -5.124167	51.765833;
                 -5.118889	51.766944;
                 -5.112222	51.766944;
                 -5.107778	51.768611;
                 -5.104167	51.771111;
                 -5.103056	51.775278;
                 -5.101944	51.779444;
                 -5.101944	51.783611;
                 -5.103056	51.787222;
                 -5.103889	51.790833;
                 -5.105833	51.794167;
                 -5.106667	51.798056;
                 -5.105833	51.802500;
                 -5.107778	51.805833;
                 -5.110278	51.808611;
                 -5.113056	51.811667;
                 -5.114722	51.815000;
                 -5.115000	51.819167;
                 -5.115000	51.823056;
                 -5.114444	51.827778;
                 -5.114722	51.831944;
                 -5.115833	51.835556;
                 -5.117500	51.838889;
                 -5.119167	51.842222;
                 -5.120000	51.845833;
                 -5.121944	51.849167;
                 -5.124444	51.852222;
                 -5.127222	51.855000;
                 -5.131944	51.856667;
                 -5.136111	51.858611;
                 -5.142222	51.858056;
                 -5.148333	51.857500;
                 -5.153333	51.859167;
                 -5.157500	51.861111;
                 -5.161667	51.863056;
                 -5.166389	51.864722;
                 -5.172500	51.866111;
                 -5.177500	51.867778;
                 -5.181667	51.869722;
                 -5.188333	51.870556;
                 -5.193611	51.869444;
                 -5.198889	51.868333;
                 -5.204722	51.867778;
                 -5.209167	51.868889;
                 -5.213333	51.870833;
                 -5.219167	51.872222;
                 -5.224722	51.871111;
                 -5.229722	51.870000;
                 -5.234722	51.868333;
                 -5.241111	51.868056;
                 -5.246389	51.868889;
                 -5.252222	51.870000;
                 -5.258889	51.870833;
                 -5.264722	51.870278;
                 -5.270833	51.869722;
                 -5.276111	51.868611;
                 -5.280000	51.866389;
                 -5.283333	51.863889;
                 -5.287222	51.861667;
                 -5.291944	51.860278;
                 -5.297778	51.861389;
                 -5.301944	51.863333;
                 -5.305833	51.861389;
                 -5.309444	51.861944;
                 -5.306389	51.864444;
                 -5.303056	51.867222;
                 -5.299167	51.869167;
                 -5.300000	51.873056;
                 -5.301111	51.876667;
                 -5.301944	51.880278;
                 -5.302222	51.884444;
                 -5.297500	51.886111;
                 -5.291667	51.885833;
                 -5.288333	51.888333;
                 -5.288611	51.892500;
                 -5.291944	51.895000;
                 -5.294444	51.898056;
                 -5.296389	51.901111;
                 -5.297500	51.905000;
                 -5.296667	51.909444;
                 -5.293889	51.912500;
                 -5.289444	51.914167;
                 -5.283333	51.914722;
                 -5.282778	51.914167;
                 -5.283333	51.913889;
                 -5.276944	51.913889;
                 -5.270000	51.914167;
                 -5.263333	51.914167;
                 -5.256667	51.914444;
                 -5.250000	51.914444;
                 -5.244722	51.915556;
                 -5.239167	51.916667;
                 -5.235278	51.918889;
                 -5.231944	51.921389;
                 -5.228611	51.923889;
                 -5.224722	51.926111;
                 -5.219167	51.927222;
                 -5.212500	51.927222;
                 -5.207778	51.928889;
                 -5.205833	51.932500;
                 -5.203889	51.936111;
                 -5.199167	51.937500;
                 -5.193889	51.938611;
                 -5.191111	51.941667;
                 -5.188056	51.944722;
                 -5.184444	51.947500;
                 -5.179167	51.948611;
                 -5.171944	51.948056;
                 -5.164444	51.947778;
                 -5.157778	51.947778;
                 -5.153611	51.950000;
                 -5.150278	51.952500;
                 -5.146944	51.955000;
                 -5.143056	51.957222;
                 -5.136944	51.957778;
                 -5.129722	51.957500;
                 -5.123056	51.956667;
                 -5.117778	51.956944;
                 -5.113889	51.958889;
                 -5.107778	51.959444;
                 -5.101667	51.960000;
                 -5.096944	51.961667;
                 -5.095000	51.965278;
                 -5.092222	51.968333;
                 -5.088333	51.970556;
                 -5.085000	51.973056;
                 -5.081667	51.975556;
                 -5.079722	51.979167;
                 -5.079167	51.982778;
                 -5.081111	51.986111;
                 -5.082778	51.989444;
                 -5.085278	51.992778;
                 -5.081389	51.994722;
                 -5.076667	51.996389;
                 -5.073889	51.999444;
                 -5.072778	52.003611;
                 -5.075278	52.006389;
                 -5.079444	52.008611;
                 -5.082778	52.011111;
                 -5.084444	52.014167;
                 -5.083889	52.018056;
                 -5.081389	52.021111;
                 -5.075833	52.022222;
                 -5.070000	52.022778;
                 -5.063889	52.023333;
                 -5.057222	52.023333;
                 -5.051389	52.024167;
                 -5.045278	52.024722;
                 -5.038333	52.024722;
                 -5.031667	52.023889;
                 -5.025833	52.022778;
                 -5.022500	52.021111;
                 -5.017500	52.019444;
                 -5.011667	52.018333;
                 -5.004444	52.017778;
                 -4.998333	52.018611;
                 -4.990833	52.018056;
                 -4.984167	52.017222;
                 -4.980000	52.015278;
                 -4.976667	52.012778;
                 -4.974167	52.010000;
                 -4.973056	52.006111;
                 -4.973333	52.003333;
                 -4.976944	52.000833;
                 -4.977778	51.998611;
                 -4.972222	51.997500;
                 -4.965556	51.996667;
                 -4.960278	51.997778;
                 -4.955278	51.999167;
                 -4.950556	52.000833;
                 -4.946111	52.002500;
                 -4.941389	52.003889;
                 -4.936111	52.005000;
                 -4.931944	52.007222;
                 -4.929722	52.008056;
                 -4.929444	52.008333;
                 -4.923611	52.008056;
                 -4.918333	52.009167;
                 -4.914722	52.011667;
                 -4.916111	52.014722;
                 -4.918611	52.017500;
                 -4.921111	52.020278;
                 -4.919167	52.023889;
                 -4.915278	52.026111;
                 -4.909167	52.025833;
                 -4.905833	52.024167;
                 -4.902222	52.021667;
                 -4.898056	52.019722;
                 -4.894167	52.017500;
                 -4.889722	52.015556;
                 -4.886389	52.013056;
                 -4.881389	52.011389;
                 -4.875556	52.011944;
                 -4.870000	52.013056;
                 -4.864444	52.014167;
                 -4.858611	52.014722;
                 -4.851111	52.014444;
                 -4.844444	52.014444;
                 -4.837778	52.014444;
                 -4.832222	52.015556;
                 -4.828333	52.017778;
                 -4.829444	52.021389;
                 -4.831667	52.024444;
                 -4.834444	52.027222;
                 -4.836944	52.030000;
                 -4.838611	52.033333;
                 -4.839722	52.037222;
                 -4.838333	52.041111;
                 -4.835556	52.044167;
                 -4.832222	52.046944;
                 -4.826944	52.047778;
                 -4.821667	52.048889;
                 -4.816111	52.050000;
                 -4.810833	52.051111;
                 -4.805556	52.052222;
                 -4.798889	52.052222;
                 -4.792778	52.053056;
                 -4.787778	52.054444;
                 -4.785278	52.057500;
                 -4.781944	52.060278;
                 -4.777222	52.061667;
                 -4.772500	52.063333;
                 -4.767778	52.065000;
                 -4.765000	52.068056;
                 -4.761667	52.070556;
                 -4.759167	52.073611;
                 -4.756944	52.077222;
                 -4.755000	52.080833;
                 -4.752500	52.083889;
                 -4.750278	52.087222;
                 -4.748333	52.090833;
                 -4.745556	52.093889;
                 -4.743611	52.097500;
                 -4.741667	52.101111;
                 -4.739444	52.104722;
                 -4.736389	52.107222;
                 -4.731389	52.108889;
                 -4.726389	52.108056;
                 -4.721111	52.107222;
                 -4.715556	52.106111;
                 -4.711111	52.104167;
                 -4.706389	52.102500;
                 -4.701944	52.101111;
                 -4.696389	52.100000;
                 -4.692222	52.101944;
                 -4.690833	52.106111;
                 -4.691667	52.109722;
                 -4.692500	52.113611;
                 -4.694444	52.116944;
                 -4.696944	52.119722;
                 -4.698056	52.123611;
                 -4.696667	52.127500;
                 -4.693333	52.130000;
                 -4.687222	52.130833;
                 -4.681389	52.129444;
                 -4.674722	52.129444;
                 -4.669167	52.130556;
                 -4.662500	52.130833;
                 -4.657222	52.131667;
                 -4.651944	52.132778;
                 -4.646944	52.134444;
                 -4.640833	52.135000;
                 -4.634167	52.135000;
                 -4.626667	52.134722;
                 -4.621111	52.133333;
                 -4.615278	52.132222;
                 -4.608333	52.132222;
                 -4.603889	52.133889;
                 -4.596111	52.133333;
                 -4.590556	52.132222;
                 -4.584444	52.132778;
                 -4.579722	52.134444;
                 -4.575556	52.136389;
                 -4.570833	52.138056;
                 -4.565556	52.138056;
                 -4.561389	52.136944;
                 -4.555556	52.135833;
                 -4.550556	52.134167;
                 -4.545000	52.132500;
                 -4.538333	52.131667;
                 -4.532222	52.131111;
                 -4.527222	52.131389;
                 -4.522222	52.133056;
                 -4.518333	52.135000;
                 -4.514167	52.137222;
                 -4.507222	52.137222;
                 -4.500000	52.136667;
                 -4.495278	52.138333;
                 -4.491944	52.140833;
                 -4.492222	52.145000;
                 -4.492222	52.149167;
                 -4.488889	52.151667;
                 -4.482778	52.152222;
                 -4.477222	52.153333;
                 -4.472778	52.155000;
                 -4.469167	52.157500;
                 -4.466944	52.161111;
                 -4.465000	52.164722;
                 -4.458889	52.165278;
                 -4.452500	52.164444;
                 -4.447222	52.164444;
                 -4.441111	52.165000;
                 -4.436389	52.166667;
                 -4.432500	52.168611;
                 -4.429444	52.171667;
                 -4.425556	52.173889;
                 -4.421389	52.175833;
                 -4.418056	52.178333;
                 -4.414167	52.180556;
                 -4.410000	52.182500;
                 -4.406111	52.184722;
                 -4.401944	52.186667;
                 -4.397778	52.188889;
                 -4.394167	52.191389;
                 -4.390278	52.193333;
                 -4.387778	52.196389;
                 -4.386111	52.200556;
                 -4.383611	52.203611;
                 -4.380833	52.206667;
                 -4.376667	52.208611;
                 -4.371389	52.209722;
                 -4.364444	52.209722;
                 -4.357222	52.209444;
                 -4.351389	52.208056;
                 -4.345556	52.206944;
                 -4.339444	52.207500;
                 -4.333889	52.208611;
                 -4.329444	52.210000;
                 -4.325278	52.212222;
                 -4.320556	52.213611;
                 -4.315556	52.215278;
                 -4.311111	52.216667;
                 -4.306111	52.218333;
                 -4.301389	52.220000;
                 -4.296667	52.221389;
                 -4.291944	52.223056;
                 -4.287778	52.225000;
                 -4.283889	52.227222;
                 -4.280556	52.229722;
                 -4.277222	52.232222;
                 -4.273611	52.234722;
                 -4.270278	52.237500;
                 -4.266944	52.240000;
                 -4.262222	52.241389;
                 -4.257222	52.243056;
                 -4.252778	52.244722;
                 -4.247222	52.245556;
                 -4.241667	52.246667;
                 -4.236389	52.247778;
                 -4.231667	52.249444;
                 -4.228333	52.251944;
                 -4.224722	52.254444;
                 -4.221389	52.256944;
                 -4.218056	52.259444;
                 -4.214722	52.262222;
                 -4.211111	52.264722;
                 -4.206944	52.266667;
                 -4.203611	52.269167;
                 -4.200278	52.271944;
                 -4.196944	52.274444;
                 -4.193333	52.276944;
                 -4.190556	52.280000;
                 -4.187222	52.282500;
                 -4.184444	52.285556;
                 -4.181944	52.288611;
                 -4.179167	52.291667;
                 -4.175000	52.293889;
                 -4.171667	52.296389;
                 -4.168333	52.298889;
                 -4.166111	52.302500;
                 -4.164167	52.306111;
                 -4.161944	52.309444;
                 -4.158611	52.312222;
                 -4.153889	52.313611;
                 -4.149722	52.315833;
                 -4.146111	52.318333;
                 -4.142778	52.320833;
                 -4.140833	52.324444;
                 -4.138056	52.327500;
                 -4.136111	52.329167;
                 -4.133611	52.332222;
                 -4.130833	52.335278;
                 -4.128611	52.338889;
                 -4.126667	52.342222;
                 -4.124444	52.345833;
                 -4.122500	52.349444;
                 -4.120278	52.353056;
                 -4.118333	52.356667;
                 -4.115556	52.359722;
                 -4.113611	52.363056;
                 -4.111389	52.366667;
                 -4.108611	52.369722;
                 -4.105833	52.372778;
                 -4.103333	52.375833;
                 -4.100278	52.378889;
                 -4.098333	52.382500;
                 -4.096944	52.386389;
                 -4.095000	52.390000;
                 -4.093611	52.394167;
                 -4.091944	52.398056;
                 -4.091389	52.402778;
                 -4.090000	52.406667;
                 -4.089167	52.411389;
                 -4.087778	52.415278;
                 -4.087222	52.420000;
                 -4.086389	52.424444;
                 -4.085556	52.429167;
                 -4.084722	52.433611;
                 -4.084167	52.438056;
                 -4.082778	52.442222;
                 -4.080556	52.445833;
                 -4.078611	52.449444;
                 -4.075556	52.452500;
                 -4.073611	52.455833;
                 -4.071667	52.459444;
                 -4.069444	52.463056;
                 -4.067500	52.466667;
                 -4.065556	52.470000;
                 -4.063333	52.473611;
                 -4.061389	52.477222;
                 -4.060556	52.481667;
                 -4.059722	52.486389;
                 -4.060556	52.490000;
                 -4.060833	52.494167;
                 -4.060833	52.498333;
                 -4.060833	52.502500;
                 -4.061111	52.506667;
                 -4.060278	52.511111;
                 -4.059444	52.515833;
                 -4.058611	52.520278;
                 -4.057500	52.524444;
                 -4.053889	52.526944;
                 -4.047222	52.526111;
                 -4.043056	52.523889;
                 -4.038889	52.521944;
                 -4.035278	52.519444;
                 -4.031111	52.518056;
                 -4.025000	52.518611;
                 -4.019444	52.519722;
                 -4.014722	52.521389;
                 -4.010000	52.522778;
                 -4.005000	52.524444;
                 -3.999722	52.525556;
                 -3.994167	52.526389;
                 -3.989444	52.528056;
                 -3.985278	52.530000;
                 -3.980556	52.531667;
                 -3.975833	52.533056;
                 -3.971667	52.535278;
                 -3.966667	52.536667;
                 -3.962778	52.538889;
                 -3.958056	52.540278;
                 -3.953889	52.542500;
                 -3.950556	52.545000;
                 -3.946944	52.547500;
                 -3.945556	52.551667;
                 -3.949722	52.553611;
                 -3.956667	52.553611;
                 -3.962778	52.553056;
                 -3.968889	52.552500;
                 -3.974167	52.551389;
                 -3.979722	52.550556;
                 -3.985000	52.549444;
                 -3.990833	52.548333;
                 -3.996111	52.547222;
                 -4.001389	52.546111;
                 -4.006944	52.545278;
                 -4.011667	52.543611;
                 -4.018056	52.543056;
                 -4.023333	52.541944;
                 -4.028889	52.541111;
                 -4.035000	52.540556;
                 -4.041389	52.540000;
                 -4.048611	52.540278;
                 -4.055556	52.541111;
                 -4.061389	52.542500;
                 -4.066111	52.544167;
                 -4.071389	52.545833;
                 -4.074444	52.548333;
                 -4.077778	52.550833;
                 -4.080556	52.553889;
                 -4.082222	52.556944;
                 -4.083333	52.560833;
                 -4.084722	52.564167;
                 -4.086667	52.567500;
                 -4.089167	52.570278;
                 -4.092500	52.572778;
                 -4.095833	52.575278;
                 -4.099444	52.577778;
                 -4.101944	52.580833;
                 -4.104167	52.583611;
                 -4.106111	52.586944;
                 -4.107778	52.590278;
                 -4.110278	52.593333;
                 -4.113889	52.595833;
                 -4.118611	52.597500;
                 -4.123889	52.599167;
                 -4.126389	52.601944;
                 -4.127222	52.605833;
                 -4.126111	52.609722;
                 -4.123889	52.613333;
                 -4.121944	52.616944;
                 -4.120278	52.620833;
                 -4.118889	52.625000;
                 -4.116667	52.628611;
                 -4.114722	52.632222;
                 -4.113333	52.636111;
                 -4.111111	52.639722;
                 -4.110000	52.643889;
                 -4.107778	52.647222;
                 -4.106111	52.651389;
                 -4.103611	52.654444;
                 -4.100833	52.657500;
                 -4.096667	52.659444;
                 -4.093333	52.661944;
                 -4.089167	52.664167;
                 -4.084167	52.665556;
                 -4.080000	52.667778;
                 -4.076667	52.670278;
                 -4.072500	52.672222;
                 -4.069167	52.675000;
                 -4.065556	52.677500;
                 -4.062222	52.680000;
                 -4.060833	52.684167;
                 -4.059444	52.688056;
                 -4.059444	52.692222;
                 -4.059444	52.696389;
                 -4.058889	52.700000;
                 -4.055556	52.702500;
                 -4.051389	52.704722;
                 -4.044444	52.704722;
                 -4.037778	52.703889;
                 -4.031111	52.703056;
                 -4.027222	52.705000;
                 -4.023056	52.706944;
                 -4.019444	52.709722;
                 -4.015278	52.711667;
                 -4.011667	52.714167;
                 -4.007778	52.716111;
                 -4.004167	52.718889;
                 -4.000278	52.720833;
                 -3.996667	52.723333;
                 -3.993056	52.725833;
                 -3.989722	52.728611;
                 -3.989167	52.732222;
                 -3.994167	52.733889;
                 -4.001111	52.733889;
                 -4.005833	52.732222;
                 -4.010833	52.730556;
                 -4.015000	52.728611;
                 -4.018333	52.726111;
                 -4.021667	52.723611;
                 -4.024444	52.720556;
                 -4.027222	52.717500;
                 -4.031389	52.715278;
                 -4.036111	52.713889;
                 -4.043056	52.713889;
                 -4.048889	52.715000;
                 -4.053056	52.717222;
                 -4.056389	52.719722;
                 -4.059722	52.722222;
                 -4.063333	52.724722;
                 -4.065833	52.727500;
                 -4.068333	52.730556;
                 -4.071667	52.733056;
                 -4.074167	52.735833;
                 -4.076944	52.737778;
                 -4.080278	52.740278;
                 -4.083611	52.742778;
                 -4.087222	52.745278;
                 -4.090556	52.747778;
                 -4.093056	52.750833;
                 -4.095833	52.753611;
                 -4.098333	52.756667;
                 -4.101667	52.759167;
                 -4.104167	52.761944;
                 -4.106667	52.765000;
                 -4.109444	52.767778;
                 -4.112778	52.770278;
                 -4.116944	52.772500;
                 -4.121944	52.774167;
                 -4.126111	52.776111;
                 -4.130278	52.778333;
                 -4.133056	52.781111;
                 -4.133056	52.785278;
                 -4.134722	52.788611;
                 -4.138056	52.791111;
                 -4.142500	52.793333;
                 -4.145833	52.795833;
                 -4.148333	52.798611;
                 -4.149444	52.802500;
                 -4.149444	52.806667;
                 -4.148056	52.810556;
                 -4.145278	52.813611;
                 -4.142500	52.816667;
                 -4.136944	52.817778;
                 -4.132222	52.819167;
                 -4.128056	52.821389;
                 -4.123889	52.823333;
                 -4.121111	52.826389;
                 -4.119167	52.830000;
                 -4.118333	52.834444;
                 -4.119167	52.838333;
                 -4.120000	52.841944;
                 -4.119444	52.846667;
                 -4.119444	52.850833;
                 -4.121944	52.853611;
                 -4.124444	52.856667;
                 -4.128056	52.859167;
                 -4.130556	52.861944;
                 -4.131667	52.865556;
                 -4.133333	52.868889;
                 -4.134167	52.872778;
                 -4.136667	52.875556;
                 -4.140000	52.878056;
                 -4.144167	52.880278;
                 -4.148611	52.882222;
                 -4.150278	52.885556;
                 -4.152222	52.888889;
                 -4.153889	52.892222;
                 -4.156389	52.895278;
                 -4.160000	52.896667;
                 -4.162500	52.899722;
                 -4.166667	52.901667;
                 -4.171944	52.903611;
                 -4.179444	52.903889;
                 -4.185556	52.903333;
                 -4.193056	52.903889;
                 -4.198889	52.905000;
                 -4.204722	52.906389;
                 -4.210000	52.908056;
                 -4.216667	52.908056;
                 -4.222500	52.906944;
                 -4.227778	52.905833;
                 -4.234167	52.905278;
                 -4.240833	52.906111;
                 -4.246944	52.905556;
                 -4.251111	52.903611;
                 -4.255278	52.901389;
                 -4.259444	52.899444;
                 -4.265556	52.898889;
                 -4.273056	52.899167;
                 -4.280833	52.899722;
                 -4.287500	52.899722;
                 -4.294444	52.899444;
                 -4.301389	52.899444;
                 -4.306944	52.898333;
                 -4.311389	52.896944;
                 -4.315556	52.894722;
                 -4.319722	52.892778;
                 -4.323889	52.890833;
                 -4.327500	52.888056;
                 -4.332222	52.886667;
                 -4.339167	52.886389;
                 -4.345833	52.887222;
                 -4.351944	52.888611;
                 -4.358333	52.889444;
                 -4.365833	52.890000;
                 -4.373611	52.890278;
                 -4.379722	52.889722;
                 -4.386111	52.889167;
                 -4.390000	52.887222;
                 -4.393611	52.884444;
                 -4.396944	52.881944;
                 -4.400278	52.879444;
                 -4.404444	52.877500;
                 -4.409444	52.875833;
                 -4.415556	52.875278;
                 -4.421667	52.874722;
                 -4.427222	52.873611;
                 -4.431389	52.871667;
                 -4.435556	52.869444;
                 -4.440278	52.868056;
                 -4.445833	52.866944;
                 -4.450556	52.865278;
                 -4.454167	52.862778;
                 -4.456111	52.859167;
                 -4.458611	52.856111;
                 -4.462778	52.854167;
                 -4.465833	52.850833;
                 -4.464722	52.847222;
                 -4.466111	52.844167;
                 -4.469444	52.841667;
                 -4.473611	52.839444;
                 -4.476944	52.836944;
                 -4.480556	52.834444;
                 -4.483056	52.831389;
                 -4.485833	52.828333;
                 -4.488056	52.824722;
                 -4.489444	52.820556;
                 -4.490000	52.816111;
                 -4.489167	52.812222;
                 -4.486667	52.809444;
                 -4.483056	52.806389;
                 -4.479722	52.803889;
                 -4.476389	52.801389;
                 -4.472778	52.798889;
                 -4.473611	52.795278;
                 -4.476389	52.792222;
                 -4.479722	52.789722;
                 -4.483889	52.787778;
                 -4.491389	52.788056;
                 -4.498333	52.788056;
                 -4.502500	52.785833;
                 -4.505000	52.782778;
                 -4.507778	52.779722;
                 -4.511111	52.777222;
                 -4.515278	52.775278;
                 -4.523056	52.775556;
                 -4.528056	52.777222;
                 -4.531389	52.779722;
                 -4.533333	52.783056;
                 -4.533333	52.787222;
                 -4.532500	52.791667;
                 -4.535833	52.794167;
                 -4.538889	52.796111;
                 -4.543056	52.798333;
                 -4.547222	52.800278;
                 -4.551389	52.802500;
                 -4.555556	52.804444;
                 -4.560833	52.806111;
                 -4.565833	52.807778;
                 -4.570000	52.810000;
                 -4.575833	52.811111;
                 -4.581111	52.812778;
                 -4.586944	52.814167;
                 -4.591944	52.815833;
                 -4.598611	52.816667;
                 -4.605556	52.817500;
                 -4.610278	52.815833;
                 -4.614167	52.813889;
                 -4.618333	52.811667;
                 -4.621111	52.808611;
                 -4.624722	52.805556;
                 -4.628056	52.803056;
                 -4.632222	52.800833;
                 -4.637778	52.800000;
                 -4.643889	52.799167;
                 -4.650556	52.800000;
                 -4.656389	52.801389;
                 -4.662500	52.800833;
                 -4.667500	52.799167;
                 -4.671667	52.797222;
                 -4.675833	52.795000;
                 -4.680556	52.793611;
                 -4.687222	52.794444;
                 -4.692222	52.796389;
                 -4.697222	52.798056;
                 -4.703056	52.799444;
                 -4.709167	52.798611;
                 -4.713333	52.796667;
                 -4.717500	52.794722;
                 -4.719444	52.791111;
                 -4.721667	52.787500;
                 -4.722778	52.783333;
                 -4.727778	52.781944;
                 -4.733889	52.781111;
                 -4.740833	52.781111;
                 -4.747500	52.781944;
                 -4.751667	52.783889;
                 -4.756111	52.786111;
                 -4.759444	52.788611;
                 -4.760000	52.789444;
                 -4.760000	52.789167;
                 -4.758889	52.793056;
                 -4.754722	52.795278;
                 -4.750556	52.797222;
                 -4.750000	52.800833;
                 -4.752778	52.803889;
                 -4.751944	52.807500;
                 -4.749167	52.810556;
                 -4.745833	52.813056;
                 -4.743056	52.816111;
                 -4.741111	52.819722;
                 -4.736944	52.822222;
                 -4.732778	52.824444;
                 -4.728889	52.826389;
                 -4.726111	52.829444;
                 -4.722778	52.831944;
                 -4.721944	52.836667;
                 -4.722778	52.840278;
                 -4.723056	52.844444;
                 -4.721111	52.847222;
                 -4.716944	52.849167;
                 -4.710833	52.849722;
                 -4.704722	52.850278;
                 -4.699167	52.851389;
                 -4.695833	52.854167;
                 -4.693611	52.857500;
                 -4.691667	52.861111;
                 -4.688889	52.864167;
                 -4.684722	52.866389;
                 -4.680000	52.867778;
                 -4.673056	52.868056;
                 -4.668333	52.869444;
                 -4.666389	52.873056;
                 -4.665556	52.877778;
                 -4.662222	52.880278;
                 -4.658056	52.882222;
                 -4.654444	52.885000;
                 -4.651111	52.887500;
                 -4.649167	52.891111;
                 -4.647778	52.895000;
                 -4.644167	52.897500;
                 -4.637500	52.897778;
                 -4.631389	52.898333;
                 -4.627222	52.900278;
                 -4.624444	52.903333;
                 -4.621667	52.906389;
                 -4.618889	52.909444;
                 -4.617778	52.913611;
                 -4.614722	52.916667;
                 -4.611111	52.916944;
                 -4.604722	52.917500;
                 -4.599444	52.918611;
                 -4.594444	52.920278;
                 -4.590278	52.922222;
                 -4.586111	52.924167;
                 -4.582778	52.926944;
                 -4.579444	52.929444;
                 -4.576667	52.932500;
                 -4.572500	52.934444;
                 -4.567222	52.934722;
                 -4.561944	52.933056;
                 -4.557778	52.931111;
                 -4.552778	52.929444;
                 -4.545556	52.930000;
                 -4.540278	52.930833;
                 -4.535278	52.932500;
                 -4.531389	52.934722;
                 -4.526667	52.936111;
                 -4.522500	52.938333;
                 -4.518333	52.940278;
                 -4.514167	52.942500;
                 -4.510000	52.944444;
                 -4.505833	52.946389;
                 -4.502500	52.949167;
                 -4.498333	52.951111;
                 -4.495000	52.953611;
                 -4.490833	52.955833;
                 -4.486667	52.957778;
                 -4.483056	52.960278;
                 -4.478889	52.962500;
                 -4.474722	52.964444;
                 -4.470833	52.966667;
                 -4.466667	52.968611;
                 -4.462500	52.970833;
                 -4.458889	52.973333;
                 -4.454722	52.975278;
                 -4.451389	52.977778;
                 -4.447222	52.980000;
                 -4.443056	52.981944;
                 -4.439722	52.984722;
                 -4.435556	52.986667;
                 -4.432222	52.989167;
                 -4.428056	52.991389;
                 -4.424444	52.993889;
                 -4.420278	52.995833;
                 -4.416944	52.998611;
                 -4.413333	53.001111;
                 -4.409444	53.003056;
                 -4.405278	53.005278;
                 -4.401944	53.007778;
                 -4.397778	53.009722;
                 -4.393611	53.011944;
                 -4.390000	53.014444;
                 -4.385833	53.016389;
                 -4.381667	53.018611;
                 -4.377500	53.020556;
                 -4.372500	53.022500;
                 -4.368333	53.024722;
                 -4.364167	53.026667;
                 -4.360833	53.029444;
                 -4.357500	53.031944;
                 -4.354722	53.035000;
                 -4.352500	53.038611;
                 -4.350556	53.041944;
                 -4.349167	53.046111;
                 -4.348333	53.050556;
                 -4.346944	53.054722;
                 -4.345556	53.058889;
                 -4.344722	53.063333;
                 -4.343889	53.067778;
                 -4.344167	53.071944;
                 -4.344167	53.076111;
                 -4.343611	53.080833;
                 -4.343889	53.085000;
                 -4.344722	53.088611;
                 -4.344722	53.092778;
                 -4.345000	53.096944;
                 -4.344167	53.101389;
                 -4.344167	53.105556;
                 -4.343056	53.109722;
                 -4.339444	53.112222;
                 -4.333889	53.113333;
                 -4.330000	53.112500;
                 -4.327500	53.109722;
                 -4.325556	53.106389;
                 -4.323889	53.103056;
                 -4.320556	53.100556;
                 -4.314444	53.100278;
                 -4.309444	53.101667;
                 -4.306944	53.104722;
                 -4.305556	53.108889;
                 -4.303889	53.113056;
                 -4.303333	53.117500;
                 -4.301944	53.121667;
                 -4.299167	53.124722;
                 -4.295556	53.127222;
                 -4.291667	53.128333;
                 -4.286944	53.130000;
                 -4.282778	53.131944;
                 -4.279167	53.134444;
                 -4.275556	53.136944;
                 -4.273056	53.140000;
                 -4.269444	53.142778;
                 -4.265833	53.145278;
                 -4.262500	53.147778;
                 -4.259167	53.150278;
                 -4.255556	53.152778;
                 -4.252222	53.155556;
                 -4.248611	53.158056;
                 -4.245000	53.160556;
                 -4.240833	53.162500;
                 -4.236111	53.164167;
                 -4.231944	53.166111;
                 -4.226944	53.167778;
                 -4.222778	53.169722;
                 -4.218056	53.171389;
                 -4.214722	53.173889;
                 -4.211111	53.176389;
                 -4.208333	53.179444;
                 -4.206667	53.183611;
                 -4.205556	53.187778;
                 -4.204722	53.192222;
                 -4.203333	53.196389;
                 -4.201944	53.200278;
                 -4.198889	53.203333;
                 -4.196389	53.206389;
                 -4.192222	53.208611;
                 -4.188056	53.210556;
                 -4.183889	53.212500;
                 -4.178889	53.214167;
                 -4.174722	53.216111;
                 -4.170000	53.217778;
                 -4.166389	53.220278;
                 -4.161667	53.221944;
                 -4.157222	53.223889;
                 -4.153056	53.226111;
                 -4.148333	53.227500;
                 -4.142778	53.228611;
                 -4.138056	53.230278;
                 -4.131111	53.230278;
                 -4.125000	53.228889;
                 -4.119167	53.227778;
                 -4.111667	53.227222;
                 -4.105278	53.227778;
                 -4.100278	53.229444;
                 -4.096111	53.231389;
                 -4.089444	53.231389;
                 -4.084167	53.229722;
                 -4.078333	53.228611;
                 -4.072222	53.227222;
                 -4.066111	53.226944;
                 -4.060833	53.228056;
                 -4.055833	53.229444;
                 -4.051111	53.231111;
                 -4.046111	53.232500;
                 -4.040556	53.233611;
                 -4.035000	53.234722;
                 -4.029444	53.235833;
                 -4.023889	53.236667;
                 -4.018333	53.237778;
                 -4.013333	53.239444;
                 -4.007778	53.240833;
                 -4.003056	53.242500;
                 -3.998889	53.244444;
                 -3.994722	53.246389;
                 -3.990556	53.248611;
                 -3.985556	53.250000;
                 -3.981389	53.252222;
                 -3.977222	53.254167;
                 -3.972778	53.256389;
                 -3.968056	53.257778;
                 -3.963056	53.259444;
                 -3.959167	53.260556;
                 -3.954444	53.261944;
                 -3.949444	53.263611;
                 -3.944444	53.265000;
                 -3.938889	53.266111;
                 -3.934167	53.267778;
                 -3.928611	53.268889;
                 -3.923056	53.269722;
                 -3.916667	53.270278;
                 -3.911111	53.271389;
                 -3.905833	53.272500;
                 -3.900833	53.273889;
                 -3.896667	53.276111;
                 -3.893056	53.278611;
                 -3.888889	53.280556;
                 -3.882500	53.281111;
                 -3.875000	53.280833;
                 -3.868889	53.280278;
                 -3.863889	53.281944;
                 -3.860000	53.283889;
                 -3.854167	53.285000;
                 -3.847778	53.284167;
                 -3.840833	53.283333;
                 -3.833333	53.282778;
                 -3.829167	53.284722;
                 -3.828333	53.288333;
                 -3.830000	53.291667;
                 -3.833611	53.294167;
                 -3.836111	53.297222;
                 -3.838611	53.300000;
                 -3.840278	53.303333;
                 -3.841389	53.307222;
                 -3.843056	53.310556;
                 -3.846389	53.313056;
                 -3.849722	53.315556;
                 -3.854167	53.317500;
                 -3.859444	53.319167;
                 -3.863611	53.321389;
                 -3.868611	53.323056;
                 -3.872778	53.325278;
                 -3.876111	53.327778;
                 -3.877222	53.331389;
                 -3.873889	53.333889;
                 -3.867500	53.334444;
                 -3.861667	53.333333;
                 -3.858611	53.332222;
                 -3.853611	53.330556;
                 -3.847778	53.329167;
                 -3.841667	53.327778;
                 -3.834722	53.328056;
                 -3.830278	53.326667;
                 -3.825833	53.324722;
                 -3.822500	53.322222;
                 -3.818333	53.320000;
                 -3.813889	53.318056;
                 -3.807500	53.316944;
                 -3.801111	53.317500;
                 -3.795278	53.318611;
                 -3.789722	53.319722;
                 -3.783611	53.320278;
                 -3.777500	53.318889;
                 -3.774444	53.316389;
                 -3.770833	53.313889;
                 -3.767222	53.311389;
                 -3.763889	53.308889;
                 -3.759722	53.306667;
                 -3.753889	53.305556;
                 -3.747778	53.304167;
                 -3.743611	53.302222;
                 -3.740278	53.299444;
                 -3.737500	53.296667;
                 -3.735833	53.293333;
                 -3.732500	53.290833;
                 -3.727500	53.289167;
                 -3.720833	53.288333;
                 -3.713611	53.288333;
                 -3.706111	53.287778;
                 -3.698611	53.287222;
                 -3.692500	53.286111;
                 -3.685833	53.285278;
                 -3.678889	53.284167;
                 -3.671389	53.283889;
                 -3.664444	53.283889;
                 -3.657500	53.283889;
                 -3.651944	53.285000;
                 -3.645000	53.285000;
                 -3.638056	53.283889;
                 -3.632222	53.282778;
                 -3.626389	53.281389;
                 -3.619722	53.280556;
                 -3.612778	53.279722;
                 -3.606667	53.280278;
                 -3.601111	53.281111;
                 -3.594722	53.281667;
                 -3.589167	53.282778;
                 -3.584444	53.284167;
                 -3.579444	53.285833;
                 -3.574722	53.287222;
                 -3.571944	53.289444;
                 -3.566389	53.290556;
                 -3.561389	53.291944;
                 -3.557222	53.294167;
                 -3.553056	53.296111;
                 -3.548889	53.298056;
                 -3.543889	53.299722;
                 -3.539444	53.301667;
                 -3.534167	53.302778;
                 -3.529167	53.304167;
                 -3.524444	53.305833;
                 -3.518889	53.306944;
                 -3.513889	53.308333;
                 -3.508333	53.309444;
                 -3.503333	53.310833;
                 -3.498611	53.312500;
                 -3.493611	53.313889;
                 -3.488889	53.315556;
                 -3.483889	53.316944;
                 -3.478889	53.318611;
                 -3.472778	53.319167;
                 -3.466389	53.319444;
                 -3.461389	53.321111;
                 -3.456667	53.322500;
                 -3.452500	53.324722;
                 -3.448333	53.326667;
                 -3.443889	53.328611;
                 -3.437778	53.329167;
                 -3.432222	53.330278;
                 -3.426389	53.331389;
                 -3.420833	53.332222;
                 -3.416111	53.333889;
                 -3.411111	53.335278;
                 -3.406389	53.336944;
                 -3.402222	53.338889;
                 -3.397222	53.340556;
                 -3.392778	53.342500;
                 -3.388056	53.343889;
                 -3.381667	53.344444;
                 -3.376111	53.345556;
                 -3.370000	53.346111;
                 -3.362778	53.346389;
                 -3.355278	53.346111;
                 -3.348889	53.346389;
                 -3.342778	53.346944;
                 -3.336389	53.347500;
                 -3.329722	53.347500;
                 -3.322500	53.347500;
                 -3.315000	53.346944;
                 -3.311389	53.345278;
                 -3.307222	53.343333;
                 -3.303056	53.341111;
                 -3.298611	53.339167;
                 -3.295278	53.336667;
                 -3.291944	53.333889;
                 -3.287500	53.331944;
                 -3.283333	53.329722;
                 -3.279167	53.327778;
                 -3.275000	53.325556;
                 -3.270833	53.323333;
                 -3.266667	53.321389;
                 -3.263056	53.318889;
                 -3.258889	53.316667;
                 -3.253889	53.315000;
                 -3.249444	53.312778;
                 -3.245278	53.310556;
                 -3.241111	53.308611;
                 -3.236111	53.306944;
                 -3.231944	53.304722;
                 -3.226944	53.303056;
                 -3.221667	53.301389;
                 -3.216667	53.299444;
                 -3.211389	53.297778;
                 -3.207222	53.295833;
                 -3.203056	53.293611;
                 -3.198611	53.291389;
                 -3.193889	53.289722;
                 -3.189444	53.287778;
                 -3.184444	53.285833;
                 -3.180278	53.283889;
                 -3.175000	53.282222;
                 -3.170833	53.280000;
                 -3.166667	53.277778;
                 -3.163333	53.275278;
                 -3.160000	53.272778;
                 -3.156389	53.270278;
                 -3.151667	53.268611;
                 -3.146389	53.266944;
                 -3.142222	53.264722;
                 -3.138056	53.262500;
                 -3.133889	53.260556;
                 -3.130278	53.257778;
                 -3.125556	53.256111;
                 -3.121111	53.254167;
                 -3.115278	53.252778;
                 -3.110000	53.251111;
                 -3.106667	53.248611;
                 -3.103611	53.245833;
                 -3.101944	53.242500;
                 -3.099167	53.239722;
                 -3.095833	53.237222;
                 -3.093889	53.235278;
                 -3.090556	53.232778;
                 -3.085556	53.230833;
                 -3.079722	53.229722;
                 -3.073333	53.230000;
                 -3.068333	53.231667;
                 -3.067222	53.234722;
                 -3.069444	53.237778;
                 -3.071389	53.241111;
                 -3.071944	53.244722;
                 -3.070556	53.248889;
                 -3.069444	53.253333;
                 -3.071389	53.256667;
                 -3.070556	53.261111;
                 -3.067500	53.264167;
                 -3.065556	53.267778;
                 -3.064722	53.271389;
                 -3.067500	53.274444;
                 -3.069722	53.277222;
                 -3.075556	53.278611;
                 -3.080833	53.280278;
                 -3.085000	53.282500;
                 -3.088333	53.285000;
                 -3.090833	53.288056;
                 -3.094167	53.290556;
                 -3.098333	53.292500;
                 -3.099444	53.296389;
                 -3.096389	53.299444;
                 -3.092778	53.301944;
                 -3.092222	53.305556;
                 -3.095556	53.308056;
                 -3.099722	53.310278;
                 -3.104722	53.311944;
                 -3.107500	53.314722;
                 -3.110000	53.317778;
                 -3.112500	53.320833;
                 -3.115000	53.323611;
                 -3.119167	53.325833;
                 -3.123611	53.327778;
                 -3.128333	53.329722;
                 -3.132778	53.331667;
                 -3.137778	53.333333;
                 -3.142222	53.335556;
                 -3.145278	53.338056;
                 -3.148056	53.341111;
                 -3.151389	53.343611;
                 -3.154722	53.346111;
                 -3.158333	53.348611;
                 -3.161389	53.351111;
                 -3.164722	53.353611;
                 -3.168333	53.356111;
                 -3.171667	53.358611;
                 -3.174167	53.361667;
                 -3.177500	53.364167;
                 -3.180833	53.366667;
                 -3.184444	53.369167;
                 -3.186667	53.372222;
                 -3.189444	53.375000;
                 -3.191111	53.378333;
                 -3.192778	53.381667;
                 -3.192778	53.385833;
                 -3.191111	53.390000;
                 -3.188611	53.393056;
                 -3.184722	53.395556;
                 -3.181111	53.398056;
                 -3.176944	53.400000;
                 -3.172778	53.402222;
                 -3.168333	53.404167;
                 -3.163611	53.405556;
                 -3.158056	53.406667;
                 -3.152222	53.407778;
                 -3.146667	53.408611;
                 -3.141944	53.410278;
                 -3.138056	53.410278;
                 -3.131944	53.410833;
                 -3.126111	53.411944;
                 -3.120556	53.412778;
                 -3.115000	53.413889;
                 -3.110556	53.415833;
                 -3.105833	53.417500;
                 -3.101667	53.419444;
                 -3.096667	53.420833;
                 -3.092222	53.423056;
                 -3.087500	53.424444;
                 -3.082500	53.426111;
                 -3.077500	53.427500;
                 -3.071944	53.428611;
                 -3.067222	53.430000;
                 -3.060833	53.430556;
                 -3.054444	53.431111;
                 -3.048333	53.431389;
                 -3.040556	53.431111;
                 -3.033611	53.430000;
                 -3.027778	53.428611;
                 -3.023611	53.426667;
                 -3.020278	53.424167;
                 -3.018611	53.420833;
                 -3.017778	53.416944;
                 -3.016944	53.413333;
                 -3.015000	53.410000;
                 -3.013611	53.406667;
                 -3.010278	53.404167;
                 -3.007500	53.401111;
                 -3.005000	53.398056;
                 -3.002778	53.395278;
                 -3.000000	53.392222;
                 -2.998333	53.388889;
                 -2.995000	53.386389;
                 -2.993333	53.383056;
                 -2.993333	53.378889;
                 -2.994167	53.374444;
                 -2.992500	53.371111;
                 -2.990000	53.368056;
                 -2.987500	53.365278;
                 -2.984167	53.362778;
                 -2.980833	53.360000;
                 -2.977222	53.357500;
                 -2.974722	53.354722;
                 -2.972500	53.351667;
                 -2.969722	53.348889;
                 -2.967222	53.345833;
                 -2.964722	53.342778;
                 -2.962222	53.340000;
                 -2.959722	53.336944;
                 -2.957222	53.334167;
                 -2.954722	53.331111;
                 -2.952222	53.328333;
                 -2.949722	53.325278;
                 -2.948056	53.321944;
                 -2.944444	53.319444;
                 -2.942222	53.316389;
                 -2.938611	53.313889;
                 -2.935278	53.311389;
                 -2.931111	53.309167;
                 -2.927778	53.306667;
                 -2.922500	53.305000;
                 -2.920000	53.303056;
                 -2.915833	53.300833;
                 -2.911667	53.298611;
                 -2.907500	53.296667;
                 -2.902222	53.294722;
                 -2.897222	53.293056;
                 -2.892222	53.291389;
                 -2.885556	53.290556;
                 -2.880278	53.288611;
                 -2.873611	53.287778;
                 -2.868611	53.286111;
                 -2.862500	53.284722;
                 -2.856667	53.283333;
                 -2.850833	53.281944;
                 -2.844167	53.281111;
                 -2.838056	53.279722;
                 -2.831389	53.279722;
                 -2.824444	53.279722;
                 -2.818056	53.280278;
                 -2.812500	53.281389;
                 -2.806944	53.282222;
                 -2.801944	53.283611;
                 -2.797778	53.285833;
                 -2.793333	53.287778;
                 -2.789444	53.289722;
                 -2.785556	53.292222;
                 -2.782222	53.294722;
                 -2.778611	53.297222;
                 -2.774444	53.299167;
                 -2.769167	53.300833;
                 -2.764444	53.302222;
                 -2.759444	53.303889;
                 -2.754722	53.305278;
                 -2.751667	53.308333;
                 -2.750833	53.312778;
                 -2.752500	53.316111;
                 -2.755000	53.319167;
                 -2.755000	53.323333;
                 -2.751944	53.326389;
                 -2.748889	53.329444;
                 -2.745000	53.331389;
                 -2.741111	53.333889;
                 -2.738333	53.336944;
                 -2.735278	53.340000;
                 -2.736389	53.343611;
                 -2.740556	53.345833;
                 -2.746111	53.344722;
                 -2.751111	53.343333;
                 -2.755833	53.341667;
                 -2.760278	53.339722;
                 -2.763611	53.337222;
                 -2.768056	53.335278;
                 -2.771389	53.332778;
                 -2.775833	53.330833;
                 -2.780000	53.328611;
                 -2.785556	53.327778;
                 -2.791389	53.326667;
                 -2.798889	53.327222;
                 -2.806389	53.327778;
                 -2.812500	53.329167;
                 -2.820000	53.329722;
                 -2.826944	53.329722;
                 -2.833889	53.329167;
                 -2.841667	53.329722;
                 -2.848611	53.329722;
                 -2.855278	53.330833;
                 -2.861111	53.332222;
                 -2.866111	53.333889;
                 -2.870278	53.335833;
                 -2.874722	53.338056;
                 -2.878056	53.340556;
                 -2.881389	53.343056;
                 -2.885556	53.345278;
                 -2.890556	53.346944;
                 -2.896389	53.348333;
                 -2.901667	53.350000;
                 -2.906667	53.351944;
                 -2.911944	53.353611;
                 -2.916667	53.355278;
                 -2.921111	53.357500;
                 -2.925278	53.359444;
                 -2.929444	53.361667;
                 -2.933889	53.363889;
                 -2.937222	53.366389;
                 -2.942222	53.368056;
                 -2.947222	53.369722;
                 -2.953056	53.371111;
                 -2.958333	53.372778;
                 -2.961667	53.374444;
                 -2.966667	53.376111;
                 -2.970278	53.378889;
                 -2.973611	53.381389;
                 -2.976944	53.383889;
                 -2.980556	53.386389;
                 -2.982778	53.389167;
                 -2.986389	53.391944;
                 -2.988889	53.394722;
                 -2.991111	53.397778;
                 -2.993056	53.401111;
                 -2.996389	53.403611;
                 -2.998889	53.406389;
                 -3.000556	53.409722;
                 -3.002222	53.413056;
                 -3.003889	53.416389;
                 -3.005000	53.420278;
                 -3.005556	53.423889;
                 -3.006389	53.427778;
                 -3.007222	53.431389;
                 -3.009167	53.434722;
                 -3.011389	53.437778;
                 -3.013889	53.440556;
                 -3.016667	53.443611;
                 -3.019167	53.446667;
                 -3.019167	53.450833;
                 -3.019722	53.454444;
                 -3.021389	53.457778;
                 -3.025000	53.460278;
                 -3.028333	53.462778;
                 -3.032500	53.465000;
                 -3.036944	53.467222;
                 -3.040000	53.469722;
                 -3.043333	53.472222;
                 -3.045278	53.475556;
                 -3.047778	53.478333;
                 -3.048889	53.481389;
                 -3.050556	53.484722;
                 -3.051389	53.488333;
                 -3.053056	53.491667;
                 -3.053611	53.495556;
                 -3.055556	53.498889;
                 -3.057222	53.502222;
                 -3.059722	53.505000;
                 -3.063333	53.507500;
                 -3.066667	53.510000;
                 -3.069722	53.512500;
                 -3.073333	53.515278;
                 -3.076667	53.517778;
                 -3.080000	53.520278;
                 -3.083611	53.522778;
                 -3.086944	53.525278;
                 -3.089444	53.528333;
                 -3.092778	53.531111;
                 -3.095278	53.534167;
                 -3.097778	53.536944;
                 -3.100278	53.540000;
                 -3.102778	53.543056;
                 -3.105556	53.545833;
                 -3.107222	53.549167;
                 -3.107778	53.553056;
                 -3.107778	53.557222;
                 -3.106389	53.561111;
                 -3.104167	53.564722;
                 -3.101944	53.568333;
                 -3.099167	53.571111;
                 -3.096944	53.574722;
                 -3.093889	53.577778;
                 -3.091111	53.580833;
                 -3.088056	53.583889;
                 -3.085278	53.586944;
                 -3.082222	53.590000;
                 -3.079444	53.593056;
                 -3.075556	53.595833;
                 -3.072778	53.598889;
                 -3.069444	53.601389;
                 -3.066389	53.604444;
                 -3.064167	53.608056;
                 -3.061389	53.611111;
                 -3.058333	53.614167;
                 -3.055556	53.617222;
                 -3.052500	53.620278;
                 -3.048889	53.622778;
                 -3.045833	53.625833;
                 -3.042222	53.628611;
                 -3.039444	53.631667;
                 -3.036389	53.634722;
                 -3.033611	53.637778;
                 -3.030833	53.640833;
                 -3.027222	53.643333;
                 -3.024167	53.646389;
                 -3.021111	53.649444;
                 -3.017500	53.651944;
                 -3.014722	53.655000;
                 -3.011667	53.658056;
                 -3.008333	53.660556;
                 -3.005278	53.663611;
                 -3.001667	53.666111;
                 -2.998056	53.668611;
                 -2.994722	53.671111;
                 -2.990833	53.673611;
                 -2.987222	53.676111;
                 -2.983611	53.678611;
                 -2.980278	53.681111;
                 -2.976667	53.683611;
                 -2.971667	53.685278;
                 -2.965278	53.685556;
                 -2.958611	53.684722;
                 -2.951667	53.683889;
                 -2.946667	53.685278;
                 -2.942500	53.687222;
                 -2.938611	53.689722;
                 -2.934444	53.691944;
                 -2.930833	53.694444;
                 -2.927222	53.696944;
                 -2.924444	53.700000;
                 -2.920556	53.702500;
                 -2.918056	53.705556;
                 -2.914444	53.708056;
                 -2.910000	53.710000;
                 -2.906389	53.712500;
                 -2.902222	53.714444;
                 -2.896944	53.716111;
                 -2.892222	53.717500;
                 -2.887222	53.718889;
                 -2.881667	53.720000;
                 -2.876389	53.721389;
                 -2.871667	53.723056;
                 -2.865833	53.723889;
                 -2.860278	53.725000;
                 -2.854722	53.726111;
                 -2.849722	53.727500;
                 -2.847500	53.731111;
                 -2.843333	53.733056;
                 -2.839722	53.735556;
                 -2.839167	53.738333;
                 -2.845278	53.738611;
                 -2.850833	53.737778;
                 -2.855833	53.736111;
                 -2.861389	53.735278;
                 -2.866389	53.733611;
                 -2.871111	53.733056;
                 -2.877500	53.732500;
                 -2.884444	53.732778;
                 -2.890000	53.731667;
                 -2.896389	53.731111;
                 -2.902778	53.730556;
                 -2.909722	53.731667;
                 -2.914722	53.733333;
                 -2.920000	53.735000;
                 -2.926667	53.736111;
                 -2.933611	53.736111;
                 -2.940000	53.735556;
                 -2.944444	53.733611;
                 -2.948611	53.731389;
                 -2.952778	53.729444;
                 -2.958889	53.730000;
                 -2.965000	53.730278;
                 -2.970833	53.729444;
                 -2.975833	53.727778;
                 -2.982222	53.727222;
                 -2.988056	53.728611;
                 -2.992500	53.730833;
                 -2.998889	53.730278;
                 -3.004444	53.729167;
                 -3.010833	53.728889;
                 -3.015556	53.730556;
                 -3.021667	53.731944;
                 -3.025833	53.733889;
                 -3.030278	53.736111;
                 -3.033611	53.738611;
                 -3.037778	53.740833;
                 -3.041389	53.743333;
                 -3.044722	53.745833;
                 -3.046667	53.747222;
                 -3.046667	53.748333;
                 -3.049444	53.749444;
                 -3.051944	53.752222;
                 -3.054444	53.755278;
                 -3.057222	53.758056;
                 -3.058889	53.761389;
                 -3.059722	53.765278;
                 -3.060556	53.768889;
                 -3.062222	53.772222;
                 -3.062222	53.776389;
                 -3.063056	53.780278;
                 -3.063611	53.784444;
                 -3.063611	53.788611;
                 -3.063611	53.792778;
                 -3.063056	53.797222;
                 -3.061389	53.801389;
                 -3.059722	53.805278;
                 -3.058056	53.809444;
                 -3.057500	53.813889;
                 -3.055833	53.818056;
                 -3.055000	53.822500;
                 -3.055000	53.826667;
                 -3.055000	53.830833;
                 -3.055000	53.835000;
                 -3.055000	53.839167;
                 -3.053889	53.843611;
                 -3.052500	53.847778;
                 -3.050833	53.851667;
                 -3.050000	53.856389;
                 -3.049167	53.860833;
                 -3.047500	53.865000;
                 -3.046667	53.869444;
                 -3.045278	53.873333;
                 -3.044167	53.878056;
                 -3.044167	53.882222;
                 -3.045278	53.885833;
                 -3.045833	53.889722;
                 -3.046667	53.893333;
                 -3.047500	53.897222;
                 -3.048333	53.900833;
                 -3.049444	53.904722;
                 -3.049444	53.908611;
                 -3.048333	53.913333;
                 -3.047500	53.916944;
                 -3.045278	53.920556;
                 -3.042500	53.923611;
                 -3.038333	53.925556;
                 -3.034722	53.928056;
                 -3.029167	53.928889;
                 -3.021944	53.928889;
                 -3.015000	53.928889;
                 -3.009167	53.930000;
                 -3.003056	53.930556;
                 -2.996944	53.929167;
                 -2.990000	53.928333;
                 -2.984444	53.929167;
                 -2.979167	53.930833;
                 -2.974444	53.932222;
                 -2.969167	53.933889;
                 -2.964444	53.935278;
                 -2.958611	53.936389;
                 -2.952778	53.937222;
                 -2.947222	53.938333;
                 -2.940833	53.938889;
                 -2.936111	53.940278;
                 -2.930833	53.941667;
                 -2.926111	53.943333;
                 -2.920278	53.944167;
                 -2.916389	53.944444;
                 -2.910278	53.945000;
                 -2.903056	53.943889;
                 -2.898889	53.941944;
                 -2.895556	53.939444;
                 -2.893889	53.936111;
                 -2.888611	53.934167;
                 -2.884167	53.936389;
                 -2.881944	53.939722;
                 -2.877778	53.941944;
                 -2.873611	53.943889;
                 -2.870000	53.946389;
                 -2.867500	53.950000;
                 -2.864444	53.952778;
                 -2.861111	53.955556;
                 -2.855833	53.956944;
                 -2.849722	53.957500;
                 -2.845278	53.959444;
                 -2.841667	53.961944;
                 -2.840000	53.966111;
                 -2.841667	53.969444;
                 -2.848611	53.970278;
                 -2.854167	53.969167;
                 -2.860000	53.968333;
                 -2.866111	53.968611;
                 -2.868056	53.971944;
                 -2.866111	53.976111;
                 -2.863333	53.979167;
                 -2.862500	53.983611;
                 -2.860000	53.987222;
                 -2.855833	53.989167;
                 -2.852222	53.991667;
                 -2.848611	53.994167;
                 -2.847778	53.997778;
                 -2.849722	54.001111;
                 -2.854722	54.002778;
                 -2.861111	54.002500;
                 -2.866111	54.000833;
                 -2.869722	53.998333;
                 -2.873333	53.995833;
                 -2.877778	53.993889;
                 -2.883333	53.992778;
                 -2.887778	53.995000;
                 -2.891111	53.997500;
                 -2.892778	54.000833;
                 -2.892778	54.005000;
                 -2.891111	54.009167;
                 -2.890000	54.013611;
                 -2.892778	54.016667;
                 -2.896944	54.018611;
                 -2.901389	54.020833;
                 -2.904722	54.023333;
                 -2.907500	54.026389;
                 -2.910000	54.029167;
                 -2.911667	54.032500;
                 -2.911667	54.036667;
                 -2.908056	54.039167;
                 -2.905000	54.042222;
                 -2.902222	54.045278;
                 -2.899167	54.048333;
                 -2.896111	54.051389;
                 -2.893056	54.054444;
                 -2.889722	54.056944;
                 -2.886667	54.060000;
                 -2.883611	54.063056;
                 -2.880556	54.065833;
                 -2.877778	54.068889;
                 -2.873889	54.071389;
                 -2.869167	54.073056;
                 -2.862778	54.073611;
                 -2.856944	54.074444;
                 -2.851944	54.076111;
                 -2.847778	54.078056;
                 -2.843889	54.080556;
                 -2.840278	54.083056;
                 -2.835556	54.084444;
                 -2.830278	54.086111;
                 -2.823889	54.086389;
                 -2.819167	54.088056;
                 -2.814722	54.090000;
                 -2.812500	54.093611;
                 -2.813333	54.097222;
                 -2.813333	54.101389;
                 -2.811389	54.105556;
                 -2.808611	54.108611;
                 -2.805556	54.111389;
                 -2.801944	54.114167;
                 -2.799167	54.116944;
                 -2.796944	54.120556;
                 -2.799444	54.123611;
                 -2.801389	54.125833;
                 -2.805556	54.128056;
                 -2.809722	54.130278;
                 -2.812222	54.133056;
                 -2.813333	54.136944;
                 -2.815000	54.140278;
                 -2.816667	54.143611;
                 -2.819167	54.146667;
                 -2.821667	54.149444;
                 -2.825000	54.151944;
                 -2.827500	54.155000;
                 -2.831111	54.157500;
                 -2.834444	54.160000;
                 -2.837222	54.163056;
                 -2.839722	54.165833;
                 -2.842222	54.168889;
                 -2.844722	54.171667;
                 -2.846389	54.175278;
                 -2.848056	54.178611;
                 -2.847222	54.183056;
                 -2.844167	54.186111;
                 -2.839444	54.187500;
                 -2.833056	54.187222;
                 -2.826111	54.186111;
                 -2.820278	54.187222;
                 -2.818056	54.190833;
                 -2.819722	54.194167;
                 -2.822500	54.196944;
                 -2.826667	54.199167;
                 -2.833611	54.200000;
                 -2.840000	54.199722;
                 -2.845833	54.198611;
                 -2.851667	54.197500;
                 -2.856389	54.196111;
                 -2.861944	54.195000;
                 -2.868611	54.194722;
                 -2.875000	54.194167;
                 -2.880000	54.192500;
                 -2.883611	54.190000;
                 -2.887222	54.187500;
                 -2.890833	54.185000;
                 -2.895278	54.183056;
                 -2.900833	54.181944;
                 -2.906111	54.180556;
                 -2.909722	54.178056;
                 -2.913333	54.175556;
                 -2.916389	54.172500;
                 -2.918056	54.168611;
                 -2.919444	54.164444;
                 -2.921667	54.160833;
                 -2.924444	54.157778;
                 -2.928333	54.155278;
                 -2.934167	54.154444;
                 -2.938889	54.152778;
                 -2.944722	54.151944;
                 -2.951111	54.151389;
                 -2.956667	54.150278;
                 -2.962500	54.149444;
                 -2.968889	54.148889;
                 -2.976667	54.149444;
                 -2.982778	54.150833;
                 -2.987778	54.152500;
                 -2.993056	54.154167;
                 -2.996667	54.156667;
                 -2.997222	54.160556;
                 -2.997222	54.164722;
                 -2.997222	54.168611;
                 -2.998056	54.172500;
                 -3.002500	54.174722;
                 -3.007500	54.176389;
                 -3.012778	54.178056;
                 -3.016111	54.180556;
                 -3.019444	54.183056;
                 -3.022222	54.186111;
                 -3.025000	54.188889;
                 -3.027222	54.191944;
                 -3.032500	54.193611;
                 -3.037500	54.192222;
                 -3.040556	54.189167;
                 -3.043333	54.186111;
                 -3.046389	54.183056;
                 -3.049444	54.180000;
                 -3.049444	54.175833;
                 -3.049444	54.171667;
                 -3.049444	54.167500;
                 -3.051111	54.163611;
                 -3.053333	54.160000;
                 -3.056111	54.156944;
                 -3.059722	54.154444;
                 -3.063611	54.151944;
                 -3.067222	54.149444;
                 -3.069444	54.145833;
                 -3.069444	54.141667;
                 -3.069444	54.137500;
                 -3.072222	54.134444;
                 -3.075833	54.131944;
                 -3.079722	54.129444;
                 -3.081111	54.125556;
                 -3.083611	54.121944;
                 -3.086389	54.118889;
                 -3.090000	54.116389;
                 -3.094167	54.114444;
                 -3.099444	54.112778;
                 -3.103611	54.110833;
                 -3.106667	54.107778;
                 -3.106667	54.103611;
                 -3.109722	54.100556;
                 -3.115278	54.099722;
                 -3.120000	54.098056;
                 -3.125278	54.096667;
                 -3.130000	54.095000;
                 -3.135278	54.093611;
                 -3.140000	54.092222;
                 -3.145278	54.090556;
                 -3.149444	54.088611;
                 -3.153889	54.086667;
                 -3.155278	54.082500;
                 -3.151944	54.080000;
                 -3.148333	54.077500;
                 -3.145000	54.075000;
                 -3.141667	54.072500;
                 -3.139167	54.069444;
                 -3.137500	54.066111;
                 -3.138056	54.061667;
                 -3.143889	54.061389;
                 -3.148056	54.063611;
                 -3.151389	54.066111;
                 -3.154722	54.068611;
                 -3.160278	54.070278;
                 -3.165278	54.071944;
                 -3.169444	54.074167;
                 -3.172778	54.076667;
                 -3.174722	54.080000;
                 -3.175556	54.083889;
                 -3.177222	54.087222;
                 -3.180556	54.089722;
                 -3.186111	54.091389;
                 -3.190278	54.093611;
                 -3.194444	54.095556;
                 -3.202222	54.096111;
                 -3.208611	54.095556;
                 -3.214722	54.096111;
                 -3.220833	54.097500;
                 -3.226111	54.099167;
                 -3.230278	54.101111;
                 -3.233889	54.103611;
                 -3.236389	54.106667;
                 -3.238056	54.110000;
                 -3.237222	54.114444;
                 -3.237222	54.118611;
                 -3.238333	54.122500;
                 -3.239722	54.125833;
                 -3.240833	54.129444;
                 -3.239722	54.134167;
                 -3.237500	54.137500;
                 -3.234722	54.140556;
                 -3.230833	54.143056;
                 -3.228889	54.146667;
                 -3.227222	54.150833;
                 -3.228889	54.154167;
                 -3.231667	54.156944;
                 -3.235833	54.159167;
                 -3.240278	54.161389;
                 -3.245278	54.163056;
                 -3.248889	54.165556;
                 -3.246111	54.167500;
                 -3.241667	54.169722;
                 -3.236111	54.170556;
                 -3.229722	54.171111;
                 -3.222778	54.171111;
                 -3.217500	54.172778;
                 -3.213333	54.174722;
                 -3.212500	54.179167;
                 -3.213333	54.183056;
                 -3.215000	54.186389;
                 -3.215000	54.190556;
                 -3.213611	54.193611;
                 -3.211389	54.197222;
                 -3.208611	54.200278;
                 -3.206389	54.203611;
                 -3.203889	54.207222;
                 -3.202500	54.211389;
                 -3.200833	54.215278;
                 -3.199167	54.219444;
                 -3.197778	54.223333;
                 -3.198611	54.227222;
                 -3.201111	54.230000;
                 -3.203889	54.233056;
                 -3.206389	54.235833;
                 -3.208889	54.238889;
                 -3.210556	54.242222;
                 -3.211667	54.245833;
                 -3.213333	54.249167;
                 -3.213333	54.253333;
                 -3.211667	54.257500;
                 -3.207222	54.259444;
                 -3.208611	54.261389;
                 -3.214722	54.262778;
                 -3.218889	54.260833;
                 -3.221944	54.257778;
                 -3.224722	54.254722;
                 -3.226944	54.251111;
                 -3.229167	54.247500;
                 -3.230278	54.243056;
                 -3.231111	54.238611;
                 -3.231111	54.234444;
                 -3.231111	54.230278;
                 -3.232778	54.226111;
                 -3.234167	54.222222;
                 -3.236667	54.218611;
                 -3.238889	54.215000;
                 -3.239444	54.211389;
                 -3.235833	54.208889;
                 -3.232500	54.206389;
                 -3.233056	54.202778;
                 -3.235000	54.200556;
                 -3.236944	54.198333;
                 -3.240000	54.195278;
                 -3.243611	54.192778;
                 -3.248889	54.191389;
                 -3.255556	54.192222;
                 -3.262500	54.193056;
                 -3.269444	54.193889;
                 -3.277222	54.194444;
                 -3.284444	54.194444;
                 -3.290000	54.193611;
                 -3.294444	54.191389;
                 -3.298611	54.189444;
                 -3.305000	54.188889;
                 -3.311111	54.190278;
                 -3.316111	54.191944;
                 -3.319722	54.194444;
                 -3.323333	54.196944;
                 -3.326667	54.199444;
                 -3.329444	54.202500;
                 -3.331667	54.205278;
                 -3.334444	54.208333;
                 -3.337222	54.211111;
                 -3.340556	54.213889;
                 -3.343056	54.216667;
                 -3.345833	54.219722;
                 -3.348333	54.222500;
                 -3.350833	54.225556;
                 -3.353611	54.228333;
                 -3.356111	54.231389;
                 -3.359722	54.233889;
                 -3.363056	54.236389;
                 -3.366389	54.238889;
                 -3.371944	54.240556;
                 -3.376111	54.242778;
                 -3.380278	54.244722;
                 -3.381944	54.248056;
                 -3.384722	54.251111;
                 -3.386389	54.254444;
                 -3.388056	54.257778;
                 -3.390833	54.260833;
                 -3.392500	54.264167;
                 -3.396111	54.266667;
                 -3.399444	54.269167;
                 -3.403056	54.271667;
                 -3.406389	54.274167;
                 -3.409167	54.276944;
                 -3.410833	54.280278;
                 -3.411667	54.284167;
                 -3.411944	54.287222;
                 -3.411944	54.291389;
                 -3.410278	54.295556;
                 -3.409722	54.300000;
                 -3.407222	54.303611;
                 -3.405833	54.307778;
                 -3.405000	54.312222;
                 -3.405000	54.316389;
                 -3.406667	54.319722;
                 -3.408333	54.323056;
                 -3.410278	54.326389;
                 -3.411944	54.329722;
                 -3.412778	54.333333;
                 -3.411944	54.338056;
                 -3.412222	54.342222;
                 -3.417222	54.343889;
                 -3.423333	54.345278;
                 -3.429444	54.346389;
                 -3.433889	54.348611;
                 -3.437222	54.351111;
                 -3.440556	54.353611;
                 -3.443333	54.356667;
                 -3.446667	54.359167;
                 -3.450278	54.361667;
                 -3.452778	54.364444;
                 -3.455556	54.367500;
                 -3.458333	54.370278;
                 -3.460556	54.373333;
                 -3.463333	54.376111;
                 -3.466111	54.379167;
                 -3.469444	54.381667;
                 -3.472222	54.384722;
                 -3.475556	54.387222;
                 -3.478889	54.389722;
                 -3.481667	54.392500;
                 -3.484444	54.395556;
                 -3.487778	54.398056;
                 -3.490556	54.400833;
                 -3.493056	54.403889;
                 -3.496667	54.406389;
                 -3.500000	54.408889;
                 -3.503611	54.411389;
                 -3.506944	54.413889;
                 -3.510556	54.416389;
                 -3.513889	54.418889;
                 -3.517500	54.421389;
                 -3.520000	54.424444;
                 -3.524444	54.426667;
                 -3.528056	54.429167;
                 -3.531389	54.431667;
                 -3.535000	54.434167;
                 -3.538611	54.436667;
                 -3.542778	54.438611;
                 -3.546389	54.441111;
                 -3.549722	54.443611;
                 -3.553333	54.446389;
                 -3.556944	54.448889;
                 -3.559444	54.451667;
                 -3.561944	54.454722;
                 -3.564722	54.457500;
                 -3.568056	54.460000;
                 -3.572500	54.462222;
                 -3.576111	54.464722;
                 -3.579722	54.467222;
                 -3.583056	54.469722;
                 -3.585556	54.472778;
                 -3.588333	54.475556;
                 -3.591111	54.478611;
                 -3.594444	54.481111;
                 -3.598889	54.483056;
                 -3.605000	54.484444;
                 -3.609444	54.486389;
                 -3.612778	54.489167;
                 -3.615556	54.491944;
                 -3.618056	54.495000;
                 -3.620833	54.497778;
                 -3.623333	54.500833;
                 -3.626944	54.503333;
                 -3.629722	54.506111;
                 -3.631944	54.509167;
                 -3.634167	54.512500;
                 -3.633333	54.516111;
                 -3.629722	54.518611;
                 -3.624722	54.520278;
                 -3.620278	54.522222;
                 -3.615278	54.523611;
                 -3.611667	54.526111;
                 -3.609444	54.529722;
                 -3.608611	54.534444;
                 -3.606389	54.537778;
                 -3.603611	54.540833;
                 -3.599722	54.543611;
                 -3.596111	54.546111;
                 -3.592500	54.548611;
                 -3.591111	54.552500;
                 -3.590278	54.556111;
                 -3.590278	54.560278;
                 -3.589722	54.565000;
                 -3.587222	54.568611;
                 -3.585000	54.571944;
                 -3.581944	54.575000;
                 -3.578333	54.577500;
                 -3.575556	54.580556;
                 -3.575556	54.584722;
                 -3.576667	54.587500;
                 -3.577500	54.591389;
                 -3.575833	54.595278;
                 -3.573611	54.598889;
                 -3.571667	54.602500;
                 -3.569444	54.606111;
                 -3.570278	54.609722;
                 -3.571111	54.613611;
                 -3.571944	54.617222;
                 -3.571944	54.621389;
                 -3.571389	54.625833;
                 -3.571389	54.630000;
                 -3.571389	54.634167;
                 -3.570556	54.638889;
                 -3.568889	54.642778;
                 -3.565833	54.645833;
                 -3.563056	54.648889;
                 -3.560000	54.651944;
                 -3.557222	54.655000;
                 -3.554167	54.658056;
                 -3.550556	54.660556;
                 -3.547500	54.663611;
                 -3.544722	54.666667;
                 -3.541667	54.669722;
                 -3.539444	54.673333;
                 -3.537222	54.676667;
                 -3.534167	54.679722;
                 -3.530556	54.682222;
                 -3.527500	54.685278;
                 -3.523889	54.688056;
                 -3.521111	54.691111;
                 -3.518056	54.694167;
                 -3.516667	54.698056;
                 -3.515000	54.702222;
                 -3.513333	54.706111;
                 -3.511111	54.709722;
                 -3.509167	54.713333;
                 -3.505278	54.715833;
                 -3.501667	54.718333;
                 -3.497222	54.720278;
                 -3.493056	54.722500;
                 -3.488611	54.724444;
                 -3.484167	54.726389;
                 -3.480833	54.728056;
                 -3.476389	54.730000;
                 -3.472778	54.732500;
                 -3.468333	54.734722;
                 -3.464722	54.737222;
                 -3.460556	54.739167;
                 -3.456667	54.741667;
                 -3.453056	54.744444;
                 -3.450000	54.747222;
                 -3.446944	54.750278;
                 -3.444167	54.753333;
                 -3.441111	54.756389;
                 -3.438333	54.759444;
                 -3.435833	54.763056;
                 -3.434444	54.767222;
                 -3.433611	54.771667;
                 -3.432500	54.776111;
                 -3.433611	54.780000;
                 -3.434444	54.783611;
                 -3.435278	54.787500;
                 -3.436389	54.791111;
                 -3.437222	54.795000;
                 -3.438056	54.798611;
                 -3.438056	54.802778;
                 -3.438056	54.806944;
                 -3.436111	54.809444;
                 -3.433056	54.807500;
                 -3.429444	54.810000;
                 -3.426389	54.813056;
                 -3.422778	54.815556;
                 -3.420000	54.818611;
                 -3.416389	54.821111;
                 -3.413333	54.824167;
                 -3.410278	54.827222;
                 -3.407222	54.830278;
                 -3.404167	54.833333;
                 -3.400556	54.835833;
                 -3.397778	54.838889;
                 -3.394722	54.841944;
                 -3.392500	54.845556;
                 -3.389444	54.848611;
                 -3.387222	54.851944;
                 -3.386389	54.856667;
                 -3.384722	54.860556;
                 -3.383333	54.864722;
                 -3.379722	54.867222;
                 -3.376111	54.869722;
                 -3.372222	54.872222;
                 -3.368611	54.874722;
                 -3.365000	54.877500;
                 -3.360278	54.879444;
                 -3.357500	54.882500;
                 -3.353889	54.885000;
                 -3.349444	54.886944;
                 -3.345000	54.888889;
                 -3.340556	54.891111;
                 -3.335556	54.892500;
                 -3.329722	54.893611;
                 -3.323889	54.894722;
                 -3.321389	54.891667;
                 -3.324167	54.888611;
                 -3.326667	54.885000;
                 -3.324722	54.882778;
                 -3.323056	54.882500;
                 -3.323611	54.883611;
                 -3.317500	54.882222;
                 -3.310278	54.882222;
                 -3.302500	54.881667;
                 -3.297222	54.880000;
                 -3.293611	54.877500;
                 -3.289444	54.875278;
                 -3.285556	54.872778;
                 -3.281389	54.870833;
                 -3.276944	54.868611;
                 -3.271944	54.869167;
                 -3.269167	54.872222;
                 -3.268056	54.876667;
                 -3.269167	54.880556;
                 -3.270000	54.884167;
                 -3.266111	54.886944;
                 -3.261111	54.888333;
                 -3.255278	54.889444;
                 -3.250278	54.890833;
                 -3.247222	54.893889;
                 -3.245000	54.897500;
                 -3.243333	54.901389;
                 -3.246944	54.903889;
                 -3.253333	54.903611;
                 -3.258611	54.901944;
                 -3.264167	54.900833;
                 -3.271389	54.900833;
                 -3.277500	54.902222;
                 -3.281944	54.904444;
                 -3.284722	54.907222;
                 -3.287500	54.909444;
                 -3.286944	54.913056;
                 -3.284722	54.916389;
                 -3.283056	54.920556;
                 -3.280000	54.923611;
                 -3.275556	54.925556;
                 -3.270556	54.927222;
                 -3.264722	54.928056;
                 -3.260278	54.930278;
                 -3.256667	54.932778;
                 -3.253056	54.935278;
                 -3.249167	54.937778;
                 -3.244167	54.939167;
                 -3.238333	54.940278;
                 -3.232500	54.941667;
                 -3.228056	54.943889;
                 -3.223611	54.946389;
                 -3.218611	54.947778;
                 -3.212778	54.948889;
                 -3.206111	54.949167;
                 -3.198889	54.949167;
                 -3.191944	54.948333;
                 -3.186389	54.946667;
                 -3.186389	54.942500;
                 -3.185833	54.938889;
                 -3.178333	54.938889;
                 -3.172500	54.939722;
                 -3.165556	54.938889;
                 -3.159444	54.937500;
                 -3.155000	54.935556;
                 -3.148889	54.934167;
                 -3.144444	54.931944;
                 -3.140000	54.929722;
                 -3.136389	54.927222;
                 -3.131389	54.925278;
                 -3.125000	54.925556;
                 -3.119167	54.926667;
                 -3.112778	54.926111;
                 -3.107500	54.924444;
                 -3.102222	54.922778;
                 -3.095278	54.921944;
                 -3.089444	54.922778;
                 -3.085000	54.925000;
                 -3.080556	54.926944;
                 -3.076944	54.929444;
                 -3.072500	54.931389;
                 -3.068056	54.933333;
                 -3.063611	54.935556;
                 -3.059444	54.937500;
                 -3.053333	54.938611;
                 -3.046111	54.938611;
                 -3.039167	54.938611;
                 -3.032500	54.938889;
                 -3.025833	54.939444;
                 -3.022222	54.941944;
                 -3.022222	54.946111;
                 -3.025000	54.949167;
                 -3.029167	54.951111;
                 -3.037222	54.951667;
                 -3.043611	54.951111;
                 -3.049444	54.950278;
                 -3.055556	54.949167;
                 -3.060556	54.947778;
                 -3.066111	54.946667;
                 -3.073611	54.946667;
                 -3.076111	54.949722;
                 -3.074722	54.952778;
                 -3.071667	54.955833;
                 -3.068611	54.958889;
                 -3.065556	54.961944;
                 -3.061944	54.964444;
                 -3.056111	54.965278;
                 -3.049167	54.964444;
                 -3.043889	54.962778;
                 -3.036944	54.961944;
                 -3.029722	54.960833;
                 -3.022500	54.960833;
                 -3.017222	54.962500;
                 -3.013611	54.965000;
                 -3.011389	54.968611;
                 -3.009722	54.972500;
                 -3.008889	54.976944;
                 -3.008889	54.981111;
                 -3.010833	54.984444;
                 -3.015000	54.986667;
                 -3.021944	54.987500;
                 -3.029167	54.987500;
                 -3.035278	54.986667;
                 -3.040278	54.985000;
                 -3.045278	54.983611;
                 -3.050556	54.981944;
                 -3.055000	54.980000;
                 -3.059444	54.978056;
                 -3.063611	54.976111;
                 -3.068889	54.974444;
                 -3.073889	54.973056;
                 -3.080278	54.972500;
                 -3.087778	54.972500;
                 -3.095556	54.973056;
                 -3.101944	54.972500;
                 -3.107778	54.971389;
                 -3.113056	54.970000;
                 -3.118056	54.968611;
                 -3.122500	54.966389;
                 -3.127778	54.965000;
                 -3.133611	54.963889;
                 -3.140000	54.963333;
                 -3.146111	54.964722;
                 -3.151389	54.966389;
                 -3.157500	54.967778;
                 -3.164444	54.968611;
                 -3.172500	54.969167;
                 -3.180556	54.969722;
                 -3.188611	54.970278;
                 -3.195556	54.970278;
                 -3.202222	54.969722;
                 -3.208056	54.968889;
                 -3.213056	54.967222;
                 -3.218889	54.966111;
                 -3.225278	54.965833;
                 -3.231944	54.965278;
                 -3.238333	54.964722;
                 -3.243333	54.963056;
                 -3.250000	54.962778;
                 -3.257222	54.962778;
                 -3.263333	54.963889;
                 -3.267778	54.966111;
                 -3.271944	54.968611;
                 -3.277222	54.970278;
                 -3.284444	54.971389;
                 -3.292500	54.971667;
                 -3.299444	54.972778;
                 -3.305556	54.973889;
                 -3.311667	54.975278;
                 -3.317778	54.976667;
                 -3.325000	54.977500;
                 -3.330000	54.976111;
                 -3.334444	54.973889;
                 -3.339722	54.972500;
                 -3.346111	54.971944;
                 -3.353333	54.971944;
                 -3.360556	54.971944;
                 -3.368333	54.972500;
                 -3.376389	54.973056;
                 -3.383611	54.973889;
                 -3.390278	54.974722;
                 -3.395833	54.976389;
                 -3.401111	54.978056;
                 -3.406389	54.980000;
                 -3.410833	54.981944;
                 -3.415278	54.984167;
                 -3.420278	54.985833;
                 -3.425833	54.987500;
                 -3.432778	54.988333;
                 -3.440556	54.988889;
                 -3.445000	54.986944;
                 -3.447500	54.983333;
                 -3.451111	54.980833;
                 -3.456944	54.979722;
                 -3.462222	54.981389;
                 -3.468889	54.981389;
                 -3.475000	54.980833;
                 -3.478889	54.983333;
                 -3.482222	54.985833;
                 -3.487778	54.985833;
                 -3.494167	54.986111;
                 -3.497778	54.988611;
                 -3.502222	54.990833;
                 -3.508611	54.991111;
                 -3.512222	54.988611;
                 -3.511111	54.985000;
                 -3.511111	54.980833;
                 -3.516389	54.979167;
                 -3.520000	54.976667;
                 -3.521111	54.972222;
                 -3.525278	54.970278;
                 -3.531667	54.969722;
                 -3.536389	54.970833;
                 -3.539167	54.973889;
                 -3.542500	54.976389;
                 -3.545278	54.979167;
                 -3.548889	54.981667;
                 -3.551389	54.984722;
                 -3.555833	54.986667;
                 -3.560278	54.988889;
                 -3.565556	54.990556;
                 -3.571111	54.991389;
                 -3.571944	54.986944;
                 -3.573611	54.982778;
                 -3.575000	54.978611;
                 -3.577778	54.975556;
                 -3.580278	54.972222;
                 -3.581111	54.967500;
                 -3.578333	54.964722;
                 -3.575000	54.962222;
                 -3.572222	54.959167;
                 -3.570556	54.955833;
                 -3.570278	54.951667;
                 -3.571944	54.947778;
                 -3.574167	54.944167;
                 -3.575556	54.940000;
                 -3.576667	54.935556;
                 -3.578889	54.931944;
                 -3.580278	54.928056;
                 -3.577500	54.925000;
                 -3.573333	54.923056;
                 -3.568889	54.920833;
                 -3.566111	54.917778;
                 -3.567222	54.913333;
                 -3.567778	54.908889;
                 -3.567778	54.904722;
                 -3.567778	54.900556;
                 -3.568611	54.895833;
                 -3.569444	54.891389;
                 -3.573889	54.889444;
                 -3.578889	54.887778;
                 -3.583889	54.886389;
                 -3.587778	54.883889;
                 -3.590556	54.880833;
                 -3.592778	54.877222;
                 -3.595278	54.873611;
                 -3.599722	54.871667;
                 -3.606667	54.871667;
                 -3.611111	54.873611;
                 -3.615556	54.875833;
                 -3.621944	54.877222;
                 -3.628889	54.878056;
                 -3.636111	54.878056;
                 -3.643056	54.878056;
                 -3.650278	54.877778;
                 -3.656944	54.877500;
                 -3.664167	54.877222;
                 -3.671389	54.877222;
                 -3.678333	54.878333;
                 -3.683611	54.880000;
                 -3.688889	54.881667;
                 -3.696111	54.881667;
                 -3.701111	54.880000;
                 -3.706389	54.878611;
                 -3.710556	54.876389;
                 -3.714444	54.873889;
                 -3.718889	54.871944;
                 -3.722222	54.869444;
                 -3.725833	54.866944;
                 -3.730278	54.864722;
                 -3.734722	54.862778;
                 -3.738889	54.860556;
                 -3.743333	54.858611;
                 -3.748611	54.857222;
                 -3.755000	54.856667;
                 -3.762222	54.856667;
                 -3.769167	54.856389;
                 -3.775556	54.857778;
                 -3.781667	54.859167;
                 -3.787222	54.860833;
                 -3.793333	54.861944;
                 -3.800278	54.863056;
                 -3.807500	54.862778;
                 -3.811667	54.860833;
                 -3.809167	54.858056;
                 -3.808889	54.855278;
                 -3.805000	54.854444;
                 -3.807222	54.850833;
                 -3.812222	54.849444;
                 -3.817500	54.850278;
                 -3.821944	54.852222;
                 -3.823889	54.855556;
                 -3.825556	54.858889;
                 -3.827500	54.862222;
                 -3.827500	54.866389;
                 -3.825556	54.870000;
                 -3.823056	54.873611;
                 -3.819444	54.876111;
                 -3.814722	54.875833;
                 -3.811389	54.873333;
                 -3.808333	54.871111;
                 -3.808611	54.874444;
                 -3.811944	54.876944;
                 -3.815556	54.879444;
                 -3.820833	54.878056;
                 -3.823056	54.874444;
                 -3.825833	54.871389;
                 -3.828056	54.867778;
                 -3.830278	54.864167;
                 -3.836111	54.863333;
                 -3.842500	54.862778;
                 -3.846111	54.860000;
                 -3.845278	54.856389;
                 -3.842778	54.853333;
                 -3.840833	54.850278;
                 -3.843056	54.846667;
                 -3.849722	54.846111;
                 -3.855278	54.845000;
                 -3.857500	54.842500;
                 -3.851944	54.840556;
                 -3.849444	54.837778;
                 -3.847778	54.834444;
                 -3.843889	54.831944;
                 -3.837778	54.830556;
                 -3.831667	54.829444;
                 -3.827500	54.827222;
                 -3.824722	54.824444;
                 -3.825556	54.820833;
                 -3.828889	54.818056;
                 -3.833333	54.816111;
                 -3.838333	54.814444;
                 -3.844722	54.814167;
                 -3.851944	54.813889;
                 -3.858611	54.813333;
                 -3.864167	54.812500;
                 -3.867222	54.809444;
                 -3.869444	54.805833;
                 -3.874444	54.804167;
                 -3.880278	54.803333;
                 -3.884444	54.801111;
                 -3.888889	54.799167;
                 -3.893333	54.797222;
                 -3.898333	54.795556;
                 -3.904167	54.794444;
                 -3.908333	54.792500;
                 -3.913611	54.790833;
                 -3.917222	54.788333;
                 -3.920833	54.785833;
                 -3.925278	54.783889;
                 -3.929444	54.781667;
                 -3.934444	54.780278;
                 -3.939722	54.778611;
                 -3.944167	54.776667;
                 -3.947500	54.774167;
                 -3.950000	54.770556;
                 -3.953333	54.768056;
                 -3.959167	54.766944;
                 -3.965000	54.765833;
                 -3.972222	54.765833;
                 -3.979444	54.765833;
                 -3.985556	54.766944;
                 -3.991667	54.768333;
                 -3.997778	54.769444;
                 -4.005000	54.770556;
                 -4.012222	54.770278;
                 -4.018611	54.769722;
                 -4.024167	54.768889;
                 -4.030833	54.768333;
                 -4.036667	54.767222;
                 -4.043333	54.768056;
                 -4.048056	54.770278;
                 -4.051389	54.772778;
                 -4.054167	54.775556;
                 -4.056111	54.778889;
                 -4.056111	54.783056;
                 -4.056111	54.787222;
                 -4.055556	54.791667;
                 -4.054722	54.796389;
                 -4.053333	54.800278;
                 -4.051111	54.803889;
                 -4.049444	54.808056;
                 -4.047500	54.811389;
                 -4.045556	54.815556;
                 -4.043333	54.819167;
                 -4.042778	54.823611;
                 -4.043056	54.827778;
                 -4.045556	54.830833;
                 -4.049167	54.833333;
                 -4.053333	54.835278;
                 -4.060000	54.834722;
                 -4.063611	54.832222;
                 -4.066667	54.829167;
                 -4.069444	54.825556;
                 -4.071111	54.821667;
                 -4.072500	54.817500;
                 -4.073333	54.813056;
                 -4.075000	54.808889;
                 -4.075556	54.804444;
                 -4.077222	54.800278;
                 -4.078611	54.796389;
                 -4.080833	54.792778;
                 -4.083611	54.789722;
                 -4.087500	54.787222;
                 -4.091667	54.785000;
                 -4.093333	54.781111;
                 -4.090556	54.778056;
                 -4.087778	54.775278;
                 -4.085833	54.771944;
                 -4.086389	54.769167;
                 -4.090833	54.766944;
                 -4.095833	54.766389;
                 -4.100833	54.768056;
                 -4.105556	54.770278;
                 -4.108889	54.772778;
                 -4.112500	54.775278;
                 -4.114167	54.778611;
                 -4.120000	54.779167;
                 -4.123333	54.776667;
                 -4.127778	54.774722;
                 -4.132778	54.773056;
                 -4.138056	54.773889;
                 -4.142500	54.776111;
                 -4.146111	54.778611;
                 -4.150000	54.781111;
                 -4.153333	54.783611;
                 -4.157778	54.785556;
                 -4.162222	54.787778;
                 -4.167500	54.789444;
                 -4.171944	54.791389;
                 -4.173889	54.794722;
                 -4.171667	54.798333;
                 -4.169444	54.801944;
                 -4.170278	54.805556;
                 -4.174722	54.807778;
                 -4.180833	54.808889;
                 -4.188333	54.808889;
                 -4.193611	54.809722;
                 -4.198056	54.811667;
                 -4.198889	54.815556;
                 -4.197500	54.819444;
                 -4.199444	54.822778;
                 -4.202778	54.825278;
                 -4.207222	54.827500;
                 -4.211944	54.829444;
                 -4.214722	54.832500;
                 -4.215556	54.836111;
                 -4.214167	54.840278;
                 -4.211944	54.843889;
                 -4.208889	54.846944;
                 -4.206667	54.850278;
                 -4.203056	54.853056;
                 -4.200833	54.856389;
                 -4.198056	54.859444;
                 -4.198333	54.862778;
                 -4.200833	54.865833;
                 -4.205833	54.866111;
                 -4.211389	54.865000;
                 -4.215000	54.862500;
                 -4.218056	54.859444;
                 -4.220833	54.856389;
                 -4.223889	54.853333;
                 -4.226944	54.850278;
                 -4.230833	54.848056;
                 -4.234722	54.845556;
                 -4.238889	54.843056;
                 -4.243333	54.840833;
                 -4.247778	54.838889;
                 -4.253611	54.837778;
                 -4.259167	54.836667;
                 -4.265000	54.835833;
                 -4.272222	54.836667;
                 -4.276667	54.838611;
                 -4.281111	54.840833;
                 -4.284722	54.843333;
                 -4.289167	54.845278;
                 -4.293333	54.847500;
                 -4.300556	54.848333;
                 -4.307500	54.848056;
                 -4.314167	54.847500;
                 -4.320278	54.848889;
                 -4.324722	54.850833;
                 -4.329444	54.853056;
                 -4.333611	54.855000;
                 -4.339167	54.856667;
                 -4.345278	54.858056;
                 -4.351389	54.859444;
                 -4.355833	54.861389;
                 -4.358611	54.864167;
                 -4.360278	54.867500;
                 -4.362222	54.870833;
                 -4.365833	54.873333;
                 -4.368611	54.876389;
                 -4.369722	54.880000;
                 -4.368889	54.884722;
                 -4.367500	54.887778;
                 -4.366667	54.892222;
                 -4.367778	54.896111;
                 -4.370278	54.898889;
                 -4.373889	54.901389;
                 -4.378333	54.903611;
                 -4.381944	54.906111;
                 -4.386389	54.908056;
                 -4.391944	54.909722;
                 -4.397778	54.908611;
                 -4.396389	54.905833;
                 -4.394444	54.902500;
                 -4.392778	54.899167;
                 -4.392500	54.895000;
                 -4.394167	54.891111;
                 -4.397778	54.888611;
                 -4.401389	54.885833;
                 -4.405833	54.883889;
                 -4.410833	54.882222;
                 -4.415833	54.880833;
                 -4.420278	54.878611;
                 -4.424444	54.876667;
                 -4.427500	54.873611;
                 -4.429444	54.870000;
                 -4.430278	54.866389;
                 -4.428333	54.863056;
                 -4.424722	54.860556;
                 -4.421111	54.858056;
                 -4.418333	54.855000;
                 -4.415556	54.852222;
                 -4.413889	54.848889;
                 -4.412778	54.845278;
                 -4.411944	54.841389;
                 -4.411667	54.837222;
                 -4.413056	54.833333;
                 -4.412222	54.830556;
                 -4.408333	54.828056;
                 -4.402222	54.826667;
                 -4.396111	54.825556;
                 -4.390833	54.823889;
                 -4.386111	54.821667;
                 -4.381111	54.820000;
                 -4.377500	54.817500;
                 -4.373056	54.815556;
                 -4.368333	54.813333;
                 -4.363889	54.811389;
                 -4.358889	54.809722;
                 -4.354167	54.807500;
                 -4.349167	54.805833;
                 -4.344444	54.803889;
                 -4.341111	54.801389;
                 -4.340000	54.797500;
                 -4.339167	54.793889;
                 -4.338889	54.789722;
                 -4.341667	54.786667;
                 -4.347778	54.785556;
                 -4.354722	54.785556;
                 -4.360000	54.783889;
                 -4.358889	54.780278;
                 -4.357778	54.776389;
                 -4.357778	54.772222;
                 -4.359167	54.768333;
                 -4.360000	54.764722;
                 -4.357222	54.761667;
                 -4.351944	54.760000;
                 -4.348056	54.757500;
                 -4.349722	54.753611;
                 -4.351944	54.750000;
                 -4.353889	54.746389;
                 -4.355556	54.742222;
                 -4.356944	54.738333;
                 -4.359167	54.734722;
                 -4.361389	54.731111;
                 -4.361944	54.726667;
                 -4.360000	54.723333;
                 -4.357500	54.720278;
                 -4.353056	54.718333;
                 -4.348611	54.716111;
                 -4.345000	54.713611;
                 -4.343889	54.710000;
                 -4.343889	54.705833;
                 -4.345556	54.701667;
                 -4.346944	54.697778;
                 -4.349722	54.694722;
                 -4.351944	54.691111;
                 -4.355556	54.688611;
                 -4.359167	54.685833;
                 -4.362778	54.683333;
                 -4.366389	54.680833;
                 -4.371389	54.679167;
                 -4.373889	54.678056;
                 -4.373056	54.678333;
                 -4.378889	54.677500;
                 -4.384444	54.676389;
                 -4.391944	54.676111;
                 -4.398611	54.676944;
                 -4.405000	54.678333;
                 -4.409444	54.680556;
                 -4.413889	54.682500;
                 -4.417500	54.685000;
                 -4.420833	54.687500;
                 -4.425278	54.689444;
                 -4.430556	54.691111;
                 -4.436111	54.692778;
                 -4.443056	54.693611;
                 -4.450000	54.694444;
                 -4.456111	54.695833;
                 -4.463056	54.696667;
                 -4.470278	54.697500;
                 -4.478056	54.698056;
                 -4.486111	54.698333;
                 -4.493889	54.698889;
                 -4.501111	54.699722;
                 -4.504444	54.702222;
                 -4.507222	54.705000;
                 -4.509167	54.708333;
                 -4.511667	54.711111;
                 -4.513611	54.714444;
                 -4.518056	54.716667;
                 -4.525278	54.717500;
                 -4.529444	54.719444;
                 -4.533333	54.721944;
                 -4.535833	54.725000;
                 -4.537500	54.728333;
                 -4.541389	54.730833;
                 -4.545556	54.732778;
                 -4.550000	54.735000;
                 -4.554722	54.736944;
                 -4.557500	54.739722;
                 -4.559444	54.743056;
                 -4.559444	54.747222;
                 -4.560278	54.751111;
                 -4.563056	54.753889;
                 -4.566667	54.756389;
                 -4.571111	54.758611;
                 -4.575556	54.760556;
                 -4.579444	54.761111;
                 -4.583889	54.763333;
                 -4.586667	54.766111;
                 -4.587778	54.770000;
                 -4.589722	54.773333;
                 -4.592222	54.776111;
                 -4.596667	54.778333;
                 -4.601944	54.780000;
                 -4.607500	54.781667;
                 -4.614167	54.783333;
                 -4.621389	54.783333;
                 -4.628611	54.783056;
                 -4.635000	54.783333;
                 -4.639444	54.785556;
                 -4.642222	54.788333;
                 -4.647500	54.790000;
                 -4.653611	54.791389;
                 -4.661389	54.791667;
                 -4.668333	54.792500;
                 -4.672222	54.795000;
                 -4.675833	54.797500;
                 -4.678333	54.800556;
                 -4.682778	54.802500;
                 -4.687222	54.804444;
                 -4.690833	54.806944;
                 -4.693611	54.810000;
                 -4.697222	54.812500;
                 -4.701944	54.814444;
                 -4.706389	54.816667;
                 -4.711667	54.818333;
                 -4.717778	54.819444;
                 -4.725556	54.820000;
                 -4.731944	54.821111;
                 -4.736389	54.823056;
                 -4.742500	54.824444;
                 -4.750556	54.824722;
                 -4.757500	54.824722;
                 -4.764722	54.825556;
                 -4.770833	54.826667;
                 -4.776111	54.828333;
                 -4.781389	54.830000;
                 -4.785278	54.832500;
                 -4.787778	54.835556;
                 -4.790556	54.838333;
                 -4.792500	54.841667;
                 -4.794444	54.845000;
                 -4.796111	54.848333;
                 -4.797500	54.851944;
                 -4.800000	54.855000;
                 -4.802778	54.857778;
                 -4.805556	54.860833;
                 -4.810833	54.862500;
                 -4.817778	54.863056;
                 -4.824444	54.862500;
                 -4.830000	54.861389;
                 -4.834444	54.859444;
                 -4.839722	54.857778;
                 -4.843889	54.855833;
                 -4.848889	54.854167;
                 -4.853056	54.852222;
                 -4.858056	54.850556;
                 -4.863889	54.849444;
                 -4.869722	54.848333;
                 -4.875556	54.847222;
                 -4.880278	54.845556;
                 -4.884722	54.843611;
                 -4.888333	54.841111;
                 -4.893333	54.839444;
                 -4.899167	54.838333;
                 -4.904722	54.837222;
                 -4.910556	54.836111;
                 -4.915833	54.834444;
                 -4.920556	54.833056;
                 -4.925833	54.831389;
                 -4.929167	54.828889;
                 -4.932222	54.825833;
                 -4.934444	54.822222;
                 -4.938611	54.820000;
                 -4.942778	54.818056;
                 -4.946944	54.815833;
                 -4.951389	54.813889;
                 -4.955000	54.811389;
                 -4.958056	54.808056;
                 -4.960556	54.805000;
                 -4.963611	54.801944;
                 -4.966389	54.798889;
                 -4.966111	54.795833;
                 -4.960556	54.794167;
                 -4.956389	54.791944;
                 -4.951944	54.790000;
                 -4.947222	54.788056;
                 -4.946389	54.784167;
                 -4.945833	54.781944;
                 -4.944722	54.779167;
                 -4.944444	54.775000;
                 -4.945278	54.770278;
                 -4.945833	54.765833;
                 -4.943056	54.762778;
                 -4.938611	54.760833;
                 -4.935000	54.758333;
                 -4.931389	54.755833;
                 -4.927778	54.753333;
                 -4.926667	54.749722;
                 -4.925833	54.745833;
                 -4.925556	54.741944;
                 -4.925556	54.737778;
                 -4.923611	54.734444;
                 -4.920000	54.731944;
                 -4.917222	54.728889;
                 -4.915278	54.725556;
                 -4.914444	54.721944;
                 -4.913333	54.718333;
                 -4.910278	54.715278;
                 -4.906111	54.713333;
                 -4.903333	54.710278;
                 -4.902222	54.706667;
                 -4.901111	54.702778;
                 -4.899444	54.699722;
                 -4.895833	54.697222;
                 -4.891389	54.695000;
                 -4.885000	54.693889;
                 -4.878889	54.692500;
                 -4.872778	54.691389;
                 -4.870000	54.688611;
                 -4.868056	54.685278;
                 -4.866111	54.681944;
                 -4.865833	54.677778;
                 -4.866667	54.673056;
                 -4.868056	54.669167;
                 -4.870278	54.665556;
                 -4.873056	54.662500;
                 -4.875278	54.658889;
                 -4.878056	54.655833;
                 -4.880000	54.652222;
                 -4.880833	54.647778;
                 -4.878056	54.644722;
                 -4.873889	54.642778;
                 -4.867778	54.641389;
                 -4.862222	54.639722;
                 -4.856944	54.638333;
                 -4.855000	54.635000;
                 -4.855833	54.631111;
                 -4.858333	54.628056;
                 -4.864167	54.626944;
                 -4.869722	54.627778;
                 -4.875000	54.629444;
                 -4.879444	54.631389;
                 -4.883889	54.633611;
                 -4.889167	54.635278;
                 -4.896111	54.635000;
                 -4.900833	54.636389;
                 -4.904444	54.638889;
                 -4.910556	54.640000;
                 -4.917778	54.640833;
                 -4.923889	54.641944;
                 -4.928333	54.644167;
                 -4.932778	54.646111;
                 -4.937222	54.648333;
                 -4.940833	54.650556;
                 -4.945278	54.652778;
                 -4.948056	54.655556;
                 -4.952500	54.657778;
                 -4.954444	54.661111;
                 -4.955278	54.664722;
                 -4.956389	54.668333;
                 -4.958333	54.671667;
                 -4.961111	54.674722;
                 -4.963889	54.677500;
                 -4.964722	54.681389;
                 -4.963333	54.685278;
                 -4.961389	54.688889;
                 -4.958611	54.691944;
                 -4.954444	54.694167;
                 -4.950556	54.696667;
                 -4.948611	54.700278;
                 -4.950556	54.703611;
                 -4.953056	54.706389;
                 -4.955000	54.709722;
                 -4.957778	54.712500;
                 -4.958611	54.716389;
                 -4.958889	54.720556;
                 -4.957500	54.724444;
                 -4.961111	54.726944;
                 -4.966667	54.728611;
                 -4.972778	54.730000;
                 -4.979722	54.730833;
                 -4.985833	54.731944;
                 -4.989444	54.734444;
                 -4.990833	54.738056;
                 -4.989167	54.742222;
                 -4.987778	54.746389;
                 -4.990556	54.749167;
                 -4.995000	54.751111;
                 -4.999444	54.753333;
                 -5.003889	54.755278;
                 -5.008611	54.757500;
                 -5.010278	54.760833;
                 -5.009167	54.764722;
                 -5.006944	54.768333;
                 -5.007778	54.771944;
                 -5.011389	54.774444;
                 -5.015833	54.776667;
                 -5.019722	54.778056;
                 -5.025278	54.779722;
                 -5.031389	54.781111;
                 -5.036667	54.782778;
                 -5.041111	54.784722;
                 -5.045556	54.786667;
                 -5.048333	54.789722;
                 -5.051111	54.792500;
                 -5.053611	54.795556;
                 -5.056667	54.798333;
                 -5.060278	54.800833;
                 -5.065556	54.802500;
                 -5.070833	54.804167;
                 -5.074444	54.806667;
                 -5.077222	54.809444;
                 -5.080833	54.811944;
                 -5.084444	54.814444;
                 -5.088889	54.816389;
                 -5.094167	54.818056;
                 -5.098889	54.820278;
                 -5.105000	54.821389;
                 -5.108611	54.823889;
                 -5.111389	54.826667;
                 -5.114167	54.829722;
                 -5.117778	54.832222;
                 -5.120556	54.835000;
                 -5.124167	54.837500;
                 -5.128056	54.840000;
                 -5.130556	54.842778;
                 -5.133333	54.845833;
                 -5.136111	54.848611;
                 -5.138889	54.851389;
                 -5.141667	54.854444;
                 -5.143611	54.857778;
                 -5.144722	54.861389;
                 -5.144722	54.865556;
                 -5.146667	54.868889;
                 -5.147778	54.872500;
                 -5.149722	54.875833;
                 -5.151667	54.879167;
                 -5.155278	54.881667;
                 -5.158056	54.884444;
                 -5.160833	54.887500;
                 -5.163611	54.890278;
                 -5.166389	54.893333;
                 -5.170000	54.895556;
                 -5.172778	54.898611;
                 -5.175556	54.901389;
                 -5.177500	54.904722;
                 -5.179444	54.907222;
                 -5.181389	54.910556;
                 -5.182500	54.914167;
                 -5.184444	54.917500;
                 -5.185556	54.921111;
                 -5.186389	54.925000;
                 -5.187500	54.928611;
                 -5.186944	54.933333;
                 -5.186389	54.937778;
                 -5.185556	54.942222;
                 -5.184444	54.946389;
                 -5.182778	54.950556;
                 -5.180556	54.954167;
                 -5.179444	54.958056;
                 -5.177222	54.961667;
                 -5.175833	54.965833;
                 -5.176944	54.969444;
                 -5.179722	54.972500;
                 -5.180000	54.976667;
                 -5.176944	54.979722;
                 -5.172778	54.981667;
                 -5.170000	54.984722;
                 -5.168333	54.988889;
                 -5.168611	54.993056;
                 -5.168056	54.997500;
                 -5.166111	55.001111;
                 -5.162222	55.003611;
                 -5.157222	55.005278;
                 -5.150833	55.005833;
                 -5.144444	55.005556;
                 -5.138056	55.004444;
                 -5.131111	55.004444;
                 -5.126667	55.006667;
                 -5.123889	55.009722;
                 -5.118889	55.011389;
                 -5.111944	55.010556;
                 -5.105556	55.009167;
                 -5.100000	55.007222;
                 -5.094722	55.005556;
                 -5.090278	55.003333;
                 -5.086667	55.001111;
                 -5.082222	54.998889;
                 -5.078611	54.996389;
                 -5.075000	54.993889;
                 -5.073056	54.990556;
                 -5.071111	54.987500;
                 -5.070833	54.983333;
                 -5.068056	54.980278;
                 -5.065556	54.977500;
                 -5.063611	54.974167;
                 -5.063333	54.970000;
                 -5.064722	54.965833;
                 -5.067500	54.962778;
                 -5.070278	54.959722;
                 -5.071667	54.956667;
                 -5.068889	54.953611;
                 -5.066111	54.950833;
                 -5.066667	54.946389;
                 -5.067500	54.941667;
                 -5.067222	54.937500;
                 -5.066944	54.933333;
                 -5.065833	54.929722;
                 -5.063889	54.926389;
                 -5.061389	54.923611;
                 -5.058611	54.920556;
                 -5.055000	54.918056;
                 -5.050278	54.916111;
                 -5.045278	54.914444;
                 -5.038889	54.913333;
                 -5.032778	54.911944;
                 -5.028056	54.910000;
                 -5.023611	54.907778;
                 -5.020000	54.905556;
                 -5.015556	54.903333;
                 -5.011111	54.901389;
                 -5.005000	54.900000;
                 -4.997500	54.900278;
                 -4.992778	54.901667;
                 -4.989722	54.903889;
                 -4.986944	54.906944;
                 -4.984167	54.910000;
                 -4.982778	54.914167;
                 -4.982222	54.918611;
                 -4.983056	54.922500;
                 -4.985000	54.925833;
                 -4.987778	54.928611;
                 -4.991389	54.931111;
                 -4.995833	54.933056;
                 -5.000556	54.935278;
                 -5.003889	54.937778;
                 -5.007500	54.940278;
                 -5.012222	54.942222;
                 -5.016667	54.944167;
                 -5.019444	54.947222;
                 -5.020556	54.950833;
                 -5.019722	54.955556;
                 -5.019167	54.959167;
                 -5.021111	54.962500;
                 -5.023889	54.965278;
                 -5.026667	54.968333;
                 -5.030278	54.970833;
                 -5.035000	54.972778;
                 -5.038333	54.975278;
                 -5.041389	54.978056;
                 -5.043889	54.981111;
                 -5.045278	54.984722;
                 -5.046944	54.988056;
                 -5.048889	54.991389;
                 -5.050833	54.994722;
                 -5.052778	54.998056;
                 -5.054722	55.001389;
                 -5.057500	55.004167;
                 -5.059444	55.007500;
                 -5.061944	55.010278;
                 -5.063889	55.013611;
                 -5.065000	55.017500;
                 -5.065278	55.021667;
                 -5.065556	55.025556;
                 -5.063889	55.029722;
                 -5.062778	55.033889;
                 -5.059722	55.036111;
                 -5.059444	55.039722;
                 -5.057222	55.043333;
                 -5.055833	55.047222;
                 -5.053611	55.050833;
                 -5.050833	55.053889;
                 -5.048056	55.056944;
                 -5.045833	55.060556;
                 -5.043611	55.064167;
                 -5.041389	55.067778;
                 -5.039444	55.071389;
                 -5.037222	55.074722;
                 -5.034444	55.078056;
                 -5.030833	55.080556;
                 -5.028056	55.083611;
                 -5.025278	55.086667;
                 -5.022222	55.089722;
                 -5.020000	55.093333;
                 -5.018889	55.097500;
                 -5.018056	55.101944;
                 -5.017222	55.106389;
                 -5.017500	55.110556;
                 -5.017778	55.114722;
                 -5.017222	55.119444;
                 -5.016389	55.123889;
                 -5.015000	55.128056;
                 -5.013611	55.131944;
                 -5.011389	55.135556;
                 -5.007778	55.138056;
                 -5.003333	55.140278;
                 -4.999167	55.142222;
                 -4.994167	55.143889;
                 -4.989722	55.146111;
                 -4.984722	55.147500;
                 -4.980556	55.149722;
                 -4.977500	55.152778;
                 -4.973889	55.155278;
                 -4.969444	55.157500;
                 -4.965278	55.159444;
                 -4.960278	55.161111;
                 -4.955000	55.162778;
                 -4.950000	55.164167;
                 -4.944722	55.165833;
                 -4.940556	55.168056;
                 -4.936944	55.170556;
                 -4.935556	55.174722;
                 -4.934722	55.179167;
                 -4.931389	55.181667;
                 -4.926389	55.183333;
                 -4.921944	55.185278;
                 -4.917500	55.187500;
                 -4.915278	55.191111;
                 -4.914722	55.195556;
                 -4.911944	55.198611;
                 -4.908889	55.201667;
                 -4.904444	55.203889;
                 -4.900278	55.205833;
                 -4.896667	55.208611;
                 -4.892222	55.210556;
                 -4.888056	55.212778;
                 -4.882778	55.214167;
                 -4.877778	55.215833;
                 -4.872778	55.217500;
                 -4.868333	55.219444;
                 -4.865556	55.222500;
                 -4.863889	55.226667;
                 -4.864167	55.230833;
                 -4.863611	55.235278;
                 -4.862778	55.240000;
                 -4.861389	55.243889;
                 -4.860000	55.248056;
                 -4.858611	55.251944;
                 -4.856389	55.255556;
                 -4.853611	55.258611;
                 -4.851389	55.262222;
                 -4.849167	55.265833;
                 -4.847778	55.270000;
                 -4.844722	55.273056;
                 -4.841389	55.275556;
                 -4.839167	55.279167;
                 -4.837778	55.283056;
                 -4.836111	55.287222;
                 -4.835556	55.291944;
                 -4.836389	55.295556;
                 -4.836667	55.299722;
                 -4.836944	55.303889;
                 -4.837778	55.307500;
                 -4.838889	55.311389;
                 -4.839167	55.315556;
                 -4.838333	55.320000;
                 -4.835556	55.323056;
                 -4.830278	55.325000;
                 -4.825278	55.326667;
                 -4.820833	55.328889;
                 -4.819722	55.331111;
                 -4.816944	55.334167;
                 -4.813889	55.337222;
                 -4.811111	55.340278;
                 -4.807500	55.342778;
                 -4.804444	55.345833;
                 -4.801111	55.348333;
                 -4.796667	55.350556;
                 -4.791389	55.351944;
                 -4.786389	55.353611;
                 -4.781389	55.355278;
                 -4.777500	55.357778;
                 -4.775278	55.361389;
                 -4.773889	55.365556;
                 -4.772500	55.369444;
                 -4.771667	55.374167;
                 -4.771111	55.378611;
                 -4.770556	55.383056;
                 -4.769722	55.387778;
                 -4.767500	55.391389;
                 -4.764722	55.394444;
                 -4.761667	55.397500;
                 -4.758889	55.400556;
                 -4.755000	55.403056;
                 -4.753056	55.406667;
                 -4.750000	55.409722;
                 -4.746389	55.412222;
                 -4.741944	55.414444;
                 -4.737778	55.416389;
                 -4.733333	55.418611;
                 -4.728889	55.420556;
                 -4.723889	55.422222;
                 -4.718889	55.423889;
                 -4.713611	55.425278;
                 -4.706944	55.425833;
                 -4.699722	55.426111;
                 -4.695278	55.428056;
                 -4.690833	55.430278;
                 -4.687222	55.432778;
                 -4.682222	55.434444;
                 -4.676389	55.435278;
                 -4.670000	55.436111;
                 -4.663333	55.436667;
                 -4.656667	55.437222;
                 -4.650833	55.438333;
                 -4.645833	55.439722;
                 -4.641389	55.441944;
                 -4.637778	55.444444;
                 -4.636111	55.448611;
                 -4.635833	55.452222;
                 -4.635833	55.456389;
                 -4.633611	55.459722;
                 -4.631389	55.463333;
                 -4.628333	55.466389;
                 -4.625556	55.469444;
                 -4.622500	55.472500;
                 -4.619722	55.475556;
                 -4.618889	55.480278;
                 -4.616667	55.483889;
                 -4.613889	55.486944;
                 -4.613333	55.491389;
                 -4.613333	55.495556;
                 -4.615833	55.498333;
                 -4.618889	55.501389;
                 -4.620833	55.504722;
                 -4.621944	55.508333;
                 -4.621944	55.512500;
                 -4.623889	55.515833;
                 -4.627500	55.518333;
                 -4.631944	55.520278;
                 -4.636389	55.522500;
                 -4.639444	55.525278;
                 -4.641389	55.528611;
                 -4.644167	55.531667;
                 -4.646111	55.535000;
                 -4.652222	55.536111;
                 -4.658333	55.537500;
                 -4.666389	55.537778;
                 -4.674444	55.538333;
                 -4.681944	55.539167;
                 -4.684444	55.541944;
                 -4.683333	55.545000;
                 -4.678889	55.547222;
                 -4.673056	55.548333;
                 -4.668056	55.549722;
                 -4.663611	55.551944;
                 -4.660556	55.555000;
                 -4.658333	55.558611;
                 -4.659444	55.562222;
                 -4.662222	55.565278;
                 -4.665000	55.568056;
                 -4.668611	55.570556;
                 -4.672500	55.573056;
                 -4.675000	55.575833;
                 -4.678056	55.578889;
                 -4.680000	55.582222;
                 -4.680833	55.585833;
                 -4.681944	55.589722;
                 -4.683889	55.592778;
                 -4.685833	55.596111;
                 -4.688611	55.599444;
                 -4.692222	55.601944;
                 -4.694722	55.605000;
                 -4.698611	55.607500;
                 -4.703056	55.609444;
                 -4.707778	55.611667;
                 -4.713056	55.613333;
                 -4.718611	55.615000;
                 -4.723889	55.616667;
                 -4.729444	55.618333;
                 -4.736667	55.618889;
                 -4.743889	55.619722;
                 -4.751944	55.620278;
                 -4.757222	55.621944;
                 -4.761111	55.624444;
                 -4.764722	55.626944;
                 -4.769167	55.628889;
                 -4.776389	55.629722;
                 -4.783611	55.629722;
                 -4.791111	55.629444;
                 -4.798333	55.629444;
                 -4.805000	55.629722;
                 -4.811389	55.631111;
                 -4.815556	55.633056;
                 -4.818611	55.636111;
                 -4.819444	55.639722;
                 -4.816667	55.642778;
                 -4.813611	55.645833;
                 -4.812500	55.648889;
                 -4.815278	55.651944;
                 -4.819722	55.653889;
                 -4.826111	55.655278;
                 -4.831667	55.656944;
                 -4.834444	55.659722;
                 -4.836389	55.663056;
                 -4.839167	55.665833;
                 -4.842778	55.668333;
                 -4.849167	55.669722;
                 -4.855556	55.670833;
                 -4.860833	55.672500;
                 -4.864444	55.675000;
                 -4.868056	55.677500;
                 -4.871944	55.680000;
                 -4.875833	55.682500;
                 -4.881111	55.684167;
                 -4.887500	55.685278;
                 -4.892778	55.686944;
                 -4.898333	55.688611;
                 -4.903889	55.690278;
                 -4.908333	55.692222;
                 -4.912222	55.694722;
                 -4.915833	55.697222;
                 -4.916667	55.701111;
                 -4.916944	55.705278;
                 -4.914167	55.708333;
                 -4.911111	55.711389;
                 -4.906667	55.713333;
                 -4.903056	55.715833;
                 -4.898611	55.718056;
                 -4.895000	55.720556;
                 -4.890556	55.722778;
                 -4.886111	55.724722;
                 -4.881944	55.726944;
                 -4.878056	55.729444;
                 -4.873889	55.731389;
                 -4.870000	55.734167;
                 -4.866389	55.736667;
                 -4.863611	55.739722;
                 -4.861389	55.743333;
                 -4.859167	55.746944;
                 -4.858333	55.751389;
                 -4.858611	55.755556;
                 -4.859722	55.759167;
                 -4.861667	55.762500;
                 -4.863611	55.765833;
                 -4.866111	55.768889;
                 -4.869167	55.771667;
                 -4.871944	55.774722;
                 -4.874722	55.777500;
                 -4.878333	55.780000;
                 -4.881389	55.782778;
                 -4.883333	55.786111;
                 -4.883333	55.790278;
                 -4.885556	55.793611;
                 -4.888056	55.796389;
                 -4.891944	55.798889;
                 -4.895556	55.801389;
                 -4.897500	55.804722;
                 -4.899444	55.808056;
                 -4.900556	55.811944;
                 -4.902222	55.815000;
                 -4.902500	55.819167;
                 -4.903889	55.823056;
                 -4.903056	55.827500;
                 -4.902222	55.832222;
                 -4.900833	55.836111;
                 -4.899444	55.840278;
                 -4.900278	55.843889;
                 -4.901667	55.847778;
                 -4.902500	55.851389;
                 -4.904444	55.854722;
                 -4.903889	55.859167;
                 -4.902222	55.863333;
                 -4.900833	55.867500;
                 -4.900278	55.871944;
                 -4.902222	55.875278;
                 -4.904167	55.878611;
                 -4.906944	55.881389;
                 -4.908056	55.885278;
                 -4.905833	55.888889;
                 -4.902778	55.891944;
                 -4.899167	55.894444;
                 -4.895556	55.896944;
                 -4.891111	55.899167;
                 -4.887500	55.901667;
                 -4.883889	55.904167;
                 -4.884167	55.906667;
                 -4.886944	55.909444;
                 -4.887222	55.913611;
                 -4.885833	55.917778;
                 -4.882778	55.920833;
                 -4.879167	55.923333;
                 -4.876944	55.926944;
                 -4.878889	55.930278;
                 -4.881944	55.933056;
                 -4.882778	55.936667;
                 -4.880000	55.940000;
                 -4.873889	55.940833;
                 -4.867222	55.941667;
                 -4.861389	55.942778;
                 -4.854722	55.943333;
                 -4.849722	55.944722;
                 -4.843889	55.945833;
                 -4.839167	55.948056;
                 -4.835556	55.950556;
                 -4.831667	55.953056;
                 -4.826667	55.954722;
                 -4.820000	55.955278;
                 -4.812500	55.955278;
                 -4.804444	55.955000;
                 -4.800000	55.956944;
                 -4.795000	55.958611;
                 -4.789167	55.959722;
                 -4.780833	55.959444;
                 -4.775278	55.957778;
                 -4.770833	55.955556;
                 -4.765000	55.953889;
                 -4.761389	55.951389;
                 -4.756944	55.949444;
                 -4.750556	55.948333;
                 -4.743333	55.947500;
                 -4.735000	55.946944;
                 -4.728889	55.945833;
                 -4.724167	55.943611;
                 -4.721389	55.940833;
                 -4.719444	55.937500;
                 -4.713889	55.935833;
                 -4.706667	55.935833;
                 -4.700000	55.936389;
                 -4.693611	55.935278;
                 -4.686389	55.934444;
                 -4.678889	55.933611;
                 -4.670833	55.933056;
                 -4.664444	55.933611;
                 -4.657778	55.934444;
                 -4.650278	55.934444;
                 -4.643889	55.933056;
                 -4.639167	55.931111;
                 -4.633056	55.930000;
                 -4.626389	55.928611;
                 -4.619444	55.927778;
                 -4.612500	55.928333;
                 -4.605833	55.928889;
                 -4.598611	55.928056;
                 -4.593333	55.926389;
                 -4.586944	55.925278;
                 -4.580556	55.923889;
                 -4.572500	55.923611;
                 -4.564167	55.923056;
                 -4.556944	55.923333;
                 -4.549444	55.922500;
                 -4.542500	55.921667;
                 -4.535278	55.920833;
                 -4.530278	55.921389;
                 -4.523333	55.921944;
                 -4.515278	55.921389;
                 -4.508889	55.922222;
                 -4.502778	55.923056;
                 -4.496944	55.924167;
                 -4.490833	55.925278;
                 -4.492222	55.928056;
                 -4.499444	55.928889;
                 -4.507500	55.929444;
                 -4.514722	55.930278;
                 -4.521944	55.931111;
                 -4.530000	55.931389;
                 -4.537222	55.932222;
                 -4.544722	55.933056;
                 -4.552778	55.933611;
                 -4.560000	55.934444;
                 -4.566389	55.935556;
                 -4.573611	55.936389;
                 -4.580833	55.937222;
                 -4.586389	55.938889;
                 -4.592778	55.940278;
                 -4.598056	55.941944;
                 -4.603889	55.943611;
                 -4.610000	55.944722;
                 -4.618056	55.945278;
                 -4.625556	55.946111;
                 -4.632500	55.946944;
                 -4.638889	55.948056;
                 -4.644444	55.949722;
                 -4.650000	55.951389;
                 -4.654444	55.953611;
                 -4.658333	55.955833;
                 -4.661944	55.958333;
                 -4.665556	55.960833;
                 -4.670278	55.963056;
                 -4.674722	55.965000;
                 -4.680556	55.966667;
                 -4.685833	55.968333;
                 -4.689444	55.970833;
                 -4.693056	55.973333;
                 -4.697778	55.975556;
                 -4.701389	55.977778;
                 -4.704444	55.980833;
                 -4.706944	55.983611;
                 -4.710000	55.986667;
                 -4.712778	55.989444;
                 -4.716667	55.991944;
                 -4.721111	55.994167;
                 -4.726667	55.995556;
                 -4.732222	55.997222;
                 -4.736667	55.999444;
                 -4.740556	56.001944;
                 -4.745833	56.003611;
                 -4.750556	56.005556;
                 -4.755833	56.007222;
                 -4.762500	56.008611;
                 -4.767778	56.010278;
                 -4.774167	56.011389;
                 -4.778889	56.013333;
                 -4.780833	56.016667;
                 -4.783333	56.019722;
                 -4.786389	56.022500;
                 -4.791667	56.024167;
                 -4.797500	56.025833;
                 -4.800278	56.028611;
                 -4.801389	56.032500;
                 -4.803333	56.035833;
                 -4.805278	56.039167;
                 -4.808889	56.041667;
                 -4.811667	56.044444;
                 -4.814444	56.047500;
                 -4.817500	56.050278;
                 -4.820278	56.053056;
                 -4.823056	56.056111;
                 -4.825833	56.058889;
                 -4.828611	56.061944;
                 -4.834167	56.063611;
                 -4.837222	56.060278;
                 -4.838611	56.056389;
                 -4.839444	56.051667;
                 -4.838056	56.048056;
                 -4.837222	56.044444;
                 -4.835278	56.041111;
                 -4.832500	56.038056;
                 -4.829722	56.035278;
                 -4.827500	56.031944;
                 -4.825556	56.028611;
                 -4.821944	56.026111;
                 -4.819167	56.023333;
                 -4.817222	56.020000;
                 -4.815278	56.016667;
                 -4.810556	56.014444;
                 -4.806111	56.012500;
                 -4.801389	56.010556;
                 -4.796944	56.008333;
                 -4.792222	56.006389;
                 -4.788611	56.003889;
                 -4.783889	56.001667;
                 -4.780278	55.999167;
                 -4.775556	55.997222;
                 -4.771944	55.994722;
                 -4.768056	55.992222;
                 -4.766389	55.988889;
                 -4.765833	55.986667;
                 -4.767222	55.982500;
                 -4.771111	55.980000;
                 -4.776111	55.978333;
                 -4.781944	55.977222;
                 -4.788889	55.976667;
                 -4.795278	55.976111;
                 -4.802778	55.976111;
                 -4.809444	55.975556;
                 -4.814167	55.976667;
                 -4.819444	55.978333;
                 -4.825833	55.979444;
                 -4.833333	55.980278;
                 -4.841667	55.980833;
                 -4.845833	55.982778;
                 -4.848889	55.985556;
                 -4.851944	55.988611;
                 -4.852778	55.992222;
                 -4.854722	55.995556;
                 -4.856667	55.998889;
                 -4.861389	56.001111;
                 -4.865000	56.003333;
                 -4.868611	56.005833;
                 -4.870556	56.009167;
                 -4.870833	56.013333;
                 -4.870000	56.018056;
                 -4.873056	56.020833;
                 -4.875833	56.023611;
                 -4.877778	56.026944;
                 -4.878611	56.030833;
                 -4.880833	56.034167;
                 -4.881944	56.037778;
                 -4.881944	56.041944;
                 -4.881944	56.046111;
                 -4.879167	56.049167;
                 -4.876944	56.052778;
                 -4.874722	56.056389;
                 -4.871944	56.059444;
                 -4.868889	56.062500;
                 -4.865833	56.065556;
                 -4.863056	56.068611;
                 -4.860833	56.072222;
                 -4.858611	56.075833;
                 -4.855833	56.078889;
                 -4.853611	56.082222;
                 -4.851389	56.085833;
                 -4.849722	56.090000;
                 -4.850833	56.093611;
                 -4.848611	56.097222;
                 -4.845000	56.099722;
                 -4.841389	56.102500;
                 -4.836667	56.104444;
                 -4.833056	56.106944;
                 -4.829444	56.109722;
                 -4.826389	56.112778;
                 -4.824167	56.116389;
                 -4.822778	56.120278;
                 -4.825833	56.120833;
                 -4.830278	56.118889;
                 -4.835000	56.116944;
                 -4.839444	56.114722;
                 -4.843056	56.112222;
                 -4.845833	56.109167;
                 -4.849722	56.106389;
                 -4.853333	56.103889;
                 -4.856111	56.100833;
                 -4.860000	56.098333;
                 -4.863889	56.095833;
                 -4.867500	56.093056;
                 -4.872500	56.091667;
                 -4.880000	56.092500;
                 -4.884444	56.094444;
                 -4.888056	56.096944;
                 -4.891111	56.099722;
                 -4.893056	56.103056;
                 -4.893333	56.107222;
                 -4.891667	56.111389;
                 -4.891111	56.115833;
                 -4.890278	56.120556;
                 -4.889722	56.124167;
                 -4.888056	56.128056;
                 -4.887778	56.131667;
                 -4.889722	56.135000;
                 -4.891667	56.138333;
                 -4.894444	56.141389;
                 -4.898056	56.143889;
                 -4.901944	56.146111;
                 -4.906667	56.147500;
                 -4.911111	56.145278;
                 -4.912500	56.141389;
                 -4.911667	56.137500;
                 -4.910278	56.133889;
                 -4.908333	56.130556;
                 -4.907500	56.126667;
                 -4.906389	56.123056;
                 -4.906389	56.118889;
                 -4.908333	56.115278;
                 -4.910000	56.111389;
                 -4.912222	56.107778;
                 -4.911944	56.103611;
                 -4.910000	56.099722;
                 -4.908056	56.096389;
                 -4.904167	56.093889;
                 -4.901389	56.091111;
                 -4.896944	56.089167;
                 -4.893056	56.086667;
                 -4.889444	56.084167;
                 -4.886389	56.081111;
                 -4.884444	56.077778;
                 -4.883611	56.074167;
                 -4.884167	56.069722;
                 -4.887222	56.066667;
                 -4.890000	56.063333;
                 -4.892222	56.060000;
                 -4.894444	56.056389;
                 -4.896667	56.052778;
                 -4.898056	56.048611;
                 -4.898056	56.044444;
                 -4.898056	56.040556;
                 -4.897778	56.036389;
                 -4.897500	56.032222;
                 -4.897222	56.028056;
                 -4.897222	56.023889;
                 -4.898056	56.019167;
                 -4.897778	56.015278;
                 -4.898333	56.010556;
                 -4.900556	56.006944;
                 -4.901389	56.002500;
                 -4.902222	55.997778;
                 -4.900833	55.994167;
                 -4.900000	55.990556;
                 -4.898056	55.987222;
                 -4.895000	55.984167;
                 -4.900278	55.982778;
                 -4.907778	55.982500;
                 -4.914722	55.983333;
                 -4.920278	55.985000;
                 -4.926667	55.986389;
                 -4.935000	55.986667;
                 -4.940833	55.985556;
                 -4.944444	55.983056;
                 -4.942500	55.979722;
                 -4.936944	55.978056;
                 -4.930556	55.976667;
                 -4.925278	55.975278;
                 -4.920556	55.973056;
                 -4.916667	55.970556;
                 -4.913056	55.968056;
                 -4.913056	55.963889;
                 -4.913611	55.959444;
                 -4.915278	55.955278;
                 -4.917222	55.951944;
                 -4.918611	55.947778;
                 -4.920833	55.944167;
                 -4.923056	55.940556;
                 -4.926111	55.937500;
                 -4.928889	55.934444;
                 -4.931944	55.931389;
                 -4.934722	55.928333;
                 -4.937778	55.925278;
                 -4.941389	55.922778;
                 -4.943611	55.919167;
                 -4.946667	55.916111;
                 -4.948056	55.911944;
                 -4.950278	55.908333;
                 -4.950833	55.903889;
                 -4.952500	55.899722;
                 -4.953889	55.895833;
                 -4.956111	55.892222;
                 -4.957500	55.888056;
                 -4.960556	55.885000;
                 -4.963333	55.881944;
                 -4.966944	55.879444;
                 -4.970833	55.876667;
                 -4.975833	55.875278;
                 -4.979444	55.872500;
                 -4.979167	55.868333;
                 -4.977222	55.865278;
                 -4.978889	55.861111;
                 -4.983056	55.858889;
                 -4.987500	55.856944;
                 -4.991944	55.854722;
                 -4.998056	55.853611;
                 -5.004722	55.853056;
                 -5.011944	55.853056;
                 -5.019167	55.853889;
                 -5.024722	55.855556;
                 -5.031111	55.856667;
                 -5.037222	55.858056;
                 -5.041944	55.860000;
                 -5.045000	55.862778;
                 -5.047500	55.865833;
                 -5.048889	55.869444;
                 -5.050833	55.872778;
                 -5.051667	55.876389;
                 -5.053056	55.880278;
                 -5.053889	55.883889;
                 -5.054167	55.888056;
                 -5.052778	55.892222;
                 -5.050556	55.895833;
                 -5.050833	55.900000;
                 -5.052778	55.903333;
                 -5.055556	55.906111;
                 -5.058333	55.908889;
                 -5.060278	55.912222;
                 -5.060556	55.916389;
                 -5.059722	55.921111;
                 -5.061667	55.924167;
                 -5.063611	55.927500;
                 -5.064722	55.931389;
                 -5.063333	55.935278;
                 -5.063611	55.939444;
                 -5.065556	55.942778;
                 -5.068333	55.945833;
                 -5.071389	55.948611;
                 -5.072222	55.952222;
                 -5.075278	55.955278;
                 -5.077778	55.958056;
                 -5.081667	55.960556;
                 -5.084444	55.963611;
                 -5.088333	55.965833;
                 -5.094722	55.967222;
                 -5.100556	55.966111;
                 -5.103611	55.963056;
                 -5.103889	55.959444;
                 -5.103056	55.955556;
                 -5.101111	55.952222;
                 -5.098056	55.949444;
                 -5.096111	55.946111;
                 -5.094167	55.942778;
                 -5.091389	55.940000;
                 -5.089444	55.936667;
                 -5.086667	55.933611;
                 -5.084722	55.930556;
                 -5.083611	55.926667;
                 -5.081667	55.923333;
                 -5.080556	55.919722;
                 -5.078611	55.916389;
                 -5.077500	55.912778;
                 -5.076389	55.908889;
                 -5.076111	55.904722;
                 -5.078333	55.901111;
                 -5.082778	55.899167;
                 -5.090000	55.898889;
                 -5.096389	55.900278;
                 -5.101944	55.901944;
                 -5.106667	55.903889;
                 -5.111944	55.905556;
                 -5.116667	55.907500;
                 -5.122222	55.909167;
                 -5.126667	55.911389;
                 -5.131389	55.913333;
                 -5.135278	55.915833;
                 -5.139722	55.917778;
                 -5.144167	55.920000;
                 -5.148889	55.921944;
                 -5.155278	55.923056;
                 -5.161667	55.924444;
                 -5.167222	55.926111;
                 -5.173611	55.927222;
                 -5.177222	55.929722;
                 -5.179167	55.933056;
                 -5.179444	55.937222;
                 -5.178611	55.941667;
                 -5.178889	55.945833;
                 -5.179167	55.950000;
                 -5.180556	55.953611;
                 -5.179722	55.958333;
                 -5.180000	55.962500;
                 -5.180833	55.966111;
                 -5.185556	55.968333;
                 -5.190556	55.966667;
                 -5.193611	55.963611;
                 -5.196667	55.960556;
                 -5.198056	55.956389;
                 -5.200278	55.952778;
                 -5.200833	55.948333;
                 -5.201389	55.943611;
                 -5.200556	55.940000;
                 -5.200278	55.935833;
                 -5.198889	55.932222;
                 -5.198056	55.928333;
                 -5.200278	55.924722;
                 -5.203889	55.922222;
                 -5.208889	55.920556;
                 -5.213333	55.918611;
                 -5.218611	55.916944;
                 -5.221389	55.913889;
                 -5.221111	55.909722;
                 -5.222778	55.905556;
                 -5.225556	55.902500;
                 -5.230000	55.900556;
                 -5.233611	55.897778;
                 -5.237222	55.895278;
                 -5.240833	55.892778;
                 -5.241389	55.889167;
                 -5.238611	55.886111;
                 -5.235000	55.883611;
                 -5.231944	55.880833;
                 -5.229167	55.878056;
                 -5.227222	55.874722;
                 -5.228889	55.870556;
                 -5.230000	55.866389;
                 -5.226944	55.863611;
                 -5.223333	55.861111;
                 -5.218889	55.859167;
                 -5.216667	55.855833;
                 -5.215833	55.851944;
                 -5.213889	55.848889;
                 -5.211944	55.845556;
                 -5.208889	55.842500;
                 -5.206389	55.839722;
                 -5.203333	55.836944;
                 -5.202222	55.833056;
                 -5.203611	55.828889;
                 -5.207222	55.826389;
                 -5.213056	55.825278;
                 -5.218889	55.826111;
                 -5.222500	55.828611;
                 -5.226389	55.831111;
                 -5.231667	55.832500;
                 -5.239722	55.833056;
                 -5.245278	55.834722;
                 -5.248333	55.837500;
                 -5.251944	55.840000;
                 -5.255556	55.842500;
                 -5.261111	55.844167;
                 -5.267500	55.845278;
                 -5.274722	55.846111;
                 -5.282222	55.845833;
                 -5.289444	55.845833;
                 -5.294167	55.846944;
                 -5.298889	55.848889;
                 -5.302500	55.851389;
                 -5.308056	55.853056;
                 -5.316111	55.853333;
                 -5.321667	55.854167;
                 -5.324722	55.856944;
                 -5.325000	55.861111;
                 -5.324444	55.864722;
                 -5.326389	55.868056;
                 -5.330000	55.870556;
                 -5.333611	55.873056;
                 -5.337778	55.875556;
                 -5.341389	55.878056;
                 -5.345000	55.880556;
                 -5.347778	55.883333;
                 -5.350833	55.886111;
                 -5.351944	55.890000;
                 -5.351944	55.894167;
                 -5.353889	55.897500;
                 -5.355278	55.901111;
                 -5.355556	55.905278;
                 -5.355833	55.909444;
                 -5.353611	55.913056;
                 -5.351944	55.916944;
                 -5.352222	55.921111;
                 -5.352500	55.925278;
                 -5.351944	55.928889;
                 -5.350556	55.933056;
                 -5.349167	55.937222;
                 -5.346944	55.940833;
                 -5.345000	55.944167;
                 -5.342778	55.947778;
                 -5.343056	55.951944;
                 -5.345833	55.955000;
                 -5.351389	55.956389;
                 -5.356944	55.958056;
                 -5.362222	55.959722;
                 -5.360833	55.963889;
                 -5.357222	55.966389;
                 -5.355833	55.970556;
                 -5.355278	55.975000;
                 -5.354444	55.979722;
                 -5.353056	55.983611;
                 -5.350000	55.986667;
                 -5.347500	55.989722;
                 -5.343889	55.992500;
                 -5.340833	55.995556;
                 -5.337222	55.998056;
                 -5.334167	56.001111;
                 -5.331389	56.004167;
                 -5.327500	56.006944;
                 -5.323889	56.009444;
                 -5.321111	56.012500;
                 -5.318056	56.015556;
                 -5.315833	56.019167;
                 -5.313611	56.022778;
                 -5.312500	56.026944;
                 -5.311111	56.030833;
                 -5.310278	56.035556;
                 -5.309167	56.039444;
                 -5.306944	56.043056;
                 -5.303889	56.046111;
                 -5.301111	56.049167;
                 -5.298056	56.052222;
                 -5.293611	56.054444;
                 -5.289167	56.056389;
                 -5.285556	56.059167;
                 -5.283333	56.062778;
                 -5.280556	56.065833;
                 -5.276111	56.067778;
                 -5.271667	56.070000;
                 -5.267222	56.071944;
                 -5.261944	56.073611;
                 -5.256944	56.075278;
                 -5.252500	56.077222;
                 -5.247222	56.078889;
                 -5.243056	56.081111;
                 -5.238611	56.083056;
                 -5.233889	56.085278;
                 -5.229444	56.087222;
                 -5.225833	56.090000;
                 -5.222222	56.092500;
                 -5.218611	56.095000;
                 -5.216389	56.098611;
                 -5.215833	56.103333;
                 -5.214444	56.107222;
                 -5.212222	56.110833;
                 -5.210000	56.114444;
                 -5.206944	56.117500;
                 -5.203333	56.120000;
                 -5.198889	56.122222;
                 -5.194444	56.124167;
                 -5.189167	56.125833;
                 -5.184167	56.127500;
                 -5.178333	56.128611;
                 -5.172500	56.129722;
                 -5.166389	56.130833;
                 -5.159722	56.131389;
                 -5.153056	56.131944;
                 -5.146944	56.133056;
                 -5.141944	56.134722;
                 -5.137500	56.136667;
                 -5.133056	56.138889;
                 -5.128333	56.140833;
                 -5.123889	56.143056;
                 -5.118889	56.144722;
                 -5.114444	56.146667;
                 -5.110000	56.148889;
                 -5.105556	56.151389;
                 -5.101944	56.153889;
                 -5.097500	56.156111;
                 -5.091667	56.157222;
                 -5.085000	56.156944;
                 -5.079722	56.158333;
                 -5.077500	56.161944;
                 -5.078611	56.165833;
                 -5.078889	56.169722;
                 -5.079167	56.173889;
                 -5.078333	56.178611;
                 -5.076944	56.182500;
                 -5.073889	56.185556;
                 -5.071111	56.188889;
                 -5.068889	56.192222;
                 -5.066667	56.195833;
                 -5.064444	56.199444;
                 -5.061389	56.202500;
                 -5.057778	56.205000;
                 -5.053333	56.207222;
                 -5.048889	56.209167;
                 -5.043611	56.210833;
                 -5.038611	56.212500;
                 -5.033333	56.214167;
                 -5.028056	56.215556;
                 -5.023056	56.217222;
                 -5.016944	56.218889;
                 -5.010278	56.219444;
                 -5.004444	56.220556;
                 -4.998889	56.221944;
                 -4.995278	56.224722;
                 -4.991667	56.227222;
                 -4.987222	56.229444;
                 -4.983333	56.231944;
                 -4.978333	56.233611;
                 -4.973056	56.235000;
                 -4.968056	56.236667;
                 -4.962778	56.238333;
                 -4.957500	56.239722;
                 -4.952500	56.241389;
                 -4.947778	56.243611;
                 -4.943333	56.245556;
                 -4.939722	56.248056;
                 -4.935278	56.250278;
                 -4.932500	56.253333;
                 -4.929167	56.256389;
                 -4.928611	56.260000;
                 -4.932500	56.262500;
                 -4.939167	56.261944;
                 -4.945000	56.260833;
                 -4.950556	56.259167;
                 -4.955556	56.257778;
                 -4.960556	56.256111;
                 -4.965278	56.253889;
                 -4.970556	56.252500;
                 -4.974722	56.250278;
                 -4.979167	56.248333;
                 -4.983889	56.246111;
                 -4.987500	56.243611;
                 -4.991111	56.241111;
                 -4.995833	56.238333;
                 -4.999444	56.235833;
                 -5.003889	56.233889;
                 -5.007500	56.231111;
                 -5.012778	56.229722;
                 -5.018889	56.228611;
                 -5.026111	56.228333;
                 -5.033056	56.227778;
                 -5.040278	56.227778;
                 -5.045833	56.229444;
                 -5.051389	56.231111;
                 -5.058611	56.231667;
                 -5.065556	56.231111;
                 -5.070556	56.229722;
                 -5.073611	56.226389;
                 -5.076389	56.223333;
                 -5.077778	56.219444;
                 -5.079444	56.215278;
                 -5.081667	56.211667;
                 -5.083056	56.207778;
                 -5.085278	56.204167;
                 -5.087500	56.200556;
                 -5.089722	56.196944;
                 -5.091667	56.193333;
                 -5.093333	56.189444;
                 -5.095556	56.185833;
                 -5.097778	56.182222;
                 -5.100556	56.179167;
                 -5.103611	56.176111;
                 -5.106389	56.173056;
                 -5.109444	56.170000;
                 -5.112222	56.166944;
                 -5.115833	56.164167;
                 -5.119722	56.161667;
                 -5.123889	56.159444;
                 -5.128333	56.157500;
                 -5.134444	56.155833;
                 -5.140556	56.154722;
                 -5.146389	56.153611;
                 -5.153056	56.153056;
                 -5.158889	56.151944;
                 -5.165000	56.150833;
                 -5.170833	56.149722;
                 -5.176667	56.148611;
                 -5.182222	56.147222;
                 -5.187222	56.145556;
                 -5.192500	56.143889;
                 -5.196667	56.141944;
                 -5.201111	56.139722;
                 -5.205833	56.137500;
                 -5.209444	56.135000;
                 -5.213889	56.133056;
                 -5.217500	56.130278;
                 -5.221944	56.128333;
                 -5.225556	56.125556;
                 -5.230000	56.123611;
                 -5.234444	56.121389;
                 -5.238889	56.119444;
                 -5.242500	56.116944;
                 -5.246944	56.114722;
                 -5.250556	56.112222;
                 -5.254444	56.109444;
                 -5.258889	56.107500;
                 -5.262500	56.105000;
                 -5.263889	56.100833;
                 -5.265833	56.097222;
                 -5.268889	56.094167;
                 -5.272500	56.091667;
                 -5.277222	56.089444;
                 -5.281389	56.087500;
                 -5.285833	56.085278;
                 -5.289444	56.082778;
                 -5.293333	56.080000;
                 -5.295278	56.076667;
                 -5.297500	56.073056;
                 -5.299444	56.069444;
                 -5.301667	56.065833;
                 -5.303889	56.062222;
                 -5.307500	56.059722;
                 -5.311389	56.056944;
                 -5.316389	56.055556;
                 -5.323333	56.054722;
                 -5.329722	56.054167;
                 -5.335556	56.053056;
                 -5.340000	56.051111;
                 -5.341667	56.046944;
                 -5.343056	56.042778;
                 -5.343611	56.038333;
                 -5.343611	56.034167;
                 -5.343889	56.029722;
                 -5.345556	56.025556;
                 -5.348333	56.022500;
                 -5.352778	56.020278;
                 -5.357778	56.018889;
                 -5.361667	56.016111;
                 -5.365833	56.014167;
                 -5.369722	56.011389;
                 -5.373333	56.008889;
                 -5.376944	56.006389;
                 -5.381389	56.004167;
                 -5.385833	56.002222;
                 -5.390833	56.000556;
                 -5.397778	56.000000;
                 -5.404167	56.000278;
                 -5.409722	56.001667;
                 -5.416111	56.003056;
                 -5.423333	56.003889;
                 -5.428611	56.005556;
                 -5.431667	56.008333;
                 -5.432778	56.011944;
                 -5.432222	56.016667;
                 -5.433333	56.020278;
                 -5.435278	56.023611;
                 -5.440833	56.025278;
                 -5.445000	56.023056;
                 -5.446667	56.019167;
                 -5.448611	56.015556;
                 -5.450278	56.011389;
                 -5.450556	56.007778;
                 -5.449444	56.004167;
                 -5.448611	56.000278;
                 -5.447222	55.996667;
                 -5.446389	55.992778;
                 -5.446111	55.988611;
                 -5.444722	55.985000;
                 -5.444444	55.980833;
                 -5.444444	55.976667;
                 -5.445833	55.972778;
                 -5.448056	55.969167;
                 -5.450000	55.965556;
                 -5.452222	55.961944;
                 -5.454444	55.958333;
                 -5.452500	55.955000;
                 -5.448611	55.952500;
                 -5.444722	55.950000;
                 -5.439444	55.948611;
                 -5.434722	55.946389;
                 -5.430278	55.944444;
                 -5.428056	55.941111;
                 -5.426111	55.937778;
                 -5.425000	55.934167;
                 -5.423889	55.930278;
                 -5.422778	55.926667;
                 -5.421667	55.923056;
                 -5.421389	55.918889;
                 -5.421111	55.914722;
                 -5.420278	55.910833;
                 -5.418889	55.907222;
                 -5.416944	55.903889;
                 -5.415000	55.900556;
                 -5.413056	55.897222;
                 -5.410278	55.894444;
                 -5.406389	55.891944;
                 -5.402500	55.889444;
                 -5.401667	55.885833;
                 -5.405278	55.883056;
                 -5.407222	55.879722;
                 -5.404444	55.876667;
                 -5.398889	55.875000;
                 -5.395278	55.872500;
                 -5.396667	55.868611;
                 -5.400278	55.866111;
                 -5.403056	55.862778;
                 -5.402222	55.859167;
                 -5.397222	55.858056;
                 -5.392778	55.856111;
                 -5.388056	55.853889;
                 -5.384167	55.851389;
                 -5.380556	55.849167;
                 -5.376111	55.846944;
                 -5.372222	55.844444;
                 -5.369444	55.841667;
                 -5.366389	55.838889;
                 -5.362778	55.836389;
                 -5.360000	55.833333;
                 -5.357222	55.830556;
                 -5.354167	55.827500;
                 -5.350556	55.825278;
                 -5.347778	55.822222;
                 -5.344722	55.819444;
                 -5.341944	55.816667;
                 -5.339167	55.813611;
                 -5.336111	55.810833;
                 -5.332500	55.808333;
                 -5.329722	55.805278;
                 -5.327500	55.802222;
                 -5.325556	55.798889;
                 -5.323611	55.795556;
                 -5.323333	55.793056;
                 -5.320278	55.790278;
                 -5.318333	55.786944;
                 -5.318056	55.782778;
                 -5.317222	55.779167;
                 -5.317778	55.774444;
                 -5.319722	55.771111;
                 -5.321389	55.766944;
                 -5.324167	55.763889;
                 -5.327778	55.761389;
                 -5.333056	55.759722;
                 -5.339722	55.759167;
                 -5.346111	55.758333;
                 -5.353611	55.758333;
                 -5.359444	55.757222;
                 -5.365278	55.756111;
                 -5.370278	55.754444;
                 -5.375556	55.752778;
                 -5.380000	55.750833;
                 -5.383611	55.748056;
                 -5.388056	55.746111;
                 -5.392222	55.743889;
                 -5.396111	55.741389;
                 -5.400278	55.739167;
                 -5.404167	55.736667;
                 -5.407500	55.734167;
                 -5.411111	55.731389;
                 -5.414722	55.728889;
                 -5.417778	55.725833;
                 -5.420556	55.722778;
                 -5.425000	55.720556;
                 -5.428333	55.718056;
                 -5.432778	55.715833;
                 -5.437222	55.713889;
                 -5.440833	55.711389;
                 -5.444444	55.708611;
                 -5.447500	55.705556;
                 -5.447222	55.701389;
                 -5.446111	55.697778;
                 -5.447500	55.693611;
                 -5.450556	55.690556;
                 -5.453056	55.687500;
                 -5.456667	55.685000;
                 -5.458611	55.682222;
                 -5.460833	55.678611;
                 -5.463889	55.675556;
                 -5.466111	55.671944;
                 -5.468889	55.668889;
                 -5.470833	55.665278;
                 -5.471667	55.660833;
                 -5.471389	55.656667;
                 -5.470278	55.652778;
                 -5.473056	55.649722;
                 -5.478333	55.648333;
                 -5.483333	55.646667;
                 -5.486389	55.643611;
                 -5.486944	55.638889;
                 -5.485833	55.635278;
                 -5.483611	55.631944;
                 -5.481667	55.628611;
                 -5.481667	55.624444;
                 -5.484444	55.621389;
                 -5.487222	55.618333;
                 -5.490278	55.615278;
                 -5.490833	55.610556;
                 -5.489722	55.606944;
                 -5.487778	55.603611;
                 -5.485000	55.600833;
                 -5.480278	55.598611;
                 -5.475556	55.596667;
                 -5.470278	55.595000;
                 -5.465556	55.593056;
                 -5.461944	55.590556;
                 -5.457222	55.588056;
                 -5.455278	55.584722;
                 -5.454444	55.581111;
                 -5.456667	55.577500;
                 -5.459167	55.574444;
                 -5.464444	55.572778;
                 -5.471667	55.572500;
                 -5.479722	55.573056;
                 -5.484167	55.570833;
                 -5.487778	55.568333;
                 -5.490000	55.564722;
                 -5.491111	55.560556;
                 -5.489167	55.557222;
                 -5.486389	55.554444;
                 -5.485278	55.550833;
                 -5.485833	55.546111;
                 -5.488056	55.542500;
                 -5.487778	55.538333;
                 -5.487500	55.534444;
                 -5.488333	55.529722;
                 -5.491944	55.527222;
                 -5.496111	55.525000;
                 -5.501111	55.523333;
                 -5.505556	55.521389;
                 -5.506389	55.516667;
                 -5.506944	55.512222;
                 -5.506667	55.508056;
                 -5.506389	55.503889;
                 -5.508611	55.500278;
                 -5.510833	55.496667;
                 -5.511944	55.492778;
                 -5.511111	55.488889;
                 -5.509722	55.485278;
                 -5.512778	55.482222;
                 -5.516111	55.479722;
                 -5.521111	55.478056;
                 -5.527222	55.476944;
                 -5.532222	55.475278;
                 -5.536667	55.473056;
                 -5.538611	55.469444;
                 -5.540000	55.465556;
                 -5.540833	55.460833;
                 -5.540556	55.456667;
                 -5.541111	55.452222;
                 -5.541667	55.447778;
                 -5.542222	55.443056;
                 -5.543611	55.438889;
                 -5.545833	55.435556;
                 -5.549444	55.432778;
                 -5.553056	55.430278;
                 -5.558056	55.428611;
                 -5.564722	55.428056;
                 -5.570278	55.426944;
                 -5.576111	55.425833;
                 -5.580556	55.423611;
                 -5.584167	55.421111;
                 -5.586944	55.418056;
                 -5.580833	55.416667;
                 -5.574167	55.417500;
                 -5.566944	55.417500;
                 -5.559444	55.416667;
                 -5.555278	55.414722;
                 -5.552222	55.411944;
                 -5.551111	55.408056;
                 -5.546667	55.406111;
                 -5.543056	55.403611;
                 -5.539167	55.401111;
                 -5.535278	55.398611;
                 -5.532778	55.395833;
                 -5.530833	55.392500;
                 -5.528611	55.389167;
                 -5.527500	55.385556;
                 -5.527222	55.381389;
                 -5.527222	55.377222;
                 -5.525278	55.373889;
                 -5.522222	55.371111;
                 -5.518611	55.368611;
                 -5.515556	55.365833;
                 -5.514722	55.361944;
                 -5.517500	55.358889;
                 -5.520278	55.355833;
                 -5.523056	55.352778;
                 -5.525833	55.349722;
                 -5.529444	55.346944;
                 -5.532500	55.343889;
                 -5.535278	55.340833;
                 -5.539444	55.338889;
                 -5.543333	55.336111;
                 -5.547500	55.334167;
                 -5.552500	55.332500;
                 -5.557500	55.330833;
                 -5.561944	55.328889;
                 -5.564722	55.325556;
                 -5.566111	55.321667;
                 -5.566944	55.316944;
                 -5.571667	55.315278;
                 -5.577500	55.314167;
                 -5.581944	55.312222;
                 -5.586944	55.310556;
                 -5.591944	55.308889;
                 -5.597778	55.307778;
                 -5.603889	55.306667;
                 -5.610000	55.306111;
                 -5.615833	55.305000;
                 -5.622500	55.304444;
                 -5.629167	55.303611;
                 -5.635000	55.302500;
                 -5.640000	55.300833;
                 -5.645833	55.299722;
                 -5.653056	55.299722;
                 -5.661111	55.300000;
                 -5.668333	55.300833;
                 -5.674167	55.299722;
                 -5.679722	55.298611;
                 -5.685556	55.297500;
                 -5.690556	55.295833;
                 -5.695000	55.293611;
                 -5.699167	55.291667;
                 -5.702778	55.288889;
                 -5.706944	55.286944;
                 -5.712222	55.285278;
                 -5.717222	55.283611;
                 -5.724444	55.283611;
                 -5.731667	55.284167;
                 -5.739722	55.284722;
                 -5.746944	55.284444;
                 -5.754167	55.284167;
                 -5.761389	55.284167;
                 -5.769444	55.284444;
                 -5.777500	55.284722;
                 -5.784722	55.285556;
                 -5.791667	55.286389;
                 -5.798056	55.287500;
                 -5.803333	55.289167;
                 -5.808056	55.291111;
                 -5.812778	55.293333;
                 -5.815556	55.296111;
                 -5.819444	55.298611;
                 -5.820278	55.302222;
                 -5.821667	55.305833;
                 -5.821667	55.310000;
                 -5.821389	55.314722;
                 -5.820556	55.319167;
                 -5.819444	55.323333;
                 -5.817222	55.326944;
                 -5.815278	55.330556;
                 -5.813611	55.334444;
                 -5.815000	55.338333;
                 -5.817500	55.341111;
                 -5.819722	55.344444;
                 -5.820833	55.348056;
                 -5.819444	55.352222;
                 -5.817500	55.355833;
                 -5.816111	55.360000;
                 -5.813889	55.363333;
                 -5.811389	55.365556;
                 -5.808611	55.368611;
                 -5.807778	55.373333;
                 -5.808056	55.377500;
                 -5.806111	55.381111;
                 -5.803889	55.384722;
                 -5.802778	55.388611;
                 -5.800556	55.392222;
                 -5.797222	55.395000;
                 -5.792778	55.396944;
                 -5.787500	55.398611;
                 -5.783333	55.400833;
                 -5.779167	55.402778;
                 -5.774722	55.405000;
                 -5.770278	55.406944;
                 -5.767500	55.410278;
                 -5.768611	55.413889;
                 -5.765833	55.416944;
                 -5.760833	55.418611;
                 -5.756667	55.420556;
                 -5.750833	55.421944;
                 -5.745556	55.423333;
                 -5.739722	55.424444;
                 -5.733889	55.425833;
                 -5.729722	55.427778;
                 -5.726944	55.430833;
                 -5.725278	55.435000;
                 -5.722778	55.438056;
                 -5.719722	55.441111;
                 -5.718611	55.445278;
                 -5.716944	55.449444;
                 -5.716667	55.453889;
                 -5.716667	55.458056;
                 -5.716944	55.462222;
                 -5.718056	55.465833;
                 -5.719167	55.469722;
                 -5.719444	55.473611;
                 -5.721389	55.476944;
                 -5.722778	55.480833;
                 -5.722778	55.485000;
                 -5.722222	55.489444;
                 -5.721667	55.493889;
                 -5.720278	55.498056;
                 -5.718889	55.502222;
                 -5.718333	55.506667;
                 -5.717778	55.511389;
                 -5.718056	55.515556;
                 -5.716667	55.519444;
                 -5.715278	55.523611;
                 -5.713056	55.527222;
                 -5.711111	55.530833;
                 -5.708333	55.533889;
                 -5.706944	55.537778;
                 -5.707222	55.541944;
                 -5.709167	55.545278;
                 -5.712222	55.548333;
                 -5.714167	55.551389;
                 -5.716667	55.554444;
                 -5.719722	55.557222;
                 -5.721667	55.560556;
                 -5.722778	55.564167;
                 -5.723056	55.568333;
                 -5.720833	55.571944;
                 -5.718333	55.575000;
                 -5.714722	55.577778;
                 -5.710833	55.580278;
                 -5.707500	55.582778;
                 -5.704722	55.586111;
                 -5.701667	55.589167;
                 -5.699722	55.592778;
                 -5.698333	55.596667;
                 -5.696944	55.600833;
                 -5.694722	55.604444;
                 -5.692500	55.608056;
                 -5.689722	55.611111;
                 -5.686944	55.614167;
                 -5.684167	55.617222;
                 -5.682778	55.621389;
                 -5.680556	55.625000;
                 -5.679444	55.628889;
                 -5.678056	55.633056;
                 -5.676389	55.637222;
                 -5.674444	55.640833;
                 -5.673056	55.644722;
                 -5.671944	55.648889;
                 -5.670278	55.652778;
                 -5.669722	55.657500;
                 -5.669167	55.661944;
                 -5.670278	55.665833;
                 -5.673056	55.668611;
                 -5.676111	55.671389;
                 -5.680556	55.673611;
                 -5.675556	55.675278;
                 -5.671111	55.677222;
                 -5.666111	55.678889;
                 -5.660833	55.680556;
                 -5.655833	55.682222;
                 -5.650833	55.683611;
                 -5.646389	55.685833;
                 -5.642778	55.688333;
                 -5.639444	55.690000;
                 -5.635833	55.692778;
                 -5.631944	55.695278;
                 -5.629444	55.698333;
                 -5.626389	55.701389;
                 -5.623611	55.704722;
                 -5.620556	55.707778;
                 -5.619444	55.711667;
                 -5.617222	55.715278;
                 -5.615000	55.718889;
                 -5.613056	55.722500;
                 -5.610833	55.726111;
                 -5.607778	55.729167;
                 -5.604167	55.731667;
                 -5.600000	55.733889;
                 -5.596389	55.736389;
                 -5.591944	55.738611;
                 -5.588333	55.741111;
                 -5.585556	55.744167;
                 -5.583611	55.747778;
                 -5.581389	55.751389;
                 -5.578333	55.754444;
                 -5.574722	55.757222;
                 -5.570278	55.759167;
                 -5.565278	55.761389;
                 -5.560000	55.762778;
                 -5.555000	55.764444;
                 -5.550556	55.766667;
                 -5.545278	55.768333;
                 -5.541111	55.770278;
                 -5.536944	55.772500;
                 -5.533333	55.775000;
                 -5.528889	55.777222;
                 -5.526667	55.780833;
                 -5.523611	55.783889;
                 -5.519167	55.785833;
                 -5.513333	55.786944;
                 -5.507500	55.788056;
                 -5.504167	55.790833;
                 -5.500556	55.793333;
                 -5.496111	55.795278;
                 -5.491667	55.797500;
                 -5.487222	55.799722;
                 -5.484444	55.802778;
                 -5.482222	55.806389;
                 -5.479167	55.809722;
                 -5.476667	55.812778;
                 -5.472778	55.815556;
                 -5.470000	55.818611;
                 -5.466944	55.821667;
                 -5.464722	55.825278;
                 -5.461944	55.828333;
                 -5.458611	55.830833;
                 -5.455556	55.833889;
                 -5.452500	55.837222;
                 -5.451944	55.841667;
                 -5.457500	55.843333;
                 -5.462778	55.841667;
                 -5.465556	55.838611;
                 -5.468333	55.835556;
                 -5.472778	55.833333;
                 -5.477778	55.831667;
                 -5.483611	55.830556;
                 -5.485833	55.827222;
                 -5.485556	55.823056;
                 -5.488611	55.820000;
                 -5.493056	55.817778;
                 -5.498056	55.816111;
                 -5.503056	55.814444;
                 -5.507500	55.812500;
                 -5.510556	55.809444;
                 -5.513333	55.806389;
                 -5.516944	55.803611;
                 -5.521111	55.801667;
                 -5.525556	55.799444;
                 -5.528611	55.796389;
                 -5.530833	55.792778;
                 -5.533333	55.789722;
                 -5.536389	55.786667;
                 -5.540000	55.784167;
                 -5.543611	55.781389;
                 -5.548056	55.779444;
                 -5.552500	55.777222;
                 -5.556944	55.775000;
                 -5.561944	55.773611;
                 -5.567222	55.771944;
                 -5.573056	55.770833;
                 -5.578056	55.769167;
                 -5.581667	55.766667;
                 -5.585278	55.763889;
                 -5.589722	55.761944;
                 -5.594722	55.760278;
                 -5.600556	55.759167;
                 -5.607778	55.758889;
                 -5.611944	55.760556;
                 -5.613056	55.764167;
                 -5.610833	55.767778;
                 -5.607222	55.770556;
                 -5.605278	55.773056;
                 -5.612500	55.773889;
                 -5.617222	55.775833;
                 -5.617222	55.780000;
                 -5.625556	55.780278;
                 -5.632778	55.780278;
                 -5.640833	55.780556;
                 -5.647222	55.781667;
                 -5.650278	55.784722;
                 -5.653056	55.787500;
                 -5.656111	55.790556;
                 -5.658056	55.793611;
                 -5.661667	55.796111;
                 -5.664444	55.799167;
                 -5.668333	55.801389;
                 -5.672222	55.803889;
                 -5.674167	55.807222;
                 -5.674444	55.811389;
                 -5.673611	55.815833;
                 -5.672500	55.820000;
                 -5.671667	55.824722;
                 -5.669722	55.828056;
                 -5.668333	55.832222;
                 -5.666667	55.836389;
                 -5.669722	55.839167;
                 -5.670833	55.843056;
                 -5.668056	55.846111;
                 -5.664444	55.848611;
                 -5.660000	55.850833;
                 -5.657222	55.853889;
                 -5.655000	55.857500;
                 -5.652222	55.860556;
                 -5.650278	55.864167;
                 -5.647222	55.867222;
                 -5.644167	55.870278;
                 -5.641389	55.873333;
                 -5.638333	55.876389;
                 -5.635000	55.878889;
                 -5.631944	55.882222;
                 -5.628333	55.884722;
                 -5.626111	55.888333;
                 -5.625833	55.892778;
                 -5.622778	55.895833;
                 -5.619167	55.898611;
                 -5.615556	55.901111;
                 -5.611944	55.903611;
                 -5.607500	55.905833;
                 -5.603056	55.908056;
                 -5.598611	55.910000;
                 -5.595000	55.912778;
                 -5.591667	55.915278;
                 -5.587778	55.917778;
                 -5.583611	55.920000;
                 -5.579722	55.922500;
                 -5.576111	55.925000;
                 -5.573889	55.928611;
                 -5.575278	55.931389;
                 -5.581944	55.930833;
                 -5.587222	55.929167;
                 -5.591667	55.927222;
                 -5.595833	55.925000;
                 -5.600000	55.923056;
                 -5.605278	55.921389;
                 -5.610556	55.919722;
                 -5.614167	55.917222;
                 -5.617778	55.914444;
                 -5.621389	55.911944;
                 -5.624167	55.908889;
                 -5.628056	55.906389;
                 -5.631667	55.903611;
                 -5.636111	55.901667;
                 -5.640278	55.899444;
                 -5.646111	55.898333;
                 -5.652778	55.897778;
                 -5.658611	55.896667;
                 -5.661667	55.893333;
                 -5.664444	55.890278;
                 -5.668056	55.887778;
                 -5.671667	55.885278;
                 -5.676667	55.883611;
                 -5.683333	55.883889;
                 -5.688056	55.885833;
                 -5.691667	55.888333;
                 -5.692778	55.891944;
                 -5.692222	55.896667;
                 -5.692500	55.900833;
                 -5.693611	55.904444;
                 -5.693889	55.908611;
                 -5.692500	55.912778;
                 -5.691111	55.916667;
                 -5.689722	55.920833;
                 -5.690833	55.924444;
                 -5.690278	55.929167;
                 -5.687500	55.932222;
                 -5.683889	55.934722;
                 -5.679444	55.936944;
                 -5.675833	55.939444;
                 -5.671389	55.941667;
                 -5.667778	55.944167;
                 -5.663333	55.946389;
                 -5.659722	55.948889;
                 -5.655278	55.951111;
                 -5.651667	55.953611;
                 -5.648056	55.956111;
                 -5.644444	55.958889;
                 -5.640833	55.961389;
                 -5.637222	55.963889;
                 -5.633611	55.966667;
                 -5.630556	55.969722;
                 -5.627778	55.972778;
                 -5.624167	55.975278;
                 -5.621389	55.978333;
                 -5.619167	55.981944;
                 -5.616111	55.985000;
                 -5.614167	55.988611;
                 -5.611389	55.991944;
                 -5.610000	55.995833;
                 -5.606944	55.998889;
                 -5.603333	56.001667;
                 -5.598056	56.003056;
                 -5.593056	56.004722;
                 -5.588611	56.006944;
                 -5.585000	56.009444;
                 -5.581389	56.012222;
                 -5.578611	56.015278;
                 -5.579722	56.018889;
                 -5.585833	56.020000;
                 -5.592778	56.019444;
                 -5.599444	56.018889;
                 -5.597500	56.020556;
                 -5.591667	56.021667;
                 -5.587222	56.023889;
                 -5.583611	56.026389;
                 -5.580556	56.029444;
                 -5.576944	56.032222;
                 -5.573333	56.034722;
                 -5.570556	56.037778;
                 -5.576111	56.038611;
                 -5.582778	56.037778;
                 -5.587778	56.036111;
                 -5.593056	56.034722;
                 -5.597500	56.032500;
                 -5.600278	56.029444;
                 -5.603889	56.026944;
                 -5.609167	56.025278;
                 -5.615000	56.024167;
                 -5.620000	56.022500;
                 -5.625278	56.020833;
                 -5.623333	56.017500;
                 -5.620278	56.014722;
                 -5.622500	56.011111;
                 -5.626111	56.008611;
                 -5.630556	56.006389;
                 -5.634167	56.003889;
                 -5.638056	56.001111;
                 -5.640000	55.997500;
                 -5.642222	55.994167;
                 -5.644167	55.990556;
                 -5.648056	55.987778;
                 -5.653056	55.986389;
                 -5.658889	55.985000;
                 -5.664722	55.983889;
                 -5.669167	55.981944;
                 -5.672222	55.978889;
                 -5.673611	55.974722;
                 -5.675556	55.971111;
                 -5.677778	55.967500;
                 -5.678333	55.963889;
                 -5.678056	55.959722;
                 -5.676667	55.956111;
                 -5.676389	55.951944;
                 -5.679444	55.948889;
                 -5.683889	55.946667;
                 -5.687500	55.944167;
                 -5.689722	55.940556;
                 -5.690833	55.936389;
                 -5.694444	55.933889;
                 -5.701111	55.933333;
                 -5.706667	55.934722;
                 -5.709722	55.937778;
                 -5.709167	55.941389;
                 -5.706944	55.945000;
                 -5.704167	55.948056;
                 -5.702778	55.952222;
                 -5.705556	55.955000;
                 -5.708611	55.957778;
                 -5.710556	55.961111;
                 -5.710278	55.964722;
                 -5.708056	55.968333;
                 -5.705833	55.971944;
                 -5.703611	55.975556;
                 -5.700833	55.978611;
                 -5.698056	55.981667;
                 -5.694444	55.984444;
                 -5.691389	55.987500;
                 -5.688611	55.990556;
                 -5.685000	55.993056;
                 -5.682222	55.996111;
                 -5.678333	55.998889;
                 -5.674722	56.001389;
                 -5.670278	56.003611;
                 -5.666667	56.006111;
                 -5.663056	56.008611;
                 -5.661111	56.012222;
                 -5.658333	56.015278;
                 -5.653056	56.016944;
                 -5.646944	56.018056;
                 -5.643333	56.020833;
                 -5.643889	56.024722;
                 -5.645833	56.028056;
                 -5.646667	56.031944;
                 -5.646111	56.036389;
                 -5.644167	56.040000;
                 -5.641944	56.043611;
                 -5.638056	56.046111;
                 -5.635556	56.049167;
                 -5.632500	56.052500;
                 -5.627500	56.053889;
                 -5.623056	56.056111;
                 -5.619444	56.058611;
                 -5.617222	56.062222;
                 -5.614167	56.065278;
                 -5.610556	56.068056;
                 -5.607778	56.071111;
                 -5.604167	56.073611;
                 -5.601389	56.076667;
                 -5.597778	56.079444;
                 -5.593333	56.081389;
                 -5.588889	56.083611;
                 -5.581944	56.084167;
                 -5.576667	56.082500;
                 -5.571111	56.080833;
                 -5.566389	56.078889;
                 -5.560833	56.077222;
                 -5.554444	56.076111;
                 -5.548889	56.074444;
                 -5.541667	56.073611;
                 -5.535556	56.074722;
                 -5.532778	56.077778;
                 -5.535000	56.081111;
                 -5.537500	56.084167;
                 -5.536389	56.088056;
                 -5.534167	56.091667;
                 -5.536111	56.095000;
                 -5.541667	56.096667;
                 -5.547500	56.095556;
                 -5.553333	56.095278;
                 -5.550556	56.098333;
                 -5.550000	56.101944;
                 -5.554722	56.104167;
                 -5.559444	56.106111;
                 -5.563889	56.108056;
                 -5.564444	56.111389;
                 -5.563611	56.115833;
                 -5.563056	56.120556;
                 -5.561111	56.124167;
                 -5.558056	56.127222;
                 -5.555278	56.130278;
                 -5.551389	56.132778;
                 -5.547778	56.135556;
                 -5.544167	56.138056;
                 -5.540556	56.140556;
                 -5.536944	56.143333;
                 -5.533889	56.146389;
                 -5.531111	56.149444;
                 -5.528056	56.152500;
                 -5.525833	56.156111;
                 -5.523056	56.159167;
                 -5.520278	56.162222;
                 -5.516667	56.164722;
                 -5.513611	56.168056;
                 -5.511389	56.171389;
                 -5.509167	56.175000;
                 -5.508889	56.179722;
                 -5.509722	56.183333;
                 -5.515278	56.185000;
                 -5.521944	56.184444;
                 -5.526389	56.182222;
                 -5.528611	56.178611;
                 -5.531389	56.175556;
                 -5.534444	56.172500;
                 -5.538056	56.170000;
                 -5.543889	56.168889;
                 -5.550000	56.167778;
                 -5.555278	56.166111;
                 -5.558056	56.163056;
                 -5.560000	56.159444;
                 -5.562222	56.155833;
                 -5.565278	56.152778;
                 -5.568056	56.149722;
                 -5.571667	56.146944;
                 -5.577500	56.145833;
                 -5.585278	56.145833;
                 -5.586389	56.141667;
                 -5.587778	56.137778;
                 -5.590833	56.134444;
                 -5.595278	56.132500;
                 -5.598889	56.129722;
                 -5.601111	56.126389;
                 -5.601667	56.121667;
                 -5.605833	56.119722;
                 -5.612778	56.118889;
                 -5.610000	56.120833;
                 -5.611944	56.123889;
                 -5.617500	56.125556;
                 -5.620278	56.128611;
                 -5.621667	56.132222;
                 -5.620000	56.136389;
                 -5.616389	56.138889;
                 -5.612778	56.141389;
                 -5.610000	56.144444;
                 -5.607222	56.147778;
                 -5.605000	56.151111;
                 -5.604167	56.155833;
                 -5.603056	56.160000;
                 -5.599444	56.162500;
                 -5.595833	56.165000;
                 -5.592778	56.168056;
                 -5.591389	56.172222;
                 -5.591667	56.176389;
                 -5.588611	56.179444;
                 -5.584167	56.181667;
                 -5.579722	56.183611;
                 -5.577500	56.187222;
                 -5.577222	56.191667;
                 -5.577500	56.195833;
                 -5.576111	56.199167;
                 -5.572500	56.201667;
                 -5.567222	56.203333;
                 -5.561944	56.205000;
                 -5.556944	56.206389;
                 -5.553333	56.209167;
                 -5.551111	56.212778;
                 -5.550556	56.217222;
                 -5.552500	56.220556;
                 -5.555556	56.223333;
                 -5.560000	56.225556;
                 -5.565556	56.226944;
                 -5.573056	56.227778;
                 -5.581111	56.228333;
                 -5.588333	56.228889;
                 -5.593889	56.230556;
                 -5.597778	56.233056;
                 -5.599722	56.236389;
                 -5.600833	56.240000;
                 -5.599722	56.244167;
                 -5.598056	56.248333;
                 -5.595833	56.251667;
                 -5.595556	56.256389;
                 -5.594722	56.260833;
                 -5.593889	56.265556;
                 -5.592778	56.269444;
                 -5.591944	56.274167;
                 -5.592222	56.278333;
                 -5.591667	56.282778;
                 -5.591944	56.286944;
                 -5.590556	56.291111;
                 -5.589167	56.295000;
                 -5.587778	56.299167;
                 -5.586389	56.303333;
                 -5.585000	56.307222;
                 -5.583611	56.311389;
                 -5.583056	56.315833;
                 -5.583333	56.320000;
                 -5.581667	56.324167;
                 -5.579722	56.327778;
                 -5.577500	56.331389;
                 -5.575556	56.335000;
                 -5.572500	56.338056;
                 -5.568889	56.340556;
                 -5.565278	56.343056;
                 -5.559167	56.344167;
                 -5.551667	56.344444;
                 -5.544167	56.343611;
                 -5.540000	56.344722;
                 -5.537778	56.348333;
                 -5.536389	56.352500;
                 -5.536667	56.356667;
                 -5.536944	56.360833;
                 -5.535556	56.364722;
                 -5.532778	56.367778;
                 -5.529722	56.371111;
                 -5.525278	56.373056;
                 -5.521389	56.375833;
                 -5.517778	56.378333;
                 -5.515000	56.381389;
                 -5.511111	56.383889;
                 -5.508333	56.386944;
                 -5.504722	56.389722;
                 -5.501111	56.392222;
                 -5.497222	56.394722;
                 -5.493056	56.396944;
                 -5.489167	56.399444;
                 -5.486389	56.402500;
                 -5.483333	56.405833;
                 -5.481111	56.409167;
                 -5.479722	56.413333;
                 -5.478889	56.417778;
                 -5.480278	56.421667;
                 -5.480556	56.425833;
                 -5.480000	56.429444;
                 -5.474722	56.431111;
                 -5.470278	56.433056;
                 -5.467500	56.436111;
                 -5.464444	56.439167;
                 -5.461389	56.442500;
                 -5.457778	56.445000;
                 -5.452500	56.446667;
                 -5.447500	56.448056;
                 -5.442222	56.449722;
                 -5.437222	56.449444;
                 -5.435278	56.446389;
                 -5.435278	56.445833;
                 -5.436389	56.445833;
                 -5.432500	56.443333;
                 -5.426389	56.444444;
                 -5.421111	56.446111;
                 -5.416111	56.447778;
                 -5.410000	56.448889;
                 -5.404167	56.448056;
                 -5.398611	56.446389;
                 -5.392778	56.447500;
                 -5.386111	56.448333;
                 -5.378333	56.448333;
                 -5.372500	56.448611;
                 -5.367500	56.450278;
                 -5.362222	56.451667;
                 -5.355556	56.452500;
                 -5.349722	56.450833;
                 -5.344167	56.449167;
                 -5.336944	56.448333;
                 -5.329722	56.448333;
                 -5.324167	56.450000;
                 -5.316944	56.450278;
                 -5.309444	56.449444;
                 -5.303611	56.447778;
                 -5.297500	56.446667;
                 -5.289167	56.446111;
                 -5.283333	56.446389;
                 -5.276667	56.446944;
                 -5.270000	56.445833;
                 -5.266389	56.443333;
                 -5.261667	56.441389;
                 -5.254167	56.440556;
                 -5.254722	56.443611;
                 -5.260278	56.445278;
                 -5.263889	56.447778;
                 -5.266944	56.450833;
                 -5.271389	56.452778;
                 -5.278889	56.453611;
                 -5.286389	56.453333;
                 -5.293333	56.452778;
                 -5.300556	56.452778;
                 -5.306389	56.453333;
                 -5.312778	56.454722;
                 -5.321111	56.455000;
                 -5.328611	56.455000;
                 -5.335000	56.456111;
                 -5.338889	56.458611;
                 -5.343611	56.460556;
                 -5.348056	56.462500;
                 -5.355833	56.462500;
                 -5.360833	56.460833;
                 -5.363889	56.457778;
                 -5.369167	56.456111;
                 -5.375000	56.455000;
                 -5.381944	56.454444;
                 -5.390000	56.454722;
                 -5.398056	56.455278;
                 -5.403889	56.456944;
                 -5.406667	56.459722;
                 -5.408056	56.463333;
                 -5.407500	56.466944;
                 -5.407500	56.471111;
                 -5.409722	56.474444;
                 -5.413333	56.476944;
                 -5.418056	56.478889;
                 -5.422778	56.481111;
                 -5.424722	56.484444;
                 -5.425833	56.488056;
                 -5.425278	56.492778;
                 -5.428333	56.495556;
                 -5.433333	56.494722;
                 -5.436389	56.491667;
                 -5.438611	56.488056;
                 -5.439722	56.484167;
                 -5.442500	56.481111;
                 -5.445556	56.478056;
                 -5.448611	56.474722;
                 -5.451389	56.471667;
                 -5.456667	56.470278;
                 -5.462778	56.469167;
                 -5.463889	56.471944;
                 -5.460833	56.475000;
                 -5.458889	56.478611;
                 -5.463889	56.477778;
                 -5.470000	56.476667;
                 -5.471111	56.480556;
                 -5.468056	56.483611;
                 -5.464444	56.486111;
                 -5.460556	56.488611;
                 -5.457778	56.491667;
                 -5.454167	56.494444;
                 -5.451111	56.497500;
                 -5.449722	56.501389;
                 -5.449167	56.506111;
                 -5.448611	56.510556;
                 -5.444722	56.513333;
                 -5.438611	56.514444;
                 -5.432778	56.515556;
                 -5.429167	56.518056;
                 -5.426389	56.521111;
                 -5.420556	56.519444;
                 -5.413333	56.518611;
                 -5.407500	56.517222;
                 -5.404722	56.514167;
                 -5.401944	56.511389;
                 -5.398056	56.508889;
                 -5.393333	56.506944;
                 -5.387778	56.505278;
                 -5.380000	56.505278;
                 -5.375000	56.506944;
                 -5.371389	56.509444;
                 -5.365833	56.511111;
                 -5.359444	56.511667;
                 -5.351111	56.511389;
                 -5.344167	56.511944;
                 -5.339722	56.514167;
                 -5.335556	56.516111;
                 -5.330000	56.517778;
                 -5.325556	56.520000;
                 -5.320556	56.521389;
                 -5.316667	56.524167;
                 -5.314444	56.527778;
                 -5.310833	56.530833;
                 -5.307222	56.533333;
                 -5.302778	56.535278;
                 -5.298056	56.537500;
                 -5.293056	56.539167;
                 -5.286944	56.540278;
                 -5.281111	56.541389;
                 -5.274167	56.541944;
                 -5.267222	56.542500;
                 -5.260833	56.543056;
                 -5.255278	56.544722;
                 -5.251111	56.546944;
                 -5.248056	56.550000;
                 -5.253611	56.550556;
                 -5.259722	56.549444;
                 -5.266389	56.548889;
                 -5.273056	56.548333;
                 -5.279167	56.547222;
                 -5.285833	56.546667;
                 -5.291944	56.545556;
                 -5.297778	56.544444;
                 -5.304722	56.543611;
                 -5.310833	56.542500;
                 -5.315833	56.541111;
                 -5.321111	56.539444;
                 -5.325556	56.537222;
                 -5.330833	56.535833;
                 -5.335278	56.533611;
                 -5.339167	56.531111;
                 -5.343611	56.528889;
                 -5.347778	56.526944;
                 -5.352222	56.524722;
                 -5.356944	56.522778;
                 -5.362778	56.521667;
                 -5.369722	56.520833;
                 -5.376111	56.521111;
                 -5.382778	56.522500;
                 -5.389167	56.523611;
                 -5.392222	56.526389;
                 -5.395000	56.529444;
                 -5.396111	56.533056;
                 -5.397222	56.536944;
                 -5.402778	56.538333;
                 -5.410278	56.539167;
                 -5.415833	56.540833;
                 -5.415278	56.545556;
                 -5.414444	56.549167;
                 -5.412222	56.552778;
                 -5.411111	56.556667;
                 -5.408333	56.559722;
                 -5.404167	56.562500;
                 -5.399722	56.561389;
                 -5.394722	56.559167;
                 -5.388889	56.559444;
                 -5.386667	56.563056;
                 -5.388056	56.566667;
                 -5.389167	56.570556;
                 -5.389444	56.574444;
                 -5.388056	56.578611;
                 -5.386389	56.582778;
                 -5.384167	56.586389;
                 -5.380556	56.588889;
                 -5.377778	56.591944;
                 -5.373889	56.594444;
                 -5.370000	56.597222;
                 -5.366389	56.599722;
                 -5.362778	56.602222;
                 -5.360556	56.605833;
                 -5.357778	56.608889;
                 -5.353889	56.611667;
                 -5.348611	56.613056;
                 -5.343611	56.614722;
                 -5.337500	56.615833;
                 -5.332222	56.617500;
                 -5.328333	56.620000;
                 -5.326111	56.623611;
                 -5.324167	56.627222;
                 -5.319444	56.629167;
                 -5.314444	56.630833;
                 -5.308333	56.631944;
                 -5.304722	56.634722;
                 -5.303333	56.637778;
                 -5.307222	56.640278;
                 -5.314444	56.641111;
                 -5.321111	56.642222;
                 -5.323056	56.645556;
                 -5.322500	56.650000;
                 -5.318611	56.652778;
                 -5.314167	56.654722;
                 -5.308056	56.655833;
                 -5.302222	56.656944;
                 -5.296944	56.658611;
                 -5.291667	56.660278;
                 -5.287222	56.662222;
                 -5.281944	56.663889;
                 -5.276667	56.665556;
                 -5.270833	56.666667;
                 -5.263889	56.667222;
                 -5.258889	56.668889;
                 -5.254167	56.670833;
                 -5.248889	56.672500;
                 -5.243611	56.674167;
                 -5.239167	56.676111;
                 -5.234722	56.678333;
                 -5.228889	56.679444;
                 -5.222778	56.680556;
                 -5.215000	56.680556;
                 -5.206667	56.680278;
                 -5.200000	56.680833;
                 -5.192500	56.680833;
                 -5.186111	56.679722;
                 -5.179444	56.678611;
                 -5.173889	56.676944;
                 -5.168333	56.675278;
                 -5.162778	56.673611;
                 -5.154444	56.673333;
                 -5.147778	56.673889;
                 -5.140833	56.674444;
                 -5.133333	56.674444;
                 -5.125000	56.674167;
                 -5.120000	56.675833;
                 -5.115278	56.677778;
                 -5.111667	56.680556;
                 -5.107778	56.683056;
                 -5.103333	56.685000;
                 -5.097222	56.686111;
                 -5.091389	56.687222;
                 -5.083611	56.687500;
                 -5.077500	56.688611;
                 -5.072500	56.690000;
                 -5.067222	56.691667;
                 -5.061944	56.693333;
                 -5.065278	56.694444;
                 -5.072778	56.694167;
                 -5.080278	56.694167;
                 -5.087778	56.694167;
                 -5.094444	56.693333;
                 -5.101389	56.692778;
                 -5.106667	56.691389;
                 -5.111944	56.689722;
                 -5.116944	56.688056;
                 -5.122222	56.686389;
                 -5.126944	56.684444;
                 -5.131389	56.682222;
                 -5.137500	56.681111;
                 -5.145833	56.681667;
                 -5.152222	56.682778;
                 -5.159722	56.683611;
                 -5.166944	56.684444;
                 -5.173333	56.685556;
                 -5.180000	56.686944;
                 -5.186389	56.688056;
                 -5.192222	56.689722;
                 -5.196667	56.691667;
                 -5.201389	56.693889;
                 -5.208611	56.694444;
                 -5.216389	56.695278;
                 -5.224722	56.695833;
                 -5.232222	56.695556;
                 -5.238889	56.695833;
                 -5.243333	56.698056;
                 -5.243611	56.701944;
                 -5.241389	56.705556;
                 -5.239167	56.709167;
                 -5.236944	56.712778;
                 -5.234167	56.715833;
                 -5.230278	56.718333;
                 -5.226667	56.721111;
                 -5.222778	56.723611;
                 -5.219167	56.726111;
                 -5.216111	56.729167;
                 -5.214167	56.732778;
                 -5.211667	56.736389;
                 -5.208611	56.739444;
                 -5.205000	56.742222;
                 -5.201389	56.744722;
                 -5.197500	56.747222;
                 -5.193889	56.749722;
                 -5.190278	56.752500;
                 -5.186389	56.755000;
                 -5.182500	56.757500;
                 -5.178889	56.760278;
                 -5.176111	56.763333;
                 -5.172222	56.765833;
                 -5.169167	56.768889;
                 -5.166389	56.771944;
                 -5.163333	56.775000;
                 -5.160278	56.778056;
                 -5.156389	56.780833;
                 -5.153611	56.783889;
                 -5.148889	56.785833;
                 -5.145278	56.788611;
                 -5.140833	56.790556;
                 -5.136944	56.793056;
                 -5.133333	56.795833;
                 -5.129722	56.798333;
                 -5.126389	56.801389;
                 -5.123611	56.804444;
                 -5.121944	56.808611;
                 -5.121389	56.813056;
                 -5.120000	56.817222;
                 -5.124444	56.815000;
                 -5.128056	56.812500;
                 -5.132778	56.810278;
                 -5.136389	56.807778;
                 -5.141111	56.805833;
                 -5.145556	56.803611;
                 -5.150000	56.801667;
                 -5.154444	56.799444;
                 -5.158333	56.796944;
                 -5.161944	56.794167;
                 -5.165833	56.791667;
                 -5.169444	56.789167;
                 -5.173333	56.786667;
                 -5.176389	56.783333;
                 -5.180556	56.781389;
                 -5.184444	56.778889;
                 -5.188889	56.776667;
                 -5.192500	56.774167;
                 -5.195833	56.771111;
                 -5.198611	56.768056;
                 -5.202500	56.765278;
                 -5.206111	56.762778;
                 -5.210556	56.760833;
                 -5.217500	56.760000;
                 -5.222778	56.761667;
                 -5.229444	56.763056;
                 -5.235556	56.761944;
                 -5.236944	56.757778;
                 -5.237500	56.755000;
                 -5.235278	56.751667;
                 -5.234444	56.748056;
                 -5.234167	56.743889;
                 -5.236944	56.740833;
                 -5.240833	56.738333;
                 -5.244722	56.735556;
                 -5.248889	56.733611;
                 -5.251111	56.730000;
                 -5.249167	56.726667;
                 -5.250833	56.722500;
                 -5.253611	56.719444;
                 -5.257222	56.716944;
                 -5.261111	56.714444;
                 -5.265000	56.711667;
                 -5.268611	56.709167;
                 -5.272222	56.706667;
                 -5.275278	56.703611;
                 -5.278333	56.700278;
                 -5.283333	56.698889;
                 -5.288056	56.700833;
                 -5.291944	56.703333;
                 -5.297500	56.705000;
                 -5.305278	56.704722;
                 -5.310278	56.703333;
                 -5.313333	56.700278;
                 -5.316389	56.696944;
                 -5.320000	56.694444;
                 -5.324444	56.692500;
                 -5.329167	56.690278;
                 -5.334167	56.688611;
                 -5.339722	56.687222;
                 -5.345556	56.686111;
                 -5.350833	56.684444;
                 -5.355833	56.682778;
                 -5.358889	56.679722;
                 -5.357778	56.676111;
                 -5.356667	56.672222;
                 -5.357222	56.668611;
                 -5.359444	56.665000;
                 -5.363333	56.662500;
                 -5.367778	56.660278;
                 -5.371944	56.658333;
                 -5.377500	56.656667;
                 -5.381944	56.654444;
                 -5.386111	56.652500;
                 -5.390000	56.649722;
                 -5.393889	56.647222;
                 -5.396667	56.644167;
                 -5.400278	56.641667;
                 -5.404167	56.638889;
                 -5.409444	56.637500;
                 -5.415000	56.638056;
                 -5.418889	56.640556;
                 -5.424444	56.642222;
                 -5.430556	56.641111;
                 -5.433611	56.638056;
                 -5.436389	56.635000;
                 -5.439444	56.631944;
                 -5.442500	56.628611;
                 -5.445278	56.625556;
                 -5.448333	56.622500;
                 -5.451111	56.619444;
                 -5.454722	56.616944;
                 -5.459167	56.614722;
                 -5.466111	56.614167;
                 -5.472778	56.615278;
                 -5.478611	56.614167;
                 -5.481389	56.611111;
                 -5.484444	56.608056;
                 -5.486667	56.604444;
                 -5.490556	56.601944;
                 -5.493056	56.598889;
                 -5.496944	56.596389;
                 -5.500000	56.593056;
                 -5.503611	56.590556;
                 -5.506667	56.587500;
                 -5.508611	56.584722;
                 -5.510833	56.581111;
                 -5.513611	56.578056;
                 -5.517222	56.575556;
                 -5.521944	56.573333;
                 -5.525556	56.570833;
                 -5.529167	56.568333;
                 -5.533056	56.565556;
                 -5.535833	56.562500;
                 -5.538889	56.559444;
                 -5.541667	56.556389;
                 -5.545278	56.553889;
                 -5.550000	56.551667;
                 -5.554444	56.549722;
                 -5.558056	56.546944;
                 -5.561111	56.543889;
                 -5.563889	56.540833;
                 -5.568333	56.538611;
                 -5.573056	56.536667;
                 -5.577500	56.534444;
                 -5.582500	56.533056;
                 -5.586111	56.530278;
                 -5.589722	56.527778;
                 -5.592778	56.524722;
                 -5.597500	56.522222;
                 -5.601944	56.520000;
                 -5.607778	56.518889;
                 -5.613889	56.517778;
                 -5.618889	56.516111;
                 -5.624167	56.514444;
                 -5.629444	56.512778;
                 -5.634444	56.511389;
                 -5.639167	56.509167;
                 -5.644167	56.507500;
                 -5.649444	56.505833;
                 -5.655556	56.504722;
                 -5.661389	56.503611;
                 -5.666667	56.501944;
                 -5.671111	56.500000;
                 -5.675556	56.497778;
                 -5.679167	56.495278;
                 -5.682778	56.492778;
                 -5.687222	56.490556;
                 -5.694167	56.490000;
                 -5.698889	56.491111;
                 -5.701944	56.493889;
                 -5.702778	56.497500;
                 -5.704167	56.501389;
                 -5.706389	56.504722;
                 -5.710000	56.507222;
                 -5.715556	56.508611;
                 -5.723056	56.508611;
                 -5.730833	56.508333;
                 -5.738056	56.508333;
                 -5.743056	56.510278;
                 -5.747500	56.512222;
                 -5.753056	56.513889;
                 -5.760833	56.513889;
                 -5.767222	56.515000;
                 -5.771111	56.517500;
                 -5.775556	56.519444;
                 -5.778611	56.522222;
                 -5.780833	56.525556;
                 -5.777778	56.528611;
                 -5.772500	56.530278;
                 -5.770556	56.533889;
                 -5.775278	56.535000;
                 -5.781389	56.533889;
                 -5.786389	56.532222;
                 -5.789444	56.529167;
                 -5.793611	56.526944;
                 -5.800556	56.526389;
                 -5.806111	56.528056;
                 -5.811667	56.529722;
                 -5.816389	56.531667;
                 -5.823056	56.532778;
                 -5.829444	56.533889;
                 -5.835833	56.535278;
                 -5.842500	56.536389;
                 -5.850833	56.536667;
                 -5.858056	56.537500;
                 -5.864444	56.538611;
                 -5.870000	56.540278;
                 -5.875000	56.542222;
                 -5.882222	56.543056;
                 -5.889167	56.542500;
                 -5.896389	56.542222;
                 -5.901389	56.544167;
                 -5.906111	56.546389;
                 -5.910000	56.548611;
                 -5.913611	56.551111;
                 -5.917500	56.553611;
                 -5.921389	56.556111;
                 -5.925278	56.558611;
                 -5.930000	56.560556;
                 -5.935556	56.562222;
                 -5.941111	56.563611;
                 -5.945833	56.565833;
                 -5.949722	56.568333;
                 -5.953611	56.570556;
                 -5.956667	56.573611;
                 -5.958611	56.576667;
                 -5.962500	56.579167;
                 -5.968056	56.580833;
                 -5.971944	56.583333;
                 -5.974722	56.586111;
                 -5.977778	56.589167;
                 -5.980000	56.592222;
                 -5.981111	56.596111;
                 -5.983056	56.599444;
                 -5.986111	56.602222;
                 -5.988333	56.605556;
                 -5.991944	56.608056;
                 -5.995833	56.610278;
                 -5.999722	56.612778;
                 -6.003611	56.615278;
                 -6.006667	56.618056;
                 -6.009444	56.621111;
                 -6.011667	56.624167;
                 -6.012778	56.628056;
                 -6.013056	56.632222;
                 -6.013333	56.636389;
                 -6.012222	56.639444;
                 -6.008611	56.641944;
                 -6.002500	56.643056;
                 -5.995556	56.643889;
                 -5.988333	56.643889;
                 -5.980000	56.643611;
                 -5.972500	56.643889;
                 -5.966389	56.645000;
                 -5.961111	56.646667;
                 -5.957500	56.649167;
                 -5.952222	56.650833;
                 -5.945556	56.651389;
                 -5.939722	56.650833;
                 -5.934167	56.649167;
                 -5.928333	56.647500;
                 -5.921111	56.646944;
                 -5.915000	56.648056;
                 -5.908333	56.648611;
                 -5.900278	56.648333;
                 -5.895278	56.646389;
                 -5.890556	56.644167;
                 -5.884167	56.643056;
                 -5.878889	56.644722;
                 -5.875278	56.647222;
                 -5.872222	56.650278;
                 -5.868611	56.653056;
                 -5.863889	56.655000;
                 -5.858889	56.656667;
                 -5.851389	56.656944;
                 -5.844722	56.657500;
                 -5.837778	56.658056;
                 -5.833611	56.660278;
                 -5.829722	56.662778;
                 -5.825833	56.665556;
                 -5.821667	56.667500;
                 -5.817222	56.669722;
                 -5.811111	56.670833;
                 -5.805833	56.672500;
                 -5.799722	56.673611;
                 -5.795278	56.675833;
                 -5.792500	56.678889;
                 -5.788056	56.680833;
                 -5.784444	56.683611;
                 -5.779722	56.685556;
                 -5.775278	56.687778;
                 -5.770278	56.689444;
                 -5.764167	56.690556;
                 -5.757222	56.691111;
                 -5.751389	56.692222;
                 -5.743889	56.692500;
                 -5.736667	56.691667;
                 -5.728333	56.691389;
                 -5.722778	56.689722;
                 -5.718889	56.687222;
                 -5.715833	56.684444;
                 -5.712778	56.681389;
                 -5.708889	56.679167;
                 -5.702500	56.678889;
                 -5.695556	56.679444;
                 -5.688889	56.678333;
                 -5.683333	56.676667;
                 -5.678611	56.674722;
                 -5.671389	56.673889;
                 -5.663889	56.673056;
                 -5.658056	56.673333;
                 -5.651389	56.673056;
                 -5.644167	56.672222;
                 -5.635833	56.671944;
                 -5.628889	56.672500;
                 -5.622778	56.673611;
                 -5.617778	56.675278;
                 -5.612500	56.676944;
                 -5.606389	56.678056;
                 -5.598889	56.678056;
                 -5.591944	56.678889;
                 -5.584444	56.678889;
                 -5.577222	56.679167;
                 -5.570278	56.679722;
                 -5.563611	56.680278;
                 -5.559722	56.682778;
                 -5.559444	56.686667;
                 -5.563889	56.688611;
                 -5.572222	56.688889;
                 -5.579722	56.688889;
                 -5.587500	56.688611;
                 -5.594167	56.688056;
                 -5.600833	56.687500;
                 -5.606944	56.686389;
                 -5.613056	56.685278;
                 -5.618889	56.683889;
                 -5.625833	56.683333;
                 -5.633333	56.683333;
                 -5.640833	56.683056;
                 -5.648056	56.683056;
                 -5.655833	56.682778;
                 -5.664167	56.683056;
                 -5.672500	56.683611;
                 -5.678889	56.684722;
                 -5.684444	56.686389;
                 -5.689167	56.688333;
                 -5.694167	56.690278;
                 -5.698056	56.692778;
                 -5.701667	56.695278;
                 -5.705556	56.697778;
                 -5.710278	56.699722;
                 -5.715833	56.701389;
                 -5.723333	56.702222;
                 -5.730833	56.702778;
                 -5.738333	56.703611;
                 -5.746667	56.703889;
                 -5.753889	56.704722;
                 -5.761111	56.705556;
                 -5.768889	56.706389;
                 -5.775278	56.705556;
                 -5.780833	56.703889;
                 -5.785278	56.701944;
                 -5.788056	56.698889;
                 -5.791667	56.696111;
                 -5.795278	56.693611;
                 -5.800000	56.691389;
                 -5.804444	56.689444;
                 -5.808056	56.686667;
                 -5.812500	56.684722;
                 -5.817778	56.683056;
                 -5.822222	56.680833;
                 -5.825833	56.678333;
                 -5.830278	56.676111;
                 -5.835000	56.674167;
                 -5.840833	56.673056;
                 -5.846944	56.671944;
                 -5.855278	56.672222;
                 -5.863611	56.672500;
                 -5.870278	56.671944;
                 -5.875556	56.670278;
                 -5.882222	56.669722;
                 -5.889722	56.670278;
                 -5.896944	56.671111;
                 -5.904444	56.671944;
                 -5.912222	56.671667;
                 -5.920278	56.671944;
                 -5.926667	56.673333;
                 -5.932500	56.674722;
                 -5.936389	56.677222;
                 -5.941111	56.679167;
                 -5.947500	56.680556;
                 -5.954444	56.680833;
                 -5.959444	56.679167;
                 -5.963889	56.676944;
                 -5.967500	56.674444;
                 -5.972778	56.672778;
                 -5.978889	56.671667;
                 -5.985278	56.672778;
                 -5.989167	56.675278;
                 -5.993889	56.677222;
                 -6.000556	56.678333;
                 -6.008889	56.678611;
                 -6.016389	56.678611;
                 -6.023056	56.678056;
                 -6.029722	56.677222;
                 -6.036667	56.676667;
                 -6.043889	56.676389;
                 -6.052222	56.676667;
                 -6.058056	56.678333;
                 -6.062778	56.680278;
                 -6.068333	56.681944;
                 -6.075556	56.682778;
                 -6.083333	56.682500;
                 -6.091667	56.682778;
                 -6.099167	56.683611;
                 -6.105833	56.683056;
                 -6.113333	56.682778;
                 -6.121667	56.683056;
                 -6.128333	56.683333;
                 -6.134167	56.682222;
                 -6.140278	56.681111;
                 -6.145556	56.679444;
                 -6.151389	56.678333;
                 -6.158333	56.677500;
                 -6.164722	56.678889;
                 -6.170278	56.680278;
                 -6.174444	56.682778;
                 -6.181944	56.682500;
                 -6.186944	56.681111;
                 -6.194444	56.681667;
                 -6.195278	56.681667;
                 -6.200833	56.682500;
                 -6.206667	56.683889;
                 -6.211389	56.686111;
                 -6.215278	56.688611;
                 -6.219167	56.690833;
                 -6.222222	56.693889;
                 -6.224444	56.696944;
                 -6.227222	56.700000;
                 -6.231111	56.702500;
                 -6.234167	56.705278;
                 -6.236389	56.708611;
                 -6.237500	56.712222;
                 -6.237778	56.716389;
                 -6.237222	56.720833;
                 -6.235000	56.724444;
                 -6.233333	56.725833;
                 -6.234167	56.725833;
                 -6.228889	56.727500;
                 -6.222222	56.728056;
                 -6.215278	56.728889;
                 -6.207778	56.728889;
                 -6.200833	56.729722;
                 -6.197222	56.732222;
                 -6.196111	56.736389;
                 -6.198056	56.739722;
                 -6.200278	56.743056;
                 -6.199722	56.746667;
                 -6.195278	56.748611;
                 -6.190278	56.750278;
                 -6.182500	56.750556;
                 -6.177778	56.748611;
                 -6.171944	56.748611;
                 -6.168333	56.751389;
                 -6.164444	56.753889;
                 -6.160278	56.756111;
                 -6.154722	56.757778;
                 -6.147222	56.757778;
                 -6.140000	56.758056;
                 -6.132222	56.757222;
                 -6.125000	56.756667;
                 -6.118056	56.757222;
                 -6.111944	56.758333;
                 -6.106111	56.759444;
                 -6.098611	56.759722;
                 -6.093056	56.758056;
                 -6.087222	56.756389;
                 -6.079722	56.755833;
                 -6.073056	56.756389;
                 -6.068611	56.758611;
                 -6.064167	56.760556;
                 -6.057500	56.761389;
                 -6.053333	56.758889;
                 -6.047778	56.757222;
                 -6.041944	56.757500;
                 -6.037500	56.759722;
                 -6.035278	56.763333;
                 -6.031111	56.765278;
                 -6.024167	56.766111;
                 -6.016944	56.765278;
                 -6.010278	56.764167;
                 -6.001944	56.763611;
                 -5.995000	56.764444;
                 -5.990833	56.766389;
                 -5.986111	56.768611;
                 -5.981667	56.770833;
                 -5.977222	56.772778;
                 -5.971111	56.773889;
                 -5.965278	56.775278;
                 -5.959722	56.773056;
                 -5.955833	56.770556;
                 -5.952778	56.767778;
                 -5.950000	56.765000;
                 -5.945000	56.762778;
                 -5.940556	56.760833;
                 -5.933889	56.759722;
                 -5.930000	56.757222;
                 -5.926111	56.754722;
                 -5.923056	56.751944;
                 -5.919167	56.749444;
                 -5.912778	56.748333;
                 -5.906667	56.748611;
                 -5.903056	56.751111;
                 -5.898611	56.753333;
                 -5.893611	56.755000;
                 -5.886111	56.754167;
                 -5.879444	56.753056;
                 -5.872778	56.753611;
                 -5.869167	56.756111;
                 -5.871944	56.759167;
                 -5.876667	56.761111;
                 -5.881667	56.763056;
                 -5.887222	56.764722;
                 -5.891111	56.767222;
                 -5.892222	56.770833;
                 -5.890833	56.775000;
                 -5.885556	56.776667;
                 -5.877222	56.776389;
                 -5.871667	56.774722;
                 -5.866944	56.772500;
                 -5.864722	56.769444;
                 -5.861944	56.766389;
                 -5.857500	56.768611;
                 -5.851944	56.770278;
                 -5.846111	56.769444;
                 -5.840556	56.767778;
                 -5.835556	56.769444;
                 -5.831667	56.772222;
                 -5.827500	56.774167;
                 -5.822778	56.776389;
                 -5.816944	56.777500;
                 -5.810000	56.778056;
                 -5.802500	56.778333;
                 -5.795556	56.778889;
                 -5.788889	56.779444;
                 -5.782222	56.780278;
                 -5.777500	56.782222;
                 -5.776389	56.786389;
                 -5.779167	56.789167;
                 -5.785000	56.790833;
                 -5.792222	56.791667;
                 -5.797778	56.793333;
                 -5.804444	56.794444;
                 -5.810000	56.796111;
                 -5.815556	56.797500;
                 -5.821389	56.799167;
                 -5.827778	56.800278;
                 -5.833611	56.801944;
                 -5.839444	56.803611;
                 -5.845000	56.805278;
                 -5.850556	56.806944;
                 -5.855278	56.808889;
                 -5.859167	56.811389;
                 -5.861944	56.814167;
                 -5.864167	56.817500;
                 -5.862778	56.821667;
                 -5.861389	56.825556;
                 -5.857778	56.828333;
                 -5.853333	56.830278;
                 -5.847778	56.831944;
                 -5.841389	56.832500;
                 -5.833611	56.831944;
                 -5.827222	56.830556;
                 -5.821944	56.831389;
                 -5.817500	56.833611;
                 -5.812222	56.835000;
                 -5.805000	56.835278;
                 -5.797500	56.834444;
                 -5.791389	56.835556;
                 -5.787500	56.838333;
                 -5.783333	56.840278;
                 -5.778889	56.842500;
                 -5.773333	56.844167;
                 -5.766667	56.844722;
                 -5.759722	56.845278;
                 -5.753333	56.844167;
                 -5.746944	56.843056;
                 -5.740278	56.841667;
                 -5.733611	56.840556;
                 -5.726389	56.839722;
                 -5.720833	56.841389;
                 -5.716667	56.843611;
                 -5.712778	56.846111;
                 -5.710556	56.849722;
                 -5.707778	56.852778;
                 -5.704722	56.855833;
                 -5.700833	56.858611;
                 -5.695833	56.860000;
                 -5.689722	56.861389;
                 -5.683611	56.862500;
                 -5.678333	56.863889;
                 -5.674167	56.866111;
                 -5.669444	56.868333;
                 -5.665833	56.870833;
                 -5.666944	56.874444;
                 -5.671667	56.876667;
                 -5.679167	56.877222;
                 -5.686111	56.876667;
                 -5.690556	56.874722;
                 -5.692500	56.871111;
                 -5.696944	56.868889;
                 -5.702500	56.867222;
                 -5.708333	56.866111;
                 -5.714444	56.865000;
                 -5.718889	56.863056;
                 -5.719444	56.858333;
                 -5.719167	56.854167;
                 -5.721389	56.850556;
                 -5.728333	56.850000;
                 -5.735556	56.850833;
                 -5.742222	56.851944;
                 -5.749722	56.852778;
                 -5.756111	56.853889;
                 -5.762778	56.855000;
                 -5.769167	56.856389;
                 -5.776667	56.856944;
                 -5.783333	56.858333;
                 -5.788889	56.859722;
                 -5.790000	56.863611;
                 -5.787222	56.866667;
                 -5.782778	56.868611;
                 -5.776667	56.870000;
                 -5.771389	56.871389;
                 -5.767500	56.874167;
                 -5.762500	56.875833;
                 -5.756389	56.876944;
                 -5.750278	56.878056;
                 -5.743611	56.878611;
                 -5.736944	56.879167;
                 -5.731389	56.880833;
                 -5.730833	56.884444;
                 -5.736667	56.886111;
                 -5.745000	56.886389;
                 -5.752500	56.887222;
                 -5.760000	56.887222;
                 -5.767500	56.886944;
                 -5.774444	56.886389;
                 -5.781389	56.885556;
                 -5.786389	56.884167;
                 -5.793333	56.883333;
                 -5.800833	56.884167;
                 -5.807500	56.884444;
                 -5.812778	56.882778;
                 -5.818611	56.881667;
                 -5.826111	56.882500;
                 -5.833611	56.882222;
                 -5.839722	56.881111;
                 -5.844167	56.879167;
                 -5.851111	56.878333;
                 -5.858611	56.878333;
                 -5.865556	56.877500;
                 -5.871389	56.876389;
                 -5.878056	56.875833;
                 -5.885000	56.875278;
                 -5.892500	56.875000;
                 -5.899167	56.876111;
                 -5.904722	56.877778;
                 -5.912222	56.878611;
                 -5.919722	56.879167;
                 -5.927222	56.880000;
                 -5.930278	56.882778;
                 -5.931389	56.886667;
                 -5.929167	56.890278;
                 -5.923056	56.891389;
                 -5.916389	56.891944;
                 -5.908611	56.892222;
                 -5.903611	56.893889;
                 -5.902778	56.898333;
                 -5.898333	56.899444;
                 -5.892778	56.898056;
                 -5.887222	56.896389;
                 -5.879722	56.895556;
                 -5.872222	56.894722;
                 -5.864722	56.895000;
                 -5.858611	56.896111;
                 -5.854167	56.898333;
                 -5.851944	56.901944;
                 -5.853333	56.905556;
                 -5.857778	56.907500;
                 -5.863889	56.909167;
                 -5.871111	56.910000;
                 -5.876667	56.911389;
                 -5.880556	56.913889;
                 -5.882778	56.917222;
                 -5.880000	56.920278;
                 -5.874444	56.921944;
                 -5.870000	56.924167;
                 -5.867222	56.927222;
                 -5.865833	56.930278;
                 -5.864444	56.934444;
                 -5.864722	56.938611;
                 -5.863611	56.942500;
                 -5.860278	56.945833;
                 -5.855833	56.947778;
                 -5.852222	56.950556;
                 -5.850000	56.953889;
                 -5.849444	56.958611;
                 -5.845000	56.960556;
                 -5.842778	56.964167;
                 -5.845000	56.967500;
                 -5.845278	56.970833;
                 -5.846389	56.974444;
                 -5.846667	56.978611;
                 -5.844722	56.982222;
                 -5.841667	56.985278;
                 -5.838611	56.988333;
                 -5.837500	56.992500;
                 -5.835833	56.996667;
                 -5.833056	56.999722;
                 -5.829444	57.002222;
                 -5.824722	57.004444;
                 -5.819444	57.005833;
                 -5.814167	57.007500;
                 -5.808056	57.008611;
                 -5.801389	57.009444;
                 -5.794722	57.009167;
                 -5.787222	57.009167;
                 -5.780278	57.010000;
                 -5.773333	57.010556;
                 -5.766667	57.011111;
                 -5.759167	57.011389;
                 -5.751389	57.011389;
                 -5.746111	57.013056;
                 -5.740000	57.014167;
                 -5.734444	57.013611;
                 -5.732222	57.010278;
                 -5.730278	57.006944;
                 -5.728056	57.003611;
                 -5.726111	57.000278;
                 -5.723889	56.996944;
                 -5.720833	56.994167;
                 -5.716944	56.991667;
                 -5.712500	56.989722;
                 -5.706667	56.988056;
                 -5.700833	56.986389;
                 -5.695278	56.984444;
                 -5.689722	56.982778;
                 -5.684167	56.981111;
                 -5.679167	56.979167;
                 -5.675278	56.976667;
                 -5.668889	56.975556;
                 -5.661111	56.975556;
                 -5.652778	56.975278;
                 -5.648889	56.972778;
                 -5.642222	56.971667;
                 -5.638056	56.972778;
                 -5.634167	56.975556;
                 -5.628056	56.976667;
                 -5.621389	56.977222;
                 -5.616111	56.978889;
                 -5.611667	56.980833;
                 -5.604722	56.981667;
                 -5.598333	56.980278;
                 -5.590556	56.980556;
                 -5.584722	56.981667;
                 -5.579444	56.983333;
                 -5.573333	56.984444;
                 -5.568056	56.986111;
                 -5.561944	56.987222;
                 -5.555833	56.988333;
                 -5.549722	56.989444;
                 -5.543611	56.990556;
                 -5.538611	56.991944;
                 -5.532500	56.993333;
                 -5.527222	56.994722;
                 -5.522778	56.996944;
                 -5.523056	57.000000;
                 -5.528889	57.001667;
                 -5.534722	57.001667;
                 -5.540000	57.000000;
                 -5.545278	56.998333;
                 -5.549722	56.996111;
                 -5.555000	56.994722;
                 -5.560278	56.993056;
                 -5.565556	56.991389;
                 -5.572500	56.990833;
                 -5.580833	56.991111;
                 -5.589167	56.991389;
                 -5.595833	56.990833;
                 -5.600556	56.988611;
                 -5.605000	56.986667;
                 -5.611944	56.986111;
                 -5.618611	56.985278;
                 -5.623889	56.983611;
                 -5.630278	56.985000;
                 -5.635278	56.986944;
                 -5.640000	56.988889;
                 -5.646389	56.990278;
                 -5.653056	56.991389;
                 -5.658056	56.993333;
                 -5.660833	56.996389;
                 -5.664722	56.998611;
                 -5.669444	57.000833;
                 -5.673333	57.003333;
                 -5.678333	57.005278;
                 -5.682778	57.007222;
                 -5.687778	57.009722;
                 -5.691389	57.012222;
                 -5.691944	57.016389;
                 -5.690556	57.019444;
                 -5.686667	57.022222;
                 -5.683056	57.024722;
                 -5.678333	57.026944;
                 -5.675556	57.030000;
                 -5.676667	57.033611;
                 -5.681389	57.035556;
                 -5.687222	57.037222;
                 -5.694444	57.038056;
                 -5.702222	57.038889;
                 -5.708889	57.038056;
                 -5.715833	57.037500;
                 -5.721944	57.036389;
                 -5.726944	57.034722;
                 -5.731667	57.032778;
                 -5.736944	57.031111;
                 -5.745278	57.031389;
                 -5.751944	57.032500;
                 -5.755833	57.035000;
                 -5.756944	57.038611;
                 -5.756944	57.042778;
                 -5.761111	57.045278;
                 -5.768611	57.045278;
                 -5.777222	57.045556;
                 -5.782778	57.047222;
                 -5.785556	57.050000;
                 -5.788889	57.052778;
                 -5.790833	57.056111;
                 -5.791111	57.060278;
                 -5.788889	57.063889;
                 -5.787500	57.066944;
                 -5.785556	57.069722;
                 -5.782778	57.072778;
                 -5.779444	57.075833;
                 -5.775833	57.078611;
                 -5.773056	57.081667;
                 -5.769167	57.084167;
                 -5.764722	57.086389;
                 -5.761111	57.088889;
                 -5.756667	57.091111;
                 -5.751111	57.092500;
                 -5.744444	57.093333;
                 -5.738333	57.094444;
                 -5.736111	57.098056;
                 -5.734722	57.101944;
                 -5.732778	57.105556;
                 -5.729444	57.108611;
                 -5.726667	57.111667;
                 -5.723611	57.115000;
                 -5.719167	57.116944;
                 -5.713056	57.118056;
                 -5.705278	57.118333;
                 -5.696944	57.117778;
                 -5.688611	57.117500;
                 -5.681111	57.116944;
                 -5.674444	57.117500;
                 -5.668889	57.119167;
                 -5.665278	57.121667;
                 -5.660000	57.123333;
                 -5.653056	57.123889;
                 -5.646111	57.124444;
                 -5.639722	57.123333;
                 -5.635000	57.121389;
                 -5.628333	57.120000;
                 -5.620833	57.119444;
                 -5.612500	57.118889;
                 -5.605833	57.117778;
                 -5.601111	57.115833;
                 -5.597222	57.113333;
                 -5.592222	57.111389;
                 -5.589444	57.108333;
                 -5.586389	57.105556;
                 -5.585278	57.101944;
                 -5.582222	57.098889;
                 -5.575556	57.097778;
                 -5.569167	57.096667;
                 -5.563611	57.095000;
                 -5.557778	57.093333;
                 -5.551944	57.091667;
                 -5.546389	57.090000;
                 -5.539722	57.088889;
                 -5.533333	57.087778;
                 -5.525000	57.087222;
                 -5.518889	57.088333;
                 -5.515833	57.091667;
                 -5.513056	57.094722;
                 -5.506944	57.095833;
                 -5.498333	57.095278;
                 -5.490000	57.095000;
                 -5.483056	57.095556;
                 -5.476389	57.096389;
                 -5.470278	57.097500;
                 -5.464167	57.098611;
                 -5.458333	57.099722;
                 -5.452222	57.100833;
                 -5.446111	57.101944;
                 -5.440000	57.103056;
                 -5.436944	57.105278;
                 -5.443056	57.104167;
                 -5.449167	57.103056;
                 -5.456111	57.102222;
                 -5.463611	57.102222;
                 -5.471111	57.103056;
                 -5.478611	57.103611;
                 -5.484444	57.105278;
                 -5.491667	57.106111;
                 -5.496944	57.104444;
                 -5.501667	57.102500;
                 -5.506111	57.100278;
                 -5.512222	57.099167;
                 -5.519722	57.099167;
                 -5.527500	57.098889;
                 -5.535278	57.098889;
                 -5.542500	57.099444;
                 -5.547500	57.101667;
                 -5.547500	57.105833;
                 -5.547778	57.110000;
                 -5.548056	57.113889;
                 -5.549444	57.117778;
                 -5.552222	57.120556;
                 -5.556111	57.123056;
                 -5.560000	57.125556;
                 -5.564722	57.127500;
                 -5.569444	57.129722;
                 -5.573611	57.131944;
                 -5.579167	57.133611;
                 -5.585556	57.134722;
                 -5.592222	57.136111;
                 -5.597778	57.137778;
                 -5.603889	57.139167;
                 -5.610278	57.140556;
                 -5.618611	57.140833;
                 -5.625556	57.140278;
                 -5.633056	57.140000;
                 -5.640000	57.139444;
                 -5.648333	57.139722;
                 -5.655000	57.140833;
                 -5.661667	57.142222;
                 -5.667222	57.143611;
                 -5.672778	57.145278;
                 -5.678611	57.146944;
                 -5.684444	57.148611;
                 -5.688333	57.151111;
                 -5.689444	57.154722;
                 -5.688056	57.158889;
                 -5.683333	57.160833;
                 -5.681111	57.164444;
                 -5.680000	57.168611;
                 -5.680278	57.172778;
                 -5.678611	57.176667;
                 -5.675000	57.179444;
                 -5.671111	57.181944;
                 -5.667500	57.184444;
                 -5.662778	57.186667;
                 -5.657778	57.188333;
                 -5.652222	57.190000;
                 -5.646111	57.191111;
                 -5.640833	57.192500;
                 -5.636389	57.194722;
                 -5.634167	57.198333;
                 -5.635556	57.201944;
                 -5.635556	57.206111;
                 -5.632500	57.209167;
                 -5.628889	57.211944;
                 -5.626111	57.215000;
                 -5.627222	57.218611;
                 -5.631944	57.220556;
                 -5.638333	57.221944;
                 -5.644167	57.223611;
                 -5.648889	57.225556;
                 -5.652778	57.228056;
                 -5.652222	57.231667;
                 -5.649444	57.234722;
                 -5.646389	57.237778;
                 -5.643333	57.240833;
                 -5.639722	57.243611;
                 -5.636111	57.246111;
                 -5.631389	57.248056;
                 -5.626111	57.249722;
                 -5.620000	57.250833;
                 -5.611667	57.250556;
                 -5.603889	57.249722;
                 -5.597222	57.250556;
                 -5.593611	57.253056;
                 -5.587500	57.254167;
                 -5.580278	57.254722;
                 -5.572778	57.255000;
                 -5.565833	57.255556;
                 -5.559722	57.256667;
                 -5.555278	57.258889;
                 -5.551389	57.261389;
                 -5.547778	57.263889;
                 -5.540000	57.264167;
                 -5.534444	57.262500;
                 -5.529444	57.260556;
                 -5.525556	57.258056;
                 -5.521667	57.255556;
                 -5.517222	57.253611;
                 -5.513056	57.251111;
                 -5.509167	57.248611;
                 -5.505278	57.246111;
                 -5.501389	57.243611;
                 -5.497500	57.241111;
                 -5.493611	57.238611;
                 -5.488889	57.236667;
                 -5.485000	57.234167;
                 -5.481111	57.231667;
                 -5.477222	57.229444;
                 -5.472778	57.227222;
                 -5.468889	57.224722;
                 -5.462778	57.223333;
                 -5.457222	57.221667;
                 -5.450556	57.220278;
                 -5.443056	57.219722;
                 -5.436389	57.218333;
                 -5.430833	57.216667;
                 -5.424444	57.215556;
                 -5.416667	57.214722;
                 -5.410000	57.215556;
                 -5.404444	57.216944;
                 -5.400278	57.219167;
                 -5.394722	57.220833;
                 -5.390833	57.223333;
                 -5.388611	57.226944;
                 -5.391667	57.229722;
                 -5.398611	57.229167;
                 -5.403889	57.227500;
                 -5.409167	57.225833;
                 -5.416667	57.225833;
                 -5.424444	57.226667;
                 -5.430833	57.227778;
                 -5.436667	57.229444;
                 -5.443333	57.230556;
                 -5.450000	57.231944;
                 -5.456667	57.233056;
                 -5.462222	57.234722;
                 -5.466944	57.236667;
                 -5.470000	57.239444;
                 -5.472778	57.242500;
                 -5.476944	57.245000;
                 -5.480833	57.247222;
                 -5.485278	57.249722;
                 -5.489167	57.252222;
                 -5.495000	57.253889;
                 -5.499722	57.255833;
                 -5.504722	57.258056;
                 -5.509167	57.260000;
                 -5.513056	57.262500;
                 -5.517222	57.265000;
                 -5.519167	57.268333;
                 -5.523056	57.270833;
                 -5.526944	57.273056;
                 -5.533333	57.274444;
                 -5.540556	57.273611;
                 -5.547222	57.275000;
                 -5.553611	57.276111;
                 -5.560278	57.276389;
                 -5.565556	57.274722;
                 -5.570278	57.272778;
                 -5.574722	57.270556;
                 -5.581667	57.270000;
                 -5.588333	57.270278;
                 -5.595833	57.271111;
                 -5.601667	57.272500;
                 -5.608056	57.273889;
                 -5.615833	57.273611;
                 -5.623611	57.274444;
                 -5.628056	57.276389;
                 -5.633056	57.278611;
                 -5.639722	57.279722;
                 -5.647222	57.279444;
                 -5.654167	57.278889;
                 -5.660278	57.277778;
                 -5.668056	57.277778;
                 -5.676389	57.278056;
                 -5.684167	57.277778;
                 -5.690278	57.276667;
                 -5.696111	57.276389;
                 -5.703889	57.276389;
                 -5.711111	57.277222;
                 -5.718056	57.278333;
                 -5.724722	57.279444;
                 -5.728333	57.281944;
                 -5.731389	57.284722;
                 -5.731667	57.288889;
                 -5.730278	57.293056;
                 -5.728056	57.296667;
                 -5.725000	57.299722;
                 -5.722778	57.303333;
                 -5.720000	57.306389;
                 -5.716667	57.309444;
                 -5.714722	57.313056;
                 -5.712778	57.316667;
                 -5.709722	57.319722;
                 -5.705000	57.321667;
                 -5.698333	57.322500;
                 -5.691111	57.323611;
                 -5.685833	57.325278;
                 -5.681389	57.327222;
                 -5.678333	57.330278;
                 -5.674444	57.332778;
                 -5.670000	57.335000;
                 -5.664167	57.336111;
                 -5.656944	57.336667;
                 -5.649444	57.336944;
                 -5.641944	57.336111;
                 -5.633611	57.335833;
                 -5.625833	57.335833;
                 -5.619722	57.336944;
                 -5.613611	57.338333;
                 -5.607500	57.339167;
                 -5.601389	57.340556;
                 -5.595833	57.341944;
                 -5.590556	57.343611;
                 -5.585556	57.345278;
                 -5.580833	57.347500;
                 -5.575556	57.348889;
                 -5.570000	57.350556;
                 -5.564722	57.352222;
                 -5.559444	57.353889;
                 -5.553333	57.355000;
                 -5.547500	57.356111;
                 -5.539444	57.356111;
                 -5.533333	57.357222;
                 -5.527222	57.358333;
                 -5.521944	57.360000;
                 -5.516667	57.361667;
                 -5.513056	57.364167;
                 -5.508333	57.366389;
                 -5.504722	57.368889;
                 -5.500000	57.371111;
                 -5.494722	57.372500;
                 -5.490000	57.374722;
                 -5.485556	57.376944;
                 -5.481667	57.379444;
                 -5.478056	57.381944;
                 -5.474167	57.384722;
                 -5.470556	57.387222;
                 -5.465833	57.389167;
                 -5.461389	57.391389;
                 -5.456667	57.393611;
                 -5.452222	57.396111;
                 -5.448333	57.398611;
                 -5.445278	57.401667;
                 -5.443056	57.405278;
                 -5.440833	57.408889;
                 -5.440278	57.413333;
                 -5.443056	57.416389;
                 -5.449167	57.416111;
                 -5.454722	57.414444;
                 -5.459167	57.412500;
                 -5.462778	57.409722;
                 -5.467500	57.407778;
                 -5.471944	57.405556;
                 -5.475833	57.403056;
                 -5.478056	57.399444;
                 -5.481667	57.396944;
                 -5.486944	57.395278;
                 -5.491667	57.393056;
                 -5.494722	57.390000;
                 -5.499167	57.388056;
                 -5.503889	57.385833;
                 -5.509167	57.384167;
                 -5.515278	57.383056;
                 -5.519722	57.381111;
                 -5.523611	57.378611;
                 -5.529167	57.376944;
                 -5.534444	57.375278;
                 -5.538056	57.372500;
                 -5.541111	57.369444;
                 -5.543889	57.366389;
                 -5.547778	57.363889;
                 -5.551389	57.361389;
                 -5.556944	57.359722;
                 -5.563056	57.358611;
                 -5.569167	57.357500;
                 -5.575278	57.356389;
                 -5.582222	57.355833;
                 -5.588056	57.354444;
                 -5.595278	57.353889;
                 -5.601944	57.353333;
                 -5.609722	57.353056;
                 -5.617222	57.353889;
                 -5.623056	57.355556;
                 -5.627778	57.357500;
                 -5.630833	57.360556;
                 -5.632778	57.363889;
                 -5.635000	57.366944;
                 -5.638056	57.370000;
                 -5.638056	57.374167;
                 -5.633611	57.376111;
                 -5.627500	57.377222;
                 -5.622222	57.378889;
                 -5.616667	57.381111;
                 -5.612222	57.383056;
                 -5.608333	57.385833;
                 -5.605556	57.388611;
                 -5.605000	57.392500;
                 -5.607778	57.395278;
                 -5.611111	57.398056;
                 -5.616667	57.399722;
                 -5.622778	57.398611;
                 -5.625000	57.395000;
                 -5.628056	57.391944;
                 -5.632500	57.389722;
                 -5.637222	57.387778;
                 -5.642222	57.386111;
                 -5.646944	57.384167;
                 -5.653056	57.382778;
                 -5.659167	57.381667;
                 -5.665278	57.380556;
                 -5.670556	57.379167;
                 -5.674444	57.376389;
                 -5.678333	57.373889;
                 -5.682500	57.371667;
                 -5.688056	57.370278;
                 -5.693333	57.368611;
                 -5.698056	57.366389;
                 -5.701667	57.363889;
                 -5.706389	57.361667;
                 -5.712500	57.360556;
                 -5.717778	57.358889;
                 -5.722778	57.357222;
                 -5.727500	57.355278;
                 -5.731111	57.352500;
                 -5.736667	57.351111;
                 -5.741944	57.349444;
                 -5.747222	57.347778;
                 -5.752778	57.346111;
                 -5.757778	57.344444;
                 -5.763889	57.343333;
                 -5.771111	57.342778;
                 -5.777500	57.343056;
                 -5.783333	57.344722;
                 -5.788333	57.346667;
                 -5.791389	57.349444;
                 -5.794167	57.352500;
                 -5.798056	57.355000;
                 -5.803056	57.356944;
                 -5.809444	57.358056;
                 -5.812500	57.360833;
                 -5.814722	57.364167;
                 -5.816944	57.367500;
                 -5.818889	57.370833;
                 -5.821111	57.374167;
                 -5.823056	57.377500;
                 -5.823333	57.381667;
                 -5.818333	57.380556;
                 -5.813056	57.381944;
                 -5.812500	57.386667;
                 -5.813611	57.390278;
                 -5.810000	57.391111;
                 -5.804167	57.390556;
                 -5.799722	57.392500;
                 -5.800000	57.396667;
                 -5.802222	57.400000;
                 -5.805278	57.402778;
                 -5.810000	57.405000;
                 -5.816667	57.406111;
                 -5.819444	57.408889;
                 -5.820833	57.412500;
                 -5.819444	57.415833;
                 -5.817500	57.419444;
                 -5.815278	57.423056;
                 -5.812222	57.426111;
                 -5.809444	57.429167;
                 -5.806111	57.432222;
                 -5.802500	57.435278;
                 -5.801111	57.438333;
                 -5.807778	57.438611;
                 -5.813889	57.437500;
                 -5.821667	57.437500;
                 -5.828611	57.436667;
                 -5.836111	57.436667;
                 -5.844722	57.436944;
                 -5.851389	57.438056;
                 -5.856111	57.440278;
                 -5.861111	57.442222;
                 -5.863333	57.445556;
                 -5.864167	57.449167;
                 -5.864722	57.453333;
                 -5.863333	57.457500;
                 -5.863611	57.461667;
                 -5.866667	57.464444;
                 -5.870278	57.466944;
                 -5.873611	57.469722;
                 -5.874722	57.473333;
                 -5.875000	57.477500;
                 -5.874444	57.482222;
                 -5.873889	57.486667;
                 -5.872222	57.490833;
                 -5.870833	57.494722;
                 -5.870278	57.499444;
                 -5.868056	57.503056;
                 -5.865833	57.506389;
                 -5.863056	57.509722;
                 -5.860833	57.513333;
                 -5.858611	57.516667;
                 -5.856389	57.520278;
                 -5.854167	57.523889;
                 -5.851944	57.527500;
                 -5.851389	57.531944;
                 -5.852500	57.535833;
                 -5.852778	57.540000;
                 -5.853889	57.543611;
                 -5.853611	57.548333;
                 -5.852778	57.552778;
                 -5.849722	57.555833;
                 -5.844444	57.557500;
                 -5.839722	57.555556;
                 -5.834444	57.556111;
                 -5.834722	57.560278;
                 -5.835833	57.563889;
                 -5.836111	57.568056;
                 -5.837500	57.571944;
                 -5.839722	57.575000;
                 -5.835833	57.575556;
                 -5.831111	57.573333;
                 -5.824444	57.572222;
                 -5.818333	57.573333;
                 -5.814444	57.576111;
                 -5.807500	57.576667;
                 -5.801944	57.575000;
                 -5.797222	57.573056;
                 -5.791389	57.571389;
                 -5.782778	57.571111;
                 -5.776111	57.570000;
                 -5.773056	57.566944;
                 -5.771944	57.563333;
                 -5.769722	57.560000;
                 -5.765833	57.557500;
                 -5.760000	57.555833;
                 -5.753333	57.554722;
                 -5.748611	57.552778;
                 -5.746667	57.549444;
                 -5.748889	57.545833;
                 -5.749167	57.541389;
                 -5.742500	57.541111;
                 -5.737222	57.542778;
                 -5.730278	57.543333;
                 -5.724722	57.541667;
                 -5.720556	57.539167;
                 -5.714722	57.538611;
                 -5.711667	57.541667;
                 -5.712778	57.545278;
                 -5.714167	57.549167;
                 -5.709722	57.550278;
                 -5.704722	57.548333;
                 -5.702500	57.545000;
                 -5.702500	57.541667;
                 -5.702222	57.537500;
                 -5.700833	57.533889;
                 -5.699722	57.530278;
                 -5.696667	57.527222;
                 -5.692500	57.524722;
                 -5.686944	57.523333;
                 -5.680556	57.521944;
                 -5.675556	57.520000;
                 -5.670556	57.518056;
                 -5.666667	57.515556;
                 -5.662778	57.513056;
                 -5.658889	57.510556;
                 -5.655000	57.508056;
                 -5.650278	57.506111;
                 -5.645000	57.506944;
                 -5.642778	57.510278;
                 -5.644722	57.513611;
                 -5.646944	57.516944;
                 -5.648889	57.520278;
                 -5.650278	57.523889;
                 -5.651389	57.527778;
                 -5.652500	57.531389;
                 -5.650278	57.534167;
                 -5.644444	57.533333;
                 -5.640556	57.530833;
                 -5.636667	57.528611;
                 -5.632778	57.526111;
                 -5.626944	57.524444;
                 -5.621111	57.523611;
                 -5.617500	57.526389;
                 -5.613611	57.528889;
                 -5.608056	57.530556;
                 -5.599722	57.530278;
                 -5.592222	57.529444;
                 -5.585556	57.528333;
                 -5.578611	57.528056;
                 -5.575000	57.530556;
                 -5.571111	57.533056;
                 -5.564167	57.533889;
                 -5.556667	57.533056;
                 -5.551111	57.531389;
                 -5.544167	57.530278;
                 -5.535833	57.529722;
                 -5.528889	57.530556;
                 -5.524167	57.532500;
                 -5.521944	57.536111;
                 -5.524167	57.539444;
                 -5.528056	57.541944;
                 -5.533889	57.543611;
                 -5.540556	57.544722;
                 -5.547222	57.545833;
                 -5.554722	57.546667;
                 -5.562222	57.547500;
                 -5.569722	57.548333;
                 -5.576389	57.549444;
                 -5.583333	57.550556;
                 -5.588889	57.552222;
                 -5.596667	57.551944;
                 -5.603611	57.551389;
                 -5.610556	57.550833;
                 -5.618889	57.551111;
                 -5.627500	57.551389;
                 -5.632778	57.550000;
                 -5.638889	57.548611;
                 -5.647500	57.549167;
                 -5.652222	57.546944;
                 -5.656389	57.544722;
                 -5.663611	57.544167;
                 -5.669167	57.545833;
                 -5.673056	57.548333;
                 -5.676389	57.551111;
                 -5.679167	57.554167;
                 -5.684167	57.556111;
                 -5.689722	57.557778;
                 -5.696667	57.558889;
                 -5.703056	57.560000;
                 -5.705278	57.563333;
                 -5.700000	57.565000;
                 -5.692778	57.565556;
                 -5.685278	57.565833;
                 -5.679167	57.566944;
                 -5.681389	57.569167;
                 -5.688056	57.570556;
                 -5.694444	57.571667;
                 -5.701389	57.572778;
                 -5.708056	57.574167;
                 -5.714722	57.575278;
                 -5.719444	57.577222;
                 -5.725278	57.578889;
                 -5.728333	57.581667;
                 -5.732222	57.584167;
                 -5.733333	57.587778;
                 -5.734722	57.591667;
                 -5.735000	57.595833;
                 -5.735000	57.600000;
                 -5.736389	57.603611;
                 -5.737500	57.607222;
                 -5.739722	57.610556;
                 -5.743611	57.613056;
                 -5.746667	57.615833;
                 -5.751389	57.618056;
                 -5.757222	57.619444;
                 -5.763056	57.621111;
                 -5.769722	57.622222;
                 -5.775278	57.623889;
                 -5.781389	57.625556;
                 -5.786111	57.627500;
                 -5.791111	57.629722;
                 -5.795556	57.631667;
                 -5.803333	57.632500;
                 -5.811111	57.633056;
                 -5.815000	57.635556;
                 -5.817222	57.638889;
                 -5.816389	57.643333;
                 -5.814167	57.646944;
                 -5.811944	57.650556;
                 -5.809167	57.653611;
                 -5.805833	57.656667;
                 -5.803611	57.660278;
                 -5.801389	57.663889;
                 -5.797778	57.666667;
                 -5.792500	57.668056;
                 -5.787778	57.670278;
                 -5.785000	57.673333;
                 -5.785278	57.677500;
                 -5.785278	57.681667;
                 -5.785556	57.685833;
                 -5.785833	57.690000;
                 -5.782222	57.692500;
                 -5.774444	57.692500;
                 -5.768333	57.693889;
                 -5.765278	57.696944;
                 -5.760833	57.698889;
                 -5.754722	57.700000;
                 -5.747500	57.700833;
                 -5.739722	57.700833;
                 -5.732222	57.700000;
                 -5.727222	57.698056;
                 -5.724444	57.695278;
                 -5.719444	57.693333;
                 -5.711667	57.693333;
                 -5.706111	57.691667;
                 -5.703889	57.688333;
                 -5.698889	57.686389;
                 -5.692222	57.686944;
                 -5.688333	57.689722;
                 -5.684444	57.692222;
                 -5.681389	57.695278;
                 -5.678333	57.698889;
                 -5.679722	57.702500;
                 -5.682500	57.705556;
                 -5.687500	57.707500;
                 -5.691389	57.710000;
                 -5.695278	57.712500;
                 -5.695556	57.716389;
                 -5.699722	57.718889;
                 -5.703611	57.721389;
                 -5.710278	57.722500;
                 -5.718056	57.722500;
                 -5.724722	57.721667;
                 -5.733333	57.722222;
                 -5.741944	57.722500;
                 -5.750556	57.722778;
                 -5.756389	57.724444;
                 -5.759167	57.727222;
                 -5.762500	57.730278;
                 -5.766389	57.732778;
                 -5.771111	57.734722;
                 -5.777778	57.735833;
                 -5.785278	57.736667;
                 -5.792222	57.737778;
                 -5.798889	57.738889;
                 -5.804722	57.740556;
                 -5.807500	57.743333;
                 -5.810833	57.746389;
                 -5.809444	57.750278;
                 -5.805556	57.753056;
                 -5.801667	57.755556;
                 -5.803056	57.759167;
                 -5.807778	57.761389;
                 -5.808056	57.765556;
                 -5.805833	57.769167;
                 -5.803056	57.772222;
                 -5.800833	57.775833;
                 -5.800000	57.780278;
                 -5.799444	57.784722;
                 -5.800556	57.788611;
                 -5.800833	57.792778;
                 -5.801944	57.796389;
                 -5.803333	57.800000;
                 -5.805556	57.803333;
                 -5.808611	57.806389;
                 -5.811389	57.809167;
                 -5.815556	57.811667;
                 -5.817500	57.814722;
                 -5.818889	57.818611;
                 -5.819167	57.822778;
                 -5.819444	57.826944;
                 -5.818889	57.831389;
                 -5.817500	57.835556;
                 -5.815833	57.839444;
                 -5.815278	57.844167;
                 -5.816389	57.847778;
                 -5.820556	57.850278;
                 -5.824444	57.852778;
                 -5.825556	57.856389;
                 -5.820278	57.858056;
                 -5.815000	57.859722;
                 -5.807222	57.858889;
                 -5.801389	57.857222;
                 -5.795556	57.855556;
                 -5.788889	57.854444;
                 -5.780278	57.854167;
                 -5.771667	57.853889;
                 -5.763889	57.853889;
                 -5.760278	57.856667;
                 -5.756389	57.859167;
                 -5.753333	57.862222;
                 -5.747778	57.863889;
                 -5.740833	57.863611;
                 -5.734444	57.862500;
                 -5.726667	57.862500;
                 -5.721111	57.864167;
                 -5.715833	57.865833;
                 -5.708056	57.865833;
                 -5.701111	57.864722;
                 -5.693611	57.863889;
                 -5.686944	57.862778;
                 -5.683889	57.860833;
                 -5.680556	57.858056;
                 -5.682222	57.853889;
                 -5.686111	57.851389;
                 -5.689722	57.848611;
                 -5.690278	57.845000;
                 -5.686389	57.842500;
                 -5.685000	57.838889;
                 -5.683056	57.835556;
                 -5.678889	57.833056;
                 -5.675000	57.830556;
                 -5.671111	57.828333;
                 -5.667222	57.825833;
                 -5.666111	57.822222;
                 -5.664722	57.818333;
                 -5.665556	57.813889;
                 -5.665278	57.809722;
                 -5.664167	57.806111;
                 -5.663889	57.801944;
                 -5.663611	57.797778;
                 -5.662222	57.793889;
                 -5.661111	57.790278;
                 -5.658056	57.787500;
                 -5.654167	57.785000;
                 -5.652222	57.781667;
                 -5.648889	57.778889;
                 -5.643333	57.777222;
                 -5.638333	57.775000;
                 -5.634444	57.772778;
                 -5.630278	57.770278;
                 -5.625556	57.768056;
                 -5.620000	57.766667;
                 -5.613056	57.765278;
                 -5.606944	57.765556;
                 -5.602500	57.767778;
                 -5.605556	57.770556;
                 -5.609444	57.773056;
                 -5.614167	57.775000;
                 -5.619167	57.777222;
                 -5.623889	57.779167;
                 -5.626111	57.782500;
                 -5.623889	57.786111;
                 -5.619444	57.788056;
                 -5.613056	57.789167;
                 -5.606389	57.788056;
                 -5.600556	57.786389;
                 -5.597778	57.783611;
                 -5.592778	57.781389;
                 -5.585000	57.781667;
                 -5.581944	57.784722;
                 -5.581389	57.788333;
                 -5.584444	57.791389;
                 -5.588333	57.793611;
                 -5.591389	57.796667;
                 -5.592500	57.800278;
                 -5.592778	57.804444;
                 -5.589722	57.807500;
                 -5.586944	57.810556;
                 -5.584444	57.814167;
                 -5.582222	57.817778;
                 -5.580833	57.821944;
                 -5.580278	57.826389;
                 -5.580556	57.830556;
                 -5.582500	57.833889;
                 -5.586389	57.836389;
                 -5.591389	57.838333;
                 -5.594444	57.841111;
                 -5.597500	57.843889;
                 -5.601389	57.846389;
                 -5.607222	57.848056;
                 -5.613889	57.849444;
                 -5.619722	57.850833;
                 -5.625556	57.852500;
                 -5.630278	57.854444;
                 -5.634167	57.856944;
                 -5.636389	57.860278;
                 -5.636667	57.864444;
                 -5.638889	57.867778;
                 -5.642778	57.870278;
                 -5.646667	57.872778;
                 -5.649722	57.875556;
                 -5.650833	57.879167;
                 -5.647222	57.881944;
                 -5.642500	57.883889;
                 -5.645556	57.886944;
                 -5.650278	57.888889;
                 -5.648889	57.892778;
                 -5.646667	57.896389;
                 -5.644444	57.900000;
                 -5.641667	57.903056;
                 -5.638333	57.906111;
                 -5.635556	57.909167;
                 -5.631667	57.911944;
                 -5.627778	57.914444;
                 -5.623056	57.916667;
                 -5.618611	57.918611;
                 -5.614722	57.921389;
                 -5.610833	57.923889;
                 -5.608889	57.920556;
                 -5.604722	57.918056;
                 -5.599722	57.916111;
                 -5.595833	57.913611;
                 -5.593056	57.910833;
                 -5.589722	57.907778;
                 -5.585000	57.905833;
                 -5.578056	57.906389;
                 -5.574167	57.909167;
                 -5.569444	57.911111;
                 -5.563611	57.912222;
                 -5.555556	57.912500;
                 -5.553333	57.909167;
                 -5.555556	57.905556;
                 -5.558056	57.901944;
                 -5.555833	57.898611;
                 -5.555556	57.894444;
                 -5.557222	57.890556;
                 -5.555833	57.886944;
                 -5.554722	57.883056;
                 -5.552500	57.879722;
                 -5.551389	57.876111;
                 -5.550000	57.873333;
                 -5.548056	57.870000;
                 -5.543333	57.868056;
                 -5.537222	57.866389;
                 -5.530833	57.865000;
                 -5.523889	57.863889;
                 -5.518056	57.862222;
                 -5.512500	57.860556;
                 -5.506667	57.859167;
                 -5.500833	57.857500;
                 -5.495833	57.855556;
                 -5.489167	57.854167;
                 -5.482500	57.853056;
                 -5.477500	57.851111;
                 -5.472778	57.848889;
                 -5.466944	57.847222;
                 -5.460000	57.848056;
                 -5.456944	57.851111;
                 -5.458056	57.854722;
                 -5.457500	57.859444;
                 -5.455000	57.862778;
                 -5.450556	57.865000;
                 -5.445833	57.866944;
                 -5.443611	57.870556;
                 -5.443056	57.875278;
                 -5.443056	57.879444;
                 -5.441667	57.883333;
                 -5.439444	57.886944;
                 -5.438056	57.891111;
                 -5.436389	57.895000;
                 -5.434444	57.898611;
                 -5.430556	57.901389;
                 -5.425000	57.902778;
                 -5.418056	57.903611;
                 -5.409444	57.903056;
                 -5.403889	57.901389;
                 -5.398056	57.900000;
                 -5.394167	57.897500;
                 -5.388056	57.895833;
                 -5.381667	57.894722;
                 -5.375833	57.893056;
                 -5.370000	57.891389;
                 -5.363889	57.889722;
                 -5.357500	57.888611;
                 -5.353611	57.886111;
                 -5.347778	57.883889;
                 -5.343889	57.881667;
                 -5.341667	57.878333;
                 -5.338611	57.875278;
                 -5.333611	57.873333;
                 -5.328889	57.871389;
                 -5.324167	57.869167;
                 -5.319444	57.867222;
                 -5.313611	57.865556;
                 -5.308611	57.863611;
                 -5.302778	57.861944;
                 -5.297222	57.860278;
                 -5.290278	57.859167;
                 -5.285278	57.856944;
                 -5.280556	57.855000;
                 -5.275000	57.853333;
                 -5.269167	57.851667;
                 -5.261389	57.850833;
                 -5.254722	57.849722;
                 -5.248889	57.848056;
                 -5.242222	57.846944;
                 -5.236389	57.845278;
                 -5.229722	57.843889;
                 -5.221944	57.844167;
                 -5.224167	57.846667;
                 -5.228056	57.848889;
                 -5.230278	57.852222;
                 -5.233056	57.855278;
                 -5.236111	57.858056;
                 -5.238889	57.860833;
                 -5.242222	57.863889;
                 -5.246111	57.866389;
                 -5.251111	57.868333;
                 -5.257500	57.869722;
                 -5.264444	57.870833;
                 -5.271111	57.871944;
                 -5.277778	57.873333;
                 -5.285278	57.873889;
                 -5.292222	57.875278;
                 -5.297778	57.876944;
                 -5.302778	57.878889;
                 -5.306667	57.881389;
                 -5.309722	57.884167;
                 -5.311667	57.887500;
                 -5.315556	57.890000;
                 -5.320556	57.891944;
                 -5.325556	57.894167;
                 -5.328333	57.896944;
                 -5.332500	57.899444;
                 -5.337222	57.901389;
                 -5.343889	57.902778;
                 -5.350833	57.903889;
                 -5.357500	57.905000;
                 -5.365000	57.905833;
                 -5.372778	57.906667;
                 -5.375278	57.907222;
                 -5.381944	57.908333;
                 -5.386667	57.910278;
                 -5.391667	57.912500;
                 -5.392778	57.916111;
                 -5.396111	57.919167;
                 -5.398889	57.921944;
                 -5.400833	57.925278;
                 -5.399444	57.929167;
                 -5.392500	57.930000;
                 -5.384167	57.929444;
                 -5.378056	57.928056;
                 -5.371389	57.927500;
                 -5.367500	57.930278;
                 -5.365278	57.933889;
                 -5.362778	57.937222;
                 -5.355833	57.936944;
                 -5.351389	57.935000;
                 -5.345556	57.933333;
                 -5.339722	57.931667;
                 -5.339444	57.928611;
                 -5.340833	57.924444;
                 -5.338611	57.921111;
                 -5.334722	57.918611;
                 -5.330833	57.916389;
                 -5.326667	57.913889;
                 -5.321944	57.911667;
                 -5.317222	57.909722;
                 -5.311389	57.908056;
                 -5.304444	57.906944;
                 -5.296944	57.906111;
                 -5.288333	57.905833;
                 -5.280556	57.905833;
                 -5.274444	57.906944;
                 -5.268889	57.908611;
                 -5.263333	57.910278;
                 -5.258889	57.912222;
                 -5.253333	57.913889;
                 -5.247500	57.913056;
                 -5.242778	57.911111;
                 -5.235000	57.910278;
                 -5.226389	57.910000;
                 -5.222500	57.907500;
                 -5.219444	57.904444;
                 -5.217222	57.901389;
                 -5.213333	57.898889;
                 -5.210556	57.895833;
                 -5.205556	57.893889;
                 -5.201667	57.891389;
                 -5.196667	57.889444;
                 -5.191111	57.887778;
                 -5.186389	57.885833;
                 -5.181389	57.883611;
                 -5.175556	57.881944;
                 -5.169722	57.880278;
                 -5.164167	57.878889;
                 -5.158333	57.877222;
                 -5.152222	57.875556;
                 -5.146667	57.873889;
                 -5.140000	57.872778;
                 -5.133333	57.871389;
                 -5.127500	57.869722;
                 -5.123611	57.867222;
                 -5.119722	57.864722;
                 -5.115833	57.862500;
                 -5.111944	57.860000;
                 -5.108611	57.856944;
                 -5.105833	57.854167;
                 -5.102778	57.851389;
                 -5.100833	57.848056;
                 -5.098611	57.844722;
                 -5.095833	57.841667;
                 -5.091667	57.839444;
                 -5.087778	57.836944;
                 -5.083889	57.834444;
                 -5.080000	57.831944;
                 -5.076944	57.829167;
                 -5.073611	57.828889;
                 -5.075556	57.832222;
                 -5.076944	57.835833;
                 -5.076944	57.840000;
                 -5.079167	57.843333;
                 -5.081111	57.846667;
                 -5.082222	57.850278;
                 -5.084167	57.853611;
                 -5.086389	57.856944;
                 -5.088333	57.860278;
                 -5.091667	57.863056;
                 -5.094444	57.866111;
                 -5.098333	57.868611;
                 -5.103333	57.870556;
                 -5.108889	57.872222;
                 -5.113889	57.874167;
                 -5.118611	57.876389;
                 -5.123611	57.878333;
                 -5.127500	57.880833;
                 -5.131389	57.883333;
                 -5.135278	57.885833;
                 -5.140000	57.887778;
                 -5.146667	57.888889;
                 -5.152500	57.890556;
                 -5.159444	57.891944;
                 -5.164444	57.893889;
                 -5.168333	57.896389;
                 -5.171111	57.899167;
                 -5.175000	57.901667;
                 -5.180000	57.903611;
                 -5.184722	57.905833;
                 -5.189722	57.907778;
                 -5.193611	57.910278;
                 -5.198611	57.912500;
                 -5.203333	57.914444;
                 -5.208889	57.916111;
                 -5.214722	57.917778;
                 -5.219722	57.919722;
                 -5.220000	57.923056;
                 -5.215278	57.925000;
                 -5.209167	57.926111;
                 -5.203056	57.927222;
                 -5.197500	57.928889;
                 -5.193611	57.931389;
                 -5.191389	57.935000;
                 -5.189167	57.938611;
                 -5.186111	57.941667;
                 -5.186389	57.945833;
                 -5.189167	57.948611;
                 -5.193056	57.951111;
                 -5.197222	57.953611;
                 -5.202778	57.955278;
                 -5.209722	57.956389;
                 -5.217222	57.957222;
                 -5.225000	57.958056;
                 -5.231667	57.959167;
                 -5.237500	57.960833;
                 -5.241389	57.963333;
                 -5.246389	57.965833;
                 -5.251111	57.967778;
                 -5.256111	57.970000;
                 -5.261667	57.971389;
                 -5.267500	57.973056;
                 -5.275278	57.973889;
                 -5.283056	57.974722;
                 -5.289722	57.975833;
                 -5.295278	57.977500;
                 -5.300278	57.979722;
                 -5.302500	57.982778;
                 -5.304444	57.986111;
                 -5.306667	57.989444;
                 -5.310556	57.991944;
                 -5.314444	57.995000;
                 -5.319444	57.996944;
                 -5.325278	57.998611;
                 -5.331667	57.999722;
                 -5.338611	58.000833;
                 -5.345278	58.002222;
                 -5.349167	58.004722;
                 -5.352222	58.007500;
                 -5.354444	58.010833;
                 -5.355556	58.014444;
                 -5.357778	58.017778;
                 -5.360556	58.020556;
                 -5.364444	58.023056;
                 -5.369722	58.025278;
                 -5.377222	58.026111;
                 -5.385833	58.026389;
                 -5.393611	58.026111;
                 -5.401389	58.026944;
                 -5.406944	58.028611;
                 -5.412778	58.030278;
                 -5.419722	58.031944;
                 -5.427222	58.032778;
                 -5.432222	58.034722;
                 -5.434167	58.038056;
                 -5.432778	58.041111;
                 -5.430000	58.044167;
                 -5.426667	58.047222;
                 -5.426389	58.050833;
                 -5.430278	58.053333;
                 -5.437778	58.054167;
                 -5.444444	58.055278;
                 -5.448611	58.057778;
                 -5.452500	58.060278;
                 -5.457500	58.062222;
                 -5.461389	58.064722;
                 -5.464444	58.067778;
                 -5.462222	58.071111;
                 -5.458333	58.073889;
                 -5.456111	58.077500;
                 -5.457222	58.081111;
                 -5.457500	58.085278;
                 -5.456111	58.089444;
                 -5.454722	58.093333;
                 -5.451389	58.096389;
                 -5.447500	58.099167;
                 -5.442778	58.101111;
                 -5.436667	58.102222;
                 -5.428889	58.102500;
                 -5.424167	58.100278;
                 -5.419167	58.098333;
                 -5.415000	58.095833;
                 -5.411111	58.093333;
                 -5.408333	58.090556;
                 -5.404167	58.088056;
                 -5.400278	58.085556;
                 -5.396111	58.083056;
                 -5.393333	58.080278;
                 -5.391111	58.076944;
                 -5.390833	58.072778;
                 -5.389722	58.069167;
                 -5.386667	58.066389;
                 -5.380833	58.064722;
                 -5.373889	58.063333;
                 -5.368056	58.063611;
                 -5.362500	58.065278;
                 -5.358611	58.067778;
                 -5.355833	58.070833;
                 -5.352500	58.073889;
                 -5.347222	58.075556;
                 -5.339444	58.074722;
                 -5.330833	58.074444;
                 -5.323056	58.074444;
                 -5.316111	58.074167;
                 -5.311389	58.072222;
                 -5.308056	58.069167;
                 -5.305278	58.066389;
                 -5.301389	58.063889;
                 -5.294444	58.062778;
                 -5.287500	58.062500;
                 -5.280556	58.063056;
                 -5.275278	58.064722;
                 -5.268889	58.065833;
                 -5.264167	58.067778;
                 -5.259444	58.070000;
                 -5.259722	58.074167;
                 -5.260000	58.078333;
                 -5.256944	58.081389;
                 -5.252222	58.083333;
                 -5.249167	58.086389;
                 -5.246111	58.089444;
                 -5.247222	58.093333;
                 -5.249444	58.096667;
                 -5.251389	58.099722;
                 -5.255278	58.102222;
                 -5.259444	58.104722;
                 -5.266111	58.106111;
                 -5.271944	58.107500;
                 -5.274167	58.110833;
                 -5.269167	58.113056;
                 -5.262500	58.113611;
                 -5.253611	58.113333;
                 -5.245000	58.112778;
                 -5.240556	58.115000;
                 -5.245278	58.116944;
                 -5.253056	58.117778;
                 -5.260833	58.118611;
                 -5.267500	58.119722;
                 -5.272500	58.121944;
                 -5.276389	58.124444;
                 -5.276667	58.128333;
                 -5.271944	58.130556;
                 -5.263056	58.130278;
                 -5.254722	58.129722;
                 -5.246944	58.130000;
                 -5.238889	58.130000;
                 -5.232778	58.131111;
                 -5.229722	58.134167;
                 -5.228333	58.138333;
                 -5.231111	58.141111;
                 -5.236111	58.143056;
                 -5.243889	58.143056;
                 -5.251667	58.143056;
                 -5.259444	58.142778;
                 -5.267222	58.143611;
                 -5.273056	58.145278;
                 -5.278889	58.146944;
                 -5.283889	58.148889;
                 -5.285833	58.152222;
                 -5.283611	58.155833;
                 -5.279167	58.157778;
                 -5.276944	58.160556;
                 -5.282778	58.162222;
                 -5.291389	58.162500;
                 -5.298056	58.163611;
                 -5.303889	58.165278;
                 -5.305833	58.168611;
                 -5.307222	58.172500;
                 -5.311111	58.174722;
                 -5.317778	58.176111;
                 -5.323611	58.177778;
                 -5.329722	58.179444;
                 -5.333611	58.181667;
                 -5.335556	58.185000;
                 -5.335000	58.188611;
                 -5.331944	58.191667;
                 -5.327500	58.193889;
                 -5.325833	58.196944;
                 -5.330833	58.199167;
                 -5.337778	58.200278;
                 -5.345556	58.201111;
                 -5.350000	58.203056;
                 -5.353333	58.205833;
                 -5.354444	58.209722;
                 -5.354722	58.213889;
                 -5.357778	58.216667;
                 -5.362500	58.218889;
                 -5.368333	58.220278;
                 -5.374167	58.221944;
                 -5.380000	58.223611;
                 -5.386111	58.225278;
                 -5.390000	58.227778;
                 -5.395000	58.229722;
                 -5.398056	58.232500;
                 -5.397500	58.236111;
                 -5.393611	58.238889;
                 -5.390278	58.241944;
                 -5.388056	58.245556;
                 -5.386667	58.249444;
                 -5.383611	58.252500;
                 -5.378056	58.254167;
                 -5.371111	58.253889;
                 -5.370000	58.250278;
                 -5.370000	58.246111;
                 -5.362778	58.246667;
                 -5.355833	58.247222;
                 -5.347778	58.246389;
                 -5.343889	58.244167;
                 -5.340000	58.241667;
                 -5.334167	58.240000;
                 -5.326389	58.239167;
                 -5.317778	58.238889;
                 -5.310000	58.238056;
                 -5.307222	58.235278;
                 -5.305000	58.231944;
                 -5.301944	58.228889;
                 -5.298889	58.226111;
                 -5.291389	58.225278;
                 -5.285556	58.226944;
                 -5.281111	58.228889;
                 -5.277778	58.232222;
                 -5.273056	58.234167;
                 -5.265278	58.234167;
                 -5.258333	58.235000;
                 -5.253611	58.236944;
                 -5.248333	58.238611;
                 -5.240278	58.238611;
                 -5.233333	58.239167;
                 -5.228611	58.241389;
                 -5.222500	58.242500;
                 -5.213611	58.242222;
                 -5.206667	58.242778;
                 -5.201111	58.244444;
                 -5.194722	58.245556;
                 -5.186944	58.245556;
                 -5.181111	58.243889;
                 -5.175278	58.242222;
                 -5.172500	58.239444;
                 -5.170278	58.236111;
                 -5.164167	58.237222;
                 -5.160833	58.240278;
                 -5.156944	58.242778;
                 -5.155556	58.246944;
                 -5.152222	58.250000;
                 -5.146944	58.251667;
                 -5.138889	58.251667;
                 -5.133611	58.253333;
                 -5.127222	58.254444;
                 -5.121389	58.252778;
                 -5.116389	58.251667;
                 -5.111944	58.253611;
                 -5.110000	58.257778;
                 -5.111389	58.261389;
                 -5.109167	58.265000;
                 -5.102778	58.266111;
                 -5.095833	58.265000;
                 -5.091111	58.262778;
                 -5.087222	58.260278;
                 -5.083889	58.257500;
                 -5.080000	58.255000;
                 -5.075278	58.253056;
                 -5.068333	58.251944;
                 -5.061667	58.250556;
                 -5.057500	58.248056;
                 -5.054722	58.245278;
                 -5.047778	58.243889;
                 -5.042500	58.245556;
                 -5.035278	58.246111;
                 -5.027500	58.245278;
                 -5.021389	58.245556;
                 -5.016111	58.247222;
                 -5.010556	58.248611;
                 -5.003611	58.248333;
                 -4.998889	58.246389;
                 -4.995833	58.243611;
                 -4.994722	58.239722;
                 -4.993333	58.236111;
                 -4.990556	58.233056;
                 -4.984722	58.231667;
                 -4.976944	58.230833;
                 -4.971111	58.229167;
                 -4.964444	58.227778;
                 -4.956667	58.226944;
                 -4.952222	58.228333;
                 -4.949722	58.231667;
                 -4.953889	58.234167;
                 -4.958611	58.236389;
                 -4.965278	58.237500;
                 -4.972222	58.238889;
                 -4.976944	58.240833;
                 -4.976389	58.245278;
                 -4.970833	58.246944;
                 -4.963611	58.247500;
                 -4.957500	58.248611;
                 -4.950556	58.249167;
                 -4.942500	58.248333;
                 -4.935833	58.248056;
                 -4.936944	58.251944;
                 -4.942778	58.253333;
                 -4.950556	58.254167;
                 -4.957222	58.255556;
                 -4.965833	58.255833;
                 -4.973611	58.255833;
                 -4.981389	58.256667;
                 -4.987222	58.258333;
                 -4.995000	58.259167;
                 -5.000556	58.257500;
                 -5.008333	58.257222;
                 -5.015000	58.257778;
                 -5.021944	58.258889;
                 -5.028333	58.257778;
                 -5.034722	58.256667;
                 -5.042222	58.257500;
                 -5.048056	58.259167;
                 -5.055000	58.260278;
                 -5.060556	58.262500;
                 -5.065556	58.264444;
                 -5.070556	58.266667;
                 -5.074444	58.268889;
                 -5.080278	58.270556;
                 -5.086111	58.272222;
                 -5.091944	58.273889;
                 -5.097778	58.275556;
                 -5.101944	58.278056;
                 -5.104722	58.280833;
                 -5.107778	58.283889;
                 -5.112778	58.285833;
                 -5.121389	58.286111;
                 -5.129167	58.286944;
                 -5.130000	58.290833;
                 -5.129722	58.294444;
                 -5.127500	58.298056;
                 -5.124167	58.301111;
                 -5.121944	58.304444;
                 -5.126667	58.306667;
                 -5.130833	58.309167;
                 -5.127778	58.312222;
                 -5.123889	58.314722;
                 -5.120556	58.317778;
                 -5.118333	58.321389;
                 -5.124167	58.323056;
                 -5.131389	58.322500;
                 -5.136111	58.320278;
                 -5.140556	58.318333;
                 -5.146111	58.316667;
                 -5.151667	58.315000;
                 -5.155556	58.317500;
                 -5.154167	58.321667;
                 -5.150833	58.324722;
                 -5.149722	58.327778;
                 -5.152500	58.330556;
                 -5.154722	58.333889;
                 -5.158611	58.336389;
                 -5.162500	58.338889;
                 -5.165833	58.341667;
                 -5.166111	58.345833;
                 -5.161111	58.348056;
                 -5.155000	58.349167;
                 -5.148611	58.350278;
                 -5.144722	58.352778;
                 -5.147778	58.355556;
                 -5.150833	58.358611;
                 -5.151111	58.362778;
                 -5.148611	58.366111;
                 -5.146389	58.369722;
                 -5.143333	58.372778;
                 -5.140000	58.375833;
                 -5.137222	58.378889;
                 -5.135556	58.383056;
                 -5.134167	58.387222;
                 -5.133333	58.391667;
                 -5.133611	58.395833;
                 -5.132778	58.400278;
                 -5.131389	58.404444;
                 -5.127500	58.406944;
                 -5.120278	58.407500;
                 -5.115556	58.405556;
                 -5.112222	58.402778;
                 -5.111111	58.398889;
                 -5.109167	58.395556;
                 -5.104167	58.393611;
                 -5.096111	58.393611;
                 -5.089167	58.394444;
                 -5.081667	58.393611;
                 -5.075556	58.391944;
                 -5.069722	58.390278;
                 -5.061111	58.390000;
                 -5.053333	58.389167;
                 -5.047500	58.387500;
                 -5.042500	58.385278;
                 -5.037500	58.383333;
                 -5.031667	58.381667;
                 -5.025278	58.382778;
                 -5.023889	58.386944;
                 -5.022500	58.390833;
                 -5.026389	58.393333;
                 -5.032500	58.393056;
                 -5.038611	58.393889;
                 -5.038889	58.398056;
                 -5.043611	58.400000;
                 -5.050833	58.399444;
                 -5.058611	58.400278;
                 -5.065278	58.401667;
                 -5.071944	58.402778;
                 -5.077778	58.404444;
                 -5.084722	58.405556;
                 -5.091667	58.406667;
                 -5.096389	58.408889;
                 -5.093333	58.411944;
                 -5.087778	58.413611;
                 -5.081667	58.414722;
                 -5.075278	58.415556;
                 -5.074722	58.418889;
                 -5.083611	58.419167;
                 -5.091389	58.419167;
                 -5.097222	58.420833;
                 -5.100278	58.423611;
                 -5.099444	58.428333;
                 -5.095833	58.430833;
                 -5.091667	58.433333;
                 -5.086111	58.435000;
                 -5.080556	58.436667;
                 -5.074167	58.437778;
                 -5.068056	58.438611;
                 -5.061111	58.439444;
                 -5.052222	58.438889;
                 -5.047500	58.441111;
                 -5.042778	58.443056;
                 -5.035000	58.443333;
                 -5.028056	58.441944;
                 -5.020278	58.441111;
                 -5.014444	58.439444;
                 -5.009444	58.437500;
                 -5.004722	58.435556;
                 -4.998889	58.433889;
                 -4.991944	58.432500;
                 -4.987222	58.433889;
                 -4.986111	58.436944;
                 -4.990833	58.438889;
                 -4.996944	58.440556;
                 -5.003611	58.441944;
                 -5.009444	58.443611;
                 -5.014444	58.445556;
                 -5.019167	58.447500;
                 -5.023056	58.450000;
                 -5.021111	58.453611;
                 -5.016389	58.455556;
                 -5.013889	58.459167;
                 -5.017778	58.461667;
                 -5.025556	58.462500;
                 -5.032778	58.461944;
                 -5.039722	58.461389;
                 -5.046944	58.460833;
                 -5.053889	58.460000;
                 -5.059722	58.461667;
                 -5.063889	58.464167;
                 -5.065833	58.467500;
                 -5.069167	58.470556;
                 -5.071944	58.473333;
                 -5.075278	58.476111;
                 -5.081111	58.476944;
                 -5.088333	58.476389;
                 -5.096111	58.476111;
                 -5.101111	58.478333;
                 -5.103889	58.481111;
                 -5.106111	58.484444;
                 -5.107500	58.488056;
                 -5.107778	58.492222;
                 -5.106111	58.496389;
                 -5.104444	58.500278;
                 -5.105000	58.503611;
                 -5.106944	58.506944;
                 -5.110000	58.509722;
                 -5.106944	58.512778;
                 -5.102222	58.515000;
                 -5.099167	58.518056;
                 -5.095833	58.521111;
                 -5.090278	58.522778;
                 -5.083333	58.523333;
                 -5.077778	58.524722;
                 -5.074722	58.528056;
                 -5.071667	58.531111;
                 -5.065278	58.531944;
                 -5.058056	58.532778;
                 -5.051111	58.533333;
                 -5.046111	58.535278;
                 -5.040833	58.536944;
                 -5.036944	58.539444;
                 -5.033056	58.542222;
                 -5.029722	58.545278;
                 -5.026667	58.548333;
                 -5.025278	58.552222;
                 -5.024167	58.556944;
                 -5.021111	58.560000;
                 -5.018889	58.563611;
                 -5.018056	58.568056;
                 -5.017222	58.571667;
                 -5.013333	58.574167;
                 -5.011111	58.577778;
                 -5.010556	58.582222;
                 -5.010833	58.586389;
                 -5.011667	58.590278;
                 -5.011944	58.594444;
                 -5.012222	58.598333;
                 -5.009722	58.601944;
                 -5.008333	58.606111;
                 -5.008611	58.610278;
                 -5.007778	58.614722;
                 -5.009167	58.618611;
                 -5.008333	58.622222;
                 -5.005000	58.625278;
                 -5.001389	58.626944;
                 -5.003056	58.628333];
  
  uki(84).xy = [ -1.073611	50.677778;
                 -1.077500	50.676667;
                 -1.082222	50.675278;
                 -1.086389	50.673333;
                 -1.091667	50.672500;
                 -1.096389	50.671111;
                 -1.101111	50.669722;
                 -1.106389	50.668611;
                 -1.111667	50.667778;
                 -1.116944	50.666944;
                 -1.121667	50.665556;
                 -1.126944	50.664444;
                 -1.131111	50.662500;
                 -1.135833	50.661111;
                 -1.139167	50.658611;
                 -1.143333	50.656667;
                 -1.146111	50.653889;
                 -1.149444	50.651389;
                 -1.153056	50.648889;
                 -1.155833	50.645833;
                 -1.158611	50.643056;
                 -1.161667	50.640000;
                 -1.164444	50.636944;
                 -1.166111	50.633056;
                 -1.166944	50.628333;
                 -1.166389	50.624722;
                 -1.165833	50.620833;
                 -1.165833	50.616944;
                 -1.165278	50.613056;
                 -1.164444	50.609444;
                 -1.165833	50.604722;
                 -1.168333	50.601944;
                 -1.172500	50.600000;
                 -1.177222	50.598333;
                 -1.181944	50.596667;
                 -1.186667	50.595278;
                 -1.191389	50.593611;
                 -1.196111	50.592222;
                 -1.200556	50.590833;
                 -1.205278	50.589444;
                 -1.210556	50.588611;
                 -1.215278	50.587222;
                 -1.220833	50.586111;
                 -1.225833	50.585278;
                 -1.231111	50.584444;
                 -1.236667	50.583333;
                 -1.241667	50.582500;
                 -1.247500	50.582222;
                 -1.253056	50.581111;
                 -1.258889	50.580833;
                 -1.265000	50.580000;
                 -1.270833	50.579444;
                 -1.276667	50.579167;
                 -1.282500	50.578889;
                 -1.289167	50.578889;
                 -1.295556	50.578889;
                 -1.301667	50.580000;
                 -1.307500	50.581389;
                 -1.311389	50.583611;
                 -1.315278	50.585833;
                 -1.318056	50.588333;
                 -1.321389	50.590833;
                 -1.325000	50.593611;
                 -1.328056	50.596111;
                 -1.331111	50.598611;
                 -1.335000	50.600833;
                 -1.338889	50.603056;
                 -1.343611	50.605278;
                 -1.349167	50.606667;
                 -1.352778	50.609444;
                 -1.356667	50.611667;
                 -1.360000	50.614167;
                 -1.363611	50.616389;
                 -1.367500	50.618611;
                 -1.373056	50.620000;
                 -1.376944	50.622222;
                 -1.381667	50.623889;
                 -1.387222	50.625278;
                 -1.391944	50.626944;
                 -1.397500	50.628611;
                 -1.402778	50.630000;
                 -1.407500	50.631667;
                 -1.413056	50.633056;
                 -1.418333	50.634444;
                 -1.424167	50.635833;
                 -1.429722	50.637222;
                 -1.434444	50.639167;
                 -1.438333	50.641389;
                 -1.442778	50.643056;
                 -1.446667	50.645278;
                 -1.451389	50.646944;
                 -1.456389	50.648889;
                 -1.460000	50.651111;
                 -1.463889	50.653056;
                 -1.466389	50.656111;
                 -1.469167	50.658611;
                 -1.473056	50.660833;
                 -1.478889	50.662222;
                 -1.484167	50.663611;
                 -1.488889	50.665556;
                 -1.493611	50.667222;
                 -1.499167	50.668611;
                 -1.506389	50.669167;
                 -1.512222	50.668889;
                 -1.518056	50.668333;
                 -1.523333	50.667500;
                 -1.528056	50.666111;
                 -1.533333	50.665000;
                 -1.538056	50.663611;
                 -1.543333	50.662778;
                 -1.548611	50.661944;
                 -1.553333	50.660278;
                 -1.558611	50.659444;
                 -1.563889	50.658611;
                 -1.569167	50.657500;
                 -1.570000	50.660556;
                 -1.565833	50.662500;
                 -1.561667	50.664444;
                 -1.557778	50.666389;
                 -1.554444	50.668889;
                 -1.553611	50.672500;
                 -1.550833	50.675278;
                 -1.546111	50.676944;
                 -1.541389	50.678333;
                 -1.537222	50.680278;
                 -1.535833	50.684167;
                 -1.532500	50.686667;
                 -1.529167	50.689167;
                 -1.526111	50.692222;
                 -1.523333	50.695278;
                 -1.522500	50.699722;
                 -1.520000	50.703056;
                 -1.515278	50.704722;
                 -1.508333	50.703889;
                 -1.501667	50.703889;
                 -1.495833	50.704444;
                 -1.491111	50.705833;
                 -1.485833	50.707222;
                 -1.481111	50.708611;
                 -1.475833	50.709444;
                 -1.471111	50.710833;
                 -1.466667	50.712500;
                 -1.461944	50.713889;
                 -1.456944	50.715278;
                 -1.452500	50.716667;
                 -1.447778	50.718056;
                 -1.443889	50.720000;
                 -1.439722	50.721944;
                 -1.435556	50.723889;
                 -1.432222	50.726389;
                 -1.427500	50.727778;
                 -1.421667	50.728333;
                 -1.417500	50.726944;
                 -1.418889	50.723056;
                 -1.422500	50.720556;
                 -1.424167	50.716667;
                 -1.421944	50.713611;
                 -1.416389	50.713056;
                 -1.411389	50.714444;
                 -1.406667	50.715833;
                 -1.402222	50.717500;
                 -1.396667	50.718333;
                 -1.392222	50.719722;
                 -1.388056	50.721667;
                 -1.388056	50.725833;
                 -1.390000	50.728889;
                 -1.388611	50.731944;
                 -1.384167	50.733333;
                 -1.379722	50.733889;
                 -1.375556	50.735833;
                 -1.370833	50.737222;
                 -1.366667	50.739167;
                 -1.362778	50.741111;
                 -1.358611	50.743056;
                 -1.354444	50.745000;
                 -1.350278	50.746944;
                 -1.346944	50.749444;
                 -1.343611	50.751944;
                 -1.340833	50.755000;
                 -1.338333	50.758333;
                 -1.333333	50.759444;
                 -1.327778	50.760278;
                 -1.323333	50.761667;
                 -1.319722	50.764167;
                 -1.316389	50.766667;
                 -1.313056	50.769167;
                 -1.308889	50.771111;
                 -1.302222	50.770833;
                 -1.296944	50.771944;
                 -1.291389	50.771389;
                 -1.288056	50.768889;
                 -1.285278	50.766111;
                 -1.279167	50.766667;
                 -1.275278	50.768611;
                 -1.270000	50.767778;
                 -1.267222	50.765000;
                 -1.263056	50.762778;
                 -1.258611	50.761111;
                 -1.254722	50.758889;
                 -1.249722	50.757222;
                 -1.245000	50.755278;
                 -1.240556	50.753611;
                 -1.236667	50.751389;
                 -1.231944	50.749444;
                 -1.227222	50.747778;
                 -1.222500	50.745833;
                 -1.217778	50.744167;
                 -1.211389	50.743056;
                 -1.205833	50.741667;
                 -1.199722	50.740556;
                 -1.194167	50.739167;
                 -1.188611	50.737778;
                 -1.183056	50.736389;
                 -1.177500	50.735000;
                 -1.171111	50.733889;
                 -1.164167	50.733333;
                 -1.157500	50.733333;
                 -1.150833	50.733056;
                 -1.144444	50.733056;
                 -1.139167	50.733889;
                 -1.133889	50.734722;
                 -1.129167	50.736111;
                 -1.124444	50.737778;
                 -1.118056	50.737500;
                 -1.113889	50.735278;
                 -1.111111	50.732778;
                 -1.107222	50.730556;
                 -1.103889	50.728056;
                 -1.101111	50.725278;
                 -1.097222	50.723333;
                 -1.093889	50.720556;
                 -1.095833	50.716667;
                 -1.096667	50.711944;
                 -1.091944	50.710278;
                 -1.086111	50.709722;
                 -1.081667	50.708056;
                 -1.079167	50.705833;
                 -1.076111	50.703333;
                 -1.074444	50.700000;
                 -1.072222	50.696944;
                 -1.070833	50.693611;
                 -1.067778	50.691111;
                 -1.062222	50.689722;
                 -1.059722	50.687500;
                 -1.063611	50.685000;
                 -1.067500	50.683056;
                 -1.071944	50.681667;
                 -1.073611	50.677778];
  
  uki(85).xy = [              -6.295000	49.933889;
                              -6.291389	49.931389;
                              -6.287500	49.929444;
                              -6.285000	49.926667;
                              -6.283333	49.923333;
                              -6.280000	49.920833;
                              -6.279167	49.918889;
                              -6.283333	49.916944;
                              -6.286389	49.913889;
                              -6.290833	49.912222;
                              -6.296389	49.912500;
                              -6.301389	49.913889;
                              -6.306389	49.912778;
                              -6.311389	49.911667;
                              -6.317222	49.911944;
                              -6.317500	49.915278;
                              -6.314444	49.916944;
                              -6.310000	49.918611;
                              -6.306944	49.921111;
                              -6.307222	49.925278;
                              -6.308056	49.928889;
                              -6.305556	49.931944;
                              -6.301389	49.933611;
                              -6.295000	49.933889];

  uki(86).xy = [              -6.336667	49.963611;
                              -6.330278	49.962778;
                              -6.324722	49.961667;
                              -6.319722	49.960000;
                              -6.318056	49.956667;
                              -6.316389	49.953611;
                              -6.313611	49.950556;
                              -6.313611	49.946667;
                              -6.315833	49.943333;
                              -6.321111	49.942222;
                              -6.327500	49.942222;
                              -6.331667	49.944167;
                              -6.332500	49.947778;
                              -6.333611	49.951389;
                              -6.336944	49.953889;
                              -6.341667	49.955556;
                              -6.345833	49.957500;
                              -6.346111	49.960833;
                              -6.341667	49.962500;
                              -6.336667	49.963611];

  uki(87).xy = [                -4.666389	53.320000;
                                -4.661944	53.318889;
                                -4.657778	53.316944;
                                -4.650833	53.316111;
                                -4.645556	53.316111;
                                -4.641389	53.318333;
                                -4.637222	53.320278;
                                -4.632222	53.321944;
                                -4.626667	53.323056;
                                -4.621667	53.321389;
                                -4.624167	53.318333;
                                -4.628333	53.316111;
                                -4.630556	53.312778;
                                -4.627222	53.310278;
                                -4.621111	53.308889;
                                -4.615833	53.310000;
                                -4.609722	53.308889;
                                -4.606111	53.306389;
                                -4.603611	53.303333;
                                -4.600000	53.300833;
                                -4.595000	53.299167;
                                -4.590833	53.297222;
                                -4.588056	53.294167;
                                -4.586111	53.290833;
                                -4.585278	53.287222;
                                -4.585833	53.282778;
                                -4.586667	53.278056;
                                -4.586389	53.275000;
                                -4.583056	53.272500;
                                -4.578611	53.270278;
                                -4.575833	53.267500;
                                -4.574167	53.264167;
                                -4.569722	53.261944;
                                -4.563333	53.261111;
                                -4.560556	53.258333;
                                -4.559444	53.254444;
                                -4.559444	53.250556;
                                -4.560833	53.246389;
                                -4.563611	53.243333;
                                -4.567222	53.240833;
                                -4.571389	53.238611;
                                -4.577500	53.238056;
                                -4.583611	53.238333;
                                -4.588611	53.240000;
                                -4.593056	53.242222;
                                -4.594722	53.245556;
                                -4.597500	53.248333;
                                -4.600000	53.251389;
                                -4.603611	53.253889;
                                -4.606944	53.256111;
                                -4.608611	53.259444;
                                -4.610556	53.262778;
                                -4.611667	53.266667;
                                -4.615000	53.269167;
                                -4.620000	53.270833;
                                -4.624167	53.272778;
                                -4.629444	53.274444;
                                -4.633056	53.276944;
                                -4.636389	53.279444;
                                -4.641667	53.281111;
                                -4.646944	53.280833;
                                -4.651944	53.279444;
                                -4.657500	53.278333;
                                -4.664444	53.278333;
                                -4.671111	53.278889;
                                -4.676944	53.280278;
                                -4.680556	53.282778;
                                -4.683056	53.285556;
                                -4.682500	53.290278;
                                -4.683333	53.293889;
                                -4.686667	53.296389;
                                -4.691111	53.298611;
                                -4.693889	53.301389;
                                -4.691944	53.304167;
                                -4.688611	53.306667;
                                -4.682778	53.307778;
                                -4.679444	53.310278;
                                -4.679444	53.314444;
                                -4.681389	53.317778;
                                -4.678611	53.320833;
                                -4.672500	53.321389;
                                -4.666389	53.320000];

  uki(88).xy = [ -4.413056	53.423889;
                 -4.406389	53.423889;
                 -4.400278	53.422778;
                 -4.396111	53.420556;
                 -4.391667	53.418611;
                 -4.386111	53.418611;
                 -4.380556	53.419722;
                 -4.373056	53.419167;
                 -4.366944	53.418056;
                 -4.361944	53.416389;
                 -4.356667	53.414722;
                 -4.350833	53.413333;
                 -4.344444	53.413889;
                 -4.338889	53.415000;
                 -4.333056	53.413889;
                 -4.326944	53.412500;
                 -4.320833	53.413056;
                 -4.313889	53.412222;
                 -4.309444	53.410278;
                 -4.305556	53.408056;
                 -4.299444	53.407778;
                 -4.292222	53.407778;
                 -4.288056	53.405833;
                 -4.283611	53.403611;
                 -4.280278	53.401111;
                 -4.276944	53.398611;
                 -4.275278	53.395278;
                 -4.273056	53.391944;
                 -4.269722	53.389167;
                 -4.268056	53.385833;
                 -4.266389	53.382500;
                 -4.268333	53.378889;
                 -4.271111	53.375833;
                 -4.271667	53.372222;
                 -4.268333	53.369722;
                 -4.265000	53.367222;
                 -4.262222	53.364167;
                 -4.261389	53.360556;
                 -4.260556	53.356944;
                 -4.256111	53.354722;
                 -4.250278	53.353333;
                 -4.244167	53.353056;
                 -4.238056	53.351944;
                 -4.235556	53.348889;
                 -4.234722	53.345278;
                 -4.233611	53.341389;
                 -4.233333	53.337222;
                 -4.230000	53.334722;
                 -4.228333	53.331389;
                 -4.227222	53.327778;
                 -4.224722	53.325000;
                 -4.222222	53.321944;
                 -4.217778	53.320000;
                 -4.213611	53.317778;
                 -4.210278	53.315278;
                 -4.207500	53.312500;
                 -4.205833	53.309167;
                 -4.204722	53.305278;
                 -4.205278	53.301667;
                 -4.208333	53.298611;
                 -4.207222	53.295000;
                 -4.203889	53.292500;
                 -4.196944	53.291667;
                 -4.189444	53.291111;
                 -4.183333	53.291667;
                 -4.177778	53.292778;
                 -4.172778	53.294167;
                 -4.168056	53.295833;
                 -4.162222	53.296944;
                 -4.156389	53.297500;
                 -4.150556	53.298611;
                 -4.145000	53.299444;
                 -4.141111	53.300833;
                 -4.137500	53.303333;
                 -4.134722	53.306389;
                 -4.130556	53.308333;
                 -4.126944	53.310833;
                 -4.121944	53.312500;
                 -4.115833	53.313056;
                 -4.110000	53.312778;
                 -4.104722	53.311111;
                 -4.099722	53.309444;
                 -4.092778	53.308333;
                 -4.085278	53.308056;
                 -4.077500	53.307500;
                 -4.069722	53.307222;
                 -4.063611	53.307778;
                 -4.056111	53.307222;
                 -4.049167	53.307222;
                 -4.042222	53.306389;
                 -4.038056	53.304167;
                 -4.040000	53.301667;
                 -4.043056	53.298611;
                 -4.045000	53.295000;
                 -4.045556	53.290556;
                 -4.047222	53.286389;
                 -4.051389	53.284444;
                 -4.056111	53.282778;
                 -4.061111	53.281389;
                 -4.064722	53.278611;
                 -4.067500	53.275556;
                 -4.070278	53.272500;
                 -4.075278	53.271111;
                 -4.078611	53.268611;
                 -4.081389	53.265556;
                 -4.084167	53.262500;
                 -4.086944	53.259444;
                 -4.090556	53.256944;
                 -4.094722	53.254722;
                 -4.098889	53.252778;
                 -4.103056	53.250556;
                 -4.107778	53.249167;
                 -4.112778	53.247500;
                 -4.117778	53.246111;
                 -4.122500	53.244444;
                 -4.126667	53.242500;
                 -4.131667	53.240833;
                 -4.136389	53.239167;
                 -4.141389	53.237778;
                 -4.146111	53.236111;
                 -4.150278	53.234167;
                 -4.155278	53.232500;
                 -4.160278	53.231111;
                 -4.165000	53.229444;
                 -4.169722	53.227778;
                 -4.174444	53.226389;
                 -4.179444	53.224722;
                 -4.183611	53.222778;
                 -4.187778	53.220556;
                 -4.192222	53.218611;
                 -4.196389	53.216667;
                 -4.200556	53.214444;
                 -4.203889	53.211944;
                 -4.208056	53.210000;
                 -4.211389	53.207222;
                 -4.214444	53.204167;
                 -4.216944	53.201111;
                 -4.218611	53.197222;
                 -4.219167	53.192500;
                 -4.219167	53.188611;
                 -4.220556	53.184444;
                 -4.222778	53.180833;
                 -4.226111	53.178333;
                 -4.229444	53.176667;
                 -4.235000	53.175556;
                 -4.240000	53.174167;
                 -4.245278	53.173056;
                 -4.250278	53.171389;
                 -4.254444	53.169444;
                 -4.258056	53.166944;
                 -4.261389	53.164167;
                 -4.265000	53.161667;
                 -4.269167	53.159722;
                 -4.272500	53.156944;
                 -4.275833	53.154444;
                 -4.280000	53.152500;
                 -4.283611	53.150000;
                 -4.287778	53.147778;
                 -4.292778	53.146389;
                 -4.297500	53.144722;
                 -4.301667	53.142778;
                 -4.306389	53.141111;
                 -4.311389	53.139444;
                 -4.316944	53.138611;
                 -4.322500	53.137500;
                 -4.330000	53.137778;
                 -4.337778	53.138333;
                 -4.339722	53.134722;
                 -4.337778	53.131389;
                 -4.336111	53.128056;
                 -4.335278	53.124444;
                 -4.340000	53.122778;
                 -4.346111	53.122222;
                 -4.352222	53.123611;
                 -4.355833	53.125000;
                 -4.360833	53.126667;
                 -4.365833	53.128333;
                 -4.371111	53.130000;
                 -4.375278	53.132222;
                 -4.380278	53.133889;
                 -4.385556	53.135556;
                 -4.390833	53.135278;
                 -4.393611	53.132222;
                 -4.397222	53.129722;
                 -4.400000	53.126667;
                 -4.403333	53.124167;
                 -4.409722	53.123611;
                 -4.410556	53.127222;
                 -4.409167	53.131389;
                 -4.408333	53.135833;
                 -4.409444	53.139722;
                 -4.411111	53.142778;
                 -4.414444	53.145278;
                 -4.421389	53.146389;
                 -4.427222	53.147500;
                 -4.432500	53.149167;
                 -4.435833	53.151667;
                 -4.437778	53.155000;
                 -4.440278	53.157778;
                 -4.442222	53.161111;
                 -4.444444	53.164167;
                 -4.447222	53.166944;
                 -4.450000	53.170000;
                 -4.453333	53.172500;
                 -4.459167	53.173611;
                 -4.466111	53.173611;
                 -4.471667	53.172500;
                 -4.478056	53.171944;
                 -4.484722	53.172778;
                 -4.488889	53.174722;
                 -4.492500	53.177222;
                 -4.495000	53.180278;
                 -4.496944	53.183611;
                 -4.496944	53.187778;
                 -4.495000	53.191389;
                 -4.492222	53.194444;
                 -4.490833	53.198333;
                 -4.492500	53.201667;
                 -4.495833	53.204167;
                 -4.500278	53.206389;
                 -4.503056	53.209167;
                 -4.506389	53.211667;
                 -4.508889	53.214444;
                 -4.511389	53.217500;
                 -4.515000	53.220000;
                 -4.517500	53.222778;
                 -4.520278	53.225833;
                 -4.523056	53.228611;
                 -4.524722	53.231944;
                 -4.528056	53.234444;
                 -4.531389	53.236944;
                 -4.534167	53.239722;
                 -4.537500	53.242222;
                 -4.541111	53.244722;
                 -4.543056	53.248056;
                 -4.543611	53.251944;
                 -4.545556	53.255278;
                 -4.547500	53.258611;
                 -4.551667	53.260556;
                 -4.555833	53.262778;
                 -4.559444	53.265000;
                 -4.561944	53.268056;
                 -4.564722	53.270833;
                 -4.567500	53.273889;
                 -4.567500	53.278056;
                 -4.566389	53.280278;
                 -4.565000	53.284167;
                 -4.563056	53.287778;
                 -4.561389	53.291944;
                 -4.561389	53.296111;
                 -4.562500	53.299722;
                 -4.563611	53.303611;
                 -4.564444	53.307222;
                 -4.566389	53.310556;
                 -4.568056	53.313889;
                 -4.570833	53.316667;
                 -4.572500	53.320000;
                 -4.572778	53.324167;
                 -4.569444	53.326667;
                 -4.566389	53.329722;
                 -4.564444	53.333333;
                 -4.563611	53.338056;
                 -4.563611	53.342222;
                 -4.562500	53.346111;
                 -4.560278	53.349722;
                 -4.558333	53.353333;
                 -4.556944	53.357222;
                 -4.557778	53.361111;
                 -4.559444	53.364444;
                 -4.560556	53.368056;
                 -4.559167	53.372222;
                 -4.558333	53.376667;
                 -4.561111	53.379722;
                 -4.563611	53.382500;
                 -4.566389	53.385556;
                 -4.569167	53.388333;
                 -4.566944	53.391944;
                 -4.563611	53.394444;
                 -4.560000	53.396944;
                 -4.553611	53.397500;
                 -4.547500	53.398056;
                 -4.540556	53.398333;
                 -4.532778	53.397778;
                 -4.528611	53.400000;
                 -4.524444	53.401944;
                 -4.521667	53.405000;
                 -4.518333	53.407500;
                 -4.512222	53.407222;
                 -4.508889	53.404722;
                 -4.505000	53.402222;
                 -4.500556	53.403056;
                 -4.496944	53.405556;
                 -4.493333	53.408056;
                 -4.490000	53.410556;
                 -4.486667	53.413333;
                 -4.483056	53.415833;
                 -4.478889	53.417778;
                 -4.474722	53.418889;
                 -4.468889	53.418611;
                 -4.463611	53.416944;
                 -4.458611	53.415278;
                 -4.454167	53.413333;
                 -4.448056	53.413056;
                 -4.443889	53.415000;
                 -4.440556	53.417500;
                 -4.436389	53.419722;
                 -4.432500	53.422222;
                 -4.429167	53.424722;
                 -4.422778	53.425278;
                 -4.416111	53.425278;
                 -4.413056	53.423889];
  
  uki(89).xy = [              -3.243611	54.147222;
                              -3.243611	54.143056;
                              -3.245833	54.139444;
                              -3.248056	54.136111;
                              -3.248889	54.131389;
                              -3.247222	54.128056;
                              -3.248056	54.123611;
                              -3.251111	54.120556;
                              -3.251944	54.115833;
                              -3.249167	54.113056;
                              -3.246944	54.110000;
                              -3.245833	54.106389;
                              -3.245000	54.102500;
                              -3.243333	54.099167;
                              -3.240833	54.096389;
                              -3.238056	54.093333;
                              -3.236389	54.090000;
                              -3.233889	54.087222;
                              -3.230278	54.084722;
                              -3.225000	54.082778;
                              -3.218333	54.081944;
                              -3.214722	54.079444;
                              -3.214167	54.075833;
                              -3.216667	54.072778;
                              -3.217500	54.070000;
                              -3.211389	54.068611;
                              -3.209722	54.065278;
                              -3.208056	54.061944;
                              -3.204722	54.059444;
                              -3.200278	54.057222;
                              -3.195000	54.055556;
                              -3.187222	54.055000;
                              -3.179444	54.054722;
                              -3.173611	54.053333;
                              -3.170278	54.050833;
                              -3.175000	54.049167;
                              -3.181389	54.048611;
                              -3.189167	54.049167;
                              -3.196111	54.050278;
                              -3.202222	54.051389;
                              -3.208889	54.052500;
                              -3.215000	54.053611;
                              -3.219167	54.055833;
                              -3.223611	54.058056;
                              -3.226111	54.060833;
                              -3.229722	54.063333;
                              -3.232222	54.066389;
                              -3.235000	54.069167;
                              -3.237500	54.072222;
                              -3.239167	54.075556;
                              -3.241667	54.078333;
                              -3.244444	54.081389;
                              -3.246111	54.084722;
                              -3.248611	54.087778;
                              -3.250278	54.091111;
                              -3.253056	54.093889;
                              -3.255556	54.096944;
                              -3.258056	54.099722;
                              -3.259722	54.103056;
                              -3.261667	54.106389;
                              -3.263056	54.109722;
                              -3.263056	54.113889;
                              -3.264167	54.117778;
                              -3.264167	54.121944;
                              -3.265833	54.125278;
                              -3.266944	54.128889;
                              -3.267500	54.132778;
                              -3.267500	54.136667;
                              -3.266111	54.140833;
                              -3.263056	54.143889;
                              -3.260278	54.146944;
                              -3.255000	54.148333;
                              -3.248056	54.148333;
                              -3.243611	54.147222];

  uki(90).xy = [ -4.352778	54.409167;
                 -4.352778	54.405278;
                 -4.352500	54.401111;
                 -4.353889	54.396944;
                 -4.356111	54.393333;
                 -4.358333	54.389722;
                 -4.360556	54.386389;
                 -4.362778	54.382778;
                 -4.364722	54.379167;
                 -4.367778	54.376111;
                 -4.370000	54.372500;
                 -4.371944	54.368889;
                 -4.373611	54.365000;
                 -4.375000	54.360833;
                 -4.375833	54.356389;
                 -4.376389	54.351667;
                 -4.376111	54.347500;
                 -4.375278	54.343889;
                 -4.374167	54.340278;
                 -4.373333	54.336389;
                 -4.372222	54.332778;
                 -4.371389	54.328889;
                 -4.369722	54.325556;
                 -4.367778	54.322222;
                 -4.365000	54.319444;
                 -4.362222	54.316389;
                 -4.357222	54.314722;
                 -4.350000	54.313889;
                 -4.345000	54.312222;
                 -4.341389	54.309722;
                 -4.336944	54.307778;
                 -4.330000	54.306944;
                 -4.323889	54.305556;
                 -4.316944	54.304722;
                 -4.311667	54.303056;
                 -4.308333	54.300556;
                 -4.307500	54.296944;
                 -4.309444	54.293333;
                 -4.312222	54.290278;
                 -4.315833	54.287778;
                 -4.318889	54.284722;
                 -4.320000	54.280556;
                 -4.321667	54.276667;
                 -4.325278	54.273611;
                 -4.329722	54.271389;
                 -4.334444	54.270000;
                 -4.336667	54.266389;
                 -4.338056	54.262222;
                 -4.341111	54.259167;
                 -4.344444	54.256667;
                 -4.348889	54.254722;
                 -4.351667	54.251667;
                 -4.354444	54.248611;
                 -4.356667	54.245000;
                 -4.358056	54.240833;
                 -4.360278	54.237222;
                 -4.363056	54.234167;
                 -4.364444	54.230278;
                 -4.366667	54.226667;
                 -4.371111	54.224444;
                 -4.377500	54.223889;
                 -4.383056	54.223056;
                 -4.388056	54.221389;
                 -4.393056	54.219722;
                 -4.396389	54.217222;
                 -4.398611	54.213611;
                 -4.397778	54.210000;
                 -4.393333	54.207778;
                 -4.390000	54.205278;
                 -4.386111	54.202778;
                 -4.385278	54.199167;
                 -4.386111	54.195556;
                 -4.388611	54.192500;
                 -4.391667	54.189444;
                 -4.394444	54.186389;
                 -4.398056	54.183889;
                 -4.402222	54.181667;
                 -4.406389	54.179722;
                 -4.410833	54.177500;
                 -4.415000	54.175556;
                 -4.419167	54.173333;
                 -4.424444	54.171944;
                 -4.428333	54.169722;
                 -4.432778	54.167778;
                 -4.436944	54.165556;
                 -4.441944	54.164167;
                 -4.448889	54.164167;
                 -4.454722	54.163056;
                 -4.459722	54.161389;
                 -4.463056	54.158889;
                 -4.467500	54.156667;
                 -4.470278	54.153611;
                 -4.472500	54.150278;
                 -4.470833	54.146944;
                 -4.468056	54.143889;
                 -4.466944	54.140278;
                 -4.470556	54.137500;
                 -4.475556	54.136111;
                 -4.480556	54.134444;
                 -4.484722	54.132500;
                 -4.487500	54.129444;
                 -4.491111	54.126667;
                 -4.497500	54.126111;
                 -4.504722	54.126111;
                 -4.510278	54.125000;
                 -4.515000	54.123611;
                 -4.519167	54.121389;
                 -4.522222	54.118333;
                 -4.524444	54.114722;
                 -4.527222	54.111667;
                 -4.532222	54.110278;
                 -4.536389	54.108056;
                 -4.540833	54.106111;
                 -4.544167	54.103333;
                 -4.546389	54.100000;
                 -4.550556	54.097778;
                 -4.556111	54.096667;
                 -4.561667	54.095833;
                 -4.568889	54.095556;
                 -4.575833	54.095556;
                 -4.580833	54.093889;
                 -4.585278	54.091944;
                 -4.588056	54.088889;
                 -4.591667	54.086389;
                 -4.595000	54.084722;
                 -4.599167	54.082500;
                 -4.601944	54.079444;
                 -4.603889	54.075833;
                 -4.603889	54.071667;
                 -4.603056	54.068056;
                 -4.603889	54.063611;
                 -4.605833	54.060000;
                 -4.609444	54.057222;
                 -4.615000	54.056389;
                 -4.618333	54.058611;
                 -4.619444	54.062500;
                 -4.620278	54.066111;
                 -4.623889	54.068611;
                 -4.630000	54.070000;
                 -4.635833	54.068889;
                 -4.640556	54.067222;
                 -4.645000	54.065278;
                 -4.650000	54.063611;
                 -4.654167	54.061667;
                 -4.659167	54.060000;
                 -4.666667	54.060278;
                 -4.670278	54.062778;
                 -4.673889	54.065278;
                 -4.674722	54.069167;
                 -4.677500	54.071944;
                 -4.680278	54.075000;
                 -4.684444	54.076944;
                 -4.690556	54.078333;
                 -4.697778	54.078889;
                 -4.704722	54.079722;
                 -4.709444	54.078333;
                 -4.714444	54.076667;
                 -4.720833	54.076111;
                 -4.724444	54.073611;
                 -4.726944	54.070556;
                 -4.730833	54.067778;
                 -4.734167	54.065278;
                 -4.738333	54.063333;
                 -4.743333	54.061667;
                 -4.749722	54.061111;
                 -4.756944	54.061111;
                 -4.762500	54.060000;
                 -4.767222	54.058333;
                 -4.772222	54.056667;
                 -4.777778	54.055556;
                 -4.783889	54.056944;
                 -4.787500	54.059444;
                 -4.788611	54.063056;
                 -4.788611	54.067222;
                 -4.785833	54.070278;
                 -4.781667	54.072500;
                 -4.776667	54.073889;
                 -4.772500	54.076111;
                 -4.769167	54.078611;
                 -4.769167	54.081944;
                 -4.773611	54.083889;
                 -4.774722	54.087778;
                 -4.772500	54.091111;
                 -4.769167	54.093889;
                 -4.765278	54.096389;
                 -4.763333	54.100000;
                 -4.761111	54.103611;
                 -4.756944	54.105556;
                 -4.753056	54.107778;
                 -4.750000	54.110833;
                 -4.747778	54.114167;
                 -4.745833	54.117778;
                 -4.743056	54.120833;
                 -4.740833	54.124444;
                 -4.737222	54.126944;
                 -4.734444	54.130278;
                 -4.730833	54.132778;
                 -4.729444	54.136667;
                 -4.728889	54.141389;
                 -4.730000	54.144167;
                 -4.730833	54.147778;
                 -4.731944	54.151667;
                 -4.732778	54.155278;
                 -4.734167	54.159167;
                 -4.735000	54.162778;
                 -4.736944	54.166111;
                 -4.736944	54.170278;
                 -4.732778	54.172222;
                 -4.728611	54.174444;
                 -4.725000	54.176944;
                 -4.721389	54.179444;
                 -4.718611	54.182500;
                 -4.717222	54.186667;
                 -4.715000	54.190278;
                 -4.714444	54.194722;
                 -4.712778	54.198889;
                 -4.713056	54.203056;
                 -4.713333	54.207222;
                 -4.715000	54.210556;
                 -4.713889	54.214444;
                 -4.710278	54.217222;
                 -4.704722	54.218056;
                 -4.698889	54.219167;
                 -4.693889	54.220833;
                 -4.688889	54.222500;
                 -4.683889	54.224444;
                 -4.678889	54.226111;
                 -4.674722	54.228056;
                 -4.669722	54.229722;
                 -4.664722	54.231111;
                 -4.660556	54.233333;
                 -4.655556	54.235000;
                 -4.650556	54.236389;
                 -4.645833	54.238056;
                 -4.641389	54.240000;
                 -4.638056	54.242778;
                 -4.633611	54.244722;
                 -4.629444	54.246944;
                 -4.625000	54.248889;
                 -4.621667	54.251389;
                 -4.618056	54.254167;
                 -4.615278	54.257222;
                 -4.612222	54.260278;
                 -4.608889	54.262778;
                 -4.605278	54.265278;
                 -4.601944	54.267778;
                 -4.598889	54.270833;
                 -4.595833	54.273889;
                 -4.593889	54.277500;
                 -4.591111	54.280556;
                 -4.588056	54.283611;
                 -4.585833	54.287222;
                 -4.583889	54.290833;
                 -4.581111	54.293889;
                 -4.578056	54.296944;
                 -4.576111	54.300556;
                 -4.573889	54.304167;
                 -4.572500	54.308056;
                 -4.570278	54.311667;
                 -4.568889	54.315833;
                 -4.566944	54.319444;
                 -4.563889	54.322500;
                 -4.560556	54.325000;
                 -4.557500	54.328056;
                 -4.554722	54.331111;
                 -4.553333	54.335278;
                 -4.553333	54.339167;
                 -4.552778	54.343889;
                 -4.549722	54.346944;
                 -4.547222	54.350000;
                 -4.544167	54.353056;
                 -4.541389	54.356111;
                 -4.537778	54.358611;
                 -4.535000	54.361667;
                 -4.531389	54.364722;
                 -4.527500	54.367222;
                 -4.522778	54.368889;
                 -4.518611	54.370000;
                 -4.513611	54.371667;
                 -4.509167	54.373611;
                 -4.505000	54.375833;
                 -4.500833	54.377778;
                 -4.496667	54.380000;
                 -4.493056	54.382500;
                 -4.489444	54.385000;
                 -4.485000	54.387222;
                 -4.480278	54.388611;
                 -4.474444	54.389722;
                 -4.468056	54.390278;
                 -4.463056	54.391944;
                 -4.458056	54.393333;
                 -4.453056	54.395000;
                 -4.448056	54.396667;
                 -4.443056	54.398056;
                 -4.437222	54.399167;
                 -4.431667	54.400278;
                 -4.425278	54.400833;
                 -4.418889	54.401389;
                 -4.412222	54.401944;
                 -4.406111	54.402500;
                 -4.400278	54.403611;
                 -4.394444	54.404722;
                 -4.389722	54.406389;
                 -4.385278	54.408333;
                 -4.381111	54.410278;
                 -4.377500	54.413056;
                 -4.373333	54.415000;
                 -4.368056	54.416667;
                 -4.360278	54.416111;
                 -4.356944	54.413611;
                 -4.354167	54.410833;
                 -4.352778	54.409167];
  
  uki(91).xy = [ -4.801667	54.057778;
                             -4.803056	54.055556;
                             -4.803611	54.051111;
                             -4.805278	54.046944;
                             -4.807778	54.043889;
                             -4.811389	54.041389;
                             -4.815556	54.039167;
                             -4.820556	54.037778;
                             -4.826667	54.038889;
                             -4.829444	54.041667;
                             -4.829722	54.045000;
                             -4.823333	54.045556;
                             -4.818333	54.047222;
                             -4.816111	54.050833;
                             -4.815000	54.054722;
                             -4.812778	54.058333;
                             -4.809444	54.061111;
                             -4.804722	54.059722;
                             -4.801667	54.057778];

  uki(92).xy = [ -6.848889	58.197778;
                 -6.781389	58.209722;
                 -6.780278	58.206111;
                 -6.779722	58.201944;
                 -6.777500	58.198611;
                 -6.778056	58.195000;
                 -6.781667	58.192500;
                 -6.787500	58.193889;
                 -6.791667	58.196389;
                 -6.796667	58.198333;
                 -6.802778	58.200000;
                 -6.808611	58.201667;
                 -6.816389	58.201389;
                 -6.823611	58.200556;
                 -6.830278	58.200833;
                 -6.833611	58.203611;
                 -6.831389	58.207222;
                 -6.833611	58.210556;
                 -6.837778	58.213056;
                 -6.841667	58.215278;
                 -6.845833	58.217778;
                 -6.849722	58.220278;
                 -6.855833	58.221667;
                 -6.862500	58.222778;
                 -6.863056	58.226944;
                 -6.865278	58.230278;
                 -6.869444	58.232778;
                 -6.876944	58.233333;
                 -6.883056	58.235000;
                 -6.885278	58.238333;
                 -6.886389	58.241944;
                 -6.888056	58.245556;
                 -6.883333	58.247778;
                 -6.877222	58.248056;
                 -6.870278	58.246944;
                 -6.861667	58.246667;
                 -6.853889	58.246944;
                 -6.846944	58.245833;
                 -6.843889	58.242778;
                 -6.839722	58.240556;
                 -6.837500	58.237222;
                 -6.834167	58.234444;
                 -6.831944	58.231111;
                 -6.830833	58.227500;
                 -6.827500	58.224444;
                 -6.820833	58.224444;
                 -6.816111	58.226389;
                 -6.810278	58.225000;
                 -6.808056	58.221667;
                 -6.805000	58.218889;
                 -6.799722	58.216944;
                 -6.793889	58.215278;
                 -6.788056	58.213611;
                 -6.781389	58.209722];
end
